# DocumentationBuild

This project provides the functionality for building Wolfram documentation authored with
[`DocuTools`][DocuTools] (internal use) or [`DocumentationTools`][DocumentationTools]
(public use).

[DocuTools]: https://stash.wolfram.com/projects/PUBS/repos/docutools/browse
[DocumentationTools]: https://stash.wolfram.com/projects/PUBS/repos/documentationtools/browse

## File Overview (work in progress)

* [`./Assets`](./Assets) contains image and WL graphics assets which are common to all
  documentation (internal and user-written).
  - In contrast, the [`PUBS/DocumentationAssets`][doc-assets] repository contains assets which are
    specific to our internal documentation.

* [`./SystemFiles/ant/Build/`](./SystemFiles/ant/Build) contains
  [Ant](https://ant.apache.org/) build scripts used by the internal documentation builds (specified in
  [re-documentation-build](https://stash.wolfram.com/projects/RE/repos/re-documentation-build/browse)).

* [`./Kernel/Build/`](./Kernel/Build/) contains top-level entry points for building
  collections of documentation. This directory is a work-in-progress, but is intended
  to be the place where logic factored out of the .xml scripts can be collected.

[doc-assets]: https://stash.wolfram.com/projects/PUBS/repos/documentationassets/browse

## Building paclet Documentation

The `DocumentationBuildNotebooks` function can be used to build any documentation arranged
in the standard `Documentation` directory structure. Simply specify a path to the input
`Documentation` directory, and a path to place the built directory.

```wolfram
Needs["DocumentationBuild`"]

DocumentationBuildNotebooks[
	"/path/to/Documentation",
	"~/Desktop/BuiltDocumentation"
]
```

Here's an example of using `DocumentationBuildNotebooks` to build the [`GitLink`][GitLink]
documentation:

![DocumentationBuildNotebooks example showing progress monitoring](./docs/images/DocumentationBuildNotebooks-Progress.png)

Assuming the build completes successfully, a `Success[..]` object will be returned with
information about the build:

![DocumentationBuildNotebooks example show Success[..] object keys](./docs/images/DocumentationBuildNotebooks-Success.png)

[GitLink]: https://stash.wolfram.com/projects/PAC/repos/gitlink/browse

#### Filtering

It's possible to restrict the set of built files (useful during testing or for quick
iteration) using a filter pattern:

```wolfram
DocumentationBuildNotebooks[
	"/path/to/Documentation",
	"~/Desktop/BuiltDocumentation",
	___ ~~ "MyFunction.nb"
]
```

The filter pattern can a `String` literal, `StringExpression`, or predicate function which
will be applied to the file name:

```wolfram
DocumentationBuildNotebooks[
	"/path/to/Documentation",
	"~/Desktop/BuiltDocumentation",
	StringContainsQ["MyFunction.nb"]
]
```

## Building System documentation

*[Work in progress]*

*TODO:* Document process of checking out the Pubs/Documentation directory from CVS.

Assuming you have the [Pubs/Documentation/](https://cvs-master.wolfram.com/viewvc/Pubs/Documentation/)
CVS repository checked out locally on your system, you may use `DocumentationBuildSystemNotebooks[]`
to build the system documentation locally.

_TODO:_ What is the difference between `Pubs/Documentation/` and `Mathematica/Documentation/`
in CVS? Are they aliases of each other?

The system documentation builds are [split into many parts][teamcity-nb-builds], with the
intention that each subset takes approximately two hours to execute. This is reflected in
the `$buildGroup` variable in the example code below.

The {3, 4} build includes all the symbol reference pages whose names start with "A" or "B".
See the source code for more details.

```wolfram
Needs["DocumentationBuild`"]

$buildGroup = 4;

DocumentationBuildSystemNotebooks[
	"/path/to/cvs-root/Pubs",
	"~/Desktop/SystemBuiltNotebooks",
	$buildGroup
]
```

[teamcity-nb-builds]: https://teamcity.wolfram.com/project/WolframLanguage_Documentation_English_Notebooks?mode=builds

#### Using Ant

*TODO:* Document what the various common options to set in one of the .properties files are.

```shell
ant -buildfile MakeMathematicaNotebooks.xml -propertyfile connorg-MakeMathematicaNotebooks.properties
ant -buildfile MakeMathematicaHTML.xml      -propertyfile connorg-MakeMathematicaHTML.properties
```

* The `outputDir` of the `MakeMathematicaNotebooks` step should be the `inputDir` of
  `MakeMathematicaHTML`.