#!/Applications/Mathematica.app/Contents/MacOS/WolframScript -script

(* https://rosettacode.org/wiki/Convert_seconds_to_compound_duration#Mathematica *)
compoundDuration[x_Integer] := 
  StringJoin @@ (Riffle[
    ToString /@ (
      (
        (
          {Floor[x/604800] , Mod[x, 604800]} /. {a_, b_} -> {a, Floor[b/86400], Mod[b, 86400]}
        ) /. {a__, b_} -> {a, Floor[b/3600], Mod[b, 3600]}
      ) /. {a__, b_} -> {a, Floor[b/60], Mod[b, 60]}
    )
  , {" wk, ", " d, ", " hr, ", " min, ", " sec"}] //. {a___, "0", b_, c___} -> {a, c}
  )

data = Sort[Import["build-time-db.m"]];
Print["Number of files analyzed: "<>ToString[Length[data], InputForm]];

(* data format:
"English/System/ReferencePages/Symbols/Accumulate.nb" -> 53
"English/System/ReferencePages/Symbols/AccuracyGoal.nb" -> 27
"English/System/ReferencePages/Symbols/Accuracy.nb" -> 45 *)

total = Ceiling @ Total[data[[All, 2]]];
Print["Total build time for all files = "<>ToString[total, InputForm]<>" seconds"];
Print[ToString[compoundDuration[total]]];

(* One can divide the web build into two separate builds with
numberOfSeparateBuilds = 2, each of them taking approximately half the needed
build time.  In that case, split 'data' into two buckets, for each of which
Total@bucket[[All, 2]] ~= total/2.

But here, let's just do the old traditional *one* set of 8 builds. *)

numberOfSeparateBuilds = 1;

targetTime = IntegerPart[total/numberOfSeparateBuilds];
Print["targetTime = "<>ToString[targetTime, InputForm]];

timings = data[[All, 2]];
temp = {};
i = 0;

While[
  Total[temp] < targetTime
, i++;
  AppendTo[temp, timings[[i]]];
];

(* Print["after loop, i = "<>ToString[i, InputForm]]; *)
(* Print["Total[temp] = "<>ToString[Total[temp], InputForm]]; *)

mainSplitPoint = i;

build1List = timings[[;;mainSplitPoint]]; 
build2List = timings[[mainSplitPoint;;]];

Print["Length[build1List] = "<>ToString[Length[build1List], InputForm]];
Print["Length[build2List] = "<>ToString[Length[build2List], InputForm]];

(* return a list of partitionings for each buildlist *)

(* evenly distribute each of the two builds into 8 bins *)
partitionLists = Module[{temp, binNumber, binSize}, 

  (* we'll destructively drop items from temp into our 8 bins, but we'll still
  want to refer to the original data at the end of the action *)
  temp = #;

  (* Print["filling bins"]; *)
  binNumber = 1; p = {};
  binSize = IntegerPart @ Total[temp/8];
  (* Print["binSize = "<>ToString[binSize, InputForm]<>" which is "<>ToString[compoundDuration[binSize], InputForm]]; *)

  While[Total[temp] > binSize,
    (* while there are still enough seconds left in temp to fill a bin.., *)
    (* Print["filling bin number "<>ToString[binNumber]]; *)

    (* start the bin-filling action with the first item in temp *)
    i = 1;

    While[Total[temp[[;;i]]] < binSize,
      (* the bin-filler: while we haven't yet filled a bin with items from temp... *)

      (* ...grab the next item from temp *)
      i++;
      
    ]; (* close the bin-filler *)

    (* now we have enough items from temp to fill a bin.  save the number of the last item... *)
    AppendTo[p, i];

    (* and remove that many items from the front of temp, since we have now
    conceptually dropped them into a bin *)

    temp = Drop[temp, i];

    binNumber++;
  ]; (* close "while there are enough items left in temp to fill a bin" *)

  (* Print["done filling bins"]; *)

  (* Print["\n========================================"]; *)
  (* Print["Partition report"]; *)
  (* Print["========================================\n"]; *)

  (* Print["Total number of notebooks built: "<>ToString[Length @ #, InputForm]]; *)

  (* report the Part numbers to use when splitting up the source files into builds *)
  (* Print["partition locations are "<>ToString[p]]; *)

  partSpecs = Total[p[[;;#]]]& /@ Range[Length[p]];
  (* Print["Part specifications are "<>ToString[partSpecs]]; *)

  PrependTo[partSpecs, 0]; AppendTo[partSpecs, "Length[webfiles]"];
  partSpecs = Partition[partSpecs, 2, 1]; partSpecs[[All, 1]] = partSpecs[[All, 1]] + 1;

  (* Print["Numbers for partitioning in MakeMathematicaHTML.xml:"]; *)
  (* Print[ToString[partSpecs]<>"\n"]; *)

  Print["Each of the 8 parts should take about "<>ToString[binSize]<>" seconds = "<>ToString[compoundDuration[IntegerPart[binSize]]]<>", depending on build machine conditions."];

  partSpecs

]& /@ {build1List, build2List};

Print[""];
Print["First half of webfiles is webfiles[[;;"<>ToString[mainSplitPoint, InputForm]<>"]]"];
Print["Second half of webfiles is webfiles[["<>ToString[mainSplitPoint, InputForm]<>";;]]"];
Print[""];
Print["Partitionings for use in MakeMathematicaHTML.xml:"];
Print[ToString[#, InputForm]]& /@ partitionLists

Quit[]

----------------------------------------------------------------------

There are cleverer ways to split a list of integers into n parts so that the
Total of each is close to a given number.  See these for background:
 - https://mathematica.stackexchange.com/questions/7511/partitioning-with-varying-partition-size
 - https://mathematica.stackexchange.com/questions/66815/partition-list-into-a-given-number-of-sub-lists
 - https://mathematica.stackexchange.com/questions/193570/divide-a-list-into-two-lists-with-equal-sums
 - https://mathematica.stackexchange.com/questions/141173/partition-list-into-n-sublists
 - https://mathematica.stackexchange.com/questions/34952/how-to-partition-a-list-of-numbers-in-two-lists-of-numbers-such-that-each-lists

list = RandomInteger[{1,10}, 100];
(* Adding the mean here allows 7 of the 8 buckets to be a bit larger than needed
so we know bucket 8 will be a bit smaller.  Spread the fuzziness over 7 buckets
instead of putting it all in the last one. *)
target = Total[list]/8. + Mean[list];
res = {}; i = 1;
While[
  Total[list] > target
, Print[i++];
  bucket = Length @ Select[Accumulate[list], # <= target&];
  AppendTo[res, bucket];
  list = Drop[list, bucket]
];
res

list = RandomInteger[{1,10}, 100];
Ordering[Abs[(Total[list] + list)/2. - Accumulate[list]], 1]