#!/usr/bin/env bash

# TODO
#  - look into capturing the output of 'rsync language cloudplatform' to
#    process only the files that have changed - see --itemize-changes as
#    mentioned in http://stackoverflow.com/a/13536737/202280

# usage:
#  - 'cloudconvert.sh' will operate on $defaultdir

logfile=/www/sites/reference.wolfram.com/htdocs/temp/00-cloudconvert.log

htdocs=/www/sites/reference.wolfram.com/htdocs
# cloud=$htdocs/cloudplatform/ChatTools
# cloud=$htdocs/cloudplatform/CloudConnectorForExcel
# cloud=$htdocs/cloudplatform
cloud=$htdocs/tmp

# convert a single file
cloudconvert () {
    perl -i -pe 'BEGIN{undef $/;}

        s|/language-assets/|/cloud-assets/|smg;
        s|<!--#include virtual="/language-assets/inc/head-inc.html.en"-->||smg;
        s|<!--#include virtual="/language-assets/inc/head-inc.html.ja"-->||smg;
        s|<!--#include virtual="/language-assets/inc/head-inc.html.zh"-->||smg;

        s|href="/language/|href="/cloudplatform/|smg;
        s|href=\x27/language/|href=\x27/cloudplatform/|smg;

        s|src="http://reference\.wolfram\.com/chars|src="/chars|smg;
        s|src="http://reference|src="//reference|smg;
        s|href="http|target="_blank"\ href="http|smg;

        s|<!DOCTYPE html>.*<!--#include virtual="/common/framework/inc/head.(..).php"-->|<!--#include virtual="/cloud-assets/inc/doc-beginning.html.\1"-->|smg;

    ' "$1" # quoted to ensure correct handling of filenames with spaces

}

indexconvert () {
    perl -i -pe 'BEGIN{undef $/;}

        s|/language-assets/|/cloud-assets/|smg;
        s|<!DOCTYPE html>|<!--#include virtual="/cloud-assets/inc/doc-beginning.html.en"-->|smg;
        s|<html lang="en" class="_footer-thin _header-thin-only">||smg;
        s|<html lang="ja" class="_footer-thin _header-thin-only">||smg;
        s|<html lang="zh" class="_footer-thin _header-thin-only">||smg;
        s|<head>||smg;
        s|<!--#include virtual="/common/framework/inc/head.en.php"-->||smg;
        s|<!--#include virtual="/common/framework/inc/head.ja.php"-->||smg;
        s|<!--#include virtual="/common/framework/inc/head.zh.php"-->||smg;
        s|<!--#include virtual="/common/framework/inc/footer.en.php"-->||smg;
        s|<!--#include virtual="/common/framework/inc/footer.ja.php"-->||smg;
        s|<!--#include virtual="/common/framework/inc/footer.zh.php"-->||smg;

        s|href="/language/|href="/cloudplatform/|smg;
        s|href=\x27/language/|href=\x27/cloudplatform/|smg;

        s|src="http://reference\.wolfram\.com/chars|src="/chars|smg;
        s|src="http://reference|src="//reference|smg;

        # add to all, then remove selectively
        s|href="http|target="_blank"\ href="http|smg;
        s|target="_blank" href="/cloudplatform/guide/AlphabeticalListing"|href="/cloudplatform/guide/AlphabeticalListing"|smg;

    ' "$1" # quoted to ensure correct handling of filenames with spaces
}

# s|href=\x27/language/|href=\x27/cloudplatform/|smg;
# s|<!--#include virtual="/common/includes/gl-header.html"-->||smg;
# s|<!--#include virtual="/common/includes/gl-footer.html"-->||smg;
# s|<!--#include virtual="/common/includes/gl-header.html.en"-->||smg;
# s|<!--#include virtual="/common/includes/gl-footer.html.en"-->||smg;

# below, a substitution operator is used to return a default value if
# the variable is undefined. Specifically, the logic is "If $1 exists
# and isn't null, return its value; otherwise, return foo." The colon
# is optional. If it's omitted, change "exists and isn't null" to only
# "exists." The minus sign specifies to return foo without setting $1
# equal to 'foo'. Substitution operators are a subclass of expansion
# operators. See section 6.1.2.1 of Robbins and Beebe's Classic Shell
# Scripting [O'Reilly].

dir=${1:-$defaultdir}

echo "Convert the main index files before doing anything else..."
indexconvert $cloud/index.html.en
indexconvert $cloud/index.html.ja
indexconvert $cloud/index.html.zh
# these three removed from web https://bugs.wolfram.com/show?number=375769
# indexconvert $cloud/guide/WolframRoot.html.en
# indexconvert $cloud/guide/WolframRoot.html.ja
# indexconvert $cloud/guide/WolframRoot.html.zh
echo "They're converted.  Moving on to the rest of the files..."

echo "Finding all the html files under $cloud..." | tee $logfile

find $cloud -type f \
     -name "*.html*" \
     -and -not -name 'index.html.en' \
     -and -not -name 'index.html.ja' \
     -and -not -name 'index.html.zh' \
     -print > /www/sites/reference.wolfram.com/htdocs/temp/01-cloudconvert-find.txt

echo "Sorting the list of target files..." | tee -a $logfile
cat /www/sites/reference.wolfram.com/htdocs/temp/01-cloudconvert-find.txt | sort > /www/sites/reference.wolfram.com/htdocs/temp/02-cloudconvert-sorted.txt

echo "Counting the number of target files..." | tee -a $logfile
length=$(wc -l /www/sites/reference.wolfram.com/htdocs/temp/02-cloudconvert-sorted.txt | awk {'print $1'})
step=$(($length / 100))
counter=0

echo "There are $length files to process." | tee -a $logfile

# process the files in the array and report the results
echo "Processing the files..." | tee -a $logfile

while read -r file; do

    # # poor man's progress meter
    # ((counter++))
    # remainder=$(($counter % $step))
    # if [ "$remainder" -eq 0 ]
    # then
    #     # +0.5 for a cheap ceiling effect with positive numbers since awk's int truncates toward zero and awk has no ceiling fn
    #     echo $counter $length | awk '{print int($1/$2 * 100 + 0.5) "% done..."}' | tee -a $logfile
    # fi
    # # end poor man's progress meter

    cloudconvert "$file"

done < <(cat /www/sites/reference.wolfram.com/htdocs/temp/02-cloudconvert-sorted.txt)

echo "done processing the html files under $cloud" | tee -a $logfile
echo "Log is in $logfile"
