#!/bin/bash

# for f in $(find ./language/guide -type f -name 'An*.html.en' -print); do diff $f $(echo $f | sed 's|/language/|/tmp/|g';) | grep "^>" | grep "<sup>"; done

# grep two directories that contain the same filenames, *while ignoring parts of the files*

# For example, compare two consecutive runs of the pricing script,
# which (at the moment) writes 296 files to a directory.

# ----------------------------------------------------------------------
# Pat User can edit stuff here as needed

echo "Files in htdocs/tmp that differ significantly from files in htdocs/language"
echo "--------------------------------------------------------------------------------"

language=en
dir1="/www/sites/reference.wolfram.com/htdocs/language"
dir2="/www/sites/reference.wolfram.com/htdocs/tmp"
outputFile=01.diff
COUNT=0

# I don't need to report diffs in the timestamps and such - of course
# they differ and I don't care.  called via 'grep -vE "$ignorePattern"'

# ignorePattern='.*(Last updated|copyright|^.*<p><em>.*|rowspan|colspan|<th>).*'
ignorePattern='.*(buildnumber|height=|width=|Files/|class="IFL").*'

# ----------------------------------------------------------------------
# but here be <("$dragons") "${great[@]}" and "$small"

# empty the output file
echo "" > "$outputFile"

for file in $(find $dir1 -type f -name '*.html*' -print | LC_COLLATE=C sort | grep "\.$language");
do
    file1="$file"
    file2="${file1/$dir1/$dir2}"

    # if file2 (the computed name) does not exist, then 'continue' to go back to the top of the loop
    if [ ! -e "$file2" ] ; then
        echo "$file2 does not exist" >> "$outputFile"
        continue ;
    fi

    # diff the files while niftily ignoring lines that generate spurious diffs
    # DIFF=$(diff -Bb <(grep -vE "$ignorePattern" "$file1") <(grep -vE "$ignorePattern" "$file2") | grep "^>" | grep "<sup>")
    DIFF=$(diff -Bb <(grep -vE "$ignorePattern" "$file1") <(grep -vE "$ignorePattern" "$file2"))
    if [ "$DIFF" != "" ]
    then
        # print a banner for every difference found
        echo ""
        echo "(ediff-files \"/ssh:webdev4:$file1\" \"/ssh:webdev4:$file2\")"
        echo "diff -Bbu $file1 $file2" >> "$outputFile"
        echo "$DIFF" >> "$outputFile"
        # COUNT=$(expr $COUNT + 1)
        # echo $COUNT ${file2/$dir2/http://reference.devel.wolfram.com/tmp}
    fi
done

echo "output written to $outputFile"
# cat "$outputFile"

# quoth man diff:
#        -B  --ignore-blank-lines
#               Ignore changes whose lines are all blank.
#        -b  --ignore-space-change
#               Ignore changes in the amount of white space.
#        -u  -U NUM  --unified[=NUM]
#               Output NUM (default 3) lines of unified context.
