#!/usr/local/bin/bash

logfile=/www/sites/reference.wolfram.com/htdocs/webm-00-conversion.log
findlog=/www/sites/reference.wolfram.com/htdocs/webm-01-find.log
sortlog=/www/sites/reference.wolfram.com/htdocs/webm-02-sort.log

defaultdir=/www/sites/reference.wolfram.com/htdocs/webMathematica/

# convert a single file
webmconvert () {
    # in the following patterns, \x27 is ascii single-quote
    perl -i -pe 'BEGIN{undef $/;}

        # remove control-m (decimal 13, octal 15)
        # ^M interferes with the other replacements in http://reference.devel2.wolfram.com/webMathematica/ref/config/URLPattern.html
        s|\015||smg;

        s|<span class="inlineSeparator"><img src="../../images/mathematicaImages/bullet.gif" style="vertical-align:middle;" width="4" height="4" alt="bullet" title="bullet" /></span>|<span class="inlineBullet">&nbsp;&#9642;&nbsp;</span>|smg;
        s|<span class="inlineSeparator"><img src="../images/mathematicaImages/bullet.gif" style="vertical-align:middle;" width="4" height="4" alt="bullet" title="bullet" /></span>|<span class="inlineBullet">&nbsp;&#9642;&nbsp;</span>|smg;

        # various fixes to make the webM files behave like language files
        s|<!DOCTYPE html>\s+?<html>\s+?<head>\s+?|<!--#include virtual="/2013/includes/global-head1.html"-->\n|smg;

        s|<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />.+?"../javascript/carousel.js"></script>|<!--#include virtual="/common/includes/gl-head-includes.html"-->\n<!--#include virtual="/2013/includes/global-head2.html"-->|smg;

        s|<div class="main-wrapper">|<div class="main-wrapper">\n<!--#include virtual="/2013/includes/reference-header.html"-->|smg;

        s|<!-- /wrapper -->|<!-- /wrapper -->\n\n<!--#include virtual="/2013/includes/reference-footer.html"-->	\n</div>\n<!--#include virtual="/common/includes/gl-footer.html"-->\n<!--#include virtual="/common/includes/gl-footer-includes.html"-->\n<!--#include virtual="/2013/includes/global-footer.html"-->|smg;

        # ref/config pages
        s|<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
		 <link rel="stylesheet" type="text/css" href="../../css/reference.css" />
		 <script language="javascript" type="text/javascript" src="../../javascript/jquery/core/1.6.1/jquery.js"></script>
		 <script language="javascript" type="text/javascript" src="../../javascript/jquery/plugins/hoverIntent/r5/jquery.hoverIntent.js"></script>
		 <script language="javascript" type="text/javascript" src="../../javascript/reference.js"></script>
		 <script language="javascript" type="text/javascript" src="../../javascript/documentation-search.js"></script>
		 <script language="javascript" type="text/javascript" src="../../javascript/inputs.js"></script>
		 <script language="javaScript" type="text/javascript" src="../../javascript/faster-page-load.js"></script>
		 <script language="javascript" type="text/javascript" src="../../javascript/jquery/plugins/jcarousel/tinycarousel/tinycarousel.js"></script>
		 <script language="javascript" type="text/javascript" src="../../javascript/jquery.dotdotdot-1.5.1.js"></script>
		 <script language="javascript" type="text/javascript" src="../../javascript/carousel.js"></script>
<script language="JavaScript" type="text/javascript" src="../../javascript/toggle-highlight.js"></script>
|<!--#include virtual="/common/includes/gl-head-includes.html"-->
<!--#include virtual="/2013/includes/global-head2.html"-->
<script language="JavaScript" type="text/javascript" src="/2013/javascript/toggle-highlight.js"></script>
|smg;

    ' "$1"
}

# below, a substitution operator is used to return a default value if
# the variable is undefined. Specifically, the logic is "If $1 exists
# and isn't null, return its value; otherwise, return foo." The colon
# is optional. If it's omitted, change "exists and isn't null" to only
# "exists." The minus sign specifies to return foo without setting $1
# equal to 'foo'. Substitution operators are a subclass of expansion
# operators. See section 6.1.2.1 of Robbins and Beebe's Classic Shell
# Scripting [O'Reilly].

dir=${1:-$defaultdir}

echo "Starting the conversion of a webMathematica directory."

echo "Finding all the eligible files under $dir..." | tee $logfile

find $dir -type f \( -name "*.html" -o -name "*.css" -o -name "*.cgi" -o -name "*.js" -o -name "*.json" \) -print  > $findlog

echo "Sorting the list of target files..." | tee -a $logfile
cat $findlog | sort > $sortlog

echo "Counting the number of target files..." | tee -a $logfile
length=$(wc -l $sortlog | awk {'print $1'})
step=$(($length / 100))
counter=0

echo "There are $length files to process." | tee -a $logfile

# process the files in the array and report the results
echo "Processing the files..." | tee -a $logfile

while read -r file; do

    # poor man's progress meter
    ((counter++))
    remainder=$(($counter % $step))
    if [ "$remainder" -eq 0 ]
    then
        result=$(echo "scale = 2; 100 * $counter / $length" | bc)
        # echo "$(printf "%.0f" $result)% done, processing $file and $step others..." | tee -a $logfile
        echo "$(printf "%.0f" $result)% done..." | tee -a $logfile
    fi
    # end poor man's progress meter

    webmconvert "$file"

done < <(cat $sortlog)

echo "Done processing the html files under $dir" | tee -a $logfile
echo "Log is in $logfile"
