#!/usr/bin/env bash

# TODO
#  - look into capturing the output of 'rsync language cloudplatform' to
#    process only the files that have changed - see --itemize-changes as
#    mentioned in http://stackoverflow.com/a/13536737/202280

# usage:
#  - 'cloudconvert.sh' will operate on $defaultdir
#  - 'cloudconvert.sh /path/to/dir' will operate on /path/to/dir

logfile=/www/sites/reference.wolfram.com/htdocs/temp/00-cloudconvert.log

htdocs=/www/sites/reference.wolfram.com/htdocs
cloud=$htdocs/cloudplatform

# convert a single file
cloudconvert () {
    perl -i -pe 'BEGIN{undef $/;}

        s|/language-assets/|/cloud-assets/|smg;
        s|<!--#include virtual="/language-assets/inc/head-inc.html.en"-->||smg;
        s|<!--#include virtual="/language-assets/inc/head-inc.html.ja"-->||smg;
        s|<!--#include virtual="/language-assets/inc/head-inc.html.zh"-->||smg;

        s|href="/language/|href="/cloudplatform/|smg;
        s|href=\x27/language/|href=\x27/cloudplatform/|smg;

        s|src="http://reference\.wolfram\.com/chars|src="/chars|smg;
        s|src="http://reference|src="//reference|smg;
        s|href="http|target="_blank"\ href="http|smg;

    ' "$1" # quoted to ensure correct handling of filenames with spaces
}

# s|href=\x27/language/|href=\x27/cloudplatform/|smg;
# s|<!--#include virtual="/common/includes/gl-header.html"-->||smg;
# s|<!--#include virtual="/common/includes/gl-footer.html"-->||smg;
# s|<!--#include virtual="/common/includes/gl-header.html.en"-->||smg;
# s|<!--#include virtual="/common/includes/gl-footer.html.en"-->||smg;

# below, a substitution operator is used to return a default value if
# the variable is undefined. Specifically, the logic is "If $1 exists
# and isn't null, return its value; otherwise, return foo." The colon
# is optional. If it's omitted, change "exists and isn't null" to only
# "exists." The minus sign specifies to return foo without setting $1
# equal to 'foo'. Substitution operators are a subclass of expansion
# operators. See section 6.1.2.1 of Robbins and Beebe's Classic Shell
# Scripting [O'Reilly].

dir=${1:-$defaultdir}

echo "Convert the main index files before doing anything else..."
cloudconvert $cloud/index.html.en
cloudconvert $cloud/index.html.ja
cloudconvert $cloud/index.html.zh
echo "They're converted.  Moving on to the rest of the files..."

echo "Finding all the html files under $cloud..." | tee $logfile

find $cloud -type f -name "*.html*" -print > /www/sites/reference.wolfram.com/htdocs/temp/01-cloudconvert-find.txt

echo "Sorting the list of target files..." | tee -a $logfile
cat /www/sites/reference.wolfram.com/htdocs/temp/01-cloudconvert-find.txt | sort > /www/sites/reference.wolfram.com/htdocs/temp/02-cloudconvert-sorted.txt

echo "Counting the number of target files..." | tee -a $logfile
length=$(wc -l /www/sites/reference.wolfram.com/htdocs/temp/02-cloudconvert-sorted.txt | awk {'print $1'})
step=$(($length / 100))
counter=0

echo "There are $length files to process." | tee -a $logfile

# process the files in the array and report the results
echo "Processing the files..." | tee -a $logfile

while read -r file; do

    ######################################################################
    ##
    ## report % progress through the files
    ##

    # poor man's progress meter
    # ((counter++))
    # remainder=$(($counter % $step))
    # if [ "$remainder" -eq 0 ]
    # then
    ## argh - bc is not available on centos 6.6
    #     # result=$(echo "scale = 2; 100 * $counter / $length" | bc)
    #     # echo "$(printf "%.0f" $result)% done, processing $file and $step others..." | tee -a $logfile
    #     # echo "$(printf "%.0f" $result)% done..." | tee -a $logfile
    # fi
    # end poor man's progress meter

    ######################################################################
    ##
    ## modify the current file
    ##

    # echo "processing $file" | tee -a $logfile

    # in the following patterns, \x27 is ascii single-quote
    cloudconvert "$file"

done < <(cat /www/sites/reference.wolfram.com/htdocs/temp/02-cloudconvert-sorted.txt)

echo "done processing the html files under $dir" | tee -a $logfile
echo "Results are in $resfile"
echo "Log is in $logfile"
