#!/usr/bin/env bash

DRY_RUN="no"

usage()
{
    echo "pull webtst4:language.en to webdev4:language.en via rsync"
    echo ""
    echo "/www/sites/reference.wolfram.com/bin/scripts/pull-webtst4.language.en-webdev4.language.en.sh"
    echo "  -t, --test: do an rsync dry run - don't actually copy anything, just report what would be done."
    echo "  -h, --help: print a help statement"
    echo ""
}

while [ "$1" != "" ]; do
    PARAM=`echo $1 | awk -F= '{print $1}'`
    case $PARAM in
        -t | --test)
            DRY_RUN="yes"
            ;;
        -h | --help)
            usage
            exit
            ;;
        *)
            echo "ERROR: unknown parameter \"$PARAM\""
            usage
            exit 1
            ;;
    esac
    shift
done


if [ "$DRY_RUN" = "no" ]; then
    echo "live command is running - files may be changed"
    echo ""
    RSYNC_ARGS="-av"
else
    echo "DRY RUN - no files will be changed"
    echo ""
    RSYNC_ARGS="-avn"
fi

# --------------------------------------------------------------------------------
echo "rsync webtst4:language to webdev4:language without deleting anything"
time rsync $RSYNC_ARGS --rsh ssh \
     --exclude 'sitemap.xml' \
     --exclude 'robots.txt' \
     \
     --include '*.html.en' \
     --include 'Files/*.en/***' \
     --include 'version.en.json' \
     \
     --exclude '*.sh' \
     \
     --exclude '*.html.ja' \
     --exclude 'Files/*.ja' \
     --exclude 'version.ja.json' \
     \
     --exclude '*.html.zh' \
     --exclude 'Files/*.zh' \
     --exclude 'version.zh.json' \
     \
     /www/sites/reference.wolfram.com/htdocs/language/ webdev4.wolfram.com:/www/sites/reference.wolfram.com/htdocs/language/
echo "rsync webtst4:language to webdev4:language... done."

if [ "$DRY_RUN" = "no" ]; then
    echo ""
    echo "live command ran - files may have been changed"

else
    echo ""
    echo "DRY RUN - no files were changed"
fi


echo ""

