#!/usr/bin/env bash

## https://stackoverflow.com/a/8597411/202280
if [[ "$OSTYPE" == "darwin"* ]]; then
    PATH=/usr/local/opt/gnu-getopt/bin/:$PATH
fi

## http://www.bahmanm.com/blogs/command-line-options-how-to-parse-in-bash-using-getopt
# -t and --test make rsync do a dry run.  they are optional, synonymous, and they take no arguments

# set an initial value for the flag
ARG_TEST=0

# read the options
TEMP=`getopt -o t --long test -n 'test.sh' -- "$@"`
eval set -- "$TEMP"

# extract options and their arguments into variables.
while true ; do
    case "$1" in
        -t|--test) ARG_TEST=1 ; shift ;;
        --) shift ; break ;;
        *) echo "Internal error!" ; exit 1 ;;
    esac
done

if [ "$ARG_TEST" = 0 ]; then
    echo "live command is running - files may be changed"
    echo ""
    RSYNC_ARGS="-av"
else
    echo "DRY RUN - no files will be changed"
    echo ""
    RSYNC_ARGS="-avn"
fi

# --------------------------------------------------------------------------------
echo "rsync webdev4:cloud-assets to webtst4:cloud-assets ..."
time rsync $RSYNC_ARGS --rsh ssh --delete \
     webdev4.wolfram.com:/www/sites/reference.wolfram.com/htdocs/cloud-assets/ /www/sites/reference.wolfram.com/htdocs/cloud-assets/
echo "rsync webdev4:cloud-assets to webtst4:cloud-assets ... done."

# --------------------------------------------------------------------------------
if [ "$ARG_TEST" = 0 ]; then
    echo ""
    echo "live command ran - files may have been changed"

else
    echo ""
    echo "DRY RUN - no files were changed"
fi


echo ""

