#!/usr/bin/env bash

# rewrite like this:
# COPY="no"
# MAKECLOUD="no"

# usage()
# {
#     echo "if this was a real script you would see something useful here"
#     echo ""
#     echo "./go"
#     echo "\t-m, --makecloud: after building, make a cloud version of the pages"
#     echo "\t-c, --copy: after building & possible cloud conversion, copy to webdev2"
#     echo "\t-h, --help: print a help statement"
#     echo ""
# }

# while [ "$1" != "" ]; do
#     PARAM=`echo $1 | awk -F= '{print $1}'`
#     case $PARAM in
#         -c | --copy)
#             COPY="yes"
#             ;;
#         -m | --makecloud)
#             MAKECLOUD="yes"
#             ;;
#         -h | --help)
#             usage
#             exit
#             ;;
#         *)
#             echo "ERROR: unknown parameter \"$PARAM\""
#             usage
#             exit 1
#             ;;
#     esac
#     shift
# done


## https://stackoverflow.com/a/8597411/202280
if [[ "$OSTYPE" == "darwin"* ]]; then
    PATH=/usr/local/opt/gnu-getopt/bin/:$PATH
fi

## http://www.bahmanm.com/blogs/command-line-options-how-to-parse-in-bash-using-getopt
# -t and --test are optional, synonymous, and they take no arguments

# set an initial value for the flag
ARG_TEST=0

# read the options
TEMP=`getopt -o t --long test -n 'test.sh' -- "$@"`
eval set -- "$TEMP"

# extract options and their arguments into variables.
while true ; do
    case "$1" in
        -t|--test) ARG_TEST=1 ; shift ;;
        --) shift ; break ;;
        *) echo "Internal error!" ; exit 1 ;;
    esac
done

if [ "$ARG_TEST" = 0 ]; then
    echo "live command is running - files may be changed"
    echo ""
    RSYNC_ARGS="-av"
else
    echo "DRY RUN - no files will be changed"
    echo ""
    RSYNC_ARGS="-avn"
fi

# --------------------------------------------------------------------------------
echo "STEP 1: rsync webdev4:cloudplatform.ja to webtst4:cloudplatform.ja ..."
time rsync $RSYNC_ARGS --rsh ssh --delete \
     \
     --exclude 'sitemap.xml' \
     --exclude 'robots.txt' \
     \
     --exclude '*.html.en' \
     --exclude 'Files/*.en/***' \
     \
     --exclude '*.sh' \
     \
     --include '*.html.ja' \
     --include 'Files/*.ja' \
     --include 'version.ja.json' \
     \
     --exclude '*.html.zh' \
     --exclude 'Files/*.zh' \
     --exclude 'version.zh.json' \
     \
     webdev4.wolfram.com:/www/sites/reference.wolfram.com/htdocs/cloudplatform/ /www/sites/reference.wolfram.com/htdocs/cloudplatform/
echo "STEP 1: rsync webdev4:cloudplatform to webtst4:cloudplatform... done."
# --------------------------------------------------------------------------------

# if [ "$ARG_TEST" = 0 ]; then
#     # --------------------------------------------------------------------------------
#     echo "STEP 2: remove workflows from root guide pages pushed from webdev4..."

#     # similarly for other languages if necessary (i.e., if workflow development extends into the period of ja and zh builds)
#     find /www/sites/reference.wolfram.com/htdocs/cloudplatform \
#          -maxdepth 1 \
#          -type f \
#          -name 'index.html.en' \
#          -print0 | xargs -0 \
#                          sed '/<h2>Workflows<\/h2>/,/<\/section>/d' > /tmp/index.tmp \
#         && mv /tmp/index.tmp /www/sites/reference.wolfram.com/htdocs/cloudplatform/index.html.en

#     find /www/sites/reference.wolfram.com/htdocs/cloudplatform \
#          -maxdepth 1 \
#          -type f \
#          -name 'index.html.ja' \
#          -print0 | xargs -0 \
#                          sed '/<h2>Workflows<\/h2>/,/<\/section>/d' > /tmp/index.tmp \
#         && mv /tmp/index.tmp /www/sites/reference.wolfram.com/htdocs/cloudplatform/index.html.ja
#     echo "STEP 2: remove workflows from root guide pages pushed from webdev4... done."
#     # --------------------------------------------------------------------------------
# fi

if [ "$ARG_TEST" = 0 ]; then
    echo ""
    echo "live command ran - files may have been changed"
else
    echo ""
    echo "DRY RUN - no files were changed"
fi


echo ""


