#!/usr/bin/env bash

DRY_RUN="no"

usage()
{
    echo "push webdev4:language.zh to webtst4:language.zh via rsync"
    echo ""
    echo "/www/sites/reference.wolfram.com/bin/scripts/push-webdev4.language.zh-webtst4.language.zh.sh"
    echo "  -t, --test: do an rsync dry run - don't actually copy anything, just report what would be done."
    echo "  -h, --help: print a help statement"
    echo ""
}

while [ "$1" != "" ]; do
    PARAM=`echo $1 | awk -F= '{print $1}'`
    case $PARAM in
        -t | --test)
            DRY_RUN="yes"
            ;;
        -h | --help)
            usage
            exit
            ;;
        *)
            echo "ERROR: unknown parameter \"$PARAM\""
            usage
            exit 1
            ;;
    esac
    shift
done


if [ "$DRY_RUN" = "no" ]; then
    echo "live command is running - files may be changed"
    echo ""
    RSYNC_ARGS="-av"
else
    echo "DRY RUN - no files will be changed"
    echo ""
    RSYNC_ARGS="-avn"
fi

# --------------------------------------------------------------------------------
echo "rsync webdev4:language to webtst4:language..."

time rsync $RSYNC_ARGS --rsh ssh --delete \
     --exclude 'sitemap.xml' \
     --exclude 'robots.txt' \
     \
     --exclude '*.sh' \
     \
     --include 'CloudConnectorForExcel' \
     \
     --include 'workflow' \
     --include 'workflowguide' \
     \
     --exclude '*.html.en' \
     --exclude 'Files/*.en/***' \
     --exclude 'version.en.json' \
     \
     --exclude '*.html.ja' \
     --exclude 'Files/*.ja' \
     --exclude 'version.ja.json' \
     \
     --include '*.html.zh' \
     --include 'Files/*.zh' \
     --include 'version.zh.json' \
     \
     webdev4.wolfram.com:/www/sites/reference.wolfram.com/htdocs/language/ /www/sites/reference.wolfram.com/htdocs/language/

echo "rsync webdev4:language.zh to webtst4:language.zh ... done."
# --------------------------------------------------------------------------------

# the following was used to remove workflow tiles from reference.test.w.c, which
# someone required sometime.  there's no need for this now since we're actually
# testing them.

# if [ "$ARG_TEST" = 0 ]; then
#     # --------------------------------------------------------------------------------
#     echo "STEP 2: remove workflows from root guide pages pushed from webdev4..."

#     # similarly for other languages if necessary (i.e., if workflow development extends into the period of ja and zh builds)
#     find /www/sites/reference.wolfram.com/htdocs/language \
#          -maxdepth 1 \
#          -type f \
#          -name 'index.html.en' \
#          -print0 | xargs -0 \
#                          sed '/<h2>Workflows<\/h2>/,/<\/section>/d' > /tmp/index.tmp \
#         && mv /tmp/index.tmp /www/sites/reference.wolfram.com/htdocs/language/index.html.en

#     echo "STEP 2: remove workflows from root guide pages pushed from webdev4... done."
#     # --------------------------------------------------------------------------------
# fi

if [ "$DRY_RUN" = "no" ]; then
    echo ""
    echo "live command ran - files may have been changed"

else
    echo ""
    echo "DRY RUN - no files were changed"
fi


echo ""

