#!/Applications/Mathematica/M-OSX-L-11.3.0-5913716-20180117.01.app/Contents/MacOS/WolframScript -script
(* -*-mode:mma;-*- *)

(* carlson: "These workflows are implemented. Anything NOT on this list will be a dead link in the build".  
also from carlson: the list of implemented workflows is maintained in WorkflowAgenda.nb

so we have this:
- guides should link only to completed workflows
- the list of completed workflows is in Pubs/Documentation/English/System/WorkflowGuides/WorkflowAgenda.nb
- the pattern for these in WorkflowAgenda.nb is Cell[s:_String, rest:___] /; MemberQ[{rest}, "\[Checkmark]", Infinity]
- the text of each entry is a string that may match a workflow title, or may merely be reminiscent of a title.  they aren't strictly maintained.
- so I wrote a script that builds a list of completed workflows from WorkflowAgenda.nb and educated guesses.  i could probably do something with EditDistance, but whatever. *)

(* associate WorkflowAgenda's titles with actual workflow notebooks.  Sometimes the
agenda's titles aren't correct. *)

implementedWorkflows = Module[
  {
    cvsWorkflows, cvsWorkflowGuides, realTitleToFilenameDB, agenda, aExpr,
    agendaTitles, realTitleToFilenameDBLowerCase
  },

  cvsWorkflows = "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/";
  cvsWorkflowGuides = "~/wri/ref/cvs/Pubs/Documentation/English/System/WorkflowGuides/";

  Print["building realTitleToFilenameDB"];
  realTitleToFilenameDB = Reap[
    Module[
      {f, e, t}
    ,
      f = #;
      e = Quiet@Get@f;
      t = Cases[e, Cell[s:_String, "WorkflowTitle" ,___] :> StringReplace[s, Whitespace -> " "], Infinity];
      Sow[f, t];
    ]& /@ FileNames["*.nb",cvsWorkflows,1], _, Rule][[2]];
  realTitleToFilenameDB[[All, 2]] = First /@ realTitleToFilenameDB[[All, 2]];

  AppendTo[realTitleToFilenameDB, {
    (* title in file: "Call External .NET Code" *)
    "Connect to an External .NET Program" ->
      "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/ConnectToAnExternalNETProgram.nb",
  
    (* title in file: "Display Notebooks White on Black" *)
    "Display Notebooks White-on-Black" ->
      "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/DisplayNotebooksWhiteOnBlack.nb",
  
    (* title in file: "Group and Ungroup Cells Manually " (note trailing space) *)
    "Group and Ungroup Cells Manually" ->
      "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/GroupAndUngroupCellsManually.nb",
  
    (* title in file: "Install Wolfram Enterprise Private Cloud on Oracle VirtualBox" *)
    "Install Wolfram Enterprise Private Cloud on Oracle Virtual Box" ->
      "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/InstallWolframEnterprisePrivateCloudOnOracleVirtualBox.nb",
  
    (* title in file: "Limit the Amount of Session History " *)
    "Limit the Amount of History Stored in the System" ->
      "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/LimitTheAmountOfHistoryStoredInTheSystem.nb",
  
    (* title in file: "Rotate, Pan and Zoom 3D Graphics" *)
    "Rotate, Pan, and Zoom 3D Graphics" ->
      "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/RotatePanAndZoom3DGraphics.nb",
  
    (* title in file: "Run a Parallel Computation" *)
    "Run a Computation in Parallel" ->
      "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/RunAComputationInParallel.nb",
  
    (* title in file: "Set Up Error Checking and Messages in a Function" *)
    "Set Up Error-Checking and Messages in a Function" ->
      "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/SetUpErrorCheckingAndMessagesInAFunction.nb",
  
    (* title in file: "Share a Cloud Notebook" *)
    "Share a Cloud Notebook <Share a Cloud Notebook with Others>" ->
      "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/ShareACloudNotebookWithOthers.nb",

    (* title in file: "Share a Cloud Notebook with Others" *)
    "Share a Cloud Notebook" -> (* title as given incorrectly in WorkflowAgenda.nb: *)
    "/Users/billw/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/ShareACloudNotebookWithOthers.nb"
  }];

  realTitleToFilenameDB = Flatten[realTitleToFilenameDB];

  agenda = FileNameJoin[{cvsWorkflowGuides, "WorkflowAgenda.nb"}];
  aExpr = Quiet@Get@agenda;
  agendaTitles = Cases[aExpr, Cell[s:_String, rest:___] /; MemberQ[{rest}, "\[Checkmark]", Infinity] -> s, Infinity];
  realTitleToFilenameDBLowerCase = realTitleToFilenameDB;
  realTitleToFilenameDBLowerCase[[All, 1]] = ToLowerCase[realTitleToFilenameDBLowerCase[[All, 1]]];

  Print["matching agenda titles to real titles, reporting ones that don't match"];
  Module[{title, nb}, Catch[
    title = #;
    nb = ToLowerCase[title] /. realTitleToFilenameDBLowerCase;
    If[
      Not[FileExistsQ[nb]]
    ,
      Print["no file in ~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/ for: "<>title]
    ,
      FileBaseName[nb]
    ]
  ]]& /@ agendaTitles
];

outfileBase = "implementedWorkflows";
Print["Exporting filenames of approved workflows to "
  <> outfileBase<>".m"
  <> " and "
  <> outfileBase<>"-lines.m"];

Export[outfileBase<>".m", implementedWorkflows];
Export[outfileBase<>"-lines.m", implementedWorkflows, "Lines"];

Print["use this list to update Export.m:implementedWorkflows"];

Quit[]

this was printed:

no file for: Connect to an External .NET Program
no file for: Display Notebooks White-on-Black
no file for: Group and Ungroup Cells Manually
no file for: Install Wolfram Enterprise Private Cloud on Oracle Virtual Box
no file for: Limit the Amount of History Stored in the System
no file for: Rotate, Pan, and Zoom 3D Graphics
no file for: Run a Computation in Parallel
no file for: Set Up Error-Checking and Messages in a Function
no file for: Share a Cloud Notebook <Share a Cloud Notebook with Others>

so i need to find which workflows these inaccurate agenda titles might refer to

~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/


{

  (* title in file: "Call External .NET Code" *)
  "Connect to an External .NET Program" ->
    "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/ConnectToAnExternalNETProgram.nb",

  (* title in file: "Display Notebooks White on Black" *)
  "Display Notebooks White-on-Black" ->
    "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/DisplayNotebooksWhiteOnBlack.nb",

  (* title in file: "Group and Ungroup Cells Manually " (note trailing space) *)
  "Group and Ungroup Cells Manually" ->
    "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/GroupAndUngroupCellsManually.nb",

  (* title in file: "Install Wolfram Enterprise Private Cloud on Oracle VirtualBox" *)
  "Install Wolfram Enterprise Private Cloud on Oracle Virtual Box" ->
    "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/InstallWolframEnterprisePrivateCloudOnOracleVirtualBox.nb",

  (* title in file: "Limit the Amount of Session History " *)
  "Limit the Amount of History Stored in the System" ->
    "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/LimitTheAmountOfHistoryStoredInTheSystem.nb",

  (* title in file: "Rotate, Pan and Zoom 3D Graphics" *)
  "Rotate, Pan, and Zoom 3D Graphics" ->
    "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/RotatePanAndZoom3DGraphics.nb",

  (* title in file: "Run a Parallel Computation" *)
  "Run a Computation in Parallel" ->
    "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/RunAComputationInParallel.nb",

  (* title in file: "Set Up Error Checking and Messages in a Function" *)
  "Set Up Error-Checking and Messages in a Function" ->
    "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/SetUpErrorCheckingAndMessagesInAFunction.nb",

  (* title in file: "Share a Cloud Notebook" *)
  "Share a Cloud Notebook <Share a Cloud Notebook with Others>" ->
    "~/wri/ref/cvs/Pubs/Documentation/English/System/Workflows/ShareACloudNotebookWithOthers.nb"
};
