Print["======================================================================"];
Print["Entering FeaturesetPages1.m"];
(* * FeaturesetPages1.m: general prep, assemble AntSetProperty["notebooks", files] *)
(* ** misc settings *)

(* full messages *)
$MessagePrePrint = .;
SetDirectory[AntProperty["basedir"]];    

(* Configure Front End *)
If[AntProperty["frontEndLaunchFlags"] =!= Null,
	SetOptions[ Developer`InstallFrontEnd, Developer`LaunchFlags -> AntProperty["frontEndLaunchFlags"]]
];

(* Add apps to $Path*)
If[
  !MemberQ[$Path, AntProperty["appPath"]]
, PrependTo[ $Path, AntProperty["appPath"]]
];

(* Make the locally checked-out DocumentationBuild override the DocumentationBuild
   which is included in 12.2 (via PacletTools). See bug #399891. *)
PacletDirectoryLoad[AntProperty["appPath"]];

(* ** User variables *)
inputDir = FileNameJoin[FileNameSplit[AbsoluteFileName[AntProperty["inputDir"]]]];
outputDir = FileNameJoin[FileNameSplit[AntProperty["outputDir"]]];
max = {0, "None"};
AntSetProperty["max", ToString@max];
lang = AntProperty["language"];
DocumentationBuild`ConversionList = {};

(* ** Load DocumentationBuild *)

Needs["DocumentationBuild`"];
DocumentationBuild`Transform`debug = ToExpression @ AntProperty["debug"];

(* ** Test Import/Export convertors *)

AntLog["_"];
AntLog["Functionality Tests"]; 

(* *** XML Import/Export Test *)

If[
  MatchQ[ImportString[ExportString["<tag>1</tag>", "XML"], "XML"], $Failed | Null]
, AntFail["Export convertor failure. Unable to Import/Export XML!"]
, AntLog["Passed XML Import/Export Test"]
];

(* *** FE Availability Test *)

If[
  Head[DocumentationBuild`Common`PubsEvaluateWithFE @ NotebookGet @ NotebookPut[Notebook[{Cell["test"]}]]] =!= Notebook
, AntFail["FrontEnd Not Available!"]
, AntLog["Passed FE Availability Test"]
];

(* ** FE settings using PubsEvaluateWithFE *)

(*
featuresetPageToolDir
DocumentationBuild`$FeaturesetStylePath
stylesheetpath *)

DocumentationBuild`Common`PubsEvaluateWithFE[ 
  SetOptions[$FrontEnd, DynamicEvaluationTimeout -> 300]; 
  Global`AntLog[ "DynamicEvaluationTimeout: " <> ToString@CurrentValue[$FrontEnd, DynamicEvaluationTimeout] ];

  featuresetPageToolDir = FileNameJoin[FileNameSplit[AntProperty["FeaturesetPageStylesheetPath"]]]; 
  Global`AntLog["featuresetPageToolDir = "<>ToString[featuresetPageToolDir, InputForm]];
    	
  SetOptions[$FrontEnd, StyleSheetPath -> Inherited];
  DocumentationBuild`$FeaturesetStylePath = With[
    {featuresetPageToolPath = FileNameJoin[FileNameSplit[featuresetPageToolDir]]}
  , FrontEnd`FileName[{ featuresetPageToolPath, "FrontEnd", "StyleSheets"}]
  ];
  Global`AntLog["DocumentationBuild`$FeaturesetStylePath = "<>ToString[DocumentationBuild`$FeaturesetStylePath, InputForm]];

  stylesheetpath = Prepend[ CurrentValue[$FrontEnd, StyleSheetPath], DocumentationBuild`$FeaturesetStylePath ];
  SetOptions[$FrontEnd, StyleSheetPath -> stylesheetpath];
  (* Global`AntLog[ "StyleSheetPath: " <> ToString@CurrentValue[$FrontEnd, StyleSheetPath] ];      	 *)
  If[
    Head[Get[ToFileName @@ With[
      {featuresetPageToolPath = FileNameJoin[FileNameSplit[featuresetPageToolDir]]}
    , FrontEnd`FileName[{featuresetPageToolPath, "FrontEnd", "StyleSheets", "Wolfram"}, "FeaturesetPageStyles.nb"]
    ]]] === Notebook
  , AntLog["Passed Stylesheet Availability Test"]
  , AntFail["Featureset Stylesheet Not Available!"]
  ]
];

(* ** log the values of other settings *)

AntLog["_"]; 
feMemory = ToString @ InputForm @ DocumentationBuild`Common`PubsEvaluateWithFE[MathLink`CallFrontEndHeld[FrontEnd`Value[FEPrivate`MemoryInUse]]];
AntSetProperty["FEMemory", feMemory];

AntLog @ StringJoin[
  "M- Version: ", ToString[Internal`$CreationID], " ", ToString@$Version, "\n", 
  "Current FE memory: ", feMemory, "\n",
  "$MaxLicenseProcesses: ", ToString[$MaxLicenseProcesses],  "\n",
  "Installation Directory: ", ToString@$InstallationDirectory, "\n",
  "DocumentationBuild Directory: ", DocumentationBuild`Common`$DocumentationBuildDirectory
];

(* ** select source notebooks: all available? recently modified? *)

singleFile = AntProperty["singleFile"];

Which[
  FileExistsQ[singleFile], files = {singleFile}
, True,
  (* Print["singleFile is not set or does not exist; finding the list of files to process..."]; *)

  (* inputDir is a local checkout of https://stash.wolfram.com/projects/PUBS/repos/featuresetpages/browse *)
  files = FileNames["*.nb", inputDir, 2]; (* select nbs from checkout/<language>/<featureset area> *)
  Print["1. files = "<>ToString[files, InputForm]];

  (* For featureset builds, consider nbs whose names begin with an uppercase letter or "z" (test files) *)
  files = Select[files, Or[
    UpperCaseQ[StringTake[FileBaseName[#], 1]],
    StringTake[FileBaseName[#], 1] == "z"
  ]&];
  Print["2. files = "<>ToString[files, InputForm]];

  (* don't build these pages: retain mechanism to be modified for featureset pages *)
  files = Select[files, !StringMatchQ[#, "*MarketingPageCategories.nb"]& ];
  files = Select[files, !StringMatchQ[#, "*BuiltInImageProcessing.nb"]& ];
  files = Select[files, !StringMatchQ[#, "*Manifest.nb"]& ];

  (* NOTE: featureset builds may honor notebook history data in the future *)
  (* files = Select[
    files
  , (DocumentationBuild`Make`Private`CheckVersion[
      ("New" /. GetNotebookHistoryData[GetQuietly[#]])
    , "ReleaseVersion" -> "12"
    , "ThresholdVersion" -> "11.3"
    ] > 0)&
  ]; *)

  (* NOTE: featureset builds honor notebook flags *)
  files = Select[
    files,
    (!MatchQ[ flag = DocumentFlag[ #] , "PreviewFlag" | "ExcisedFlag" | (* "FutureFlag" | *) "InternalFlag" | "TemporaryFlag"])&
  ];

  fileBasenames = Map[FileBaseName, files];

  AntSetProperty["numberTotalNotebooks", ToString@Length@files];
  AntLog["Number of notebooks to process: "<>ToString@Length@files];
  AntLog["_"];

  AntLog[files];
  If[
    ToExpression @ AntProperty["fileLimit"] =!= Null
  , files = Take[files, ToExpression @ AntProperty["fileLimit"]]
  ]; 

];

files = ToString@Sequence[files];
files = StringTake[StringReplace[files, RegularExpression["\\s*,\\s*"]->"," ], {2,-2}];
AntSetProperty["notebooks", files];
AntLog["AntProperty[\"notebooks\"] = "<>ToString[AntProperty["notebooks"], InputForm]];
AntLog["Files to process: "<>ToString[files]];

