(* * frontmatter *)

If[ToExpression[AntProperty["debug"]], debugOn[]];

dprint["======================================================================"];
dprint["Entering FeaturesetPages2.m"];

inFile = FileNameJoin[FileNameSplit[AbsoluteFileName[AntProperty["individualFilename"]]]]; Print["inFile = "<>ToString[inFile, OutputForm]];
dprint["inFile = "<>ToString[inFile, InputForm]];

inputDir = FileNameJoin[FileNameSplit[AbsoluteFileName[AntProperty["inputDir"]]]];
outputDir = FileNameJoin[FileNameSplit[AntProperty["outputDir"]]];
dprint["inputDir = "<>ToString[inputDir, InputForm]];
dprint["outputDir = "<>ToString[outputDir, InputForm]];

commonassetsDir = FileNameJoin[{inputDir, "_CommonAssets"}];
dprint["commonassetsDir = "<>ToString[commonassetsDir, InputForm]];

metadataFolder = FileNameJoin[{ParentDirectory[inputDir, 1], "MetadataFolder"}];
pageCategories = Get[FileNameJoin[{metadataFolder, "PageCategories.wl"}]];
pageColors = <|"Application" -> "yellow", "Core" -> "orange", "Data" -> "green", "Interfaces" -> "blue"|>;
pageColor = pageColors[pageCategories[FileBaseName[inFile]]] /. _Missing -> "orange";
dprint["pageColor = "<>ToString[pageColor, InputForm]];

placeholderExpr = Rasterize[Show[
  ImagePad[ConstantImage[GrayLevel[.6], {1100, 600} - 20], 10, GrayLevel[.4]],
  Graphics[Style[Text["Placeholder", {550, 300}], FontSize -> 50]]
], RasterSize -> 1100, ImageSize -> 1100];
(* debugOn[]; *)
dprint["placeholderExpr is:"]; dfile[placeholderExpr, "DeleteLargeItems" -> False];
(* debugOff[]; *)

(* Print["======================================================================"]; *)
(* Print["Entering FeaturesetPages2.m"]; *)

  (* the other possible value of "CellLabel" is "Notebook Archive - PRD" *)
  getNotebookArchiveLink[url_String] := Module[{examplenb, ex, link},
    dprint["entering getNotebookArchiveLink, arg 'url' is "<>ToString[url, InputForm]];
    examplenb = StringReplace[url, {
      "http://reference.wolfram.com/language/featureexample/" -> exampleCheckout,
      ".html" ~~ EndOfString -> ".nb"
    }];
    dprint["examplenb = "<>ToString[examplenb, InputForm]];

    ex = Get[examplenb];
    (* remove these in case an old or futurized notebookarchive.org link is among them *)
    ex = DeleteCases[ex, Cell[CellGroupData[{Cell[___, "FutureExample" | "TentativeExample" | "ExcludedMaterial", ___], ___}, ___], ___], Infinity];
    ex = DeleteCases[ex, Cell[___, "FutureExample" | "TentativeExample" | "ExcludedMaterial", ___], Infinity];
    ex = DeleteCases[ex, Cell[___, CellDingbat->Cell[__, "TentativeID" | "BrightFlag", ___], ___], Infinity];

    (* this must be the example's link to notebookarchive.org.  if none is provided by inheritance, then there is no link *)
    dprint["examplenb expr 'ex' = "]; dfile[ex];
    link = Quiet @ Check[
      First @ Cases[ex, Cell[TextData[ButtonBox[__, ButtonData -> {URL[s:_String], ___}]], "Categorization", ___, CellLabel -> "Notebook Archive - DVL", ___] -> s, Infinity], ""];
    dprint["final harvested link = "<>ToString[link, InputForm]];
    link
  ];

(* this is cheaper than IntegerName[4, "Ordinal"], which requires cloud access. *)
ordinal[n_Integer] := Switch[n,
  1, "first",
  2, "second",
  3, "third",
  4, "fourth",
  5, "fifth",
  6, "sixth",
  7, "seventh",
  8, "eighth",
  9, "ninth",
  10, "tenth",
  _, "implement ordinal["<>ToString[n]<>"]"];
  
dprint["ordinal[4] is "<>ToString[ordinal[4], InputForm]];

(* Print["individualFilename ant property = "<>ToString[AntProperty["individualFilename"], InputForm]]; *)

exampleCheckout = AntProperty["exampleCheckout"];
dprint["exampleCheckout = "<>ToString[exampleCheckout, InputForm]];

(* * MakeFeaturesetPageNotebook *)

MakeFeaturesetPageNotebook::usage = "MakeFeaturesetPageNotebook[nb] takes a production notebook and returns a massaged notebook ready for inclusion in product/on web.";

MakeFeaturesetPageNotebook[nb_String, language_String:"English", opts___?OptionQ ] := Module[
  {expr}
,
  dprint["entering _String _String, opts version of MakeFeaturesetPageNotebook"];
  dprint["  nb = "<>ToString[nb, InputForm]];
  dprint["  language = "<>ToString[language, InputForm]];
  expr = Get@nb;
  dprint["  Head[expr] = "<>ToString[Head[expr], InputForm]];
  MakeFeaturesetPageNotebook[expr, language, opts];
];
MakeFeaturesetPageNotebook[nbExpr_Notebook, language_String:"English", opts___?OptionQ ] := Module[
  {transform, expr=nbExpr, pageTitle = "None", keywordList = {}, seeAlsoGroup, poss, exampleCaption }
,
  dprint["entering _Notebook, _String, opts version of MakeFeaturesetPageNotebook"];
  dprint["earliest possible expr entering MakeFeaturesetPageNotebook: "]; dfile[expr];

(* ** remove unneeded/flagged items *)

  (* remove future/invisible items *)
  dprint[" - remove future/invisible items"];
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[___, "FutureExample" | "TentativeExample"(* | "ExcludedMaterial"*), ___], ___}, ___], ___], Infinity];
  expr = DeleteCases[expr, Cell[___, "FutureExample" | "TentativeExample" | "ExcludedMaterial", ___], Infinity];
  expr = DeleteCases[expr, Cell[___, CellDingbat->Cell[__, "TentativeID" | "BrightFlag", ___], ___], Infinity];

  dprint["after removing future/invisible items, expr = "]; dfile[expr];

  (* Remove empty cell groups (update for featuresets) *)
  dprint[" - Remove empty cell groups (update for featuresets)"];
  expr = RemoveEmptyCellGroups[expr, {
    "KeywordsSection", "DescriptionSection", "MarketingContentsSection"
  , "FeaturesetMoreAboutSection", "FeaturesetRelatedDemonstrationsSection"
  , "FeaturesetRelatedFunctionsSection", "FeaturesetSeeAlsoSection"
  , "FeaturesetSubpageMoreAboutSection", "FeaturesetSubpageRelatedDemonstrationsSection"
  , "FeaturesetSubpageRelatedFunctionsSection", "FeaturesetSubpageSection"
  , "FeaturesetSubpageSeeAlsoSection", "FeaturesetThumnailSection" , "Keywords"
  , "Description", "FeaturesetRelatedLinksSection", "DocumentStatusSection", "DocumentStatus", "ContributorSection"
  }];

dprint["MakeFeaturesetPageNotebook: after removing unneeded/flagged items, expr is "]; dfile[expr];

(* ** convert Function string to buttons *)

  dprint[" - convert Function string to buttons"];
  expr = expr /. Cell[c_String, "FeaturesetPageRelatedFunctions", o___] :>
    DocumentationBuild`Make`Private`inlineFunctionStringToButtonBox @ Cell[c, "FeaturesetPageRelatedFunctions", o];

(* ** reorganize SeeAlso (Related Featuresets) and RelatedLinks (Additional Resources) *)

  dprint["before reworking SeeAlso and RelatedLinks, expr is "]; dfile[expr];

  expr = expr /.
   Cell[CellGroupData[{Cell[c_, "FeaturesetSeeAlsoSection", o___], p___}, q___], r___] :> 
   Cell[CellGroupData[{
     Cell[c, "FeaturesetSeeAlsoSection", o],
     p
   }, q], r];
   
   expr = expr /.
   Cell[CellGroupData[{Cell[c_, "FeaturesetMoreAboutSection", o___], p___}, q___], r___] :> 
   Cell[CellGroupData[{
     Cell[c, "FeaturesetMoreAboutSection", o],
     p
   }, q], r];
   
   expr = expr /.
   Cell[CellGroupData[{Cell[c_, "FeaturesetRelatedDemonstrationsSection", o___], p___}, q___], r___] :> 
   Cell[CellGroupData[{
     Cell[c, "FeaturesetRelatedDemonstrationsSection", o, CellGroupingRules->{"SectionGrouping", 50}],
     p
   }, q], r];
   
dprint["expr before reorganizing RelatedLinks: "]; dfile[expr];
expr = expr /.
{
  first:___,
  Cell[CellGroupData[{
    c:Cell[_, "FeaturesetRelatedLinksSection", ___],
    d:___,
    e:Longest[Cell[_, "FeaturesetRelatedLinks", ___]..],
    f:Cell[CellGroupData[{__}, ___], ___]
  }, g:___], h:___],
  rest:___
} :>
{
  first,
  Cell[CellGroupData[{
    c, d, e
  }, g], h],
  f,
  rest
};
dprint["expr after reorganizing RelatedLinks: "]; dfile[expr];

(* ** localize section titles *)

(* TODO: add featureset localization to DocumentationBuild/Assets/Data/Localization.m *)

  (* Change See Also title per sw *)
  dprint[" - Change See Also title per sw"];
  expr = expr /. Cell[_, "FeaturesetSeeAlsoSection", o___] :> Cell[Localized["FeaturesetSeeAlsoSection", language], "FeaturesetSeeAlsoSection", o];
  expr = expr /. Cell[_, "FeaturesetMoreAboutSection", o___] :> Cell[Localized["FeaturesetMoreAboutSection", language], "FeaturesetMoreAboutSection", o];
  expr = expr /. Cell[_, "FeaturesetRelatedFunctionsSection", o___] :> Cell[Localized["FeaturesetRelatedFunctionsSection", language], "FeaturesetRelatedFunctionsSection", o];
  expr = expr /. Cell[_, "FeaturesetRelatedLinksSection", o___] :> Cell[Localized["FeaturesetRelatedLinksSection", language], "FeaturesetRelatedLinksSection", o];
  expr = expr /. Cell[_, "FeaturesetRelatedDemonstrationsSection", o___] :> Cell[Localized["FeaturesetRelatedDemonstrationsSection", language], "FeaturesetRelatedDemonstrationsSection", o];

(* ** move page elements to correct order *)

  (* Move See Also group to bottom per sw *)
  dprint[" - Move See Also group to bottom per sw"];
  seeAlsoGroup = Cases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetSeeAlsoSection", ___], ___},___],___], Infinity];
  moreAboutGroup = Cases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetMoreAboutSection", ___], ___},___],___], Infinity];
  relatedFunctionsGroup=Cases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetRelatedFunctionsSection", ___], ___},___],___], Infinity];
  relatedLinksGroup=Cases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetRelatedLinksSection", ___], ___},___],___], Infinity];
  relatedDemonstrationsGroup=Cases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetRelatedDemonstrationsSection", ___], ___},___],___], Infinity];
  
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetSeeAlsoSection", ___], ___},___],___], Infinity];
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetMoreAboutSection", ___], ___},___],___], Infinity];
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetRelatedFunctionsSection", ___], ___},___],___], Infinity];
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetRelatedLinksSection", ___], ___},___],___], Infinity];
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetRelatedDemonstrationsSection", ___], ___},___],___], Infinity];
  
  expr[[1]] = Flatten[{
    First[expr]
  , relatedFunctionsGroup
  , moreAboutGroup
  , relatedDemonstrationsGroup
  , seeAlsoGroup
  , relatedLinksGroup
  }];

(* ** change paclets to URLs *)

  dprint[" - Change paclets to URLs"];

  expr = expr /. ButtonBox[
    c__
  , BaseStyle -> _
  , o1___
  , ButtonData -> str_String /; StringMatchQ[str, "paclet*"]
  , o2___
  ]
  :>
  ButtonBox[
    c
  , BaseStyle -> "Hyperlink"
  , ButtonData -> {
      URL[
        Which[
          Length[StringSplit[str, "#"]] === 2 ,
          "http://reference.wolfram.com/language/"
          <> StringReplace[StringSplit[str, "#"][[1]], "paclet:" -> ""]
          <> ".html#"
          <> StringReplace[StringSplit[str, "#"][[2]], "paclet:" -> ""]
        , StringContainsQ[str, "paclet:featureset"],
          DocumentationBuild`Utils`decamel[
            StringReplace[str, {"paclet:" -> "", "featureset/" -> ""}],
            "Lowercase" -> True
          ]<>".php.en" (* update when translations arrive *)
        , True,
            "http://reference.wolfram.com/language/" <>
            StringReplace[str, "paclet:" -> ""] <>
            ".html"
        ]
      ]
    , None
    }
  , BaseStyle->"Hyperlink"
  , o1, o2
  ];

  expr = expr /. Cell[BoxData[ButtonBox[c_, BaseStyle->"Link", ___]], "InlineFormula", ___] :> 
    Cell[BoxData[ButtonBox[c, ButtonData->{URL["http://reference.wolfram.com/language/ref/"<> StringReplace[c, {"\""->""}] <>".html"], None}, BaseStyle->"Hyperlink"]], "InlineFormula"];

(* ** miscellaneous cleanup *)

  (* change EmptyVerySmallSquare to "something else" *)
  dprint[" - change EmptyVerySmallSquare to \"something else\""];
  expr = expr //. Cell[TextData[{a___, s_String /; StringMatchQ[s, RegularExpression["\\s+\[EmptyVerySmallSquare]\\s+"] ], b___}], 
    "FeaturesetRelatedFunctions", o___] :> 
    Cell[TextData[{a, " ", b}], "FeaturesetRelatedFunctions", o];

  (* Change WindowSize to a width for nicer line wrapping in gifs *)
  dprint[" - Change WindowSize to a width for nicer line wrapping in gifs"];
  expr = DocumentationBuild`Utils`Private`ResetOptions[expr, 
    WindowSize -> {700, 700},
    ShowCellBracket -> True,
    StyleDefinitions -> Notebook[{Cell[ StyleData[ StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "MarketingPageStyles.nb"]]]}]
  ];

(* ** return the final manipulated expression *)

dprint["Make.wl: leaving MakeFeaturesetPageNotebook, returning expr"];

  expr

];

(* * ExportFeaturesetWebPage: MakeFeaturesetPageNotebook -> expr munging -> Transmogrify *)
(* ** prep *)
(* *** begin function definitions *)

Options[ExportFeaturesetWebPage] = {DefaultParameters->{}, Keywords->{}, LogFile->None, Test->False};

ExportFeaturesetWebPage[_,$Failed,___] = $Failed;

ExportFeaturesetWebPage[file_String, nb_String, language_String:"English", opts___?OptionQ]:= Module[{expr},
  dprint["entering _String, _String, _String, opts form of ExportFeaturesetWebPage; args:"];
  dprint["  file = "<>ToString[file, InputForm]];
  dprint["  nb = "<>ToString[nb, InputForm]];
  dprint["  language = "<>ToString[language, InputForm]];
  dprint["  opts = "<>ToString[{opts}, InputForm]];
  expr = Get@nb;
  dprint["  Head[expr] = "<>ToString[Head[expr], InputForm]];
  dprint["now calling _String, _Notebook, _String form of ExportFeaturesetWebPage"];
  dprint["  Head[file] = "<>ToString[Head[file], InputForm]];
  dprint["  Head[expr] = "<>ToString[Head[expr], InputForm]];
  dprint["  Head[language] = "<>ToString[Head[language], InputForm]];
  ExportFeaturesetWebPage[file, expr, language]
];

ExportFeaturesetWebPage[file_String, nbExpr_Notebook, language_String:"English"] := Module[
  {
    ext, properties={}, outfile=file, transform, keywordList={}, titleName,
    res, nbObj, nbfile, nbobj, nbobj2, nbget, tmp, parentDir, nbFile, localcss,
    layoutfile, popupLayoutfile, urlBaseRemote, expr, infilename, filename,
    jatitle, description, ExamplesRawMaterialDir, WhatsNewID,
    MarketingCategoryName, additionalstyles, defaultstylesheet
  }
,
  dprint["** Entering ExportFeaturesetWebPage **"];
  dprint["entering _String, _Notebook, _String, opts form of ExportFeaturesetWebPage; args:"];
  dprint["  file = "<>ToString[file, InputForm]];
  dprint["  Head[nbExpr] = "<>ToString[Head[nbExpr], InputForm]];
  dprint["  language = "<>ToString[language, InputForm]];
  dprint["  opts = "<>ToString[{opts}, InputForm]];

  dprint["ExportFeaturesetWebPage: earliest possible incoming expr: "]; dfile[nbExpr];

(* *** prep for 'in-product' notebook build *)

  infilename = AntProperty["individualFilename"];
  infilename = FileNameJoin[FileNameSplit[AbsoluteFileName[infilename]]];

  UseDocumentationBuildXMLTransforms[];

  urlBaseRemote = "http://reference.wolfram.com/language/";
  ext = Switch[language,
    "Japanese", "ja"
  , "Russian", "ru"
  , "Korean", "ko"
  , "French", "fr"
  , "German", "de"
  , "Portuguese", "pt-br"
  , "Spanish", "es"
  ,"ChineseSimplified", "zh"
  , _, "en"
  ];
  
  transform = {"HTML","Featuresets.m"};
  layoutfile = DocsAsset["HTMLTransformLayout", "Featuresets.html"];

  (* keywords *)
  keywords = Cases[nbExpr, Cell[s:_String /; Not@StringMatchQ[s, "XXXX"], "Keywords", ___] -> s, Infinity];
  dprint["incoming expr: keywords = "<>ToString[keywords, InputForm]];

  titleName = DocumentationBuild`Info`GetNotebookTitle[nbExpr, "Featureset Page"];
  dprint["FeaturesetPages2.m:ExportFeaturesetWebPage: titleName = "<>ToString[titleName, InputForm]];

  LogIt["  Title name: "<>ToString[titleName]];
  dprint["FeaturesetPages2.m:ExportFeaturesetWebPage: ext = "<>ToString[ext, InputForm]];
  dprint["FeaturesetPages2.m:ExportFeaturesetWebPage: outfile = "<>ToString[outfile, InputForm]];

(* ** generate a notebook expr: MakeFeaturesetPageNotebook *)

  (* Create preprocess notebook *)
  
  dprint["Calling MakeFeaturesetPageNotebook with args:"];
  dprint["  Head[nbExpr] = "<>ToString[Head[nbExpr], InputForm]];
  dprint["  language = "<>ToString[language, InputForm]];
  dprint["  {opts} = "<>ToString[{opts}, InputForm]];
  xxx = ToString[Information[MakeFeaturesetPageNotebook]];
  expr = MakeFeaturesetPageNotebook[nbExpr, language];
  dprint["expr after MakeFeaturesetPageNotebook:"]; dfile[expr];

(* ** modify the notebook expr *)

  dprint["after MakeFeaturesetPageNotebook, expr is:"]; dfile[expr];

(* *** remove and rearrange items *)

expr = DeleteCases[expr, Cell[_, "History", ___], Infinity];
expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "CategorizationSection", ___], ___}, ___], ___], Infinity];
expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "ContributorSection", ___], ___}, ___], ___], Infinity];
expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "KeywordsSection", ___], ___}, ___], ___], Infinity];
expr = expr /. {
  Rule[CellChangeTimes, _] -> Sequence[],
  Rule[ExpressionUUID, _] -> Sequence[]
};

dprint["changing hero FeaturesetText to FeaturesetTitleText, expr = "]; dfile[expr /. (_Image -> xxxImage)];

(* prep for hero section later *)
expr = expr /.
  {
    first:___,
    a:Cell[_, "FeaturesetTitle", ___],
    Shortest[b:___Cell],
    c:Cell[_, "FeaturesetText", ___],
    rest:___
  } :>
  {first, a, b, c /. "FeaturesetText" -> "FeaturesetTitleText", rest};

dprint["after changing hero FeaturesetText to FeaturesetTitleText, expr = "]; dfile[expr /. (_Image -> xxxImage)];

(* *** example/subhero sections *)

(* debugOn[]; *)

(* **** example/subhero title, images, blurb *)

exampleSections = Cases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetExampleSection", ___], ___}, ___], ___], Infinity];
dprint["exampleSections = "; dfile[exampleSection]];
expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetExampleSection", ___], ___}, ___], ___], Infinity];

exampleSectionCounter = 0;

exampleSectionList = Module[{
    section, sectionTitle, sectionTitleDecamel, sectionBlurb, sectionImage,
    sectionImageCell, sectionImageCellRight, sectionImageCellLeft, links = {},
    featureLinkCells, subheroCounter, multipleImagesQ = False, sectionImageCount
  },

  section = #;
  exampleSectionCounter++;
  dprint["======================================================================\nprocessing example section "<>ToString[exampleSectionCounter]];
  dprint["section expression 1 is "]; dfile[section];
  dprint["full section expression 1 is "]; dfile[section, "DeleteLargeItems" -> False];

  section = section /. Cell[_, "FeatureImageInput", ___] -> Sequence[];

  section = section /.
    {a:___, b:Cell[_, "FeaturesetExampleSection", ___], c:___, d:Cell[_, "FeaturesetText", ___], e:___} :>
    {a, b, c, d /. "FeaturesetText" -> "FeaturesetSectionBlurb", e};

  (* ---[ title ]--------------------------- *)
  sectionTitle = Quiet @ Check[First @ Cases[section, Cell[s:_String, "FeaturesetExampleSection", ___] -> s, Infinity], "No section title"];
  sectionTitleDecamel = DocumentationBuild`Utils`decamel[sectionTitle, "Lowercase" -> True];
  dprint["sectionTitle = "<>ToString[sectionTitle, InputForm]];
  dprint["sectionTitleDecamel = "<>ToString[sectionTitleDecamel, InputForm]];

  (* ---[ blurb ]--------------------------- *)
  sectionBlurb = Quiet @ Check[First @ Cases[section, Cell[s:_String, "FeaturesetSectionBlurb", ___] -> s, Infinity], "No section blurb"];
  dprint["sectionBlurb = "<>ToString[sectionBlurb, InputForm]];

  dprint["section expression step 2 is "]; dfile[section];
  dprint["full section expression step 2 is "]; dfile[section, "DeleteLargeItems" -> False];

  (* ---[ image or video ]--------------------------- *)

  dprint["entering section image/video processing..."];
  dprint["checking the section expr for a FeatureImageOutput cell..."];
  dprint["there are "<>ToString[Count[section, Cell[_, "FeatureImageOutput", ___], Infinity], InputForm]<>" FeatureImageOutput cell(s) in this section expr"];
  dprint["(multiple subhero images in a section will be displayed as a slideshow/carousel); if zero images are provided in the source nb, use a placeholder"];
  multipleImagesQ = Count[section, Cell[_, "FeatureImageOutput", ___], Infinity] > 1;

  dprint["begin processing every FeatureImageOutput cell in the section"];

  (* debugOn[]; *)
  sectionImageCount = 0;
  sectionImageResult = 
    Cases[
      section
    , Cell[i:_, "FeatureImageOutput", ___] :>
        Module[{image = i, c, res, imageFilename, durations = {}, videoQ = False},

          (* general cleanup and massage if needed*)
          image = image /. TagBox[a:_, _EventHandlerTag] :> a;

          If[MatchQ[image, _TextData],
            dprint["MatchQ[image, _TextData]"];
            (* image is now a string that may represent the basename of an image or video file *)
            image = Quiet @ Check[First[Cases[image, ButtonBox[f:_String, ___] -> f, Infinity]], ""]];

          If[MatchQ[image, _BoxData], image = ToExpression[image]];

          Which[
            MatchQ[image, _Graphics3D],

              dprint["MatchQ[image, _Graphics3D]"];
              (* Rasterize per shadi: https://rocketchat.wolfram.com/direct/TF2TmESutnNRxABtkuuWY8ipttmRNTW9xM?msg=rAqjRfPRYgAkmrNHg *)
              image = Rasterize[image, Background -> None, ImageSize -> {Automatic, 600}, ImageResolution -> 144];
              imageFilename = sectionTitleDecamel<>".webp"

          , MatchQ[image, _String],

              dprint["MatchQ[image, _String]"];
              (* by authoring convention,'image' *should* (but may not) represent
                 the basename of an image or video file in the same dir as the input nb *)
              imageFilename = FileNameJoin[{FileNameDrop[infilename], image}];
              If[FileExistsQ[imageFilename]
              , image = Import[imageFilename];
                durations = If[Length[image] > 1, Import[imageFilename, "DisplayDurations"], {}];
              , image = placeholderExpr;
                imageFilename = "feature-image-placeholder.webp";
              ];

          , MatchQ[image, _Image],
              dprint["MatchQ[image, _Image]"];
              imageFilename = sectionTitleDecamel<>".webp"

          , True,
              dprint["True"];
              image = placeholderExpr;
              imageFilename = "feature-image-placeholder.webp";
          ];

          c = If[Length[image] > 1
          , ImageResize[#, {Automatic,600}]& /@ image
          , ImageResize[image, {Automatic, 600}]
          ];
          dprint["  in: c = ImageResize[image, {Automatic, 600}],"];
          dprint["  short c is "]; dfile[c /. _Image -> xxxImage];
          dprint["Head[c] = "<>ToString[Head[c], InputForm]];
          dprint["imageFilename = "<>ToString[imageFilename, InputForm]];

          videoQ = FileExtension[imageFilename] == "mp4";

          (* video files do not receive cropping/resizing processing *)
          If[Not @ videoQ,
            (* 'image' may be _Image or {_Image..} *)
            With[{
              w = If[Length[c] > 1
              , ImageDimensions[First[c]][[1]]
              , ImageDimensions[c][[1]]]
              }
            , dprint["w = "<>ToString[w, InputForm]];
              res = Which[
                w < 850,
                  dprint["w < 850: ImageCrop"];
                  If[Length@c > 1,
                    dprint["ImageDimensions /@ c = "<>ToString[ImageDimensions /@ c, InputForm]],
                    dprint["ImageDimensions[c] = "<>ToString[ImageDimensions[c], InputForm]]];
                  If[Length[c] > 1
                  , ImageCrop[#, {850, 600}, Padding -> Transparent]& /@ c
                  , ImageCrop[c, {850, 600}, Padding -> Transparent]
                  ]
              ,
                w > 1100,
                  dprint["w > 1100: ConformImages"];
                  If[Length@c > 1,
                    dprint["ImageDimensions /@ c = "<>ToString[ImageDimensions /@ c, InputForm]],
                    dprint["ImageDimensions[c] = "<>ToString[ImageDimensions[c], InputForm]]];
                  xx = ConformImages[Flatten[{c}], {1100, 600}, "Fit", Padding -> Transparent];
                  dprint["xx in width calc is "<>ToString[xx /. (_Image -> xxxImage)]];
                  xx
              ,
                True,
                  dprint["w >= 850 and <= 1100: leave as-is"];
                  If[Length@c > 1,
                    dprint["ImageDimensions /@ c = "<>ToString[ImageDimensions /@ c, InputForm]],
                    dprint["ImageDimensions[c] = "<>ToString[ImageDimensions[c], InputForm]]];
                  c
              ]
            ]
          ,
            res = image
          ]; (* close Not @ videoQ *)
          dprint["after possible sizing and cropping, {imageFilename, res} = "<>ToString[{imageFilename, res /. _Image -> xxxImage}, InputForm]];

          If[multipleImagesQ, imageFilename = StringReplace[imageFilename, FileBaseName[imageFilename] -> FileBaseName[imageFilename]<>"-"<>ToString[++sectionImageCount]]];
          dprint["final imageFilename = "<>ToString[imageFilename, InputForm]];

          {"ImageFilename" -> imageFilename, "videoQ" -> videoQ, "Durations" -> durations, "Image" -> res}
        ]
    , Infinity
    ];
  dprint["initial sectionImageResult = "<>ToString[sectionImageResult /. _Image -> xxxImage, InputForm]];
  If[MatchQ[sectionImageResult, {}], sectionImageResult = {{"ImageFilename" -> "feature-image-placeholder.webp", "Durations" -> {}, "Image" -> placeholderExpr}}];

  dprint["after processing all FeatureImageOutput cells in the section, sectionImageResult is "<>ToString[sectionImageResult /. _Image -> xxxImage, InputForm]];

  dprint["now turn sectionImageResult into a list of cells"];
  subheroImageCounter = 0;

  sectionImageCells = Module[{
    image, sourceImageFilename, x, res, durations, videoQ = False
  },
    ++subheroImageCounter;

    {sourceImageFilename, durations, image} = {"ImageFilename", "Durations", "Image"} /. #;
    dprint["  {sourceImageFilename, durations, image} = "]; dfile[{sourceImageFilename, durations, image} /. _Image -> xxxImage];
    videoQ = FileExtension[sourceImageFilename] == "mp4";
    dprint["generating section image cells, videoQ = "<>ToString[videoQ, InputForm]];

    dprint["building an ExampleSectionImage cell for subhero image number "<>ToString[subheroImageCounter, InputForm]];

    x = Cell[
      If[videoQ, image, BoxData[ToBoxes[image]]],
      "ExampleSectionImage",
      "SourceImageFilename" -> sourceImageFilename,
      "DisplayDurations" -> durations
    ];
    AppendTo[x, "SectionCounter" -> exampleSectionCounter];
    AppendTo[x, "SubheroImageCounter" -> subheroImageCounter];
    res = AppendTo[x, "SectionTitle" -> sectionTitleDecamel];
    dprint["  result for subhero image number "<>ToString[subheroImageCounter, InputForm]<>" is "]; dfile[res];
    res
  ]& /@ sectionImageResult;

  dprint["sectionImageCells is: "]; dfile[sectionImageCells];

  multipleSubheroImagesQ = Length[sectionImageCells] > 1;
  dprint["multipleSubheroImagesQ = "<>ToString[multipleSubheroImagesQ, InputForm]];

(* **** example/subhero links *)

  links1FeatureLinkURL = Cases[
    section
  , Cell[
      TextData[
        ButtonBox[
          title:_String,
          ___,
          Rule|RuleDelayed_[ButtonData, {URL[link:_String], ___}], ___], ___]
    , "FeatureLink"
    , ___
    ] :> Cell[title, "ExampleSectionLink", "Link" -> link, "Style" -> "links1"]
  , Infinity
  ];
  dprint["links1FeatureLinkURL = "<>ToString[links1FeatureLinkURL, InputForm]];

  links2FeatureLinkString = Cases[
    section
  , Cell[
      TextData[
        ButtonBox[
          title:_String,
          ___,
          Rule|RuleDelayed_[ButtonData, link:_String],
          ___
        ],
      ___
    ]
    , "FeatureLink"
    , ___
    ] :> Cell[title, "ExampleSectionLink", "Link" -> link, "Style" -> "links2"]
  , Infinity
  ];
  dprint["links2FeatureLinkString = "<>ToString[links2FeatureLinkString, InputForm]];

  links3FeaturesetExampleSectionDocLinkURL = Cases[
    section
  , Cell[
      TextData[
        ButtonBox[
          title:_String,
          ___,
          Rule[ButtonData, List[URL[link:_String], ___]],
          ___
        ],
      ___
    ]
    , "FeaturesetExampleSectionDocLink"
    , ___
    ] :> Cell[title, "ExampleSectionLink", "Link" -> link, "Style" -> "links3"]
  , Infinity
  ];
  dprint["links3FeaturesetExampleSectionDocLinkURL = "<>ToString[links3FeaturesetExampleSectionDocLinkURL, InputForm]];

  links4FeaturesetExampleSectionDocLinkString = Cases[
    section,
    Cell[
      TextData[
        ButtonBox[
          title:_String,
          ___,
          Rule|RuleDelayed_[ButtonData, link:_String],
          ___
        ],
        ___
      ],
      "FeaturesetExampleSectionDocLink",
      ___
    ] :> Cell[title, "ExampleSectionLink", "Link" -> link, "Style" -> "links4"]
    Infinity
  ];
  dprint["links4FeaturesetExampleSectionDocLinkString = "<>ToString[links4FeaturesetExampleSectionDocLinkString, InputForm]];

  links = DeleteDuplicates[Flatten[{
    links1FeatureLinkURL,
    links2FeatureLinkString,
    links3FeaturesetExampleSectionDocLinkURL,
    links4FeaturesetExampleSectionDocLinkString
  }]];

  (* ---[ resolve any example links to notebookarchive.org ]---------------------------------------- *)
  links = links /. (
    s:_String /; StringContainsQ[s, StartOfString ~~ "http://reference.wolfram.com/language/featureexample/"] :>
    getNotebookArchiveLink[s]
  );

(* **** subhero assembly *)

dprint["------------------------------\nassembling the current subhero section"];

  subheroRes = Cell[CellGroupData[Flatten @ {
    Cell[sectionTitle, "ExampleSection", "SectionCounter" -> exampleSectionCounter],
    Cell[CellGroupData[Flatten @ {
      Cell["", "ExampleSectionFeatureLeft", "SectionCounter" -> exampleSectionCounter],
      Cell[sectionTitle, "ExampleSectionHead", "SectionCounter" -> exampleSectionCounter],
      Cell[sectionBlurb, "ExampleSectionBlurb", "SectionCounter" -> exampleSectionCounter],
      If[Length[sectionImageCells] == 1
      , Cell[CellGroupData[Append[#, "Class" -> "hide show__900"]& /@ sectionImageCells]]
      , Cell[CellGroupData[sectionImageCells /. "ExampleSectionImage" -> "SubheroSlideImageTop"]]
      ],
      Cell[CellGroupData[Flatten @ {links}]]
    }]],
    If[Length[sectionImageCells] == 1
    , Cell[CellGroupData[Append[#, "Class" -> "feature-right hide__900"]& /@ sectionImageCells]]
    , Cell[CellGroupData[sectionImageCells /. "ExampleSectionImage" -> "SubheroSlideImageBottom"]]
    ]
  }]];
  dprint["subheroRes = "]; dfile[subheroRes];
  subheroRes
]& /@ exampleSections;

(* debugOff[]; *)

(* *** related courses section *)

dprint["entering related courses, expr ="]; dfile[expr];
dprint["do i need to clean this section up in MakeFeaturesetPageNotebook?"];

relatedCourses = Cases[expr, Cell[CellGroupData[{Cell["Related Courses", "FeaturesetRelatedCourseSection", ___], ___}, ___], ___], Infinity];
dprint["relatedCourses 1 = "]; dfile[relatedCourses];

relatedCourses = Cases[
  relatedCourses
, {
    title:Cell[_, "FeaturesetRelatedCourseSection", ___],
    seeall:__,
    Cell[_, "ExampleDelimiter", ___],
    rest:__
  } :> {
    "seeall" -> {seeall},
    "rest" -> {rest}
  }
, Infinity];

seeAllSection = "seeall" /. relatedCourses;
dprint["initial seeAllSection = "]; dfile[seeAllSection];
relatedCoursesSection = "rest"/. relatedCourses;

(* **** related courses: "see all" items *)

dprint["incoming seeAllSection = "]; dfile[seeAllSection];
dprint["from seeAllSection I just need an image and two links"];

seeAllImage = Quiet @ Check[First @ Cases[seeAllSection, Cell[i_, "FeaturesetCourseImage", ___] -> i, Infinity], {}];
dprint["seeAllImage = "]; dfile[seeAllImage];
dprint["Head[seeAllImage] = "<>ToString[Head[seeAllImage, InputForm]]];

seeAllCoursesLink = Quiet @ Check[First @ Cases[
  seeAllSection,
  Cell[TextData[ButtonBox["See all courses", ___, ButtonData -> {URL[url:_String], ___}, ___]], "FeaturesetRelatedCourseTitle", ___] -> url,
  Infinity], ""];
dprint["seeAllCoursesLink = "<>ToString[seeAllCoursesLink, InputForm]];

seeAllCertificatesLink = Quiet @ Check[First @ Cases[
  seeAllSection,
  Cell[TextData[ButtonBox["See all certificates", ___, ButtonData -> {URL[url:_String], ___}, ___]], "FeaturesetRelatedCourseTitle", ___] -> url,
  Infinity], ""];
dprint["seeAllCertificatesLink = "<>ToString[seeAllCertificatesLink, InputForm]];

(* **** related courses: course items *)

dprint["relatedCoursesSection = "]; dfile[relatedCoursesSection];
dprint["for each course I need an image, a description, a type, and a link"];

courseCount = 0;
relatedCoursesData = Cases[
  relatedCoursesSection,
  Cell[CellGroupData[{
    Cell[TextData[ButtonBox[desc:_String, ___, ButtonData -> {URL[url:_String], None}, ___]], "FeaturesetRelatedCourseTitle", ___],
    Cell[_, "FeaturesetRelatedCourseType", Rule[TaggingRules, courseType:_], ___],
    Cell[image:_, "FeaturesetCourseImage", ___]
  }, ___], ___] :> {"courseDescription" -> desc, "courseLink" -> url, "courseType" -> courseType, "courseImage" -> image},
  Infinity
];
dprint[ToString[Length[relatedCoursesData]]<>" related courses provided by "<>infilename];
relatedCoursesData = Take[relatedCoursesData, UpTo[2]];
dprint["relatedCoursesData = "]; dfile[relatedCoursesData];

(* ***** code *)

dprint["near final relatedCoursesData = "]; dfile[relatedCoursesData];

boilerplateCourse = {"courseDescription" -> "boilerplate", "courseLink" -> "", "courseType" -> "", "courseImage" -> ""};

relatedCoursesData = Switch[Length[relatedCoursesData],
  0, AppendTo[relatedCoursesData, boilerplateCourse]; AppendTo[relatedCoursesData, boilerplateCourse],
  1, AppendTo[relatedCoursesData, boilerplateCourse],
  _, Take[relatedCoursesData, UpTo[2]]
];

dprint["final relatedCoursesData = "]; dfile[relatedCoursesData];

count = 0;
relatedCourses = Module[{data, descr, image, link, type},
  count++;
  data = #;
  dprint["building relatedcourses, data #"<>ToString[count]<>" is:"]; dfile[data];

  descr = "courseDescription" /. data;
  image = "courseImage" /. data;
  link  = "courseLink"  /. data;
  type  = "courseType"  /. data;

  If[descr == "boilerplate"
  , Cell[CellGroupData[{
      Cell["", If[count == 1, "UpperCoursesLeftColumn", "UpperCoursesCenterColumn"]],
      Cell["", "CourseImageBoilerplate"],
      Cell["", "CourseTypeBoilerplate"],
      Cell["", "CourseLinkBoilerplate"]
    }]]
  , Cell[CellGroupData[{
      Cell["", If[count == 1, "UpperCoursesLeftColumn", "UpperCoursesCenterColumn"]],
      Cell[image, "CourseImage", "url" -> link],
      (* Cell[type, "CourseType"], *)
      Cell["Interactive", "CourseType"],
      Cell[link, "CourseLink", "Description" -> descr]
    }]]
  ]
]& /@ relatedCoursesData;

dprint["antefinal relatedCourses = "]; dfile[relatedCourses];

relatedCourses = 
  Cell[CellGroupData[{
    Cell["", "CoursesSection"],
    Cell[CellGroupData[{
        Cell["", "UpperCoursesPod"],
        relatedCourses[[1]],
        relatedCourses[[2]],
        Cell[CellGroupData[{
            Cell["", "UpperCoursesRightColumn"],
            If[Or[
                MatchQ[seeAllImage, _List],
                Positive @ Count[seeAllImage, s:_String /; StringContainsQ[s, "Related Course Thumbnail"], Infinity]
              ]
            , Cell["", "CourseImageBoilerplate"]
            , Cell[seeAllImage, "CourseImage", "url" -> seeAllCoursesLink]
            ],
            Cell[CellGroupData[{
              Cell["", "SeeAllListWrapper"],
              Cell["See all courses", "SeeAllCoursesLink", "url" -> seeAllCoursesLink],
              Cell["See all certificates", "SeeAllCertificatesLink", "url" -> "https://www.wolfram.com/wolfram-u/certification/"]
            }]]
        }]]
    }]],
    Cell[CellGroupData[{
        Cell["", "LowerCoursesPod"],
        Cell["", "GetCertifiedBoilerplate"],
        Cell["", "WolframUBoilerplate"]
    }]]
  }]];

dprint["the final relatedCourses = "]; dfile[relatedCourses];

(* *** "Secondary Links": Related Featuresets and Additional Resources *)

(* This section contains two columns: the left column is populated from the
source notebook's "FeaturesetSeeAlsoSection" expr; the right is from
"FeaturesetRelatedLinksSection". *)

dprint["entering SecondaryLinks, expr is "]; dfile[expr];

(* **** FeaturesetSeeAlsoSection *)

seeAlsos = Cases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetSeeAlsoSection", ___], ___}, ___], ___], Infinity];
expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetSeeAlsoSection", ___], ___}, ___], ___], Infinity];
dprint["seeAlsos = "]; dfile[seeAlsos];

(* **** FeaturesetRelatedLinksSection *)

relatedLinks = Cases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetRelatedLinksSection", ___], ___}, ___], ___], Infinity];
expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "FeaturesetRelatedLinksSection", ___], ___}, ___], ___], Infinity];
dprint["relatedLinks = "]; dfile[relatedLinks];

(* **** assemble SecondaryLinksSection *)

secondaryLinksSection = Cell[CellGroupData[Flatten @ {
  Cell["", "SecondaryLinksSection"],
  seeAlsos,
  relatedLinks
}]];

(* remove stuff that shouldn't be here (Chemistry.nb) *)
secondaryLinksSection = secondaryLinksSection /. Cell[CellGroupData[{Cell[_, "FeaturesetRelatedCourseSection", ___], ___}, ___], ___] :> Sequence[];

(* fill out empty link lists *)
secondaryLinksSection = secondaryLinksSection /.
  Cell[CellGroupData[{c:Cell[None, "FeaturesetSeeAlsoSection", ___]}, ___], ___] :>
  Cell[CellGroupData[{c, Cell["XXXX", "FeaturesetSeeAlso"], Cell["XXXX", "FeaturesetSeeAlso"]}]];

secondaryLinksSection = secondaryLinksSection /.
  Cell[CellGroupData[{c:Cell[None, "FeaturesetRelatedLinksSection", ___]}, ___], ___] :>
  Cell[CellGroupData[{c, Cell["XXXX", "FeaturesetRelatedLinks"], Cell["XXXX", "FeaturesetRelatedLinks"]}]];

dprint["secondaryLinksSection = "]; dfile[secondaryLinksSection];

(* *** Documentation section with doc screenshots *)

dprint["expr entering doc section:"]; dfile[expr];

docFuncList = Cases[expr, Cell[CellGroupData[{Cell["Related Functions", "FeaturesetRelatedDocumentationSubsection", ___], ___}, ___], ___], Infinity];
docFuncList = Cases[docFuncList, ButtonBox[func:_String, ___, BaseStyle -> "Hyperlink", ___] -> func, Infinity];
dprint["docFuncList = "<>ToString[docFuncList, InputForm]];

docGuideList = Cases[
  expr,
  {___,
    Cell[s_String /; StringContainsQ[s, "Related Guides (for add'l functions)"], "FeaturesetRelatedDocumentationSubsection", ___],
    c:Longest[__Cell],
    ___
  } :> c,
  Infinity
];

docGuideList = Cases[
  docGuideList,
  _[title:_String, ___, Rule[ButtonData, {URL[uri:_String], ___}], ___]
  :> StringReplace[uri, {"http://reference.wolfram.com/language/" -> "", ".html" -> ""}],
  Infinity
];

dprint["docGuideList = "<>ToString[docGuideList, InputForm]];

docFunctions = DocumentationBuild`Utils`GetFeaturesetFunctions[docFuncList, docGuideList];
dprint["docFunctions = "<>ToString[docFunctions, InputForm]];

docThumbs = Cases[expr, Cell[CellGroupData[{Cell["Thumbnailed Pages", "FeaturesetRelatedDocumentationSubsection", ___], c:__Cell}, ___], ___] -> c, Infinity];
dprint["initial docThumbs:"]; dfile[docThumbs];
docThumbs = Cases[
  docThumbs,
  ButtonBox[caption:_String, ___, Rule[ButtonData, List[URL[url:_String], None]], ___]
  :> {
    "caption" -> StringTrim[caption],
    "url" -> url,
    "shotfile" -> StringReplace[url, {
      ".html" -> ".webp",
      "System/WorkflowGuides" -> "workflowguide",
      "PDEModels/Tutorials" -> "PDEModels/tutorial",
      "http://reference.wolfram.com/language/PDEModels/" -> "assets/img/pages.en/PDEModels/",
      "http://reference.wolfram.com/language/" -> "assets/img/pages.en/system/"
    }]
  },
  Infinity
];

(* delete items with only whitespace or nothing as a caption - these are
invisible buttons left behind after an editing session *)
docThumbs = DeleteCases[docThumbs, {"caption" -> s:_String /; StringMatchQ[s, Whitespace...], ___}];

dprint["docThumbs 2 = "]; dfile[docThumbs];

AppendTo[docThumbs, {
  "caption" -> "Wolfram Language Documentation Center",
  "url" -> "http://reference.wolfram.com/language/",
  "shotfile" -> "assets/img/pages.en/system/guide/WolframRoot.webp"
}];


dprint["final docThumbs = "<>ToString[docThumbs, InputForm]];
dprint["and look at DocScreenshot below"];

docs = Cell[CellGroupData[{
  Cell["", "DocSection"],
  Cell["", "DocColorBars"],
  Cell[CellGroupData[Flatten @ {
    Cell[#, "DocFunctions"]& /@ docFunctions
  }]],
  Cell[CellGroupData[Flatten @ {
    Cell["", "DocContent"],
    Cell["", "DocScreenshot", Sequence @@ (Flatten[#])]& /@ docThumbs
  }]]
}]];

dprint["final 'docs' = "]; dfile[docs /. _Image -> xxxImage];

(* *** hero section *)

dprint["hero section - incoming expr is:"]; dfile[expr];

(*{
  Cell["Image", "FeaturesetTitle", ___],
  Cell["Process, Analyze and Compute with Images.", "FeaturesetBoldTagline", ___],
  Cell["For beginners or experts, Wolfram Image provides blurb.", "FeaturesetTitleText", ___],
  [...]
}*)


heroTitle   = Cases[expr, Cell[_, "FeaturesetTitle", ___], Infinity];
heroTagline = Cases[expr, Cell[_, "FeaturesetBoldTagline", ___], Infinity];
heroBlurb   = Cases[expr, Cell[_, "FeaturesetTitleText", ___], Infinity];

dprint["heroTitle = "<>ToString[heroTitle, InputForm]];
dprint["heroTagline = "<>ToString[heroTagline, InputForm]];
dprint["heroBlurb = "<>ToString[heroBlurb, InputForm]];

heroImage = First @ Cases[expr, Cell[i:_, "FeaturesetTopImage", ___] :> ToExpression[i], Infinity];
If[heroImage == First[{}], heroImage = placeholderExpr];
dprint["hero image 1: heroImage = "]; dfile[heroImage];
(* Note: do not process hero images.  They're carefully designed for a specific size & resolution. *)

(* accept a filename for the hero image *)
If[MatchQ[heroImage, _String]
, (* by authoring convention,'image' *should* (but may not) represent
     the basename of an image file in the same dir as the input nb *)
  dprint["'heroImage' is a String"];
  imageFilename = FileNameJoin[{FileNameDrop[infilename], heroImage}];
  dprint["imageFilename = "<>ToString[imageFilename, InputForm]];
  dprint["FileExistsQ[imageFilename] = "<>ToString[FileExistsQ[imageFilename], InputForm]];
  If[FileExistsQ[imageFilename]
  , dprint["  exists, Importing..."];
    heroImage = Import[imageFilename];
    dprint["hero image 2: heroImage = "]; dfile[heroImage];
    dprint["hero image 2: Information[heroImage] = "<>ToString[Information[heroImage], InputForm]];
  , dprint["  does not exist, using placeholderExpr"];
    heroImage = placeholderExpr;
  ]
];

heroSection = Cell[CellGroupData[{
      Cell["", "Hero", "Color" -> pageColor],
      Cell["", "Breadcrumb"], (* empty for now, may have content later *)
      heroTitle,
      Cell[CellGroupData[{
          Cell["", "FeatureGrid"],
          Cell[CellGroupData[Flatten @ {
              Cell["", "FeatureLeft"],
              Cell[BoxData[ToBoxes[heroImage]], "HeroImage"],
              Cell[CellGroupData[Flatten @ {
                heroBlurb,
                heroTagline
              }]],
              Cell["Get Started", "HeroButton"] (* perhaps content will vary? *)
          }]],
          Cell[CellGroupData[Flatten @ {
              Cell["", "FeatureRight"],
              Cell[BoxData[ToBoxes[heroImage]], "HeroImage"]
          }]]
      }]]
  }]];

(* *** assemble the final expression *)

expr = Notebook[Flatten @ {
  heroSection
, exampleSectionList
, docs
, relatedCourses
, secondaryLinksSection
  (* use the page title for "Get Started with X" *)
, Cell[CellGroupData[{Cell[DocumentationBuild`Info`GetNotebookTitle[Get[inFile], "Featureset Page"], "GetStartedSection"]}]]
, Cell[CellGroupData[{Cell["", "LogoSection"]}]]
}];

dprint["expr to transmog is: "]; dfile[expr];

(* *** get metadata to pass to the Notebook transform *)

(* /Users/billw/git/featuresetpages/English/_MetadataFolder/SEOMetadata.wl *)
seoMetadataFile = FileNameJoin[{inputDir, "_MetadataFolder", "SEOMetadata.wl"}];
seoMetadata = Import[seoMetadataFile];

pageTitle = DocumentationBuild`Info`GetNotebookTitle[Get[inFile], "Featureset Page"];
pageMetadata = Lookup[seoMetadata, pageTitle, {}];

metaTitle = Lookup[pageMetadata, "MetaTitle", pageTitle];
metaDescription = Lookup[pageMetadata, "MetaDescription", pageTitle];
metaURL = "https://www.devel.wolfram.com/core/"<>pageTitle<>".php.en";
metaKeywords = keywords; (* harvested from the incoming notebook expression *)

dprint["metaTitle = "<>ToString[metaTitle, InputForm]];
dprint["metaDescription = "<>ToString[metaDescription, InputForm]];
dprint["metaURL = "<>ToString[metaURL, InputForm]];
dprint["metaKeywords = "<>ToString[metaKeywords, InputForm]];

(* ** generate a webpage: Transmogrify *)

  (* now call Transmogrify with the appropriate parameters set.
     note that language extension by default goes before the .html *)
  LogIt["  Off to transmogrify"];
  dprint["** Off to Transmogrify **"];
  dprint["gronk"];

  res = Developer`UseFrontEnd @ Transmogrify`Transmogrify[
    outfile
  , expr
  , transform
  , Transmogrify`DefaultParameters -> {
      "CompleteHTMLQ" -> False
    , "ExamplesRawMaterialDir" -> ExamplesRawMaterialDir
    , "InFileName" -> infilename
    , "inputDir" -> inputDir
    , "IsSolutionsQ" -> False
    , "IsTMJQ" -> False
    , "Language" -> language
    , "description" -> description
    , "entityType" -> "Featureset"
    , "outfile" -> outfile
    , "outputDir" -> outputDir
    , "htmlBase" -> urlBaseRemote
    , "keywords" -> keywordList
    , "la" -> DocumentationBuild`Utils`LanguageAbbreviation[language]
    , "languageAbbreviations" -> DocumentationBuild`Utils`languageList[[All, 2]]
    , "layoutfile" -> layoutfile
    , "localcss" -> localcss
    , "metaDescription" -> metaDescription
    , "metaKeywords" -> metaKeywords
    , "metaTitle" -> metaTitle
    , "metaURL" -> metaURL
    , "pacletName" -> ""
    , "popupLayoutfile" -> popupLayoutfile
    , "synonyms" -> synonymList
    , "titleName" -> titleName
    , "url" -> ""
    , "urlBaseRemote" -> urlBaseRemote
    , "workflowQ" -> False
    }
  ];

  dprint["** leaving Transmogrify **"];

  If[Head@res =!= String, First@res, res]


  dprint["XXX"];


];

(* * do the work *)

dprint[ DateString[{"Hour", ":", "Minute", ":", "Second", " ", "MonthName", " ", "Day", ", ", "Year"}] ];
$MessagePrePrint = .;
AntSetProperty["notebooks", files];
dprint["files = "<>ToString[files, InputForm]];
AntSetProperty["numberTotalNotebooks", Length @ files];
appPath = AntProperty["appPath"];
If[ !MemberQ[$path, appPath], PrependTo[ $Path, appPath] ];
(* Make the locally checked-out DocumentationBuild override the DocumentationBuild which is included in 12.2 (via PacletTools). See bug #399891. *)
PacletDirectoryLoad[AntProperty["appPath"]];

(* Load DocumentationBuild *)
Needs["DocumentationBuild`"];

finaltime = UsingFrontEnd @ Module[{

    res = "", time = {0, ""}, requests = 0, results = 0, inputDir, outputDir, lang, language, langopt, inFile, pageTitle, outFile,
    exitCodePath, failedFiles, failedFilesExpr, max, numberTotalNotebooks, pf = {}, requestLimit, successfulFiles, successfulFilesExpr,
    history, urlOverride
  }
,

(* ** boilerplate and bookkeeping *)

  dprint["entering FeaturesetPages2.m:finaltime"];
  res = Catch[
  inFile = FileNameJoin[FileNameSplit[AbsoluteFileName[AntProperty["individualFilename"]]]]; Print["inFile = "<>ToString[inFile, OutputForm]];
  dprint["FeaturesetPages2.m: initial inFile = "<>ToString[inFile, InputForm]];
  If[MatchQ[flag = DocumentFlag[inFile], "PreviewFlag" | "ExcisedFlag" | (* "FutureFlag" | *) "InternalFlag" | "TemporaryFlag"]
  , dprint["            " <> flag <> " -- files not included in build."];
    Throw[{0,""}]; (* timing, res *)
  ]; 
  (* ====================================================================== *)
  (* now inFile can only be unflagged or futureflagged *)

  (* history = GetNotebookHistoryData[GetQuietly[inFile]]; *)
  (* If[ *)
  (*   Not@Positive@DocumentationBuild`Make`Private`CheckVersion[ *)
  (*     ("New" /. history) *)
  (*   , "ReleaseVersion" -> "13" *)
  (*   , "ThresholdVersion" -> "1" *)
  (*   ] *)
  (* , dprint["            NewIn < 6 or > 13 -- file not included in build."]; *)
  (*   Throw[{0,""}] *)
  (* ]; *)
  (* ====================================================================== *)
  (* now inFile satisfies 6 <= NewIn <= 13 *)

  (* Set requests variable *)
  If[IntegerQ[requests], requests++, results=1];

  (* Fixing a loading issue for Graphic Directives per Lou *)
  Scan[
    ToExpression[StringJoin["System`", #1]]&
  , {
      "Absolute", "DrawEdges", "DrawFrontFaces", "EdgeCapForm", "EdgeColor",
      "EdgeDashing", "EdgeJoinForm", "EdgeOpacity", "EdgeThickness", "FrontFaceColor",
      "FrontFaceOpacity", "GraphicsColor", "GraphicsHighlightColor", "LineColor",
      "LineOpacity", "NewPrimitiveStyle", "StrokeForm", "RectangleBoxOptions"
    }
  ];

  (* User variables *)
  inputDir = FileNameJoin[FileNameSplit[AbsoluteFileName[AntProperty["inputDir"]]]]; Print["inputDir = "<>ToString[inputDir, InputForm]];
  outputDir = FileNameJoin[FileNameSplit[AntProperty["outputDir"]]]; Print["outputDir = "<>ToString[outputDir, OutputForm]];
  Quiet @ CreateDirectory[FileNameJoin[Flatten[{FileNameSplit[outputDir]}]]];

  language = lang = AntProperty["language"]; Print["language = "<>ToString[language, OutputForm]]; Print["lang = "<>ToString[lang, OutputForm]];
  langopt = {LanguageExtension->".en"}; (* update when we have translations *)

  dprint["building output filename"];
  pageTitle = DocumentationBuild`Info`GetNotebookTitle[Get[inFile], "Featureset Page"];
  dprint[" - pageTitle = "<>ToString[pageTitle, InputForm]];
  urlOverride = Quiet @ Check[First @ Cases[Get[inFile], Cell[s:_String, "Categorization", ___, CellLabel -> "URL Override", ___] :> s, Infinity], $None];
  dprint[" - urlOverride = "<>ToString[urlOverride, InputForm]];
  outFileBase = If[StringQ[urlOverride] && Positive@StringLength[urlOverride], urlOverride, pageTitle];
  dprint[" - outFileBase = "<>ToString[outFileBase, InputForm]];

  outFile = FileNameJoin[{
    outputDir
  , DocumentationBuild`Utils`decamel[outFileBase , "Lowercase" -> True, "Separator" -> "-"]<>".php.en"
  }];
  dprint[" - outFile = "<>ToString[outFile, OutputForm]];

  exitCodePath = AntProperty["exitCodePath"];
  numberTotalNotebooks = AntProperty["numberTotalNotebooks"];
  pf = ToExpression@AntProperty["processedFiles"];
  pf = ToString@Plus[ToExpression@pf, 1];
  AntSetProperty["processedFiles", pf];
  successfulFiles = AntProperty["successfulFiles"];
  successfulFilesExpr = ToExpression@successfulFiles;
  failedFiles = AntProperty["failedFiles"];
  failedFilesExpr = ToExpression@failedFiles;
  requestLimit = AntProperty["requestLimit"];
  max = AntProperty["max"];
  max = If[Head@max =!= String, "{0, None}", max];
  max = StringSplit[StringReplace[max, "{" | "}" -> ""], RegularExpression[",\\s+"]];
  max = {ToExpression@max[[1]], max[[2]]};

  (* If Frontend quits, then rerun stylesheet definitions *)
  DocumentationBuild`Common`PubsEvaluateWithFE[
    featuresetPageToolDir = FileNameJoin[FileNameSplit[AntProperty["FeaturesetPageStylesheetPath"]]]; 
    SetOptions[$FrontEnd, StyleSheetPath -> Inherited];
    DocumentationBuild`$FeaturesetStylePath = With[
      {featuresetPageToolPath = FileNameJoin[FileNameSplit[AntProperty["FeaturesetPageStylesheetPath"]]]}
    , FrontEnd`FileName[{ featuresetPageToolPath, "FrontEnd", "StyleSheets"}]
    ];
    stylesheetpath = Prepend[ CurrentValue[$FrontEnd, StyleSheetPath], DocumentationBuild`$FeaturesetStylePath];
    SetOptions[$FrontEnd, StyleSheetPath -> stylesheetpath]
  ];
  AppendTo[$Path, featuresetPageToolDir];

  (* AppendTo[pf, inFile]; *)

  dprint["InFile: " <> inFile ];
  dprint["OutputFile: " <> outFile ];
  dprint["PageType: Featureset"];
  dprint["FrontEnd Memory: " <>ToString[InputForm @ DocumentationBuild`Common`PubsEvaluateWithFE[MathLink`CallFrontEndHeld[FrontEnd`Value[FEPrivate`MemoryInUse]]/1024./1024.]]<>" Megabytes"];
  dprint["FrontEnd ProcessID: " <>ToString[DocumentationBuild`Common`PubsEvaluateWithFE[SystemInformation["FrontEnd", "ProcessID"]]]];
  dprint["Kernel Memory: " <>ToString[MemoryInUse[]/1024./1024.]<>" Megabytes"];
  dprint["Kernel ProcessID: " <>ToString[$ProcessID]];
  dprint[" "];

  dprint["FeaturesetPages2.m: calling ExportFeaturesetWebPage[_String, _String, _String]; args:"];
  dprint["  outFile = "<>ToString[outFile, InputForm]];
  dprint["  inFile = "<>ToString[inFile, InputForm]];
  dprint["  language = "<>ToString[language, InputForm]];
(* ** call ExportFeaturesetWebPage[outFile, inFile, language] *)
  time = AbsoluteTiming[ 
    res = DocumentationBuild`Common`PubsEvaluateWithFE[ExportFeaturesetWebPage[outFile, inFile, language]]
  ];
  (* dprint["time = "<>ToString[time, InputForm]]; *)

  (* Output file test *)
  dprint["point 1"];
  dprint["FeaturesetPages2.m: outFile = "<>ToString[outFile, InputForm]];
  If[(FileType[outFile] === None || ByteCount[outFile] < 1 || res === $Failed)
  , dprint["*** Output file test failed *** "];
    res = $Failed;
    failedFilesExpr++;
    AntSetProperty["failedFiles", ToString@failedFilesExpr];
    dprint["*** Failed *** "]
  , dprint["*** Success ***"]
  ];
  dprint["point 2"];

  Throw[time]
]];

dprint["finaltime = "<>ToString[finaltime, InputForm]];

(* dprint["killing feprocess: "<>ToString[DocumentationBuild`Common`PubsEvaluateWithFE[SystemInformation["FrontEnd", "ProcessID"]]]]; *)
(* Run["kill -9 "<>ToString[DocumentationBuild`Common`PubsEvaluateWithFE[SystemInformation["FrontEnd", "ProcessID"]]]]; *)

