AntLog["Entering MakeMathematicaHTML1.m: html partitioning"];
(* full messages *)

$MessagePrePrint = .;
SetDirectory[AntProperty["basedir"]];
AntLog["_"];

(* specific to osx - generalize later *)
AntLog[Import["!system_profiler SPSoftwareDataType", "Text"] <> "\n"];

(* Configure Front End *)
If[AntProperty["frontEndLaunchFlags"] =!= Null,
  SetOptions[ Developer`InstallFrontEnd, Developer`LaunchFlags -> AntProperty["frontEndLaunchFlags"]]
];

(* kill if reference.nb is 0k or missing *)
If[
  Not @ AllTrue[Positive][ (* Positive[n] returns False for both 0 and $Failed *)
    FileByteCount /@ {
      FileNameJoin[{$InstallationDirectory, "SystemFiles", "FrontEnd", "StyleSheets", "Wolfram", "Reference.nb"}],
      FileNameJoin[{$InstallationDirectory, "SystemFiles", "FrontEnd", "StyleSheets", "Wolfram", "Reference-ja.nb"}],
      FileNameJoin[{$InstallationDirectory, "SystemFiles", "FrontEnd", "StyleSheets", "Wolfram", "Reference-zh.nb"}]
    }
  ]
,
  AntLog["One or more of {Reference.nb, Reference-ja.nb, Reference-zh.nb} is corrupted or missing.  Terminating."];
  AntLog["Build failed"];
  (* Quit with a non-zero exit code *)
  Quit[1]
];

(* Add apps to $Path*)
If[
  Not @ MemberQ[$Path, AntProperty["appPath"]],
  PrependTo[$Path, AntProperty["appPath"]]];

(* Make the locally checked-out DocumentationBuild override the DocumentationBuild
   which is included in 12.2 (via PacletTools). See bug #399891. *)
PacletDirectoryLoad[AntProperty["appPath"]];

(* User variables *)
inputDir = AntProperty["inputDir"];
outputDir = AntProperty["outputDir"];
language = AntProperty["language"];
linkBase = AntProperty["app.name"];
buildgroup = AntProperty["buildgroup"];
buildFileGroup1 = AntProperty["buildFileGroup1"];
buildFileGroup2 = AntProperty["buildFileGroup2"];
buildFileGroup3 = AntProperty["buildFileGroup3"];
buildNumber = AntProperty["buildNumber"];
buildmode = AntProperty["buildMode"];
mydebug = ToExpression[AntProperty["mydebug"]];

singleFile = AntProperty["singleFile"];

(* See https://ant.apache.org/ant_task_guidelines.html for ant's $-handling.
Use MemberQ[{list}, string] for easy additions to {list} in the future. *)
If[MemberQ[{"$Media.nb"}, FileNameTake[singleFile]],
  singleFile = FileNameJoin[{FileNameDrop[singleFile], "$$Media.nb"}]];

RobotTools`Java9JDK = AntProperty["java9JDK"];
AntLog["RobotTools`Java9JDK = "<>ToString[RobotTools`Java9JDK, InputForm]];

AntLog["MakeMathematicaHTML1.m: 'inputDir' = "<>ToString[inputDir, InputForm]];
(* inputDir is "/Volumes/Jenkins/workspace/Web.Reference-Part1.English.12.1/Mathematica" *)

(* For FindFile["dir/file.ext"] to find file.ext, the *parent of dir* must be
mentioned explicitly in $Path.  FindFile does not do recursive searches through
directory trees as you might expect if you've used gnu or bsd 'find'.

Demonstration:

$ mkdir -p /tmp/dir1/dir2/ExampleData

$ touch /tmp/dir1/dir2/ExampleData/asdf.asdf

$ math
Mathematica 12.1.0 Kernel for Mac OS X x86 (64-bit)
Copyright 1988-2019 Wolfram Research, Inc.

In[1]:= FindFile["ExampleData/asdf.asdf"]
Out[1]= $Failed

In[2]:= Block[{$Path = {"/tmp/"}}, FindFile["ExampleData/asdf.asdf"]]
Out[2]= $Failed

In[3]:= Block[{$Path = {"/tmp/dir1"}}, FindFile["ExampleData/asdf.asdf"]]
Out[3]= $Failed

In[4]:= Block[{$Path = {"/tmp/dir1/dir2"}}, FindFile["ExampleData/asdf.asdf"]]
Out[4]= /private/tmp/dir1/dir2/ExampleData/asdf.asdf
*)

(* Locate an ExampleData directory, preferring the official one under
$InstallationDirectory if it exists.  If it does not exist, continue searching
through an "AlternativeDirectoryList". *)

Options[getExampleDataFolder] = {"AlternativeDirectoryList" -> {"/tmp/"}};

getExampleDataFolder[OptionsPattern[]] := Module[
  {tmp, pathElement, optAlternativeDirectoryList},

  optAlternativeDirectoryList = OptionValue["AlternativeDirectoryList"];

  (* first, look for a known ExampleData/file.ext in $Path.  If a real & official
  ExampleData dir exists, we don't need to look elsewhere. *)

  tmp = FindFile["ExampleData/car.mp3"]; (* returns a string or $Failed *)

  pathElement = If[
    FileExistsQ[tmp]
  ,
    (* then we've found a file in a working ExampleData dir somewhere in $Path.
    It's possible that $Path may be munged later so this ExampleData dir would
    no longer be findable, so save its *parent* to be prepended to $Path when
    needed later in the build.  See accompanying discussion of FileFile
    above. *)
    FileNameDrop[tmp, -2]
  ,
    (* we have no standard ExampleData dir (hello, jenkins).  RE has put one for
    our convenience in the HighResolution nb build's artifact, which lives under
    'inputDir'.  To find it: getExampleDataFolder["AlternativeDirectoryList" ->
    {inputDir}] *)

    tmp = FileNames["ExampleData", optAlternativeDirectoryList, Infinity]; (* returns a list of "ExampleData" dir paths, or an empty list *)
    If[
      (* On using 'First' here: could we ever be looking for one of many
      ExampleData dirs?  Maybe, but ignore that possibility for now.  If multiple
      ones are returned, just use the first one found. *)
      StringQ @ Quiet @ First @ tmp && FileExistsQ[First @ tmp]
    ,
      FileNameDrop[First @ tmp] (* save the dir's parent for reasons discussed above *)
    ,
      $Failed
    ]
  ];

  (* this is either $Failed or a path to a dir that contains ExampleData *)
  pathElement
];

(* inputDir is "/Volumes/Jenkins/workspace/Web.Reference-Part1.English.12.1/Mathematica" *)
exampleDataPath = getExampleDataFolder["AlternativeDirectoryList" -> {inputDir}];

If[exampleDataPath =!= $Failed, AntSetProperty["exampleDataPath", exampleDataPath]];
AntLog["AntProperty[\"exampleDataPath\"] = "<>ToString[AntProperty["exampleDataPath"], InputForm]];

(* Load DocumentationBuild *)
(* Needs["Transmogrify`Debug`"]; *)
If[FailureQ @ Needs["DocumentationBuild`"],
  AntFail["Needs[\"DocumentationBuild`\"] failed."];
];

appPath = AntProperty["appPath"];

(* SetOptions[$Debug,  *)
(*   Trace -> {Expressions,(\*NBProfile,ImageCreation,*\)Recursion,  *)
(*     TransformParsing, TransformRules}, TraceHandler -> Save,  *)
(*   TraceFile -> "/tmp/trace.m", Verbosity -> 99]; *)

DocumentationBuild`Transform`debug =
  DocumentationBuild`Export`$DocumentationExportDebug =
  ToExpression @ AntProperty["debug"];

  (* Initialize various $-variables in DocumentationBuild`Settings`. *)
  DocumentationBuild`Internal`LoadInternalSettings[];

(* DocumentationBuild`Export`$DocumentationExportDebug = True; *)
(* Global`$AntBuildQ = True; *)

feMemory = InputForm @
  DocumentationBuild`Common`PubsEvaluateWithFE[MathLink`CallFrontEndHeld[FrontEnd`Value[FEPrivate`MemoryInUse]]];
AntSetProperty["FEMemory", ToString@feMemory];

AntLog @ StringJoin[
  "M- Version: ", ToString[Internal`$CreationID], " ", ToString@$Version, "\n",
  "ReleaseID: ", ToString[SystemInformation["Kernel", "ReleaseID"]], "\n",
  "Current FE memory: ", ToString @ feMemory, "\n",
  "Installation Directory: ", ToString@$InstallationDirectory, "\n",
  "DocumentationBuild Directory: ", DocumentationBuild`Common`$DocumentationBuildDirectory, "\n",
  "DocumentationAssets Directory: ", DocumentationBuild`Internal`InternalAssetsDirectory[], "\n"
];

(* Test Import/Export convertors *)
If[MatchQ[ImportString[ExportString["<tag>1</tag>", "XML"], "XML"], $Failed | Null],
  AntFail["Export convertor failure. Unable to Import/Export XML!"]
];

AntLog["point 2"];

If[!DirectoryQ[inputDir],
  AntFail["inputDir does not exist: " <> inputDir];
];

AntLog["Notebook folder: "<>ToString[FileNameJoin[FileNameSplit[AbsoluteFileName[inputDir]]]]];

(* AntLog["Notebook files: "<>ToString[FileNames["*", FileNameJoin[FileNameSplit[AbsoluteFileName[inputDir]]], Infinity]]]; *)

AntLog["inputDir (source of webfiles list) is "<>ToString[inputDir, InputForm]];
webfiles = FileNames["*.nb", FileNameJoin[FileNameSplit[AbsoluteFileName[inputDir]]], Infinity];
(* remove dotfiles: https://jira.wolfram.com/jira/browse/RE-510387 *)
webfiles = Select[webfiles, StringContainsQ[FileNameTake[#], StartOfString ~~ Except["."]]&];
(* filesnames that contain these patterns should never be built - they aren't documentation notebooks *)
webfiles = DeleteCases[webfiles, s_String /; StringContainsQ[s, "/LocalPalettes/"], All];

packagemirrorfiles = Select[webfiles, StringMatchQ[#, "*" ~~ ($PathnameSeparator | "/") ~~ "PackagesMirror" ~~ "*"] &];

(* remove files which aren't under 'language' *)
webfiles = Select[webfiles, StringMatchQ[#, "*" ~~ ($PathnameSeparator | "/") ~~ language ~~ "*"]& ];

(* remove files which aren't notebooks *)
webfiles = Select[webfiles, StringMatchQ[#, "*.nb"]& ];

globalExcludeList = {
   FileNameJoin[{"Guides", "Mathematica.nb"}]
 , FileNameJoin[{"System", "ExampleData"}]
 , "FirstFiveMinutesWithMathematica.nb"
 , "VirtualBookContents.nb"
 , "FullNotationPalette.nb"
 , "NotationPalette.nb"
 , "HandsOnFirstLook01.nb"
 , "HandsOnFirstLook02.nb"
 , "HandsOnFirstLook03.nb"
 , "HandsOnFirstLook04.nb"
 , "HandsOnFirstLook05.nb"
 , "HandsOnFirstLook06.nb"
 , "HandsOnFirstLook07.nb"
 , "HandsOnFirstLook08.nb"
 , "HandsOnFirstLook09.nb"
 , "HandsOnFirstLook10.nb"
 , "HandsOnFirstLook11.nb"
 , "HandsOnFirstLook12.nb"
 , "HandsOnFirstLook13.nb"
 , "HandsOnFirstLook14.nb"
 , "HandsOnFirstLook15.nb"
 , "HandsOnFirstLook16.nb"
 , "HandsOnFirstLook17.nb"
 , "HandsOnFirstLook18.nb"
 , "HandsOnFirstLook19.nb"
 , "HandsOnFirstLook20.nb"
 , "HandsOnFirstLook21.nb"
 , "HandsOnFirstLook22.nb"
 , "HandsOnTour01.nb"
 , "HandsOnTour02.nb"
 , "HandsOnTour03.nb"
 , "HandsOnTour04.nb"
 , "HandsOnTour05.nb"
 , "HandsOnTour06.nb"
 , "HandsOnTour07.nb"
 , "HandsOnTour08.nb"
 , "HandsOnTour09.nb"
 , "HandsOnTour10.nb"
 , "HandsOnTour11.nb"
 , "HandsOnTour12.nb"
 , "HandsOnTour13.nb"
 , "HandsOnTour14.nb"
 , "HandsOnTour15.nb"
 , "HandsOnTour16.nb"
 , "HandsOnTour17.nb"
 , "QuickOverview.nb"
 };

(* remove files which are excluded *)
If[
  buildmode === "Future"
,
  (* buildmode==="Future": true clause *)
  webfiles = Select[webfiles, !StringMatchQ[#, "*" ~~ ($PathnameSeparator | "/") ~~ "ExampleData" | "PackagesMirror" | "Legacy" ~~ "*"]& ];
  AntLog["Choosing "<>DateString["DayName"] <>" Files"];
  webfiles = Switch[DateString["DayName"],
    "Monday", Take[webfiles, {1, 1730}],
    "Tuesday", Take[webfiles, {1730, 3462}],
    "Wednesday", Take[webfiles, {3463, 5193}],
    "Thursday", Take[webfiles, {5194, 6924}],
    "Friday", Take[webfiles, {6925, 8655}],
    "Saturday", Take[webfiles, {8656, 10386}],
    "Sunday", Take[webfiles, {10387, Length[webfiles]}],
    _, webfiles
  ];
  excisednblist ={};
  excluded = Map[($PathnameSeparator ~~ #)&, Join[excisednblist , globalExcludeList]];
,
  (* buildmode==="Future": false clause *)
  excluded = Map[($PathnameSeparator ~~ #)&, globalExcludeList]
];
webfiles = (DeleteCases[webfiles, f_ /; ! StringFreeQ[f, Alternatives @@ #, IgnoreCase -> True]] &[excluded]);

Print["Number of files to process: Length[webfiles] = "<>ToString[Length[webfiles], InputForm]];

(* partition webfiles to distribute runtime evenly over all parts *)

AntLog["point 3"];

(* if no packages are present in webfiles, then use the first partitioning below;
otherwise, use the second.  note that all & only package filenames contain the
string "/Packages/" *)

(* new in October 2017: same partitioning for all languages, which is what
I've been doing by hand for the past few years. *)

(* Which's first two clauses - if we're attempting to build:
 - just one file by setting the .properties file's 'singleFile', or
 - some number of files stuffed into the .properties file's 'inputDir'
then don't try to partition those lists - just process them as-is *)

webfiles = Which[
  FileExistsQ[singleFile], {singleFile}
,
  (* 9000 is an arbitrary large number that's less than the typical length of
  webfiles but allows for lots of ad-hoc files to be built from inputDir *)
  Length[webfiles] < 9000, webfiles
,
  True, If[
    Positive @ Count[webfiles, s_String /; StringContainsQ[s, "/Packages/"], All]
  ,
    (* this is what normally happens with a web documentation build *)
    Print["Partitioning 'webfiles' into 8 builds"];

(*
 - build times are in build-time-db.m, updated periodically.
 - filenames in the database are language-agnostic.
 - given the list webfiles = {file1, file2, ...}, lookup the build time of each member of the list to arrive at {time1, time2, ...}.
 - calculate a partitioning specification to split 'webfiles' into 8 lists of approximately equal build time
 - apply those partitioning specs here *)
 
    partitionSpecs = Module[{manifest, db},

      (* make 'webfiles' system-agnostic and language-agnostic by removing the
      filesystem location and changing, e.g., "English" to "Language", for
      lookup in the language-agnostic build-time database. *)
      manifest = StringReplace[webfiles, {StartOfString ~~ Shortest[__] ~~ $PathnameSeparator ~~ language -> $PathnameSeparator ~~ "Language", language -> "Language"}];

      dprint["some filenames from manifest: "<>ToString[RandomChoice[manifest, 5], InputForm]];

      db = Import[FileNameJoin[{appPath, "DocumentationBuild", "Scripts", "BuildPartitioning", "build-time-db.m"}]];
      (* if a filename isn't in the database, assume a default build time of 20 seconds *)
      manifest = Lookup[db, manifest, 20]; (* convert filenames to build times *)
      res = partitionList[manifest];
      Print["Average build time per notebook is "<>ToString[IntegerPart @ Mean[manifest]]<>" seconds"];
      res
    ];

    Assert[MemberQ[{"1", "2", "3", "4", "5", "6", "7", "8"}, buildgroup]];
    Take[webfiles, Part[partitionSpecs, ToExpression[buildgroup]]]
  ,
    Print["partitioning without packages"];
    Switch[buildgroup,
      "1", Take[webfiles, {1, 5305}]
    , "2", Take[webfiles, {5306, 6027}]
    , "3", Take[webfiles, {6028, 6947}]
    , "4", Take[webfiles, {6948, 7726}]
    , "5", Take[webfiles, {7727, 8271}]
    , "6", Take[webfiles, {8272, 8941}]
    , "7", Take[webfiles, {8942, 9809}]
    , "8", Take[webfiles, {9810, Length[webfiles]}]
    , _, webfiles
    ]
  ]
];

AntLog["point 4"];

AntSetProperty["numberTotalWebNotebooks", ToString@Length@webfiles];
AntLog["___"];
AntLog["Web Notebooks to process: "<>ToString@Length[webfiles]];
AntLog["First: "<>ToString[First[webfiles, InputForm]]];
AntLog["Last: "<>ToString[Last[webfiles, InputForm]]];
AntLog["___"];

webfiles = ToString@Sequence[webfiles]; (* AntLog["HTMLPartitioning.m: webfiles step 1 = "<>ToString[webfiles, InputForm]]; *)
webfiles = StringTake[StringReplace[webfiles, RegularExpression["\\s*,\\s*"]->"," ], {2,-2}]; (* AntLog["HTMLPartitioning.m: webfiles step 2 = "<>ToString[webfiles, InputForm]]; *)
AntSetProperty["webnotebooks", webfiles ];

AntSetProperty["webfilesCounter", "0"];
AntSetProperty["webfilesTotal", ToString[Length[webfiles]]];

(* AntLog["Building future landing page"];
MakeLandingContent[inputDir, FileNameJoin[Flatten[{FileNameSplit[outputDir], "mathematica.en.html"}]]]; *)

If[buildmode==="Future",
  AntLog["Building future landing page"];
  MakeLandingContent[inputDir, FileNameJoin[Flatten[{FileNameSplit[outputDir],"mathematica.en.html"}]]]
];

If[buildmode==="Future",
  AntLog["Adding .htaccess file"];
  CopyFile[
    DocsAsset["Internal:Web", FileNameJoin[{"html", "mathematica.htaccess"}]],
    FileNameJoin[Flatten[{FileNameSplit[outputDir],".htaccess"}]]
  ];
];

If[buildmode==="Future",
  AntSetProperty["buildtimelimit", "9000"],
  AntSetProperty["buildtimelimit", "9000"]
];

AntLog["point 5 and exit"];
AntLog["leaving HTMLPartitioning.m"];
AntLog["_"];
