(* * MakeMathematicaHTML2.m *)
(* debugOn[]; *)
AntLog["Entering MakeMathematicaHTML2.m: html conversion"];

webindividualFilename = AntProperty["webindividualFilename"];

(* See https://ant.apache.org/ant_task_guidelines.html for ant's $-handling.
Use MemberQ[{list}, string] for easy additions to {list} in the future. *)
If[MemberQ[{"$Media.nb"}, FileNameTake[webindividualFilename]],
  webindividualFilename = FileNameJoin[{FileNameDrop[webindividualFilename], "$$Media.nb"}]];

Catch[

  (* Log time *)
  AntLog["___"];
  AntLog@DateString[{"Hour", ":", "Minute", ":", "Second", " ", "MonthName", " ", "Day", ", ", "Year"}];

(* ** poor man's progress meter *)
  AntSetProperty["webfilesCounter", ToString[1 + ToExpression[AntProperty["webfilesCounter"]]]];
  AntLog[
    StringJoin[
      "Processing notebook "
    ,  AntProperty["webfilesCounter"]
    , " of "
    , AntProperty["numberTotalWebNotebooks"]
    , "; "
    , ToString[
        NumberForm[
          100 * N[ToExpression[AntProperty["webfilesCounter"]] / ToExpression[AntProperty["numberTotalWebNotebooks"]]]
        , {Infinity, 2}
        , DigitBlock -> 3
        , ExponentFunction -> (Null &)
        ]
      ]
    , "%"
    ]
  ];

(* ** fe reset for difficult files *)

  feResetList = {
    "/System/ReferencePages/Symbols/Graphics3D.nb"
  };

  If[
    StringContainsQ[webindividualFilename, feResetList]
  ,
    (
      AntLog["https://jira.wolfram.com/jira/browse/WLDOCS-2049: running UninstallFrontEnd in preparation for difficult files"];
      Developer`UninstallFrontEnd[];
      AntLog["Finished quitting FE"];
    )
  ];

(* ** miscellaneous variables *)

  (* full messages *)
  $MessagePrePrint = .;

  appPath = AntProperty["appPath"];

  $RecursionLimit = 1000;

  (* Set requests variable *)
  If[IntegerQ[requests], requests++, results=1];

(* ** set up $Path and load the DocumentationBuild package *)

  (* Add apps to $Path*)
  If[!MemberQ[$Path, appPath], PrependTo[$Path, appPath]];

  (* Make the locally checked-out DocumentationBuild override the DocumentationBuild
    which is included in 12.2 (via PacletTools). See bug #399891. *)
  PacletDirectoryLoad[AntProperty["appPath"]];

  (* Add ExampleData to $Path*)
  exampleDataPath = AntProperty["exampleDataPath"];
  If[!MemberQ[$Path, exampleDataPath], PrependTo[$Path, exampleDataPath]];
  $Path = DeleteDuplicates[$Path];
  (* AntLog["MakeMathematicaHTML2.m: exampleDataPath: "<>ToString[exampleDataPath, InputForm]]; *)
  (* AntLog[StringJoin["$Path:\n", ToString[TableForm[StringJoin["   "<>#]& /@ $Path, TableSpacing -> {0, 3}]], "\n"]]; *)
  (* AntLog["FindFile[\"ExampleData/car.mp3\"] returns: "<>ToString[FindFile["ExampleData/car.mp3"], InputForm]]; *)

  (* Load DocumentationBuild *)
  Needs["DocumentationBuild`"];

(* ** variables for directories, files, language; find the nb's url *)

  (* User variables *)

  (*
  On FileNameJoin[FileNameSplit[dir]], see the discussion starting at
  https://mail-archive.wolfram.com/archive/l-programming/2019/Dec00/0012.html
  
  This code may run on windows, and it may receive unix-style file pathnames
  supplied by an ant config file.  This normalizes those pathname separators for
  windows runs.  It also drops a trailing $PathnameSeparator on mac:

  In[5]:= {$OperatingSystem, FileNameJoin[FileNameSplit["/a/b/c/"]]}//InputForm
  Out[5]//InputForm= {"MacOSX", "/a/b/c"} 
  *)

  webInputDir = FileNameJoin[FileNameSplit[AntProperty["inputDir"]]];
  AntLog["webInputDir: "<>webInputDir];

  webOutputDir = FileNameJoin[FileNameSplit[AntProperty["outputDir"]]];
  AntLog["webOutputDir: "<>webOutputDir];

  webinFile = FileNameJoin[FileNameSplit[webindividualFilename]];
  AntLog["webinFile: "<>webinFile];

  (*weboutFile = StringReplace[webinFile, {webInputDir->""}] ;
  weboutFile = DocumentationBuild`Common`FilePathToDocURI[StringJoin[webOutputDir, StringReplace[weboutFile, {".nb" -> ""}], ".html"], $PathnameSeparator];*)

  language = AntProperty["language"];
  AntLog["language: "<>language];

  langext = Switch[language,
    "ChineseSimplified", ".zh",
    "Spanish", ".es",
    "Japanese", ".ja",
    "Korean", ".ko",
    "Russian", ".ru",
    "Portuguese", ".pt-br",
    _, ".en"
  ];
  AntLog["langext: "<>langext];

	If[!FileExistsQ[webinFile],
		AntLog["FAILURE: input notebook "<>webinFile<>" does not exist.  Nothing more can be done with this file."];
		Throw["exit: notebook does not exist."]
	];

	weboutURI = NestedLookup[
		NotebookTools`NotebookFileOptions[webinFile],
		{TaggingRules, "Metadata", "uri"},
		(
			AntLog["FAILURE: "<>webinFile<>" does not contain a uri.  Nothing more can be done with this file."];
			Throw["exit: notebook has no uri."];
		)
	];

	If[!StringQ[weboutURI] || weboutURI === "uri",
		AntLog["FAILURE: "<>webinFile<>" does not contain a uri.  Nothing more can be done with this file."];
		Throw["exit: notebook has no uri."]
	];

  (* leave this here temporarily as a reminder in case it's needed in a
     workflow-development emergency:
  If[
    weboutURI == "none" && StringContainsQ[webinFile, "/Workflows/"],
    weboutURI = "workflow/"<>FixedPoint[FileBaseName, FileNameTake[webinFile]]
  ]; *)

  weboutBasename = weboutURI<>".html"<>langext;

  weboutFile = FileNameJoin[{
    webOutputDir,
    weboutBasename
  }];

  weboutFilesDir = FileNameJoin[Flatten[{
    webOutputDir,
    Most[FileNameSplit[weboutBasename]],
    "Files",
    StringReplace[Last[FileNameSplit[weboutBasename]], ".html" -> ""]
  }]];

  AntLog["weboutURI: "<>weboutURI];
  AntLog["weboutBasename: "<>weboutBasename];
  AntLog["weboutFile: "<>weboutFile];
  AntLog["weboutFilesDir: "<>weboutFilesDir];

(* ** skip pages *)

(* Maintain a list of pages that cause problems in builds - if current nb
matches, Throw.  For list format, see assetList below.  This works exactly the
same way.  The purpose here is to get the file out of the way so a build can
finish, then go back and investigate/fix things later. 

Many of these can be made to run by bitmapping their Output cells.  Devise a way
to do that to a given notebook while building, and perhaps change this to a
bitmapList.
*)

(* notes for ja 12.3.1
 - first, without repartitioning the ja build, reduce the build time of all 8 builds.
 - then also skip the rest of the top 10%
 - if this significantly speeds up builds on all machines,
perhaps look into adding a separate part for the expensive pages, or splitting
others into 7 builds and put the most expensive in part 1 (to ensure it runs
first) *)

skipList = {};

If[StringContainsQ[webinFile, skipList],
  AntLog["Skipping skipList item "<>webinFile];
  Throw["Skipping "<>webinFile]];

(* ** AdditionalAssets *)

  (* 

  If a page causes problems in the build or can't be built, add it to assetList
  and put a pre-built copy of it and all its files in
  DocsAsset["Internal:Web", "AdditionalAssets"].  Then during the build the pre-built
  page and its files will be copied into the layout.

  Note that you add (part of) the *notebook* wholename.  For example,
  Guides/WolframRoot and not guide/WolframRoot.

  The DocumentationBuild maintainer has to ensure that html versions of
  these are available and up-to-date in DocsAsset["Internal:Web", "AdditionalAssets"].

  Sample entries for assetList from previous versions of this file:
    , "/System/ReferencePages/Symbols/GraphData.nb"
    , "ReferencePages/Symbols/ImageResize.nb"
    , "Guides/WolframRoot.nb"

   - use notebook filenames, not web filenames.
   - for nbs whose basenames aren't unique, it's safest to specify a leading directory name. 
   - doc files with non-unique basenames:
     x = DeleteCases[GatherBy[Select[FileNames["*.nb", $InstallationDirectory, Infinity], StringContainsQ[#, "/Documentation/English/"]&], FileBaseName], {_}, Infinity];
  *)

  assetList = {
    "WolframRoot.nb"
  , "/System/Guides/RecentlyAddedFeatures.nb"
  , "/System/ReferencePages/Symbols/WebAudioSearch.nb" (* many pink fails *)
  , "/System/ReferencePages/Symbols/DynamicGeoGraphics.nb" (* $Failed appears in outputs *)
  , "/System/ReferencePages/Symbols/DiscretizeRegion.nb" (* timed out *)
  };

  (* when used as a StringExpression pattern here, assetList is equivalent to Alternatives[string1, string2, ...] *)
  If[StringContainsQ[webinFile, assetList]
  ,
    assetStart = Now;
    AntLog["Begin asset code..."];
    AntLog[webinFile<>" is a member of the asset list"];

	assetFile = StringReplace[weboutFile, {
		webOutputDir -> DocumentationBuild`Common`DocsAsset["Internal:Web", "AdditionalAssets"]
	}];
    AntLog["  assetFile = "<>ToString[assetFile, InputForm]];

    assetFilesDir = StringReplace[weboutFilesDir, {
		webOutputDir -> DocumentationBuild`Common`DocsAsset["Internal:Web", "AdditionalAssets"]
	}];
    AntLog["  assetFilesDir = "<>ToString[assetFilesDir, InputForm]];

    (* if weboutfile exists, delete it; if its parent directory does not exist, create it *)
    If[FileExistsQ[weboutFile], DeleteFile[weboutFile]];
    If[Not[FileExistsQ[DirectoryName[weboutFile]]], CreateDirectory[DirectoryName[weboutFile]]];

    (* if weboutfilesdir exists, delete it; if its parent directory does not exist, create it *)
    If[FileExistsQ[weboutFilesDir], DeleteDirectory[weboutFilesDir, DeleteContents -> True]];
    If[Not[FileExistsQ[DirectoryName[weboutFilesDir]]], CreateDirectory[DirectoryName[weboutFilesDir]]];

    (* ---------------------------------------------------------------------- *)
    (* copy 1: copy the asset page's html file and report results *)

    copyfileResult = If[FileExistsQ[assetFile]
    ,
      AntLog["  copy "<>assetFile<>" to "<>weboutFile];
      CopyFile[assetFile, weboutFile]
    ,
      AntLog[assetFile<>" doesn't exist - investigate"];
      $Failed
    ];
    AntLog["  CopyFile return value: "<>ToString[copyfileResult, InputForm]];
    AntLog["  weboutFile "<>weboutFile<>If[FileExistsQ[weboutFile], " exists, carry on.", " does not exist; investigate."]];

    (* ---------------------------------------------------------------------- *)
    (* copy 2: copy the asset page's Files dir and report results *)

    copydirResult = If[
      FileExistsQ[assetFilesDir]
    ,
      AntLog["  copy "<>assetFilesDir<>" to "<>weboutFilesDir];
      CopyDirectory[assetFilesDir, weboutFilesDir]
    ,
      AntLog[assetFilesDir<>" doesn't exist - if it should exist, investigate"];
      $Failed
    ];
    AntLog["  CopyFile return value: "<>ToString[copydirResult, InputForm]];
    AntLog["  weboutFilesDir "<>weboutFilesDir<>If[FileExistsQ[weboutFilesDir], " exists, carry on.", " does not exist; investigate."]];

    (* ---------------------------------------------------------------------- *)
    (* find and delete any copied CVS directories *)

    cvsDirs = FileNames["CVS", weboutFilesDir, Infinity];
    If[
      Positive @ Length @ cvsDirs
    ,
      AntLog[ToString[Length[cvsDirs]]<>" CVS "<>If[1==Length[cvsDirs], "directory", "directories"]<>" to be deleted:"];
      AntLog[#]& /@ cvsDirs;
      DeleteDirectory[#, DeleteContents -> True]& /@ cvsDirs;
    ,
      AntLog["  there are no CVS directories in the copied asset."];
    ];

    AntLog["Asset timing: "<>TextString[DateDifference[assetStart, Now, "Second"]]];
    AntLog["Asset copied: "<>weboutFile];
    AntLog["End asset code."];

(* ** build code *)
  ,
    exitCodePath = AntProperty["exitCodePath"];
    AntLog["  exitCodePath: "<>exitCodePath];

    linkBase = AntProperty["app.name"];
    AntLog["  linkBase: "<>linkBase];

    buildmode = AntProperty["buildMode"];
    AntLog["  buildmode: "<>buildmode];

    buildNumber = AntProperty["buildNumber"];
    AntLog["  buildNumber: "<>buildNumber];

    feprocessid = ToString[DocumentationBuild`Common`PubsEvaluateWithFE[SystemInformation["FrontEnd", "ProcessID"]]];

    DocumentationBuild`Common`PubsEvaluateWithFE[SetOptions[$FrontEnd, NotebookSecurityOptions->{"TrustByDefault"->True}, RenderingOptions -> {"HardwareAntialiasingQuality" -> 1}]];
    DocumentationBuild`Common`PubsEvaluateWithFE[SetOptions[$FrontEnd, DynamicEvaluationTimeout -> 60]];
    DocumentationBuild`Common`PubsEvaluateWithFE[SetOptions[$FrontEnd, RenderingOptions -> {"HardwareAntialiasingQuality" -> 1}]];
    DocumentationBuild`Common`PubsEvaluateWithFE[SetOptions[$FrontEndSession, RenderingOptions -> {"HardwareAntialiasingQuality" -> 1}]];
    AntLog["  FrontEnd Memory: " <>ToString[InputForm @ DocumentationBuild`Common`PubsEvaluateWithFE[MathLink`CallFrontEndHeld[FrontEnd`Value[FEPrivate`MemoryInUse]]]]];
    AntLog["  Kernel Memory: " <>ToString@MemoryInUse[]];
    AntLog["  FrontEnd ProcessID: " <>feprocessid];
    AntLog["  Kernel ProcessID: " <>ToString[$ProcessID]];
    AntLog["  Quality Level: " <> ToString[InputForm @ DocumentationBuild`Common`PubsEvaluateWithFE[CurrentValue[$FrontEnd, {RenderingOptions, "HardwareAntialiasingQuality"}]]]];
    AntLog["  Processing: " <> FixedPoint[StringReplace[#, {webInputDir -> "", RegularExpression["^\\s*\\/(.+)"] -> "$1"}] &, webinFile]];

	(* Load the System core information. *)
	(* TODO: Do this only once, at the start of the HTML build, instead of loading it
	         in each separate <mathematica> instance that handles a single file. *)
	DocumentationBuild`Info`SetCoreInformation[language, Automatic, {"System"}];

	DocumentationBuild`Info`SetSystemEnglishSourceInformationIfNeeded["Mathematica", language];
	
	DocumentationBuild`Settings`$DocumentationBuildLanguage = language;

    If[
      buildmode === "Future"
    ,

      (* ----------------------------------- *)
      (* buildmode === "Future": true clause *)

      AntLog["  buildmode is Future; using ExportDocumentation"];
      time = AbsoluteTiming[
        res = DocumentationBuild`Common`PubsEvaluateWithFE @ ExportDocumentation[
          weboutFile,
          webinFile,
          "BuildMode"->"Future",
          "BuildNumber"->buildNumber,
          "ExportFormats"->"HTML",
          "Cache"->False,
          "CacheLocation"->webOutputDir,
          "Language"->language,
          "CompleteHTMLQ" -> False,
          "IsManipulate" -> True
        ]
      ];
      (* Output file test *)
      webexportfile = "HTML" /. res;
      AntLog["Exported: " <> webexportfile];
      If[(FileType[webexportfile]===None || FileByteCount[webexportfile]<1),
        AntLog["*** Output file test failed *** "];
        AntLog[webexportfile]
      ]
    ,

      (* ------------------------------------ *)
      (* buildmode === "Future": false clause *)

      AntLog["  buildmode is not Future; using ExportWebPageFromResultNB"];
      If[
        (FileType[weboutFile] === File)
      ,
        AntLog["  Skipping, output file already exists: "<>ToString[weboutFile, InputForm]];
      ,
        time2 = AbsoluteTiming[
          TimeConstrained[
            webexportfile = DocumentationBuild`Common`PubsEvaluateWithFE[
              DocumentationBuild`Export`ExportWebPageFromResultNB[
                weboutFile,
                webinFile,
                "Language" -> language,
                "IsManipulate" -> True,
                "Cache"->False,
                "CacheLocation"->webOutputDir,
                "BuildNumber"->buildNumber,
                "Debug" -> mydebug
              ]
            ];
          ,
            7000 (* jenkins timeout is 2h = 7200s, and it kills the entire build *)
          ,
            AntLog["  *** Build timeout ***  "]
          ];
        ];
        AntLog["  ExportWebPageFromResultNB timing: "<>ToString[time2[[1]]]];

        If[
          StringQ[webexportfile]
          && FileType[webexportfile] === File
          && Positive @ FileByteCount[webexportfile]
        ,
          AntLog["  ExportWebPageFromResultNB success, exported " <> ToString[webexportfile, InputForm]]
        ,
          AntLog["  *** Output file test failed *** "];
          Which[
            Not[StringQ[webexportfile]],
              AntLog["  ExportWebPageFromResultNB failure, return value is not a String: "<>ToString[webexportfile, InputForm]];
              Throw["ExportWebPageFromResultNB failure: not a String"]
          ,
            FileType[webexportfile] === None,
              AntLog["  ExportWebPageFromResultNB failure, file does not exist on the filesystem: "<>ToString[webexportfile, InputForm]];
              Throw["ExportWebPageFromResultNB failure: file does not exist"]
          ,
            FileByteCount[webexportfile] < 1,
              AntLog["  ExportWebPageFromResultNB failure, file exists but is empty: "<>ToString[webexportfile, InputForm]];
              Throw["ExportWebPageFromResultNB failure: file is empty"]
          ]
        ]
      ]
    ]
  ];

  (* after that, the output file exists and is not zero-length *)

  If[StringContainsQ[webexportfile, "file/wstp."],
    CopyFile[
		DocsAsset["Internal:Web", FileNameJoin[{"html", "file.htaccess"}]],
      FileNameJoin[Flatten[{Drop[FileNameSplit[webexportfile], -1], ".htaccess"}]]
    ]
  ];

  (* special handling for the root guide page: copy it from AdditionalAssets or the webpage that was just built *)
  If[Or @@ StringContainsQ[assetList, "WolframRoot"],

    (* we're processing WolframRoot when it's also in the assetList: *)
    If[StringContainsQ[webinFile, "WolframRoot"],
      AntLog["we're processing WolframRoot and it's in the assetList, so also copy the index.html.la asset"];
      Module[{rootHTMLFile},
        rootHTMLFile = "index.html"<>langext;
        If[
          FileExistsQ[FileNameJoin[{webOutputDir, rootHTMLFile}]],
          DeleteFile[FileNameJoin[{webOutputDir, rootHTMLFile}]]
        ];
        CopyFile[
          FileNameJoin[{DocumentationBuild`Common`DocsAsset["Internal:Web", "AdditionalAssets"], rootHTMLFile}],
          FileNameJoin[{webOutputDir, rootHTMLFile}]
        ]
      ]
    ]
  ,
    (* we're processing WolframRoot when it is *not* in assetList, so a value was assigned to webexportfile above. *)
    If[StringContainsQ[webinFile, "WolframRoot"],
      AntLog["  we're processing WolframRoot and it's not in the assetList, so copy guide/WolframRoot just built above."];
      CopyFile[
        webexportfile,
        FileNameJoin[Flatten[{Drop[FileNameSplit[webexportfile], -2], "index" <> ".html" <> langext}]]
      ];
      CopyFile[
        DocsAsset["Internal:Web", FileNameJoin[{"html", "mathematica.htaccess"}]],
        FileNameJoin[Flatten[{Drop[FileNameSplit[webexportfile], -2], ".htaccess"}]]
      ];
    ]
  ];

  (* remove guide/WolframRoot from the web layout *)
  If[
    StringContainsQ[webinFile, "WolframRoot"]
    && FileExistsQ[FileNameJoin[{webOutputDir, "guide/WolframRoot.html"<>langext}]]
  , DeleteFile[FileNameJoin[{webOutputDir, "guide/WolframRoot.html"<>langext}]]
  ];

  If[
    DocumentationBuild`Common`PubsEvaluateWithFE[MathLink`CallFrontEndHeld[FrontEnd`Value[FEPrivate`MemoryInUse]]] > 5*^8
  ,
    (
      AntLog[
        "Noticed the FE memory was "
        <> ToString[InputForm[DocumentationBuild`Common`PubsEvaluateWithFE[MathLink`CallFrontEndHeld[FrontEnd`Value[FEPrivate`MemoryInUse]]]]]
        <> " which is greater than 5*^8, will run UninstallFrontEnd to alleviate this issue..."
      ];
      Developer`UninstallFrontEnd[];
      AntLog["Finished quitting FE"];
    )
  ];
  AntLog["leaving HTMLConversion.m"];

  AntLog["___"];
] (* close outer Catch *)
