AntLog["Entering SourceInformationNotes.m"];

AntLog["Ant properties available to the Mathematica code:"];
AntLog["   AntProperty[\"appPath\"]                       = "<>ToString[AntProperty["appPath"], InputForm]];
AntLog["   AntProperty[\"scriptDir\"]                     = "<>ToString[AntProperty["scriptDir"], InputForm]];
AntLog["   AntProperty[\"language\"]                      = "<>ToString[AntProperty["language"], InputForm]];
AntLog["   AntProperty[\"NotesStylesheetPath\"]           = "<>ToString[AntProperty["NotesStylesheetPath"], InputForm]];
AntLog["   AntProperty[\"inputDir\"]                      = "<>ToString[AntProperty["inputDir"], InputForm]];
AntLog["   AntProperty[\"outputDir\"]                     = "<>ToString[AntProperty["outputDir"], InputForm]];
AntLog["   AntProperty[\"mathematica.installation.dir\"]  = "<>ToString[AntProperty["mathematica.installation.dir"], InputForm]];
AntLog["   AntProperty[\"jlinkpath\"]                     = "<>ToString[AntProperty["jlinkpath"], InputForm]];
AntLog["   AntProperty[\"dist\"]                          = "<>ToString[AntProperty["dist"], InputForm]];
AntLog["   AntProperty[\"log.dir\"]                       = "<>ToString[AntProperty["log.dir"], InputForm]];
AntLog["   AntProperty[\"JLinkLoaded\"]                   = "<>ToString[AntProperty["JLinkLoaded"], InputForm]];
(* AntLog["   $Path                                          = "<>ToString[$Path, InputForm]]; *)

PrependTo[$Path, AntProperty["appPath"]];

(* Make the locally checked-out DocumentationBuild override the DocumentationBuild
   which is included in 12.2 (via PacletTools). See bug #399891. *)
PacletDirectoryLoad[AntProperty["appPath"]];

<<DocumentationBuild`;

(* Workaround: https://bugs.wolfram.com/show?number=385974 *)
DocumentationBuild`SystemResourcesFromDocs`Private`loadSystemModel[];

DocumentationBuild`Common`PubsEvaluateWithFE[
  SetOptions[$FrontEnd, DynamicEvaluationTimeout -> 300];

  AntLog[ "DynamicEvaluationTimeout: " <> ToString@CurrentValue[$FrontEnd, DynamicEvaluationTimeout]];

  SetOptions[$FrontEnd, StyleSheetPath -> Inherited];

  DocumentationBuild`$NotesStylePath =
    With[
      {notesPath = FileNameJoin[FileNameSplit[AntProperty["NotesStylesheetPath"]]]}
    ,
      FrontEnd`FileName[{notesPath, "FrontEnd", "StyleSheets"}]
    ];
AntLog["DocumentationBuild`$NotesStylePath = "<>ToString[DocumentationBuild`$NotesStylePath, InputForm]];


  stylesheetpath = Prepend[
    CurrentValue[$FrontEnd, StyleSheetPath],
    DocumentationBuild`$NotesStylePath
  ];
(* AntLog["stylesheetpath = "<>ToString[stylesheetpath, InputForm]]; *)

  SetOptions[$FrontEnd, StyleSheetPath -> stylesheetpath]
];

kill := Module[{},
  AntLog["Reference.nb or Reference-{ja,zh}.nb is corrupted or missing. Terminating. "];
  (* Failed build *)
  AntLog["Build failed"];
  (* Quit with a non-zero exit code *)
  Quit[1]
];

enRef = FileNameJoin[{$InstallationDirectory, "SystemFiles", "FrontEnd", "StyleSheets", "Wolfram", "Reference.nb"}];
jaRef = FileNameJoin[{$InstallationDirectory, "SystemFiles", "FrontEnd", "StyleSheets", "Wolfram", "Reference-ja.nb"}];
zhRef = FileNameJoin[{$InstallationDirectory, "SystemFiles", "FrontEnd", "StyleSheets", "Wolfram", "Reference-zh.nb"}];

(* kill if Reference.nb and/or Reference-{ja,zh}.nb are empty or missing *)
If[Not[FileExistsQ[enRef]] || FileByteCount[enRef] == 0, kill[]];
Switch[lang,
  "Japanese",
  If[Not[FileExistsQ[jaRef]] || FileByteCount[jaRef] == 0, kill[]]
,
  "ChineseSimplified",
  If[Not[FileExistsQ[zhRef]] || FileByteCount[zhRef] == 0, kill[]]
];

fbcountReference = FileByteCount[enRef];

otherStyleSheets =
  FileNames["*.nb", FileNameJoin[{$InstallationDirectory, "SystemFiles", "FrontEnd", "StyleSheets"}], Infinity];

fbcountOtherStyles =
 FileByteCount /@ FileNames["*.nb", FileNameJoin[{$InstallationDirectory, "SystemFiles", "FrontEnd", "StyleSheets"}], Infinity];

(* AntLog["fbcountReference: " <> ToString[fbcountReference]]; *)
(* antLog["otherStyleSheets: " <> ToString[otherStyleSheets]]; *)
(* AntLog["fbcountOtherStyles: " <> ToString[fbcountOtherStyles]]; *)

buildInputDir = FileNameJoin[{AntProperty["inputDir"]}];
If[
  Not[FileExistsQ[AntProperty["outputDir"]]],
  CreateDirectory[AntProperty["outputDir"]]
];

ExportNotes[{inputNB_String, outputDirectory_String}, lang_String] := Module[
  {languageExt, outputHTML, res}
,
Block[{
	DocumentationBuild`Settings`$DocumentationBuildLanguage = lang
},
  AntLog["Language: " <> lang];
  languageExt = Switch[lang, "Japanese", ".ja", "ChineseSimplified", ".zh", _, ".en"];
  AntLog["LanguageExt: " <> languageExt];

  AntLog["Processing input notebook: " <> inputNB];

  outputHTML = FileNameJoin[{outputDirectory, "note", FileBaseName[inputNB]<>".html"<>languageExt}];
  AntLog["Output html file: " <> outputHTML];

  res = ExportWebPage[
    outputHTML,
    MakeNotebook[inputNB],
    GetNotebookInformation[inputNB, ExportFormat -> "HTML"]
  ];

  If[
    FileExistsQ[res],
    AntLog["file size:" <> ToString[FileByteCount[res], InputForm]],
    AntLog["did ExportWebPage fail?  Its return value is a filename when it succeeds, but the returned value does not exist as a file."]
  ]
]];

buildList = FileNames["*.nb", buildInputDir];
buildOutputDir = AntProperty["outputDir"];

releaseVersion = AntProperty["version"];

(* test each notebook for version and flag as is done for system notebooks *)

(* AntLog["initial buildList: " <> ToString[format@buildList]]; *)
AntLog["initial buildList length: " <> ToString[Length@buildList]];

buildList = Flatten @ Reap[Module[{nb, nbInfo, entityType, historyData, flag}, 
  nb = #;
  nbInfo = GetNotebookInformation[nb];
  entityType = "EntityType" /. nbInfo;
  historyData = "HistoryData" /. nbInfo;
  flag = "Flag" /. nbInfo;

  Which[
		entityType === $Failed,
      AntLog[nb];
			AntLog["            File's Entity Type not found -- file not included in build."];
		,

		(* pass over flagged nbs which shouldn't be included within distribution *)
		TrueQ[
			(DocumentationBuild`Make`Private`CheckVersion[
				("New" /. historyData),
				"ReleaseVersion" -> releaseVersion,
				"ThresholdVersion" -> Automatic
			] > 1) ||
			(DocumentationBuild`Make`Private`CheckVersion[
				("Modified" /. historyData),
				"ReleaseVersion" -> releaseVersion,
				"ThresholdVersion" -> Automatic
			] > 1)
		],
      AntLog[nb];
			AntLog["             -- file not included in build due to history data greater than " <> releaseVersion];
		,

		(* pass over flagged nbs which shouldn't be included within distribution *)
		MatchQ[flag, "ExcisedFlag" | "FutureFlag" | "InternalFlag" | "TemporaryFlag"],
      AntLog[nb];
			AntLog["            " <> flag <> " -- file not included in build."];
		,

		(* Convert everything else *)
		True, Sow[nb]
	];

]& /@ buildList][[2]];

(* AntLog["final buildList: " <> ToString[format@buildList]]; *)
AntLog["final buildList length: " <> ToString@Length[buildList]];

NotesBuildSuccessQ[outputDir_String, lang_String]:= Module[
  {languageExt, outList, inList}
,
  languageExt = Switch[lang, "Japanese", ".ja", "ChineseSimplified", ".zh", _, ".en"];

  (* a note on outList below:

  In[24]:= FileBaseName["/a/b/c.html.en"]
  Out[24]= c.html

  In[25]:= FileBaseName@FileBaseName["/a/b/c.html.en"]
  Out[25]= c
  *)

  outList = Sort[FileBaseName /@ FileBaseName /@ FileNames["*.html."<>languageExt, outputDir, 1]];
  inList = Sort[FileBaseName /@ buildList];

  (* if all and only the input files were converted to output files, success! *)
  outList === inList
];

NotesBuildSuccessQ[outputDir_String, lang_String]:= True;

BuildAllNotes[blist_List, outputDir_String, lang_String] := Module[
  {feMemory}
,
  (
    ExportNotes[{#, outputDir}, lang];
    feMemory = Developer`UseFrontEnd[MathLink`CallFrontEndHeld[FrontEnd`Value[FEPrivate`MemoryInUse]]];
    AntLog["FEMemory"<>ToString[feMemory, InputForm]]
  )& /@ blist

];

DocumentationBuild`Common`PubsEvaluateWithFE[
  BuildAllNotes[buildList, buildOutputDir, AntProperty["language"]]
];

NotesBuildSuccessQ[buildOutputDir, AntProperty["language"]];

AntLog["Build Succeeded?: "<> ToString@NotesBuildSuccessQ[buildOutputDir, AntProperty["language"]] ];

AntLog["OutputDir: "<> buildOutputDir];

(* full messages *)
$MessagePrePrint = .;

Print["done."];

AntLog["Leaving SourceInformationNotes.m"];
