(* :Title: Functions.m *)

(* :Author:
	Andrew Hunt, andy@wolfram.com
	Greg Martel, gregm@wolfram.com
*)

(* :Mathematica Version: 5 *)
(* :Package Version: $Revision: 1.8 $ $Date: 2016/11/13 05:30:54 $ *)                  
(* :Copyright: (c) 2005, Wolfram Research, Inc. All rights reserved. *)
(* :Summary: Add-ons specific rules and general transform for Mathematica Documentation *)


XMLTransform[{

{Notebook} :> 
WithParameters[
  { (* "FileName"-> StringReplace[GetAbsoluteFileName[], {".en"->"", ".ja"->""}],  *)
  		
   		"FileName"-> StringReplace[GetBasenameLa[], {".en"->"", ".ja"->""} ],
		"prev" -> (PrevLink /. Cases[
				Cases[GetParameter["properties"], __],
 				_~(Rule|RuleDelayed)~_]),  
		"next" -> (NextLink /. Cases[
				Cases[GetParameter["properties"], __],
 				_~(Rule|RuleDelayed)~_]), 
		"titlename" ->First[Cases[GetParameter["properties"], __]],
		"linkTrailSeperator" -> XMLElement["img", {"src" -> 
			"/common/images2003/linktrail-separator.gif", "border" -> "0", "width" -> "10", 
			"height" -> "10", "alt" -> " / ", "hspace" -> "3"}, {}], 
		"linkTrailPath" -> First[Last[(LinkTrail /. Cases[
					Cases[GetParameter["properties"], __],
					_~(Rule|RuleDelayed)~_])]],
		"AppName" -> First[First[(LinkTrail /. Cases[
				Cases[GetParameter["properties"], __], 
				_~(Rule|RuleDelayed)~_])]]
				
  
  }, 
  
  Print[GetParameter["FileName"]];
  Print[GetParameter["properties"]];

(
  
Block[{},

Export[
  StringJoin[GetBasenameLa[], ".html-head"],
  XML`RawXML[ StringJoin[ExportString[#, "XML"] & /@ 
	Flatten[  {
		XMLElement["title", {}, {GetParameter["FileName"]}],
		XMLElement["meta", 
          {
            "name"->"keywords", 
            "content"->StringList[GetParameter["keywords"]]
          }, {}
        ]
	}   ]  
  ]]
  , "XML"];
  
    
  (* fill in layout file with content *)
  (* symbolic XML template *)
 FillInLayout[
    GetParameter["layoutfile"],
    "Text",    
    "ProdName"-> ExportString[XMLElement["a", {"href" -> GetParameter["AppName"]}, {
				XMLElement["img", {"src" -> GetParameter["AppName"] <> "images/productName.gif",
					"border" -> "0" }, {}]   }], "XML", "AttributeQuoting"->"\"", "Entities"->{"\[LongDash]"->"&mdash;"}, 
					"ElementFormatting" -> DocumentationBuild`DocumentationConversion`Private`indentFunction],

    "body"->
      ExportString[
XMLElement["td", {"id"-> "ApplicationTable"}, { Recurse[] }],
        "XML",
     "AttributeQuoting"->"\"", "Entities"->{"\[LongDash]"->"&mdash;"},    
"ElementFormatting" -> DocumentationBuild`DocumentationConversion`Private`indentFunction] 


]  
  ]
)
],  (* end WithParam *)



(* Remove in-product decorations *)
{Cell, "LinkTrail" | "AnchorBar" | "History" | "HistoryData" | "AnchorBarGrid"} :> "", 

{Cell, "Usage"} :> 
	(* DIV[{"id" -> "UT"}, { *) 
	DIV[{"id" -> "UsageTable"}, { 
		(* DIV[{"class" -> "UM"}, { *) 
		DIV[{"class" -> "UsageMessage"}, { WithParameters[{"newline2div" -> True}, Recurse[] ] }] } ], 


{Cell, "BulletText" | "BulletedList"} :>
(* DIV[{"id" -> "BulletedList"}, {Recurse[]} ], *)
UL[{"id" -> "BulletedList"}, {
Li[{}, {Recurse[]} ]
}], 


{Cell, "CompactText"} :>
WithParameters[{"newline2div" -> True}, 
DIV[{"class" -> "SmallText"}, {Recurse[]}]
],


{Cell, "Contents"} :>
WithParameters[{"newline2div" -> True, 	"styleUsageMessageBreak" -> "</div>\n<div class=\"Contents\">"}, 
DIV[{"class" -> "Contents"}, {Recurse[]}]
],


{Cell, "MathCaption"} :>
XMLElement["div", {"class" -> "MathCaption"}, 
  If[HasOption[CellTags],
  Flatten[ {
    Map[
    (XMLElement["a", {"name" -> #}, {}])&, Flatten[{GetOption[CellTags]}] ],
    Recurse[]  }],
  {Recurse[]}  ]    
],

{Cell, "References"} :>
WithParameters[{"newline2div" -> True, 	"styleUsageMessageBreak" -> "</div>\n<div class=\"References\">"}, 
DIV[{"class" -> "References"}, {Recurse[]}]
],



{Cell, "Text"} :> WithParameters[{"newline2div" -> True, 	"styleUsageMessageBreak" -> "</div>\n<div class=\"Text\">"}, 
DIV[{"class" -> "Text"}, {Recurse[]} ]
],


{Cell, "ItemizedText" | "ItemizedList" | "ItemizedSubList"} :>
(* DIV[{"id" -> "ItemizedList"}, {Recurse[]}],  *)
UL[{"id" -> "BulletedList"}, {
Li[{}, {Recurse[]} ]
}], 


(* {Cell, "ItemizedText" | "ItemizedList" | "ItemizedSubList"} :>
Module[{content},
content = SelectChildren[];

If[MatchQ[content[[1,1,1]], StyleBox[s_/;StringMatchQ[s,RegularExpression["\\d+\\."]],_]],  
	DIV[{"id" -> "ItemizedList"}, {Transmogrify[content]}], 
	Li[{"id" -> "ItemizedList"}, {Transmogrify[content]}] 
	]
], *)



{Cell, "NumberedEquation" | "DisplayFormula"} :> 
DIV[{"class" -> GetStyle[]} , {
( transmogrifyImage[
    ToFileName[{"HTMLImages", GetBasenameLa[]}, GetBasenameLa[] <> "_ne_" <> IncrementCounter["NumberedEquation"] <> ".gif"],
    {"alt" -> " "},
		Inline -> False, (* changed for TimeSeries--why inline anywahy?  *)
    "RetinaImage" -> True
  ]
)   
}],  


{Cell, "DisplayMath"} :> 
DIV[{"class" -> GetStyle[]} , {
(
  transmogrifyImage[
    ToFileName[{"HTMLImages", GetBasenameLa[]}, GetBasenameLa[] <> "_ne_" <> IncrementCounter["NumberedEquation"] <> ".gif"],
    {"alt" -> " "},
		Inline -> False,
    "RetinaImage" -> True
  ]
)
}],  


{Cell, "Chapter"} :> H1[{"class"-> "Chapter"}, {Recurse[]}], 


{Cell, "TOCSection"} :> DIV[{"class"-> "ContentsSection"}, {
XMLElement["b", {}, {Recurse[]}]
}], 

{Cell, "TOCSubsection"} :> DIV[{"class"-> "ContentsSubsection"}, {Recurse[]}], 

{Cell, "TOCSubsubsection"} :> DIV[{"class"-> "ContentsSubsubsection"}, {Recurse[]}], 

{Cell, "TOCPage"} :> Span[{"class"-> "TOCPage"}, {Recurse[]}], 

{Cell,  "2ColumnTable" | "3ColumnTable"} :>
DIV[{"class" -> "FunctionTable", "name"-> GetStyle[]}, {Recurse[]}], 

{Cell,  "AlphabeticalListingTable"} :> P[{"class" -> "MathTable", "name"-> GetStyle[]}, {Recurse[]}],

{Cell, "BulletedSubList"} :>
DIV[{"class" -> "BulletedSubList"}, {Recurse[]}], 

{CellGroupData, "TopBox" | "TopSpanBox"} :>
DIV[{"class" -> "MathTable"}, {Recurse[]}], 	

{Cell, "TopBox" | "TopSpanBox"} :>
DIV[{"class" -> "TopBox"}, {Recurse[]}], 

{Cell, "MiddleBox" | "MiddleSpanBox"} :>
DIV[{"class" -> "MiddleBox"}, {Recurse[]} ],

{Cell, "BottomBox"} :>
DIV[{"class" -> "BottomBox"}, {Recurse[]}],


(***** Cell Definitions *****)
{Cell, "Notes"} :> XMLElement["div", {"class" -> "Notes"}, {Recurse[]} ],


{Cell, "Picture"} :> {
(XMLElement["img", {
		"src" -> GetParameter["linkTrailPath"] <> ("URL" /. #),
		"width" -> ToString[Floor[.89 ToExpression["Width" /. #]]],
		"height" -> ToString[Floor[.89 ToExpression["Height" /. #]]], "alt" -> " "
	}, {}]&[BoxToImage[ToFileName[{"HTMLImages", GetBasenameLa[]},
		GetBasenameLa[] <> "_pict_" <> IncrementCounter["Picture"] <> ".gif"],
		Inline -> True]])
}, 


{Cell, "Graphics"} :>
XMLElement["p", {"class" -> GetStyle[]}, {
  transmogrifyImage[
    ToFileName[{"HTMLImages", GetBasenameLa[]}, GetBasenameLa[] <> "_a_" <> IncrementCounter["Animation"] <> ".gif"],
    {"alt" -> " "},
    Inline -> False,
    "RetinaImage" -> True
  ]
}],


{CellGroupData, "PictureGroup" | "Graphics"} :>
(If[HasOption[First[SelectChildren[Cell]], AnimationCycleRepetitions],
	XMLElement["p", {"class" -> "Animation"}, {
		WithParameters[{"src" -> ToFileName[{"HTMLImages", GetBasenameLa[]},
				GetBasenameLa[] <> "_a_" <> IncrementCounter["Animation"] <> ".gif"]},
			CreateDirectory[DirectoryName[GetParameter["src"]]];
			Export[GetParameter["src"],
				SelectLiteral[SelectChildren[]], "GIF", 
				ConversionOptions -> {"Loop" -> 3}];
			XMLElement["img", {
				"src" -> StringReplace[GetParameter["src"], $PathnameSeparator -> "/"]
			}, {}]
		]
	}],
	XMLElement["p", {"class" -> GetStyle[]}, {
    transmogrifyImage[
      ToFileName[{"HTMLImages", GetBasenameLa[]}, GetBasenameLa[] <> "_a_" <> IncrementCounter["Animation"] <> ".gif"],
      {"alt" -> " "},
			Inline -> False,
      "RetinaImage" -> True
    ]
}]]), 



{CellGroupData, "Input"} :> 
  DIV[{"class"->"inoutdiv"}, {Recurse[]}],


{Cell, "FlushInput" | "Input" | "InputOnly"} :>	
Sequence@@{
  XMLElement["table", {"class" -> "InputCell"
  }, {
    XMLElement["tr", {}, {
      XMLElement["td", {"class" -> "celllabel"}, 
        If[HasOption[CellLabel],
          (* fill it with the CellLabel text *)
		{GetOption[CellLabel]},
          {" "} ] (* otherwise it's empty end If *)
      ], 
      (* input <td> *)
      TD[{"class" -> "Input"}, {
          ( 
            transmogrifyImage[
              ToFileName[{"HTMLImages", GetBasenameLa[]}, GetBasenameLa[] <> "_i_" <> IncrementCounter["Input"] <> ".gif"],
              {"alt" -> " "},
              Inline -> True,
              "RetinaImage" -> True
            ]
          ) } ] (* end <td> *)
    }]  (* end <tr> *)
  }] (* end <table> *)
},  


{Cell, "Output"} :>	
Sequence@@{
  XMLElement["table", {"class" -> "OutputCell"
  }, {
    XMLElement["tr", {}, {
      XMLElement["td", {"class" -> "celllabel"}, 
        If[HasOption[CellLabel],
          (* fill it with the CellLabel text *)
		{GetOption[CellLabel]},
          {" "} ] (* otherwise it's empty end If *)
      ], 
      (* output <td> *)
      TD[{"class" -> "Output"}, {
          ( 
            transmogrifyImage[
              ToFileName[{"HTMLImages", GetBasenameLa[]}, GetBasenameLa[] <> "_i_" <> IncrementCounter["Input"] <> ".gif"],
              {"alt" -> " "},
              Inline -> True,
              "RetinaImage" -> True
            ]
          ) } ] (* end <td> *)
    }]  (* end <tr> *)
  }] (* end <table> *)
},  



{RawData} :> ( 
XMLElement["img", {
		"src" -> GetParameter["linkTrailPath"] <> ("URL" /. #),
		"width" -> ("Width" /. #), "height" -> ("Height" /. #), "alt" -> " "
	}, {}]&[BoxToImage[ToFileName[{"HTMLImages", GetBasenameLa[]},
		GetBasenameLa[] <> "_" <> IncrementCounter["GraphicsData"] <> ".gif"],
		Inline -> True]]),  


{Cell, "MR"} :> Span[{"class"->"MR"}, {Recurse[]}],


{StyleBox, "MR"} :> Module[ { output , replacements},
output = Recurse[]; 
replacements = Import[DocsAsset["WebData", "MRReplacementList.m"]];

WithParameters[ { "url"->( output /.  replacements )}, 

If[
Head[ GetParameter["url"] ] === XMLElement,  
GetParameter["url"],
  Span[{"class" -> "MR"}, {output} ] ]  
  ]
], 




{ButtonBox} :> Block[
{BoxContent, BoxButtonData, RawBoxOptions, BoxOptions, exampleReplacements},

exampleReplacements = Import[
ToFileName[{$UserAddOnsDirectory, "Applications", "DocumentationBuild", "Internal", "web", "data"}, "ApplicationsButtonReplacements.m"] 
]; 

BoxContent := Recurse[];
BoxButtonData = Switch[GetOption[ButtonData],
	URL[_], GetOption[ButtonData][[1]],
	{URL[_], _}, GetOption[ButtonData][[1, 1]],
	{___, _String?((StringLength[#] >= 3 && StringTake[#, {-3,-1}] == ".nb")&)},
		Last[GetOption[ButtonData]],
	_String?((StringTake[#, {-3,-1}] == ".nb")&), GetOption[ButtonData],
	{___, _String}, If[StringSplit[GetParameter["AppName"], "/"][[2]] === "timeseries" && Length@GetOption[ButtonData] == 2, 
		First@GetOption[ButtonData] <> ".html" <> "#" <> Last@GetOption[ButtonData],  
		Last@GetOption[ButtonData] <> ".html"],
(*	{___, _String}, Last@GetOption[ButtonData] <> ".html",  *)
	_String, GetOption[ButtonData] <> ".html",
	{FrontEnd`FileName[{__}, file_String, ___], ___}, StringReplace[GetOption[ButtonData][[1, 2]], ".nb"->""] <> ".html",
	{FrontEnd`FileName[{}, file_String, ___], ___}, StringReplace[GetOption[ButtonData][[1, 2]], ".nb"->""] <> ".html",
	{_String, None}, GetOption[ButtonData][[1]],
	_,  "FIXME"]; 
BoxButtonData = ( BoxButtonData /. exampleReplacements ); 
BoxButtonData = StringReplace[BoxButtonData, exampleReplacements];


RawBoxOptions =  Rest@Cases[SelectSelf[], _];
BoxOptions = Which[
	MemberQ[RawBoxOptions, _ -> "MainBookLink"], "href" -> "/language/tutorial/" <> BoxContent,   
	MemberQ[RawBoxOptions, _ -> "RefGuideLink"], "href" -> "/language/ref/" <> BoxContent,   
	MemberQ[RawBoxOptions, _ -> "Hyperlink"], "href" -> BoxButtonData ,   
	MemberQ[RawBoxOptions, _ -> "AddOnsLinkText"], "href" -> BoxButtonData ,   
	MemberQ[RawBoxOptions, _ -> "AddOnsLink"], "href" -> BoxButtonData ,   
	GetOption[ButtonData] == None && StringMatchQ[ToString[GetOption[ButtonEvaluator]], "Automatic"], 
		"href" -> StringReplace[Transmogrify[SelectChildren[][[1]]], " " -> ""] <> ".tar.gz",
	GetOption[ButtonData] == None && StringMatchQ[ToString[GetOption[ButtonFunction]], "NotebookOpen*"], 
		{NBOPEN = GetOption[ButtonFunction];  
     		CopyFile[NBOPEN[BoxContent], StringDrop[BoxContent, -3] <> "2.nb"];,   
		"href" -> BoxContent},
	True, "href" -> BoxButtonData ];

Which[
	MatchQ[GetOption[ButtonStyle], "UserDocsLink" | "PageLink"], Recurse[], 
	True, A[{BoxOptions}, {BoxContent}]
	]
],



(* {ButtonBox} :> Block[
{BoxContent, BoxButtonData, RawBoxOptions, BoxOptions, exampleReplacements},

exampleReplacements = Import[
ToFileName[{$UserAddOnsDirectory, "Applications", "DocumentationBuild", "Internal", "web", "data"}, "ApplicationsButtonReplacements.m"] 
]; 

BoxContent := Recurse[];
BoxButtonData = Switch[GetOption[ButtonData],
	URL[_], GetOption[ButtonData][[1]],
	{URL[_], _}, GetOption[ButtonData][[1, 1]],   
	{___, _String?((StringLength[#] >= 3 && StringTake[#, {-3,-1}] == ".nb")&)},
		Last[GetOption[ButtonData]],
	_String?((StringTake[#, {-3,-1}] == ".nb")&), GetOption[ButtonData],
	{___, _String}, Last@GetOption[ButtonData] <> ".html",  
	_String, GetOption[ButtonData] <> ".html",
	{FrontEnd`FileName[{__}, file_String, ___], ___}, StringReplace[GetOption[ButtonData][[1, 2]], ".nb"->""] <> ".html",
	{FrontEnd`FileName[{}, file_String, ___], ___}, StringReplace[GetOption[ButtonData][[1, 2]], ".nb"->""] <> ".html",
	{_String, None}, GetOption[ButtonData][[1]],
	_,  "FIXME"]; 
BoxButtonData = ( BoxButtonData /. exampleReplacements ); 

RawBoxOptions =  Rest@Cases[SelectSelf[], _];
BoxOptions = Which[
	MemberQ[RawBoxOptions, _ -> "MainBookLink"], "href" -> "/language/tutorial/" <> BoxContent,   
	MemberQ[RawBoxOptions, _ -> "RefGuideLink"], "href" -> "/language/ref/" <> BoxContent,   
	MemberQ[RawBoxOptions, _ -> "Hyperlink"], "href" -> BoxButtonData ,   
	MemberQ[RawBoxOptions, _ -> "AddOnsLinkText"], "href" -> BoxButtonData ,   
	MemberQ[RawBoxOptions, _ -> "AddOnsLink"], "href" -> BoxButtonData ,   
	GetOption[ButtonData] == None && StringMatchQ[ToString[GetOption[ButtonEvaluator]], "Automatic"], 
		"href" -> StringReplace[Transmogrify[SelectChildren[][[1]]], " " -> ""] <> ".tar.gz",
	GetOption[ButtonData] == None && StringMatchQ[ToString[GetOption[ButtonFunction]], "NotebookOpen*"], 
		{NBOPEN = GetOption[ButtonFunction];  
     		CopyFile[NBOPEN[BoxContent], StringDrop[BoxContent, -3] <> "2.nb"];,   
		"href" -> BoxContent},
	True, "href" -> BoxButtonData ];

Which[
	StringSplit[GetParameter["AppName"], "/"][[2]] === "timeseries" && MatchQ[GetOption[ButtonData], {a_String, b_String}], Print@"FIRST ONE"; Recurse[], 
	MatchQ[GetOption[ButtonStyle], "UserDocsLink" | "PageLink"], Print@"Second ONE"; Recurse[], 

	True, Print@"Third ONE";A[{BoxOptions}, {BoxContent}]
	]
],  *)






{Cell, None} :> If[htmlBoxesQ[ SelectLiteral[ SelectSelf[] ] ], 
  Recurse[], 
transmogrifyImage[
  ToFileName[{"HTMLImages", GetBasenameLa[]}, "Box_" <> IncrementCounter["Inline"] <> ".gif"],
  {alt -> ""}, 
  Inline -> True,
  CropImage -> False,
  "RetinaImage" -> True
]
     ],
     



(* {Cell, None} :>
If[
  htmlBoxesQ[ SelectLiteral[ SelectSelf[] ] ], 
  Transmogrify[ SelectChildren[] ],
  (  
    XMLElement["img", { 
      "class" -> "FlushInput", "src" -> ("URL" /. #), 
      "width" -> ("Width" /. #), "height" -> ("Height" /. #),  
      "alt" -> " "  }, {}]&[
      BoxToImage[ToFileName[{"HTMLImages", GetBasenameLa[]},
    "Box_" <> IncrementCounter["Inline"] <> ".gif"],
    Inline -> True, CropImage -> False]
    ]
  )  
],   *)



{OutputFormData} :> Transmogrify[SelectChildren[][[2]]]

},

IncludeXMLTransforms -> {{"HTML","Components","Defaults.m"}, {"HTML","Components","GridBox.m"}, {"HTML","Components","StyleBox.m"}, {"HTML","Components","ButtonBox.m"}},

DefaultParameters -> {
	(*  "newline2br" -> False,  *)
	"newline2br" -> False,
	"newline2div" -> False,
	"styleUsageMessageBreak" -> "</div>\n<div class=\"UsageMessage\">",

	(* No Operation code *)
	"noop" -> {
		$LanguageExtension = "en";
	}
}

]
