(* * Defaults.m *)
(* :Title: CoreHTMLRules.m *)

(* :Author:
  Andrew Hunt, andy@wolfram.com
  Greg Martel, gregm@wolfram.com
    Stephen Layland, layland@wolfram.com
*)

(* :Mathematica Version: 5.1 *)
(* :Mathematica Version: 6.0 *)
(* :Package Version: $Revision: 1.326 $ $Date: 2018/05/30 16:54:53 $ *)                  
(* :Copyright: (c) 2005, Wolfram Research, Inc. All rights reserved. *)
(* :Summary: CoreTransformRules for Mathematica Documentation *)

(* This includes the other *Box files in the Components directory *)


XMLTransform[{

(* *** Notebook transform *)

(* note that XMLTransforms/HTML/ExamplePages.m defines its own Notebook transform *)

{Notebook} :> Module[
  {
    pagetitle, pagetitlemodifier, language, buildNumber,
    history, l, completeHTMLQ, htmlBase, summary, fileBase, tagrules,
		flag, versionNumber, keywords, specialKeywords,insertedKeywords,listLength,listDivisor, subtractor, buildmode,
    entityrules, filledinlayoutexpr, featuredexampledata, relativeBase,
    newin, updatedin, experimentalQ, preservedInputForm, prerequisite
  },

	dprint["Defaults.m: entering Notebook transform, SelectSelf[] = "];
	dfile[SelectSelf[], "Format" -> ".m"];

	dprint["   GetParameter[\"buildmode\"] = "<>ToString[GetParameter["buildmode"], InputForm]];
	dprint["   GetParameter[\"BuildNumber\"] = "<>ToString[GetParameter["BuildNumber"], InputForm]];
	dprint["   GetParameter[\"Cache\"] = "<>ToString[GetParameter["Cache"], InputForm]];
	dprint["   GetParameter[\"CacheLocation\"] = "<>ToString[GetParameter["CacheLocation"], InputForm]];
	dprint["   GetParameter[\"CompleteHTMLQ\"] = "<>ToString[GetParameter["CompleteHTMLQ"], InputForm]];
	dprint["   GetParameter[\"contextName\"] = "<>ToString[GetParameter["contextName"], InputForm]];
	dprint["   GetParameter[\"entityType\"] = "<>ToString[GetParameter["entityType"], InputForm]];
	dprint["   GetParameter[\"exampleSections\"] = "<>ToString[GetParameter["exampleSections"], InputForm]];
	dprint["   GetParameter[\"experimentalQ\"] = "<>ToString[GetParameter["experimentalQ"], InputForm]];
	dprint["   GetParameter[\"featuredexampledata\"] = "<>ToString[GetParameter["featuredexampledata"], InputForm]];
	dprint["   GetParameter[\"flag\"] = "<>ToString[GetParameter["flag"], InputForm]];
	dprint["   GetParameter[\"guideMoreAbout\"] = "<>ToString[GetParameter["guideMoreAbout"], InputForm]];
	dprint["   GetParameter[\"guideTutorials\"] = "<>ToString[GetParameter["guideTutorials"], InputForm]];
	dprint["   GetParameter[\"HistoryData\"] = "<>ToString[GetParameter["HistoryData"], InputForm]];
	dprint["   GetParameter[\"htmlBase\"] = "<>ToString[GetParameter["htmlBase"], InputForm]];
	dprint["   GetParameter[\"IsManipulate\"] = "<>ToString[GetParameter["IsManipulate"], InputForm]];
	dprint["   GetParameter[\"IsSolutionsQ\"] = "<>ToString[GetParameter["IsSolutionsQ"], InputForm]];
	dprint["   GetParameter[\"IsTMJQ\"] = "<>ToString[GetParameter["IsTMJQ"], InputForm]];
	dprint["   GetParameter[\"keywords\"] = "<>ToString[GetParameter["keywords"], InputForm]];
	dprint["   GetParameter[\"la\"] = "<>ToString[GetParameter["la"], InputForm]];
	dprint["   GetParameter[\"language\"] = "<>ToString[GetParameter["language"], InputForm]];
	dprint["   GetParameter[\"languageAbbreviations\"] = "<>ToString[GetParameter["languageAbbreviations"], InputForm]];
	dprint["   GetParameter[\"layoutfile\"] = "<>ToString[GetParameter["layoutfile"], InputForm]];
	dprint["   GetParameter[\"lwgQ\"] = "<>ToString[GetParameter["lwgQ"], InputForm]];
	dprint["   GetParameter[\"monographObjectNameGridQ\"] = "<>ToString[GetParameter["monographObjectNameGridQ"], InputForm]];
	dprint["   GetParameter[\"moreAboutGuides\"] = "<>ToString[GetParameter["moreAboutGuides"], InputForm]];
	dprint["   GetParameter[\"newin\"] = "<>ToString[GetParameter["newin"], InputForm]];
	dprint["   GetParameter[\"packageQ\"] = "<>ToString[GetParameter["packageQ"], InputForm]];
	dprint["   GetParameter[\"pacletName\"] = "<>ToString[GetParameter["pacletName"], InputForm]];
	dprint["   GetParameter[\"relatedTutorials\"] = "<>ToString[GetParameter["relatedTutorials"], InputForm]];
	(* dprint["   GetParameter[\"rootCaptions\"] = "<>ToString[GetParameter["rootCaptions"], InputForm]]; *)
	dprint["   GetParameter[\"seeAlsoFunctions\"] = "<>ToString[GetParameter["seeAlsoFunctions"], InputForm]];
	dprint["   GetParameter[\"specialKeywords\"] = "<>ToString[GetParameter["specialKeywords"], InputForm]];
	dprint["   GetParameter[\"summary\"] = "<>ToString[GetParameter["summary"], InputForm]];
	dprint["   GetParameter[\"synonyms\"] = "<>ToString[GetParameter["synonyms"], InputForm]];
	dprint["   GetParameter[\"guideFunctions\"] = "<>ToString[GetParameter["guideFunctions"], InputForm]];
	dprint["   GetParameter[\"taggingRules\"] = "<>ToString[GetParameter["taggingRules"], InputForm]];
	dprint["   GetParameter[\"title\"] = "<>ToString[GetParameter["title"], InputForm]];
	dprint["   GetParameter[\"titlemodifier\"] = "<>ToString[GetParameter["titlemodifier"], InputForm]];
	dprint["   GetParameter[\"tutorialFunctions\"] = "<>ToString[GetParameter["tutorialFunctions"], InputForm]];
	dprint["   GetParameter[\"tutorialMoreAbout\"] = "<>ToString[GetParameter["tutorialMoreAbout"], InputForm]];
	dprint["   GetParameter[\"tutorials\"] = "<>ToString[GetParameter["tutorials"], InputForm]];
	dprint["   GetParameter[\"updatedin\"] = "<>ToString[GetParameter["updatedin"], InputForm]];
	dprint["   GetParameter[\"url\"] = "<>ToString[GetParameter["url"], InputForm]];
	dprint["   GetParameter[\"urlBaseRemote\"] = "<>ToString[GetParameter["urlBaseRemote"], InputForm]];
	dprint["   GetParameter[\"webmQ\"] = "<>ToString[GetParameter["webmQ"], InputForm]];
	dprint["   GetParameter[\"workflowOverviewQ\"] = "<>ToString[GetParameter["workflowOverviewQ"], InputForm]];
	dprint["   GetParameter[\"workflowQ\"] = "<>ToString[GetParameter["workflowQ"], InputForm]];
	dprint["   GetParameter[\"wsmQ\"] = "<>ToString[GetParameter["wsmQ"], InputForm]];
	dprint["   GetParameter[\"prerequisite\"] = "<>ToString[GetParameter["prerequisite"], InputForm]];

	entityrules = {"\[LongDash]"->"&mdash;", "\[FilledRightTriangle]"->"&#9658;", "\[FilledLeftTriangle]"->"&#9668;"};

	versionNumber = StringSplit[$Version, "."][[1]];

	pagetitlemodifier = GetParameter["titlemodifier"];
	pagetitle = GetParameter["title"];

	pagetitle = Which[
		pagetitlemodifier === "",
			StringReplace[pagetitle, {"\""->"", " - Wolfram Mathematica"->""}]
	,
		pagetitlemodifier === "XXXX",
			StringReplace[pagetitle, {"\""->"", " - Wolfram Mathematica"->""}]
	,
		True,
			StringReplace[pagetitle <> " " <> pagetitlemodifier, {
				"\"" -> "",
				" - Wolfram Mathematica" -> ""
			}]
	];
	pagetitle = If[StringQ[pagetitle],
		StringReplace[pagetitle, " ("~~_~~")"->""]
		,
		pagetitle
	];
	pagetitle = StringReplace[
		pagetitle,
		Prepend[
			Get[DocsAsset["TextCharacterReplacementRules"] ],
			"\[Rule]"->"->"
		]
	];

	keywords = GetParameter["keywords"];
	specialKeywords = GetParameter["specialKeywords"];
	keywords = Complement[Flatten[{StringSplit[keywords, ","]}],specialKeywords];
	(* Formatting note: e33aac04:
		Limit related search keywords to 20 or less. *)
	listLength = Length[keywords];
	listDivisor = If[listLength > 0,
		Round[Extract[Solve[listLength/x == 20, x], {{1, 1, 2}}]][[1]],
		1
	];
	subtractor = listDivisor - 1;
	keywords = If[Length[keywords] < 24,
		keywords,
		Extract[
			keywords,
			Partition[Table[listDivisor(i)-subtractor, {i,Round[listLength/listDivisor]}], 1]
		]
	];
	insertedKeywords = ToStringList[keywords];

(* Print["Notebook transform point 1"]; *)

	buildmode = GetParameter["buildmode"];
	buildNumber = GetParameter["BuildNumber"];
	history = GetParameter["HistoryData"];
	IsManipulate = GetParameter["IsManipulate"];
	cache = GetParameter["Cache"];
	cachelocation = GetParameter["CacheLocation"];
	featuredexampledata = GetParameter["featuredexampledata"];
	language = GetParameter["language"];
	htmlBase = GetParameter["htmlBase"];
	completeHTMLQ = GetParameter["CompleteHTMLQ"];

	(* Print["Notebook transform point 2"]; *)
	dprint["buildmode: "<>ToString[buildmode]];

	(* Formatting note: cc1db57b:
		Add vertical fudge for inline math. *)
	fudgeValignProp[x_] := Module[{var, newvar},
		var = ToExpression[x];
		newvar = Which[
			var<14, "-10",
			13<var<18, "-12",
			17<var<20, "-16",
			19<var<23, "-22",
			22<var<27, "-35",
			26<var<30, "-45",
			29<var<32, "-55",
			31<var<35, "-65",
			True, "-75"
		];
		newvar
	];

	(* convert charaacters to entities - IE Windows won't render pages if meta description contains escaped characters *)
	(* Formatting note: a70e10a9:
		Fix twice-escaped entities in meta description
		https://jira.wolfram.com/jira/browse/WLDOCS-3280
	*)
	summary = ExportString[
		XMLElement["String", {}, { GetParameter["summary"] }],
		"XML",
		"Entities" -> entityrules
	];
	summary = If[Head@summary === String,
		StringReplace[summary, {"<String>" -> "", "</String>" -> ""}]
		,
		""
	];

	fileBase = GetBasenameLa[];
	tagrules = GetOption[TaggingRules];
	flag = Quiet@Check[Extract[tagrules, Position[tagrules, "status"->a_String]][[1,2]], ""];

WithParameters[{
	"FileName"-> StringReplace[GetAbsoluteFileName[], {".en"->"", ".ja"->""}]
},
(
	(* **** create a .html-conf file if there's a filename *)
	If[
		(fileBase =!= "")
		&& (completeHTMLQ =!= True)
		&& (
			StringReplace[fileBase, {".en"->"", ".ja"->"", ".zh"->""}] === "Mathematica"
			&& GetParameter["entityType"] === "Guide"
		),
		Export[StringJoin[fileBase, ".html-conf"],
			"#SetEnv STYLE_CustomHeaderFile $$DOCUMENT_ROOT$$/mathematicaIncludes/mathematicaHead.html
			#SetEnv STYLE_CustomFooterFile $$DOCUMENT_ROOT$$/mathematicaIncludes/mathematicaFoot.html
			#SetEnv STYLE_CustomFooterFile 0
			SetEnv STYLE_InternalPageWidth 100%

			SetEnv STYLE_Body '<body style=\"margin:0px;\" onLoad=\"javascript:set_domain()\">'",
			"TEXT"
		]
	];

	(* Print["Notebook transform point 3"]; *)
Block[{styleswitch},

	(* *** fill in layout file with content *)

	styleswitch = GetParameter["entityType"];

	(* Formatting note:
		* fe0703b7
		* 60d6e8ae
		* a410487c
	*)
	(* **** relativeBase: special for completeHTMLQ, otherwise standard.
		TODO: what about web WSM which is not completeHTMLQ?  it needs to link to /system-modeler/css/... *)

	relativeBase = GetParameter["relativeBase"];

	dprint["Defaults.m: relativeBase = "<>ToString[relativeBase, InputForm]];

	(* **** filledinlayoutexpr = FillInLayout[much code] *)

	(* Print["Notebook transform point 4"]; *)
	(* symbolic XML template *)
	filledinlayoutexpr = FillInLayout[
		GetParameter["layoutfile"],
		"Text",
		"develContent" -> "[ Internal Document ]",
		"pageTitle" -> pagetitle,
		"BuildNumber" -> buildNumber,
		"title" -> ExportString[
			XMLElement[
				"title",
				{},
				{
					(* Formatting note: e956a0f0 *)
					If[
						GetParameter["pacletName"] === "Wolfram SystemModeler"
						|| GetParameter["pacletName"] === "Wolfram System Modeler"
						|| GetParameter["pacletName"] === "Wolfram SystemModeler Library"
					,
						(* WSM documentation, do not touch title *)
						pagetitle
					,
						(* Wolfram Language documentation, make sure title ends with "Wolfram Language Documentation" *)
						If[
							StringFreeQ[pagetitle, "Wolfram Mathematica Documentation Center"|"Wolfram Mathematica: Documentation Center"]
						,
							pagetitle <>"\[LongDash]"<>Localized["TitleV10WebHeading",GetParameter["language"]]
						,
							StringReplace[
								pagetitle,
								"Wolfram Mathematica: Documentation Center"
								| "Wolfram Mathematica Documentation Center" -> "Wolfram Language Documentation"
							]
						]
					]
				}
			],
			"XML",
			"Entities"->entityrules
		],
		"metabuildnumber" -> "<meta name=\"buildnumber\" content=\""<>buildNumber<>"\">",
		"metahistorydata" -> ExportString[
			XMLElement["meta", {"name"->"historydata","content"->ToString[history] }, {}],
			"XML",
			"AttributeQuoting" -> "\"",
			"Entities" -> entityrules
		],
		"metakeywords" ->If[Length@keywords > 0,
			ExportString[
				XMLElement["meta", {"name"->"keywords", "content"->insertedKeywords}, {}],
				"XML",
				"AttributeQuoting" -> "\"",
				"Entities" -> entityrules
			],
			""
		],
		"metasynonyms" ->If[Length@GetParameter["synonyms"] > 0,
			ExportString[
				XMLElement["meta", {"name"->"synonyms", "content"->ToStringList[GetParameter["synonyms"]] }, {}],
				"XML",
				"AttributeQuoting" -> "\"",
				"Entities" -> entityrules
			],
			""
		],
		(* Formatting note: eb4f66cd53e3b843a96c9741592c959465b1b530:
			Translate metadata for root guide page

			https://jira.wolfram.com/jira/browse/WLDOCS-2467
			https://jira.wolfram.com/jira/browse/WLDOCS-2537

			- when inserting translated metadata into the root guide page, bypass
			ExportString[..., "XML"] to avoid converting ja and zh text to html
			entities like &#25991.
		*)
		"rootguidetitle" -> ExportString[
			XMLElement["title", {}, {"ROOTGUIDETITLE"}],
			"XML",
			"AttributeQuoting" -> "\"",
			"Entities" -> entityrules
		],
		"metadescription" -> If[GetParameter["url"] === "guide/WolframRoot",
			ExportString[
				XMLElement["meta", {"name" -> "description", "content" -> "ROOTGUIDEMETADATADESCRIPTION" }, {}],
				"XML",
				"AttributeQuoting" -> "\"",
				"Entities" -> entityrules
			]
		,
			(* Formatting note: a70e10a9:
				Fix twice-escaped entities in meta description
				https://jira.wolfram.com/jira/browse/WLDOCS-3280
			*)
			If[
				MatchQ[summary, List[_String] | _String]
				, "<meta name=\"description\" content=\""<>If[MatchQ[summary, List[_String]], summary[[1]], summary]<>"\" />"
				, ""
			]
		],
		"baselangline" -> ExportString[
			XMLElement["script", {}, {XML`RawXML["var baselang = '" <> fileBase <> "';"]}],
			"XML",
			"AttributeQuoting" -> "\"",
			"Entities" -> entityrules
		],
		"WebRootHeadingText" -> Localized["RootWebHeading", GetParameter["language"]],
		"WebRootGetStartedText" -> Localized["RootGSVideos", GetParameter["language"]],
		"WebRootWhatsNewText" -> Localized["RootWhatsNew", GetParameter["language"]],
		"WebRootHowTosText" -> Localized["RootHowTo", GetParameter["language"]],
		"WebRootFooterLinks" -> Localized["WebRootFooterLinks", GetParameter["language"]],
		"la" -> GetParameter["la"], (* guaranteed 2-letter language code *)
		"keywords" -> insertedKeywords,

		"gs" -> ExportString[
			XMLElement["span", {"class"->"RootFootSpan"}, {Localized["RootGSVideos", language]}],
			"XML",
			"Entities"->{"'" -> "&#39;"}
		],
		"path" -> ExportString[
			XMLElement["span", {"class"->"RootFootSpan"}, {Localized["RootLearningPath", language]}],
			"XML",
			"Entities"->{"'" -> "&#39;"}
		],
		"howto" -> ExportString[
			XMLElement["span", {"class"->"RootFootSpan"}, {Localized["RootHowTo", language]}],
			"XML",
			"Entities"->{"'" -> "&#39;"}
		],

		"body"-> If[
			MemberQ[exceptionsList, GetParameter["entityType"]]
		,
			(* QUERY: what is the purpose of exceptionsList?
			see Functions.m: all refpage types are listed there:
			exceptionsList:= {"Symbol", "Menu Item", "File", "System Program", "Front End Object", "MathLink C Function", "LibraryLink C Function","System Program", "Widget", "Message", "Indicator"} *)

			(* note Monday, June 12, 2017 at 11:45:31: does this output fit workflows, or do i need yet another exception here? *)

			(********** TRUE clause: entityType is a member of exceptionsList **********)
			StringJoin @ Map[
				ExportString[
					#,
					"XML",
					"AttributeQuoting"->"\"",
					"Entities"->{"\[LongDash]"->"&mdash;", "\[FilledRightTriangle]"->"&#9658;", "\[FilledLeftTriangle]"->"&#9668;"},
					"ElementFormatting" -> DocumentationBuild`Utils`Private`indentFunction
				]&,
				{
					(* Formatting note: 6ea33491:
						Remove "inner main-content" div from all pages

						https://jira.wolfram.com/jira/browse/WLDOCS-1728, item 1

						For the div to be deleted, I changed it to what becomes a distinctive
						pair of tags, <DeleteThisElement> ... </DeleteThisElement>.  This it's
						dead simple to delete them after filledinlayoutexpr is built.
					*)
					XMLElement["DeleteThisElement", {}, {
						Transmogrify[SelectChildren[] //. Cell[linkContent__, "LinkTrail", linkOptions___] :> Unevaluated[Sequence[]]]
					}]
				}
			]
		,
			(********** FALSE clause: entytyType is NOT a member of exceptionsList **********)
			ExportString[
				If[
					GetParameter["flag"] =!= "RootGuidePage"
				,
					(********** TRUE clause: NOT the root guide page **********)
					Module[{contents, thirdArg},
						contents = SelectChildren[] //. Cell[linkContent__, "LinkTrail", linkOptions___] :> Unevaluated[Sequence[]];

						thirdArg = Which[
							MemberQ[{"Guide Overview" | "Standard Extra Packages"}, GetParameter["entityType"]],
							{Transmogrify[contents]}
						,
							MemberQ[{"HowTo"}, GetParameter["entityType"]],
							AddFirstAndLastToexpandWraps[{Transmogrify[contents]}]
						,
							True,
							(* not refpages, not root guide, not guide overviews, not howtos *)
							{Transmogrify[contents]}
						];

						XMLElement["DeleteThisElement", {}, thirdArg]
					]
				,
					(********** FALSE clause: the root guide page **********)
					XMLElement["ROOTGUIDETAG", {}, Flatten[{Recurse[]}]]
				],
				"XML",
				"AttributeQuoting" -> "\"",
				"Entities" -> {
					"\[RightGuillemet]" -> "<span class=\"rightguillemet\">&#187;</span>",
					"\[LongDash]" -> "&mdash;",
					"\[FilledRightTriangle]" -> "&#9658;",
					"\[FilledLeftTriangle]" -> "&#9668;"
				},
				"ElementFormatting" -> DocumentationBuild`Utils`Private`indentFunction
			]
		]
	];

	dprint["Defaults.m: initial value of filledinlayoutexpr before various mungings: "]; dfile[filledinlayoutexpr, "Format" -> ".txt"];

	(* **** modify filledinlayoutexpr: special characters, misc fixes *)

	filledinlayoutexpr = StringReplace[filledinlayoutexpr, {
		"<span class=\"specialcharacters\"><span class=\"verticalbar\">|</span></span>"      -> "<span class=\"verticalbar\">|</span>",
		"<span class=\"specialcharacters\"><span class=\"comma\">,</span></span>"            -> "<span class=\"comma\">,</span>",
		"<span class=\"specialcharacters\"><span class=\"semicolon\">;</span></span>"        -> "<span class=\"semicolon\">;</span>",
		"<span class=\"specialcharacters\"><span class=\"openbrace\">{</span></span>"        -> "<span class=\"openbrace\">{</span>",
		"<span class=\"specialcharacters\"><span class=\"closebrace\">}</span></span>"       -> "<span class=\"closebrace\">}</span>",
		"<span class=\"specialcharacters\"><span class=\"openbracket\">[</span></span>"      -> "<span class=\"openbracket\">[</span>",
		"<span class=\"specialcharacters\"><span class=\"closebracket\">]</span></span>"     -> "<span class=\"closebracket\">]</span>",
		"<span class=\"specialcharacters\"><span class=\"openparenthesis\">(</span></span>"  -> "<span class=\"openparenthesis\">(</span>",
		"<span class=\"specialcharacters\"><span class=\"closeparenthesis\">)</span></span>" -> "<span class=\"closeparenthesis\">)</span>",
		"<ROOTGUIDETAG>" -> "",
		"</ROOTGUIDETAG>" -> "",
		(* Formatting note: 252988e3d1aae20dab56be0de08772d03b72d19c:
				- remove some old root guide items that are no longer localized via
				Localization.m
		*)
		(* "ROOTGUIDEMETADATADESCRIPTION" -> Localized["RootGuideMetaDataDescription", GetParameter["language"]], *)
		(* "ROOTGUIDETITLE" -> Localized["RootGuideTitle", GetParameter["language"]], *)
		"<div class=\"rootfooter\">" -> "</div>\n<div class=\"footer\">",
		"&#160;<span class=\"inlineBullet\">&nbsp;&#9642;&nbsp;</span> <" -> "<span class=\"inlineBullet\">&nbsp;&#9642; </span><",
		"<span class=\"ThinSpace\"><!-- --></span>" -> "<span class=\"ThinSpace\"></span>",
		"&#160;&#160;<span>(" -> "&#160;<span>(",
		(* "<ul class=\"dropdown\">" -> "", *)
		"<span class=\"MenuNameDelimiter\"> &#9658; </span>" -> "<span class=\"MenuNameDelimiter\">&nbsp;&#9658;&nbsp;</span>",
(* Note: the following multi-line string literals cannot be indented. *)
		(* -------------------- *)
  "<div class=\"breadcrumb\">
 <div class=\"wrapper\">
  <em><a href=\"/language/guide/WolframRoot.html\">Mathematica</a></em>
 </div>
</div>" -> "",
		(* -------------------- *)
"<em><a href=\"/language/guide/WolframRoot.html\">Mathematica</a></em>
  <span class=\"breadcrumbSep\">&gt;</span>" -> "",
		(* -------------------- *)
		"(<a href=\"http://www.wolframscience.com/nksonline/\">A New Kind of Science</a>)" -> "(<a href=\"http://www.wolframscience.com/nksonline/\"><em>A New Kind of Science</em></a>)"
	}];

	(* **** modify filledinlayoutexpr when completeHTMLQ == True *)

	dprint["Defaults.m: filledinlayout expr just before testing for wsmQ vs completeHTMLQ: "]; dfile[filledinlayoutexpr, "Format" -> ".txt"];
	dprint["entering StringReplace action,
		GetParameter[\"webmQ\"] = "<>ToString[GetParameter["webmQ"], InputForm]<>"
		GetParameter[\"lwgQ\"] = "<>ToString[GetParameter["lwgQ"], InputForm]<>"
		GetParameter[\"wsmQ\"] = "<>ToString[GetParameter["wsmQ"], InputForm]<>"
		completeHTMLQ = "<>ToString[completeHTMLQ, InputForm]
	];

	(*
	GetParameter["webmQ"] = False
	GetParameter["lwgQ"] = False
	GetParameter["wsmQ"] = True
	completeHTMLQ = True *)

	(* Formatting note: a410487c:
		Updates for in-product WSM documentation
		Note: this commit is paired with a commit of wsm-assets in the DocumentationAssets repo.

		- use wsmQ=True and completeHTMLQ=True to set up in-product WSM use of wsm-assets
		- build on previous standalone work done for WebM and LightweightGrid
	*)
	filledinlayoutexpr = Which[
		(* Formatting note: 736980733d5a5f17794b1809bc485a65c8c6770a:
			Fix web wsm docs

			https://jira.wolfram.com/jira/browse/SYSTEMMODELER-2295

			We currently generate stand-alone in-product documentation for three
			products: Wolfram SystemModeler, webMathematica, and Lightweight Grid.
			Wolfram SystemModeler also has a web version at reference.wolfram.com.

			In Defaults.m, this modification of filledinlayoutexpr should happen
			only for in-product docs: webm, lwg, and in-product wsm. Web wsm docs
			should not be modified in this way.
		*)
		(* wsmQ has both web (completeHTMLQ == False) and in-product (completeHTMLQ ==
		True) versions.  Do not do the following for web wsm. *)
		(TrueQ[GetParameter["wsmQ"]] && TrueQ[completeHTMLQ])
		|| TrueQ[GetParameter["webmQ"]]
		|| TrueQ[GetParameter["lwgQ"]],
			StringReplace[filledinlayoutexpr, {
				If[TrueQ[GetParameter["wsmQ"]] && TrueQ[completeHTMLQ],
					"<body id=\"techNote\">" -> "<body id=\"techNote\"><main class=\"main\"><div class=\"inner main-content\">"
					,
					Nothing
				],

				(* ------------------------------ *)

				"<!--#include virtual=\"/language-assets/inc/feedback.html."<>GetParameter["la"]<>"\"-->" :> "",

				(* ------------------------------ *)

				"(<a href=\"http://www.wolframscience.com/nksonline/\">A New Kind of Science</a>)"
				:> "(<a href=\"http://www.wolframscience.com/nksonline/\"><em>A New Kind of Science</em></a>)"

			}]
		,
		(* WSM, and webMathematica's in-product documentation, for example *)
		completeHTMLQ === True,
			StringReplace[filledinlayoutexpr, {
				(* ------------------------------ *)
				(* TODO: Remove this branch entirely once we're confident it isn't used. *)
				"<!--#include virtual=\"/2013/includes/global-head2.html\"-->"
				| "<!--#include virtual=\"/2013/includes/global-head1.html\"-->"
				| "<!--#include virtual=\"/2013/includes/reference-header.html\"-->"
				| "<!--#include virtual=\"/2013/includes/reference-footer.html\"-->"
				| "<!--#include virtual=\"/2013/includes/global-footer.html\"-->"
					:> DocsError["Unexpected match of '/2013/includes/*' file. Contact connorg."],

				"<!--#include virtual=\"/common/includes/gl-header.html\"-->" :> "",

				"<!--#include virtual=\"/common/includes/gl-footer.html\"-->" :> "",

				"<!--#include virtual=\"/common/includes/gl-footer-includes.html\"-->" :> "",

				"(<a href=\"http://www.wolframscience.com/nksonline/\">A New Kind of Science</a>)"
					:> "(<a href=\"http://www.wolframscience.com/nksonline/\"><em>A New Kind of Science</em></a>)",
				(* ------------------------------ *)
				"<!--#include virtual=\"/language-assets/inc/feedback.html."<>GetParameter["la"]<>"\"-->" -> ""
			}]
		,
		True,
			filledinlayoutexpr
	];

	Module[{
		includes,
		docStart, headEnd, bodyStart, bodyEnd,
		rootGuideHeadStart, rootGuideBodyEnd,
		rules
	},
		includes = GetParameter["HTMLTemplateIncludes"];

		DocsAssert[AssociationQ[includes]];
		DocsAssert[MatchQ[includes, Association[(_?StringQ -> _?StringQ) ...]]];

		(* TODO:
			Warn when defaulting to None? Enable this warning once this template include
			logic is more elegantly threaded through the top level DocumentationBuild*[]
			API functions. Fix/change default value of "HTMLTemplateIncludes" in
			Options[ExportWebPage]? *)
		(* DocsWarning["Missing HTML server-side include expansion for '``'", label]; *)
		docStart  = Lookup[includes, "doc-start", None];
		headEnd   = Lookup[includes, "head-tag-end", None];
		bodyStart = Lookup[includes, "body-tag-start", None];
		bodyEnd   = Lookup[includes, "body-tag-end", None];

		(* The template include value should either be a string or None. *)
		DocsAssert[
			MatchQ[{docStart, headEnd, bodyStart, bodyEnd}, {Repeated[None | _?StringQ]}],
			"Invalid HTML template include"
		];

		(* TODO:
			Remove the `None` rules and the associated hacky
			`filledinlayoutexpr = StringReplace[...]` calls to replace the
			`<!--#include virtual=".."-->` strings embedded in the Layout/*.html files,
			and instead use ${doc-start/etc.} template parameters in the Layout/*.html
			files directly. At the moment this is just an extra layer of indirection. *)
		rules = {
			"<!--#include virtual=\"/language-assets/inc/doc-beginning.html."<>GetParameter["la"]<>"\"-->"
				-> docStart,
			"<!--#include virtual=\"/language-assets/inc/head-inc.html."<>GetParameter["la"]<>"\"-->"
				-> headEnd,
			"<!--#include virtual=\"/language-assets/inc/header.html."<>GetParameter["la"]<>"\"-->"
				-> bodyStart,
			"<!--#include virtual=\"/language-assets/inc/footer-inc.html."<>GetParameter["la"]<>"\"-->"
				-> bodyEnd
		};

		(* If this is a Root Guide page, add rules only used only by Root Guide pages. *)
		If[GetParameter["taggingRules"]["Metadata"]["type"] === "Root Guide",
			rootGuideHeadStart = Lookup[includes, "root-guide-head-tag-start", None];
			rootGuideBodyEnd   = Lookup[includes, "root-guide-body-tag-end", None];

			rules = Join[rules, {
				"<!--#include virtual=\"/language-assets/inc/footer-inc.html\"-->"
					-> bodyEnd,
				"<!--#include virtual=\"/common/framework/inc/head.en.php\"-->"
					-> rootGuideHeadStart,
				"<script src=\"/language-assets/js/root-guide.js\"></script>"
					-> rootGuideBodyEnd
			}];
		];

		(* Select the rules which aren't None. *)
		rules = Cases[rules, _ ~(Rule|RuleDelayed)~ _?StringQ];

		(*--------------------------*)
		(* Perform the replacement. *)
		(*--------------------------*)

		filledinlayoutexpr = StringReplace[filledinlayoutexpr, rules];
	];

	(* **** modify filledinlayoutexpr: special handling for WSM styles and headers *)

	filledinlayoutexpr = If[
		GetParameter["pacletName"] === "Wolfram SystemModeler"
		|| GetParameter["pacletName"] === "Wolfram System Modeler"
		|| GetParameter["pacletName"] === "Wolfram SystemModeler Library"
		,
		StringReplace[filledinlayoutexpr, {
			"<link rel=\"stylesheet\" type=\"text/css\" href=\"" <> relativeBase <> "css/reference.css\">"
				:> DocsError["Unexpected match of 'css/reference.css' in System Modeler HTML. Contact connorg."]
		}]
		,
		filledinlayoutexpr
	];

	filledinlayoutexpr = If[
		completeHTMLQ === False
		&& (StringMatchQ[GetParameter["pacletName"], "*SystemModeler*"|"*System Modeler*"])
		,
			(* TODO: Remove this code entirely after experimentally verifying that no
			         documentation builds start to fail due to the DocsError call below.
					 I believe this StringReplace is dead code. *)
			StringReplace[filledinlayoutexpr, {
				"<!--#include virtual=\"/2013/includes/global-head2.html\"-->"
				| "<!--#include virtual=\"/2013/includes/reference-header.html\"-->"
				| "<!--#include virtual=\"/2013/includes/global-footer.html\"-->"
					:> DocsError["Unexpected match of '/2013/includes/*' file in System Modeler HTML. Contact connorg."]
			}]
		,
		filledinlayoutexpr
	];

	(* **** modify filledinlayoutexpr: set up a standard header to modify: newin, icons, highlightlink *)

	(*
	<div class="main-heading">
		<h1 class="main-title">Plot</h1>
		<div class="iconography" data-src="s=t d=available m=available c=available u=t l= n= e=t"></div>
	</div>

	n -> GetParameter["newin"]
	u -> GetParameter["updatedin"]
	e -> GetParameter["experimentalQ"]
  c -> GetParameter["prerequisite"]
	*)

	{newin, updatedin, experimentalQ, prerequisite} = {GetParameter["newin"], GetParameter["updatedin"], GetParameter["experimentalQ"], GetParameter["prerequisite"]};
	dprint["Defaults.m: GetParameter[\"newin\"] = "<>ToString[GetParameter["newin"], InputForm]];
	dprint["Defaults.m: GetParameter[\"updatedin\"] = "<>ToString[GetParameter["updatedin"], InputForm]];
	dprint["Defaults.m: GetParameter[\"experimentalQ\"] = "<>ToString[GetParameter["experimentalQ"], InputForm]];
	dprint["Defaults.m: GetParameter[\"prerequisite\"] = "<>ToString[GetParameter["prerequisite"], InputForm]];

	filledinlayoutexpr = StringReplace[filledinlayoutexpr, {
		"<span class=\"UnfinishedDraft\"></span>" ~~ WhitespaceCharacter... -> "",
		"<span class=\"NewIn10Graphic\"></span>" ~~ WhitespaceCharacter... -> "",
		"<a href=\"#\" class=\"highlight-link\"></a>" ~~ WhitespaceCharacter... -> ""
	}];

	(* dprint["before inserting iconography, filledinlayoutexpr is:"]; dfile[filledinlayoutexpr, "Format" -> ".txt"]; *)

	(* guides can also have an EXPERIMENTAL banner, so don't limit this to functions *)
	filledinlayoutexpr = StringReplace[filledinlayoutexpr, "EICONSTATUS" :> If[experimentalQ, "t", ""]];
	filledinlayoutexpr = StringReplace[filledinlayoutexpr, "NICONSTATUS" :> GetParameter["newin"]]; (* this is either "11.3", for example, or "" *)
	filledinlayoutexpr = StringReplace[filledinlayoutexpr, "UICONSTATUS" :> GetParameter["updatedin"]];
	filledinlayoutexpr = StringReplace[filledinlayoutexpr, "SICONSTATUS" :> ""];
	filledinlayoutexpr = StringReplace[filledinlayoutexpr, "DICONSTATUS" :> ""];
	filledinlayoutexpr = StringReplace[filledinlayoutexpr, "MICONSTATUS" :> ""];
	filledinlayoutexpr = StringReplace[filledinlayoutexpr, "CICONSTATUS" :> Which[
    StringContainsQ[prerequisite, "Unsupported"], "u",
    StringContainsQ[prerequisite, "Restricted"], "r",
    True, ""
  ]];

	dprint["Defaults.m: after filling out iconography data, filledinlayoutexpr = "];
	dfile[filledinlayoutexpr, "Format" -> ".txt"];

	(* Formatting note: f2bb541d:
		https://jira.wolfram.com/jira/browse/WLDOCS-1572
		- when processing filledinlayoutexpr, move FullListing to a point just outside
		of and after the h1 per suef:
	*)
	If[StringContainsQ[filledinlayoutexpr, "<span class=\"FullListing\">"],
		filledinlayoutexpr = Module[{
			fullListing,
			e = filledinlayoutexpr,
			patt = "<span class=\"FullListing\"><a href=" ~~ Shortest[__] ~~ "</span></a></span>"
		},
			fullListing = StringCases[e, patt];
			e = StringReplace[e, patt -> ""];
			e = StringReplace[e, icon:"<div class=\"iconography\"" :> (Sequence @@ fullListing) ~~ "\n" ~~ icon]
		]
	];

	(* Formatting note: 68231d3a:
		- fix icon sizes in guide/RecentlyAddedFeatures without spending a day tracing
		through obscure transforms.
	*)
	(* Formatting note: f61b8123:
		Fix icon sizes in the 12.1 RecentlyAddedFeatures
		https://jira.wolfram.com/jira/browse/WLDOCS-2643
		- adjusting the numbers from the 12.0 build
	*)
	(* quick little icon resizing *)

	(* yes, it's brittle, but it beats ab-initio tracing of transforms I've never
	seen before when it has to get done because there's other stuff to get done. *)

	(* dprint["Defaults.m: fileBase = "<>ToString[fileBase, InputForm]]; *)

	If[StringContainsQ[fileBase, "RecentlyAddedFeatures"],
		filledinlayoutexpr = StringReplace[
			filledinlayoutexpr,
			{
				"height=\"21\" width=\"29\"" -> "width=\"15\"",
				"height=\"23\" width=\"29\"" -> "width=\"15\"",
				"height=\"24\" width=\"29\"" -> "width=\"15\"",
				"height=\"26\" width=\"29\"" -> "width=\"15\"",
				"height=\"35\" width=\"41\"" -> "width=\"21\"",
				"height=\"43\" width=\"55\"" -> "width=\"28\""
			}
		]
	];

	(* Formatting note: 6b529a3a6761dfbf8eb148cf9e7bb7f665488d52:
		Improve formatting of history info
		WLDOCS-1581

		Export.m:
			- per the jira ticket, add non-breaking spaces in the history info's
			"date (version)" units so they aren't broken apart at line breaks.  I
			don't see a way to use XML`RawXML["&nbsp;"] or otherwise get that
			string into the output here, so use a standard technique to resolve
			placeholders in Defaults.m

		XMLTransforms/HTML/Components/Defaults.m
			- resolve |NONBREAKINGSPACE| placeholders
	*)
	filledinlayoutexpr = StringReplace[filledinlayoutexpr, "|NONBREAKINGSPACE|" -> "&nbsp;"];

	(* Formatting note: 147b3b22d4e0f7eb019d09b5eae1fe74d11467b6:
		Implement new clipboard markup, part 1

		https://jira.wolfram.com/jira/browse/WLDOCS-844
		https://jira.wolfram.com/jira/browse/WLDOCS-2083

		XMLTransforms/HTML/Components/Defaults.m
			CLOSE-THIS-DIV:
				- the new In and Out divs need to be explicitly closed with a separate
				</div>, but I don't see a way to convince Transmogrify to do that.
				So emit a unique string to be cleaned up at the end of
				filledinlayoutexpr processing where similar things already happen.
	*)
	filledinlayoutexpr = StringReplace[filledinlayoutexpr, "CLOSE-THIS-DIV=\"True\" />" -> "></div>"];

	filledinlayoutexpr = StringReplace[filledinlayoutexpr, {"<DeleteThisElement>" -> "", "</DeleteThisElement>" -> ""}];

	(* Formatting note: eb725f83:
		Full web implementation of citations
		https://jira.wolfram.com/jira/browse/WLDOCS-3223
	*)
	filledinlayoutexpr = StringReplace[filledinlayoutexpr, {"CITATIONTIMESTAMP" -> "<!--#include virtual=\"/language-assets/inc/timestamp.html.en\"-->"}];

	If[!StringQ[filledinlayoutexpr],
		DocsError["Internal implementation error producing final HTML content."];
	];

	(* Print["Notebook transform point 6"]; *)
	filledinlayoutexpr

]
)
]  (* end WithParameters *)
], (* end Notebook transform *)

(* *** the rest of the file *)

(* **** History information *)

(* these were in Functions.m but are needed globally: *)
{Cell, "History"} :> XMLElement["div", {"class" -> "modified"}, {Recurse[]}],
{Cell, "HistoryDelimiter"} :> XMLElement["div", {"class" -> "history-delimiter"}, {Recurse[]}],
{Cell, "HistoryVersion"} :> XMLElement["div", {"class" -> "history-version"}, {Recurse[]}],

(* **** continuing *)

{Cell, "feBitmap"} :> Module[
  {filename, bitmapname, export, imageSize}
,
  bitmapname = First[SelectSelf[]];
  filename = FileNameJoin[{"Files", GetBasenameLa[], "FEBitmap_" <> IncrementCounter["Image"] <> ".png"}];
  export = DocumentationBuild`Utils`exportFEBitmap[filename, bitmapname, "PNG"];
  If[export == $Failed, Return[$Failed]];

  imageSize = Import[filename, "ImageSize"];

  XMLElement["img", {
    "src" -> StringReplace[filename, {"\\"->"/"}],
    "width" -> ToString@First@imageSize,
    "height" -> ToString@Last@imageSize,
    "alt" -> ""
  }, {}]
],

{Cell, "UnfinishedDraft"} :> XMLElement["span", {"class"->"UnfinishedDraft"},{""}],

{Cell, "GuideColorBar" | "SymbolColorBar" | "TutorialColorBar"} :> Unevaluated[Sequence[]],

(* define how to handle box types *)
{Cell, "DefinitionBox" | "DefinitionBox1Col" |"DefinitionBox3Col" | "DefinitionBox4Col" | "DefinitionBox6Col" | "OptionBox3Col" | "DoubleBox"| "FramedBox" | "1ColumnBox" | "2ColumnBox" | "3ColumnBox" | "4ColumnBox" | 
"4ColumnBoxClean" |"2ColumnSmallBox" | "2ColumnTable" | "6ColumnBox" | "43Picture" | "PrecedenceTable" | 
"47Picture" | "47PictureLeft"} :> 
DIV[ {"class" -> "MathTable "<>GetStyle[]}, {DocumentationBuild`Utils`AddSpans[Recurse[] ]} ],

{Cell, "1ColumnTableModStandalone"
  | "2ColumnTableStandalone"
  | "3ColumnTableStandalone"
  | "2ColumnTableModStandalone"
  | "3ColumnTableModStandalone"
  | "3ColumnIllustratedTableModStandalone"} :>
  XMLElement[
    "ul",
    {"class" -> "functionList"},
    Li[
      {"class" -> "NotesTable"},
      {DocumentationBuild`Utils`AddSpans[Recurse[]]}
    ]
  ], 

{Cell, "AlphabeticalListing"} :> 
DIV[ {"class" -> "AlphabeticalListing"}, {Recurse[]} ],

{Cell, "LinkTrail"} :>
DIV[ {"class" -> "wrapper"}, {Recurse[]}],

(* TODO use brendan's CellToStringCode to add sensible Alt/Title tags  *)
FractionBox | RadicalBox | SqrtBox | OverscriptBox | UnderscriptBox | UnderoverscriptBox | SubsuperscriptBox :>
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> "OverscriptBox.png"],
  {"style"->"vertical-align:middle;", "alt"->"", "title"->""},
  Inline -> True,
  CropImage -> False,
  ImageFormat -> "PNG",
  TransparentBackground->True,
  ConversionOptions->{Background->None},
  "RetinaImage" -> True
], 


(* TODO How to add sensible Alt/Title tags?  *)
SliderBox | Slider2DBox | OpenerBox :>
(
Export[ToFileName[{"Files", GetBasenameLa[]},
  "Control_" <> IncrementCounter["Image"] <> ".gif"],
  SelectLiteral[ SelectSelf[] ]
]
), 


(***** Cell Definitions *****)


{CellGroupData, "TOCChapter"} :> 
Block[{name, style, ex, count},
count = IncrementCounter["TOCChapter"];
style = "TOC";
name = "TOC"; 
(*ex = If[GetParameter["pacletName"]==="Alpha","_Ex","_Co"];*)
ex = If[MemberQ[{"Alpha", "webMathematica"},GetParameter["pacletName"]],"_Ex","_Co"];
(*ex =  "_Co";*) (* ex =  "_Ex"; *)

DIV[{"class" -> name<>ex, "name" -> name<>count, "id" -> name<>count}, {
Transmogrify[First[SelectChildren[]]],
DIV[{"class" -> name, "name" -> name<>"G", "id" -> name<>"G" },{
  Transmogrify[Rest[SelectChildren[]]] }]
}] ], 


{Cell, "TOCChapter" | "TOCPart"} :> 
Module[{
name, style, ex, newcount},
newcount = Switch[Head[count], 
		Symbol, "",
		String, count];
style === "TOCH";
name = "TOC";
ex = "closed";

Switch[Head[count], 
Symbol, H1[ {"id" -> name<>"H"}, {Recurse[]}],
String, 

  DIV[ {"class" -> name<>"H", "name" -> name<>newcount<>"H"}, {
H1[{"class" -> "TOCChapter"}, {
    A[{"href" -> ("javascript:toggle('" <> name <> newcount <> "')"),
       "onmouseover" -> "javascript:return(true);" }, {
    Img[{"src"->(GetParameter["htmlBase"]<>"/mathematicaImages/"<>ex<>"SectionGroup.gif"), "id"->name<>newcount<>"_", "border"->"0", "width"->"8", "height"->"8", "alt"->""}], 
      " ",
      Span[{"class"->name<>"Span"}, { Recurse[] }]
    }]
}] (* end h1 *)
}] (* end Div *)

] (* end Switch *)
], 


{Cell, "UnderDevelopment"} :> 
  DIV[{"class"->"UnderDevelopment"}, {"U N D E R ", XML`RawXML["&nbsp;"], " ", XML`RawXML["&nbsp;"], "D E V E L O P M E N T"}],
  
{CellGroupData, "TOCChapter" | (*"TOCSection" |*) (* "TOCSubsection" |*) "TOCSubsubsection"} :> 
	DIV[{"class"->GetStyle[]}, {Recurse[]}],

{CellGroupData, "TOCSection"} :> 
Block[{name, style, ex, tocSectionCount},
tocSectionCount = IncrementCounter["TOCSection"];
style = "TOCSEC";
name = "TOCSEC"; 
(*ex =  "_Co";*)
(*ex =  If[GetParameter["pacletName"]=== "Alpha", "_Ex", "_Co"];*)
ex =  If[MemberQ[{"Alpha","webMathematica"}, GetParameter["pacletName"]], "_Ex", "_Co"];

DIV[{"class" -> name<>ex, "name" -> name<>tocSectionCount, "id" -> name<>tocSectionCount}, {
Transmogrify[First[SelectChildren[]]],
DIV[{"class" -> name, "name" -> name<>"G", "id" -> name<>"G" },{
  Transmogrify[Rest[SelectChildren[]]] }]
}] ], 


{Cell, "TOCSection"} :> 
Module[{
name, style, ex, newcount},
newcount = Switch[Head[tocSectionCount], 
		Symbol, "",
		String, tocSectionCount];
style === "TOCSECH";
name = "TOCSEC";
ex = "closed";

Switch[Head[tocSectionCount], 
Symbol, H2[ {"id" -> name<>"H"}, {Recurse[]}],
String, 

  DIV[ {"class" -> name<>"H", "name" -> name<>newcount<>"H"}, {
H2[{"class" -> "TOCSection"}, {
    A[{"href" -> ("javascript:toggle('" <> name <> newcount <> "')"),
       "onmouseover" -> "javascript:return(true);" }, {
    Img[{"src"->(GetParameter["htmlBase"]<>"/mathematicaImages/"<>ex<>"SectionGroup.gif"), "id"->name<>newcount<>"_", "border"->"0", "width"->"8", "height"->"8", "alt"->""}], 
      " ",
      Span[{"class"->name<>"Span"}, { Recurse[] }]
    }]
}] (* end h1 *)
}] (* end Div *)

] (* end Switch *)
], 


{CellGroupData, "TOCSubsection"} :> 
Block[{name, style, ex, tocSubsectionCount},
tocSubsectionCount = IncrementCounter["TOCSubsection"];
style = "TOCSU";
name = "TOCSU"; 
ex =  "_Co"; (* ex =  "_Ex"; *)

DIV[{"class" -> name<>ex, "name" -> name<>tocSubsectionCount, "id" -> name<>tocSubsectionCount}, {
Transmogrify[First[SelectChildren[]]],
DIV[{"class" -> name, "name" -> name<>"G", "id" -> name<>"G" },{
  Transmogrify[Rest[SelectChildren[]]] }]
}] ], 


{Cell, "TOCSubsection"} :> 
Module[{
name, style, ex, newcount},
newcount = Switch[Head[tocSubsectionCount], 
		Symbol, "",
		String, tocSubsectionCount];
style === "TOCSUH";
name = "TOCSU";
ex = "closed";

Switch[Head[tocSubsectionCount], 
Symbol, H2[ {"id" -> name<>"H"}, {Recurse[]}],
String, 

  DIV[ {"class" -> name<>"H", "name" -> name<>newcount<>"H"}, {
H2[{"class" -> "TOCSubsection"}, {
    A[{"href" -> ("javascript:toggle('" <> name <> newcount <> "')"),
       "onmouseover" -> "javascript:return(true);" }, {
    Img[{"src"->(GetParameter["htmlBase"]<>"/mathematicaImages/"<>ex<>"SectionGroup.gif"), "id"->name<>newcount<>"_", "border"->"0", "width"->"8", "height"->"8", "alt"->""}], 
      " ",
      Span[{"class"->name<>"Span"}, { Recurse[] }]
    }]
}] (* end h1 *)
}] (* end Div *)

] (* end Switch *)
], 

{CellGroupData, "GuideReferenceSection"} :> 
  XMLElement["section", If[GetParameter["url"] == "guide/RecentlyAddedFeatures", {"id" -> "Reference"}, {}],	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["GuideReferenceSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],

{Cell, "GuideReferenceSection"} :> Unevaluated[Sequence[]],

{Cell, "MathCaption"} :> WithParameters[{"newline2br" -> True},
	DIV[{"class" -> "MCap"}, {Recurse[]}]  ],

{Cell, "TOCChapter" | "TOCSection" | "TOCSubsection" | "TOCSubsubsection"} :> Ol[{"class"->GetStyle[]}, {
Li[{"class"->GetStyle[]}, {Recurse[]}]
}], 

{Cell, "InlineCharacterDisplay"} :> 
  Span[{"class"->"InlineCharacterDisplay"}, {Recurse[]}],

(* no FooterCell has content as of V11.1.1, so make no provision for it *)
{Cell, "FooterCell"} :> RawXML["\n  <!--#include virtual=\"/language-assets/inc/feedback.html."<>GetParameter["la"]<>"\"-->\n"],

{Cell, "WorkflowFooterCell"} :> RawXML["\n  <!--#include virtual=\"/language-assets/inc/feedback.html."<>GetParameter["la"]<>"\"-->\n"],

{Cell, "TI"} :> Span[{"class"->GetStyle[]}, Recurse[]],

{CheckboxBox | RadioButtonBox} :> 
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, "CellNone_" <> IncrementCounter["Image"] <> ".png"],
  {"alt"->"","style" -> "vertical-align:middle;"},
  Inline -> True,
  CropImage -> False,
  ImageFormat -> "PNG",
  TransparentBackground->True,
  ConversionOptions->{Background->None},
  "RetinaImage" -> True
], 

{Cell,None} :> Module[{
  tmp1=SelectChildren[],
  tmp2=Recurse[]
  (*tmp3=SelectAncestors[Cell] *)
},
  If[
    Head[tmp1[[1]]]===CellGroupData, 
    If[Cases[tmp1, __]==={CellGroupData[{}, Open]}, "", tmp2 ],
    If[tmp2===Null, "",
    Which[htmlBoxesQ[SelectLiteral[SelectSelf[]]], tmp2, 
		Cases[tmp1, _CheckboxBox, Infinity]=!={}, tmp2,
		Cases[tmp1, _RadioButtonBox, Infinity]=!={}, tmp2,
		True,
        transmogrifyImage[
          ToFileName[{"Files", GetBasenameLa[]}, "CellNone_" <> IncrementCounter["Image"] <> ".png"],
          {"alt"->"","style" -> "vertical-align:middle;"},
          Inline -> True,
          CropImage -> False,
          ImageFormat -> "PNG",
          TransparentBackground->True,
          ConversionOptions->{Background->None},
          "RetinaImage" -> True
        ]
      ]
  ]
  ]

],  

(* TODO How to add sensible Alt/Title tags? *)
{Cell, "Graphics"} :> 
DIV[{"class"->GetStyle[]}, {
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Graphics"] <> "_g.gif"],
  {"alt"->""},
  Inline -> True,
  CropImage -> False,
  "RetinaImage" -> True
]}], 

{Cell, "ItemizedPicture" | "ItemPicture"} :> 
DIV[{"class"->GetStyle[]}, {
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Graphics"] <> "_g.gif"],
  {"alt"->""},
  Inline -> True,
  CropImage -> False,
  "RetinaImage" -> True
]}], 

(* spacer added by Greg for InputCells with no Output followed by ExampleDelimiter *)
{Cell, "WeakDivider" | "StrongDivider" | "InternalFlag" | "ImportExportSection" |
"ImportExportSection" | "NotebookInterfaceSection" | "ElementsSection" | "ExportElementsSection" | "Spacer" | "TableHeader" | "MonographTableHeader"} :> 
DIV[ {"class" -> (GetStyle[]/.None->"FIXME")
}, {Recurse[]}],


 {Cell, "ObsolescenceNote"} :>  XMLElement["div", {"class"->"ObsolescenceNote"}, {Recurse[]}],


{Cell, "TableRowIcon"}:>  Sequence[],
{Cell, "CloudIconGraphic"}:> Sequence[],

{Cell, "BrighterFlag"}:> DIV[{"class"->"BrighterFlag"},{
	XMLElement["img", {"src"->"/mathematicaImages/FutureFlag.gif"}, {}]}
	],

{Cell, "MenuName"} :>
Span[ {"class" -> "MenuName"}, {Recurse[]}],

{Cell, "InlineCharacterName"} :>
Span[ {"class" -> "ICN"}, {Recurse[]}],

(* not changed *)
{Cell, "ModInfo"} :> 
Span[ {"class" -> "ModInfo", "id" -> "ModInfo_"<>"8", "name" -> "ModInfo_"<>"8"}, {Recurse[]}],


{Cell, "TableText"|"MonographTableText"} :> 
Span[ {"class" -> "TT"}, {Recurse[] } ],


(* not changed *)
{Cell, "ListNoteBox"} :> 
DIV[ {"class" -> "MathTable"}, {DocumentationBuild`Utils`AddSpans[Recurse[] ]} ], 



(*   removed: ExampleText should be handled by the Cell, All rule.  *)


(* not changed *)
{Cell, "Caption" | "CaptionText"} :> 
P[ {"class" -> "Caption"}, {Recurse[]} ], 

{Cell, "UpgradeLabel"} :> 
P[{"class" -> "UpgradeLabel"}, {Recurse[]} ], 

{Cell, "UpgradeDetail"} :> 
P[{"class" -> "UpgradeDetail"}, {Recurse[]} ],

{Cell, "Text"} :> Module[
  {cellPrintQ},
  cellPrintQ = 
	If[
    StringQ[preservedInputForm]
  ,
    TrueQ[StringTake[preservedInputForm, 9] === "CellPrint" || StringTake[preservedInputForm, 12] === "Do[CellPrint"]
  ,
		False
	];

  If[
    cellPrintQ
  ,
    transmogrifyImage[
      ToFileName[{"Files", GetBasenameLa[]}, "oText_" <> IncrementCounter["oText"] <> ".gif"],
      {"alt" -> ""},
      Inline -> False,
      MaxImageSize->580,
      CropImage -> False,
      "RetinaImage" -> True]
	,
    DIV[ {"class" -> "Text"}, {Recurse[]} ]
  ]
], 

{Cell, "ExampleText"} :> P[{}, {Recurse[]} ], 

{Cell, "CR" | "CompactText" | "GroupDivider" | (*"Item" |*) "Leader"} :>
P[ {"class" -> "Text"}, {Recurse[]} ], 


{Cell, "DisplayText" | "Compact12Text" (* | "DisplayFormula" *)} :>
P[ {"class" -> "DT"}, {Recurse[]} ], 


(* not changed *)
{CellGroupData, "BulletedList" | "BulletedText" } :> XMLElement["ul", {}, {Recurse[]}  ],

{Cell, "Item" } :> WithParameters[{"newline2br" -> True},
  (* AntLog["entering {Cell, \"Item\" } with SelectSelf[] = "<>ToString[SelectSelf[], InputForm]]; *)
  XMLElement["ul", {"class"->"Text"}, {Li[{"class" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]} ]}  ]],

{Cell, "Subitem" } :> XMLElement["ul", {"class"->"Text"}, {Li[{"class" -> "Subitem"}, {Recurse[]} ]}  ],

{Cell, "Subsubitem" } :> XMLElement["ul", {"class"->"Text"}, {Li[{"class" -> "Subsubitem"}, {Recurse[]} ]}  ],

(* not changed *)
{CellGroupData, (* "ItemizedText" | *)"ItemizedList" | "ItemizedSubList"} :>
Ol[{"class" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]} ],


{Cell, "BulletText" | "BulletedList" | (* "ItemizedText" | *)"ItemizedList" | "ItemizedSubList"} :>
Li[{"class" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]} ], 


{Cell, "ItemizedText"} :> P[{"class" -> (GetStyle[]/.None->"FIXME")}, 
Sequence@{XMLElement["span", {}, {IncrementCounter["ItemizedText"] <> ". ", XML`RawXML["&nbsp;"]}],
Recurse[]
} ],

{Cell, "ItemNumbered"} :> P[{"class" -> (GetStyle[]/.None->"FIXME")}, 
Sequence@{XMLElement["span", {}, {IncrementCounter["ItemNumbered"] <> ". ", XML`RawXML["&nbsp;"]}],
Recurse[]
} ],

{Cell, "ItemParagraph"} :> P[{"class" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]} ], 

{Cell, "SectionHowTo"} :> H2[{"class" -> "SectionHowTo"}, {Recurse[]}],
{Cell, "SubsectionHowTo"} :>
  XMLElement["section", {}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h3", {"class" -> "SubsectionHowTo"}, {Recurse[]}]
    }]
  }],
(* H3[{"class" -> "SubsectionHowTo"}, {Recurse[]}], *)
{Cell, "SubsubsectionHowTo"} :> H4[{"class" -> "SubsubsectionHowTo"}, {Recurse[]}],

{Cell, "DeviceSection"} :> H2[{"class" -> "SectionHowTo"}, {Recurse[]}],
{Cell, "DeviceSubsection"} :> H3[{"class" -> "SubsectionHowTo"}, {Recurse[]}],
{Cell, "DeviceSubsubsection"} :> H4[{"class" -> "SubsubsectionHowTo"}, {Recurse[]}],

{Cell, "BulletedText"} :>
UL[{}, {
Li[{"class" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]} ]
}], 


{Cell, "Notes"} :> Li[Flatten[
	{If[HasOption[Background], 
          If[MatchQ[ GetOption[Background], _Dynamic], {"class" -> "modified-text"}, {}], {}]}
          ], {Recurse[]}],

{Cell, "MessageNote"} :> Li[Flatten[
	{If[HasOption[Background], 
          If[MatchQ[ GetOption[Background], _Dynamic], {"class" -> "modified-text"}, {"class"->"MessageNote"}], {"class"->"MessageNote"}]}
          ], {Recurse[]}],

{Cell, "FormatNotes"} :>
  Li[Flatten[
	{If[HasOption[Background], 
          If[MatchQ[ GetOption[Background], _Dynamic], {"class" -> "modified-text"}, {}], {}]}
          ], {Recurse[]}],
          
{Cell, "InterpreterNotes"} :>
  Li[Flatten[
	{If[HasOption[Background], 
          If[MatchQ[ GetOption[Background], _Dynamic], {"class" -> "modified-text"}, {}], {}]}
          ], {Recurse[]}], 

{Cell, "DeviceNotes"} :>
  Li[Flatten[
	{If[HasOption[Background], 
          If[MatchQ[ GetOption[Background], _Dynamic], {"class" -> "modified-text"}, {}], {}]}
          ], {Recurse[]}], 

{Cell, "FormatNotesFirst"} :>
  XMLElement["ul", {If[
        HasOption[CellDingbat], 
          If[MatchQ[ GetOption[CellDingbat], (Cell["6+","ModInfo"] | StyleBox[_, "ModInfo", ___])],
           Sequence@@{"class" -> "FN1", "id"->"ModInfo_"<>"8", "name"->"ModInfo_"<>"8"}, 
            "class" -> "FN1"],   
        "class" -> "FN1"]}, {   
      XMLElement["li", {}, {Recurse[]}] 
  }], 



(* not changed *)
{Cell,  "ItemizedTextNote" | "ItemizedNote" | "ListNote"} :> 
P[ {"class" -> "Item2", "name" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]} ],

{Cell,  "FunctionEssay"} :> 
Module[{functionessaycells}, 
	functionessaycells = Flatten[Cases[SelectSelf[], If[_, a_, b_] :> {a}, Infinity]];
	functionessaycells = Flatten[functionessaycells /. Cell[BoxData[{w___}], "FunctionEssay"]:>{w}];
	If[Length[Cases[functionessaycells, StyleBox["Show Less \[LeftGuillemet]", "FunctionEssay"], Infinity]] > 0,
	    {},
		If[Length[functionessaycells]>0,
		Transmogrify[functionessaycells],
		Li[ {"class" -> "functionessay"}, {Recurse[]} ]
		]
	]
],

{Cell, "IndentedText" | "NotesList" | "TextBelow" | "Item1" | "Contents"} :>
(* P[ {"class" -> "IndentedText", "name" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]} ], *)
P[ {"class" -> "IndTex", "name" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]} ],



(* not changed *)
{Cell, "TextSans" | "SmallTextSans" | "S9" | "HelpLabels"} :>
P[ {"class" -> "TextSans", "name" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]} ],


(* not changed *)
{Cell, "SmallText" | "CopyrightText"} :>
P[ {"class" -> "SmallText", "name" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]} ],



{Cell, "IndexSection" | "Reference" | "LineHead"} :>
(* P[ {"class" -> "IndexSection", "name" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]} ], *)
P[ {"class" -> "IS"<>" "<>GetStyle[]/.None->"FIXME"}, {Recurse[]} ],


{Cell, "Program" | "Code"} :> WithParameters[{"newline2br" -> True},
  If[
    htmlBoxesQ[SelectLiteral[SelectSelf[]]]
  ,
    XMLElement[
      "div",
      {"class" -> (GetStyle[]/.None->"FIXME")},
      {
        XMLElement[
          "pre",
          {},
          Flatten@{
            If[
              HasOption[CellTags]
            ,
              Flatten[{Map[(XMLElement["a", {"name" -> #}, {}])&, Flatten[{GetOption[CellTags]}] ], Recurse[]  }]
            ,
              {Recurse[]  }
            ]
          }
        ]
      }
    ]
  ,
    DIV[
      {"class" -> GetStyle[]}, {
        transmogrifyImage[
          ToFileName[{"Files", GetBasenameLa[]}, "Box_" <> IncrementCounter["Inline"] <> ".gif"],
          {"class"->"FlushInput"},
          Inline -> False,
          CropImage -> False,
          "RetinaImage" -> True
        ]
      }
    ]
  ]
],


{Cell, "Sound"} :>
A[{"href" ->
    WithParameters[{"href" ->
          (StringReplace[GetBasenameLa[], ".en" -> ""] <> "_" <> IncrementCounter["Sound"] <> ".wav")}, 
      Export[GetParameter["href"], SelectSelf[], "WAV"];
      GetParameter["href"]
  ]}, {Recurse[]} 
],  

{Cell, "SectionHeaderSpacer"}:>Unevaluated[Sequence[]],
{Cell, "SectionFooterSpacer"}:>Unevaluated[Sequence[]],

{Cell, "Chapter" | "Title" | "ChapTitle"} :>
P[ {"class" -> "Title", "name" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]} ],


{Cell, "Section" | "SelArg" | "F12" | "Line1" | "LineSpace" |
"MagicInlineCellMarker" | "PathText" | "SectionFirst" | "SymbolicExpression" | "TOCPage" | 
"TOCSection" | "TOCSubsection" | "TOCSectionDingbat" | "TOCSubsectionDingbat" | "TOCTitle" |
"TOCTitleDingbat" | "TOCTitleDingbat2" | "ObjectName" | "ObjectNameSmall"} :> 
H1[{"class" -> If[styleswitch==="HowTo",(GetStyle[]/.None->"FIXME")<>styleswitch,(GetStyle[]/.None->"FIXME")]}, {Recurse[]}],


{Cell, "Subsection" | "SubsectionIcon" | "SubsectionFirst" | "Command" | "ContentsTitle" | "InspectorLevel1"} :>
H2[{"class" -> If[styleswitch==="HowTo",(GetStyle[]/.None->"FIXME")<>styleswitch,(GetStyle[]/.None->"FIXME")]}, {Recurse[]}],

{Cell, "Subsubsection" | "SubsubsectionIcon" | "InspectorLevel2"} :>
If[styleswitch==="HowTo",
	DIV[{"class" -> "howto-main-links"}, {Recurse[]}],
	H3[{"class" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]}]],

{Cell, "Subsubtitle" | "Subsubsubsection" | "ItemDingbat"} :>
WithParameters[{"newline2br" -> True},
	H4[{"class" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]}]
],

{Cell, "NotesSubsection"} :> XMLElement["h5", {}, {Recurse[]}],

{Cell, "SessionLabel"} :> XMLElement["h3", {}, {Recurse[]}],

{Cell, "Subtitle"} :> WithParameters[{"newline2br" -> True},
H3[{"class" -> (GetStyle[]/.None->"FIXME")}, {
	XMLElement["font", {"color" -> "#000066", "face" -> "italic"}, {Recurse[]}] 
	} ]
],  


{Cell, "SubsubsectionColor"} :> WithParameters[{"newline2br" -> True},
H3[{"class" -> (GetStyle[]/.None->"FIXME")}, {
	XMLElement["font", {"color" -> "#330066"}, {Recurse[]}] 
	} ]
],  


{Cell, "CharacterObjectWarningMessages"} :>
(*  P[ {"class" -> "CharacterObjectName"}, {Recurse[]} ],   *)
P[ {"class" -> "CON"}, {Recurse[]} ], 


{Cell, "CopyrightHead"} :>
  WithParameters[{"newline2br" -> True},
    P[{"class" -> "SmallText"}, {
      XMLElement["b", {}, {Recurse[]}]
    }]  ],


{Cell, "Header"} :>
  WithParameters[{"newline2br" -> True},
    P[{"class" -> "SmallText"}, {
      XMLElement["i", {}, {Recurse[]}]
    }]  ],
  
{Cell, "KeyEvent"} :>XMLElement["span", {"class"->"KeyEvent"}, {Recurse[]}],

(* TODO use Brendan's code to add sensible Alt/title tags. *)
{Cell, "Message" | "StandardForm"} :>
  XMLElement["div", {"class" -> "Message"}, {
    transmogrifyImage[
      ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
      {"alt"->""},
      Inline -> True,
      CropImage -> False,
      "RetinaImage" -> True
    ]
  }],


(* TODO use Brendan's code to add sensible Alt/title tags. *)
{Cell, "Print"} :>
  DIV[
    {"class" -> "Print"}, {
      transmogrifyImage[
        ToFileName[{"Files", GetBasenameLa[]}, "Image_" <> IncrementCounter["Image"] <> ".gif"],
        {"class" -> "FlushInput"},
        Inline -> False,
        CropImage -> False,
        "RetinaImage" -> True
      ]
    }
  ],


{TextData, All} :> Recurse[],



(** Input/Ouput Cell Groups **)
{CellGroupData, "Input"} :> Recurse[],

(** Input Cells **)
(* TODO use Brendan's code to add sensible Alt/title tags to 'InlineInput'. *)
{Cell, "InlineInput"} :>
If[
  htmlBoxesQ[ SelectLiteral[ SelectSelf[] ] ]
, 
  Transmogrify[ SelectChildren[] ]
,
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, "Box_" <> IncrementCounter["Inline"] <> ".gif"],
    {"style" -> "vertical-align:middle", "class" -> "FlushInput", "alt" -> " "},
    Inline -> True,
    CropImage -> False,
    "RetinaImage" -> True
  ]
],



(** Input Cells **)
(* TODO USe Brendan's code to add sensible Alt/Title tags to Deemphasis. *)
(* TODO Update to use transmogrifyImage[]. *)
{Cell, "Deemphasis"(* | "Code"  *)} :>
DIV[{"class" -> GetStyle[]}, {
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, "Box_" <> IncrementCounter["Inline"] <> ".gif"],
    {"class" -> "FlushInput", "alt" -> " "},
    Inline -> False,
    CropImage -> False,
    "RetinaImage" -> True
  ]
}],





{Cell, "IndexSection" | "Reference" | "LineHead"} :>
P[{"class" -> "IS", "name" -> (GetStyle[]/.None->"FIXME")}, 
  If[HasOption[CellTags],
  Flatten[ {
    Map[
    (XMLElement["a", {"name" -> #}, {"yyyyyyy"}])&, Flatten[{GetOption[CellTags]}] ],
    Recurse[]  }],
  {Recurse[]}  ]    
],


(* TODO USe Brendan's code to add sensible Alt/Title tags to LegacyInput. *)
{Cell, "LegacyInput"} :>
Module[{iForm},
iForm = CreateInputForm[ SelectSelf[] ];
iForm = If[Head@iForm === String, iForm, ToString@iForm];

DIV[{"class" -> "LegacyInput"}, 
	{
    XMLElement["span", {"class"->"LegacyVersion"}, {"Version 5.2"}],
    transmogrifyImage[
      ToFileName[{"Files", GetBasenameLa[]}, "legacy_" <> IncrementCounter["legacy"] <> ".gif"],
      {"alt" -> iForm},
      Inline -> True,
      CropImage -> False,
      "RetinaImage" -> True
    ]}]
],

{Cell, "LegacyInputV7"} :>
Module[{iForm},
iForm = CreateInputForm[ SelectSelf[] ];
iForm = If[Head@iForm === String, iForm, ToString@iForm];

DIV[{"class" -> "LegacyInputV7"}, 
	{
    XMLElement["span", {"class"->"LegacyVersion"}, {"Version 6.0"}],
    transmogrifyImage[
      ToFileName[{"Files", GetBasenameLa[]}, "legacy_" <> IncrementCounter["legacy"] <> ".gif"],
      {"alt" -> iForm},
      Inline -> True,
      CropImage -> False,
      "RetinaImage" -> True
    ]}]
],

{Cell, "LegacyInputV8"} :>
Module[{iForm},
iForm = CreateInputForm[ SelectSelf[] ];
iForm = If[Head@iForm === String, iForm, ToString@iForm];

DIV[{"class" -> "LegacyInputV7"}, 
	{
    XMLElement["span", {"class"->"LegacyVersion"}, {"Version 7.0"}],
    transmogrifyImage[
      ToFileName[{"Files", GetBasenameLa[]}, "legacy_" <> IncrementCounter["legacy"] <> ".gif"],
      {"alt" -> iForm},
      Inline -> True,
      CropImage -> False,
      "RetinaImage" -> True
    ]}]
],

{Cell, "LegacyInputV9"} :>
Module[{iForm},
iForm = CreateInputForm[ SelectSelf[] ];
iForm = If[Head@iForm === String, iForm, ToString@iForm];

DIV[{"class" -> "LegacyInputV7"}, 
	{
    XMLElement["span", {"class"->"LegacyVersion"}, {"Version 8.0"}],
    transmogrifyImage[
      ToFileName[{"Files", GetBasenameLa[]}, "legacy_" <> IncrementCounter["legacy"] <> ".gif"],
      {"alt" -> iForm},
      Inline -> True,
      CropImage -> False,
      "RetinaImage" -> True
    ]}]
],

{Cell, "LegacyInputWE"} :>
Module[{iForm},
iForm = CreateInputForm[ SelectSelf[] ];
iForm = If[Head@iForm === String, iForm, ToString@iForm];

DIV[{"class" -> "LegacyInputV7"}, 
	{
    XMLElement["span", {"class"->"LegacyVersion"}, {"Wavelet Explorer"}],
    transmogrifyImage[
      ToFileName[{"Files", GetBasenameLa[]}, "legacy_" <> IncrementCounter["legacy"] <> ".gif"],
      {"alt" -> iForm},
      Inline -> True,
      CropImage -> False,
      "RetinaImage" -> True
    ]}]
],

{Cell, "LegacyInputError"} :>
Module[{iForm},
iForm = CreateInputForm[ SelectSelf[] ];
iForm = If[Head@iForm === String, iForm, ToString@iForm];

DIV[{"class" -> "LegacyInputError"}, 
	{
    XMLElement["span", {"class"->"LegacyVersion"}, {"Version 5.2"}],
    transmogrifyImage[
      ToFileName[{"Files", GetBasenameLa[]}, "legacy_" <> IncrementCounter["legacy"] <> ".gif"],
      {"alt" -> iForm},
      Inline -> True,
      CropImage -> False,
      "RetinaImage" -> True
    ], XMLElement["span", {"class"->"LegacyError"}, {"returns unevaluated"}]}]
],

{Cell, "LegacyInputErrorV7"} :>
Module[{iForm},
iForm = CreateInputForm[ SelectSelf[] ];
iForm = If[Head@iForm === String, iForm, ToString@iForm];

DIV[{"class" -> "LegacyInputErrorV7"}, 
	{
    XMLElement["span", {"class"->"LegacyVersion"}, {"Version 6.0"}],
    transmogrifyImage[
      ToFileName[{"Files", GetBasenameLa[]}, "legacy_" <> IncrementCounter["legacy"] <> ".gif"],
      {"alt" -> iForm},
      Inline -> True,
      CropImage -> False,
      "RetinaImage" -> True
    ], XMLElement["span", {"class"->"LegacyError"}, {"returns unevaluated"}]}]
],

{Cell, "LegacyInputErrorV8"} :>
Module[{iForm},
iForm = CreateInputForm[ SelectSelf[] ];
iForm = If[Head@iForm === String, iForm, ToString@iForm];

DIV[{"class" -> "LegacyInputErrorV7"}, 
	{
    XMLElement["span", {"class"->"LegacyVersion"}, {"Version 7.0"}],
    transmogrifyImage[
      ToFileName[{"Files", GetBasenameLa[]}, "legacy_" <> IncrementCounter["legacy"] <> ".gif"],
      {"alt" -> iForm},
      Inline -> True,
      CropImage -> False,
      "RetinaImage" -> True
    ], XMLElement["span", {"class"->"LegacyError"}, {"returns unevaluated"}]}]
],

{Cell, "LegacyInputErrorV9"} :>
Module[{iForm},
iForm = CreateInputForm[ SelectSelf[] ];
iForm = If[Head@iForm === String, iForm, ToString@iForm];

DIV[{"class" -> "LegacyInputErrorV7"}, 
	{
    XMLElement["span", {"class"->"LegacyVersion"}, {"Version 8.0"}],
    transmogrifyImage[
      ToFileName[{"Files", GetBasenameLa[]}, "legacy_" <> IncrementCounter["legacy"] <> ".gif"],
      {"alt" -> iForm},
      Inline -> True,
      CropImage -> False,
      "RetinaImage" -> True
    ], XMLElement["span", {"class"->"LegacyError"}, {"returns unevaluated"}]}]
],

(* TODO USe Brendan's code to add sensible Alt/Title tags to SetterBox. *)
SetterBox :>
Span[{"class" -> ""}, 
	{(
    transmogrifyImage[
      ToFileName[{"Files", GetBasenameLa[]}, "I_" <> IncrementCounter["Image"] <> ".gif"],
      {"alt" -> "Setter"},
      Inline -> True,
      CropImage -> False,
      "RetinaImage" -> True
    ]
    )}],


CounterBox :> Span[
{"class"->"RefNumber"}, {IncrementCounter["CounterBox"]}   ],


(* TODO USe popup info as alt/title tags? *)
(* TODO test for links & add a/link wrapper if positive test. *)
{Cell, "FlushInput" | "Input" | "InputOnly" | "InputDeemphasis" | "WorkflowAnnotatedInputPrompts" | "WorkflowInput"} :> Module[
  {
    iInnerCounter = IncrementCounter["iInner"], iId, iInner, iOuter,
    inputForm, inputFormQueue, celtag, url,
    isManipulate, id, urlOrig, urlEncoded, inputVals, base, inBase,
    res, imageFile, image, inPrompt, inNumber,
    (* when this input has dual images to be swapped out for narrow or wide
    browser windows, imageSwapQ is set to True *)
    imageSwapQ = False
  },

dprint["----------------------------------------"];
dprint["Defaults.m: entering Input xform"];

  iId = "i_" <> iInnerCounter;
  iInner = StringJoin[iId, "_in"];
  iOuter = StringJoin[iId, "_out"];

  (* Note that preservedInputForm is declared at the top of this file in the
  Notebook transform so that it's available everywhere throughout the
  transmogrification of the notebook.  It's updated here every time we encounter
  an Input cell.  After each update, it contains the code of the most recent
  Input cell.

  preservedInputForm is used in the Output transform to determine whether the
  current example's output should be exported as a video.  More details there. *)

  inputForm = CreateInputForm[ SelectSelf[] ];
  preservedInputForm = inputForm;
  inputForm = If[Head@inputForm === String, inputForm, ToString@inputForm];
  inputFormQueue = inputQueue[inputForm, GetOption[CellLabel]];

  celtag = If[HasOption[CellTags], GetOption[CellTags], False];
  url = GetParameter["url"];

(* dprint["inputForm = "<>ToString[inputForm, InputForm]]; *)
(* dprint["preservedInputForm = "<>ToString[preservedInputForm, InputForm]]; *)
(* dprint["inputFormQueue = "<>ToString[inputFormQueue, InputForm]]; *)

  (* dprint["Defaults.m:Input, inputForm = "<>ToString[inputForm, InputForm]]; *)

  isManipulate = If[
    IsManipulate === True
  , (DocumentationBuild`Utils`IsManipulateQ[ SelectLiteral[SelectSelf[] ]] (*&& GetParameter["entityType"] === "Symbol"*)  )
  , False
  ];

  id = If[HasOption[CellID], "#"<>ToString@GetOption[CellID], ""];
  urlOrig = "https://reference.wolfram.com/language/" <> GetParameter["url"] <> ".html" <> id;
  urlOrig = If[!StringFreeQ[urlOrig, "FIXME"], False, urlOrig];
  urlEncoded = If[StringQ[urlOrig], Pubs`ShortenURL@urlOrig, ""];

  (* For long ages now, the Input transform has incremented iInner at the
  beginning of the transform, and then incremented it again when assembling the
  output filename.  Which produces an annoyance: I_2.png to go with O_1.png and
  i_1.txt, etc.  I'm going to stop that now.  In the short term this will leave
  extraneous unused input images in all the Files directories, but there's
  probably a clever way to clear those out using find -mtime. *)

  imageFile = ToFileName[{"Files", GetBasenameLa[]}, "I_" <> iInnerCounter <> ".png"];
  dprint["Defaults.m:Input, imageFile = "<>ToString[imageFile, InputForm]];

  res = Sequence@@{  
    XMLElement["table", {"class" -> If[celtag==="ErrorCellBracket","InCellBracket","InCell"]},  {  
      XMLElement["tr", {}, {

        (* cell label <td> *)
        XMLElement["td", {"class" -> "lab" (* , ""-""<> *)}, 
          Sequence @@ {
            If[
              HasOption[CellTags]
            ,
              Flatten[{
                If[
                  HasOption[CellLabel]
                ,
                  (* fill it with the CellLabel text *)
                  { GetOption[CellLabel] }
                ,
                  (* otherwise it's empty *)
                  {}
                ] (* end If *)
              }]
            ,  
              If[
                HasOption[CellLabel]
              ,
                (* fill it with the CellLabel text *)
                { GetOption[CellLabel] }
              ,
                (* otherwise it's empty *)
                {}
              ]
            ] 
          }
        ],
    
        (* input <td> *)
        XMLElement["td", {"class" -> "Input"}, {

          (* hidden input layer *)
          DIV[{}, {
            (* Check for Manipulate *) 
            If[
              isManipulate
            ,
              transmogrifyImage[
                imageFile,
                { "alt" -> "Click for copyable input" },
                Inline -> True,
                CropImage -> False,
                ImageFormat -> "PNG",
                "NarrowImage" -> If[Or[GetParameter["pacletName"] == "webMathematica", StringContainsQ[GetParameter["pacletName"], "SystemModeler"|"System Modeler"]], False, True],
                "RetinaImage" -> If[GetParameter["workflowQ"], False, True],
                "Magnification" -> If[GetParameter["workflowQ"], 1, 1.1],
                "Debug" -> False
              ]
            ,
              transmogrifyImage[
                imageFile,
                {"alt" -> "Click for copyable input" },
                Inline -> True,
                CropImage -> False,
                ImageFormat -> "PNG",
                "NarrowImage" -> If[Or[GetParameter["pacletName"] == "webMathematica", StringContainsQ[GetParameter["pacletName"], "SystemModeler"|"System Modeler"]], False, True],
                "RetinaImage" -> If[GetParameter["workflowQ"], False, True],
                "Magnification" -> If[GetParameter["workflowQ"], 1, 1.1],
                "Debug" -> False
              ]
            ]
          }], (* end <div> *) 

          Module[{str},
            str = ExportString[inputFormQueue, "String"];
			Quiet[
				CreateDirectory[ToFileName[{"Files", GetBasenameLa[]}]],
				(* Note: `filex` is emitted in WL 12.2-, `eexist` in 12.3+. *)
				{CreateDirectory::filex, CreateDirectory::eexist}
			];
            Export[ToFileName[{"Files", GetBasenameLa[]}, iId<>".txt"], str, "Text"];    
          ]
        }], (* end <td> *) 

        If[
          celtag==="ErrorCellBracket"
        ,
          XMLElement["td", {"class" -> "RedCellBracket"}, {}]
        ,
          Unevaluated@Sequence[]
        ]
      }] (* end <tr> *) 
    }] (* end <table> *)
  };

  dprint["Defaults.m: initial res = "<>ToString[res, InputForm]];

image = If[
  Positive @ Count[res, "data-src", Infinity]
, (* true clause: a 405 image-swap input *)
  dprint["Defaults.m Input xform: processing a 405 input."];
  imageSwapQ = True;
  Check[
    First[
      Cases[
        res,
        (* wxh = "width x height" *)
        XMLElement["img", {"src" -> "", "data-src" -> imageFile:_String, "data-big" -> bigwxh:_String, "data-small" -> smallwxh:_String, ___}, ___] :>
          Association["imageFile" -> imageFile, "data-big" -> bigwxh, "data-small" -> smallwxh],
        Infinity
      ]
    ],
    Association[]
  ]
, (* false clause: a standard input *)
  dprint["Defaults.m Input xform: processing a standard non-405 input."];
  Check[
    First[
      Cases[
        res,
        XMLElement["img", {"src" -> imageFile:_String, "height" -> height:_String, "width" -> width:_String, "alt" -> alt:_String}, ___] :>
          Association["imageFile" -> imageFile, "height" -> height, "width" -> width],
        Infinity
      ]
    ],
    Association[]
  ]
];

dprint["Defaults.m Input xform: image = "<>ToString[image, InputForm]];

inPrompt = Quiet @ Check[First[Cases[res, XMLElement["td", {"class" -> "lab"}, {s:_String}] -> s, Infinity]], ""];
inNumber = Quiet @ Check[First[StringCases[inPrompt, NumberString..]], ""];

res = If[
  imageSwapQ
, XMLElement["div", {
    If[HasOption[CellID], "id" -> ToString@GetOption[CellID], Nothing]
  , "class" -> "clipboard-input"
  , "data-in" -> inPrompt
  , "data-in-num" -> inNumber
  , "data-in-src" -> "data:image/gif;base64,R0lGODlhAQABAIAAAP///wAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw=="
  , "data-src-swap" -> image["imageFile"]
  , "data-big" -> image["data-big"]
  , "data-small" -> image["data-small"]
  , If[TrueQ @ GetParameter["workflowQ"], Nothing, "data-link" -> urlEncoded]
  , "data-code" -> FileNameJoin[{"Files", GetBasenameLa[], iId<>".txt"}]
  , "CLOSE-THIS-DIV" -> "True"
  }, {}]
, XMLElement["div", {
    If[HasOption[CellID], "id" -> ToString@GetOption[CellID], Nothing]
  , "class" -> "clipboard-input"
  , "data-in" -> inPrompt
  , "data-in-num" -> inNumber
  , "data-in-src" -> "data:image/gif;base64,R0lGODlhAQABAIAAAP///wAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw=="
  , "data-src" -> image["imageFile"]
  , "data-in-width" -> image["width"]
  , "data-in-height" -> image["height"]
  , If[TrueQ @ GetParameter["workflowQ"], Nothing, "data-link" -> urlEncoded]
  , "data-code" -> FileNameJoin[{"Files", GetBasenameLa[], iId<>".txt"}]
  , "CLOSE-THIS-DIV" -> "True"
  }, {}]
];

  dprint["Defaults.m: final processed res = "<>ToString[res, InputForm]];
  res
],
    
(** 
  Output Cells
  right now turn all output cells into images.
**)

(* TODO USe Brendan's code alt/title tags? *)
(* TODO test for links & add a/link wrapper if positive test. *)

(* 20170222: the varieties of Output cell will be output at 100% magnification
(i.e., no magnification).  Later we can work on producing some sort of
non-bitmapped outputs for these *)

{Cell, "Output" | "OutputOnly" | "Echo"} :> Module[
  {
    outCell, soundPath, soundID, soundExpr, ret, outputVals, outBase, multipleSWFQ,
    videoID, previewFormat, previewExtension, previewFile,
    videoFormat, videoExtension, videoEncoding, videoFile,
    manipulateExpr, opts, time,
    previewWidth, previewHeight, videoWidth, videoHeight, count, res,
    exportResult, image, outPrompt, outNumber, audioID, audioPath,
    isEcho = False, isManipulate = False, isSound = False,
    isSoundNote = False, isAudio = False
  },

dprint["======================================================================"];
dprint["Defaults.m: entering Output transform"];
(* dprint["testing values from the Input xform:"]; *)
(* dprint[" - inputForm = "<>ToString[inputForm, InputForm]]; *)
(* dprint[" - preservedInputForm = "<>ToString[preservedInputForm, InputForm]]; *)
(* dprint[" - inputFormQueue = "<>ToString[inputFormQueue, InputForm]]; *)

  outCell = SelectLiteral[SelectSelf[] ];
  (* dprint["outCell (SelectLiteral[SelectSelf[] ]) = "<>ToString[outCell, InputForm]]; *)

dprint["uppercase 'is': IsManipulate (an option passed to ExportWebPage) = "<>ToString[IsManipulate, InputForm]];
dprint["DocumentationBuild`Utils`IsManipulateQ[ outCell] = "<>ToString[DocumentationBuild`Utils`IsManipulateQ[ outCell], InputForm]];

  (* IsManipulate is a global option passed to ExportWebPage: shall we export
  Manipulates as videos, or not bother with all that video rigamarole in this
  build?  When it's True, we export videos.  It seems it was poorly named in the
  First Age; nowadays I'd go with something like exportManipulateVideosQ -
  something to describe the obvious purpose of the option.

  isManipulate is a local flag initialized to False when the Output transform
  runs.  When TrueQ[IsManipulate] (the global option), we check each example for
  signs that its output should be exported as a video.  If it should, then we
  set TrueQ[isManipulate] (the local flag).  This will select the video output
  branch of this transform's central Which statement. *)

  isManipulate =

    (* is global video export activated? *)
    TrueQ[IsManipulate]

    (* does this example's Output contain Manipulate items? *)
    && DocumentationBuild`Utils`IsManipulateQ[ outCell]

    (* is this example's Input expected to produce a single Manipulate object
    suitable for export and playback as a video?  We don't want to make a video
    of Table[Manipulate[_, __], __].  Note that this is our use of the "global"
    variable preservedInputForm as mentioned in the Input transform.  *)
    && StringContainsQ[preservedInputForm, StartOfString ~~ {"Manipulate", "ControllerManipulate"} ~~ WordBoundary];

dprint["lowercase 'is': isManipulate = "<>ToString[isManipulate, InputForm]];

dprint["----------------------------------------"];
  isSound = DocumentationBuild`Utils`IsSoundQ[ outCell];
  isSoundNote = !FreeQ[ outCell, _SoundNote];
  isEcho = Positive @ Count[{outCell}, Cell[_, "Echo", ___]];
  isAudio = DocumentationBuild`Utils`IsAudioQ[ outCell];
  (* DocumentationBuild/XMLTransforms/HTML/MarketingPages.m *)
  (* If[isAudio && Length[Cases[First@SelectChildren[], _Audio, Infinity]]>1, isAudio=False]; *)

  (* Use ImageList[...] to obtain Baseline value *)
  (* TODO *)
  (*
  outputVals=First@Cases[ImageList[
        BoxToImage[ToFileName[{"Files", GetBasenameLa[]},
      "Box_" <> IncrementCounter["Inline"] <> ".gif"],
      Inline -> True, CropImage -> False]
  ], _];
  outBase=StringDrop[Cases[outputVals, "Baseline"~(Rule|RuleDelayed)~x_ -> x], -1];
  *)

res = If[
  HasOption[CellLabel] && StringMatchQ[GetOption[CellLabel], "*Form*" |"*Short*"],

    (* true clause *)
    If[StringMatchQ[GetOption[CellLabel], "*TableForm*"],
      optMaxImageSize = Infinity];

(* *** handle //*Form output *)
    XMLElement["table", {"class"->"OCell"}, {
      XMLElement["tr", {}, {
        XMLElement["td", {"colspan" -> "2"}, {
        (* this extra span just snags the inline styles from the celllabel block styles. width is overridden in the colspan="2" attribute *)
          If[
            HasOption[CellLabel]
          ,
            XMLElement[
              "div",
              {"class" -> "celllabelTableForm"},
              (* https://jira.wolfram.com/jira/browse/WLDOCS-1056 *)
              If[StringContainsQ[GetOption[CellLabel], "//MatrixForm="] ,
                {StringReplace[GetOption[CellLabel], "//MatrixForm=" -> ""], XMLElement["span", {}, {"//MatrixForm="}]} ,
                {GetOption[CellLabel]}
              ]
            ]
          ,
            XMLElement["div", {"class" -> "celllabelTableForm"}, {}]
          ]
        }]
      }], (* end tr  *)

      XMLElement["tr", {}, {
        (* empty td for fixed width *)
        XMLElement["td", {"class" -> "lab"}, {" "}],
          XMLElement["td", {"class" -> If[isEcho, "Echo", "Output"]}, {
            (
              If[MemberQ[{"XXXXXXX"}, StringDrop[GetBasenameLa[], -3]],
                (* true (this clause is an old kludge) *)
                Module[{imageName, imageExport, imageSizes, xportInfo},
                  imageName = ToFileName[{"Files", GetBasenameLa[]}, "O_" <> IncrementCounter["iOutput"] <> ".png"];
                  xportInfo = System`ConvertersDump`ToRasterDataPacket[
                    Evaluate[DeleteCases[Transmogrify`Private`mangleCell[SelectSelf[], True], Rule[GridBoxOptions, __], Infinity]],
                    "PNG"
                  ];
                  imageExport = Export[
                    imageName,
                    System`ConvertersDump`FromRasterDataPacket[ReplacePart[xportInfo, Take[xportInfo[[3]], All, Min[550, Dimensions[xportInfo[[3]]][[2]]]], 3]],
                    "PNG"
                  ];
                  imageSizes = Import[ToFileName[{Directory[]}, imageName], "ImageSize"];

                  XMLElement["img", {
                    "src" -> StringReplace[imageName, {"\\"->"/"}],
                    "width" -> ToString@First@imageSizes,
                    "height" -> ToString@Last@imageSizes,
                    "alt" -> ""
                  }, {}]

                ]
              ,
                (* false *)
                If[StringMatchQ[GetOption[CellLabel], "*TableForm*"],
                  (* true: we're working with a TableForm cell, wrap with div to allow for wide images *)
                  XMLElement["div", {"class"->"TableForm"}, {
                    (* Print["ti 1"]; *)
                    transmogrifyImage[
                      ToFileName[{"Files", GetBasenameLa[]}, "O_" <> IncrementCounter["iOutput"] <> ".png"],
                      {"alt" -> ""},
                      Inline -> True,
                      MaxImageSize->optMaxImageSize,
                      CropImage -> False,
                      ImageFormat -> "PNG",
                      "RetinaImage" -> True,
                      "Debug" -> False,
                      "OutputImage" -> True
                    ]
                  }],
                  (* false *)
                  (* Print["ti 2"]; *)
                  transmogrifyImage[
                      ToFileName[{"Files", GetBasenameLa[]}, "O_" <> IncrementCounter["iOutput"] <> ".png"],
                      {"alt" -> ""},
                      Inline -> True,
                      MaxImageSize->optMaxImageSize,
                      CropImage -> False,
                      ImageFormat -> "PNG",
                      "RetinaImage" -> True,
                      "Debug" -> False,
                      "OutputImage" -> True
                  ] (* close false clause *)
                ] (* close If TableForm *)
              ] (* close If {"XXXXXXX"} *)
            ) (* close Output td body *)
            }] (*  close Output td *)
          }] (*  close tr *)
      }] (*  close OCell table *)
  ,

(* *** otherwise handle normal output *)
  XMLElement["table",
    {"class"->If[isManipulate, "OCell animation", "OCell"]}, {
      XMLElement["tr",
        {}, {
          (* cell label <td> *)
          XMLElement["td",
            {"class" -> "lab"},
            If[HasOption[CellLabel],
              (* fill it with the CellLabel text *)
              {GetOption[CellLabel]}
            ,
              (* otherwise it's empty *)
              {}
            ] (* end If *)
          ], (* end cell label <td> *)

          XMLElement["td",
            {"class" -> If[isEcho, "Echo", "Output"]}, {
Which[

(* *** isManipulate *)

isManipulate, (
  dprint["Defaults.m Output xform, entering isManipulate"];


(* **** preliminaries: set up variables *)

  videoID = "Video_" <> IncrementCounter["video"];

  previewFormat = "JPEG";
  previewExtension = "jpg";
  videoFormat = "MP4";
  videoExtension = "mp4";

  previewFile = StringReplace[FileNameJoin[{"Files", GetBasenameLa[], videoID <> "." <> previewExtension}],  {"\\" -> "/"}];
  videoFile = StringReplace[FileNameJoin[{"Files", GetBasenameLa[], videoID <> "." <> videoExtension}],  {"\\" -> "/"}];
  Global`AntLog["-------------------------------------------------------"];
  Global`AntLog["videoID = "<>ToString[videoID, InputForm]];
  Global`AntLog[""];

(* **** build the expression to be exported *)

  manipulateExpr = ReleaseHold@MakeExpression[First@SelectLiteral[SelectChildren[]], StandardForm];

  opts = {
    "ShowControls" -> True
  , "ControlAppearance" -> Automatic

    (* the following is True by default, but we may as well mention it here to
    document the choice.  see tutorial/ImportingAndExportingVideo#1156435029,
    especially its note about the minimum required version of ffmpeg *)
  , VideoProcessingOptions -> {TryUsingSystemFFmpeg -> True}

    (* Streamable per shadi to encourage playback in firefox; see
    https://jira.wolfram.com/jira/browse/CLOUD-18202 *)
  , "Streamable" -> True
  };

(* **** export preview image and get its dimensions *)

  Catch[
    (* on the use of Rasterize here, see https://mail-archive.wolfram.com/archive/l-importexport/2018/Jan00/0037.html *)
    time = First @ AbsoluteTiming[exportResult = Export[previewFile, Rasterize @ manipulateExpr, previewFormat, Sequence @@ opts];];
    Global`AntLog["previewFile Export time: "<>TextString[time]<>" seconds"]; Clear[time];

    (* test for existence *)
    If[exportResult == $Failed || Not@FileExistsQ[previewFile],
      Global`AntLog["previewFile Export failed; setting {previewWidth, previewHeight} to {0, 0}"];
      Clear[exportResult]; Throw[{previewWidth, previewHeight} = {0, 0}]];

    (* at this point, Export succeeded and previewFile exists *)
    time = First @ AbsoluteTiming[{previewWidth, previewHeight} = ImageDimensions[Import[previewFile]];];
    Global`AntLog["previewFile ImageDimensions time: "<>TextString[time]<>" seconds"]; time =.;

    If[Not[NumberQ[previewWidth] && NumberQ[previewHeight]],
      Global`AntLog["previewFile exists but ImageDimensions failed; {previewWidth, previewHeight} = "<>ToString[{previewWidth, previewHeight}, InputForm]];
      Global`AntLog["Setting {previewWidth, previewHeight} to {100, 100}"];
      Throw[{previewWidth, previewHeight} = {100, 100}];
    ,
      Global`AntLog["{previewWidth, previewHeight} = "<>ToString[{previewWidth, previewHeight}, InputForm]];
    ]
  ];

  Global`AntLog["final {previewWidth, previewHeight} = "<>ToString[{previewWidth, previewHeight}, InputForm]];
  Global`AntLog[""];

(* **** export a video file and get its dimensions *)

  Catch[

    (* AntLog["manipulateExpr = "<>ToString[manipulateExpr, InputForm]]; *)

    (* these three encodings produce working video manipulates in OSX's firefox, chrome, and safari: *)

    videoEncoding = "H264-AVF"; (* test 1 for jenkins builds.  result - firefox: xxx, chrome: xxx, safari: xxx *)
    (* videoEncoding = "H264-VIDEOTOOLBOX"; *)
    (* videoEncoding = "LIBX264"; *)

    time = First @ AbsoluteTiming[exportResult = Export[videoFile, manipulateExpr, videoFormat, VideoEncoding -> videoEncoding, Sequence @@ opts];];
    Global`AntLog["videoFile Export time: "<>TextString[time]<>" seconds"]; Clear[time];

    (* test for existence *)
    If[exportResult == $Failed || Not@FileExistsQ[videoFile],
      Global`AntLog["videoFile Export failed for "<>videoFile];
      Clear[exportResult]; Throw[{videoWidth, videoHeight} = {100, 100}]];

    (* at this point, video Export succeeded *)
    {videoWidth, videoHeight} = ImageDimensions[First @ Import[videoFile, "ImageList"]];

  ];

  Global`AntLog["final {videoWidth, videoHeight} = "<>ToString[{videoWidth, videoHeight}, InputForm]];

(* **** return XML code with video & image information *)

  isManipulateRes = XMLElement["div", {
    "class" -> "clipboard-output"
  , "data-out" -> GetOption[CellLabel]
  , "data-out-num" -> Check[First[StringCases[GetOption[CellLabel], NumberString..]], ""]
  , "data-out-src" -> previewFile
  , "data-out-width" -> ToString[previewWidth]
  , "data-out-height" -> ToString[previewHeight]
  , "data-video" -> StringJoin[ToString[videoWidth], " ", ToString[videoHeight]]
  , "data-media" -> videoFile
  , "CLOSE-THIS-DIV" -> "True"
  }, {}];
  dprint["Defaults.m:Output xform, isManipulateRes = "<>ToString[isManipulateRes, InputForm]];


),  (* end isManipuate branch of the Output td *)

(* *** isSoundNote *)

isSoundNote, (
  dprint["Defaults.m Output xform, entering isSoundNote"];
  (*Print["<" <> GetBasenameLa[] <> ": SoundNote Output cell>"]; *)
  soundID = "SoundNote_" <> IncrementCounter["soundnote"];
  soundPath = StringReplace[ToFileName[{"Files", GetBasenameLa[]}, soundID <> ".mid"],  {"\\" -> "/"}];
  If[
    Length[Cases[First@outCell, _Sound, Infinity]] === 1
  , soundExpr = First@Cases[outCell, _Sound, Infinity];
    ret = Export[soundPath, soundExpr];
    A[
      {"href"->soundPath}
    , {
        XMLElement[
          "span",
          {"id"->soundID},
          {
            transmogrifyImage[
              ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".png"],
              {"alt"->""},
              Inline -> True,
              CropImage -> False,
              ImageFormat -> "PNG",
              "RetinaImage" -> True,
              "Debug" -> False,
              "OutputImage" -> True
            ]
          }
        ]
      }
    ]
  , XMLElement[
      "span",
      {"id"->soundID},
      {
        transmogrifyImage[
          ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".png"],
          {"alt"->""},
          Inline -> True,
          CropImage -> False,
          ImageFormat -> "PNG",
          "RetinaImage" -> True,
          "Debug" -> False,
          "OutputImage" -> True
        ]
      }
    ]
  ]
), (* end isSoundNote branch of the Output td *)

(* *** isSound *)

isSound, (
  dprint["Defaults.m Output xform, entering isSound"];
  (*Print["<" <> GetBasenameLa[] <> ": Sound Output cell>"]; *)
  soundID = "Sound_" <> IncrementCounter["sound"];
  soundPath = StringReplace[ToFileName[{"Files", GetBasenameLa[]}, soundID <> ".wav"],  {"\\" -> "/"}];

  (* reminder: when GetAbsoluteFileName[] returns
  "/Users/billw/wri/docs/www/language/workflow/LoadAPackage.html.en",
  GetBaseNameLa[] returns "LoadAPackage.en". *)
  Which[
    StringDrop[GetBasenameLa[], -3] === "Music" || StringContainsQ[GetAbsoluteFileName[], "/workflow"],
    XMLElement[
      "span",
      {"id"->soundID},
      {
        transmogrifyImage[
          ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".png"],
          {"alt"->""},
          Inline -> True,
          CropImage -> False,
          ImageFormat -> "PNG",
          "RetinaImage" -> True,
          "Debug" -> False,
          "OutputImage" -> True
        ]
      }
    ]
  , Length[Cases[First@outCell, _Sound, Infinity]] === 1,
    soundExpr=Sound@@Cases[First@outCell, _Sound, Infinity];
    Quiet[ret = Export[soundPath, soundExpr]];
    A[
      {"href"->soundPath},
      {
        XMLElement[
          "span",
          {"id"->soundID},
          {
            transmogrifyImage[
              ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".png"],
              {"alt"->""},
              Inline -> True,
              CropImage -> False,
              ImageFormat -> "PNG",
              "RetinaImage" -> True,
              "Debug" -> False,
              "OutputImage" -> True
            ]
          }
        ]
      }
    ]
  ,
    (* Length@Cases[outCell, CompressedData[a__], Infinity]>0, Last@Cases[outCell, _Sound, Infinity],*)
    True,
    XMLElement[
      "span",
      {"id"->soundID},
      {
        transmogrifyImage[
          ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".png"],
          {"alt"->""},
          Inline -> True,
          CropImage -> False,
          ImageFormat -> "PNG",
          "RetinaImage" -> True,
          "Debug" -> False,
          "OutputImage" -> True
        ]
      }
    ]
  ] (* end isSound Which *)
), (* end isSound branch of the Output td *)

(* *** isAudio *)

isAudio, (
  dprint["Defaults.m Output xform, entering isAudio"];

  audioID = "Audio_" <> IncrementCounter["audio"];
  audioPath = StringReplace[ToFileName[{"Files", GetBasenameLa[]}, audioID <> ".mp3"],  {"\\" -> "/"}];
  Global`AntLog["About to create: "<>audioPath];
  audioExpr = Audio@@Cases[First@SelectChildren[], _Audio, Infinity];
  audioReturn = Export[audioPath, audioExpr];
  audioReturnString = ToString[audioReturn];

  (* automatically provide a horizontal scrollbar for images wider than the css output-image width *)
  optMaxImageSize = Infinity;
  XMLElement["div",
    {"class" -> "output-image"}, {
      (* Print["ti 9"]; *)
      transmogrifyImage[
        ToFileName[{"Files", GetBasenameLa[]}, "O_" <> IncrementCounter["iOutput"] <> ".png"],
        {"alt"->""},
        Inline -> True,
        MaxImageSize->optMaxImageSize,
        CropImage -> False,
        ImageFormat -> "PNG",
        "IsAudio" -> True,
        (* the Audio gui implementation can't handle magnification and shrinking *)
        "RetinaImage" -> False,
        "Debug" -> False,
        "OutputImage" -> True
      ]
    }
  ]
),

True, (
  dprint["Defaults.m Output xform, entering default branch"];
  (* automatically provide a horizontal scrollbar for images wider than the css output-image width *)
  optMaxImageSize = Infinity;
  XMLElement["div",
    {"class" -> "output-image"}, {
      transmogrifyImage[
        ToFileName[{"Files", GetBasenameLa[]}, "O_" <> IncrementCounter["iOutput"] <> ".png"],
        {"alt"->""},
        Inline -> True,
        MaxImageSize->optMaxImageSize,
        CropImage -> False,
        ImageFormat -> "PNG",
        "RetinaImage" -> True,
        "Debug" -> False,
        "OutputImage" -> True
      ]
    }
  ]
)
]

    }] (* end Output <td> *)

    }] (*  close tr  *)
  }] (*  close table  *)
];


dprint["Defaults.m, Output transform: initial res = "<>ToString[res, InputForm]];


res = If[
  isManipulate
, isManipulateRes
, image = Check[
    First[
      Cases[
        res,
        XMLElement["img", {"src" -> imageFile:_String, "height" -> height:_String, "width" -> width:_String, "alt" -> alt:_String}, ___] :>
          Association["imageFile" -> imageFile, "height" -> height, "width" -> width],
        Infinity
      ]
    ]
  , Association[]
  ];
  dprint["Defaults.m, Output transform: image association = "<>ToString[image, InputForm]];
  outPrompt = Quiet @ Check[First[Cases[res, XMLElement["td", {"class" -> "lab"}, {s:_String}] -> s, Infinity]], ""];
  outNumber = Quiet @ Check[First[StringCases[outPrompt, NumberString..]], ""];
  XMLElement[
    "div"
  ,
    {
      "class" -> "clipboard-output"
    , "data-out" -> outPrompt
    , "data-out-num" -> outNumber
    , "data-out-src" -> "data:image/gif;base64,R0lGODlhAQABAIAAAP///wAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw=="
    , "data-src" -> image["imageFile"]
    , "data-out-width" -> image["width"]
    , "data-out-height" -> image["height"]
    ,  If[isAudio && audioReturnString =!= "$Failed", "data-media" -> audioPath, Nothing]
    , "CLOSE-THIS-DIV" -> "True"
    }
  ,
    {}
  ]
];

dprint["Defaults.m, Output transform: final res = "<>ToString[res, InputForm]];
dprint["Defaults.m: ----------------------------------------------------------------------"];

res

 ], (* end "Output" rule *)

{Cell, "DataPageEntityNote"} :>
  XMLElement["div", {"class" -> "DataPageEntityNote"}, {
    XMLElement["div", {"class" -> "inner"}, {Recurse[]}]
  }],

{Cell, "PlatformAvailabilityNote"} :>
  XMLElement["div", {"class" -> "PlatformAvailabilityNote"}, {
    XMLElement["div", {"class" -> "inner"}, {Recurse[]}]
  }],

{Cell, "InlineFormulaTableText"} :> XMLElement["span", {"class"->"IF"}, {Recurse[]}],

(* https://jira.wolfram.com/jira/browse/WLDOCS-790 *)
{Cell, "InlineFormula"} :> Module[
  {res, recurseResults}
,
  (* dprint["InlineFormula entrance: children = "<>ToString[SelectChildren[], InputForm]]; *)
  recurseResults = DocumentationBuild`Utils`AddSpans[Recurse[]];
  If[
    recurseResults == Null
  ,
    dprint["[error] InlineFormula error: returned Null while building "<>GetAbsoluteFileName[]<>", SelectChildren[] = "<>ToString[{SelectChildren[]}, InputForm]];
    Global`AntLog["[error] InlineFormula error: returned Null while building "<>GetAbsoluteFileName[]<>", SelectChildren[] = "<>ToString[{SelectChildren[]}, InputForm]];
    recurseResults = "Null"
  ];
  (* dprint["InlineFormula: recurseResults = "<>ToString[{recurseResults}, InputForm]]; *)
  res = XMLElement[
    "span",
    {"class" -> If[GetStyle[] === "InlineMath", "IM", "IF"]}, 
    {recurseResults}
  ];
  (* dprint["InlineFormula exit: res = "<>ToString[res, InputForm]]; *)
  res
],

{Cell, "InlineCode" | "InlineOutput" | "InlineMath" | "FunctionTemplate"|"PlatformDynamic"} :> Module[
  {thestyle, res}
,
  (* Print["InlineMath et al. entrance: children = "<>ToString[SelectChildren[], InputForm]]; *)
  thestyle = GetStyle[];
  Span[
    {"class"-> If[thestyle==="InlineMath", "IM", "IF"]}, 
    res = If[Cases[SelectSelf[], _ButtonBox|_TemplateBox, Infinity]==={}, 
    	{
        (* Print["InlineMath et al.: transmogrifyImage"]; *)
    		transmogrifyImage[
          ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".png"],
          {"alt"->""},
          Inline -> True,
          CropImage -> False,
          ImageFormat -> "PNG",
          TransparentBackground->True,
          ConversionOptions->{Background->None},
          "RetinaImage" -> True
        ]
      }, 
      (* Print["InlineMath et al.: Recurse"]; *)
      {DocumentationBuild`Utils`AddSpans[Recurse[] ]}
    ];
    (* Print["InlineMath et al. exit: res = "<>ToString[res, InputForm]]; *)
    res
  ]
], 

{Cell, "DisplayFormula" | "DisplayMath"} :> 
DIV[{"class"->"DisplayFormula"}, 
    {
      transmogrifyImage[
        ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
        {"alt"->""},
        Inline -> False,
        CropImage -> False,
        "RetinaImage" -> True
      ]
    }
  ], 

{Cell, "AnnotatedExpression"|"AnnotatedInput"|"AnnotatedOutput"} :> 
DIV[{"class"->"AnnotatedExpression"}, 
    {
      transmogrifyImage[
        ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".png"],
        {"alt"->""},
        Inline -> True,
        CropImage -> False,
        ImageFormat -> "PNG",
        TransparentBackground->True,
        ConversionOptions->{Background->None},
        "RetinaImage" -> True
      ]
    }
  ], 


(* Sans version of InlineFormula *)
(* InlineFunctionSans         -> <span class="IFSans"> *)
(* InlineFunctionSansUnlinked -> <span class="IFSansUnlinked"> *)
{Cell, "InlineFunctionSans"} :> 
  If[Positive@Count[SelectSelf[], TemplateBox[_, "EntityTypeLink", ___], Infinity]
  , XMLElement["span", {"class"->"IFSans sub-ref"}, {Recurse[]}]
  , XMLElement["span", {"class"->"IFSans"}, { Recurse[] }]
  ],
{Cell, "InlineFunctionSansUnlinked"} :> XMLElement["span", {"class"->"IFSansUnlinked"}, { Recurse[] }],

{Cell,
  "SeeAlsoBlockchain"
| "SeeAlsoC"
| "SeeAlsoCallback"
| "SeeAlsoClassifier"
| "SeeAlsoCharacter"
| "SeeAlsoCompiledtype"
| "SeeAlsoDatastructure"
| "SeeAlsoDevice"
| "SeeAlsoEmbeddingformat"
| "SeeAlsoEntity"
| "SeeAlsoExcelfunction"
| "SeeAlsoExternalevaluationsystem"
| "SeeAlsoFile"
| "SeeAlsoFormat"
| "SeeAlsoFrontendobject"
| "SeeAlsoIndicator"
| "SeeAlsoInterpreter"
| "SeeAlsoMenuitem"
| "SeeAlsoMethod"
| "SeeAlsoNetdecoder"
| "SeeAlsoNetencoder"
| "SeeAlsoPredictor"
| "SeeAlsoProgram"
| "SeeAlsoService"
| "SeeAlsoTextcontent"
} :> XMLElement["span", {"class" -> "sub-ref-heading"}, {Recurse[]}],

{BoxData}:> Recurse[],

  
{SuperscriptBox} :> Module[{},
  If[
    !htmlBoxesQ[SelectSelf[]]
  , transmogrifyImage[
      ToFileName[{"Files", GetBasenameLa[]}, "Box_"<>IncrementCounter["Image"] <> ".png"],
      {"style"->"vertical-align:middle;", "alt"->"", "title"->""},
      Inline -> True,
      CropImage -> False,
      ImageFormat -> "PNG",
      TransparentBackground->True,
      ConversionOptions->{Background->None},
      "RetinaImage" -> True
    ]
  , Sequence@@{
      Transmogrify[SelectChildren[][[1]] ],
      XMLElement["sup", {} , {Transmogrify[SelectChildren[][[2]]]} ]
    }
  ]
],


{SubscriptBox} :> Module[{}, If[!htmlBoxesQ[SelectSelf[]], transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, "Box_"<>IncrementCounter["Image"] <> ".png"],
  {"style"->"vertical-align:middle;", "alt"->"", "title"->""},
  Inline -> True,
  CropImage -> False,
  ImageFormat -> "PNG",
  TransparentBackground->True,
  ConversionOptions->{Background->None},
  "RetinaImage" -> True
], Sequence@@{
  Transmogrify[SelectChildren[][[1]] ],
  XMLElement["sub", {}, {Transmogrify[SelectChildren[][[2]]]} ]
}]],



{CellGroupData, "PictureGroup"} :>
(
  If[HasOption[First[SelectChildren[Cell]], AnimationCycleRepetitions],
  {
  XMLElement["p", {"class" -> "Animation"}, {
    WithParameters[{"src" -> ToFileName[{"Files", GetBasenameLa[]},
        "A_" <> IncrementCounter["Animation"] <> ".gif"]},
      CreateDirectory[DirectoryName[GetParameter["src"]]];
      Export[GetParameter["src"],
        SelectLiteral[SelectChildren[]], "GIF", 
        ConversionOptions -> {"Loop" -> 3}];
      XMLElement["img", {
        "src" -> StringReplace[GetParameter["src"], $PathnameSeparator -> "/"]
      }, {}]
    ]
  }]

   }
   ,
   
   XMLElement["p", If[GetStyle===None, {}, {"class"->GetStyle[]}], {
     transmogrifyImage[
       ToFileName[{"Files", GetBasenameLa[]}, "A_" <> IncrementCounter["Animation"] <> ".gif"],
       {"alt" -> " "},
       Inline -> False
     ]
   }]
]
),

{Cell, "Usage"} :> DIV[{"id" -> "UT"}, Block[
  {usagegrid = True}
,
  dprint["Defaults.m: entering transform for Usage; expr = "]; dfile[expr];
  Module[
    {xml = {Recurse[]}, pos, repl, first = True}
  ,
    pos = Position[xml, XMLElement["tr", _, _], Infinity];
    repl = Map[
        Module[{tr, attr, children, modinfo},
            tr = Extract[xml, #];
            modinfo = False;
            modinfo = MatchQ[tr[[3, 1]],
                XMLElement["td", _, {
                    XMLElement["span", {___, "id"->"ModInfo_"<>"8", ___}, _] }] ];
            attr = Join[
                DeleteCases[tr[[2]], _["name"|"id", _]],
                If[modinfo, {"name"->"ModInfo_"<>"8", "id"->"ModInfo_"<>"8"}, {}] ];
            If[first,
                first = False,
                attr = Join[
                    DeleteCases[attr, _["class", _]],
                    {"class" -> ("class" /. attr /. {"class" -> ""}) <> " lineAbove"}]];
            children = Map[
                Function[{td}, Module[{attr},
                    attr = td[[2]];
                    If[MatchQ[attr, {___, "class" -> _, ___}],
                        attr = ReplaceAll[attr, _["class", _] -> ("class" -> "UM")],
                        attr = Join[attr, {"class" -> "UM"}]];
                    attr = Join[
                        DeleteCases[attr, _["name"|"id", _]],
                        If[modinfo, {"name"->"ModInfo_"<>"8", "id"->"ModInfo_"<>"8"}, {}] ];
                    XMLElement["td", attr, td[[3]]]
                ]],
                Rest[tr[[3]]] ];
            (# -> XMLElement["tr", attr, children])
        ]&,
        pos];
    ReplacePart[xml, repl] ]]
], (* end block  *)

(* **** {CellGroupData, "FormatUsage"} *)

(*
FormatUsageWrapper   : <li class="functionTable">
 - FormatUsageColumn : <ul class="col-1|2">
   - FormatUsage     : <li>
   - FormatUsage     : <li>
   - FormatUsage     : <li>
   - FormatUsage     : <li>
 - FormatUsageColumn : <ul class="col-1|2">
   - FormatUsage     : <li>
   - FormatUsage     : <li>
   - FormatUsage     : <li>
 *)

{Cell, "FormatUsageWrapper"} :> XMLElement["li", {"class" -> "functionTable"}, {Recurse[]}],
{Cell, "FormatUsageColumnOne"} :> XMLElement["ul", {"class" -> "col-1"}, {Recurse[]}],
{Cell, "FormatUsageColumnTwo"} :> XMLElement["ul", {"class" -> "col-2"}, {Recurse[]}],
{Cell, "FormatUsage"} :> XMLElement["li", {}, {Recurse[]}],

{CellGroupData, "FormatUsage"} :> If[
  Or[GetParameter["entityType"] == "Interpreter", GetParameter["entityType"] == "External Evaluation System", GetParameter["entityType"] == "Predictor"]
, 
  {Recurse[]}
, 
  {
    Transmogrify[First[SelectChildren[]]], 
    XMLElement["ul", {"class"->"functionList afterIntro"}, {Transmogrify[Rest[SelectChildren[]]]}]
  }
],

(* **** {Cell, "InterpreterUsage" | "DeviceUsage"} *)

{Cell, "InterpreterUsage" | "DeviceUsage"} :> Module[{},
  XMLElement["div", {"class"->"formatIntroWrap"}, {
    XMLElement["div", {"class"->"formatIntro"}, {Recurse[]}]
  }]
],

{CellGroupData, "InterpreterUsage"} :>  {Recurse[]},
{CellGroupData, "EntityUsage"} :>  {Recurse[]},
{CellGroupData, "DeviceUsage"} :> {
  Transmogrify[First[SelectChildren[]]],
  XMLElement["ul", {"class"->"functionList afterIntro"}, {Transmogrify[Rest[SelectChildren[]]]}]
},  

{CellGroupData, "EntityAbstract"} :> 
  XMLElement["section", {"class" -> "format-wrapper"}, {
    XMLElement["div", {"class" -> "inner"}, {Recurse[]}]
  }],

{Cell, "EntityAbstract"} :> 
  XMLElement["div", {"class"->"formatIntroWrap"}, {
    XMLElement["div", {"class"->"formatIntro"}, {Recurse[]}]
  }],

{Cell, "AnchorBarGrid" } :> Module[
  {
    MenuCounter, SeeAlsoFunctionList, MoreAboutGuideList, ExampleSectionList, TutorialList, GuideMoreAboutList, GuideTutorialList, 
		GuideFunctionList, showAnchorBarQ, TutorialFunctionList, TutorialMoreAboutList, RelatedTutorialsList, thisElementContent,
		uri = GetParameter["url"],
		pulldownmenu, pulldowncontent,
    newAnchorBarData,context, urlBaseRemote = GetParameter["urlBaseRemote"], workflowOverviewQ = GetParameter["workflowOverviewQ"],
    monographHeads, monographLinks
  },
   
dprint["----------------------------------------------------------------------"]
dprint["Defaults.m:AnchorBarGrid - entering transform"];
(* dprint["SelectSelf[] = "<>ToString[SelectSelf[], InputForm]]; *)

  breakDownMenuCell[a_Cell] := Module[
    {menuStyle, menuList, res},
    menuStyle = StringReplace[Cases[a, z:Rule[MenuStyle,_], Infinity][[1,2]], "Section"->""]; 
    menuList = DeleteCases[a, _FrameBox, Infinity][[1, 1, 1, 1]];

    dprint["Defaults.m:AnchorBarGrid - entering breakDownMenuCell with a_Cell = "<>ToString[a, InputForm]];
    dprint["Defaults.m:AnchorBarGrid:breakDownMenuCell - menuStyle = "<>ToString[menuStyle, InputForm]];
    dprint["Defaults.m:AnchorBarGrid:breakDownMenuCell - menuList = "<>ToString[menuList, InputForm]];

    res = {
      menuStyle,
      Map[
        Function[{item},
          newitem = Switch[item

          , Delimiter,
              {"Delimiter", "Delimiter"}

          , RuleDelayed[StyleBox[_String, "SeeAlsoRelated", ___], _Documentation`HelpLookup],
              (* these "subheadings" will carry a third element, "RelatedLink", so they can
              receive a special css tag; otherwise they look just like main list items: *)
              {ToExpression[item[[1]]], item[[2, 1]], "RelatedLink"}

          , RuleDelayed[_StyleBox, _NotebookFind],
              Flatten[{ToExpression[Cases[item, s:_StyleBox :> First[s], Infinity]], "Section"}]

          , _,
              {ToExpression[item[[1]]], item[[2, 1]]}
          ];
          newitem
        ]
      , menuList
      ]
    };
    (* dprint["Defaults.m:AnchorBarGrid:breakDownMenuCell - res = "<>ToString[res, InputForm]]; *)
    res
  ];
   
  If[GetParameter["entityType"] == "Monograph",
    monographHeads = Quiet @ Check[First[Cases[SelectSelf[], Cell[CellGroupData[{Cell[_, "MonographHeader", ___], ___}, ___], ___], Infinity]], {}];
    monographLinks = Quiet @ Check[First[Cases[SelectSelf[], Cell[CellGroupData[{Cell[_, "MonographHamburgerGroup", ___], ___}, ___], ___], Infinity]], {}];
    (* dprint["Defaults.m:AnchorBarGrid - monographHeads = "<>ToString[monographHeads, InputForm]]; *)
    (* dprint["Defaults.m:AnchorBarGrid - monographLinks = "<>ToString[monographLinks, InputForm]]; *)
  ];

	pulldowncontent = Cases[
    SelectSelf[],
    Cell[BoxData[TagBox[ActionMenuBox[___], ___], ___], ___],
    Infinity
  ];
  (* dprint["Defaults.m:AnchorBarGrid - initial pulldowncontent = "<>ToString[pulldowncontent, InputForm]]; *)

  pulldowncontent = DeleteCases[
    pulldowncontent,
    Cell[BoxData[TagBox[ActionMenuBox[___, Rule[MenuStyle, "TranslationMenu" | "URLMenu"], ___], ___], ___], ___],
    Infinity
  ];
  (* dprint["Defaults.m:AnchorBarGrid - final pulldowncontent = "<>ToString[pulldowncontent, InputForm]]; *)

  newAnchorBarData = Map[
    breakDownMenuCell[#]&,
    pulldowncontent
  ];

	(* TODO:
		Validate and replace this with a DocsAssert instead of a warning. My understanding of
		this logic isn't sufficient to make me confident this covers all cases, but it does
		help to serve as a type annotation making the code more readable, as the logic above
		is somewhat subtle. *)
	If[
		!MatchQ[
			newAnchorBarData,
			{
				{
					"SeeAlso" | "MoreAbout" | "Tutorials"
					| "GuideFunction" | "GuideMoreAbout" | "GuideTutorial"
					| "TutorialFunction"
					| "TutorialMoreAbout"
					| "RelatedTutorials",
					{
						{
							_?StringQ, (* label *)
							_?StringQ, (* uri *)
							RepeatedNull["RelatedLink", 1]
						} ...
					}
				} ...
			}
		],
		DocsWarning[
			"Internal warning: newAnchorBarData has unexpected format: ``",
			InputForm[newAnchorBarData]
		]
	];

(* dprint["Defaults.m:AnchorBarGrid - newAnchorBarData = "<>ToString[newAnchorBarData, InputForm]]; *)

  context = Cases[
    SelectSelf[],
    Cell[_, "PacletNameCell", ___],
    Infinity
  ];
  (* dprint["Defaults.m:AnchorBarGrid - context step 1 = "<>ToString[context, InputForm]]; *)
  (* text for the badge cell at the top left of a page *)
  context = context /. {
    "TUTORIAL" -> "TUTORIAL",
    "OTHERS" -> "AS NEEDED"
  };
  (* dprint["Defaults.m:AnchorBarGrid - context step 2 = "<>ToString[context, InputForm]]; *)

  If[workflowOverviewQ, context = context /. "WORKFLOW GUIDE" -> "WORKFLOW OVERVIEW"];
  (* dprint["Defaults.m:AnchorBarGrid - context step 3 = "<>ToString[context, InputForm]]; *)

  MenuCounter = IncrementCounter["MenuCounter"];
  SeeAlsoFunctionList = GetParameter["seeAlsoFunctions"];
  (* dprint["Defaults.m:AnchorBarGrid - SeeAlsoFunctionList = "<>ToString[SeeAlsoFunctionList, InputForm]]; *)
  TutorialFunctionList = GetParameter["tutorialFunctions"];
  MoreAboutGuideList = GetParameter["moreAboutGuides"];
  ExampleSectionList = GetParameter["exampleSections"];
  TutorialList = GetParameter["tutorials"];
  TutorialMoreAboutList = GetParameter["tutorialMoreAbout"];
  RelatedTutorialsList = GetParameter["relatedTutorials"];

  GuideMoreAboutList = GetParameter["guideMoreAbout"];
  GuideFunctionList = GetParameter["guideFunctions"];
  GuideTutorialList = GetParameter["guideTutorials"];

dprint["Defaults.m:AnchorBarGrid - MenuCounter = "<>ToString[MenuCounter, InputForm]];

	showAnchorBarQ = MatchQ[GetParameter["entityType"], (
			"AppleScript"
		| "Blockchain"
		| "Character Name"
		| "Character"
		| "Classifier"
		| "Compiled Type"
		| "Data Structure"
		| "Device Connection"
		| "Embedding Format"
		| "Entity"
		| "External Evaluation System"
		| "File"
		| "Format"
		| "Front End Object"
		| "HowTo"
		| "Indicator"
		| "Interpreter"
		| "LibraryLink C Function"
		| "MLModel"
		| "MathLink C Function"
		| "Menu Item"
		| "Message"
		| "Method"
		| "Net Decoder"
		| "Net Encoder"
		| "Note"
		| "Predictor"
		| "Program"
		| "Remote Batch Computation Provider"
		| "ResourceObject"
		| "Service Connection"
		| "Symbol"
		| "System Program"
		| "Text Content"
		| "Upgrade Information"
		| "Workflow"
		| "Tutorial"
		| "Overview"
		| "Monograph"
		| "Monograph Overview"
		| "Guide"
		| "Workflow Guide"
		| "Guide Overview"
		| "Standard Extra Packages"
		| "Example"
	)];

	(* If[!showAnchorBarQ, *)
		(* TODO: Are there any pages where we _shouldn't_ show an anchor bar? *)
		(* DocsWarning[
			"Internal warning: showAnchorBarQ is not True for `` page",
			InputForm[GetParameter["entityType"]]
		]; *)
	(* ]; *)

  thisElementContent := XMLElement[If[GetParameter["workflowQ"], "div", "nav"], {"class" -> "topContentWrap"}, Flatten@{
    Recurse[context], (* badge *)
    If[
      TrueQ[showAnchorBarQ]
    ,
      (* map a function over newAnchorBarData and generate XMLElement expressions *)
      pulldownmenu = Flatten@Map[
        Function[{data},
          Module[
            {
              dataid = First[data] /. {
                "GuideFunction"->Localized["FunctionsSectionText", GetParameter["language"]], 
                "GuideTutorials"->Localized["TutorialsSectionText", GetParameter["language"]], 
                "GuideTutorial"->Localized["TutorialsSectionText", GetParameter["language"]], 
                "GuideMoreAbout"->Localized["MoreAboutSectionText", GetParameter["language"]], 
                "FeaturedExampleMoreAbout"->Localized["MoreAboutSectionText", GetParameter["language"]], 
                "TutorialFunction"->Localized["FunctionsSectionText", GetParameter["language"]], 
                "TutorialMoreAbout"->Localized["MoreAboutSectionText", GetParameter["language"]],
                "RelatedTutorials"->Localized["TutorialsSectionText", GetParameter["language"]], 
                "Related Guides"->Localized["RelatedGuidesSectionText", GetParameter["language"]], 
                "RelatedGuides"->Localized["RelatedGuidesSectionText", GetParameter["language"]], 
                "Related Entities"->Localized["RelatedEntitySectionText", GetParameter["language"]], 
                "RelatedEntities"->Localized["RelatedEntitySectionText", GetParameter["language"]], 
                "Functions"->Localized["FunctionsSectionText", GetParameter["language"]], 
                "Tutorials"->Localized["TutorialsSectionText", GetParameter["language"]],
                "SeeAlso"->Localized["SeeAlsoSectionText", GetParameter["language"]], 
                "Workflow"->Localized["WorkflowsSectionText", GetParameter["language"]], 
                "MoreAbout"->Localized["MoreAboutSectionText", GetParameter["language"]],
                "Related Functions"->Localized["RelatedFunctionsSectionText", GetParameter["language"]],
                "RelatedFunctions"->Localized["RelatedFunctionsSectionText", GetParameter["language"]]
              }, 
              datalist = Last[data]
            }, (* end local vars *)

						If[dataid === "PDF",
							(* TODO:
								This warning was added during a refactoring removing special handling of
								the `dataid === "PDF"` case. I believe that that special logic is dead
								code, but I'm leaving this warning around for now just in case. In the
								future, upgrade this to an error for a while, and then remove it
								completely. *)
							DocsWarning["Internal warning: dataid is unexpectedly equal to \"PDF\""];
						];

            (* dprint["Defaults.m: datalist = "<>ToString[datalist, InputForm]]; *)
            (* dprint["Defaults.m: initial dataid = First[data] = "<>ToString[First[data], InputForm]]; *)
            (* dprint["Defaults.m: final dataid with localization = "<>ToString[dataid, InputForm]]; *)

            If[
              (Length[datalist] > 0) && (datalist =!= {{}})
            ,
              {
                Flatten@{
                  XMLElement["li", {}, {
                    XMLElement["a",
                      {
                        "href"->"#",
                        "class" -> Switch[dataid,
                          (* this returns "menu see-also" for any dataid phrases that aren't in english. *)
                          (* note that above, we replaced english with localized text in dataid. *)
                          (* either Switch using the unlocalized First[data] or add localized versions here (or just use things like Localized["SeeAlsoSectionText", GetParameter["language"]] here) *)
                          "See Also", "menu see-also",
                          "Related Guides", "menu related-guides",
                          "Workflows", "menu workflows",
                          "Tutorials", "menu tutorials",
                          "Functions", "menu see-also",
                          _, "menu see-also"
                        ]
                      },
                      {dataid}
                    ],
                    XMLElement["ul",
                      {"class"->"sub_menu"},
                      Map[
                        (* a function that's mapped over datalist *)
                        XMLElement["li",
                          (* dprint["----------------------------------------"]; *)
                          (* dprint["function mapped over datalist: # = "<>ToString[#, InputForm]]; *)
                          {
                            attr = If[
                              MatchQ[#, {_String, "Section"}]
                            , dprint["li attr: \"class\" -> \"other sub-ref-heading\""];
                              "class" -> "other sub-ref-heading"
                            , dprint["li attr: no attr"];
                              Unevaluated[Sequence[]]
                            ];
                            dprint["final li attr = "<>ToString[attr, InputForm]];
                            attr
                          },
                          {
                            Which[
                              # == {"Delimiter", "Delimiter"},
                                dprint["inner li Which branch 1, # = "<>ToString[#, InputForm]];
                                XMLElement["hr", {}, {}],
                              MatchQ[#, {_String, "Section"}],
                                dprint["inner li Which branch 2, # = "<>ToString[#, InputForm]];
                                Transmogrify[#[[1]]],
                              True,
                                dprint["inner li Which default, # = "<>ToString[#, InputForm]];
                                A[
                                  {
                                    innerAttr = If[
                                      MatchQ[#, {_String, _String, "RelatedLink"}]
                                    , dprint["inner li attr: \"class\" -> \"sub-ref\""];
                                      "class" -> "sub-ref"
                                    , dprint["inner li attr: none"];
                                      Unevaluated[Sequence[]]
                                    ];
                                    dprint["final inner li attr = "<>ToString[innerAttr, InputForm]];
                                    innerAttr
                                  ,
																		"href" -> DocumentationBuild`Export`DocLinkURL[
																			{GetParameter["pacletName"], uri},
																			#[[2]],
																			urlBaseRemote,
																			GetParameter["CompleteHTMLQ"]
																		]
                                  },
                                  {XMLElement["span", {"class"->"dropdownPad"}, {Transmogrify[#[[1]]]}]}
                                ]
                              ] (* end Which for delimiter, section or "a" link *)
                            }
                          ]&, (* end of XMLElement["li", _, _] *)
                          datalist
                        ]
                      ]
                    }]
                  }
                }
            ,
              {}
            ]
          ]], (* end of function to map over newAnchorBarData *)
          newAnchorBarData
        ]; (* end of pulldownmenu definition*)
dprint["pulldownmenu (put in dropdown) = "<>ToString[pulldownmenu, InputForm]];
        If[
          Positive @ Length[pulldownmenu]
        , 
          XMLElement["ul", {"class"->"dropdown"}, Flatten@{pulldownmenu}]
        ,
          XML`RawXML[""]
        ]
    , (* end of TrueQ[showAnchorBarQ] *)
      {}
    ],
    If[StringContainsQ[GetParameter["entityType"], "Monograph"]
    , {
        mHeads = Transmogrify[monographHeads];
        dprint["AnchorBarGrid: transmogrified monographHeads = "<>ToString[mHeads, InputForm]];
        mLinks = Transmogrify[monographLinks];
        dprint["AnchorBarGrid: transmogrified monographLinks = "<>ToString[mLinks, InputForm]];
        mHeads,
        mLinks
      }
    , Unevaluated[Sequence[]]
    ]
  }]; (* end of <nav> content *)

  thisElementContent
], (* closes "AnchorBarGrid" *)

{br} -> XMLElement["br", {}, {}],  
{Cell, "OpenCloseItemizedPicture"} :> "",
{Cell, "PacletNameCell"}:>Span[{"class"->"badge"}, {Recurse[]}],
{Cell, "ItemizedPicture" | "ItemPicture"} :> 
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
  {"alt"->""},
  Inline -> True,
  CropImage -> False,
  "RetinaImage" -> True
], 

{Cell, "HeaderCell"} :> "",
{CellGroupData, All} :> Recurse[], 

{Cell, "LaTex"} :> Span[{"class"-> GetStyle[]}, {Recurse[]}],

{AdjustmentBox}:>
Which[HasOption[BoxBaselineShift],
	XMLElement["sub", {"style" -> "vertical-align:-10%;"}, {Recurse[]} ],
	True, Recurse[]
	],


{DynamicBox}:>(
testDynamicBox[a_DynamicBox] := Module[{strip,strip2},

  strip=Cases[a, b_If :> Apply[List, Defer[b], 1], Infinity][[1]];
  strip2=Evaluate[strip][[1]];
  strip2[[3]]
   ];
   testDynamicBox[SelectSelf[]]
),


{Cell, All} :>
Module[{id},
id = If[HasOption[CellID], "id" -> ToString[GetOption[CellID]], Hold[Sequence[]]];
Which[GetStyle[]===None && !HasOption[CellID], Recurse[],
	GetStyle[]===None, 
  XMLElement["div", {ReleaseHold[id]}, 
    If[HasOption[CellTags],
    Flatten[{
      Map[
      (XMLElement["a", {"name" -> #}, {}])&, Flatten[{GetOption[CellTags]}] ],
      Recurse[] }],
    {Recurse[]} ] 
  ],  
  True, 
  DIV[{"class" -> GetStyle[], ReleaseHold[id]}, {Recurse[]}]
]],



(***** Other Box structures *****)
{RawData} :>  
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
  {"alt"->""},
  Inline -> True,
  CropImage -> False,
  "RetinaImage" -> True
], 


{PopupMenuBox} :>  Span[{}, {
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, "Popup_" <> IncrementCounter["Popup"] <> ".gif"],
    {"style" -> "vertical-align:middle;", "alt" -> " "},
    Inline -> True,
    CropImage -> False,
    "RetinaImage" -> True
  ]
 } 
], 


{TooltipBox} :> Block[{tooltip = SelectChildren[][[2]]}, 
	Which[Length[Cases[SelectSelf[], "TradFormChar", Infinity]]>0, 
		transmogrifyImage[
      ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
      {"alt"->""},
      Inline -> True,
      CropImage -> False,
      "RetinaImage" -> True
    ], 
Length[Cases[SelectSelf[], "StandFormChar", Infinity]]>0,
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
  {"alt"->""},
  Inline -> True,
  CropImage -> False,
  "RetinaImage" -> True
],
Length[Cases[SelectSelf[], GraphicsBox[___, Rule[FrameStyle, ___], ___], Infinity]]>0,
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
  {"alt"->""},
  Inline -> True,
  CropImage -> False,
  "RetinaImage" -> True
],
True, Transmogrify[SelectChildren[][[1]]]]
],


{GraphicsBox | PanelBox | PaneBox | Graphics3DBox} :>
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".png"],
    {"alt" -> ""},
    Inline -> True,
    CropImage -> False,
    ImageFormat -> "PNG",
    TransparentBackground -> True,
    ConversionOptions -> {Background -> None},
    "RetinaImage" -> True
  ],

{DynamicBox } :> Module[{dBoxContent}, 
dBoxContent=SelectSelf[][[1]];
If[MatchQ[Head@dBoxContent, String], StringReplace[dBoxContent, "\""->""],
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
  {"alt"->""},
  Inline -> True,
  CropImage -> False,
  "RetinaImage" -> True
]
]
], 


{Cell, "Picture"} :> 
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
  {"alt"->""},
  Inline -> True,
  ImageFormat -> "GIF",
  CropImage -> False,
  "RetinaImage" -> False
], 

(* If graphics are wider/taller than n, create popup window *)
{GraphicsData} :> 
(
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, "Box_" <> IncrementCounter["GraphicsData"] <> ".gif"],
    {"alt" -> " "},
    Inline -> False,
    CropImage -> False,
    "RetinaImage" -> True
  ]
 ),


{OutputFormData} :> Transmogrify[SelectChildren[][[2]]],

ValueBox["Year"] :> ToString[Date[][[1]]],

{InterpretationBox} :> Transmogrify[SelectChildren[]],

{FormBox} :> Module[
  {boxChildren, res},
  boxChildren=SelectChildren[];
  res = Which[
    MatchQ[boxChildren, {"Mathematica"}],
    Transmogrify[StyleBox["Mathematica", FontSlant->"Italic"]]
  ,
    MatchQ[boxChildren, {StyleBox[_, "KeyEvent", ___]}],
    Transmogrify[boxChildren]
  ,
    MatchQ[boxChildren, {_SuperscriptBox}],
    Transmogrify[boxChildren]
  ,
    True,
    transmogrifyImage[
      ToFileName[{"Files", GetBasenameLa[]}, "Image_"<> IncrementCounter["Image"] <> ".png"],
      ImageFormat -> "PNG",
      TransparentBackground->True,
      ConversionOptions->{Background->None},
      Inline -> If[GetStyle[]===TraditionalForm, True, False],
      MaxImageSize->580,
      CropImage -> False,
      "RetinaImage" -> True
    ]
  ];
  res
];

(*{FormBox} :> Module[{boxChildren,traditionalFormQ,altTag},
boxChildren=SelectChildren[];
traditionalFormQ = If[GetStyle[] === TraditionalForm, True, False];
altTag = DocumentationBuild`Utils`CellToString@SelectSelf[];
If[MatchQ[boxChildren, {"Mathematica"}],Transmogrify[StyleBox["Mathematica", FontSlant->"Italic"]],
If[MatchQ[boxChildren, {StyleBox[_, "KeyEvent", ___]}], Transmogrify[boxChildren],
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
  {"style"->"vertical-align:middle;", "alt"->altTag, "title"->altTag},
  "TransparentColor" -> White,
  Inline -> If[GetStyle[]===TraditionalForm, True, False],
  "RetinaImage" -> True
]
]
]
],*)


(***** String *****)
{String} :> Module[{stringContent, stringTest}, 
	
	stringContent=ReplaceSpecialCharacters[SelectSelf[]];
	stringTest[x_] := Which[
		Head[x] === XMLElement, x,
		Head[x] === RawXML, x,
  StringLength[x] === 1 && StringMatchQ[x, RegularExpression["[a-z]"]], XMLElement["I", {}, {x}], 
  		True, x];
	
  (* handle inline formatted strings... *)
  Which[InlineFormattedQ[SelectLiteral[SelectSelf[]]],
    		Transmogrify[FromInlineString[SelectSelf[]]],
    	
    	traditionalFormQ === True, 
			Sequence@@Map[stringTest[#]&,  {stringContent}],
    
    True, stringContent
    
  ]
],

(* a scratch cell for debugging. use as Cell["point 1", "gronk"] or similar *)
{Cell, "gronk"} :> Module[{},
  Print[ToString[InputForm[SelectSelf[]]]];
  XMLElement["div", {"class" -> "gronk"}, {Recurse[]}]
]


}]
