(* :Title: Functions.m *)

(* :Author:
	Andrew Hunt, andy@wolfram.com
	Greg Martel, gregm@wolfram.com
    Stephen Layland, layland@wolfram.com
*)

(* :Mathematica Version: 5.1 *)
(* :Package Version: $Revision: 1.23 $ $Date: 2017/12/07 15:32:51 $ *)
(* :Copyright: (c) 2005, Wolfram Research, Inc. All rights reserved. *)
(* :Summary: Add-ons specific rules and general transform for Mathematica Documentation *)



XMLTransform[{

(** Primary Examples **)

(*
Under the Example header, the first ExampleSection must always be open and the
rest closed.  An "Open All"/"Close All" button is provided when there are
multiple ExampleSections (and thus all but the first are closed).  If there is
only one ExampleSection, it is open by default and we do not want an "Open
All"/"Close All" button.  All the examples are already open.

https://jira.wolfram.com/jira/browse/WLDOCS-1231?focusedCommentId=2538730&page=com.atlassian.jira.plugin.system.issuetabpanels:comment-tabpanel#comment-2538730

tests for the 2017 redesign:
 - ref/device/Serial (1 ExampleSection, so open with no "open all"/"close all" button),
 - ref/device/Camera (1 ExampleSection, so open with no "open all"/"close all" button),
 - ref/format/PNG (many ExampleSections, so closed with an "open all"/"close all" button)

This is set up in Export.m: if a PrimaryExamplesSection has multiple
ExampleSections, it becomes "PrimaryExamplesSectionWithOpener"; if it has only
one ExampleSection, it becomes "PrimaryExamplesSectionWithoutOpener"
*)

{CellGroupData, "PrimaryExamplesSectionWithOpener"} :> 
  XMLElement["section", {"id" -> "Examples"}, {
      XMLElement["h1", {"class" -> "toggle open"}, {Localized["ExamplesSectionText", GetParameter["language"]]}],
      XMLElement["span", {"class" -> "open-all on"}, {ToLowerCase[Localized["Open All", GetParameter["language"]]]}],
      XMLElement["span", {"class" -> "close-all"}, {ToLowerCase[Localized["Close All", GetParameter["language"]]]}],
      XMLElement["div", {"class" -> "hideable"}, {
        Recurse[],
        XMLElement["div", {"class" -> "clear"}, {XML`RawXML["<!-- -->"]}]
      }]
  }],

{CellGroupData, "PrimaryExamplesSectionWithoutOpener"} :> 
  XMLElement["section", {"id" -> "Examples"}, {
      XMLElement["h1", {"class" -> "toggle open"}, {Localized["ExamplesSectionText", GetParameter["language"]]}],
      XMLElement["div", {"class" -> "hideable"}, {
        Recurse[],
        XMLElement["div", {"class" -> "clear"}, {XML`RawXML["<!-- -->"]}]
      }]
  }],

(** Primary Examples **)
{CellGroupData, "IndicatorExampleSection"} :> 
  XMLElement["section", {"id" -> "Examples"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle open"}, {Localized["ExamplesSectionText", GetParameter["language"]]}],
      XMLElement["span", {"class" -> "open-all on"}, {ToLowerCase[Localized["Open All", GetParameter["language"]]]}],
      XMLElement["span", {"class" -> "close-all"}, {ToLowerCase[Localized["Close All", GetParameter["language"]]]}],
      XMLElement["div", {"class" -> "hideable"}, {
        Recurse[],
        XMLElement["div", {"class" -> "clear"}, {XML`RawXML["<!-- -->"]}]
      }]
    }]
  }],
							
{Cell, "PrimaryExamplesSection" | "PrimaryExamplesSectionWithOpener" | "PrimaryExamplesSectionWithoutOpener" | "IndicatorExampleSection"} :> Unevaluated[Sequence[]],

{CellGroupData, "ExampleSection" | "ExtendedExamplesSection"} :> WithParameters[
  {
    "ExampleSectionCounter" -> IncrementCounter["ExampleSectionCounter"]
  },

  (* an anonymous container for the following h2 and div XMLElements *)
  XMLElement["div", {}, {
    XMLElement["h2",
      (* in the default view, "Basic Examples" is open and other sections are closed *)
      {"class" -> If[
        Positive @ Length[Cases[First[SelectChildren[]], s_String /; StringMatchQ[s, Localized["BasicExamplesSectionText", GetParameter["language"]]], Infinity]]
      ,
        "toggle open"
      ,
        "toggle"
      ]},
      {Transmogrify[First[SelectChildren[]]]}
    ],
    XMLElement["div", {"class" -> "hideable"}, {Transmogrify[Rest[SelectChildren[]]]}]
  }]
],

(* this is just the section name and the number of examples it contains *)
{Cell, "ExampleSection" | "ExtendedExamplesSection"} :> Recurse[],

{CellGroupData, "ExampleSubsection"} :> 
  (* anonymous div to contain these two elements - is there a more legit way to do this? *)
  XMLElement["div", {}, {
    XMLElement["h3", {"class" -> "toggle"}, {Transmogrify[First[SelectChildren[]]]}],
    XMLElement["div", {"class" -> "hideable"}, {Transmogrify[Rest[SelectChildren[]]]}]
  }],
        
{Cell, "ExampleSubsection"} :> Recurse[],

{CellGroupData, "ExampleSubsubsection"} :> 
  (* anonymous div to contain these two elements - is there a more legit way to do this? *)
  XMLElement["div", {}, {
    XMLElement["h4", {"class" -> "toggle"}, {Transmogrify[First[SelectChildren[]]]}],
    XMLElement["div", {"class" -> "hideable"}, {Transmogrify[Rest[SelectChildren[]]]}]
  }],

{Cell, "ExampleSubsubsection"} :> Recurse[],

{Cell, "ExampleCount"} :> XMLElement["span", {"class" -> "total-examples"}, {Recurse[]}]

}]
