(* 
  Title: ButtonBox XMLTransform
  Authors: Greg Martel, Andy Hunt, Stephen Layland
*)



XMLTransform[{

{TemplateBox} :> Block[
  {
    output, 
  	urlBaseRemote = GetParameter["urlBaseRemote"], 
  	htmlBase = GetParameter["htmlBase"],
  	pacletName = GetParameter["pacletName"], 
  	completeHTMLQ = GetParameter["CompleteHTMLQ"], 
  	isALink,
    isAPackageLink = False,
    isANonWLLink = False,
    isASeeAlsoSublink = False,
    isAStringTypeLink = False,
  	link,
  	versionNumber, 
  	hasOptionBaseStyle,
  	getOptionBaseStyle,
  	hasOptionButtonData,
  	getOptionButtonData, 
  	anchorButtons, 
  	anchorContent,
  	hasOptionButtonFrame, 
  	uri = GetParameter["url"],
  	link2, res
	},

Module[{
	templateTag,
	linkLabel = None,
	linkDestination
},
	(* Extract the TemplateBox tag name, or generate an error. *)
	templateTag = Replace[SelectSelf[], {
		TemplateBox[_, tag_?StringQ, ___?OptionQ] :> tag,
		self_ :> (
			DocsError["TemplateBox has an unexpected structure: ``", InputForm[self]];
		)
	}];

	Assert[StringQ[templateTag]];

  (* dprint["TemplateBox.m: SelectSelf[] = "<>ToString[SelectSelf[], InputForm]]; *)

  versionNumber = StringSplit[$Version, "."][[1]];

	isALink = MemberQ[
		{
			"GrayLink", "TealLink", "GrayWebLink", "OrangeLink", "WFOrangeLink",
			"PackageLink", "RefLink", "RefLinkPlain", "MenuGrayLink", "WebLink",
			"GrayLinkWithIcon", "StringTypeLink", "StringTypeSublink", "EntityTypeLink",
			"NonWLLink"
		},
		templateTag
	];

	isASeeAlsoSublink = isALink && MemberQ[{"StringTypeSublink"}, templateTag];

  (* https://jira.wolfram.com/jira/browse/WLDOCS-1022: links to package /ref/ pages should have a different style. *)
  (* links to packages contain ^paclet:[A-Z].+/ref/; other links match ^paclet:[a-z] *)
  isAPackageSymbolLink = Positive @ Length @ Cases[
    SelectSelf[],
    s_String /; StringContainsQ[s, StartOfString ~~ "paclet:" ~~ CharacterRange["A", "Z"] ~~ __ ~~ "/ref/"],
    Infinity
  ];

  (* https://jira.wolfram.com/jira/browse/WLDOCS-2203: links to non-package non-WL
  pages should have their own unique style.  ip nbs use NonWLLink, to be
  transformed to <a class="non-wl-link">. *)

  (* brianv: Just to clarify, I'll be fixing this in the notebooks too, eventually.
  I'll be implementing this fix like this: instead of the TemplateBox object
  with a style "WebLink", it'll be something else like "WebLinkGreen".  So if
  you want to future proof what you're doing (if there's a chance that that
  change might clash), plan for that. *)

  (* SelectSelf[] = TemplateBox[{Cell[TextData["WolframLanguageSession"]], "https://reference.wolfram.com/language/WolframClientForPython/api/wolframclient.evaluation.html#wolframclient.evaluation.WolframLanguageSession"}, "WebLink", BaseStyle -> {"GuideText"}] *)

	If[isALink,
		{linkLabel, linkDestination} = Replace[SelectSelf[], {
			TemplateBox[{label_, dest_?StringQ, ___}, _?StringQ, ___] :> {label, dest},
			self_ :> (
				DocsError["TemplateBox link has an unexpected structure: ``", InputForm[self]];
			)
		}];

		Assert[linkLabel =!= None];
		Assert[StringQ[linkDestination]];

		(* Formatting note: 9d41f5a9*)
		isANonWLLink = Or[
			SameQ[templateTag, "NonWLLink"],
			StringContainsQ[linkDestination, StartOfString ~~ "https://reference.wolfram.com/language/"]
		];

	];

	(* https://jira.wolfram.com/jira/browse/WLDOCS-2178: StringTypeLink -> non-wl-link-grey *)
	isAStringTypeLink = SameQ[templateTag, "StringTypeLink"];

  (* https://jira.wolfram.com/jira/browse/WLDOCS-2190 *)
  isEntityRelatedInterpreterLink = Positive @ Count[SelectSelf[], "EntityRelatedInterpreter", Infinity];

(* dprint["TemplateBox.m: isALightweightGridLink = "<>ToString[isALightweightGridLink, InputForm]]; *)

  {output, link} = If[
    isALink
  ,
		Assert[linkLabel =!= None];
		Assert[StringQ[linkDestination]];

		(* if SelectSelf[] is one of the link objects: *)
		{{Transmogrify[linkLabel]}, linkDestination}
  ,
    (* if SelectSelf[] is NOT one of the link objects: *)
    {
      transmogrifyImage[
        ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".png"],
        {"alt"->""},
        Inline -> True, CropImage -> False, ImageFormat -> "PNG", TransparentBackground->True, ConversionOptions->{Background->None}, "RetinaImage" -> True
      ]
    ,
      transmogrifyImage[
        ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".png"],
        {"alt"->""},
        Inline -> True, CropImage -> False, ImageFormat -> "PNG", TransparentBackground->True, ConversionOptions->{Background->None}, "RetinaImage" -> True
      ]
    }
  ];

	link = If[isALink,
		Switch[templateTag,
			(* if isALink and templateTag matches this: *)
			"RefLink" | "RefLinkPlain" | "GrayLink" | "TealLink" | "OrangeLink" | "WFOrangeLink"
			| "PackageLink" | "StringTypeLink" | "StringTypeSublink" | "EntityTypeLink" | "NonWLLink"
			| "MenuGrayLink" | "GrayLinkWithIcon",
				DocumentationBuild`Export`DocLinkURL[
					{pacletName, uri},
					link,
					urlBaseRemote,
					completeHTMLQ
				],

			(* if isALink and templateTag matches this: *)
			"WebLink" | "GrayWebLink",
				If[!completeHTMLQ,
					convertWSMLink @ convertModelicaLink @ ConvertToString[link]
					,
					link
				],

			(* if isALink and templateTag matches this: *)
			_,
				link
		]
		,
		(* when isALink is False: *)
		transmogrifyImage[
			ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".png"],
			{"alt" -> ""},
			Inline -> True,
			CropImage -> False,
			ImageFormat -> "PNG",
			TransparentBackground -> True,
			ConversionOptions -> {Background -> None},
			"RetinaImage" -> True
		]
	];

  res = If[
    isALink
  ,
    (* if this TemplateBox contains a link, return XMLElement["a", ...] *)
    XMLElement[
      "a"
    ,
      Which[
        isANonWLLink,
        {"class" -> "non-wl-link", "href"->link}
      ,
        isAStringTypeLink,
        {"class" -> "non-wl-link-grey", "href"->link}
      ,
        isAPackageSymbolLink,
        {"class" -> "package-symbol", "href"->link}
      ,
        isASeeAlsoSublink,
        {"class" -> "sub-ref", "href"->link}
      ,
        isEntityRelatedInterpreterLink,
        {"class" -> "related-interpreter", "href"->link}
      ,
        True,
        {"href"->link}
      ]
    ,
      Flatten[{output}]
    ]
  ,
    (* if this TemplateBox does not contain a link, return what we computed for 'output' *)
    output
  ];

  res

](* end Module*)
](* end Block *)
}] (* end the TemplateBox XMLTransform *)

