(* :Title: Featuresets.m *)

(* :Author:
  Bill White, billw@wolfram.com
*)

(* :Mathematica Version: 13 *)
(* :Package Version: $Revision:  $ $Date:  $ *)                  
(* :Copyright: (c) 2021, Wolfram Research, Inc. All rights reserved. *)
(* :Summary: General transform for Mathematica Featureset Pages *)

(* m- things to be defined before filling in layout *)
(* lang *)
(* pageTitle = heroHeading *)
(* pageDescription *)
(* pageURL *)
(* pageBasename *)
(* heroHeading *)
(* heroIntroText *)

XMLTransform[{

(* Transform Notebook *)
{Notebook} :> Block[
  {title, nbExpr, restNBExpr, mainPrev, mainNext, exampleholder,
  mainFileNameBase, buildNumber, $NewIn7MarketingNBList, ext, descriptionfile, descriptionfileQ,
  descriptionfromfile}
,
dprint["** Featuresets.m: entering Notebook transform **"];

dprint["Exporting incoming nb expr to /tmp/file.m"];
(* Export["/tmp/file.m", SelectSelf[]]; *)

ext = Switch[GetParameter["Language"], 
	"ChineseSimplified" , "zh", 
	"Japanese"          , "ja", 
	"Spanish"           , "es", 
	"Russian"           , "ru",
	"Korean"            , "ko",
	"Portuguese"        , "pt-br",
	"German"            , "de",
	"French"            , "fr",
	 _                  , "en"];
dprint["ext = "<>ToString[ext, InputForm]];

nbExpr = SelectSelf[];

exampleholder = 0;

pageURL = "TODO: Transmogrify parameter";

mainFileNameBase = GetBasenameLa[];
dprint["Transform file Featuresets.m: mainFileNameBase = "<>ToString[mainFileNameBase, InputForm]];

dprint["Featuresets.m transform: checking needed layout parameters; some are set inside FillInLayout"];
dprint[" - lang = "<>ToString[
  Switch[GetParameter["Language"]
  , "ChineseSimplified", "zh"
  , "Spanish", "es"
  , "Japanese", "ja"
  , "Korean", "ko"
  , "Russian", "ru"
  , "Portuguese", "pt-br"
  , "German", "de"
  , "French", "fr"
  , _, "en"
  ],
  InputForm
]];
dprint["checking parameters coming into transform file"];
dprint["GetParameter[\"metaTitle\"] = "<>ToString[GetParameter["metaTitle"], InputForm]];
dprint["GetParameter[\"metaDescription\"] = "<>ToString[GetParameter["metaDescription"], InputForm]];
dprint["GetParameter[\"metaURL\"] = "<>ToString[GetParameter["metaURL"], InputForm]];
dprint["GetParameter[\"metaKeywords\"] = "<>ToString[GetParameter["metaKeywords"], InputForm]];
dprint["GetParameter[\"metaKeywords\"] = "<>ToString[GetParameter["metaKeywords"], InputForm]];


(* layout file *)
FillInLayout[
  GetParameter["layoutfile"]
, "Text"
, "pageTitle" -> GetParameter["metaTitle"]
, "lang" -> "en"
, "pageDescription" -> GetParameter["metaDescription"]
, "pageURL" -> GetParameter["metaURL"]
, "metakeywords" -> If[Length@GetParameter["metaKeywords"] > 0,
    ExportString[
      XMLElement["meta", {"name" -> "keywords", "content" -> ToStringList[GetParameter["metaKeywords"]]}, {}],
      "XML",
      "AttributeQuoting" -> "\"",
      "Entities" -> entityrules
    ],
    ""
  ]
, "body" -> StringReplace[
    ExportString[
      XMLElement["main", {}, {Recurse []}]
    , "XML"
    , "AttributeQuoting" -> "\""
    , "Entities" -> {"\[LongDash]"->"&mdash;"}
    , "ElementFormatting"->Automatic
    ]
  , {
      "VIRTUAL-INCLUDES-BEGIN" -> "<!--#include virtual='",
      "VIRTUAL-INCLUDES-END" -> "'-->",
      "[NBSP][RGILL]" -> "\[NonBreakingSpace]\[RightGuillemet]",
      "[NBSP]" -> "\[NonBreakingSpace]",
      "[BEGINPHP]" -> "<?= ",
      "[ENDPHP]" -> " ?>"
    }
  ]
] 
],

(* **** outline *)

(*
,----[ "SecondaryLinksSection" ]
| <section id="resource-lists">
|   <div id="resource-lists-content">
|     <div class="middle-wrapper">
|       <div class="grid cols-2 heirs-width-1-2 cols-1__600 heirs-width-full__600">
| ,----[ "FeaturesetSeeAlsoSection" ]
| |         <div>
| |           <div class="list">
| |             <span class="header">Secondary Header</span>
| |             <ul>
| | ,----[ "FeaturesetSeeAlso" ]
| | |             <li><a href="">List item »</a></li>
| | `----
| | ,----[ "FeaturesetSeeAlso" ]
| | |             <li><a href="">List item »</a></li>
| | `----
| | ,----[ "FeaturesetSeeAlso" ]
| | |             <li><a href="">List item »</a></li>
| | `----
| | ,----[ "FeaturesetSeeAlso" ]
| | |             <li><a href="">List item »</a></li>
| | `----
| |             </ul>
| |           </div>
| |         </div>
| `----
| ,----[ "FeaturesetRelatedLinksSection" ]
| |         <div>
| |           <div class="list">
| |             <span class="header">Secondary Header</span>
| |             <ul>
| | ,----[ "FeaturesetRelatedLinks ]
| | |             <li><a href="">List item »</a></li>
| | `----
| | ,----[ "FeaturesetRelatedLinks ]
| | |             <li><a href="">List item »</a></li>
| | `----
| | ,----[ "FeaturesetRelatedLinks ]
| | |             <li><a href="">List item »</a></li>
| | `----
| | ,----[ "FeaturesetRelatedLinks ]
| | |             <li><a href="">List item »</a></li>
| | `----
| |             </ul>
| |           </div>
| |         </div>
| `----
|       </div>
|     </div>
|   </div>
| </section>
`----
*)

(* **** transforms *)

{CellGroupData, "SecondaryLinksSection"} :>
  XMLElement["section", {"id" -> "resource-lists"}, {
    XMLElement["div", {"id" -> "resource-lists-content", "class" -> "narrow feature-grid"}, {
      Recurse[]
    }]
  }],
{Cell, "SecondaryLinksSection"} :> Unevaluated[Sequence[]],

{CellGroupData, "FeaturesetSeeAlsoSection" | "FeaturesetRelatedLinksSection"} :>
  XMLElement["div", {}, {
    XMLElement["h3", {}, {
      Which[
        Positive@Count[SelectSelf[], Cell[_, "FeaturesetSeeAlsoSection", ___], Infinity], "Related Core Areas"
      , Positive@Count[SelectSelf[], Cell[_, "FeaturesetRelatedLinksSection", ___], Infinity], "Additional Resources"
      , True, "Secondary Header"
      ]
    }],
    XMLElement["ul", {}, {
      Recurse[]
    }]
  }],
{Cell, "FeaturesetSeeAlsoSection" | "FeaturesetRelatedLinksSection"} :> Unevaluated[Sequence[]],

{Cell, "FeaturesetSeeAlso" | "FeaturesetRelatedLinks"} :> Module[{children, res},

  (* If an entire button-link structure is styled (very easy to do in the fe),
  move its styling into the structure so the following chevron (which must be
  expressed as an attribute of the link) will be outside the styling directive.
  doing this now avoids more complicated handling of the transmogrified xml expr *)

  children = SelectChildren[] /. StyleBox[ButtonBox[s:_String, rest:___], styleinfo:___] -> ButtonBox[StyleBox[s, styleinfo], rest];

  res = XMLElement["li", {}, {Transmogrify[children]}];
  res = res /. {"href" -> s:_String} :> {"class" -> "chevron-after", "href" -> s};

  If[StringContainsQ["href" /. Cases[res, _Rule, Infinity], Alternatives["stackexchange.com", "populate-this-as-needed.com"]], 
    res = res /. XMLElement["a", attrList:_, contentsList:_] :> XMLElement["a", Join[attrList, {"target" -> "_blank"}], contentsList]];
  res
],

{Cell, "GetStartedSection"} :> Recurse[],
{CellGroupData, "GetStartedSection"} :>
XML`RawXML["
<section id=\"cta\">
    <div id=\"cta-content\" class=\"narrow\">
        <h2>Get Started with Wolfram "<>Recurse[]<>"</h2>

        <div class=\"cta-grid\">
            <div class=\"img\"><img src=\"assets/img/shared/icon-wolfram-one.svg\" width=\"77\" height=\"79\" alt=\"\"></div>
            <div class=\"txt\">
                <h3><span>Access the Wolfram Language through</span>
                <a href=\"https://www.wolfram.com/wolfram-one/\">Wolfram|One</a> or <a href=\"https://www.wolfram.com/mathematica/\">Mathematica</a></h3>
                <p>The state of the art in technical computing and the principle programming software for millions of innovators, educators and students around the world.</p>
                <?php include $_SERVER['DOCUMENT_ROOT'] . '/core/assets/includes/sitebutton-cta.php'; ?>
                <div class=\"place-holder\" id=\"form-anchor\"></div>
            </div>
        </div>

        <div class=\"cta-grid\">
            <div class=\"img\"><img src=\"assets/img/shared/icon-consulting.svg\" width=\"56\" height=\"53\" alt=\"\"></div>
            <div class=\"txt\">
                <h3 class=\"red\">Get Help with Your Project</h3>
                <p>Enlist the world's computation experts to help with your project, no matter its size.</p>
                <div class=\"buttons\"><a class=\"button secondary form-trigger two\" href=\"#talk-to-a-wolfram-expert\">Talk to a Wolfram Expert</a></div>
                <div id=\"talk-to-a-wolfram-expert\" class=\"hide\">
                    <iframe id=\"talk-to-a-wolfram-expert-form\" scrolling=\"no\" height=\"100%\" width=\"100%\" title=\"Talk to a Wolfram Expert\" src=\"assets/form/?feat=<?php echo basename($_pageURL); ?>\" style=\"height: 0px;\"></iframe>
                </div>
            </div>
        </div>

    </div>
</section>
"],

{Cell, "LogoSection"} :> Sequence[],
{CellGroupData, "LogoSection"} :>
XML`RawXML["
<section id=\"logos\">
        <div id=\"logos-content\">
            <h2>Wolfram Supports Organizations <span class=\"white-space-nw\">Large and Small</span></h2>

            <ul class=\"flex flex-wrap-w justify-content-fs justify-content-se__600\">
                <li><a href=\"//www.boeing.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/boeing.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.intel.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/intel.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.lockheed-martin.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/lockheed-martin.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//lanl.gov\" target=\"_blank\"><img src=\"/core/assets/img/companies/los-alamos.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.apple.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/apple.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.ibm.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/ibm.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.microsoft.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/microsoft.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.nike.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/nike.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.ntt.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/ntt.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.nasa.gov\" target=\"_blank\"><img src=\"/core/assets/img/companies/nasa.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.ge.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/ge.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.bosch.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/bosch.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.deere.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/john-deere.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.ford.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/ford.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.airbus.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/airbus.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.gm.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/gm.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.db.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/deutsche-bank.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.epa.gov\" target=\"_blank\"><img src=\"/core/assets/img/companies/epa.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.roche.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/roche.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//corporate.exxonmobil.com/\" target=\"_blank\"><img src=\"/core/assets/img/companies/exxon-mobil.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.pfizer.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/pfizer.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.ups.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/ups.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.bayer.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/bayer.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.3m.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/3m.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.amd.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/amd.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.amgen.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/amgen.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.xerox.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/xerox.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.honeywell.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/honeywell.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.canon.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/canon.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.cummins.com\" target=\"_blank\"><img src=\"/core/assets/img/companies/cummins.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.goldmansachs.com/\" target=\"_blank\"><img src=\"/core/assets/img/companies/goldman-sachs.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.agilent.com/\" target=\"_blank\"><img src=\"/core/assets/img/companies/agilent.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.alcoa.com/global/en/home\" target=\"_blank\"><img src=\"/core/assets/img/companies/alcoa.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.asc-csa.gc.ca/eng/default.asp\" target=\"_blank\"><img src=\"/core/assets/img/companies/canadian-space-agency.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.airbus.com/en\" target=\"_blank\"><img src=\"/core/assets/img/companies/eurocopter.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.faa.gov/\" target=\"_blank\"><img src=\"/core/assets/img/companies/faa.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//global.fujifilm.com/en/\" target=\"_blank\"><img src=\"/core/assets/img/companies/fujifilm.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.mpg.de/en\" target=\"_blank\"><img src=\"/core/assets/img/companies/max-planck-gesellschaft.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.ll.mit.edu/\" target=\"_blank\"><img src=\"/core/assets/img/companies/mit-ll.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"http://www.mitsubishielectric.com/en/\" target=\"_blank\"><img src=\"/core/assets/img/companies/mitsubishi.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//optics.synopsys.com/\" target=\"_blank\"><img src=\"/core/assets/img/companies/synopsys.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.thalesdsi.com/\" target=\"_blank\"><img src=\"/core/assets/img/companies/thales.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.federalreserve.gov/\" target=\"_blank\"><img src=\"/core/assets/img/companies/us-federal-reserve.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
                <li><a href=\"//www.usgs.gov/\" target=\"_blank\"><img src=\"/core/assets/img/companies/usgs.svg\" alt=\"\" width=\"70\" height=\"40\"></a></li>
           </ul>
        </div>
    </section>"
],

(* **** hero section *)

{Cell, "FeaturesetTitle"} :>
  XMLElement["h1", {}, {
    XMLElement["span", {"class" -> "wolfram"}, {"Wolfram"}],
    XMLElement["span", {"class" -> "core-area"}, {Recurse[]}],
    XMLElement["a", {"class" -> "subtitle", "href" -> "/language/"}, {
      "A Core Part of the",
      XML`RawXML["<br class=\"hide__900\">"],
      " Wolfram Language"
    }]
  }],

{Cell, "Hero"} :> Sequence[],
{CellGroupData, "Hero"} :> Module[{color},
  color = "Color" /. Options[First @ SelectChildren[]]; (* the first child is the "Hero" cell, by definition *)
  XMLElement["header", {"id" -> "hero", "class" -> color}, {
    XMLElement["div", {"id" -> "hero-content"}, {
      Recurse[]
    }]
  }]
],

{Cell, "Breadcrumb"} :>
  XMLElement["div", {"id" -> "breadcrumb"}, {
    XMLElement["a", {"class" -> "arrow-l-before", "href" -> "#"}, {"Featured Areas"}]
  }],

{Cell, "FeatureGrid"} :> Sequence[],
{CellGroupData, "FeatureGrid"} :>
  XMLElement["div", {"class" -> "feature-grid"}, {
    Recurse[]
  }],

{Cell, "FeatureLeft"} :> Sequence[],
{CellGroupData, "FeatureLeft"} :>
  XMLElement["div", {"class" -> "feature-left"}, {
    Recurse[]
  }],

{Cell, "FeaturesetTitleText"} :> Recurse[],
{CellGroupData, "FeaturesetTitleText"} :> Module[{tagline},
  tagline = Quiet @ Check[First @ Cases[SelectSelf[], Cell[s:_String, "FeaturesetBoldTagline", ___] -> s, Infinity], ""];
  blurbCell = First @ SelectChildren[];
  XMLElement["p", {}, Flatten @ {
    If[Positive @ StringLength[tagline]
    , {XMLElement["strong", {}, {tagline}],
        " ",
        Transmogrify[blurbCell]
      }
    , Transmogrify[blurbCell]
    ]
  }]
],

{Cell, "HeroImage"} :> Module[{
  image,
  outfile,
  imagefile = FileNameJoin[{"Files", GetBasenameLa[], "hero.webp"}]
},

  outfile = FileNameJoin[{GetParameter["outputDir"], imagefile}];

  If[Not @ FileExistsQ[outfile],
    image = ToExpression[First @ SelectChildren[]];
    Export[imagefile, image]];

  (* use the exported hero image *)
  XMLElement["img", {
    "class" -> "hide show__900",
    "src" -> imagefile
  }, {""}]
],

{Cell, "HeroButton"} :>
  XMLElement["div", {"class" -> "buttons"}, {
    XMLElement["a", {"class" -> "button primary", "href" -> "#cta"}, {"Get Started"}]
  }],

{CellGroupData, "FeatureRight"} :> Module[{
  image,
  outfile,
  imagefile = FileNameJoin[{"Files", GetBasenameLa[], "hero.webp"}],
},

  outfile = FileNameJoin[{GetParameter["outputDir"], imagefile}];

  If[Not @ FileExistsQ[outfile],
    image = First @ Cases[SelectSelf[], Cell[i:_, "HeroImage", ___] :> ToExpression[i], Infinity];
    Export[imagefile, image]
  ];

  XMLElement["div", {"class" -> "feature-right hide__900"}, {
    XMLElement["img", {"src" -> imagefile}, {""}]
  }]
], 
{Cell, "FeatureRight"} :> Sequence[],

(* **** "example" sections *)

{Cell, "ExampleSection"} :> Sequence[],

{CellGroupData, "ExampleSection"} :> Module[{sectionName, sectionCounter, class},
  dprint["======================================================================"];
  dprint["ExampleSection xform: SelectSelf[] is "]; dfile[SelectSelf[]];

  sectionCounter = First @ Cases[SelectSelf[], c:Cell[_, "ExampleSection", ___] :> ("SectionCounter" /. Options[c]), Infinity];
  dprint["section number "<>ToString[sectionCounter]];

  sectionName = Quiet @ Check[
    First@Cases[SelectSelf[], Cell[s:_String, "ExampleSection", ___] -> s, Infinity],
    "No Section Name"
  ];
  sectionName = DocumentationBuild`Utils`decamel[sectionName, "Lowercase" -> True, "Separator" -> "-"];

  class = {"feature-grid"};
  (* If[sectionCounter == 1, AppendTo[class, "no-divider"]]; *)
  If[OddQ[sectionCounter], AppendTo[class, "img-on-left"]];
  class = StringJoin[Riffle[class, " "]];
  dprint["class = "<>ToString[class, InputForm]];

  XMLElement["section", {"id" -> sectionName, "class" -> "feature"}, {
    XMLElement["div", {"id" -> sectionName<>"-content", "class" -> class}, {
      Recurse[]
    }]
  }]
],

{Cell, "ExampleSectionFeatureLeft"} :> Sequence[],

{CellGroupData, "ExampleSectionFeatureLeft"} :> Module[{},
  dprint["ExampleSectionFeatureLeft xform: SelectSelf[] is "]; dfile[SelectSelf[]];
  XMLElement["div", {"class" -> "feature-left"}, {Recurse[]}]
 ],

{Cell, "ExampleSectionHead"} :> XMLElement["h2", {}, {Recurse[]}],
{Cell, "ExampleSectionBlurb"} :> XMLElement["p", {}, {Recurse[]}],

(* coming in with  "ImageFilename" ->"/Users/billw/git/featuresetpages/English/Audio/audio-identify.png" *)

{Cell, "ExampleSectionImage"} :> Module[{
  sectionTitle,
  class,
  durations,
  sourceImageName = "feature-image-placeholder.webp",
  sectionCounter,
  videoQ,
  outputImageFilename
},
  dprint["----------------------------------------"];
  dprint["entering ExampleSectionImage transform"];
  dprint["SelectSelf[] = "]; dfile[SelectSelf[]];
  dprint["SelectChildren[] = "<>ToString[SelectChildren[] /. _NumericArray -> xxxNumericArray[], InputForm]];

(*
Cell[
  Video[
    "/Users/billw/Dropbox/homedir/git/featuresetpages/English/zReference/sin-x-sin-ax.mp4",
    Appearance -> Automatic,
    AudioOutputDevice -> Automatic,
    PlaybackSettings -> <||>,
    SoundVolume -> Automatic
  ],
  "ExampleSectionImage",
  "SourceImageFilename" -> "/Users/billw/Dropbox/homedir/git/featuresetpages/English/zReference/sin-x-sin-ax.mp4",
  "DisplayDurations" -> {},
  "SectionCounter" -> 4,
  "SubheroImageCounter" -> 1,
  "SectionTitle" -> "video",
  "Class" -> "hide show__900",
  "NodeID" -> 44
]
*)

  {sectionTitle, class, durations, sourceImageFilename, sectionCounter} = {"SectionTitle", "Class", "DisplayDurations", "SourceImageFilename", "SectionCounter"} /. Options[SelectSelf[]];
  videoQ = FileExtension[sourceImageFilename] == "mp4";

  outputImageFilename = If[
    FileBaseName[sourceImageFilename] == "feature-image-placeholder"
  , FileNameJoin[{"assets", "img", "shared", FileNameTake[sourceImageFilename]}]
  , FileNameJoin[{"Files", GetBasenameLa[], FileNameTake[sourceImageFilename]}]];

  (* just in case *)
  outputImageFilename = StringReplace[outputImageFilename, ".png" ~~ EndOfString -> ".webp"];

  dprint["{sectionTitle, class, durations, sourceImageFilename} = "<>ToString[{sectionTitle, class, durations, sourceImageFilename}, InputForm]];
  dprint["videoQ = "<>ToString[videoQ]];
  dprint["outputImageFilename = "<>ToString[outputImageFilename, InputForm]];

  (* write the web file *)
  If[videoQ
  , CopyFile[sourceImageFilename, outputImageFilename]
  , image = ToExpression @ First[SelectChildren[]];
    If[FileBaseName[sourceImageFilename] =!= "feature-image-placeholder" && Not @ FileExistsQ[outputImageFilename],
      If[Positive @ Length @ durations
      , Export[outputImageFilename, image, "DisplayDurations" -> durations]
      , Export[outputImageFilename, image]
      ]
    ]
  ];

  dprint["FileExistsQ[outputImageFilename] = "<>ToString[FileExistsQ[outputImageFilename], InputForm]];

  Switch[class,
  
    "hide show__900",
    If[videoQ
    , XMLElement["div", {"class" -> class<>" wri-video basic"}, {
        XMLElement["video", {"loop" -> "", "muted" -> "", "playsinline" -> "", "id" -> "vid_"<>ToString[sectionCounter]<>"a"}, {
          XMLElement["source", {"src" -> outputImageFilename}, {""}]
        }]
      }]
    , XMLElement["img", {    
        "class" -> class,
        "src" -> outputImageFilename,
        "alt" -> ""
      }, {" "}]
    ]

  , "feature-right hide__900",
    If[videoQ
    , XMLElement["div", {"class" -> class<>" wri-video basic"}, {
        XMLElement["video", {"loop" -> "", "muted" -> "", "playsinline" -> "", "id" -> "vid_"<>ToString[sectionCounter]<>"b"}, {
          XMLElement["source", {"src" -> outputImageFilename}, {""}]
        }]
      }]
    , XMLElement["div", {"class" -> class}, {
        XMLElement["img", {
          "src" -> outputImageFilename,
          "alt" -> ""
        }, {" "}]
      }]
    ]

  , _, XMLElement["p", {}, {"UNKNOWN IMAGE CLASS"}]
  ]
],

{CellGroupData, "ExampleSectionLink"} :> XMLElement["ul", {}, {Recurse[]}],
{Cell, "ExampleSectionLink"} :> Module[{},
  dprint["ExampleSectionLink xform"];
  dprint["SelectSelf[] is "<>ToString[SelectSelf[], InputForm]];
  link = Quiet @ Check[First[Cases[SelectSelf[], Rule["Link", s:_String] -> s, Infinity]], ""];
  XMLElement["li", {}, {
    XMLElement["a", {
      "class" -> "chevron-after",
      "href" -> If[link == "", "javascript:void(0)", link]
    }, {Recurse[]}]
  }]
],

              (* <li><a class="chevron-after" href="">Guide to Computer Vision</a></li> *)

(* **** Related Courses *)

{CellGroupData, "CoursesSection"} :> Module[{},
  dprint["entering CoursesSection, SelectSelf[] is"]; dfile[SelectSelf[]];
  XMLElement["section", {"id" -> "resources"}, {
    XMLElement["div", {"id" -> "resources-content"}, {
      Recurse[]
    }]
  }]
],
{Cell, "CoursesSection"} :> XMLElement["h2", {}, {"Introductory Courses"}],

{CellGroupData, "UpperCoursesPod"} :> XMLElement[
  "div",
  {"class" -> "grid cols-3 heirs-width-1-3 cols-1__900 heirs-width-full__900"},
  {Recurse[]}
],
{Cell, "UpperCoursesPod"} :> Sequence[],

{CellGroupData, "UpperCoursesLeftColumn"} :>
  XMLElement["div", {"class" -> "left-col"}, {
    XMLElement["div", {"class" -> "resources-pod"}, {
      Recurse[]
    }]
  }],
{Cell, "UpperCoursesLeftColumn"} :> Sequence[],

(* ok to here *)

{CellGroupData, "UpperCoursesCenterColumn"} :>
  XMLElement["div", {"class" -> "center-col"}, {
    XMLElement["div", {"class" -> "resources-pod"}, {
      Recurse[]
    }]
  }],
{Cell, "UpperCoursesCenterColumn"} :> Sequence[],

{CellGroupData, "UpperCoursesRightColumn"} :>
  XMLElement["div", {"class" -> "right-col"}, {
    XMLElement["div", {"class" -> "resources-pod"}, {Recurse[]}]
  }],
{Cell, "UpperCoursesRightColumn"} :> Sequence[],

(* ok to here *)

{Cell, "CourseImageBoilerplate"} :>
  XMLElement["div", {"class" -> "resources-image"}, {
    XMLElement["a", {"href" -> ""}, {
      XMLElement["img", {"src" -> "assets/img/shared/course-boilerplate.png"}, {""}]
    }]
  }],
{Cell, "CourseTypeBoilerplate"} :> XMLElement["h3", {}, {"Interactive"}],
{Cell, "CourseLinkBoilerplate"} :> XMLElement["a", {"class" -> "chevron-after", "href" -> ""}, {"Course Description"}],


{Cell, "CourseImage"} :> Module[{url, image, imagefile},
  dprint["Entering CourseImage transform, SelectSelf[] = "]; dfile[SelectSelf[]];
  dprint["Entering CourseImage transform, SelectChildren[] = "]; dfile[SelectChildren[]];

  url = "url" /. Options[SelectSelf[]];
  image = ToExpression[First @ SelectChildren[]];
  imagefile = FileNameJoin[{"Files", GetBasenameLa[], FileNameTake[url]<>".webp"}];

  XMLElement["div", {"class" -> "resources-image"}, {
    XMLElement["a", {"href" -> url}, {
      XMLElement["img", {"src" -> Export[imagefile, image]}, {""}]
    }]
  }]
],

{Cell, "CourseType"} :> XMLElement["h3", {}, {Recurse[]}],

(* Cell["https://www.wolfram.com/wolfram-u/introduction-to-cryptography/", "CourseLink", "Description" -> "Introduction to Cryptography", "NodeID" -> 51] *)
{Cell, "CourseLink"} :> Module[{link, description},
  link = First[SelectSelf[]];
  description = "Description" /. Options[SelectSelf[]];
  XMLElement["a", {"class" -> "chevron-after", "href" -> link}, {description}]
],

{Cell, "SeeAllCoursesLink" | "SeeAllCertificatesLink"} :> Module[{link},
  link = "url" /. Options[SelectSelf[]];
  XMLElement["li", {}, {
    XMLElement["a", {"class" -> "chevron-after", "href" -> link}, {Recurse[]}]
  }]
],

{Cell, "SeeAllListWrapper"} :> Sequence[],
{CellGroupData, "SeeAllListWrapper"} :> XMLElement["ul", {}, {Recurse[]}],

{CellGroupData, "LowerCoursesPod"} :>
  XMLElement[
    "div",
    {"id" -> "get-certified"},
    {Recurse[]}
],
{Cell, "LowerCoursesPod"} :> Unevaluated[Sequence[]],

{Cell, "GetCertifiedBoilerplate"} :>
  XMLElement[
    "a",
    {"class" -> "mini-t", "href" -> "https://www.wolfram.com/wolfram-u/certification/"}, {
      XMLElement["div", {"class" -> "img hide__900"}, {
        XMLElement["img", {"src" -> "assets/img/shared/icon-certificate.svg", "width" -> "64", "height" -> "55", "alt" -> ""}, {""}]
      }],
      XMLElement["div", {"class" -> "txt"}, {
        XMLElement["h3", {}, {"Get Certified"}],
        XMLElement["p", {}, {"Showcase your Wolfram Language skills by earning a certification with a free, self-paced course."}]
      }]
    }
],

{Cell, "WolframUBoilerplate"} :>
  XMLElement[
    "a",
    {"href" -> "https://www.wolfram.com/wolfram-u/", "class" -> "wolfram-u"}, {
      XMLElement["img", {"src" -> "assets/img/shared/logo-wolfram-u.png", "width" -> "330", "height" -> "66"}, {""}]
    }
  ],

{CellGroupData, "SubheroSlideImageTop"} :> Module[{slideCount},
  slideCount = Count[SelectSelf[], Cell[_, s_String /; StringContainsQ[s, "SubheroSlideImage"], ___], Infinity];
  XMLElement["div", {"class" -> "hide show__900 slides"}, {
    Recurse[],
    XMLElement["div", {"class" -> "arrows"}, {
      XMLElement["span", {"class" -> "prev"}, {"‹"}], (* unicode name: SINGLE LEFT-POINTING ANGLE QUOTATION MARK *)
      XMLElement["span", {"class" -> "next"}, {"›"}]  (* unicode name: SINGLE RIGHT-POINTING ANGLE QUOTATION MARK *)
    }],
    XMLElement["div", {"class" -> "nav-links show"},
      Table[XMLElement["span", {"class" -> If[i == 1, "nav show", "nav"]}, {""}], {i, slideCount}]
    ]
  }]
],
{CellGroupData, "SubheroSlideImageBottom"} :> Module[{slideCount},
  slideCount = Count[SelectSelf[], Cell[_, s_String /; StringContainsQ[s, "SubheroSlideImage"], ___], Infinity];
  XMLElement["div", {"class" -> "feature-right hide__900 slides"}, {
    Recurse[],
    XMLElement["div", {"class" -> "arrows"}, {
      XMLElement["span", {"class" -> "prev"}, {"‹"}], (* unicode name: SINGLE LEFT-POINTING ANGLE QUOTATION MARK *)
      XMLElement["span", {"class" -> "next"}, {"›"}]  (* unicode name: SINGLE RIGHT-POINTING ANGLE QUOTATION MARK *)
    }],
    XMLElement["div", {"class" -> "nav-links show"},
      Table[XMLElement["span", {"class" -> If[i == 1, "nav show", "nav"]}, {""}], {i, slideCount}]
    ]
  }]
],

{Cell, "SubheroSlideImageTop" | "SubheroSlideImageBottom"} :> Module[{
  type, count, image, imagefile
},
  dprint["Entering SubheroSlideImageTop|Bottom transform, SelectSelf[] = "]; dfile[SelectSelf[] /. _NumericArray -> xxxNumericArray];
  dprint["Entering SubheroSlideImageTop|Bottom transform, SelectChildren[] = "]; dfile[SelectChildren[] /. _NumericArray -> xxxNumericArray];

(*
Cell[
  BoxData[xxxGraphicsBox[]],
  "SubheroSlideImageTop",
  "SourceImageFilename" -> "/Users/billw/Dropbox/homedir/git/featuresetpages/English/zReference/sin-x-sin-2x.png",
  "DisplayDurations" -> {},
  "SectionCounter" -> 2,
  "SubheroImageCounter" -> 1,
  "SectionTitle" -> "image-carousel",
  "NodeID" -> 23
]
*)

  type = SelectSelf[][[2]]; (* the style of the cell *)
  dprint["type = "<>ToString[type, InputForm]];
  count = ToString["SubheroImageCounter" /. Options[SelectSelf[]]];
  dprint["count = "<>ToString[count, InputForm]];
  image = ToExpression[First @ SelectChildren[]];
  dprint["Head[image] = "<>ToString[Head[image], InputForm]];
  outputImageFilename = FileNameJoin[{"Files", GetBasenameLa[], StringReplace[FileNameTake["SourceImageFilename" /. Options[SelectSelf[]]], ".png" -> ".webp"]}];
  dprint["outputImageFilename = "<>ToString[outputImageFilename, InputForm]];

  XMLElement["div", {"id" -> "slide"<>count<>If[type == "SubheroSlideImageTop", "a", "b"], "class" -> "slide"<>If[count == "1", " show", ""]}, {
    XMLElement["img", {
      "src" -> If[Not @ FileExistsQ[outputImageFilename], Export[outputImageFilename, image], outputImageFilename],
      "alt" -> ""
    }, {""}]
  }]
],

(* documentation section *)

{Cell, "DocSection"} :> Sequence[],
{CellGroupData, "DocSection"} :> XMLElement["section", {"id" -> "reference"}, {Recurse[]}],

{Cell, "DocColorBars"} :>
  XMLElement["div", {"id" -> "ref-bars"}, {
    XMLElement["div", {"class" -> "red"},    {XMLElement["div", {}, {" "}], XMLElement["div", {}, {" "}], XMLElement["div", {}, {" "}], XMLElement["div", {}, {" "}]}],
    XMLElement["div", {"class" -> "orange"}, {XMLElement["div", {}, {" "}], XMLElement["div", {}, {" "}], XMLElement["div", {}, {" "}], XMLElement["div", {}, {" "}]}],
    XMLElement["div", {"class" -> "green"},  {XMLElement["div", {}, {" "}], XMLElement["div", {}, {" "}], XMLElement["div", {}, {" "}], XMLElement["div", {}, {" "}]}],
    XMLElement["div", {"class" -> "blue"},   {XMLElement["div", {}, {" "}], XMLElement["div", {}, {" "}], XMLElement["div", {}, {" "}], XMLElement["div", {}, {" "}]}]
  }],

{CellGroupData, "DocFunctions"} :> XMLElement["div", {"id" -> "ref-words"}, {Recurse[]}],
{Cell, "DocFunctions"} :> XMLElement["span", {}, {Recurse[]}],

{Cell, "DocContent"} :> Sequence[],
{CellGroupData, "DocContent"} :>
  XMLElement["div", {"id" -> "reference-content"}, {
    XMLElement["img", {
      "src" -> "assets/img/shared/icon-wolf-solid.svg",
      "width" -> "135",
      "height" -> "135",
      "alt" -> ""
    }, {""}],
    XMLElement["h2", {}, {"Wolfram ", GetParameter["titleName"], XML`RawXML["&nbsp;"], "Documentation"}],
    XMLElement["p", {}, {
      "Thousands of working examples, tutorials, guides and workflows"
    }],
    XMLElement["div", {"id" -> "reference-grid"}, {Recurse[]}]
  }],

(*
SelectSelf[] = Cell[
  "",
  "DocScreenshot",
  "caption" -> "Image processing & Analysis Guide",
  "url" -> "http://reference.wolfram.com/language/guide/ImageProcessing.html",
  "shotfile" -> "assets/img/pages.en/system/guide/ImageProcessing.png",
  "NodeID" -> 304
]*)

{Cell, "DocScreenshot"} :> Module[{shotfile, caption, url},
  dprint["entering DocScreenshot, SelectSelf[] = "<>ToString[SelectSelf[] /. _Image -> xxxImage, InputForm]];

  shotfile = "shotfile" /. Options[SelectSelf[]];
  caption  = "caption" /. Options[SelectSelf[]];
  url      = "url" /. Options[SelectSelf[]];

  dprint["shotfile = "<>ToString[shotfile, InputForm]];
  dprint["caption = "<>ToString[caption, InputForm]];
  dprint["url = "<>ToString[url, InputForm]];

  res = XMLElement["a", {"href" -> url}, {
    XMLElement["img", {"src" -> shotfile, "width" -> "262", "height" -> "200", "alt" -> ""}, {""}],
    XML`RawXML["<br>"],
    XMLElement["span", {"class" -> "chevron-after"}, {Transmogrify[caption]}]
  }];
  dprint[" res = "<>ToString[res, InputForm]];
  res
],

(* *** endmatter *)

(* all above this should be followed by a comma *)
{Cell, "xxxendxxx"} :> Sequence[]
},
IncludeXMLTransforms -> {{"HTML","Components","Includes.m"}},
DefaultParameters -> { "newline2br" -> False, "newline2div" -> False}

]