(* :Title: Functions.m *)

(* :Author:
  Andrew Hunt, andy@wolfram.com
  Greg Martel, gregm@wolfram.com
*)

(* :Mathematica Version: 5 *)
(* :Package Version: $Revision: 1.117 $ $Date: 2018/05/17 16:41:54 $ *)                  
(* :Copyright: (c) 2005, Wolfram Research, Inc. All rights reserved. *)
(* :Summary: Add-ons specific rules and general transform for Mathematica Documentation *)

(* :Title: Functions.m *)

XMLTransform[{

(* * other Functions.m stuff *)

(* Remove in-product decorations *)

{Cell, "NotebookImage"} :> 
DIV[{"class"->"NotebookImage"}, {
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
    {"alt"->"", "title"->""},
    Inline -> True,
    "RetinaImage" -> True
  ]
}],
(*{Cell, "NotebookImage"} :> 
Span[{"class"->"NotebookImage"}, {
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
    Inline -> True,
    "RetinaImage" -> True
  ]
}],*)

{Cell, "PackageHeader"} :> Unevaluated[Sequence[]],

{Cell, "ObjectNameGrid"} :>  Block[{objgrid = True}, Recurse[] ],
  
{Cell, "PlatformIconGrid"} :> Recurse[],

{StyleBox, "EntityRelatedInterpreter"} :> Recurse[],

{Cell, "EntityRelatedInterpreter"} :> Recurse[],

{Cell, "ObjectName" | "ObjectNameSmall" | "ObjectNameAlt" | "CFunctionName"} :> Module[
  {
    uri, context, cell, result, relatedInterpreterCell
  },
  dprint["Functions.m: entering ObjectName and friends"];
  dprint["  where SelectSelf[] is"]; dfile[SelectSelf[]];

  cell = SelectSelf[];
  {uri, context} = {GetParameter["taggingRules"]["Metadata"]["uri"], GetParameter["taggingRules"]["Metadata"]["context"]};
  dprint["Functions.m ObjectNameAlt: {uri, context} = "<>ToString[{uri, context}, InputForm]];

  relatedInterpreterCell = Transmogrify[Quiet @ Check[First[Cases[cell, Rule["RelatedInterpreterCell", c:_] -> c, Infinity]], {}]];
  cell = DeleteCases[cell, Rule["RelatedInterpreterCell", _]];

  dprint["relatedInterpreterCell = "<>ToString[relatedInterpreterCell, InputForm]];
  dprint["cell = "<>ToString[cell, InputForm]];

  dprint["Functions.m: ObjectName* context = "<>ToString[context, InputForm]];
  dprint["Functions.m: ObjectName* uri = "<>ToString[uri, InputForm]];

  result = If[
    StringContainsQ[uri, StartOfString ~~ CharacterRange["A", "Z"]] (* && context =!= "CloudConnectorForExcel`" *)
  ,
    (* for packages, provide a place for the ContextName` above the main title *)
    Unevaluated[Sequence[
      XMLElement["span", {"class" -> "main-title-top"}, {context}]
    , XMLElement["h1", {"class" -> "main-title"}, {Recurse[]}]
    , XML`RawXML["<div class=\"iconography\" data-src=\"s=SICONSTATUS d=DICONSTATUS m=MICONSTATUS c=CICONSTATUS u=UICONSTATUS l= n=NICONSTATUS e=EICONSTATUS\"></div>"]
    , relatedInterpreterCell
    ]]
  ,
    If[
      MemberQ[{
        "Embedding Format"
      , "File"
      , "Front End Object"
      , "Interpreter"
      , "MathLink C Function"
      , "Method"
      }, GetParameter["entityType"]]
    , XMLElement["div", {"class" -> "main-heading"}, {
        XMLElement["h1", {"class" -> "main-title"}, {Recurse[]}]
      , XML`RawXML["<div class=\"iconography\" data-src=\"s=SICONSTATUS d=DICONSTATUS m=MICONSTATUS c=CICONSTATUS u=UICONSTATUS l= n=NICONSTATUS e=EICONSTATUS\"></div>"]
      , relatedInterpreterCell
      }]
    , Unevaluated[Sequence[
        XMLElement["h1", {"class" -> "main-title"}, {Recurse[]}]
      , XML`RawXML["<div class=\"iconography\" data-src=\"s=SICONSTATUS d=DICONSTATUS m=MICONSTATUS c=CICONSTATUS u=UICONSTATUS l= n=NICONSTATUS e=EICONSTATUS\"></div>"]
      , relatedInterpreterCell
      ]]
    ]
  ];

(* I may need to munge res here to wrap Title quotes in their own spans *)
  dprint["Functions.m: exiting ObjectName transform, {result} = "<>ToString[{result}, InputForm]];
  result
],

{StyleBox, "FullListing"} :> XMLElement["span", {"class" -> "FullListing"}, {Recurse[]}],

{CellGroupData, "IndicatorUsage"} :> {
  Transmogrify[First[SelectChildren[]]],
  XMLElement["ul", {"class"->"functionList afterIntro"}, {Transmogrify[Rest[SelectChildren[]]]}]
},  

{Cell, "IndicatorUsage"} :>  Module[{},
  XMLElement["div", {"class"->"indicatorIntroWrap"}, {
    XMLElement["div", {"class"->"indicatorIntro"}, {Recurse[]}]
  }]
],
	
(* a scratch cell for debugging. use as Cell["point 1", "gronk"] or similar *)
{Cell, "gronk"} :> Module[{},
  Print[ToString[InputForm[SelectSelf[]]]];
  XMLElement["div", {"class" -> "gronk"}, {Recurse[]}]
],

{CellGroupData, "DeviceAbstract"} :>
  XMLElement["section", {}, {
    XMLElement["div", {"class" -> "inner"}, {
      Recurse[]
    }]
  }],
{Cell, "DeviceAbstract"} :> P[{"class" -> "intro"}, {Recurse[]}],

(* these two are copied from "GuideFunctionsSubsection" in Services.m - these need
to look like a GuideFunctionsSubsection in ref/service/Reuters
https://reference.devel.wolfram.com/language/ref/service/Reuters.html *)

{StyleBox, "CompiledTypeSubsection"} :> Recurse[],
{Cell, "CompiledTypeSubsection"} :>
Module[{flag = GetParameter["flag"]},
  If[flag =!= "RootGuidePage",
    H3[If[HasOption[CellTags],
		{"id"->StringReplace[ToString[Flatten[{GetOption[CellTags]}]], {"{"->"", "}"->""}]}, {}], {Recurse[]} ],
    DIV[{},{}]
  ]
],

(* HowTo styles *)
{Cell, "HowToTitle"} :> XMLElement["div", {"class" -> "main-heading"}, Flatten[{XMLElement["h1", {"class" -> "main-title"}, {Recurse[]}]}]],

{Cell, "HowToTitlePrefix"} :> Unevaluated @ Sequence[XMLElement["span", {}, {Recurse[]}], XMLElement["span", {"class"->"titleSep"}, {XML`RawXML["|"]}]],

{CellGroupData, "HowToAbstract"} :> XMLElement["section", {}, {XMLElement["div", {"class" -> "inner"}, {Recurse[]}]}],
{Cell, "HowToAbstract"} :> P[{"class" -> "intro"}, {Recurse[]}],

{CellGroupData, "HowToDelimiter"} :> XMLElement["section", {}, {XMLElement["div", {"class" -> "inner"}, {Recurse[]}]}],
{Cell, "HowToDelimiter"} :> XMLElement["div", {"class"->"HowToDelimiter"}, {Recurse[]}],

{CellGroupData, "HowToSubsection"} :>
  XMLElement["section", {}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h3", {"class" -> "SubsectionHowTo"}, {Recurse[]}]
    }]
  }],
{CellGroupData, "Subsection"} :>
  XMLElement["section", {}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h2", {"class" -> "Section"}, {Recurse[]}]
    }]
  }],
{CellGroupData, "HowToSubsubsection"} :>
  XMLElement["section", {}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h3", {"class" -> "SubsubsectionHowTo"}, {Recurse[]}]
    }]
  }],

{CellGroupData, "HowToScreencastLink"} :> XMLElement["section", {}, {XMLElement["div", {"class" -> "inner"}, {Recurse[]}]}],
{Cell, "HowToScreencastLink"} :> Module[
  {iconText,iconSource,bdata,h,w}
,
  bdata = First[Cases[SelectSelf[], (ButtonData->{URL[bbdd_], _}) :> bbdd, Infinity, 1]];
  {w,h} = First[Cases[SelectSelf[], Rule[ImageSizeRaw,{width_, height_}] :> {ToString[width], ToString[height]}, Infinity, 1]];
  If[
    StringMatchQ[bdata, __~~"vimeo"~~__]
  ,
    XMLElement["div", {"class"->"embedWrap"}, {
      XMLElement[
        "iframe",
        {"src"->"//player.vimeo.com/video/"<>Last[StringSplit[bdata, "/"]], "width"->w, "height"->h, "frameborder"->"0", "webkitallowfullscreen"->"","mozallowfullscreen"->"","allowfullscreen"->""},
        {""}
      ]
    }]
  ,
    XMLElement["div", {"class"->"embedWrap"}, {
      XML`RawXML["<!--#include virtual=\"/common/video/includes/embed.php?v="<>StringReplace[StringCases[bdata,"v=" ~~ DigitCharacter ..], "v="->""]<>"&share=false\"-->"],
      XMLElement["div", {"id"->"videoplayer"}, {XML`RawXML[""]}]
    }]
  ]
],

{Cell, "DeviceSubtitle" | "InterpreterSubtitle" | "FilenameExtension" | "ServiceSubtitle" | "DataStructureSubtitle"} :>
  XMLElement["span", {"class"->"main-heading-type"}, {Recurse[]}],

{StyleBox, "FilenameExtension"} :>  XMLElement["span", {"class"->"main-heading-type"}, {Recurse[]}],
{Cell, "InterpreterTitleQuotes"} :> XMLElement["span", {"class" -> "quote"}, {Recurse[]}],

{CellGroupData, "DeviceList"} :> XMLElement["ul", {"class"->"DeviceList"}, {Recurse[]}],
{Cell, "DeviceList"} :> Sequence[],

{CellGroupData, "DeviceListA"} :>
  XMLElement["section", {}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["ul", {"class"->"functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "DeviceListA"} :> Sequence[],

{CellGroupData, "FormatList"} :> XMLElement["ul", {"class"->"FormatList"}, {Recurse[]}],

{Cell, "FormatList"} :> Sequence[],

{Cell, "ScreencastPointerText"} :> P[{"class" -> GetStyle[]}, {Recurse[]}],
(* end howto styles *)
{Cell, "HowToText"} :> P[{"class" -> GetStyle[]}, {Recurse[]}],
  
{Cell, "AwaitingReviewNote"} :>  XMLElement["div", {"class"->"AwaitingReviewNote"}, {Recurse[]}],


{Cell, "CharacterImage"}:> 
Block[{CharacterImageData, CharacterImageDataLeft, CharacterImageDataRight},
CharacterImageData = SelectSelf[];
CharacterImageDataLeft={Transmogrify[Cell[BoxData[CharacterImageData[[1, 1, 1, 1, 1]]], "CharacterImageLeftSide"]]};
CharacterImageDataRight={Transmogrify[Cases[CharacterImageData[[1, 1, 1, 1, 2]], Cell[___, "CharacterNotes", ___], Infinity]]};
XMLElement["div", {"class"->"functionIntroWrap"}, {
	XMLElement["div", {"class"->"functionIntro"}, 
	{XMLElement["table", {}, {XMLElement["tr", {}, Flatten[{XMLElement["td", {"class"->"character-img"}, Flatten[{CharacterImageDataLeft}]], 
	XMLElement["td", {}, {XMLElement["ul", {"class"->"functionList afterIntro"}, Flatten[{CharacterImageDataRight}]]}]}]]}]}]}]
 ],   


{Cell, "CharacterImageLeftSide"}:>
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, "characterimage_"<>IncrementCounter["CharacterImage"] <> ".gif"],
    {"alt"->"", "title"->""},
    Inline -> True,
    "RetinaImage" -> True
  ],
  
{Cell, "CharacterNotes"}:> XMLElement["li", {}, {Recurse[]}],

{CellGroupData, "CharacterName"}:> Module[{},

  XMLElement["div", {"class" -> "main-heading"}, {
    XMLElement["h1", {"class" -> "main-title"}, {Transmogrify[SelectChildren[][[1, 1]]]}]
  , XML`RawXML["<div class=\"iconography\" data-src=\"s=SICONSTATUS d=DICONSTATUS m=MICONSTATUS c=CICONSTATUS u=UICONSTATUS l= n=NICONSTATUS e=EICONSTATUS\"></div>"]
  }]
],
{Cell, "CharacterName"} :> Unevaluated[Sequence[]],

{Cell, "1ColumnTableMod"
  | "2ColumnTable"
  | "3ColumnTable"
  | "2ColumnTableMod"
  | "3ColumnTableMod"
  | "3ColumnIllustratedTableMod"} :> Li[{"class"->"NotesTable"}, {DocumentationBuild`Utils`AddSpans[Recurse[]]}], 


{Cell,  "AlphabeticalListingTable"} :>
P[ {"class" -> "MathTable", "name" -> (GetStyle[]/.None->"FIXME")}, 
  {Recurse[]}
], 


(***** Cell Definitions *****)

(*
the official structure of one section of a Usage cell:

 - code
 - explicit linebreak
 - description, possibly with more explicit linebreaks

and a Usage cell consists of any number of these sections.

Also, the third argument of an XMLElement is a list that may contain only a
string, an XMLElement, an XMLObject, or any combination of those.  Hence the use
of Flatten here and there below since the argument may not contain sublists.
*)

{Cell, "Usage" | "DeviceDoubleUsage" | "CFunctionUsage"} :> Module[
  {usageTemplateList = {}, usageContent, usageCount = 1, exportUsageQ, title}
,
  If[
    (* only refpages in the top level of the maindoc ref dir have usage messages.  none outside maindoc ref and none under it *)
    StringMatchQ[GetParameter["url"], StartOfString ~~ "ref/" ~~ Except["/"].. ~~ EndOfString]
  ,
    title = GetParameter["taggingRules"]["Metadata"]["title"];
    usageTemplateList = DocumentationBuild`TemplateTools`GenerateFunctionTemplate[title];
    (* InlineFormula style prompts the fe to query the user with a dialog box.
       Input allows the fe to silently accept the expression. *)
    usageTemplateList = usageTemplateList //. Cell[first_, "InlineFormula", rest:___] -> Cell[first, "Input", rest];
    exportUsageQ = TrueQ[Positive @ Length[usageTemplateList]]
  ,
    exportUsageQ = False
  ];

  usageDir = FileNameJoin[{"Files", GetBasenameLa[]}];

	Quiet[
		CreateDirectory[usageDir],
		(* Note: `filex` is emitted in WL 12.2-, eexist in 12.3+. *)
		{CreateDirectory::filex, CreateDirectory::eexist}
	];

  usageContent := XMLElement["div", {"class" -> "functionIntroWrap"},
    Block[
      {usagegrid = True},

      Module[
        {
          xml = {Recurse[]}, pos, repl, first = True,
          versionNumber, modinfobackground, modinfoposition
        },

(* dprint["Functions.m:Usage transform: initial transmogrified xml expr = "]; dfile[xml]; *)
        versionNumber = StringSplit[$Version, "."][[1]];
        pos = Position[xml, XMLElement["tr", _, _], Infinity];

        modinfobackground = Cases[GetOption[GridBoxOptions], Rule["Rows", a__] :> a, Infinity];
        modinfobackground = Map[MemberQ[{#}, Dynamic[__]]&, Flatten[modinfobackground]];
        modinfoposition = Pick[pos, modinfobackground];

        (* map this over 'pos', a list returned by Position of instances of 'tr'
        in Recurse[<usage expr>].  So the position of each top-level section of
        the Usage cell. *)
        repl = Map[
          Module[
            {
              tr, attr, children, modinfo
            },

            tr = Extract[xml, #]; (* # is a Position of a tr in the transmogrified Usage cell *)

            (* deal with modinfo and attributes *)
            modinfo = If[MemberQ[modinfoposition, #], True, False];
            attr = DeleteCases[tr[[2]], _["name"|"id", _]];
            (* take care of the 'first' switch; also, it seems that only the first tr should have a "class" attribute: *)
            If[first, first = False, attr = DeleteCases[attr, _["class", _]]];

            XMLElement[
              "div",
              {"class" -> "functionIntro"<>If[modinfo, " modified-text", ""]},
              Flatten @ Map[
                (* map this Function over the td's in this table row, but skip this tr's first td by way of Rest *)
                Function[
                  {td},
                  Module[{attr, usageTemplate, usageFile, exportRes},
                    attr = td[[2]];
                    attr = If[MatchQ[attr, {___, "class" -> _, ___}] , ReplaceAll[attr, _["class", _] -> ("class" -> "UM")], attr];
                    attr = Join[
                      DeleteCases[attr, _["name"|"id", _]],
                      If[modinfo, {"name" -> "ModInfo_"<>versionNumber, "id" -> "ModInfo_"<>versionNumber}, {}]
                    ];

                    td[[3]] /. {first:___, " "..., XMLElement["br", __], rest:___} :> {
                      If[
                        exportUsageQ
                      ,
                        usageTemplate = Part[usageTemplateList, usageCount];
                        usageFile = FileNameJoin[{"Files", GetBasenameLa[], "usage_"<>ToString[usageCount++]<>".txt"}];
                        exportRes = Export[usageFile, usageTemplate];
                        If[exportRes =!= usageFile, Global`AntLog["ERROR: usage file "<>usageFile<>" not written"]];
                        XMLElement["p", {"class" -> "code clipboard-inline", "data-code" -> usageFile}, Flatten[{first}]]
                      ,
                        XMLElement["p", {"class" -> "code"}, Flatten[{first}]]
                      ],
                      XMLElement["p", {"class" -> "code-description"}, Flatten[{rest}]]
                    }
                  ]
                ], (* map Function over following: *)
                Rest[tr[[3]]]
              ]
            ]
          ]&,
          pos
        ];
        repl
      ]
    ]
  ];

usageContent
], 




{CellGroupData, "UsageSection" | 
	"LegacyMaterialSection" | 
	"ApplicationNotesSection" | 
	"ExtensionsSection" | 
	"DesignDiscussionSection"  | 
	"TutorialMoreAboutSection" | 
	"TutorialCollectionSection"} :> 
Recurse[],  

{CellGroupData, "SeeAlsoSection"} :>
  XMLElement["section", {"id" -> "SeeAlso", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["SeeAlsoSectionText", GetParameter["language"]]}], 
      Recurse[]
    }]
  }], 
{Cell, "SeeAlso"} :> XMLElement["p", {"class"->"inlineFunctionList"}, {Recurse[]}],

{CellGroupData, "TutorialsSection"} :> 
  XMLElement["section", {"id" -> "Tutorials", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["TutorialsSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }], 
{Cell, "Tutorials"} :> Li[{}, {Recurse[]}],

{CellGroupData, "MoreAboutSection"} :>
  XMLElement["section", {"id" -> "RelatedGuides", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["MoreAboutSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }], 
{Cell, "MoreAbout"} :> Li[{}, {Recurse[]}],

{CellGroupData, "RelatedWorkflowsSection"} :>
  XMLElement["section", {"id" -> "RelatedWorkflows", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedWorkflowsSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }], 
{Cell, "RelatedWorkflows"} :> Li[{}, {Recurse[]}],

{CellGroupData, "RelatedLinksSection"} :>
  XMLElement["section", {"id" -> "RelatedLinks", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedLinksSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "RelatedLinks"} :> Module[{class},
  (* if an item has no RelatedLinkStyle, then don't give it a class *)
  class = Quiet@Check[First[Cases[SelectSelf[], Rule["RelatedLinkStyle", s_String] :> {Rule["class", s]}, Infinity]], {}];
  XMLElement["li", class, {Recurse[]}]
],

{CellGroupData, "RelatedDecoderSection"} :>
  XMLElement["section", {"id" -> "RelatedDecoderSection", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedDecoderSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "RelatedDecoderSection"} :> Unevaluated[Sequence[]],
{Cell, "RelatedDecoders"} :> Li[{}, {Recurse[]}],

{CellGroupData, "RelatedEncoderSection"} :>
  XMLElement["section", {"id" -> "RelatedEncoderSection", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedEncoderSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "RelatedEncoderSection"} :> Unevaluated[Sequence[]],
{Cell, "RelatedEncoders"} :> Li[{}, {Recurse[]}],

{CellGroupData, "RelatedEntitySection"} :>
  XMLElement["section", {"id" -> "RelatedEntity", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedEntitySectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "RelatedEntitySection"} :> Unevaluated[Sequence[]],

{CellGroupData, "RelatedTutorialsSection"} :>
  XMLElement["section", {"id" -> "RelatedTutorials", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedTutorialsSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],

{CellGroupData, "TutorialRelatedLinksSection"} :> 
  XMLElement["section", {"id" -> "RelatedLinks", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["TutorialRelatedLinksSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
			
{CellGroupData, "NotesSection" } :> Module[{nstitle, toggle},
  (* Print["entering NotesSection transform; First[SelectChildren[]] = "<>ToString[InputForm[First[SelectChildren[]]]]]; *)
  nstitle = Flatten[{Transmogrify[First[SelectChildren[]]]}];
  toggle = If[MemberQ[{"Interpreter", "MathLink C Function"}, GetParameter["entityType"]], "toggle open", "toggle"];
  XMLElement["section", {"id" -> "DetailsAndOptions"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> toggle}, nstitle],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, Flatten[{Transmogrify[Rest[SelectChildren[]]]}]]
      }]
    }]
  }]
],

{Cell, "NotesSection"} :> Recurse[],
{Cell, "NotesThumbnailsSection"} :> Recurse[],

{Cell, "NotesThumbnail"} :> (
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, "details_" <> IncrementCounter["details"] <> ".png"],
    {"alt" -> ""},
    Inline -> True,
    MaxImageSize->200,
    CropImage -> False,
    ImageFormat -> "PNG",
    "RetinaImage" -> False,
    "Debug" -> False
  ]
),

{CellGroupData, "NotesThumbnailsSection" } :> Module[
  {children, nstitle, highlightableSectionQ, originalCellStyle}
,
  dprint["Functions.m: entering {CellGroupData, \"NotesThumbnailsSection\" }"];
  dprint["Functions.m: NotesThumbnailsSection: SelectSelf[] = "]; dfile[SelectSelf[]];
  children = SelectChildren[];
  dprint["Functions.m: NotesThumbnailsSection: children = "]; dfile[children];
  nstitle = Flatten[{Transmogrify[children[[1]]]}];
  dprint["Functions.m: NotesThumbnailsSection: nstitle = "<>ToString[nstitle, InputForm]];
  thumbs = Transmogrify[{children[[2]]}] /. Null -> Sequence[];
  dprint["Functions.m: NotesThumbnailsSection: thumbs = "]; dfile[thumbs];

  highlightableSectionQ = Or[
    Positive @ Count[SelectSelf[], Cell[__, "BrightFlag", ___], Infinity]
  , Positive @ Count[SelectSelf[], Cell[_, _String, Rule[Background, d:_Dynamic /; Positive @ Count[d, "ModificationHighlight", Infinity]], ___], Infinity]
  ];
  dprint["NotesThumbnailsSection: highlightableSectionQ = "<>ToString[highlightableSectionQ, InputForm]];

  (* originalCellStyle = First @ Cases[SelectSelf[], Rule["OriginalCellStyle", s:_String] -> s, Infinity]; *)
  (* dprint["NotesThumbnailsSection: originalCellStyle = "<>ToString[originalCellStyle, InputForm]]; *)

  XMLElement["section", If[highlightableSectionQ, {"id" -> "DetailsAndOptions", "class" -> "HighlightableSection"}, {"id" -> "DetailsAndOptions"}], {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, nstitle],
      thumbs,
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, Flatten[{Transmogrify[children[[3;;]]]}]]
      }]
    }]
  }]
],

{CellGroupData, "NotesThumbnails"} :> (
  XMLElement["div", {"class" -> "NotesThumbnails"}, {Recurse[]}]
),

{Cell, "NotesThumbnails"} :> Unevaluated[Sequence[]],

{CellGroupData, "FunctionEssaySection" } :> 
	XMLElement["section", {"id"->"FunctionEssay"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, {Localized["FunctionEssaySectionText", GetParameter["language"]]}],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, Flatten[{Transmogrify[Rest[SelectChildren[]]]}]]
      }]
    }]
  }],

{CellGroupData, "FormatBackground" } :> 
	XMLElement["section", {"id"->"FormatBackground"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, {Localized["FormatBackgroundSectionText", GetParameter["language"]]}],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, Flatten[{Transmogrify[Rest[SelectChildren[]]]}]]
      }]
    }]
  }],

{CellGroupData, "EmbeddingFormatSection" } :> Module[{efstitle},
  efstitle = Flatten[{Transmogrify[First[First[First[SelectChildren[]]]]]}];
	XMLElement["section", {"id"->"EmbeddingFormatSection"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, efstitle],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, Flatten[{Transmogrify[Rest[SelectChildren[]]]}]]
      }]
    }]
  }]
],

{CellGroupData, "QuestionInterfaceSection" } :> Module[{efstitle},
  efstitle = Flatten[{Transmogrify[First[First[First[SelectChildren[]]]]]}];
	XMLElement["section", {"id"->"QuestionInterfaceSection"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, efstitle],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, Flatten[{Transmogrify[Rest[SelectChildren[]]]}]]
      }]
    }]
  }]
],

{CellGroupData, "ImportExportSection" } :>
	XMLElement["section", {"id"->"ImportAndExport"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, {Localized["ImportExportSectionText", GetParameter["language"]]}],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
      }]
    }]
  }],

{CellGroupData, "ElementsSection" } :> 
	XMLElement["section", {"id"->"Elements"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, {Localized["ElementsSectionText", GetParameter["language"]]}],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
      }]
    }]
  }],

{CellGroupData, "ExportElementsSection" } :>
	XMLElement["section", {"id"->"Elements"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, {Localized["ExportElementsSectionText", GetParameter["language"]]}],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
      }]
    }]
  }],

{CellGroupData, "OptionsSection" } :>
	XMLElement["section", {"id"->"Options"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, {Localized["OptionsSectionText", GetParameter["language"]]}],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
      }]
    }]
  }],
			
{CellGroupData, "NotebookInterfaceSection" } :>
	XMLElement["section", {"id"->"NotebookInterface"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, {Localized["NotebookInterfaceSectionText", GetParameter["language"]]}],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
      }]
    }]
  }],

{CellGroupData, "DeviceSubsection" | "EntitySection"} :> Module[{dstitle},
  dstitle = Quiet @ Check[First[Flatten[{Transmogrify[First[First[SelectChildren[]]]]}]], ""];
	XMLElement["section", {"id"->"Elements"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" ->
        If[
          MemberQ[{"Sample Entities", "Sample Entity Classes"}, dstitle]
        , dprint["toggle open"]; "toggle open"
        , dprint["toggle"]; "toggle"
        ]
      }, dstitle],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
      }]
    }]
  }]
],

{CellGroupData, "RelatedInterpreterSection" } :> Module[{dstitle},
  dstitle = Flatten[{Transmogrify[First[First[SelectChildren[]]]]}];
  XMLElement["section", {"id" -> "RelatedInterpreter", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, dstitle], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }]
], 

{CellGroupData,
  "InterpreterSection"
| "MethodSection"
| "ExternalEvaluateSystemSection"
| "CompiledTypeSection"
| "BlockchainSection"
| "DataStructureSection"
| "DatabaseConnectionSection"
| "ClassifierSection"
| "TextContentSection"
| "PredictorSection"
| "ResourceObjectSection"
} :> Module[{dstitle, toggle},
  (* Print["entering InterpreterSection, SelectSelf[] = "<>ToString[InputForm[SelectSelf[]]]]; *)
  dstitle = Flatten[{Transmogrify[First[First[SelectChildren[]]]]}];
  toggle = If[MemberQ[{"Interpreter"}, GetParameter["entityType"]], "toggle open", "toggle"];
	XMLElement["section", {"id"->"InterpreterSection"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> toggle}, dstitle],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, Flatten[{Transmogrify[Rest[SelectChildren[]]]}]]
      }]
    }]
  }]
],

{CellGroupData, "RelatedDemonstrationsSection"} :> Module[{dstitle},
  dstitle = Flatten[{Transmogrify[First[First[SelectChildren[]]]]}];
  XMLElement["section", {"id" -> "RelatedDemonstrationsSection", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, dstitle], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }]
], 

{Cell, "UsageSection" | "LegacyMaterialSection" | "ApplicationNotesSection" | "DesignDiscussionSection" | "ExtensionsSection" 
} :> 
Module[{name, style, ex, versionNumber},
versionNumber = StringSplit[$Version, "."][[1]];
style = GetStyle[];
Which[ style === "ElementsSection", name = "ELMS",  
style === "ExtensionsSection", name = "EXTS",  
True, name = StringJoin@Flatten@{If[First@ToCharacterCode[#] < 91, #, ""] & /@ Characters[ GetStyle[] ]};
];
ex = If[MemberQ[
  {"NotesSection", "ImportExportSection", "NotebookInterfaceSection", "ElementsSection", "DeviceSubsection", "OptionsSection"},
  style], "closed", "open"];
A[{"name" -> GetStyle[], "id"->"hello" }, {
  DIV[ 
  	(*If[name==="NS",{"class" -> name<>"H", "id"->"ModInfo_"<>versionNumber, "name"->"ModInfo_"<>versionNumber},{"class" -> name<>"H" }]*)
  	{"class" -> name<>"H"}
  	, {
    A[{"href" -> ("javascript:toggle('" <> name <> "')"), "onmouseover" -> "javascript:return(true);" }, {
    Img[{"src"->(GetParameter["htmlBase"]<>"/mathematicaImages/"<>ex<>"SectionGroup.gif"), "id"->name<>"_", "border"->"0", "width"->"8", "height"->"8", "alt"->""}], 
      " ",
      Span[{"class"->name<>"Span"}, { Recurse[] }]
    }]
  }]
}] ],

{Cell,
  "RelatedInterpreterSection"
| "InterpreterSection"
| "MethodSection"
| "ExternalEvaluateSystemSection"
| "EmbeddingFormatSection"
| "DeviceSubsection"
| "MoreAboutSection"
| "SeeAlsoSection"
| "TutorialsSection"
| "RelatedLinksSection"
| "RelatedTutorialsSection"
| "TutorialRelatedLinksSection"
| "TutorialMoreAboutSection"
| "RelatedDemonstrationsSection"
| "TutorialCollectionSection"
| "ElementsSection"
| "ImportExportSection"
| "NotebookInterfaceSection"
| "OptionsSection"
| "EntitySection"
| "CompiledTypeSection"
| "BlockchainSection"
| "DatabaseConnectionSection"
| "ClassifierSection"
| "PredictorSection"
} :> Unevaluated[Sequence[]],

(* for some reason <p class="ExampleDelimiter" /> doesn't bode well with browsers.
   therefore, I have added a " " *)
{Cell, "ExampleDelimiter"} :> 
DIV[{"class" -> "ExLine"}, {XML`RawXML["<!-- -->"]}],

{Cell, "ExampleDelimiterInput"} :> 
DIV[{"class" -> "ExLineI"}, {XML`RawXML["<!-- -->"]}],

{Cell, "TutorialMoreAbout" | "RelatedDemonstrations" | "RelatedLinks" | "LegacyMaterial" | 
"ApplicationNotes" | "DesignDiscussion" | "Extensions"} :> 
  Li[{}, {Recurse[]}],
  
{Cell, "TutorialCollection"} :> Unevaluated[Sequence[]],
(* Scale picture smaller *)
(* TODO find more suitable alt & title tags for GraphicsData and RawData *)
{Cell, "Picture"} :> (
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, "Image_" <> IncrementCounter["Image"] <> ".gif"],
    {"class"->If[GetParameter["entityType"]==="HowTo","HowToPicture",""], "alt" -> ""},
    Inline -> False,
    ImageFormat -> "GIF",
    MaxImageSize->580,
    CropImage -> False,
    "RetinaImage" -> True
  ]
),

{Cell, "FlushInput"} :>
  WithParameters[{"newline2br" -> True},
    XMLElement["p", {"class" -> "StandardForm", "id" -> GetStyle[]}, {
      XMLElement["b", {}, {Recurse[]}]}
      ] ],


(* Change bullets between 'see also' items *)

{StyleBox, "InlineSeparator"} :> 
  XMLElement["span", {"class" -> "inlineSeparator"}, {""}],

{Cell, "RelatedInterpreters"} :> XMLElement["p", {"class"->"inlineFunctionList"}, {Recurse[]}],

{FormBox} :> Module[
  {
    boxChildren,traditionalFormQ,altTag
  },
  boxChildren=SelectChildren[];
  traditionalFormQ = If[GetStyle[] === TraditionalForm, True, False];
  altTag = DocumentationBuild`Utils`CellToString@SelectSelf[];

  If[
    MatchQ[boxChildren, {"Mathematica"}]
  ,
    Transmogrify[StyleBox["Mathematica", FontSlant->"Italic"]]
  ,
    If[
      MatchQ[boxChildren, {StyleBox[_, "KeyEvent", ___]}]
    ,
      Transmogrify[boxChildren]
    ,
      transmogrifyImage[
        ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".png"],
        {"style"->"vertical-align:middle;", "alt"->altTag, "title"->altTag},
        Inline -> True,
        ImageFormat -> "PNG",
        TransparentBackground->True,
        ConversionOptions->{Background->None},
        "RetinaImage" -> True
      ]
    ]
  ]
],

 
(* TODO find more suitable alt & title tags for GraphicsData and RawData *)
{GraphicsData | RawData} :>  
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
  {"alt" -> "", "title" -> ""},
  Inline -> True,
  "RetinaImage" -> True
],


{OutputFormData} :> Transmogrify[SelectChildren[][[2]]],

(* * workflows *)
(* ** WorkflowMainHeadingWrapper *)

{CellGroupData, "WorkflowMainHeadingWrapper"} :>
  XMLElement["div", {"class" -> "main-heading-wrapper"}, {
    XMLElement["div", {"class" -> "inner"}, {
      Recurse[]
    }]
  }],
{Cell, "WorkflowMainHeadingWrapper"} :> Unevaluated[Sequence[]],

(* ** WorkflowTabsList and WorkflowTabItem *)

{CellGroupData, "WorkflowTabsList"} :>
  XMLElement["ul", {"class" -> "workflow-tabs"}, {Recurse[]}],
{Cell, "WorkflowTabsList"} :> Unevaluated[Sequence[]],

{Cell, "WorkflowTabItem"} :> Module[{platform},
  dprint["WorkflowTabItem transform: SelectChildren[] = "<>ToString[SelectChildren[], InputForm]];
  platform = StringTrim @ ToLowerCase[Transmogrify[SelectChildren[]]];
  dprint["WorkflowTabItem transform: platform = "<>ToString[platform, InputForm]];
  XMLElement["li", {}, {
    XMLElement[
      "a", {
        "href" -> "#"<>platform<>"-tab", 
        If[IncrementCounter["WorkflowTabItem"] == "1",
          "class" -> platform<>"-tab selected",
          "class" -> platform<>"-tab"
        ]
      },
      {
        Which[
          StringContainsQ[platform, "desktop"],
          Localized["WorkflowDesktopTab", GetParameter["language"]],
          StringContainsQ[platform, "cloud"],
          Localized["WorkflowCloudTab", GetParameter["language"]],
          StringContainsQ[platform, "mobile"],
          Localized["WorkflowMobileTab", GetParameter["language"]]
        ]
      }
    ]
  }]
],

(* ** WorkflowPlatform: a large enclosing group - SelectChildren[] can be very large *)

{CellGroupData, "WorkflowPlatform"} :> Module[{platform, class, count},
  platform = StringTrim @ SelectChildren[][[1, 1]];
  (* don't hide the first platform section, whatever platform it may be *)
  class = If[IncrementCounter["WorkflowPlatform"] == "1", "platform-container", "hide platform-container"];
  XMLElement["div", {"id" -> platform, "class" -> class}, {Transmogrify[Rest[SelectChildren[]]]}]
],
{Cell, "WorkflowPlatform"} :> Recurse[],

(* ** WorkflowNotesSection *)

{CellGroupData, "WorkflowNotesSection"} :>
  XMLElement["section", {"id" -> "WorkflowNotes"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle open"}, {Localized["WorkflowNotesSectionText", GetParameter["language"]]}],
      XMLElement["div", {"class" -> "hideable"}, {Transmogrify[Rest[SelectChildren[]]]}]
    }]
  }],
{Cell, "WorkflowNotesSection"} :> Unevaluated[Sequence[]],

(* ** WorkflowHeader *)

{CellGroupData, "WorkflowHeader" } :> Module[{title, id},
  title = Transmogrify[First[SelectChildren[]]];
  id = If[
    Positive @ Count[{title}, _XMLElement]
  ,
    StringReplace[Transmogrify`ConvertToString[title], {RegularExpression["[[:punct:]\\s-]+"] -> ""}]
  ,
    StringReplace[
      StringReplace[
        title,
        WordBoundary ~~ c:LetterCharacter :> ToUpperCase[c]
      ]
    ,
      {RegularExpression["[[:punct:]\\s-]+"] -> ""}
    ]
  ];

  XMLElement["section", {"id" -> id}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle open"}, {title}],
      XMLElement["div", {"class" -> "hideable"}, {Transmogrify[Rest[SelectChildren[]]]}]
    }]
  }]
],
{Cell, "WorkflowHeader" } :> Recurse[],

(* ** WorkflowHeaderLast *)

{CellGroupData, "WorkflowHeaderLast" } :> Module[{title, id},
  title = Transmogrify[First[SelectChildren[]]];
  id = StringReplace[title, WordBoundary ~~ c:LetterCharacter :> ToUpperCase[c]];
  id = StringReplace[id, {RegularExpression["[[:punct:]\\s-]+"] -> ""}];

  XMLElement["section", {"id" -> id, "class" -> "last"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle open"}, {Capitalize[title]}],
      XMLElement["div", {"class" -> "hideable"}, {Transmogrify[Rest[SelectChildren[]]]}]
    }]
  }]
],
{Cell, "WorkflowHeaderLast" } :> Recurse[],

(* ** WorkflowHeaderPlaceholder *)

(*
when there is no WorkflowHeader cell in a notebook, a placeholder should be
inserted to come out like this with non-hideable div and a simple section.

      <section>
        <div>
*)

{CellGroupData, "WorkflowHeaderPlaceholder" } :> Module[{},
  (* Print["entering WorkflowHeaderPlaceholder CellGroup transform"]; *)
  XMLElement["section", {}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["div", {}, {Transmogrify[Rest[SelectChildren[]]]}]
    }]
  }]],
{Cell, "WorkflowHeaderPlaceholder" } :> Module[{},
  (* Print["entering WorkflowHeaderPlaceholder cell transform"]; *)
  Recurse[]
],

(* ** WorkflowTitle *)

{Cell, "WorkflowTitle"} :> 
  XMLElement["div", {"class" -> "main-heading"}, {
    XMLElement["h1", {"class" -> "main-title"}, {Recurse[]}]
  }],

(* ** WorkflowDescription *)

{Cell, "WorkflowDescription"} :> XMLElement["div", {"id" -> "intro"}, {XMLElement["p", {}, {Recurse[]}]}],

(* ** WorkflowStep *)

{Cell, "WorkflowStep"} :> Module[{},
  IncrementCounter["WorkflowStep"];
  Sequence[
    If[GetCounter["WorkflowStep"] > 1, XMLElement["hr", {}, {}]],
    XMLElement["h2", {}, {Recurse[]}]
  ]
],

(* ** WorkflowText *)

{Cell, "WorkflowText"} :> XMLElement["p", {}, {Recurse[]}],

(* ** WorkflowAnnotatedInputImage *)

(* but AnnotatedInput with just images can be handled outside the Input infrastructure *)
{Cell, "WorkflowAnnotatedInputImage"} :> Recurse[],

(* ** WorkflowParenthetical *)

{CellGroupData, "WorkflowParenthetical"} :> {
  XMLElement["ul", {"class"->"functionList"}, {
    Transmogrify[SelectChildren[]]
  }]
},
{Cell, "WorkflowParenthetical"} :> XMLElement["li", {}, {Recurse[]}],


(* ** WorkflowFooterTopDelimiter *)

{CellGroupData, "WorkflowFooterTopDelimiter"} :> Recurse[],

(* ** FooterHeader* *)

{CellGroupData, "FooterHeaderSeeAlso"} :>
  XMLElement["section", {"id" -> "SeeAlso", "class" -> "related-links"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {"See Also"}],
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "FooterHeaderSeeAlso"}:> Unevaluated[Sequence[]],

{CellGroupData, "FooterHeaderRelatedFunctions"} :>
  XMLElement["section", {"id" -> "RelatedFunctions", "class" -> "related-links"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {"Related Functions"}],
      XMLElement["p", {"class" -> "inlineFunctionList"}, {Recurse[]}]
    }]
  }],
{Cell, "FooterHeaderRelatedFunctions"}:> Unevaluated[Sequence[]],

{CellGroupData, "FooterHeaderRelatedGuides"} :>
  XMLElement["section", {"id" -> "RelatedGuides", "class" -> "related-links"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {"Related Guides"}],
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "FooterHeaderRelatedGuides"}:> Unevaluated[Sequence[]],

{CellGroupData, "FooterHeaderRelatedLinks"} :>
  XMLElement["section", {"id" -> "RelatedLinks", "class" -> "related-links"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {"Related Links"}],
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "FooterHeaderRelatedLinks"}:> Unevaluated[Sequence[]],

{CellGroupData, "FooterHeaderRelatedTechNotes"} :>
  XMLElement["section", {"id" -> "RelatedTechNotes", "class" -> "related-links"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {"Related TechNotes"}],
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "FooterHeaderRelatedTechNotes"}:> Unevaluated[Sequence[]],

(* ** FooterOffset *)

(* RawXML because this XMLElement["div", ...] is self-closing, which doesn't give the desired effect *)
{Cell, "FooterOffset"} :> XML`RawXML["\n\n<div id=\"_footer-offset\"></div>\n\n"],

(* ** WorkflowRelatedLinksSection *)

{CellGroupData, "WorkflowRelatedLinksSection"} :>
  XMLElement["section", {"id" -> "RelatedLinks", "class" -> "related-links"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedLinksSectionText", GetParameter["language"]]}],
      XMLElement["ul", {"class" -> "functionList"}, {Transmogrify[Rest[SelectChildren[]]]}]
    }]
  }],
{Cell, "WorkflowRelatedLinks"} :> XMLElement["li", {}, {Recurse[]}],

(* ** WorkflowRelatedWorkflowsSection *)

{CellGroupData, "WorkflowRelatedWorkflowsSection"} :>
  XMLElement["section", {"id" -> "RelatedWorkflows", "class" -> "related-links"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedWorkflowsSectionText", GetParameter["language"]]}],
      XMLElement["ul", {"class" -> "functionList"}, {Transmogrify[Rest[SelectChildren[]]]}]
    }]
  }],
{Cell, "WorkflowRelatedWorkflows"} :> XMLElement["li", {}, {Recurse[]}],

(* ** WorkflowRelatedFunctionsSection *)

{CellGroupData, "WorkflowRelatedFunctionsSection"} :>
  XMLElement["section", {"id" -> "RelatedFunctions", "class" -> "related-links"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedFunctionsSectionText", GetParameter["language"]]}],
      Transmogrify[Rest[SelectChildren[]]]
    }]
  }],
{Cell, "WorkflowRelatedFunctions"} :> XMLElement["p", {"class" -> "inlineFunctionList"}, {Recurse[]}],

(* for use in Workflow Related Functions sections *)
{Cell, "WorkflowIF"} :> XMLElement["span", {"class"->"IF"}, {Recurse[]}],

(* ** WorkflowRelatedGuidesSection and WorkflowGuideRelatedGuidesSection *)

{CellGroupData, "WorkflowRelatedGuidesSection"|"WorkflowGuideRelatedGuidesSection"} :>
  XMLElement["section", {"id" -> "RelatedGuides", "class" -> "related-links"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedGuidesSectionText", GetParameter["language"]]}],
      XMLElement["ul", {"class" -> "functionList"}, {Transmogrify[Rest[SelectChildren[]]]}]
    }]
  }],
{Cell, "WorkflowRelatedGuides"|"WorkflowGuideRelatedGuide"} :> XMLElement["li", {}, {Recurse[]}],

(* ** WorkflowRelatedGuidesSection *)

(* ** WorkflowTechNotesSection *)

{CellGroupData, "WorkflowTechNotesSection"} :>
  XMLElement["section", {"id" -> "RelatedTechNotes", "class" -> "related-links"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["TechNotesSectionText", GetParameter["language"]]}],
      XMLElement["ul", {"class" -> "functionList"}, {Transmogrify[Rest[SelectChildren[]]]}]
    }]
  }],
{Cell, "WorkflowTechNotes"} :> XMLElement["li", {}, {Recurse[]}],

(* AnnotatedInput with just images can be handled outside the Input infrastructure *)
{Cell, "WorkflowAnnotatedInputImage"} :> Recurse[],

(* * workflow guides *)

{CellGroupData, "WorkflowGuideLeftColumn"|"WorkflowGuideRightColumn"} :>
  XMLElement["div", {"class" -> "half"}, {Recurse[]}],
{Cell, "WorkflowGuideLeftColumn"|"WorkflowGuideRightColumn"} :> Unevaluated[Sequence[]],

{CellGroupData, "WorkflowGuideTwoColumns"} :>
  XMLElement["section", {"class" -> "guide grid cols-2 heirs-width-1-2"}, {
    XMLElement["div", {"class" -> "inner"}, {
      Recurse[]
    }]
  }],
{Cell, "WorkflowGuideTwoColumns"} :> Unevaluated[Sequence[]],

{CellGroupData, "WorkflowGuideTitle"} :> 
  XMLElement["div", {"class" -> "main-heading-wrapper"}, {
    XMLElement["div", {"class" -> "inner"}, {
      Recurse[]
    }]
  }],
{Cell, "WorkflowGuideTitle"} :>
  XMLElement["div", {"class" -> "main-heading"}, {
    XMLElement["h1", {"class" -> "main-title"}, {Recurse[]}]
  }],

{CellGroupData, "WorkflowGuidePlatform"} :> 
  XMLElement["div", {"id" -> "desktop", "class" -> "platform-container"}, {
    XMLElement["section", {"class" -> "guide"}, {
      XMLElement["div", {"class" -> "inner"}, {
        Recurse[]
      }]
    }]
  }],
{Cell, "WorkflowGuidePlatform"} :> Unevaluated[Sequence[]],

{Cell, "WorkflowGuideTitle"} :> {
  XMLElement["div", {"class" -> "main-heading"}, {
    XMLElement["h1", {"class" -> "main-title"}, {Recurse[]}]
  }]
},

{Cell, "WorkflowGuideDelimiter"} :> 
XML`RawXML["<hr class=\"WorkflowDelimiter\"></hr>\n"],

{CellGroupData, "WorkflowGuideDescription"} :>
  XMLElement["div", {"class" -> "intro"}, {Recurse[]}],

{CellGroupData, "WorkflowGuideSection"|"WorkflowGuideSectionPlaceholder"} :> Sequence[
    XMLElement["h3", {"class" -> "special"}, {Transmogrify[First[SelectChildren[]]]}],
    XMLElement["ul", {}, {Transmogrify[Rest[SelectChildren[]]]}]
  ],
{Cell, "WorkflowGuideSection"} :> Recurse[],
{Cell, "WorkflowGuideSectionPlaceholder"} :> Recurse[],

{CellGroupData, "WorkflowOverviewSection"|"WorkflowOverviewSectionPlaceholder"} :> 
  Sequence[
    XMLElement["h3", {"class" -> "special"}, {Transmogrify[First[SelectChildren[]]]}],
    XMLElement["ul", {}, {Transmogrify[Rest[SelectChildren[]]]}]
  ],
{Cell, "WorkflowOverviewSection"} :> Recurse[],
{Cell, "WorkflowOverviewSectionPlaceholder"} :> Recurse[],

{CellGroupData, "WorkflowGuideOverviewSection"} :> Module[{}
  ,
  (* Print["WorkflowGuideOverviewSection: First[SelectChildren[]] = "<>ToString[InputForm[First[SelectChildren[]]]]]; *)
  XMLElement["section", {"class" -> "guide grid cols-2 heirs-width-1-2"}, {
    XMLElement["div", {"class" -> "inner"}, {
      Transmogrify[First[SelectChildren[]]],
      XMLElement["ul", {}, {Transmogrify[Rest[SelectChildren[]]]}]
    }]
  }]
],
{Cell, "WorkflowGuideOverviewSection"} :> Recurse[],

(* this is wrapped around a bare WorkflowGuideEntry to ensure correct margins *)
{CellGroupData, "WorkflowGuideEntry"} :> 
  XMLElement["section", {"class" -> "guide"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["ul", {}, {Transmogrify[Rest[SelectChildren[]]]}]
    }]
  }],
{Cell, "WorkflowGuideEntry"} :> XMLElement["li", {}, {Recurse[]}],

{CellGroupData, "WorkflowOverviewEntry"} :> 
  XMLElement["ul", {}, {Transmogrify[Rest[SelectChildren[]]]}],

{Cell, "WorkflowOverviewEntry"} :> XMLElement["li", {}, {Recurse[]}],

(* * wolframscript *)

(* ---------------------------------------------------------------------- *)
(* wolframscript *)

{Cell, "ProgramName"} :>
  XMLElement["div", {"class" -> "main-heading"}, {
    XMLElement["div", {"class" -> "main-title"}, {Recurse[]}]
  }],

{CellGroupData, "ProgramSection"} :> XMLElement["section", {}, {XMLElement["div", {"class" -> "inner"}, {Recurse[]}]}],

{Cell, "ProgramSection"} :> XMLElement["h2", {}, {Recurse[]}],

{Cell, "ProgramSubsection"} :> H3[ {}, {Recurse[]}],

{Cell, "ProgramCodeText"} :> XMLElement["code", {}, {Recurse[]}],

{StyleBox, "ProgramCodeText"} :> XMLElement["strong", {}, {XMLElement["code", {}, {Recurse[]}]}],

{Cell, "ProgramInput"|"ProgramOutput"|"ProgramInputOnly"} :> XMLElement["pre", {}, {XMLElement["code", {}, {Recurse[]}]}],

{Cell, "Script"} :> DIV[{"class" -> "with-file"}, {Span[{"class" -> "file-name"}, {"file.wls"}], XMLElement["pre", {}, {XMLElement["code", {}, {Recurse[]}]}]}],

{Cell, "ProgramNotes"} :> P[{"class" -> "code"}, {Recurse[]}],

{Cell, "ProgramNotesDescriptionSection"} :> P[{}, {Recurse[]}],

{Cell, "ProgramNotesExamplesSection"} :> P[{}, {Recurse[]}],

{Cell, "ProgramNotesDetailsSection"} :> P[{}, {Recurse[]}],

(* * Citations *)

{Cell, "CitationTitle"} :> XMLElement["h4", {}, {Recurse[]}],
{Cell, "CitationText"} :> XMLElement["p", {"class" -> "clipboard-inlineblock clipboard-hover"}, {Recurse[]}],


{Cell, "CitationTextJS"} :> Module[{},
  dprint["CitationTextJS: SelectSelf[] = "<>ToString[SelectSelf[], InputForm]];
  dprint["CitationTextJS: SelectChildren[] = "<>ToString[SelectChildren[], InputForm]];
  XMLElement["p",
    {"class" -> "clipboard-inlineblock clipboard-hover citation-timestamp"},
    {Recurse[]}
  ]
],

(* or let TMESTAM go on through to default.m and let fillinlayoutexpr munging take carek of it *)

{Cell, "CitationGroup"} :> Unevaluated[Sequence[]],
{CellGroupData, "CitationGroup"} :> Module[{defaultCitation, children},

  dprint["Functions.m: entering CitationsGroup"];
  dprint["SelectSelf[] = "]; dfile[SelectSelf[]];
  dprint["SelectChildren[] = "]; dfile[SelectChildren[]];

  children = Rest[SelectChildren[]];
  defaultCitation = First @ Cases[First[children], Cell[s:_String, "CitationText", ___] -> s, Infinity];

  XMLElement["section", {"id" -> "Citations", "class" -> "citations"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["div", {"class" -> "citation"}, {
        XMLElement["span", {"class" -> "citation-tooltip"}, {
          XMLElement["button", {"type" -> "button", "class" -> "citation-tooltip-button"}, {XMLElement["span", {"class" -> "citation-text"}, {Localized["CiteThisAsText", GetParameter["language"]]}]}],
          XMLElement["span", {"class" -> "citation-default clipboard-inlineblock clipboard-hover"}, {Transmogrify[defaultCitation]}],
          XMLElement["span", {"class" -> "citation-tooltip-content"}, Flatten @ {
            XMLElement["button", {"class" -> "x-close-svg"}, {
              XMLElement["img", {"src" -> "/language-assets/img/CitationCloseButton.svg", "height" -> "10px", "width" -> "10px"}, {}]
            }],
            Transmogrify /@ children
          }],
          XMLElement["span", {"class" -> "citation-overlay"}, {""}]
        }]
      }]
    }]
  }]
],

(* History section *)

{Cell, "HistorySection"} :> Unevaluated[Sequence[]],

{CellGroupData, "HistorySection"} :> Module[{},
  dprint["Functions.m: HistorySection: SelectSelf[] = \n"<>ToString[ResourceFunction["ReadableForm"][SelectSelf[], "PrefixForm" -> False], InputForm]];
  dprint["Functions.m: HistorySection: SelectChildren[] = \n"<>ToString[ResourceFunction["ReadableForm"][SelectChildren[], "PrefixForm" -> False], InputForm]];

  XMLElement["section", {"id" -> "History", "class" -> "related-links"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {"History"}],
      XMLElement["p", {"class" -> "inlineFunctionList"}, {
        Recurse[]
      }]
    }]
  }]
],

{Cell, "History" | "HistoryText" | "HistoryVersion" | "HistoryDelimiter"} :> Recurse[],

{Cell, "HistoryItem"} :> Module[{text, versionText, versionInteger, image, link},
  dprint["Functions.m: HistoryItem: SelectSelf[] = \n"<>ToString[ResourceFunction["ReadableForm"][SelectSelf[], "PrefixForm" -> False], InputForm]];
  dprint["Functions.m: HistoryItem: SelectChildren[] = \n"<>ToString[ResourceFunction["ReadableForm"][SelectChildren[], "PrefixForm" -> False], InputForm]];

  text = First @ Cases[SelectSelf[], Cell[_String, "HistoryText", ___], Infinity];
  dprint["HistoryItem: text = "<>ToString[text, InputForm]];

  versionNumber =
    StringReplace[
      First @ Cases[SelectSelf[], StyleBox[s:_String, "HistoryVersion", ___] -> s, Infinity]
    , {
        __ ~~ "(" -> "("
      , ")" ~~ __ -> ")"
      }
    ];
  dprint["HistoryItem: versionNumber = "<>ToString[versionNumber, InputForm]];

  link = FirstCase[SelectChildren[], s:_String /; StringContainsQ[s, StartOfString ~~ "paclet:"], Missing[], Infinity];
  dprint["HistoryItem: link = "<>ToString[link, InputForm]];

  If[StringQ[link]
  , XMLElement["span", {"class" -> "history-text"}, {
      XMLElement["a", {"href" -> StringReplace[link, "paclet:" -> "/language/"]}, {
        Recurse[text]<>" ",
        XMLElement["span", {"class" -> "history-version"}, {versionNumber}]
      }]
    }]
  , XMLElement["span", {"class" -> "history-text"}, {
      Recurse[text]<>" ",
      XMLElement["span", {"class" -> "history-version"}, {versionNumber}]
    }]
  ]
]

}, (* close the body of this file's XMLTransform *)

(* * closing arguments *)

IncludeXMLTransforms -> {{"HTML","Components","Includes.m"}},

DefaultParameters -> {
  (*  "newline2br" -> False,  *)
  "newline2br" -> False,
  "newline2div" -> False,
  "styleUsageMessageBreak" -> "</div>\n<div class=\"usageDelimiter\" > </div><div class=\"UM\">",
  (* No Operation code *)
  "noop" -> {
    $LanguageExtension := Switch[GetParameter["language"], "Japanese", "ja", "ChineseSimplified", "zh", "Spanish", "es", _, "en"];
    (* exceptionsList:= {True}, *)
    exceptionsList:= {
      "Character Name"
    , "Classifier"
    , "Compiled Type"
    , "Database Connection"
    , "Data Structure"
    , "Device Connection"
    , "Embedding Format"
    , "Entity"
    , "External Evaluation System"
    , "File"
    , "Format"
    , "Front End Object"
    , "Indicator"
    , "Interpreter"
    , "MathLink C Function"
    , "Menu Item"
    , "Message"
    , "Method"
    , "Net Decoder"
    , "Net Encoder"
    , "Predictor"
    , "Program"
    , "Remote Batch Computation Provider"
    , "ResourceObject"
    , "Service Connection"
    , "Symbol"
    , "System Program"
    , "Text Content"
    };
  }
}

]
