(* :Title: Guides.m *)

(* :Author:
	Andrew Hunt, andy@wolfram.com
	Greg Martel, gregm@wolfram.com
*)

(* :Mathematica Version: 6.0 *)

(* :Package Version: $Revision: 1.53 $ $Date: 2017/10/03 19:47:44 $ *)
                  
(* :Copyright: (c) 2005, Wolfram Research, Inc. All rights reserved. *)

(* :Summary: XMLTransform for Documentation Guide pages *)

(* :ToDo: 

*)


XMLTransform[{


(***** Match Notebook expression *****)


(***** Cell Definitions *****)
{Cell, "AnchorBar" | "History"} :> Unevaluated[Sequence[]],

{Cell, "Picture"} :> 
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
  {"alt"->""},
  Inline -> True,
  ImageFormat -> "GIF",
  CropImage -> False,
  "RetinaImage" -> False
], 

{Cell,  "2ColumnTable" | "3ColumnTable" | "2ColumnTableMod" | "3ColumnTableMod" | "3ColumnIllustratedTableMod"} :>
  DIV[ {"class" -> "FunctionTable", "name" -> (GetStyle[]/.None->"FIXME")}, 
  {{Recurse[] }}
], 

{Cell, "GuideTOCTitle"} :> 
H1[{},  {Recurse[]}],  

{Cell, "GuideTOCLink"} :> XMLElement["h2", {}, {Recurse[]}],

{Cell, "GuideTOCAbstract"} :>
P[{"class" -> "intro"}, {Recurse[]}],

{BoxData}:> Recurse[],

{Cell, "GuideTitle"} :> Module[
  {flag = GetParameter["flag"]},

  If[
    flag =!= "RootGuidePage"
  ,
    XMLElement["div", {"class" -> "main-heading"}, {
      XMLElement["h1", {"class" -> "main-title"}, {Recurse[]}],
      XML`RawXML["<div class=\"iconography\" data-src=\"s=SICONSTATUS d=DICONSTATUS m=MICONSTATUS c=CICONSTATUS u=UICONSTATUS l= n=NICONSTATUS e=EICONSTATUS\"></div>"]
    }]
  ,
    XMLElement["div", {}, {}]
  ]
],

{Cell, "GuideTitleGrid"} :>  Block[{objgrid = True}, 
  DIV[{"class"->"GuideTitleGrid"}, {Recurse[]}] ],

{Cell, "IntroLinkCell"} :> Sequence[],
(*
XMLObject["Comment"][ToString@DIV[{"class"->"IntroLinkCell"}, {Recurse[] } ] ],
*)

{StyleBox, "GuideFunctionsSubsection"} :> Recurse[],

{Cell, "GuideAbstract"} :> 
Module[{flag = GetParameter["flag"]}, 
  If[
    flag =!= "RootGuidePage"
  ,
    WithParameters[{"newline2br" -> False, "newline2div" -> True, "styleUsageMessageBreak" -> "<span class='br'>&nbsp;</span>"},
      P[{"class"->"intro"}, {Recurse[]} ]
    ]
  ,
    DIV[{},{}]
  ]
],

{CellGroupData, "GuideReferenceLinksSection"} :> 
  XMLElement["section", If[GetParameter["url"] == "guide/RecentlyAddedFeatures", {"id" -> "Reference"}, {}], {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["div", {"class" -> "GuideDelimiterTop"}, {XML`RawXML["<!-- -->"]}],
      Recurse[]
    }]
  }],
{Cell, "GuideReferenceLinksSection"} :> Unevaluated[Sequence[]],

{CellGroupData, "SeeAlsoSection"} :>
  XMLElement["section", {"id" -> "SeeAlso", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["SeeAlsoSectionText", GetParameter["language"]]}], 
      Recurse[]
    }]
  }],
{Cell, "SeeAlso"} :> XMLElement["p", {"class"->"inlineFunctionList"}, {Recurse[]}],


{Cell, "GuideFunctionsSubsection"} :> Module[
  {flag = GetParameter["flag"]}, 
  If[flag =!= "RootGuidePage",
    H3[
      If[HasOption[CellTags],
        {"id" -> StringReplace[
          ToString[Flatten[{GetOption[CellTags]}]],
          {"{" -> "", "}"->"", "$" -> "dollar"}]},
        {}
      ],
      {Recurse[]}
    ],
    DIV[{},{}]
  ]
],

{Cell, "GuideFunctionsSubsubsection"} :> 
Module[{flag = GetParameter["flag"]}, 
  If[flag =!= "RootGuidePage",
    H4[{}, {Recurse[]} ],
    DIV[{},{}]
  ]
],


{Cell, "GuideDelimiterSubsection"} :> 
Module[{flag = GetParameter["flag"]}, 
  If[flag =!= "RootGuidePage",
    XMLElement["div", {"class"->"GuideDelimiterSubsection"}, {XML`RawXML["<!-- -->"]} ],
    DIV[{},{}]
  ]
],

{Cell, "GuideDelimiter"} :> 
Module[{flag = GetParameter["flag"]}, 
  If[flag =!= "RootGuidePage",
    XMLElement["div", {"class"->"GuideDelimiter"}, {XML`RawXML["<!-- -->"]} ],
    DIV[{},{}]
  ]
],

{Cell, "FeaturedExamplesGridCell"} :> Unevaluated[Sequence[]],

{Cell, "GuideMainTutorial" | "GuideTestDrive" | "GuideTitle" | "GuideFunctionGroupCaption"} :> 
Module[{flag = GetParameter["flag"]}, 
  If[flag =!= "RootGuidePage",
    DIV[{"class"->GetStyle[]}, {
    	Recurse[]
    	} ],
    XMLElement["span",{}, {Recurse[]}]
  ]
],

(* standard handling for 2017 redesign openable headers with arbitrary text *)
{CellGroupData, "GuideFunctionsSection"} :> Module[{summaryQ, openallQ},
  openallQ = TrueQ["GuideOpenAll" /. Options[First[SelectChildren[]]]];
  summaryQ = StringContainsQ[GetBasenameLa[], {"SummaryOfNewFeaturesIn", "HowToTopics"}];
  XMLElement["section", If[summaryQ, {}, If[GetParameter["url"] == "guide/RecentlyAddedFeatures", {"id" -> "Reference"}, {}]], {
    XMLElement["div", {"class" -> "inner"}, Flatten@{
      XMLElement["h1", {"class" -> "toggle open"}, {Transmogrify[First[SelectChildren[]]]}],
      If[openallQ, {
        XMLElement["span", {"class" -> "open-all"}, {ToLowerCase[Localized["Open All", GetParameter["language"]]]}],
        XMLElement["span", {"class" -> "close-all on"}, {ToLowerCase[Localized["Close All", GetParameter["language"]]]}]
      }, {}],
      XMLElement["div", {"class" -> "hideable"}, {Transmogrify[Rest[SelectChildren[]]]}]
    }]
  }]
],

{Cell, "GuideFunctionsSection"} :> Recurse[],

{CellGroupData, "GuideTutorialsSection" } :>
  XMLElement["section", {"id" -> "RelatedTutorials", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedTutorialsSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }], 
{Cell, "GuideTutorialsSection" } :> Unevaluated[Sequence[]],

{CellGroupData, "GuideMoreAboutSection" } :>
  XMLElement["section", {"id" -> "RelatedGuides", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["MoreAboutSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }], 
{Cell, "GuideMoreAboutSection" } :> Unevaluated[Sequence[]],
{Cell, "MoreAbout"} :> XMLElement["li", {}, {Recurse[]}],

{CellGroupData, "GuideRelatedLinksSection" } :>
  XMLElement["section", {"id" -> "RelatedLinks", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedLinksSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }], 
{Cell, "GuideRelatedLinksSection" } :> Unevaluated[Sequence[]],

{Cell, "GuideRelatedLinks"} :> Module[{class},
  (* if an item has no RelatedLinkStyle, then don't give it a class *)
  class = Quiet@Check[First[Cases[SelectSelf[], Rule["RelatedLinkStyle", s_String] :> Rule["class", s], Infinity]], ""];
  XMLElement["li", {class}, {Recurse[]}]
],

{Cell, "RelatedLinksParenText"} :> XMLElement["span", {"class" -> "paren-text"}, {Recurse[]}],

{Cell, "RelatedLinks"} :> XMLElement["li", {}, {Recurse[]}],

{CellGroupData, "GuideWorkflowGuidesSection"} :>
  XMLElement["section", {"id" -> "RelatedWorkflowGuides", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["WorkflowGuidesSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "GuideWorkflowGuidesSection" } :> Unevaluated[Sequence[]],
{Cell, "GuideWorkflowGuide"} :> XMLElement["li", {}, {Recurse[]}],

{CellGroupData, "GuideTutorialCollectionSection"} :>
  XMLElement["section", {"id" -> "TutorialCollection"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["TutorialCollectionSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "GuideTutorialCollectionSection" } :> Unevaluated[Sequence[]],

(* {Cell, "RootMoreAboutDelimiter"} :> 
    DIV[{"class"->"GuideDelimiter"}, {Recurse[] } ],  *)
    
{CellGroupData, "GuideFeaturedExamplesSection"} :> Module[{cfp},
  cfp = CreateFeaturedPage[GetParameter["featuredexampledata"]];
	XMLElement["div", {"class"->"expandWrap", "id"->"FeaturedExamples"},
	{XMLElement["div", {"class"->"expandOuter", "id"->"mycarousel"},{
		XMLElement["h2", {"class"->"expandInner"}, Flatten[{
			XMLElement["a", {"class"->"expand hide", "title"->Localized["show", GetParameter["language"]], "href"->""},{Localized["GuideFeaturedExamplesSectionText", GetParameter["language"]]}], 
			XMLElement["a", {"class"->"collapse", "title"->Localized["hide", GetParameter["language"]], "href"->""},{Localized["GuideFeaturedExamplesSectionText", GetParameter["language"]]}], 
			If[Length[cfp]>4, {XML`RawXML["<span class=\"jcarousel-scroll\"> 
							<a href=\"#\" class=\"buttons prev\"></a>
							<span class=\"page-out-of\"></span><span class=\"divider\">|</span><span class=\"total-pages\"></span>
							<a href=\"#\" class=\"buttons next\"></a>
							<a href=\"#\" class=\"buttons\" id=\"mycarousel-minimize\"></a>
							<a href=\"#FeaturedExamples\" id=\"mycarousel-show-all\"></a>
					</span>"]}, {}]
			}]], 
			XMLElement["div",{"class"->"expanded"},{XMLElement["div",{"class"->"viewport"},{XMLElement["ul",{"class"->"overview"},Flatten[{Map[Transmogrify,cfp]}]], XMLElement["div", {"class"->"clearboth"}, {XML`RawXML["<!-- -->"]}] }]}]}]}]],
			
{Cell, "GuideFeaturedExampleSection"} :> Unevaluated[Sequence[]],

{CellGroupData, "FeaturedExampleLink"} :> XMLElement["li", {},{XMLElement["a", {"href"->Cases[SelectSelf[], Cell[a_, "FeaturedExampleLink", ___]:>a, Infinity][[1]]},{Recurse[]}]}],
			
{Cell, "FeaturedExampleLink"} :> Unevaluated[Sequence[]],

{Cell, "FeaturedExampleThumbnailImage"} :>
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, "featured_example"<>IncrementCounter["FeaturedExampleThumbnail"] <> ".png"],
    {"alt"->"carousel-img"<>GetCounter["FeaturedExampleThumbnail"]},
    Inline -> True,
    CropImage -> False,
    ImageFormat->"PNG",
    "RetinaImage" -> True
  ],

{Cell, "FeaturedExampleText"} :> XMLElement["span", {"class"->"carousel-caption"},{Recurse[]}],
    
{CellGroupData, "GuideLearningResourcesSection"} :> Module[{}, XMLElement["div", {"class"->"expandWrap closed", "id"->"LearningResources"},
	{XMLElement["div", {"class"->"expandOuter"},{
		XMLElement["h2", {"class"->"expandInner"}, {
			XMLElement["a", {"class"->"expand hide", "title"->Localized["show", GetParameter["language"]], "href"->"#"},{Localized["GuideLearningResourcesSectionText", GetParameter["language"]]}], 
			XMLElement["a", {"class"->"collapse", "title"->Localized["hide", GetParameter["language"]], "href"->"#"},{Localized["GuideLearningResourcesSectionText", GetParameter["language"]]}]}], 
			XMLElement["div",{"class"->"expanded"}, 
				{XMLElement["table", {"cellspacing" -> "0", "cellpadding" -> "0", "border" -> "0"}, {XMLElement["tr", {}, {Transmogrify[CreateLearningResources[SelectSelf[], ""]]}]}]}]}]}]],

{CellGroupData, "GuideFeaturedExamplesSection"} :> Unevaluated[Sequence[]],
{Cell, "GuideFeaturedExamplesSection"} :> Unevaluated[Sequence[]],

{Cell, "GuideLearningResourcesSection"} :> Unevaluated[Sequence[]],

{CellGroupData, "GuideLearningResourcesLeftSection"} :> XMLElement["td", {"class" -> "resourcesLeft"}, {XMLElement["div", {}, Flatten[{Recurse[]}]]}],

{Cell, "GuideLearningResourcesLeftSection"} :> Unevaluated[Sequence[]],

{CellGroupData, "GuideLearningResourcesTutorialSection"}:> {Transmogrify[First[SelectChildren[]]], XMLElement["div", {"class"->"learning-links"}, {Transmogrify[Rest[SelectChildren[]]]}]},

{Cell, "GuideLearningResourcesTutorialSection"} :> XMLElement["h3", {"class"->"learning-tutorials"}, {Recurse[]}],

{Cell, "LearningResourceTutorial"} :> Recurse[], 

{CellGroupData, "GuideLearningResourcesHowToSection"} :> {Transmogrify[First[SelectChildren[]]], XMLElement["div", {"class"->"learning-links"}, {Transmogrify[Rest[SelectChildren[]]]}]}, 

{Cell, "GuideLearningResourcesHowToSection"} :> XMLElement["h3", {"class"->"learning-howtos"}, {Recurse[]}],

{Cell, "LearningResourceHowTo"} :> Recurse[],

{CellGroupData, "GuideLearningResourcesRightSection"} :> XMLElement["td", {"class" -> "resourcesRight"}, {Recurse[]}],

{Cell, "GuideLearningResourcesRightSection"} :> Unevaluated[Sequence[]],

{Cell, "SmallTitle"} :> XMLElement["p", {"class"->"smallTitle"}, {Recurse[]}],

{Cell, "ResourceTraining"} :> Recurse[],
{Cell, "ResourceVideos"} :> Recurse[],
{Cell, "ResourceDemonstrations"} :> Recurse[],
{Cell, "ResourceCommunity"} :> Recurse[],

{Cell, "RootMoreAboutDelimiter"} :> 
    DIV[{"class"->"RootMoreAboutDelimiter"}, {Recurse[] } ],

{Cell, "RootSectionCaption"} :> 
    DIV[{"class"->"GuideFunctionGroupCaption"}, {Recurse[] } ],

{Cell, "AlphabeticalListingAnchor"} :>
  DIV[
    {"class" -> "AlphabetListingJumpTo"},
    Map[
      XMLElement[
        "a",
        {"href" -> "#" <> StringReplace[#, "$" -> "dollar"]},
        {#}
      ]&,
      Cases[SelectSelf[], Rule[False, w_] :> w, Infinity]
    ]
  ],

{Cell, "GuideMoreAbout" | "GuideMoreAboutSub" | "GuideTutorial" | "GuideTutorialCollection"} :> Module[
  {flag = GetParameter["flag"]},

  If[flag =!= "RootGuidePage",
    Li[{Recurse[]}],
    DIV[{"class"->"OtherItems"},  {
      UL[{"class"->GetStyle[]}, { 
        Li[{ Recurse[] }]
      }]
    }]
  ]
],
	

{Cell, "RootMoreAbout"} :> {},

{Cell, "GuideFunctionListingText"} :>
  XMLElement["span", {"class"->"singleFunctionText"},  { Recurse[] }],

{CellGroupData, "GuideSymbolIcon"} :> XMLElement["span", {"class" -> "icon-text-container"}, {Recurse[]}],
{Cell, "GuideSymbolIcon"} :> XMLElement["span", {"class" -> "GuideSymbolIcon"}, {Recurse[]}],

{Cell, "GuideText"} :> Module[{flag = GetParameter["flag"], summaryQ, res},

  dprint["Guides.m: entering GuideText xform; SelectChildren[] = "]; dfile[SelectChildren[]];
  summaryQ = StringContainsQ[GetBasenameLa[], {"SummaryOfNewFeaturesIn", "HowToTopics"}];

  res = If[
    flag =!= "RootGuidePage"
  , P[{"class"->"singleFunction"},  { Recurse[] }]
  , Module[{guidelinks,guidelinksNew678,class="left"},
      guidelinks = Cases[SelectSelf[], _ButtonBox, Infinity];
      (*guidelinksNew678 = Insert[Take[guidelinks, {-3, -2, -1}], " | ", 2];*)
      guidelinksNew678 = Riffle[Take[guidelinks, {-3, -1}], " | "];
      guidelinks = Join[Drop[guidelinks, -3], {guidelinksNew678}];
      guidelinks = If[EvenQ@Length@guidelinks === True, guidelinks, Append[guidelinks, {}]];
      guidelinks = Partition[guidelinks, 2];
      guidelinks = Transpose[guidelinks];
      guidelinks = Map[
        XMLElement["ul", {"class" -> (class="right";#)&[class]}, {Map[Li[{}, {Transmogrify[#]}]&, #]}]&,
        guidelinks];
      XMLElement["div",{
          "class" -> "accordion-content "<> GetCounter["RootSection"]
        }, {
        XMLElement["tr",{}, {
        TD[{"class"->"rootLinkCol"}, {guidelinks[[1]]}],
        TD[{"class"->"rootLinkCol"}, {DeleteCases[guidelinks[[2]], XMLElement["li",{__},{}], Infinity]}]
        }]
      }]
    ]
  ];

  dprint["GuideText res = "<>ToString[res, InputForm]];
  res
],

{CellGroupData, "GuideTextSectionWrapper"} :>
  XMLElement["div", {"class" -> "inline"}, {Recurse[]}],
{Cell, "GuideTextSectionWrapper"} :> Unevaluated[Sequence[]],

{Cell, "RootLinks" | "RootLink"} :> 
Module[{flag = GetParameter["flag"]},
  If[flag =!= "RootGuidePage",
    DIV[{"class"->"GuideText"},  { Recurse[] }],
    Module[{guidelinks,guidelinksNew678,class="left"},
      guidelinks = Cases[SelectSelf[], _ButtonBox, Infinity];

      guidelinks = GatherBy[guidelinks, #[[-1]] &];

      (*guidelinksNew678 = Insert[Take[guidelinks, {-3, -2, -1}], " | ", 2];*)
      guidelinksNew678 = Riffle[Take[guidelinks, {-3, -1}], " | "];
      guidelinks=Join[Drop[guidelinks, -3], {guidelinksNew678}];
	  guidelinks=If[EvenQ@Length@guidelinks === True, guidelinks, Append[guidelinks, {}]];
	  guidelinks = Partition[guidelinks, 2];
	  guidelinks = Transpose[guidelinks];
      guidelinks = Map[
        XMLElement["ul", {"class" -> (class="right";#)&[class]}, Flatten[{Map[XMLElement["li", {}, {Transmogrify[#]}]&, #]}]]&,
        guidelinks];
      guidelinks = DeleteCases[guidelinks, XMLElement["li",{},{}], Infinity];
      XMLElement["div",{
          "class" -> "accordion-content "<> (GetCounter["RootSection"]/.{"1"->"one","2"->"two","3"->"three","4"->"four","5"->"five","6"->"six","7"->"seven","8"->"eight"})
        }, Flatten[{guidelinks[[1]], guidelinks[[2]]}]]
    ]
]],

{Cell, "RootDarkLink"} :> Recurse[],

{Cell, "RootText"} :> 
Module[{rows},
  rows = First@First@SelectNearestDescendants[GridBox];
  DIV[{"class" -> "GuideText"}, {
    XMLElement["ul", {"class" -> "RootText"}, Flatten[{
      Map[
        XMLElement["li", {}, 
          If[MatchQ[#, {_, Cell[TextData[_List, ___], ___]}],
            Flatten@Map[
              If[MatchQ[#, {StyleBox[" \[FilledSmallSquare] ", ___]}],
                {" ", XMLElement["img",
                  {"src" -> "http://reference.wolfram.com/mathematicaImages/bullet.gif",
                   "style" -> "vertical-align: middle", "alt"->"bullet", "title"->"bullet"},
                  {}], " "},
                Transmogrify /@ #
              ]&,
              Split[#[[2, 1, 1]], Not[Apply[Or, Map[
                MatchQ[#, StyleBox[" \[FilledSmallSquare] ", ___]]&,
                {#1, #2}]]]&]
            ],
            (* Else *)
            {Transmogrify[#]}
        ]]&,
        Most[rows] ],
      XMLElement["li", {}, {
        Transmogrify[rows[[-1, 2]]] /. XMLElement["a", atr_, cnt_] :>
          XMLElement["a", atr, {
            XMLElement["span", {"style" -> "color:#454545"}, {
              StringDrop[cnt[[1]], -1] }],
            StringTake[cnt[[1]], -1]
          }]
      }]
    }] ]
  }]
],

{StyleBox, "InlineSeparator"} :>If[StringReplace[GetBasenameLa[], {".en"->"", ".ja"->""}]==="Mathematica",
Span[{"class" -> "inlineBullet"}, {XML`RawXML["&nbsp;"], XML`RawXML["&#9642;"], XML`RawXML["&nbsp;"]}],
Span[{"class" -> "inlineBullet"}, {XML`RawXML["&nbsp;"], XML`RawXML["&#9642;"], XML`RawXML["&nbsp;"]}]
],


{StyleBox, "LeadingBullet"} :>
Span[{"class" -> "LeadingBullet"}, {
    Img[{"src" -> htmlBase<>"/mathematicaImages/bullet.gif", "width"->"4", "height"->"4", "style"->"vertical-align:middle;", "alt"->"bullet", "title"->"bullet"}] }],

{Cell, "InlineListing"} :>
DIV[{"class" -> "InlineListing"}, {Recurse[] }],
{Cell, "InlineListing3Col"} :>
DIV[{"class" -> "InlineListing3Col"}, {Recurse[] }],
{Cell, "InlineListing4Col"} :>
DIV[{"class" -> "InlineListing4Col"}, {Recurse[] }],


{Cell, "InlineGuideFunction"} :> 
Span[{"class"->"InlineGuideFunction"}, {Recurse[]} ],


{Cell, "InlineGuideFunctionListing"} :> 
Module[{ml},
  ml = Recurse[];
  If[(Length@{ml} === 1) && (Head@ml === String),
   ml = StringSplit[ml, RegularExpression["\\s+\\.\\s+"] ];
   ml = Span[{"class"->"InlineGuideFunction"}, 
          {XMLElement["span", {}, {#}] }]&/@ ml;
   ml = Riffle[ml,  
     Span[{"class" -> "inlineGuideFunctionListingBullet"}, 
       {" ", Img[{"src" -> htmlBase<>"/mathematicaImages/bullet-guide.gif", "width"->"6", "height"->"12", "style"->"vertical-align:middle;", "alt"->"bullet-guide", "title"->"bullet-guide"}],
       " "}]
     ] ];
  P[{"class"->"inlineFunctionList"}, Flatten@{ml}]
],

PaneSelectorBox:>Transmogrify@SelectSelf[][[1,2,2]],

{CellGroupData, "GuideReferenceLinksSection"} :> 
  XMLElement["section", If[GetParameter["url"] == "guide/RecentlyAddedFeatures", {"id" -> "Reference"}], {XMLElement["div", {"class" -> "inner"}, {Recurse[]}]}],

{Cell, "GuideRecentlyAddedDate"} :> XMLElement["span", {"class" -> "date"}, {Recurse[]}],
{Cell, "GuideRecentlyAddedUpdates"} :> XMLElement["span", {"class" -> "updates"}, {Recurse[]}],

{CellGroupData, "RootSectionHeading"} :> Module[{headCell, linkCells, res},
  dprint["entering RootSectionHeading cellgroup"];

  headCell = First[SelectChildren[]]; dprint["headCell 1 = "<>ToString[headCell, InputForm]];
  linkCells = Rest[SelectChildren[]];
  class = Which[
    First[headCell] == "DEFAULT",
    "container"
  , Or[
      First[headCell] == "WORKFLOWS",
      First[headCell] == "\:30ef\:30fc\:30af\:30d5\:30ed\:30fc",
      First[headCell] == "\:5de5\:4f5c\:6d41\:7a0b"
    ],
    "container left"
  , Or[
      First[headCell] == "RESOURCES",
      First[headCell] == "\:30ea\:30bd\:30fc\:30b9",
      First[headCell] == "\:8d44\:6e90"
    ],
    "container left resources"
  , ___, "container left"
  ];
  dprint["headCell 2 = "<>ToString[headCell, InputForm]];
  dprint["class = "<>ToString[class, InputForm]];

  res = If[First[headCell] == "DEFAULT",
    XMLElement["section", {"class" -> class}, {Transmogrify[linkCells]}]
  ,
    {
      XMLElement["h2", {}, {Transmogrify[headCell]}],
      XMLElement["section", {"class" -> class}, {Transmogrify[linkCells]}]
    }
  ];

  dprint["RootSectionHeading cellgroup res = "]; dfile[res];

  res
],

{Cell, "RootSectionHeading"} :> Module[{res},
  dprint["entering RootSectionHeading cell, SelectSelf[] = "<>ToString[SelectSelf[], InputForm]];
  res = Recurse[];
  dprint["RootSectionHeading cell: res = "<>ToString[res, InputForm]];
  res
],

{CellGroupData, "RootFooterLink"} :> XMLElement["div", {"class" -> "root-guide-footer-links"}, {
  XMLElement["div", {"class" -> "inner"}, {Transmogrify[SelectChildren[]]}]
}],

{Cell, "RootFooterLink"} :> Module[{res},
  dprint["Guides.m: RootFooterLink, SelectSelf[] = "<>ToString[SelectSelf[], InputForm]];
  res = Recurse[];
	(* Use Join so that the "class" style here overrides `r` if `r` is already a "class"
	   style. This works around the error generating when trying to export an element
	   like XMLElement["a", {"class" -> "foo", "class" -> "chevron-after"}, {..}] *)
	res = res /. {r:_Rule} :> Normal @ Join[
		<| r |>,
		If[
			Positive@Count[res, s_String /; StringContainsQ[s, "/guide/SummaryOfNewFeatures"], All],
			<| "class" -> "chevron-after" |>,
			<| "class" -> "chevron-after", "target" -> "_blank" |>
		]
	];
  dprint["Guides.m: RootFooterLink: final res = "<>ToString[res, InputForm]];
  res
],

{CellGroupData, "RootSectionTitle"} :> Module[{tileCount, tileColor, title, processedTitle, head, body, linkTarget, workflowTileQ},
  dprint["entering RootSectionTitle"];
  cell = First[SelectChildren[]];
  dprint["cell = "<>ToString[cell, InputForm]];

  tileCount = IncrementCounter["rootTileCounter"];
  dprint["RootSectionTitle: tileCount = "<>ToString[tileCount, InputForm]];

  title = Transmogrify[cell];
  processedTitle = If[title == "Recent Features",
    "LatestFeatures", (* TODO: match css for now; later, change css to match calculated value RecentFeatures *)
    title /. s:_String :> StringReplace[s, {"\"" -> "", WhitespaceCharacter.. -> "", "&" -> "And", "-" -> "", "," -> ""}]
  ];
  
  dprint["RootSectionTitle: processedTitle = "<>ToString[processedTitle, InputForm]];

  (* only workflow tile cells link to workflowguides *)
  workflowTileQ = TrueQ[Positive @ Count[cell, s_String /; StringContainsQ[s, "paclet:workflowguide/"], Infinity]];

  resourceTileQ = MemberQ[{"FastIntroductions", "IntroBookAndCourse", "ArchivesAndRepositories", "DeveloperResources", "WolframLanguageProducts", "WolframU"}, processedTitle];
    resourceIconName = <| (* usage: resourceIconName[processedTitle] *)
    "FastIntroductions"       -> "fast-intro",
    "IntroBookAndCourse"      -> "intro-book",
    "ArchivesAndRepositories" -> "archives-repositories",
    "DeveloperResources"      -> "developer-resources",
    "WolframLanguageProducts" -> "language-products",
    "WolframU"                -> "wolfram-u"
  |>;

  linkTarget = If["SingleLink" =!= ("SingleLink" /. Options[cell]),
    StringReplace["SingleLink" /. Options[cell], "paclet:" -> "/language/"],
    "#"<>processedTitle];

  tileColor = Which[
    workflowTileQ || resourceTileQ,
    "w1bg",
    StringContainsQ[linkTarget, StartOfString~~"#"] || StringContainsQ[linkTarget, "RecentlyAddedFeatures"],
      Switch[tileCount,
        "1"|"2"|"3"|"4"|"5", "redbg",
        "6"|"7"|"8"|"9"|"10"|"11", "orangebg",
        "12"|"13"|"14"|"15"|"16"|"17"|"18", "greenbg",
        "19"|"20"|"21"|"22"|"23", "bluebg",
        "24", "maingreybg",
        ___, "redbg"
      ],
    True,
      "w1bg"
  ];
  dprint["RootSectionTitle: tileColor = "<>ToString[tileColor, InputForm]];

  If[
    processedTitle == "FastIntroductions"
  , XMLElement["div", {"class" -> "thumb "<>tileColor<>" has-links dropdown"}, {
      XMLElement["div", {"class" -> "menu-wrapper"}, {
        XMLElement["div", {"class" -> "podheader menu", "id" -> "FastIntroductions"}, {
          XMLElement["div", {"class" -> "podheader-inner"}, {
            XMLElement["a", {"href" -> "#"<>processedTitle, "class" -> "title"}, {title}],
            XMLElement["div", {"class" -> "svg"}, {
              XMLElement["svg", {"viewBox" -> "0 0 22 22"}, {
                XMLElement["use", {"xlink:href" -> "/language-assets/img/root-guide-small.svg#fast-intro"}, {}]
              }]
            }]
          }]
        }],
        Transmogrify[Rest[SelectChildren[]]]
      }]
    }]
  , XMLElement["div", {"class" -> "thumb "<>tileColor}, Flatten@{
      XMLElement["div", If[workflowTileQ || resourceTileQ,
          {"class" -> "podheader"},
          {"class" -> "podheader icon"<>tileCount, "id" -> "tab-"<>processedTitle}], {
        XMLElement["div", {"class" -> "podheader-inner"}, {
          XMLElement["a", {"href" -> linkTarget, "class" -> "title"}, {title}],
          Which[
            workflowTileQ,
            XMLElement["div", {"class" -> "svg"}, {XML`RawXML["&raquo;"]}]
          , resourceTileQ, 
            XMLElement[
              "div",
              {"class" -> "svg"},
              {XMLElement["svg", {"viewBox" -> "0 0 22 22"}, {XMLElement["use", {"xlink:href" -> "/language-assets/img/root-guide.svg#"<>resourceIconName[processedTitle]}, {}]}]}
            ]
          , True,
            XMLElement[
              "div",
              {"class" -> "svg"},
              {XMLElement["svg", {"viewBox" -> "0 0 33 33"}, {XMLElement["use", {"xlink:href" -> "/language-assets/img/root-guide.svg#icon_"<>tileCount}, {}]}]}
            ]
          ]
        }]
      }],
      If[StringContainsQ[linkTarget, StartOfString~~"#"],
        XMLElement["div", {"class" -> "links-list"}, {
          Transmogrify[Rest[SelectChildren[]]]
        }],
        {}
      ]
    }]
  ]
],
{Cell, "RootSectionTitle"} :> Recurse[],

{CellGroupData, "RootItemFastIntroductions"} :> XMLElement["ul", {"class" -> "sub_menu"}, {Recurse[]}],
{CellGroupData, "RootItem"} :> XMLElement["ul", {}, {Recurse[]}],
{Cell, "RootItemFastIntroductions"} :> XMLElement["li", {}, {Recurse[]}],

{Cell, "RootItem"} :> XMLElement["li", {}, {Recurse[]}],

{CellGroupData, "RootItemDelimiter"} :> Unevaluated[Sequence[]],
{Cell, "RootItemDelimiter"} :> Unevaluated[Sequence[]]

}, (* end of the transform definitions *)

IncludeXMLTransforms -> {{"HTML","Components","Includes.m"}},

DefaultParameters -> {
  "newline2br" -> True,
  "newline2div" -> False,
  "styleUsageMessageBreak" -> "</div>\n<div class='UsageMessage'>",
  "noop" -> {
    $LanguageExtension = "en";
   }
}

]
