(* :Title: MarketingPages.m *)

(* :Author:
  Andrew Hunt, andy@wolfram.com
*)

(* :Mathematica Version: 6 *)
(* :Package Version: $Revision: 1.230 $ $Date: 2017/07/07 19:45:31 $ *)                  
(* :Copyright: (c) 2006, Wolfram Research, Inc. All rights reserved. *)
(* :Summary: General transform for Mathematica Marketing Pages *)


XMLTransform[{

(* Transform Notebook *)
_Notebook :>  Block[{title, nbExpr, restNBExpr, mainPrev, mainNext, 
	exampleholder, mainFileNameBase, popupFileNames, iNumber, mainFileNameNumber, 
	filename, buildNumber, $NewIn7MarketingNBList, ext, descriptionfile, descriptionfileQ, descriptionfromfile},

Global`AntLog["** Inside Transform Code **"];

$NewIn7SortedList =  {
  "BuiltInImageProcessing",
  "BuiltInParallelComputing",
(* Visualization & Graphics *)
  "AutomatedChartingGraphics",
  "NewGraphicsPrimitives",
  "VectorAndFieldVisualization",
  "ComprehensiveSplineSupport",
(* Mathematics & Algorithms *)
  "IndustrialStrengthBooleanComputation",
  "DiscreteCalculus",
  "DelayDifferentialEquations",
  "DifferentialAndDifferenceRoots",
  "TranscendentalRoots",
  "FiniteGroupTheory",
  "IntegerSequenceAnalysis",
  "NewNumberTheoryCapabilities",
  "NewCategoriesOfSpecialFunctions",
  "EnhancedFourierAnalysis",
(* Computable Data *)
  "IntegratedGenomicAndProteinData",
  "DynamicAstronomicalComputation",
  "CurrentAndHistoricalWeatherData",
  "ExtendedChemicalData",
  "IntegratedGeodesyAndGIS",
(* Data Operations Innovations *)
  "StatisticalModelAnalysis",
  "AutomaticHistogramGeneration",
  "IntegratedProgrammaticEmail",
  "SecureWebsiteImport",
(* Core Language *)
  "SequenceAlignmentAndAnalysis",
(* Interface & User Experience Innovations *)
  "SpeechOutput",
  "EnhancedTypesettingAutomation",
  "OptimizedNotebookErgonomics",
  "QuickStartAssistantPalettes",
  "NewInterfaceElements"
};

$NewIn7MarketingNBList = If[Length[MarketingNBList]<=0,
$NewIn7SortedList,
Sort[MarketingNBList,
  OrderedQ[Map[
    If[Length[#]>0, #[[1, 1]], Infinity]&[Position[$NewIn7SortedList, #]]&,
    {#1, #2}]]&
]];

ext = Switch[GetParameter["Language"], 
	"ChineseSimplified", "zh", 
	"Japanese", "ja", 
	"Spanish", "es", 
	"Russian", "ru",
	"Korean", "ko",
	"Portuguese", "pt-br",
	"German", "de",
	"French", "fr",
	 _, "en"];

speaktext[filename_String, id_String]:="
<script language=\"JavaScript\" type=\"text/javaScript\">
function loadurl(url,frame) {document.getElementById(frame).src=url;}
</script>

<div id=\"sound\">
 <a href=\"#\" onClick=\"javascript:loadurl('../../flash/flashSound.cgi?sound="<>filename<>"','soundFrame"<>id<>"'); return true\"<img src=\"../../images/playbutton.gif\" alt=\"Listen to Output\" border=\"0\">&nbsp;Listen to Output</a>
 </div>
 
<iframe frameborder=\"0\" width=\"1\" height=\"1\" src=\"../../flash/blank.html\" id=\"soundFrame"<>id<>"\" style=\"width:1px; height:1px; border:0px; scrolling:no\" scrolling=\"no\" name=\"soundFrame"<>id<>"\"></iframe>";


  nbExpr = SelectSelf[];
  
  topBody = First@DeleteCases[nbExpr, 
             Cell[CellGroupData[{Cell[_, "MarketingSubpageSection", ___], ___}, ___], ___], Infinity];
             
  featuredimagepath=Transmogrify`Utilities`Private`createDirectory[ToFileName[{"assets"<>"."<>ext}]];

  If[Length[Cases[topBody, Cell[CellGroupData[{Cell[___, "MarketingImageSection", ___], ___,Cell[___, "MarketingTopImage",___], ___}, ___], ___], Infinity]]>0, 
		Export[FileNameJoin[Flatten[{FileNameSplit[featuredimagepath], "featuredimage.png."<>ext}]], Cases[topBody, Cell[CellGroupData[{Cell[___, "MarketingImageSection", ___], ___, b:Cell[___, "MarketingTopImage",___], ___}, ___], ___]:>b, Infinity][[1]], "PNG"],
		Export[FileNameJoin[Flatten[{FileNameSplit[featuredimagepath], "featuredimage.png."<>ext}]], Import[DocsAsset["InternalOnlineImage", "featured-image-default.png"], "PNG"], "PNG"]];

  topBody = DeleteCases[topBody, 
             Cell[CellGroupData[{Cell[___, "MarketingImageSection", ___], ___}, ___], ___], Infinity];
  
  exampleholder = 0;
  title = GetParameter["titleName"];
  mainFileNameBase = GetBasenameLa[];
  mainFileNameNumber = 0;
  filename = StringReplace[DirectoryName@GetParameter["filename"], {DirectoryName@DirectoryName@GetParameter["filename"]->"", "/" | "\\" -> ""}];
  (* find prev/next links *)  
  {mainPrev, mainNext} = DocumentationBuild`Utils`PreviousNextElementList[filename, $NewIn7MarketingNBList];
  iNumber = 0;
  popupFileNames = Switch[GetParameter["Language"], 
  	"Spanish",Flatten @ Cases[nbExpr, Cell[c_, "MarketingSubpageSection", ___,Rule[CellLabel,d_], ___] :> d, Infinity], 
  	"Japanese",Flatten @ Cases[nbExpr, Cell[c_, "MarketingSubpageSection", ___,Rule[CellLabel,d_], ___] :> d, Infinity], 
  	"Korean",Flatten @ Cases[nbExpr, Cell[c_, "MarketingSubpageSection", ___,Rule[CellLabel,d_], ___] :> d, Infinity], 
  	"French",Flatten @ Cases[nbExpr, Cell[c_, "MarketingSubpageSection", ___,Rule[CellLabel,d_], ___] :> d, Infinity], 
    "German",Flatten @ Cases[nbExpr, Cell[c_, "MarketingSubpageSection", ___,Rule[CellLabel,d_], ___] :> d, Infinity],   	  	
  	"Russian",Flatten @ Cases[nbExpr, Cell[c_, "MarketingSubpageSection", ___,Rule[CellLabel,d_], ___] :> d, Infinity], 
  	"Portuguese",Flatten @ Cases[nbExpr, Cell[c_, "MarketingSubpageSection", ___,Rule[CellLabel,d_], ___] :> d, Infinity], 
  	"ChineseSimplified", Flatten @ Cases[nbExpr, Cell[c_, "MarketingSubpageSection", ___,Rule[CellLabel,d_], ___] :> d, Infinity], 
  	_, Flatten @ Cases[nbExpr, Cell[c_, "MarketingSubpageSection", ___] :> c, Infinity]];
  popupFileNames = 
    Flatten[ 
      {
      name = ToLowerCase[StringReplace[ Transmogrify`ConvertToString[#], {"|"->"",","->"","."->"","\""->"", "'"->"","\'"->"", "^"->"", "&" -> "And", "\[CloseCurlyQuote]"->"", "\[CloseCurlyDoubleQuote]"->"", "\[OpenCurlyQuote]"->"", "\[OpenCurlyDoubleQuote]"->"", " "->"-","\[EAcute]"->"e","\[Pi]"->"pi", "\[ScriptCapitalD]"->"D", ""->"e", "\[Hyphen]"->"-", "\[Dash]"->"-", "\[LongDash]"->"-","-"->"-","/"->"-"}]];
      name = If[StringLength[name]>80, StringTake[name, 80], name];
      name = StringTrim[name, ("+" | "-" | ".") ...];
      name = deleteNonASCII[name];
      If[ (name ===
          "MarketingSubpageTitle" || name ===
          "Marketing Subpage Title..."), "marketing-subpage-title-"<>IncrementCounter["CommonTitle"], name]
      }& /@ popupFileNames ];

  (* determine build number *)
  buildNumber = StringCases[GetAbsoluteFileName[], RegularExpression["\\d\\d\\d\\d\\d\\d"]];
  buildNumber = If[Length@buildNumber > 0, First@buildNumber, "Indeterminable"];
  descriptionfile = FileNameJoin[Flatten[{FileNameSplit[DirectoryName[FileNameJoin[Flatten[{FileNameSplit[AbsoluteFileName[GetParameter["ExamplesRawMaterialDir"]]]}]]]], "Descriptions.m"}]];
  descriptionfileQ = FileExistsQ[descriptionfile];
  descriptionfromfile = If[descriptionfileQ, filename /. Import[descriptionfile, "Package"], filename];
 
  (* layout file *)
  FillInLayout[
    GetParameter["layoutfile"],
    "Text",
   (* "localcss" -> GetParameter["localcss"], *)
    "la" -> GetParameter["la"], (* guaranteed 2-letter language code *)
    "langext"->Switch[GetParameter["Language"], 
    	"ChineseSimplified", ".zh", 
    	"Spanish", ".es", 
    	"Japanese", ".ja",
    	"Korean", ".ko",
    	"Russian", ".ru",
    	"Portuguese", ".pt-br",
    	"German", ".de",
    	"French", ".fr",
    	 _, ".en"],
    "lang"->Switch[GetParameter["Language"], 
    	"ChineseSimplified", "zh", 
    	"Spanish", "es", 
    	"Japanese", "ja",
    	"Korean", "ko",
    	"Russian", "ru",
    	"Portuguese", "pt-br",
    	"German", "de",
    	"French", "fr",
    	 _, "en"],
    "contentheading"->title,
    "uri"->filename,
    "titleandmeta"->ExportString[
      XML`RawXML[ StringJoin[ExportString[#, "XML"] & /@ 
        Flatten[{
          XMLElement["title", {}, {title<>": "<>Localized["MarketingTitleAddition", GetParameter["Language"]]}],
          XMLElement["meta", {"name"->"original-source", "content"->FileBaseName[GetParameter["InFileName"]]<>".nb" }, {}],
          Which[
          	StringLength[GetParameter["description"]] > 0, 
            XMLElement["meta", {
            "name"->"description", 
            "content"->GetParameter["description"] }, {}], 
            descriptionfromfile =!= filename ,
            XMLElement["meta", {
            "name"->"description", 
            "content"->descriptionfromfile }, {}],
            True, {}],
          If[Length@GetParameter["keywords"] > 0, 
            XMLElement["meta", {
            "name"->"keywords", 
            "content"->StringList[GetParameter["keywords"]] }, {}], {}],
          If[Length@GetParameter["synonyms"] > 0, 
            XMLElement["meta", {
            "name"->"synonyms", 
            "content"->StringList[GetParameter["synonyms"]] }, {}], {}],
          If[GetParameter["Language"] ==="Russian", 
            XMLElement["meta", {
            "http-equiv"->"Content-Type", 
            "content"->"text/html; charset=utf-8" }, {}], {}],
          If[GetParameter["Language"] ==="German", 
            XMLElement["meta", {
            "http-equiv"->"Content-Type", 
            "content"->"text/html; charset=utf-8" }, {}], {}],
          If[GetParameter["Language"] ==="French", 
            XMLElement["meta", {
            "http-equiv"->"Content-Type", 
            "content"->"text/html; charset=utf-8" }, {}], {}],
          If[GetParameter["Language"] ==="Korean", 
            XMLElement["meta", {
            "http-equiv"->"Content-Type", 
            "content"->"text/html; charset=utf-8" }, {}], {}]
        }]]], 
      "XML"],
    "topicname"->"",
    "body"->
       StringReplace[ExportString[
         DIV[{"class"->"page-width grid"}, Flatten@{
           Recurse [DeleteCases[topBody, Cell[___,"MarketingTitleAlt"|"MarketingTitleSubheader", ___], Infinity]]
         }],
        "XML",
          "AttributeQuoting"->"\"", "Entities"->{"\[LongDash]"->"&mdash;","\[LeftGuillemet]"->"<span class=\"chevron\">&laquo;</span>","\[RightGuillemet]"->"<span class=\"chevron\">&#187;</span>"}, "ElementFormatting"->None],
          {"VIRTUAL-INCLUDES-BEGIN"->"<!--#include virtual='","VIRTUAL-INCLUDES-END"->"'-->"}]
  ] 
],


(************************

Define cell transforms 

************************)

(* General styles *)

{Cell,"MarketingTitleSubheader"} :> Sequence[],

{Cell,"MarketingTitleAlt"} :> H1[{}, {Recurse[]} ],

{Cell, "MarketingTitle"} :> Sequence[],

{CellGroupData,"MarketingMainSection"} :> Recurse[],

{Cell, "MarketingMainSection"} :> Sequence[],

{Cell, "MarketingSubpageRow"} :> Sequence[],

{Cell, "MarketingThumbnailInput"} :> Sequence[], 

{CellGroupData, "MarketingText"} :>
  {	CreateDirectory["assets"<>"."<>ext];
	CreateDirectory["assets"<>"."<>ext<>"/inc"];
  	Export["assets"<>"."<>ext<>"/inc/thumbnails.html."<>ext, StringReplace[ExportString[MakeMarketingThumbnailList[ Transpose[{
             Recurse /@ Cases[nbExpr, Cell[CellGroupData[{Cell[_, "MarketingSubpageSection", ___], ___}, ___], ___], Infinity],
             Recurse /@ Cases[nbExpr, Cell[_, "MarketingSubpageSection", ___], Infinity]
             }], GetParameter["Language"]], "XML", "ElementFormatting"->None, "AttributeQuoting"->"\""], {" />"->">"}], "Text"]; 
   Recurse[]},

{Cell, "MarketingText"} :> Module[{},
	CreateDirectory["assets"<>"."<>ext];
	CreateDirectory["assets"<>"."<>ext<>"/inc"];
	Export["assets"<>"."<>ext<>"/inc/introparagraph.html."<>ext, P[{}, {Recurse[]}], "XML"];
	{}],

{Cell, "Delimiter"} :> DIV[{"class" -> "line"}, {" "}],


(* Thumnails and Popup pages *)
{Cell, "MarketingSubpageSection"} :> 
Module[{r, f, sn},
  r = Recurse[];
  sn = Switch[GetParameter["Language"], 
  	"Spanish", Cases[SelectSelf[], Rule[CellLabel,d_]:>d, Infinity][[1]], 
  	"Japanese", Cases[SelectSelf[], Rule[CellLabel,d_]:>d, Infinity][[1]], 
  	"ChineseSimplified", Cases[SelectSelf[], Rule[CellLabel,d_]:>d, Infinity][[1]], 
  	"Russian", Cases[SelectSelf[], Rule[CellLabel,d_]:>d, Infinity][[1]], 
  	"Korean", Cases[SelectSelf[], Rule[CellLabel,d_]:>d, Infinity][[1]],
    "German", Cases[SelectSelf[], Rule[CellLabel,d_]:>d, Infinity][[1]], 
  	"French", Cases[SelectSelf[], Rule[CellLabel,d_]:>d, Infinity][[1]], 
  	"Portuguese", Cases[SelectSelf[], Rule[CellLabel,d_]:>d, Infinity][[1]], 
  	_, SelectSelf[]];
  sn = ToLowerCase[StringReplace[ Transmogrify`ConvertToString[sn], {"|"->"",","->"", "."->"", "\""->"", "'"->"", "\'"->"", "^"->"", "&" -> "And"," "->"-","\[CloseCurlyQuote]"->"", "\[CloseCurlyDoubleQuote]"->"", "\[OpenCurlyQuote]"->"", "\[OpenCurlyDoubleQuote]"->"", "\[EAcute]"->"e","\[Pi]"->"pi","\[ScriptCapitalD]"->"D",""->"e", "\[Hyphen]"->"-", "\[Dash]"->"-","\[LongDash]"->"-", "-"->"-", "/"->"-"}]];
  sn = If[StringLength[sn]>80, StringTake[sn, 80], sn];
  sn = StringTrim[sn, ("+" | "-" | ".") ...];
  sn = deleteNonASCII[sn];
  sn = If[(sn === "MarketingSubpageTitle" || sn ==="Marketing Subpage Title..."), "marketing-subpage-title-"<>IncrementCounter["CommonTitle"], sn];
  XMLElement["a",{"href"->sn<>".html", "onclick"->"return pop('"<>sn<>".html"<>"')"},{r}]
 ], 

{CellGroupData, "MarketingSubpageSection"} :> 
Block[{sectionSource, initialcaptionSource, captionSource, styleList, caption, styles, windowSize, file, inputType, stuff, 
vTemp, hTemp, prev, next, thumb, popup, group, textGroup, sectionName, popupOutput, thumFilename,thumURL,
thumbnail, srcThumb, wThumb, hThumb, count, srcHR, wHR, hHR, labelClass, filepathname, thumbnailfilepathname, ext, isInputHiddenQ},
ext = Switch[GetParameter["Language"], 
	"ChineseSimplified", "zh", 
	"Japanese", "ja", 
	"Spanish", "es", 
	"Russian", "ru",
	"Korean", "ko",
	"German", "de",
	"French", "fr",
	"Portuguese", "pt-br",
	 _, "en"];
  sectionSource = SelectSelf[];
  sectionSource = sectionSource /. {Cell[c_, "MarketingText", o___] :> Cell[c,"MarketingExampleDetail", o]};
  initialcaptionSource=First@SelectChildren[];
  initialcaptionSource = initialcaptionSource /. {Cell[c_, "MarketingText", o___] :> Cell[c,"MarketingExampleDetail", o]};
  captionSource = initialcaptionSource;
  captionSource = Switch[GetParameter["Language"], 
  	"Spanish", Cases[captionSource, Rule[CellLabel,d_]:>d, Infinity][[1]], 
  	"Japanese", Cases[captionSource, Rule[CellLabel,d_]:>d, Infinity][[1]], 
  	"ChineseSimplified", Cases[captionSource, Rule[CellLabel,d_]:>d, Infinity][[1]], 
  	"Russian", Cases[captionSource, Rule[CellLabel,d_]:>d, Infinity][[1]], 
  	"Korean", Cases[captionSource, Rule[CellLabel,d_]:>d, Infinity][[1]], 
  	"German", Cases[captionSource, Rule[CellLabel,d_]:>d, Infinity][[1]], 
  	"French", Cases[captionSource, Rule[CellLabel,d_]:>d, Infinity][[1]], 
  	"Portuguese", Cases[captionSource, Rule[CellLabel,d_]:>d, Infinity][[1]], 
  	_, captionSource];
  caption = Recurse@initialcaptionSource;
  If[ DocumentationBuild`Transform`debug, Global`AntLog["*", ToString@caption ] ];
  isInputHiddenQ = If[Length@Cases[sectionSource, "InputHiddenMaterial", Infinity] > 0, True, False];

  (* custom WindowSize? *)
  windowSize = DocumentationBuild`Make`Private`GetWindowSizeCellTag[sectionSource];

  windowSize = 
    If[Length@windowSize>0, 
    If[ DocumentationBuild`Transform`debug, Global`AntLog["___Custom WindowSize -> : " <> ToString@windowSize ] ];
    windowSize, 
    {595, 700} ];
  
  mainFileNameNumber++;

  sectionName = ToLowerCase[StringReplace[ Transmogrify`ConvertToString[captionSource], {"|"->"",","->"","."->"","\""->"", "'"->"", "\'"->"", "\[CloseCurlyQuote]"->"", "\[CloseCurlyDoubleQuote]"->"", "\[OpenCurlyQuote]"->"", "\[OpenCurlyDoubleQuote]"->"","^"->"", "&" -> "And"," "->"-","\[EAcute]"->"e","\[Pi]"->"pi","\[ScriptCapitalD]"->"D", ""->"e", "\[Hyphen]"->"-", "\[LongDash]"->"-","\[Dash]"->"-", "-"->"-", "/"->"-"}]];
  sectionName = If[StringLength[sectionName] > 80, StringTake[sectionName, 80], sectionName];
  sectionName = StringTrim[sectionName, ("+" | "-" | ".") ...];
  sectionName = deleteNonASCII[sectionName];
  sectionName = If[(sectionName === "MarketingSubpageTitle" || sectionName ==="Marketing Subpage Title..."), "marketing-subpage-title-"<>IncrementCounter["CommonTitle"], sectionName];
  (* styles used in group *)
  Global`AntLog[ToString[sectionName]];
  Global`AntLog[ToString[Date[]]];
  styleList = DocumentationBuild`Utils`GetBlockStyleList[Cell@sectionSource];
  labelClass = If[ DocumentationBuild`Make`Private`HideCellLabelQ[styleList], "label_Hide", "label_Show"];
  
  (* find prev/next links *)  
  {prev, next} = DocumentationBuild`Utils`PreviousNextElementList[sectionName, popupFileNames];
  
If[!ValueQ[DocumentationBuild`ConversionList], DocumentationBuild`ConversionList = {}];
  AppendTo[DocumentationBuild`ConversionList, 
      { "\"Filename\" -> \"" <> filename <> "\", \"PopupFilename\" -> \"" <> sectionName <> "\", \"Styles\" -> {\"" <> StringJoin @@ Riffle[styleList, "\",\""] <> "\"}, \"HiddenInputQ\"->" <> ToString@DocumentationBuild`Make`Private`HiddenInputQ[styleList] <> ", \"HideCellLabelQ\"->" <> ToString@DocumentationBuild`Make`Private`HideCellLabelQ[styleList] <> ", CellLabel->\"" <> labelClass <> "\"}" }];

  file = sectionName <>".html."<>ext;
  count = IncrementCounter["Image"];
  group = Rest[SelectChildren[]];
  group = group /. {Cell[c_, "MarketingText", o___] :> Cell[c,"MarketingExampleDetail", o]};
  group = group /. Cell[c_, "MarketingText", o___] :> Cell[c,"PopupCaptionText", o];
(*  group = group /. Cell[BoxData[InterpretationBox[stuff:Cell[a_, b___], ___]], o___] :> Cell[a, o]; *)
  
  textGroup = Cases[group, Cell[_, "PopupCaptionText", ___], Infinity];
  group = DeleteCases[group, Cell[_, "PopupCaptionText", ___], Infinity];

  popupOutput = Cases[group, Cell[CellGroupData[{Cell[_,"InputThumbnailMaterial", ___], ___}, ___], ___], Infinity];
 
  If[Length@popupOutput < 1,
    popupOutput = Cases[ group, Cell[_, "InputThumbnailMaterial", ___], Infinity]
  ];
  (* no marked cells for thumbnail *)
  If[Length@popupOutput < 1,
    popupOutput = Cases[  DeleteCases[group, Cell[___, "WolframAlphaShortInput"|"WolframAlphaLong", ___], Infinity], 
        Cell[_, Alternatives@@{"MarketingThumbnailOutput"}, ___], Infinity] 
  ];
  
  group = group /. { Cell[con_, sty_String, o___] -> Cell[con, sty, o] };
  popupOutput = thumbnail = { Flatten[popupOutput /. Cell[a_, "ContentOutput", b___] :> Cell[a, "Output", b] ] };

  group = DeleteCases[group, Cell[__, CellTags -> tags_ /; MemberQ[Flatten[{tags}], "SuppressInput"], ___], Infinity];

  thumbnail = DeleteCases[thumbnail, Cell[___, "OutputSpecial", ___], Infinity];
  
  thumbnail = If[Length[Cases[thumbnail, Cell[___, "MarketingThumbnailOutput", ___], Infinity]]>0, Cases[thumbnail, Cell[___, "MarketingThumbnailOutput", ___], Infinity],thumbnail];

  thumbnail = If[Length[Flatten[{thumbnail}]] == 0, Cell["\[SadSmiley]", "MarketingThumbnailOutput", FontSize->300], thumbnail];

  thumbnail = Cell[BoxData[ToBoxes@Rasterize[
    Notebook[Flatten[{thumbnail}], 
           WindowSize -> windowSize, ShowCellLabel->False, ShowCellBracket->False, CellBracketOptions->{"Color"->GrayLevel[1]}, (*PageWidth->WindowWidth,*)
           StyleDefinitions -> 
             Notebook[{(*Cell[ StyleData[ StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "MarketingPageStyles.nb"]]],*) 
               Cell[StyleData[All, "Working"], ShowCellBracket->False, CellBracketOptions->{"Color"->GrayLevel[1]}],
               Cell[StyleData["InlineFormula"], FontFamily->"Source Sans Pro", FontSize->Inherited 1.08],
               Cell[StyleData["MR"], FontFamily->"Source Sans Pro"],
               Cell[StyleData["StandardForm"],FontFamily->"Source Sans Pro"],
               Cell[StyleData["InputForm"],FontFamily->"Source Sans Pro"],
               Cell[StyleData["Input"],  FontFamily->"Source Sans Pro", FontSize->13, LineSpacing->{1.2, 0}, PageWidth->525],
               Cell[StyleData["Output"], FontSize->13],
               Cell[StyleData["InputHiddenMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputHiddenMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputSuppressedMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputThumbnailMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1]
             }]]]], "Output", ShowCellLabel->False, ShowCellBracket->False];


filepathname=Transmogrify`Utilities`Private`createDirectory[ToFileName[{"assets"<>"."<>ext, sectionName}]];
Export[FileNameJoin[Flatten[{FileNameSplit[filepathname], "smallthumb_" <> count <> ".png"}]], ImageResize[Rasterize[thumbnail], 300], "PNG"];
  srcThumb = StringJoin["assets"<>"."<>ext<>"/", sectionName, "/smallthumb_" <> count <> ".png"];
 {wThumb, hThumb} = Import[FileNameJoin[Flatten[{FileNameSplit[filepathname], "smallthumb_" <> count <> ".png"}]], "ImageSize"];

  (* Popup layout *)
  Export[ file, 
  FillInLayout[
    GetParameter["popupLayoutfile"],
    "Text",
    "langext"->Switch[GetParameter["Language"], 
    	"ChineseSimplified", ".zh", 
    	"Spanish", ".es", 
    	"Japanese", ".ja",
    	"Korean", ".ko",
    	"Russian", ".ru",
    	"Portuguese", ".pt-br",
    	"German", ".de",
    	"French", ".fr",
    	 _, ".en"],
        "lang"->Switch[GetParameter["Language"], 
    	"ChineseSimplified", "zh", 
    	"Spanish", "es", 
    	"Japanese", "ja",
    	"Korean", "ko",
    	"Russian", "ru",
    	"Portuguese", "pt-br",
    	"German", "de",
    	"French", "fr",
    	 _, "en"],
    "title" -> ExportString[XMLElement["title", {}, {Transmogrify`ConvertToString[initialcaptionSource]<>": "<>Localized["MarketingTitleAddition", GetParameter["Language"]]}], "XML","AttributeQuoting"->"\"", "Entities"->{"\[LongDash]"->"&mdash;"}, "ElementFormatting"->None], 
    "body"->
       StringReplace[ExportString[
         XMLElement["div", {"class"->"page-content"}, Flatten[{
         	XMLElement["a", {"href"->prev<>".html", "class"->"prev"},{XMLElement["span", {}, {XML`RawXML["&lsaquo;"]}]}],
         	XMLElement["a", {"href"->next<>".html", "class"->"next"},{XMLElement["span", {}, {XML`RawXML["&rsaquo;"]}]}],
         	XMLElement["div", {"class"->"breadcrumb-links"}, {
         		XMLElement["a", {"href"->"index.html", "class"->"breadcrumb arrow-before"}, {Recurse@GetParameter["titleName"]}],
         		XMLElement["a", {"href"->"/language/new-in-12/", "class"->"latest-feature-link"}, {Localized["MarketingLatestFeature", GetParameter["Language"]]}]
         	}],
         	H2[{}, Flatten@{DeleteCases[DeleteCases[Last[caption], XMLElement["span",{"class"->"chevron"},{_}], Infinity], "\[RightGuillemet]", Infinity]}],
			Recurse@textGroup,
            FixedPoint[Replace[#, {x___, y : Longest@PatternSequence[XMLElement["section", {"class" -> "suppress-hover hide-show-section"|"suppress-hover-normal hide-show-section"}, {___}] ..], z___} :> 
           	{x, XMLElement["div", {"class" -> "toggle-controls"}, {XML`RawXML["<i></i>"], Localized["MarketingShowInput", GetParameter["Language"]]}],XMLElement["div", {"class" -> "toggle-me"}, {y}], z}] &, Flatten@{Recurse[group]}]
           			}]],
        "XML",
          "AttributeQuoting"->"\"", "Entities"->{"\[LongDash]"->"&mdash;"}, "ElementFormatting"->None], "href=\"http://"->"target=\"_blank\" href=\"http://"]
    ], "Text"
  ];

  If[(Length@popupOutput > 0),
    A[{"href"->StringReplace[file, {".en."->".",".ja."->".",".zh."->"."}]}, {
        XMLElement["img", {"src"->srcThumb, "height"->ToString[Floor[hThumb/2]], "width"->ToString[Floor[wThumb/2]], "alt"->""}, {}]
        }],
    ""
  ]
],



(** Input Cell Groups **)
{CellGroupData, "Input" | "InputHiddenMaterial" | "InputSuppressedMaterial" | "ExternalLanguage" | "InputThumbnailMaterial" | "InputPopupMaterial" | "InputMainPageMaterial" | "InputMainPopupMaterial"} :> 
Block[{style, showInput=True (*, labelClass="sessionLabel"*)},
  style = GetStyle[];
  Flatten@{
    Recurse@Cases[SelectSelf[], Cell[_, "calloutbox", ___], Infinity],
    Flatten@{
      Recurse@Cases[SelectSelf[], Cell[_, "Input"|"Output"|"ExternalLanguage"|"Message"|"NotebookImage"|"InputHiddenMaterial" | "InputSuppressedMaterial" | "InputThumbnailMaterial" | "Picture" | "Print" | "Echo" | "ShowLinkedVideo" | "ShowLinkedFile" | "Graphics", ___], 5]
      }
  }
],

(*  Input / Output Images *)
{Cell, "Input" | "WolframAlphaShortInput" | "WolframAlphaLong" | "ExternalLanguage" | "InputHiddenMaterial" | "InputThumbnailMaterial" | "InputPopupMaterial" | "InputMainPageMaterial" | "InputMainPopupMaterial" | "InputNoClipboardAction"} :> 
Module[{expr, data, w, h, thumb, inputCell, inManipulateCount, graphic, duration, incount, iId, ext, iInner, iOuter, inputForm, inputFormQueue, animatedGifQ, isManipulate, isExternalLanguageQ, style, cTags, suppressInputPopup, altText, hasGraphicQ, isHiddenQ},
  ext = Switch[GetParameter["Language"], 
	"ChineseSimplified", "zh", 
	"Japanese", "ja", 
	"Spanish", "es", 
	"Russian", "ru",
	"Korean", "ko",
	"German", "de",
	"French", "fr",
	"Portuguese", "pt-br",
	 _, "en"];
  expr = SelectSelf[];
  style = GetStyle[];
  iNumber++;
  cTags = If[HasOption[CellTags], Flatten[ {GetOption[CellTags]}], {}];
    If[ DocumentationBuild`Transform`debug && Length@cTags>0, Global`AntLog["___CellTags: "<>ToString@cTags]];
  suppressInputPopup = MemberQ[cTags, "SuppressPopup"] || SameQ[style, "InputNoClipboardAction"];
  altText = If[ !suppressInputPopup, "Click for copyable input", ""];
  animatedGifQ = MemberQ[cTags, "AnimatedGIF"];
  cTags = cTags //. {a___, sty_String /; StringMatchQ[sty, "SuppressPopup" | "{WindowSize*"], b___} :> {a, b};
  
  iId = StringJoin["i_", ToString@iNumber];
  iInner = StringJoin[iId, "_in"];
  iOuter = StringJoin[iId, "_out"];
  inputForm = CreateInputForm[ SelectSelf[] ];
  inputForm = If[Head@inputForm === String, inputForm, ToString@inputForm];
  inputFormQueue = If[MemberQ[cTags, "DoNotGetPreviousInputs"], inputForm, inputQueue[inputForm, GetOption[CellLabel]]];
  (* 4 MB limit for copyable input *)
  If[ByteCount[inputFormQueue] > 4000000, suppressInputPopup = True];
  preservedInputForm = inputForm;

  
  isManipulate = DocumentationBuild`Utils`IsManipulateQ[ SelectSelf[] ];
  hasGraphicQ = If[Length@Cases[expr, GraphicsBox[___], Infinity] > 0, True, False];
  isHiddenQ = If[Length@Cases[expr, "InputHiddenMaterial", Infinity] > 0, True, False];
  isExternalLanguageQ = If[Length@Cases[expr, "ExternalLanguage", Infinity] > 0, True, False];

  incount = IncrementCounter["iInner"];
  If[isExternalLanguageQ,
  inputCell=Notebook[Flatten[{
  AppendTo[expr, AutoStyleOptions->{"HighlightExcessArguments"->False,"HighlightUnknownOptions"->False, "UnknownOptionStyle"->{FontColor -> RGBColor[0., 0., 0.]}}],
  AppendTo[expr, AutoStyleOptions->{"HighlightExcessArguments"->False,"HighlightUnknownOptions"->False, "UnknownOptionStyle"->{FontColor -> RGBColor[0., 0., 0.]}}],
  AppendTo[expr, AutoStyleOptions->{"HighlightExcessArguments"->False,"HighlightUnknownOptions"->False, "UnknownOptionStyle"->{FontColor -> RGBColor[0., 0., 0.]}}]}], 
    WindowSize -> 1000, ShowCellLabel->False, ShowCellBracket->False, Magnification->2, CellBracketOptions->{"Color"->GrayLevel[1]},
           StyleDefinitions -> 
             Notebook[{
               Cell[StyleData[StyleDefinitions -> "Core.nb"]],
               Cell[StyleData[ StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "MarketingPageStyles.nb"]]], 
               Cell[StyleData[All, "Working"], ShowCellBracket->False, CellBracketOptions->{"Color"->GrayLevel[1]}],
               Cell[StyleData["InlineFormula"], FontFamily->"Source Sans Pro", FontSize->Inherited 1.08],
               Cell[StyleData["MR"], FontFamily->"Source Sans Pro"],
               Cell[StyleData["StandardForm"],FontFamily->"Source Sans Pro"],
               Cell[StyleData["InputForm"],FontFamily->"Source Sans Pro"],
               Cell[StyleData["ExternalLanguage"], PageWidth->495],
               Cell[StyleData["Input"],  FontFamily->"Source Sans Pro", FontSize->13, LineSpacing->{1.2, 0}, PageWidth->525],
               Cell[StyleData["Output"], FontSize->13],
               Cell[StyleData["InputNoClipboardAction", StyleDefinitions->StyleData["Input"]], CellID -> 1],
               Cell[StyleData["InputHiddenMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputSuppressedMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputThumbnailMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1]
             }]],
    inputCell=Notebook[Flatten[{AppendTo[expr, AutoStyleOptions->{"HighlightExcessArguments"->False,"HighlightUnknownOptions"->False, "UnknownOptionStyle"->{FontColor -> RGBColor[0., 0., 0.]}}]}], 
           WindowSize -> windowSize, ShowCellLabel->False, ShowCellBracket->False, Magnification->2, CellBracketOptions->{"Color"->GrayLevel[1]},
           StyleDefinitions -> 
             Notebook[{
               Cell[StyleData[StyleDefinitions -> "Core.nb"]],
               Cell[StyleData[ StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "MarketingPageStyles.nb"]]], 
               Cell[StyleData[All, "Working"], ShowCellBracket->False, CellBracketOptions->{"Color"->GrayLevel[1]}],
               Cell[StyleData["InlineFormula"], FontFamily->"Source Sans Pro", FontSize->Inherited 1.08],
               Cell[StyleData["MR"], FontFamily->"Source Sans Pro"],
               Cell[StyleData["StandardForm"],FontFamily->"Source Sans Pro"],
               Cell[StyleData["InputForm"],FontFamily->"Source Sans Pro"],
               Cell[StyleData["Input"],  FontFamily->"Source Sans Pro", FontSize->13, LineSpacing->{1.2, 0}, PageWidth->525],
               Cell[StyleData["Output"], FontSize->13],
               Cell[StyleData["InputNoClipboardAction", StyleDefinitions->StyleData["Input"]], CellID -> 1],
               Cell[StyleData["InputHiddenMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputSuppressedMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputThumbnailMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1]
             }]]
             ];
              
              If[isExternalLanguageQ,
              {image, {width, height, baseline}} = Rasterize[inputCell, {"Image", "BoundingBox"}, TransparentBackground -> True];
              image = ImagePartition[image, {width, Ceiling[height/3]}][[2]][[1]];
              {width, height} = ImageDimensions[image]
              ,
              {image, {width, height, baseline}} = Rasterize[inputCell, {"Image", "BoundingBox"}, TransparentBackground -> True, Background -> None]];
	          padwidth = 2 - Mod[width, 2];
	     	  padheight = 2 - Mod[height, 2];
	     	  image = ImagePad[image, {{0, padwidth}, {padheight, 0}}, Transparent];
	     	  {width, height} = ImageDimensions[image];
              
              Quiet[CreateDirectory[ToFileName[{"assets"<>"."<>ext, sectionName}]]];
                
              Export[
                ToFileName[{"assets"<>"."<>ext, sectionName}, "In_" <> incount <> ".png"],
                image,
                "PNG",
                Background -> None
              ];
              
              {w, h} = Ceiling[{width, height}/2];
             

Sequence@@Flatten@{  

XMLElement["section", {"class"->If[ !suppressInputPopup,"suppress-hover", "suppress-hover-normal"]<>If[ !isHiddenQ,"", " hide-show-section"]},  Flatten@{  
    XMLElement["div", Flatten@{
		    		"class" -> "clipboard-input"<>If[!suppressInputPopup,"", " not-copiable"]<>If[isExternalLanguageQ," externallanguage", ""],
		    		"data-in"->If[
		            HasOption[CellLabel],
		            GetOption[CellLabel],
		            ""
		          ],
		          "data-in-num" -> StringReplace[If[
		            HasOption[CellLabel],
		            GetOption[CellLabel],
		            ""
		          ], {"In["->"", "]:="->""}],
		    		  "data-in-src" -> StringJoin[ToFileName[{"assets."<>ext, sectionName},"In_" <> incount <> ".png"]],
		    		  "data-in-width" -> ToString[w],
		    		  "data-in-height" -> ToString[h],
		    		  If[ !suppressInputPopup, Export[ToFileName[{"assets"<>"."<>ext, sectionName}, "In_" <> incount <> ".txt"], inputFormQueue, "Text"]; Rule["data-code", StringJoin[ToFileName[{"assets."<>ext, sectionName},"In_" <> incount <> ".txt"]]], {}]
		    		  }, {""}
		      ]
		    }
  ]}
 ],
   
{Cell, "ShowLinkedVideo"}:>Module[{vimeonumber, ffmpeg = DocumentationBuild`$ffmpeg, videoarray, videocounter, jpgoutputlocation, videoCopyQ, videoPath},
	Which[
		StringMatchQ[Transmogrify`ConvertToString[SelectSelf[]], "*vimeo*"],
		  vimeonumber = Last[StringSplit[Transmogrify`ConvertToString[SelectSelf[]], "/"]];
		  XMLElement["div", {"class"->"embeddedvideo"}, {XMLElement["iframe", {
		  "src" -> "//player.vimeo.com/video/"<>vimeonumber, 
		  "width" -> "550", "height" -> "309", 
		  "frameborder" -> "0", "webkitallowfullscreen" -> "", 
		  "mozallowfullscreen" -> "", 
		  "allowfullscreen" -> ""}, {" "}]}],
		  Length[Cases[SelectSelf[], TemplateBox[___,"LocalVideoFile",___], Infinity]]==1,
		  
		    videoarray = Flatten[Cases[SelectSelf[], TemplateBox[va___,"LocalVideoFile",___]:>va, Infinity]];
		  
		    videocounter = IncrementCounter["videocount"];
	    
		    CreateDirectory[ToFileName[{"assets"<>"."<>ext, sectionName}]];
		    
		    jpgoutputlocation = ToFileName[{"assets"<>"."<>ext, sectionName}, "Video_" <> videocounter <> ".jpg"];
		    
			exit = Run[
				StringJoin[
				  ffmpeg,
				  " -ss 0. -i ", 
				  FileNameJoin[Flatten[{FileNameSplit[AbsoluteFileName[GetParameter["ExamplesRawMaterialDir"]]], videoarray[[1]]}]],
				  " -t 1 -s ",
				  videoarray[[2]],"x",videoarray[[3]],
				  " -f image2 -y ", jpgoutputlocation
				]
			  ];

			If[Not[FileExistsQ[jpgoutputlocation]],
				Print["WARNING: MP4 JPG conversion failed"];
				Global`AntLog["WARNING: MP4 JPG conversion failed"];
			  ];
		    
			{width, height} = ToExpression[{videoarray[[2]], videoarray[[3]]}];

			Global`AntLog["Copy file from: "<>FileNameJoin[Flatten[{FileNameSplit[AbsoluteFileName[GetParameter["ExamplesRawMaterialDir"]]], videoarray[[1]]}]]];

			videoCopyQ=CopyFile[FileNameJoin[Flatten[{FileNameSplit[AbsoluteFileName[GetParameter["ExamplesRawMaterialDir"]]],videoarray[[1]]}]],
			ToFileName[{"assets"<>"."<>ext, sectionName}, "Video_" <> videocounter <> ".mp4"], OverwriteTarget->True];

			If[videoCopyQ =!= $Failed, Global`AntLog["Video file copied"], Global`AntLog["Video file not copied"]];

			videoPath = "assets"<>"."<>ext<>"/"<>sectionName<>"/"<>"Video_" <> videocounter <> ".mp4";

			Global`AntLog["Video file path: "<>videoPath];
		  
			XMLElement["section", {}, {XMLElement["div", Flatten[{
	    		"class" -> "clipboard-output",
	    		"data-out"->"",
	              "data-out-num" -> "",
	    		  "data-out-src" -> jpgoutputlocation,
	    		  "data-out-width" -> ToString[width],
	    		  "data-out-height" -> ToString[height],
	    		  "data-video" -> ToString[width]<>" "<>ToString[height],
	    		  "data-media" -> videoPath
	    		  }], {""}]}],
			True,{}
			]
		  
		  ],
	  
{Cell, "ShowLinkedFile"}:>Module[{videoCopyQ, videoPath, videocount, jpgPath, width=400, height=400, slfexpr, vimeonumber,cellTags, scale},
	Which[
	StringMatchQ[Transmogrify`ConvertToString[SelectSelf[]],"*vimeo*"],
	vimeonumber=Last[StringSplit[Transmogrify`ConvertToString[SelectSelf[]],"/"]];
	XMLElement["div",{"class"->"embeddedvideo"},{XMLElement["iframe",{"src"->"//player.vimeo.com/video/"<>vimeonumber,"width"->"550","height"->"309","frameborder"->"0","webkitallowfullscreen"->"","mozallowfullscreen"->"","allowfullscreen"->""},{" "}]}]
	,
	StringMatchQ[Transmogrify`ConvertToString[SelectSelf[]],"*.zip*"],
	zipfile=FileNameJoin[Flatten[{FileNameSplit[AbsoluteFileName[GetParameter["ExamplesRawMaterialDir"]]],FileNameSplit[Cases[{SelectSelf[]},Cell[a_String,___]:>a,Infinity][[1]]]}]];
	zipfilebasename=FileBaseName[zipfile];
	CreateDirectory[ToFileName[{"assets"<>"."<>ext,sectionName}]];
	ExtractArchive[zipfile,ToFileName[{"assets"<>"."<>ext,sectionName}]];
	XMLElement["section",{},{XMLElement["div",{"class"->"iframe-content"},{XMLElement["iframe",{"src"->StringJoin[ToFileName[{"assets."<>ext, sectionName},zipfilebasename<>"/index.html"]], "width"->"525px","height"->"385px","frameborder"->"0", "scrolling"->"no"},{""}]}]}]
	,
	True,
	videocount=IncrementCounter["videocount"];
	slfexpr=DeleteCases[SelectSelf[],Rule[PageWidth,_],Infinity];
	CreateDirectory[ToFileName[{"assets"<>"."<>ext,sectionName}]];
	Export[ToFileName[{"assets"<>"."<>ext,sectionName},"Video_"<>videocount<>".jpg"],slfexpr,"JPG"];
	{width,height}=Import[ToFileName[{"assets"<>"."<>ext,sectionName},"Video_"<>videocount<>".jpg"],"ImageSize"];
	Global`AntLog["Copy file from: "<>FileNameJoin[Flatten[{FileNameSplit[AbsoluteFileName[GetParameter["ExamplesRawMaterialDir"]]],Cases[{SelectSelf[]},Cell[a___,Rule[CellLabel,c_],___]:>c,Infinity][[1]]}]]];
	videoCopyQ=CopyFile[FileNameJoin[Flatten[{FileNameSplit[AbsoluteFileName[GetParameter["ExamplesRawMaterialDir"]]],Cases[{SelectSelf[]},Cell[a___,Rule[CellLabel,c_],___]:>c,Infinity][[1]]}]],ToFileName[{"assets"<>"."<>ext,sectionName},"Video_"<>videocount<>".mp4"],OverwriteTarget->True];
	If[videoCopyQ=!=$Failed,Global`AntLog["Video file copied"],Global`AntLog["Video file not copied"]];
	videoPath="assets"<>"."<>ext<>"/"<>sectionName<>"/"<>"Video_"<>videocount<>".mp4";
	Global`AntLog["Video file path: "<>videoPath];
	res=XMLElement["section",{},{XMLElement["div",Flatten[{"class"->"clipboard-output","data-out"->"","data-out-num"->"","data-out-src"->StringReplace[ToFileName[{"assets"<>"."<>ext,sectionName},"Video_"<>videocount<>".jpg"],{"\\"->"/"}],"data-out-width"->ToString[width],"data-out-height"->ToString[height],"data-video"->ToString[width]<>" "<>ToString[height],"data-media"->videoPath}],{""}]}];
	res]],
             
{Cell, "Message"} :>
Module[{ext}, 
	ext = Switch[GetParameter["Language"], 
	"ChineseSimplified", "zh", 
	"Japanese", "ja", 
	"Spanish", "es", 
	"Russian", "ru",
	"Korean", "ko",
	"German", "de",
	"French", "fr",
	"Portuguese", "pt-br",
	 _, "en"];
	DIV[{"class" -> "Message"}, {
  Export[ToFileName[{"assets"<>"."<>ext, GetBasenameLa[]},
      "M_" <> IncrementCounter["iMessage"] <> ".gif"], outCell];
  transmogrifyImage[
    ToFileName[{"assets"<>"."<>ext, GetBasenameLa[]}, "M_" <> GetCounter["iMessage"] <> ".gif"],
    {"alt"->""},
    Inline -> True,
    CropImage -> False,
    ImageFormat -> "GIF",
    "RetinaImage" -> True
  ]
}]],


{Cell, "Output" | "Graphics" | "Picture" | "Print" | "Echo" | "OutputSpecial" | "OutputOnly"} :> 
Module[{style, cTags, outCell, ext, manipulatetimingvalue, isManipulate, isPrint, isEcho, isPicture, isSound, 
isSoundNote, soundPath, soundID, soundExpr,isAudio, audioPath, audioID, audioExpr, manipulateExpr, ret, retstring, val, outputVals, outBase, width, height, w1, h1, flashID, swfPath, opts,
ffmpeg = DocumentationBuild`$ffmpeg, optMaxImageSize = 580, exit},

ext = Switch[GetParameter["Language"], 
	"ChineseSimplified", "zh", 
	"Japanese", "ja", 
	"Spanish", "es", 
	"Russian", "ru",
	"Korean", "ko",
	"German", "de",
	"French", "fr",
	"Portuguese", "pt-br",
	 _, "en"]; 
outCell = SelectSelf[];
style = GetStyle[];

cTags = Flatten[{Cases[outCell, Rule[CellTags, s_] :> s, Infinity]}]; 
isManipulate = DocumentationBuild`Utils`IsManipulateQ[ outCell ] && !MemberQ[cTags, "NoFlash"];
(*isManipulate = If[style === "OutputSpecial", isManipulate = False, isManipulate];*)

isPrint = If[Length[Cases[{outCell}, Cell[___, "Print", ___], Infinity]]>0, True, False];
isEcho = If[Length[Cases[{outCell}, Cell[___, "Echo", ___], Infinity]]>0, True, False];
isPicture = If[Length[Cases[{outCell}, Cell[___, "Picture", ___], Infinity]]>0, True, False];

isSound = DocumentationBuild`Utils`IsSoundQ[ outCell];
isSoundNote = !FreeQ[ outCell, _SoundNote];

isAudio = DocumentationBuild`Utils`IsAudioQ[ outCell];
If[isAudio && Length[Cases[First@SelectChildren[], _Audio, Infinity]]>1, isAudio=False];

Which[
isPrint|| isEcho || isPicture,

		  outcount = IncrementCounter["iOutput"];
            
          outputCell = Notebook[
            {outCell},
            WindowSize -> 1300, 
            ShowCellLabel -> False,
            ShowCellBracket -> False,
            ShowStringCharacters -> False,
            Magnification->2,
            CellBracketOptions -> {"Color" -> GrayLevel[1]},
            StyleDefinitions -> Notebook[{ 
              Cell[StyleData[StyleDefinitions -> "Core.nb"]],
              Cell[StyleData[ StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "MarketingPageStyles.nb"]]],
              Cell[StyleData[All, "Working"], ShowCellBracket -> False, CellBracketOptions -> {"Color" -> GrayLevel[1]}],
              Cell[StyleData["InlineFormula"], FontFamily -> "Source Sans Pro", FontSize -> Inherited 1.08],
              Cell[StyleData["MR"], FontFamily -> "Source Sans Pro"],
              Cell[StyleData["StandardForm"], FontFamily -> "Source Sans Pro"],
              Cell[StyleData["Input"], FontFamily -> "Source Sans Pro", FontSize -> 16, LineSpacing -> {1.2, 0}, PageWidth -> 620], 
              Cell[StyleData["Output"], PageWidth -> 620, ShowStringCharacters -> False],
              Cell[StyleData["Print", PageWidth -> 620], CellID -> 1],
              Cell[StyleData["InputHiddenMaterial", StyleDefinitions -> StyleData["Input"]], CellID -> 1], 
              Cell[StyleData["InputSuppressedMaterial", StyleDefinitions -> StyleData["Input"]], CellID -> 1], 
              Cell[StyleData["InputThumbnailMaterial", StyleDefinitions -> StyleData["Input"]], CellID -> 1]
            }]
          ];
          
          {image, {width, height, baseline}} = Rasterize[outputCell, {"Image", "BoundingBox"}, TransparentBackground -> True, Background -> None];
          padwidth = 2 - Mod[width, 2];
     	  padheight = 2 - Mod[height, 2];
     	  image = ImagePad[image, {{0, padwidth}, {padheight, 0}}, Transparent];
     	  {width, height} = ImageDimensions[image];
          
           Quiet[CreateDirectory[ToFileName[{"assets"<>"."<>ext, sectionName}]]];
                
           Export[
                ToFileName[{"assets"<>"."<>ext, sectionName}, "O_" <> outcount <> ".png"],
                image,
                "PNG",
                Background -> None
              ];
          
          {w, h} = Ceiling[{width, height}/2];

          Which[ 
		      Length[Cases[SelectSelf[], TemplateBox[{_,a_},"HyperlinkURL"] :> a, Infinity]] > 0,
              cloudlink = {"data-out-link"->Cases[SelectSelf[], TemplateBox[{_,a_},"HyperlinkURL"] :> a, Infinity][[1]]},
              Length[Cases[SelectSelf[], Rule[ButtonData, {URL[a_String], ___}] :> a, Infinity]] > 0,
              cloudlink = {"data-out-link"->Cases[SelectSelf[], Rule[ButtonData, {URL[a_String], ___}] :> a, Infinity][[1]]},
              True,
              cloudlink = {}
          ];

		   res = XMLElement["section", {"class"-> Which[isEcho, "echo", isPrint, "print", isPicture, "picture", True, ""]}, {XMLElement["div", Flatten[{
	    		"class" -> "clipboard-output",
	    		"data-out"->If[
	            HasOption[CellLabel] && !isPrint && !isPicture && !isEcho,
	            GetOption[CellLabel],
	            If[isEcho, "&raquo;", ""]
	          ],
	              cloudlink,
	              "data-out-num" -> "",
	    		  "data-out-src" -> StringJoin[ToFileName[{"assets."<>ext, sectionName},"O_" <> outcount <> ".png"]],
	    		  "data-out-width" -> ToString[w],
	    		  "data-out-height" -> ToString[h]
	    		  }], {""}]}];

	res
, isSoundNote, 
			Print["<" <> GetBasenameLa[] <> ": SoundNote Output cell>"];
            audioID = "Audio_" <> IncrementCounter["audio"];
            audioPath = StringReplace[ToFileName[{"assets"<>"."<>ext, sectionName}, audioID <> ".mp3"],  {"\\" -> "/"}];
            Print["About to create: "<>audioPath];
            audioExpr = Cases[First@SelectChildren[], _Sound, Infinity][[-1]];
            ret = Export[audioPath, audioExpr];
            retstring = ToString[ret];
            Export[ToFileName[{"assets"<>"."<>ext, sectionName}, audioID <> ".png"], SelectSelf[], "PNG"];
            {w, h} = Import[ToFileName[{"assets"<>"."<>ext, sectionName}, audioID <> ".png"], "ImageSize"];
            res = XMLElement["div", {
            "class" -> "clipboard-output", 
			"data-out" -> If[
							HasOption[CellLabel],
							GetOption[CellLabel],
							""
						  ], 
			  "data-out-num" -> StringReplace[If[HasOption[CellLabel],GetOption[CellLabel], ""], {"Out["->"", "]="->""}], 
			  "data-out-src" -> StringJoin[ToFileName[{"assets."<>ext, sectionName}, audioID <>".png"]], 
			  "data-out-width" -> ToString[w], 
			  "data-out-height" -> ToString[h], 
			  "data-media" -> If[retstring == "$Failed", "", audioPath]}, {""}];
			  res
, isSound,
			Print["<" <> GetBasenameLa[] <> ": Sound Output cell>"];
            audioID = "Audio_" <> IncrementCounter["audio"];
            audioPath = StringReplace[ToFileName[{"assets"<>"."<>ext, sectionName}, audioID <> ".mp3"],  {"\\" -> "/"}];
            Print["About to create: "<>audioPath];
            audioExpr = Sound@@Cases[First@SelectChildren[], _Sound, Infinity];
            ret = Export[audioPath, audioExpr];
            retstring = ToString[ret];
            Export[ToFileName[{"assets"<>"."<>ext, sectionName}, audioID <> ".png"], SelectSelf[], "PNG"];
            {w, h} = Import[ToFileName[{"assets"<>"."<>ext, sectionName}, audioID <> ".png"], "ImageSize"];
            res = XMLElement["div", {
            "class" -> "clipboard-output", 
			"data-out" -> If[
							HasOption[CellLabel],
							GetOption[CellLabel],
							""
						  ], 
			  "data-out-num" -> StringReplace[If[HasOption[CellLabel],GetOption[CellLabel], ""], {"Out["->"", "]="->""}], 
			  "data-out-src" -> StringJoin[ToFileName[{"assets."<>ext, sectionName}, audioID <>".png"]], 
			  "data-out-width" -> ToString[w], 
			  "data-out-height" -> ToString[h], 
			  "data-media" -> If[retstring == "$Failed", "", audioPath]}, {""}];
			  res
, isAudio,

			Print["<" <> GetBasenameLa[] <> ": Audio Output cell>"];
            audioID = "Audio_" <> IncrementCounter["audio"];
            audioPath = StringReplace[ToFileName[{"assets"<>"."<>ext, sectionName}, audioID <> ".mp3"],  {"\\" -> "/"}];
            Print["About to create: "<>audioPath];
            audioExpr = Audio@@Cases[First@SelectChildren[], _Audio, Infinity];
            ret = Export[audioPath, audioExpr];
            retstring = ToString[ret];
            Export[ToFileName[{"assets"<>"."<>ext, sectionName}, audioID <> ".png"], SelectSelf[], "PNG"];
            {w, h} = Import[ToFileName[{"assets"<>"."<>ext, sectionName}, audioID <> ".png"], "ImageSize"];
            res = XMLElement["div", {
            "class" -> "clipboard-output", 
			"data-out" -> If[
							HasOption[CellLabel],
							GetOption[CellLabel],
							""
						  ], 
			  "data-out-num" -> StringReplace[If[HasOption[CellLabel],GetOption[CellLabel], ""], {"Out["->"", "]="->""}], 
			  "data-out-src" -> StringJoin[ToFileName[{"assets."<>ext, sectionName}, audioID <>".png"]], 
			  "data-out-width" -> ToString[w], 
			  "data-out-height" -> ToString[h], 
			  "data-media" -> If[retstring == "$Failed", "", audioPath]}, {""}];
			  res
, isManipulate, 

		dprint["Defaults.m:Output, isManipulate code"];

		  videoID = "Video_" <> IncrementCounter["videocount"]; dprint["videoID = "<>ToString[videoID]];
		  Print[videoID];
		  Global`AntLog[videoID];

		  previewFormat = "jpg";
		  exportFormat  = "mov";
		  videoFormat   = "mp4";

		  exportFile  = StringReplace[FileNameJoin[{"assets."<>ext, sectionName, videoID <> "." <> exportFormat}],  {"\\" -> "/"}];
		  previewFile = StringReplace[FileNameJoin[{"assets."<>ext, sectionName, videoID <> "." <> previewFormat}],  {"\\" -> "/"}];
		  videoFile   = StringReplace[FileNameJoin[{"assets."<>ext, sectionName, videoID <> "." <> videoFormat}],  {"\\" -> "/"}];

		(* **** build the expression to be exported *)

		  opts = {
			"ShowControls" -> True,
			"ControlAppearance" -> Automatic,
			"VideoEncoding" -> "Uncompressed"
		  };

		  manipulateExpr = ReleaseHold@MakeExpression[First@SelectLiteral[SelectChildren[]], StandardForm];

		(* **** export preview image and get its dimensions *)

		  (* on the use of Rasterize here, see https://mail-archive.wolfram.com/archive/l-importexport/2018/Jan00/0037.html *)

		  Export[previewFile, Rasterize @ manipulateExpr, Sequence @@ opts];

		  (* test for existence and complain if False *)
		  If[
			Not[StringQ[previewFile] && Positive@StringLength[previewFile] && FileExistsQ[previewFile]]
		  ,
			dprint["WARNING: preview export failed; Defaults.m:Output transform:isManipulate."];
			Global`AntLog["WARNING: preview export failed; Defaults.m:Output transform:isManipulate."]
		  ];

		  {previewWidth, previewHeight} = Import[previewFile, "ImageSize"];

		(* **** export an intermediate video file to be converted to mp4 *)

		  Export[exportFile, manipulateExpr, Sequence @@ opts];
	
		  (* test for existence and complain if False *)
		  If[Not[StringQ[exportFile] && Positive@StringLength[exportFile] && FileExistsQ[exportFile]], Print["dprint:WARNING: intermediate export failed; Defaults.m:Output transform:isManipulate."]; Global`AntLog["WARNING: intermediate export failed; Defaults.m:Output transform:isManipulate."]];

		(* **** and get its dimensions for use after sue's js jpg -> mp4 swap *)

		  (* this will give us the maximum size attained by the video *)

		  {videoWidth, videoHeight} = ImageDimensions[Import[exportFile, {"QuickTime", "Frames", 1}]];

		(* **** ffmpeg: convert the export file to mp4, our preferred format on the web *)

		  exit = Run[
			StringJoin[
			  ffmpeg,
			  " -v -8 ", (* absolute silence; without this, ffmpeg's copious output appears the build log *)
			  " -i ", exportFile,
			  (* a series of mystical incantations that summon firefox compatability *)
			  " -pix_fmt yuv420p ",
			  " -profile:v baseline ",
			  " -level 3 ",
			  " -vf pad=\"width=ceil(iw/2)*2:height=ceil(ih/2)*2:color=white\" ", 
			  " -y ", videoFile (* overwrite output files without asking. *)
			]
		  ];

		  (* test for existence and complain if False *)
		  If[
			Or[
			  0 =!= exit,
			  Not[StringQ[videoFile] && Positive@StringLength[videoFile] && FileExistsQ[videoFile]]
			]
		  ,
			Print["WARNING: Manipulate video conversion failed; Defaults.m:Output transform:isManipulate."];
			Global`AntLog["WARNING: Manipulate video conversion failed; Defaults.m:Output transform:isManipulate."];
		  ];

		  (* clean up after ourselves *)
		  If[StringQ[exportFile] && FileExistsQ[exportFile] && Positive@FileByteCount[exportFile], DeleteFile[exportFile]];

		(* **** return XML code with video & image information *)
		  
		   res = XMLElement["section", {}, {XMLElement["div", Flatten[{
	    		"class" -> "clipboard-output",
	    		"data-out"->If[
	            HasOption[CellLabel],
	            GetOption[CellLabel],
	            ""
	          ],
	              "data-out-num" -> StringReplace[If[HasOption[CellLabel],GetOption[CellLabel], ""], {"Out["->"", "]="->""}],
	    		  "data-out-src" -> previewFile,
	    		  "data-out-width" -> ToString[previewWidth],
	    		  "data-out-height" -> ToString[previewHeight],
	    		  "data-video" -> ToString[videoWidth]<>" "<>ToString[videoHeight],
	    		  "data-media" -> videoFile
	    		  }], {""}]}];
		  res,
		True, 

		  outcount = IncrementCounter["iOutput"];
            
          outputCell = Notebook[
            {outCell},
            WindowSize -> 1300, 
            ShowCellLabel -> False,
            ShowCellBracket -> False,
            ShowStringCharacters -> False,
            Magnification->2,
            CellBracketOptions -> {"Color" -> GrayLevel[1]},
            StyleDefinitions -> Notebook[{ 
              Cell[StyleData[StyleDefinitions -> "Core.nb"]],
              Cell[StyleData[StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "MarketingPageStyles.nb"]]],
              Cell[StyleData[All, "Working"], ShowCellBracket -> False, CellBracketOptions -> {"Color" -> GrayLevel[1]}],
              Cell[StyleData["InlineFormula"], FontFamily -> "Source Sans Pro", FontSize -> Inherited 1.08],
              Cell[StyleData["MR"], FontFamily -> "Source Sans Pro"],
              Cell[StyleData["StandardForm"], FontFamily -> "Source Sans Pro"],
              Cell[StyleData["Input"], FontFamily -> "Source Sans Pro", FontSize -> 16, LineSpacing -> {1.2, 0}, PageWidth -> 620], 
              Cell[StyleData["Output"], PageWidth -> 620, ShowStringCharacters -> False],
              Cell[StyleData["Print", PageWidth -> 620], CellID -> 1],
              Cell[StyleData["InputHiddenMaterial", StyleDefinitions -> StyleData["Input"]], CellID -> 1], 
              Cell[StyleData["InputSuppressedMaterial", StyleDefinitions -> StyleData["Input"]], CellID -> 1], 
              Cell[StyleData["InputThumbnailMaterial", StyleDefinitions -> StyleData["Input"]], CellID -> 1]
            }]
          ];
          
          {image, {width, height, baseline}} = Rasterize[outputCell, {"Image", "BoundingBox"}, TransparentBackground -> True, Background -> None];
          padwidth = 2 - Mod[width, 2];
     	  padheight = 2 - Mod[height, 2];
     	  image = ImagePad[image, {{0, padwidth}, {padheight, 0}}, Transparent];
     	  {width, height} = ImageDimensions[image];
          
           Quiet[CreateDirectory[ToFileName[{"assets"<>"."<>ext, sectionName}]]];
                
           Export[
                ToFileName[{"assets"<>"."<>ext, sectionName}, "O_" <> outcount <> ".png"],
                image,
                "PNG",
                Background -> None
              ];
          
          {w, h} = Ceiling[{width, height}/2];
          
          Which[ 
		      Length[Cases[SelectSelf[], TemplateBox[{_,a_},"HyperlinkURL"] :> a, Infinity]] > 0,
              cloudlink = {"data-out-link"->Cases[SelectSelf[], TemplateBox[{_,a_},"HyperlinkURL"] :> a, Infinity][[1]]},
              Length[Cases[SelectSelf[], Rule[ButtonData, {URL[a_String], ___}] :> a, Infinity]] > 0,
              cloudlink = {"data-out-link"->Cases[SelectSelf[], Rule[ButtonData, {URL[a_String], ___}] :> a, Infinity][[1]]},
              True,
              cloudlink = {}
          ];
          
          XMLElement["div", Flatten[{
	    		"class" -> "clipboard-output",
	    		"data-out"->If[
	            HasOption[CellLabel],
	            GetOption[CellLabel],
	            ""
	          ],
	          cloudlink,
	          "data-out-num" -> StringReplace[If[HasOption[CellLabel],GetOption[CellLabel], ""], {"Out["->"", "]="->""}],
	    		  "data-out-src" -> StringJoin[ToFileName[{"assets."<>ext, sectionName},"O_" <> outcount <> ".png"]],
	    		  "data-out-width" -> ToString[w],
	    		  "data-out-height" -> ToString[h],
	    		  "data-out-separate"->If[StringMatchQ[If[HasOption[CellLabel],GetOption[CellLabel], ""], "*Form*"], "true", "false"]
	    		  }], {""}]

		]
], (* end "Output" rule *)



{StyleBox, "InlineSeparator"} :> Sequence[],


(* Bulleted lists *)
{CellGroupData, "HighlightSection"} :> Module[{hghlghttm}, 
	hghlghttm=Cases[SelectSelf[], Cell[___, "HighlightItem", ___], Infinity];
	If[Length[hghlghttm]>0,
	 {XMLElement["section", {"class"->"list row"}, Map[XMLElement["div", {"class"->"column width-50per"}, {XMLElement["ul", {"class"->"styled-list"}, {Transmogrify[#]}]}]&, Partition[hghlghttm, Ceiling[Length[hghlghttm]/2], Ceiling[Length[hghlghttm]/2], 1, {}]]], XML`RawXML["<!--#include virtual=\"assets"<>"."<>ext<>"/inc/thumbnails.html."<>ext<>"\"-->"]}
	 ]],
{Cell, "FutureFlag"} :> Sequence[],
{Cell, "PrintTemporary"} :> Sequence[],
{Cell, "HighlightSection"} :> Sequence[],
{Cell, "HighlightItem"} :> 
Module[{u, cell, res},
  res = Recurse[];
  cell = SelectSelf[];
  u = Cases[cell, ButtonBox[">>" | "\[RightGuillemet]", ___, 
                    Rule[ButtonData, d_String | {URL[d_String, ___], ___}], ___] :> d, Infinity];
  XMLElement["li", {}, {
    If[Length@u > 0, 
      If[Length[u]===1, res,
      res],
      res
    ]
  }] 
],


{StyleBox, "InlineSeparator"} :>
Span[{"class" -> "separator"}, {XML`RawXML["&nbsp;"], XML`RawXML["&#9642;"], " "}],

{Cell, "Delimiter" | "ExampleDelimiter"} :> Sequence[],


{Cell, "InputSuppressedMaterial"} :> 
  DIV[{"class" -> "InputSuppressedMaterial"}, {" "}],

{Cell, "ProgramHiddenMaterial" | "Program" | "Code"} :>
If[htmlBoxesQ[SelectLiteral[SelectSelf[]]],
XMLElement["section", {}, {
	XMLElement["code", {"class"->"code-plain"}, Flatten@{
  If[HasOption[CellTags],
  Flatten[ {
    Map[
    (XMLElement["a", {"name" -> #}, {}])&, Flatten[{GetOption[CellTags]}] ],
    Recurse[]  }],
  {Recurse[]  }  ]    }]} ],

XMLElement["section", {}, {XMLElement["code", {"class"->"code-plain"}, {
  (
    XMLElement["img", { 
      "class" -> "FlushInput", "src" -> ("URL" /. #), 
      "width" -> ("Width" /. #), "height" -> ("Height" /. #),  
      "alt" -> " "  }, {}]&[
      BoxToImage[ToFileName[{"assets"<>"."<>ext, sectionName},
    "Box_" <> IncrementCounter["Inline"] <> ".gif"],
    Inline -> False, CropImage -> False]
    ]
  )}]}]
],

(* Callouts *)
{Cell, "calloutbox" } :>
Module[{calloutID},
  exampleholder++;
  (*  Define callout counter *)
  calloutID  = "cbox"<>ToString@exampleholder<>"-"<>IncrementCounter["callout"];
  DIV[{"class" -> "callout", "id" -> calloutID, "name" -> calloutID }, {
    DIV[{"class" -> "calloutbox"}, {
      Recurse[]
    }],
  DIV[{"class"->"calloutline", "id"->calloutID, "name"->calloutID}, {
    Img[{"src"->"/common/images2003/spacer.gif", "height"->"1", "width"->"1", "alt"->""}]
    }]
  }]
],

{Cell, "MarketingExampleDetail"} :>
Module[{myexpr},
	myexpr = DeleteCases[SelectSelf[],"MarketingExampleDetail", Infinity];
  XMLElement["p", {}, {Transmogrify[myexpr]}]
],

{GraphicsBox | PanelBox | PaneBox} :> 
transmogrifyImage[
  ToFileName[{"assets"<>"."<>ext, GetBasenameLa[]}, "GB_"<>IncrementCounter["GBImage"] <> ".png"],
  {"alt"->""},
  Inline -> True,
  CropImage -> False,
  ImageFormat -> "PNG",
  TransparentBackground->True,
  ConversionOptions->{Background->None},
  "RetinaImage" -> True
], 
      
 {Cell,None} :> Module[{
  tmp1=SelectChildren[],
  tmp2=Recurse[]
  (*tmp3=SelectAncestors[Cell] *)
},
  If[
    Head[tmp1[[1]]]===CellGroupData, 
    If[Cases[tmp1, __]==={CellGroupData[{}, Open]}, "", tmp2 ],
    If[tmp2===Null, "",
    Which[htmlBoxesQ[SelectLiteral[SelectSelf[]]], tmp2, 
		Cases[tmp1, _CheckboxBox, Infinity]=!={}, tmp2,
		Cases[tmp1, _RadioButtonBox, Infinity]=!={}, tmp2,
		True,
        transmogrifyImage[
          ToFileName[{"assets"<>"."<>ext, GetBasenameLa[]}, "CellNone_" <> IncrementCounter["Image"] <> ".gif"],
          {"style"->"vertical-align:middle;", "alt"->"", "title"->""},
          Inline -> True,
          CropImage -> False,
          ImageFormat -> "PNG",
          TransparentBackground->True,
          ConversionOptions->{Background->None},
          "RetinaImage" -> True
        ]
      ]
  ]
  ]
],

{Cell,  "MarketingExampleCaption"} :>
H3[{"class" -> "MarketingSubsubsection"}, {
      Recurse[]
    }],

{File} :> 
  DIV[{"class" -> "file"}, { "content"}],

{Cell, "PopupCaptionText"} :> 
  XMLElement["p", {}, {Recurse[]}],
 


{Cell, "MarketingThumbnailSection" | "MarketingThumnailSection" | "MarketingContentsSection"} :> Sequence[],
(*
{Cell, "ThumbOutput" | "ThumbnailOutput" | "ThumnailOutput"} :> 
  transmogrifyImage[
    ToFileName[{"assets", mainFileNameBase}, "smallthumb_" <> IncrementCounter["Images"] <> ".gif"],
    Inline -> True,
    CropImage -> False,
    "RetinaImage" -> True
  ],
*)
{Cell, "ContentOutput"} :> 
Module[{ext},  
	ext = Switch[GetParameter["Language"], 
	"ChineseSimplified", "zh", 
	"Japanese", "ja", 
	"Spanish", "es", 
	"Russian", "ru",
	"Korean", "ko",
	"German", "de",
	"French", "fr",
	"Portuguese", "pt-br",
	 _, "en"];
	transmogrifyImage[
    ToFileName[{"assets"<>"."<>ext, mainFileNameBase}, "content_" <> IncrementCounter["Images"] <> ".gif"],
    Inline -> True,
    CropImage -> False,
    ImageFormat -> "GIF",
    "RetinaImage" -> True
  ]],

{Cell, "NotebookImage"} :> 
Module[{ext},
  ext = Switch[GetParameter["Language"], 
	"ChineseSimplified", "zh", 
	"Japanese", "ja", 
	"Spanish", "es", 
	"Russian", "ru",
	"Korean", "ko",
	"German", "de",
	"French", "fr",
	"Portuguese", "pt-br",
	 _, "en"];
  transmogrifyImage[
    ToFileName[{"assets"<>"."<>ext, mainFileNameBase}, "G_" <> IncrementCounter["Graphics"] <> ".gif"],
    Inline -> True,
    CropImage -> False,
    ImageFormat -> "GIF",
    "RetinaImage" -> True
  ]],

{Cell, "AltThumbnail"} :> {},
{Cell, "AlternateThumbnail"} :> {},
{Cell, "MarketingThumbnailOutput"} :> {},

{CellGroupData, "MarketingRelatedLinksSection"} :> 
XMLElement["section", {"id"->"relatedlinks"}, {
      Recurse[First[SelectChildren[]]],
      XMLElement["ul",{"class"->"square-between-each-child"}, {Recurse[Rest[SelectChildren[]]]}]
  }],

{CellGroupData, "MarketingMoreAboutSection"} :> 
XMLElement["section", {"id"->"guides"}, {
      Recurse[First[SelectChildren[]]],
      XMLElement["ul",{"class"->"square-between-each-child"}, {Recurse[Rest[SelectChildren[]]]}]
  }],

{CellGroupData, "MarketingSeeAlsoSection"} :> 
XMLElement["section", {"id"->"also"}, {
      Recurse[First[SelectChildren[]]],
      XMLElement["ul",{"class"->"square-between-each-child"}, {Recurse[Rest[SelectChildren[]]]}]
  }],

{CellGroupData, "MarketingRelatedFunctionsSection"} :> 
XMLElement["section", {"id"->"functions"}, {
      Recurse[First[SelectChildren[]]],
      XMLElement["ul",{"class"->"square-between-each-child"}, DeleteCases[Map[XMLElement["li",{}, {#}]&, {Recurse[Rest[SelectChildren[]]]}], XMLElement["li",{}, {" "}]]]
  }],
 
 {CellGroupData, "MarketingRelatedDemonstrationsSection"} :> 
XMLElement["section", {"id"->"demonstrations"}, {
      Recurse[First[SelectChildren[]]],
      XMLElement["ul",{"class"->"square-between-each-child"}, {Recurse[Rest[SelectChildren[]]]}]
  }], 

{Cell, "MarketingRelatedLinksSection"|"MarketingMoreAboutSection"|"MarketingSeeAlsoSection"|"MarketingRelatedFunctionsSection"|"MarketingRelatedDemonstrationsSection"} :> 
  XMLElement["h3",{}, {Recurse[]}],

{Cell, "MarketingUnorderedList"} :> Recurse[],

{Cell, "MarketingRelatedLinks"|"MarketingMoreAbout"|"MarketingSeeAlso"|"MarketingRelatedDemonstrations"} :> XMLElement["li", {}, {Recurse[]}],

{Cell, "MarketingRelatedFunctions"} :>   Module[{}, Flatten[{Recurse[]}]/.XMLElement["a", {a___},{b___}]:>XMLElement["a", {a}, {b}]],

(* Some basic styles*)
{Cell, "InlineFormula" | "InlineCode" | "FunctionTemplate"} :> 
Module[{thestyle, ext},
ext = Switch[GetParameter["Language"], 
	"ChineseSimplified", "zh", 
	"Japanese", "ja", 
	"Spanish", "es", 
	"Russian", "ru",
	"Korean", "ko",
    "German", "de",
	"French", "fr",
	"Portuguese", "pt-br",
	 _, "en"];
thestyle = GetStyle[];
If[htmlBoxesQ[SelectSelf[]],
	Span[{"class" -> GetStyle[]},{DocumentationBuild`Utils`AddSpaces[ Recurse[] ]}],
    ( CreateDirectory[ToFileName[{"assets"<>"."<>ext, mainFileNameBase}]];
    	Export[ToFileName[{"assets"<>"."<>ext, mainFileNameBase},
         "IM_"<>IncrementCounter["IMimage"] <> ".gif"], SelectSelf[]];
        Img[{"src" -> StringJoin["assets"<>"."<>ext<>"/", mainFileNameBase, "/IM_" <> GetCounter["IMimage"] <> ".gif"], "style"->"vertical-align:middle;", "alt"->""
            } ]
      )
    ]
],

{Cell, "InlineOutput" | "InlineMath" } :> 
Module[{thestyle, ext},
ext = Switch[GetParameter["Language"], 
	"ChineseSimplified", "zh", 
	"Japanese", "ja", 
	"Spanish", "es", 
	"Russian", "ru",
	"Korean", "ko",
	"German", "de",
	"French", "fr",
	"Portuguese", "pt-br",
	 _, "en"];
thestyle = GetStyle[];
Span[{}, 
  If[htmlBoxesQ[SelectSelf[]],
    { CreateDirectory[ToFileName[{"assets"<>"."<>ext, mainFileNameBase}]];
    	Export[ToFileName[{"assets"<>"."<>ext, mainFileNameBase},
         "IM_"<>IncrementCounter["IMimage"] <> ".gif"], SelectSelf[]];
        Img[{"src" -> StringJoin["assets"<>"."<>ext<>"/", mainFileNameBase, "/IM_" <> GetCounter["IMimage"] <> ".gif"], "style"->"vertical-align:middle;", "alt"->""
            } ]
      },
    { CreateDirectory[ToFileName[{"assets"<>"."<>ext, mainFileNameBase}]];
    	Export[ToFileName[{"assets"<>"."<>ext, mainFileNameBase},
         "IM_"<>IncrementCounter["IMimage"] <> ".gif"], SelectSelf[]];
        Img[{"src" -> StringJoin["assets"<>"."<>ext<>"/", mainFileNameBase, "/IM_" <> GetCounter["IMimage"] <> ".gif"], "alt"->"","style"->"vertical-align:middle;"
            } ]
      }
    ]
  ]
],

(* {Cell, "WolframAlphaShortInput" | "WolframAlphaLong"} :> 
Module[{thestyle, ext},
ext = Switch[GetParameter["Language"], "ChineseSimplified", "zh", "Japanese", "ja", _, "en"];
thestyle = GetStyle[];
Span[{}, 
  If[htmlBoxesQ[SelectSelf[]],
    {  CreateDirectory[ToFileName[{"assets"<>"."<>ext, mainFileNameBase}]];
    	Export[ToFileName[{"assets"<>"."<>ext, mainFileNameBase},
         "WA_"<>IncrementCounter["WAimage"] <> ".gif"], SelectSelf[]];
        Img[{"src" -> StringJoin["assets"<>"."<>ext<>"/", mainFileNameBase, "/WA_" <> GetCounter["WAimage"] <> ".gif"]
            } ]
      },
    { CreateDirectory[ToFileName[{"assets"<>"."<>ext, mainFileNameBase}]];
    	Export[ToFileName[{"assets"<>"."<>ext, mainFileNameBase},
         "WA_"<>IncrementCounter["WAimage"] <> ".gif"], SelectSelf[]];
        Img[{"src" -> StringJoin["assets"<>"."<>ext<>"/", mainFileNameBase, "/WA_" <> GetCounter["WAimage"] <> ".gif"]
            } ]
      }
    ]
  ]
],
*)
{FormBox} :> 
Module[{boxChildren,traditionalFormQ,altTag,image,height,fudge,width, url},
	fudgeValignProp[x_]:= Module[{var,newvar},
    	var=ToExpression[x];
    	newvar=Which[
    	var<14, "-10",
    	13<var<18, "-12",
    	17<var<20, "-16",
    	19<var<23, "-22",
    	22<var<27, "-35",
    	26<var<30, "-45",
    	29<var<32, "-55",
        31<var<35, "-65",
    	True, "-75"
    ];
    newvar
    ];
	
boxChildren=SelectChildren[];
traditionalFormQ = If[GetStyle[] === TraditionalForm, True, False];
altTag = DocumentationBuild`Utils`CellToString@SelectSelf[];
(* image = BoxToImage[ToFileName[{"assets"<>"."<>ext, mainFileNameBase},
         "FB_"<>IncrementCounter["FBimage"] <> ".gif"],  "TransparentColor" -> White,
            Inline -> If[GetStyle[]===TraditionalForm, True, False], CropImage -> False];  
height = ("Height" /. #)&[image];
width = ("Width" /. #)&[image];
url = ("URL" /. #)&[image];
fudge = fudgeValignProp[height];*)
If[MatchQ[boxChildren, {"Mathematica"}],Transmogrify[StyleBox["Mathematica", FontSlant->"Italic"]],
If[MatchQ[boxChildren, {StyleBox[_, "KeyEvent", ___]}], 
        Transmogrify[boxChildren],
If[MatchQ[boxChildren, {SuperscriptBox["\[Null]", "th"]}],
        XMLElement["sup",{}, {"th"}],
If[MatchQ[boxChildren, {SuperscriptBox["\[Null]", "\[EGrave]me"]}],
        XMLElement["sup",{}, {XML`RawXML["&#232;me"]}], 
          CreateDirectory[ToFileName[{"assets"<>"."<>ext, mainFileNameBase}]];
    	Export[ToFileName[{"assets"<>"."<>ext, mainFileNameBase},
         "FB_"<>IncrementCounter["FBimage"] <> ".png"], Cell[BoxData[SelectSelf[]]], Background->None];
        Img[{"src" -> StringJoin["assets"<>"."<>ext<>"/", mainFileNameBase, "/FB_" <> GetCounter["FBimage"] <> ".png"], "alt"->"","style"->"vertical-align:middle;"
            } ]]]
]
]
]

}, 

IncludeXMLTransforms -> {{"HTML","Components","Includes.m"}},
DefaultParameters -> { "newline2br" -> False, "newline2div" -> False}

]