(* :Title: Functions.m *)

(* :Author:
  Andrew Hunt, andy@wolfram.com
  Greg Martel, gregm@wolfram.com
*)

(* :Mathematica Version: 5 *)
(* :Package Version: $Revision: 1.16 $ $Date: 2018/03/05 04:45:32 $ *)                  
(* :Copyright: (c) 2005, Wolfram Research, Inc. All rights reserved. *)
(* :Summary: Add-ons specific rules and general transform for Mathematica Documentation *)

(* :Title: Functions.m *)

XMLTransform[{

(* Remove in-product decorations *)

{Cell, "NotebookImage"} :> 
DIV[{"class"->"NotebookImage"}, {
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
    {"alt"->"", "title"->""},
    Inline -> True,
    "RetinaImage" -> True
  ]
}],
(*{Cell, "NotebookImage"} :> 
Span[{"class"->"NotebookImage"}, {
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
    Inline -> True,
    "RetinaImage" -> True
  ]
}],*)

{Cell, "PackageHeader"} :> Unevaluated[Sequence[]],

{Cell, "ObjectNameGrid"} :>  Block[{objgrid = True}, Recurse[] ],
  
{Cell, "ObjectName" | "ObjectNameSmall" | "ObjectNameAlt" | "CFunctionName"} :> Module[
  {
    uri, context, cell, result, relatedInterpreterCell
  },
  dprint["Services.m: entering ObjectName and friends"];
  dprint["  where SelectSelf[] is"]; dfile[SelectSelf[]];

  cell = SelectSelf[];
  {uri, context} = {GetParameter["taggingRules"]["Metadata"]["uri"], GetParameter["taggingRules"]["Metadata"]["context"]};
  dprint["Services.m ObjectNameAlt: {uri, context} = "<>ToString[{uri, context}, InputForm]];

  relatedInterpreterCell = Transmogrify[Quiet @ Check[First[Cases[cell, Rule["RelatedInterpreterCell", c:_] -> c, Infinity]], {}]];
  cell = DeleteCases[cell, Rule["RelatedInterpreterCell", _]];

  dprint["relatedInterpreterCell = "<>ToString[relatedInterpreterCell, InputForm]];
  dprint["cell = "<>ToString[cell, InputForm]];

  dprint["Services.m: ObjectName* context = "<>ToString[context, InputForm]];
  dprint["Services.m: ObjectName* uri = "<>ToString[uri, InputForm]];
(* debug[2020-02-14T16:46:56]: Services.m: ObjectName* context = "System`" *)
(* debug[2020-02-14T16:46:56]: Services.m: ObjectName* uri = "ref/indicator/RangeIndicator" *)

  result = If[
    StringContainsQ[uri, StartOfString ~~ CharacterRange["A", "Z"]] (* && context =!= "CloudConnectorForExcel`" *)
  ,
    (* for packages, provide a place for the ContextName` above the main title *)
    Unevaluated[Sequence[
      XMLElement["span", {"class" -> "main-title-top"}, {context}]
    , XMLElement["h1", {"class" -> "main-title"}, {Recurse[]}]
    , XML`RawXML["<div class=\"iconography\" data-src=\"s=SICONSTATUS d=DICONSTATUS m=MICONSTATUS c=CICONSTATUS u=UICONSTATUS l= n=NICONSTATUS e=EICONSTATUS\"></div>"]
    , relatedInterpreterCell
    ]]
  ,
    If[
      MemberQ[{"Interpreter", "Indicator"}, GetParameter["entityType"]]
    , XMLElement["div", {"class" -> "main-heading"}, {
        XMLElement["h1", {"class" -> "main-title"}, {Recurse[]}]
      , XML`RawXML["<div class=\"iconography\" data-src=\"s=SICONSTATUS d=DICONSTATUS m=MICONSTATUS c=CICONSTATUS u=UICONSTATUS l= n=NICONSTATUS e=EICONSTATUS\"></div>"]
      , relatedInterpreterCell
      }]
    , Unevaluated[Sequence[
        XMLElement["h1", {"class" -> "main-title"}, {Recurse[]}]
      , XML`RawXML["<div class=\"iconography\" data-src=\"s=SICONSTATUS d=DICONSTATUS m=MICONSTATUS c=CICONSTATUS u=UICONSTATUS l= n=NICONSTATUS e=EICONSTATUS\"></div>"]
      , relatedInterpreterCell
      ]]
    ]
  ];

(* I may need to munge res here to wrap Title quotes in their own spans *)
  dprint["Services.m: exiting ObjectName transform, result = "<>ToString[result, InputForm]];
  result
],
 
{Cell, "IndicatorUsage"} :>
  XMLElement[
    "div",
    {"class"->"indicatorIntroWrap"},
    {XMLElement["div", {"class"->"indicatorIntro"}, {Recurse[]}]}
  ],
 
{CellGroupData, "IndicatorUsage"} :> 
  XMLElement[
    "ul",
    {"class"->"functionList afterIntro"},
    {Recurse[]}
  ],
	
{Cell, "IndicatorUsageItem"} :> XMLElement["p", {}, {Recurse[]}],

(* HowTo styles *)
{Cell, "HowToTitle"} :> H1[{"class" -> "main-title"}, {Recurse[]}],
{Cell, "HowToTitlePrefix"} :> {XMLElement["span", {}, {Recurse[]}], XMLElement["span", {"class"->"titleSep"}, {XML`RawXML["|"]}]},
{Cell, "HowToAbstract"} :> P[{"class" -> "intro"}, {Recurse[]}],
{Cell, "HowToScreencastLink"} :> Module[{iconText,iconSource,bdata,h="100px",w="100px"},
  bdata = First[Cases[SelectSelf[], (ButtonData->{URL[bbdd_], _}) :> bbdd, Infinity, 1]];
  XMLElement["div", {"class"->"embedWrap"}, {
      	XML`RawXML["<script language=\"JavaScript\" type=\"text/javascript\" src=\"/common/javascript/swfobject.js\" ></script>"],
      	XML`RawXML["<!--#include virtual=\"/common/includes/flash/vdbembed.cgi?v="<>StringReplace[StringCases[bdata,"video=" ~~ DigitCharacter ..], "video="->""]<>"&o=74&db=prd\"-->"],
      	 XMLElement["div", {"id"->"videoplayer"}, {XML`RawXML[""]}]
      	}]
],

{Cell, "DeviceSubtitle"} :> XMLElement["span", {"class"->"main-heading-type"}, {Recurse[]}],
{Cell, "InterpreterSubtitle"} :> XMLElement["span", {"class"->"main-heading-type"}, {Recurse[]}],
{Cell, "InterpreterTitleQuotes"} :> XMLElement["span", {"class" -> "quote"}, {Recurse[]}],

{Cell, "ServiceSubtitle"} :> XMLElement["span", {"class"->"main-heading-type"}, {Recurse[]}],

{Cell, "ScreencastPointerText"} :> P[{"class" -> GetStyle[]}, {Recurse[]}],
(* end howto styles *)
{Cell, "HowToText"} :> P[{"class" -> GetStyle[]}, {Recurse[]}],

{Cell, "HowToDelimiter"} :> XMLElement["div", {"class"->"HowToDelimiter"}, {Recurse[]}],

 {Cell, "AwaitingReviewNote"} :>  XMLElement["div", {"class"->"AwaitingReviewNote"}, {Recurse[]}],


{Cell, "CharacterImage"}:> 
Block[{CharacterImageData, CharacterImageDataLeft, CharacterImageDataRight},
CharacterImageData = SelectSelf[];
CharacterImageDataLeft={Transmogrify[Cell[BoxData[CharacterImageData[[1, 1, 1, 1, 1]]], "CharacterImageLeftSide"]]};
CharacterImageDataRight={Transmogrify[Cases[CharacterImageData[[1, 1, 1, 1, 2]], Cell[___, "CharacterNotes", ___], Infinity]]};
XMLElement["div", {"class"->"functionIntroWrap"}, {
	XMLElement["div", {"class"->"functionIntro"}, 
	{XMLElement["table", {}, {XMLElement["tr", {}, Flatten[{XMLElement["td", {"class"->"character-img"}, Flatten[{CharacterImageDataLeft}]], 
	XMLElement["td", {}, {XMLElement["ul", {"class"->"functionList afterIntro"}, Flatten[{CharacterImageDataRight}]]}]}]]}]}]}]
 ],   


{Cell, "CharacterImageLeftSide"}:>
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, "characterimage_"<>IncrementCounter["CharacterImage"] <> ".gif"],
    {"alt"->"", "title"->""},
    Inline -> True,
    "RetinaImage" -> True
  ],
  
{Cell, "CharacterNotes"}:> XMLElement["li", {}, {Recurse[]}],

{Cell, "CharacterName"}:> H1[{"class" -> "main-title"}, {Recurse[]} ],
 



{Cell, "1ColumnTableMod" | "2ColumnTable" | "3ColumnTable" | "2ColumnTableMod" | "3ColumnTableMod" | "3ColumnIllustratedTableMod"} :> Li[{"class"->"NotesTable"}, {Recurse[]}], 


{Cell,  "AlphabeticalListingTable"} :>
P[ {"class" -> "MathTable", "name" -> (GetStyle[]/.None->"FIXME")}, 
  {Recurse[]}
], 


(***** Cell Definitions *****)

{Cell, "Usage" | "DeviceDoubleUsage"} :> (
usageContent := XMLElement["div", {"class" -> "functionIntroWrap"},
Block[{usagegrid = True},
    Module[{xml = {Recurse[]}, pos, repl, first = True, versionNumber, modinfobackground,modinfoposition},
    modinfobackground=Cases[GetOption[GridBoxOptions], Rule["Rows", a__]:>a, Infinity];
    modinfobackground=Map[MemberQ[{#}, Dynamic[__]] &, Flatten[modinfobackground]];
    versionNumber = StringSplit[$Version, "."][[1]];
    pos = Position[xml, XMLElement["tr", _, _], Infinity];
    modinfoposition=Pick[pos, modinfobackground];
    repl = Map[
        Module[{tr, attr, children, modinfo},
            tr = Extract[xml, #];
            modinfo = False;
            modinfo = If[MemberQ[modinfoposition, #], True, False];
            attr = Join[
                DeleteCases[tr[[2]], _["name"|"id", _]],
                If[modinfo, {}(*{"name"->"ModInfo_"<>versionNumber, "id"->"ModInfo_"<>versionNumber}*), {}] ];
            If[first,
                first = False,
                attr = Join[
                    DeleteCases[attr, _["class", _]],
                    {}]];
            children = Map[
                Function[{td}, Module[{attr},
                    attr = td[[2]];
                    If[MatchQ[attr, {___, "class" -> _, ___}],
                        attr = ReplaceAll[attr, _["class", _] -> ("class" -> "UM")],
                        attr = Join[attr, {}]];
                    attr = Join[
                        DeleteCases[attr, _["name"|"id", _]],
                        If[modinfo, {"name"->"ModInfo_"<>versionNumber, "id"->"ModInfo_"<>versionNumber}, {}] ];
                    XMLElement["p", {}, td[[3]]]
                ]],
                Rest[tr[[3]]] ];
            XMLElement["div", {"class"->"functionIntro"<>If[modinfo, " modified-text", ""]}, children]
        ]&,
        pos];
        repl]]
];

usageContent
), 


{StyleBox, "GuideFunctionsSubsection"} :> Recurse[],

{Cell, "GuideAbstract"} :> 
Module[{flag = GetParameter["flag"]}, 
  If[flag =!= "RootGuidePage",
    WithParameters[{"newline2br" -> False, "newline2div" -> True, "styleUsageMessageBreak" -> "<span class='br'>&nbsp;</span>"},P[{"class"->"intro"}, {Recurse[]} ]],
    DIV[{},{}]
  ]
],

{Cell, "GuideFunctionsSubsection"} :> 
Module[{flag = GetParameter["flag"]},
  If[flag =!= "RootGuidePage",
    H3[If[HasOption[CellTags], 
		{"id"->StringReplace[ToString[Flatten[{GetOption[CellTags]}]], {"{"->"", "}"->""}]}, {}], {Recurse[]} ],
    DIV[{},{}]
  ]
],

{Cell, "GuideFunctionsSubsubsection"} :> 
Module[{flag = GetParameter["flag"]}, 
  If[flag =!= "RootGuidePage",
    H4[{}, {Recurse[]} ],
    DIV[{},{}]
  ]
],


{Cell, "GuideDelimiterSubsection"} :> 
Module[{flag = GetParameter["flag"]}, 
  If[flag =!= "RootGuidePage",
    XMLElement["div", {"class"->"GuideDelimiterSubsection"}, {XML`RawXML["<!-- -->"]} ],
    DIV[{},{}]
  ]
],

{Cell, "InlineGuideFunction"} :> 
Span[{"class"->"InlineGuideFunction"}, {Recurse[]} ],


{Cell, "InlineGuideFunctionListing"} :> 
Module[{ml},
  ml = Recurse[];
  If[(Length@{ml} === 1) && (Head@ml === String),
   ml = StringSplit[ml, RegularExpression["\\s+\\.\\s+"] ];
   ml = Span[{"class"->"InlineGuideFunction"}, 
          {XMLElement["span", {}, {#}] }]&/@ ml;
   ml = Riffle[ml,  
     Span[{"class" -> "inlineBullet"}, {XML`RawXML["&nbsp;"], XML`RawXML["&#9642;"], XML`RawXML["&nbsp;"]}]
     ] ];
  P[{"class"->"inlineFunctionList"}, Flatten@{ml}]
],

{Cell, "GuideText"} :> 
Module[{flag = GetParameter["flag"]},
  If[flag =!= "RootGuidePage",
    P[{"class"->"singleFunction"},  { Recurse[] }],
    Module[{guidelinks,guidelinksNew678,class="left"},
      guidelinks = Cases[SelectSelf[], _ButtonBox, Infinity];
      (*guidelinksNew678 = Insert[Take[guidelinks, {-3, -2, -1}], " | ", 2];*)
      guidelinksNew678 = Riffle[Take[guidelinks, {-3, -1}], " | "];
      guidelinks=Join[Drop[guidelinks, -3], {guidelinksNew678}];
	  guidelinks=If[EvenQ@Length@guidelinks === True, guidelinks, Append[guidelinks, {}]];
	  guidelinks = Partition[guidelinks, 2];
	  guidelinks = Transpose[guidelinks];
      guidelinks = Map[
        XMLElement["ul", {"class" -> (class="right";#)&[class]}, {Map[Li[{}, {Transmogrify[#]}]&, #]}]&,
        guidelinks];
      XMLElement["div",{
          "class" -> "accordion-content "<> GetCounter["RootSection"]
        }, {
        XMLElement["tr",{}, {
        TD[{"class"->"rootLinkCol"}, {guidelinks[[1]]}],
        TD[{"class"->"rootLinkCol"}, {DeleteCases[guidelinks[[2]], XMLElement["li",{__},{}], Infinity]}]
        }]
      }]
    ]
]],

{StyleBox, "InlineSeparator"} :>If[StringReplace[GetBasenameLa[], {".en"->"", ".ja"->""}]==="Mathematica",
Span[{"class" -> "inlineBullet"}, {XML`RawXML["&nbsp;"], XML`RawXML["&#9642;"], XML`RawXML["&nbsp;"]}],
Span[{"class" -> "inlineBullet"}, {XML`RawXML["&nbsp;"], XML`RawXML["&#9642;"], XML`RawXML["&nbsp;"]}]
],


{StyleBox, "LeadingBullet"} :>
Span[{"class" -> "LeadingBullet"}, {
    Img[{"src" -> htmlBase<>"/mathematicaImages/bullet.gif", "width"->"4", "height"->"4", "style"->"vertical-align:middle;", "alt"->"bullet", "title"->"bullet"}] }],

{Cell, "InlineListing"} :>
DIV[{"class" -> "InlineListing"}, {Recurse[] }],
{Cell, "InlineListing3Col"} :>
DIV[{"class" -> "InlineListing3Col"}, {Recurse[] }],
{Cell, "InlineListing4Col"} :>
DIV[{"class" -> "InlineListing4Col"}, {Recurse[] }],


{CellGroupData, "UsageSection" | 
	"LegacyMaterialSection" | 
	"ApplicationNotesSection" | 
	"ExtensionsSection" | 
	"DesignDiscussionSection"  | 
	"TutorialMoreAboutSection" | 
	"TutorialCollectionSection"} :> 
Recurse[],  

{Cell, "ServiceAbstract"} :> 
Module[{flag = GetParameter["flag"]}, 
  If[flag =!= "RootGuidePage",
    WithParameters[{"newline2br" -> False, "newline2div" -> True, "styleUsageMessageBreak" -> "<span class='br'>&nbsp;</span>"},P[{"class"->"intro"}, {Recurse[]} ]],
    DIV[{},{}]
  ]
],

{CellGroupData, "SeeAlsoSection"} :>
  XMLElement["section", {"id" -> "SeeAlso", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["SeeAlsoSectionText", GetParameter["language"]]}], 
      Recurse[]
    }]
  }], 
{Cell, "SeeAlso"} :> XMLElement["p", {"class"->"inlineFunctionList"}, {Recurse[]}],

{CellGroupData, "TutorialsSection"} :> 
  XMLElement["section", {"id" -> "Tutorials", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["TutorialsSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }], 
{Cell, "Tutorials"} :> Li[{}, {Recurse[]}],
			
{CellGroupData, "RelatedTutorialsSection"} :>
  XMLElement["section", {"id" -> "RelatedTutorials", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedTutorialsSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],

{CellGroupData, "TutorialRelatedLinksSection"} :> 
  XMLElement["section", {"id" -> "RelatedLinks", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["TutorialRelatedLinksSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],

{CellGroupData, "RelatedLinksSection"} :>
  XMLElement["section", {"id" -> "RelatedLinks", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedLinksSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
			
{CellGroupData, "MoreAboutSection"} :>
  XMLElement["section", {"id" -> "RelatedGuides", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["MoreAboutSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }], 
{Cell, "MoreAbout"} :> Li[{}, {Recurse[]}],

{CellGroupData, "NotesSection" } :> Module[{nstitle},
  (* Print["entering NotesSection transform; First[SelectChildren[]] = "<>ToString[First[SelectChildren[]]]]; *)
  nstitle = Flatten[{Transmogrify[First[SelectChildren[]]]}];
  XMLElement["section", {"id" -> "DetailsAndOptions"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, nstitle],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, Flatten[{Transmogrify[Rest[SelectChildren[]]]}]]
      }]
    }]
  }]
],

{CellGroupData, "ImportExportSection" } :>
	XMLElement["section", {"id"->"ImportAndExport"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, {Localized["ImportExportSectionText", GetParameter["language"]]}],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
      }]
    }]
  }],

{CellGroupData, "ElementsSection" } :>
	XMLElement["section", {"id"->"Elements"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, {Localized["ElementsSectionText", GetParameter["language"]]}],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
      }]
    }]
  }],

{CellGroupData, "OptionsSection" } :>
	XMLElement["section", {"id"->"Options"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, {Localized["OptionsSectionText", GetParameter["language"]]}],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
      }]
    }]
  }],
			
{CellGroupData, "NotebookInterfaceSection" } :>
	XMLElement["section", {"id"->"NotebookInterface"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, {Localized["NotebookInterfaceSectionText", GetParameter["language"]]}],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
      }]
    }]
  }],

{CellGroupData, "DeviceSubsection" } :> Module[{dstitle},
  dstitle = Flatten[{Transmogrify[First[First[SelectChildren[]]]]}];
	XMLElement["section", {"id"->"Elements"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, dstitle],
      XMLElement["div", {"class" -> "hideable"}, {
        XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
      }]
    }]
  }]
],

{CellGroupData, "ServiceSubsection" } :> Module[{dstitle},
  dstitle = Flatten[{Transmogrify[First[First[SelectChildren[]]]]}];
	XMLElement["section", {"id"->"ServiceSubsection"}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {"class" -> "toggle"}, dstitle],
      XMLElement["div", {"class" -> "hideable"}, Flatten[{Transmogrify[Rest[SelectChildren[]]]}]]
    }]
  }]
],

{CellGroupData, "RelatedDemonstrationsSection"} :> Module[{dstitle},
  dstitle = Flatten[{Transmogrify[First[First[SelectChildren[]]]]}];
  XMLElement["section", {"id" -> "RelatedDemonstrationsSection", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, dstitle], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }]
], 

{Cell, "UsageSection" | "LegacyMaterialSection" | "ApplicationNotesSection" | "DesignDiscussionSection" | "ExtensionsSection" 
} :> 
Module[{name, style, ex, versionNumber},
versionNumber = StringSplit[$Version, "."][[1]];
style = GetStyle[];
Which[ style === "ElementsSection", name = "ELMS",  
style === "ExtensionsSection", name = "EXTS",  
True, name = StringJoin@Flatten@{If[First@ToCharacterCode[#] < 91, #, ""] & /@ Characters[ GetStyle[] ]};
];
ex = If[MemberQ[
  {"NotesSection", "ImportExportSection", "NotebookInterfaceSection", "ElementsSection", "DeviceSubsection", "ServiceSubsection", "OptionsSection"},
  style], "closed", "open"];
A[{"name" -> GetStyle[], "id"->"hello" }, {
  DIV[ 
  	(*If[name==="NS",{"class" -> name<>"H", "id"->"ModInfo_"<>versionNumber, "name"->"ModInfo_"<>versionNumber},{"class" -> name<>"H" }]*)
  	{"class" -> name<>"H"}
  	, {
    A[{"href" -> ("javascript:toggle('" <> name <> "')"), "onmouseover" -> "javascript:return(true);" }, {
    Img[{"src"->(GetParameter["htmlBase"]<>"/mathematicaImages/"<>ex<>"SectionGroup.gif"), "id"->name<>"_", "border"->"0", "width"->"8", "height"->"8", "alt"->""}], 
      " ",
      Span[{"class"->name<>"Span"}, { Recurse[] }]
    }]
  }]
}] ],

{Cell, "DeviceSubsection" |  "ServiceSubsection" | "MoreAboutSection" | "SeeAlsoSection" | "TutorialsSection" | "RelatedLinksSection" | "RelatedTutorialsSection" | "TutorialRelatedLinksSection" | "TutorialMoreAboutSection" | "RelatedDemonstrationsSection" | "TutorialCollectionSection" | "ElementsSection" | "ImportExportSection" | "NotebookInterfaceSection" | "OptionsSection"} :> Unevaluated[Sequence[]],

{Cell,"NotesSection"} :> Recurse[],

(* for some reason <p class="ExampleDelimiter" /> doesn't bode well with browsers.
   therefore, I have added a " " *)
{Cell, "ExampleDelimiter"} :> 
DIV[{"class" -> "ExLine"}, {XML`RawXML["<!-- -->"]}],


{Cell, "ExampleDelimiterInput"} :> 
DIV[{"class" -> "ExLineI"}, {XML`RawXML["<!-- -->"]}],
  

{Cell, "MoreAbout" | "TutorialMoreAbout" | "Tutorials" | "RelatedDemonstrations" | "RelatedLinks" | "LegacyMaterial" | 
"ApplicationNotes" | "DesignDiscussion" | "Extensions"} :> 
  Li[{}, {Recurse[]}],
  
{Cell, "TutorialCollection"} :> Unevaluated[Sequence[]],
(* Scale picture smaller *)
(* TODO find more suitable alt & title tags for GraphicsData and RawData *)
{Cell, "Picture"} :> (
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, "Image_" <> IncrementCounter["Image"] <> ".gif"],
    {"class"->If[GetParameter["entityType"]==="HowTo","HowToPicture",""], "alt" -> ""},
    Inline -> False,
    MaxImageSize->580,
    CropImage -> False,
    ImageFormat -> "GIF",
    "RetinaImage" -> False
  ]
),

{Cell, "FlushInput"} :>
  WithParameters[{"newline2br" -> True},
    XMLElement["p", {"class" -> "StandardForm", "id" -> GetStyle[]}, {
      XMLElement["b", {}, {Recurse[]}]}
      ] ],




{FormBox} :> Module[{boxChildren,traditionalFormQ,altTag},
boxChildren=SelectChildren[];
traditionalFormQ = If[GetStyle[] === TraditionalForm, True, False];
altTag = DocumentationBuild`Utils`CellToString@SelectSelf[];
If[MatchQ[boxChildren, {"Mathematica"}],Transmogrify[StyleBox["Mathematica", FontSlant->"Italic"]],
If[MatchQ[boxChildren, {StyleBox[_, "KeyEvent", ___]}], Transmogrify[boxChildren],
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".png"],
  {"style"->"vertical-align:middle;", "alt"->altTag, "title"->altTag},
  Inline -> True,
  ImageFormat -> "PNG",
  TransparentBackground->True,
  ConversionOptions->{Background->None},
  "RetinaImage" -> True
]
]
]
],

 
(* TODO find more suitable alt & title tags for GraphicsData and RawData *)
{GraphicsData | RawData} :>  
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["Image"] <> ".gif"],
  {"alt" -> "", "title" -> ""},
  Inline -> True,
  "RetinaImage" -> True
],


{OutputFormData} :> Transmogrify[SelectChildren[][[2]]]  

},

IncludeXMLTransforms -> {{"HTML", "Functions.m"}},

DefaultParameters -> {
  (*  "newline2br" -> False,  *)
  "newline2br" -> False,
  "newline2div" -> False,
  "styleUsageMessageBreak" -> "</div>\n<div class=\"usageDelimiter\" > </div><div class=\"UM\">",


  (* No Operation code *)
  "noop" -> {
    $LanguageExtension := Switch[GetParameter["language"], "Japanese", "ja", "ChineseSimplified", "zh", 
    	"Spanish", "es", _, "en"];
(* exceptionsList:= {True}, *)
    exceptionsList:= {"Symbol", "Menu Item", "File", "System Program", "Front End Object", 
    	"MathLink C Function", "LibraryLink C Function","System Program", "Widget", "Message", "Indicator"}; 
  }
}

]