(* :Title: Tutorials.m *)

(* :Author:
	Andrew Hunt, andy@wolfram.com
	Greg Martel, gregm@wolfram.com
*)

(* :Mathematica Version: 5 *)
(* :Package Version: $Revision: 1.15 $ $Date: 2018/03/16 01:35:15 $ *)                  
(* :Copyright: (c) 2005, Wolfram Research, Inc. All rights reserved. *)
(* :Summary: Add-ons specific rules and general transform for Mathematica Documentation *)


XMLTransform[{
(* Remove in-product decorations *)
{Cell, "AnchorBar"} :> Unevaluated[Sequence[]],

{Cell, "UpgradeSummary"} :>
		XMLElement["p",{"class" -> "UpgradeSummary"}, { 
		Recurse[]
		 }],

{Cell, "UpgradePackageListing"} :>
		XMLElement["p", {"class" -> "UpgradePackageListing"}, { 
		Recurse[]
		 }],
		 
{Cell, "UpgradeMessage"} :>
		XMLElement["span", {"class" -> "UpgradeMessage"}, { 
		Recurse[]
		 }],

{CellGroupData, "MathCaption"} :> 
	XMLElement["table", {"class" -> "MCapTab"}, {
		XMLElement["tr", {}, {
			XMLElement["td", {}, {Recurse[]}   ]
		 }]    
	}],  

{Cell, "TOCDocumentTitle"} :> XMLElement["div", {"class" -> "main-heading"}, {XMLElement["h1", {"class" -> "main-title"}, {Recurse[]}]}],

{Cell, "TutorialAbstract"} :> Unevaluated[Sequence[
  XMLElement["div", {"class" -> "TutorialAbstract"}, {
    XMLElement["div", {"class" -> "inner"}, {Recurse[]}]
  }]
]],

(*
h2:
  if CellGroupData:
    open a section
    h2 "overviews toggle"
  else
    undefined

h3:
  if CellGroupData:
    opens a hideable div
    h3 "overviews toggle"
  else
    h3 "overviews"

h4:
  if CellGroupData:
    opens a hideable div
    h4 "overviews toggle"
  else
    h4 "overviews"

h5:
  if CellGroupData:
    opens a hideable div
    h5 "overviews toggle"
  else
    h5 "overviews"

======================================================================

Recurse::usage = "Recurse[] is a shorthand for Transmogrify[SelectChildren[]] for use in XMLTransforms."

SelectSelf[] is the entire CellGroupData expression
SelectChildren[] is the top-level list enclosing all cells in the CellGroupData

SelectSelf[]: this is always the single TOCSection cell
SelectSiblings[]: when part of a CellGroupData, this is all the remaining TOCSubsection cells forming the remainder of the current top-level list of the parent CellGroupData

when there is a parent TOCSection CellGroupData, it Recurse[]s into here first.
then when we arrive here (when GetParameter["TOCSectionChildren"] is True), -
SelectSelf[] is the TOCSection cell and SelectSiblings[] is the rest of the cells
in the parent cellgroup list

h2:
  if CellGroupData:
    open a section
    h2 "overviews toggle"
  else
    undefined
*)

{CellGroupData, "MonographTOCChapter"} :> Module[{},
  dprint["entering {CellGroupData, \"MonographTOCChapter\"}, SelectSelf[] = "<>ToString[SelectSelf[], InputForm]];
  XMLElement["section", {}, {Recurse[]}]
],
{Cell, "MonographTOCChapter"} :> Module[{},
  dprint["entering {Cell, \"MonographTOCChapter\"}, SelectSelf[] = "<>ToString[SelectSelf[], InputForm]];
  If[
    GetParameter["TOCChildren"]===True
  , XMLElement["h2", {"class" -> "toggle open"}, {Recurse[]}]
  , XMLElement["section", {}, {
      XMLElement["h2", {}, {Recurse[]}]
    }]
  ]
],

{CellGroupData, "TOCChapter"} :> Module[{},WithParameters[{"TOCChildren" -> True},
  XMLElement["section", {}, {
    XMLElement["div", {"class" -> "inner"}, {
      Transmogrify[ SelectSelf[][[1, 1]] ],
      XMLElement["div", {"class" -> "hideable"}, {Transmogrify[ Rest[SelectChildren[]] ]}]
    }]
  }]
]],

{Cell, "TOCChapter"} :> If[
	GetParameter["TOCChildren"]===True, 
	XMLElement["h2", {"class" -> "toggle open"}, {Recurse[]}], 
	XMLElement["section", {}, {
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h2", {}, {Recurse[]}]
    }]
  }]
	],

{CellGroupData, "TOCSection"} :> Module[{},WithParameters[{"TOCSectionChildren" -> True},
  XMLElement["div", {}, {
    Transmogrify[ SelectSelf[][[1, 1]] ],
    XMLElement["div", {"class" -> "hideable"}, {Transmogrify[ Rest[SelectChildren[]] ]}]
  }]
]],

{Cell, "TOCSection"} :> If[
	GetParameter["TOCSectionChildren"]===True, 
	XMLElement["h3", {"class" -> "toggle"}, {Recurse[]}], 
	XMLElement["h3", {}, {Recurse[]}]
	],

{CellGroupData, "TOCSubsection"} :> WithParameters[{"TOCSubsectionChildren" -> True},
  XMLElement["div", {}, {
    Transmogrify[ SelectSelf[][[1, 1]] ],
    XMLElement["div", {"class" -> "hideable"}, {Transmogrify[Rest[SelectChildren[]]]}]
  }]
],
        
{Cell, "TOCSubsection"} :> If[
	GetParameter["TOCSubsectionChildren"]===True, 
	XMLElement["h4", {"class" -> "toggle"}, {Recurse[]}], 
	XMLElement["h4", {}, {Recurse[]}]],

{CellGroupData, "TOCSubsubsection"} :> WithParameters[{"TOCSubsubsectionChildren" -> True},
  XMLElement["div", {}, {
    Transmogrify[ SelectSelf[][[1, 1]] ],
    XMLElement["div", {"class" -> "hideable"}, {Transmogrify[Rest[SelectChildren[]]]}]
  }]],

{Cell, "TOCSubsubsection"} :> 
	If[GetParameter["TOCSubsubsectionChildren"]===True, 
	XMLElement["h5", {"class" -> "toggle"}, {Recurse[]}], 
	XMLElement["h5", {}, {Recurse[]}]],

{CellGroupData, "TOCSubsubsubsection"} :> WithParameters[{"TOCSubsubsubsection" -> True},
  XMLElement["div", {}, {
    Transmogrify[ SelectSelf[][[1, 1]] ],
    XMLElement["div", {"class" -> "hideable"}, {Transmogrify[Rest[SelectChildren[]]]}]
  }]],

{Cell, "TOCSubsubsubsection"} :> If[GetParameter["TOCSubsubsubsection"]===True, 
	XMLElement["h6", {"class" -> "toggle"}, {Recurse[]}], 
	XMLElement["h6", {}, {Recurse[]}]],
(* ====================================================================== *)

{Cell, "Title"} :> (ResetCounters["ItemizedText"];  H1[{"class" -> "Title"}, {Recurse[]}]),
{Cell, "Section"} :> (ResetCounters["ItemizedText"];  H2[{"class" -> "Section"}, {Recurse[]}]),
{Cell, "Subsection"} :> (ResetCounters["ItemizedText"];  H3[{"class" -> "Subsection"}, {Recurse[]}]),
{Cell, "Subsubsection"} :> (ResetCounters["ItemizedText"];  H4[{"class" -> "Subsubsection"}, {Recurse[]}]),
{Cell, "Subsubsubsection"} :> (ResetCounters["ItemizedText"];  XMLElement["H5", {"class" -> "Subsubsubsection"}, {Recurse[]}]),

{Cell, "Text"} :> WithParameters[{"newline2br" -> True},
	If[MatchQ[SelectParent[], Cell[BoxData[PopupMenuBox[a__]], b___]], 
		Span[{"class" -> "Text"}, {Recurse[]}], 
	P[{"class" -> "Text"}, {Recurse[]}]
	] 
],


{Cell,  "2ColumnTable" | "3ColumnTable"} :>
 				WithParameters[{"tableStyle" -> GetStyle[]},
					(
XMLElement["div", {"class" -> "FTab", "id"-> GetStyle[]}, {
	If[HasOption[CellTags], 
		Map[XMLElement["a", {"name" -> #}, {}] &, 
		StringReplace[Flatten[{GetOption[CellTags]}], ":" -> "_"]     ]],
	Recurse[]	}] 
)  ], 



{Cell, "Picture" | "Table"} :>
Module[{iCounter},
iCounter = IncrementCounter["Image"];

P[{"class" -> "Picture"}, {
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, iCounter <> ".gif"],
    {"alt" -> iCounter <> ".gif", "title" -> iCounter <> ".gif"},
    Inline -> True,
    CropImage -> False,
    ImageFormat -> "GIF",
    "RetinaImage" -> False
  ]
}]
],

{Cell, "Subsection" | "SubsectionIcon" | "SubsectionFirst" | "Command" | 
"ContentsTitle" | "InspectorLevel1"} :>
		H2[{"id" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]}],


{Cell, "FlushInput"} :>
	WithParameters[{"newline2br" -> True},
		P[{"class" -> "StandardForm", "id" -> GetStyle[]}, {
			XMLElement["b", {}, {Recurse[]}]  }
			] ], 


{Cell, "ItemizedText"} :> P[{"class" -> (GetStyle[]/.None->"FIXME")}, 
Sequence@{XMLElement["span", {}, {IncrementCounter["ItemizedText"] <> ". ", XML`RawXML["&nbsp;"]}],
Recurse[]
} ],

{Cell, "ItemizedTextParagraph"} :> P[{"class" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]} ], 


(* {CellGroupData, "TOCSection" | "TOCSubsection"} :> Recurse[], 


{Cell,  "TOCSection" | "TOCSubsection" | "TOCSubsubsection"} :> 
	Ol[{"id"->GetStyle[]}, {Li[{"id"->GetStyle[]}, {Recurse[]}]
	}], *)

(***** Cell Definitions *****)

{CellGroupData, "RelatedTutorialsSection"} :>
  XMLElement["section", {"id" -> "RelatedTutorials", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedTutorialsSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "RelatedTutorialsSection" } :> Unevaluated[Sequence[]],
{Cell, "RelatedTutorials"} :> XMLElement["li", {}, {Recurse[]}], 

{CellGroupData, "TutorialMoreAboutSection"} :>
  XMLElement["section", {"id" -> "RelatedGuides", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["MoreAboutSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "TutorialMoreAboutSection" } :> Unevaluated[Sequence[]],
{Cell, "TutorialMoreAbout"} :> XMLElement["li", {}, {Recurse[]}], 

{CellGroupData, "TutorialRelatedLinksSection"} :> 
  XMLElement["section", {"id" -> "RelatedLinks", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["TutorialRelatedLinksSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "TutorialRelatedLinksSection" } :> Unevaluated[Sequence[]],

{CellGroupData, "TutorialsTutorialCollectionSection"} :> 
  XMLElement["section", {"id" -> "TutorialCollection"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["TutorialCollectionSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "TutorialsTutorialCollectionSection" } :> Unevaluated[Sequence[]],

{CellGroupData, "TutorialCollectionSection"} :> 
  XMLElement["section", {"id" -> "TutorialCollection"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["TutorialCollectionSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "TutorialCollectionSection" } :> Unevaluated[Sequence[]],

{Cell, "TutorialRelatedLinks" | "TutorialCollection"} :> Li[{Recurse[]}],

{Cell, "Graphics" | "ItemizedPicture"} :> 
P[{"class"->GetStyle[]}, {
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["TutorialGraphics"] <> "_tg.gif"],
  {"alt"->""},
  Inline -> True,
  CropImage -> False,
  "RetinaImage" -> True
]}],

{OutputFormData} :> Transmogrify[SelectChildren[][[2]]]

},

IncludeXMLTransforms -> {{"HTML","Components","Includes.m"}},

DefaultParameters -> {
	(*  "newline2br" -> False,  *)
  "TOCChildren" -> False,
  "TOCSectionChildren" -> False,
  "TOCSubsectionChildren" -> False,
  "TOCSubsubsectionChildren" -> False,
  "TOCSubsubsubsection" -> False,
	"newline2br" -> False,
	"newline2div" -> False,
	"styleUsageMessageBreak" -> "</div>\n<div class='UsageMessage'>",

	(* No Operation code *)
	"noop" -> {
		$LanguageExtension = "en";

	}
}

]
