(* :Title: Tutorials.m *)

(* :Author:
	Andrew Hunt, andy@wolfram.com
	Greg Martel, gregm@wolfram.com
*)

(* :Mathematica Version: 5 *)
(* :Package Version: $Revision: 1.24 $ $Date: 2018/03/14 20:43:42 $ *)                  
(* :Copyright: (c) 2005, Wolfram Research, Inc. All rights reserved. *)
(* :Summary: Add-ons specific rules and general transform for Mathematica Documentation *)


XMLTransform[{

(* a scratch cell for debugging. use as Cell["point 1", "gronk"] or similar *)
{Cell, "gronk"} :> Module[{},
  Print[ToString[InputForm[SelectSelf[]]]];
  XMLElement["div", {"class" -> "gronk"}, {Recurse[]}]
],

(* Remove in-product decorations *)
{Cell, "AnchorBar"} :> Unevaluated[Sequence[]],

{Cell, "UpgradeSummary"} :>
		XMLElement["p",{"class" -> "UpgradeSummary"}, { 
		Recurse[]
		 }],

{Cell, "UpgradePackageListing"} :>
		XMLElement["p", {"class" -> "UpgradePackageListing"}, { 
		Recurse[]
		 }],
		 
{Cell, "UpgradeMessage"} :>
		XMLElement["span", {"class" -> "UpgradeMessage"}, { 
		Recurse[]
		 }],

{CellGroupData, "MathCaption"|"MonographMathCaption"} :> 
	XMLElement["table", {"class" -> "MCapTab"}, {
		XMLElement["tr", {}, {
			XMLElement["td", {}, {Recurse[]}   ]
		 }]    
	}],  

{Cell, "DataSourceTitle"} :>
  XMLElement["div", {"class" -> "main-heading"}, {
    XMLElement["h1", {"class" -> "main-title"}, {Recurse[]}]
  }],
		 
{Cell, "DataSourceSummary"} :>
		XMLElement["p", {"class" -> "DataSourceSummary"}, { 
		Recurse[]
		 }],
		 
{Cell, "DataSourceSection"} :>
		XMLElement["p", {"class" -> "DataSourceSection"}, { 
		Recurse[]
		 }],

{Cell, "DataSourceContact"} :>
		XMLElement["p", {"class" -> "DataSourceContact"}, { 
		Recurse[]
		 }],


{Cell, "DataSourceText"} :>
		XMLElement["p", {"class" -> "DataSourceText"}, { 
		Recurse[]
		 }],

{Cell, "Title"|"MonographTitle"|"TechNoteTitle"} :> (
  ResetCounters["ItemizedText"];
  ResetCounters["ItemNumbered"];
  (* XMLElement["div", {"class" -> "main-heading"}, {XMLElement["h1", {"class" -> "Title main-title"}, {Recurse[]}]}] *)
  If[
    Or[
      GetParameter["packageQ"], (* avoid doubled main-heading div, the other provided by GridBox.m *)
      GetParameter["monographObjectNameGridQ"]
    ]
  , XMLElement["h1", {"class" -> "Title main-title"}, {Recurse[]}]
  , XMLElement["div", {"class" -> "main-heading"}, {XMLElement["h1", {"class" -> "Title main-title"}, {Recurse[]}]}]
  ]
),

{CellGroupData, "MonographBodyWrapper"} :>
  XMLElement["section", {}, {
    XMLElement["div", {"class" -> "inner"}, {
      Recurse[]
    }]
  }],
{Cell, "MonographBodyWrapper"} :> Unevaluated[Sequence[]],

{Cell, "UpgradeObjectName"} :> XMLElement["h1", {"class" -> "UpgradeObjectName"}, {Recurse[]}],
{Cell, "Section"} :> (ResetCounters["ItemizedText"];  ResetCounters["ItemNumbered"]; H2[{"class" -> "Section"}, {Recurse[]}]),
{Cell, "Subsection"} :> (ResetCounters["ItemizedText"]; ResetCounters["ItemNumbered"]; H3[{"class" -> "Subsection"}, {Recurse[]}]),
{Cell, "UpgradeDetailLabel"} :> H3[{"class" -> "UpgradeDetailLabel"}, {Recurse[]}],
{Cell, "Subsubsection"} :> (ResetCounters["ItemizedText"]; ResetCounters["ItemNumbered"]; H4[{"class" -> "Subsubsection"}, {Recurse[]}]),
{Cell, "Subsubsubsection"} :> (ResetCounters["ItemizedText"]; ResetCounters["ItemNumbered"]; XMLElement["H5", {"class" -> "Subsubsubsection"}, {Recurse[]}]),

{CellGroupData, "MonographHeader"} :> Module[{},
  dprint["entering {CellGroup, MonographHeader}, SelectSelf[] = "<>ToString[SelectSelf[], InputForm]];
  XMLElement["div", {"class" -> "breadcrumb"}, {Recurse[]}]
],
{Cell, "MonographHeader"} :> Module[{},
  dprint["entering {Cell, MonographHeader}, SelectSelf[] = "<>ToString[SelectSelf[], InputForm]];
  XMLElement["a", {"class" -> "overview"}, {Recurse[]}]
],
{Cell, "MonographSubheader"} :> XMLElement["p", {"class" -> "current"}, {Recurse[]}],

{Cell, "MonographBackLink"} :> Module[{},
  dprint["entering MonographBackLink, SelectSelf[] = "<>ToString[SelectSelf[], InputForm]];
	XMLElement[
		"a",
		{"href" -> Quiet @ Check[
			First @ Cases[
				SelectSelf[],
				Rule[ButtonData, s_String] :> DocumentationBuild`Export`DocLinkURL[
					{GetParameter["pacletName"], GetParameter["url"]},
					s,
					GetParameter["urlBaseRemote"],
					GetParameter["CompleteHTMLQ"]
				],
				Infinity
			],
			""
		]},
		{XML`RawXML["&lsaquo;"]}
	]
],
{Cell, "MonographInactiveBackLink"} :> Module[{},
  dprint["MonographInactiveBackLink, SelectSelf[] = "<>ToString[SelectSelf[], InputForm]];
  XMLElement["a", {"class" -> "inactive"}, {XML`RawXML["&lsaquo;"]}]
],

{Cell, "MonographForeLink"} :> Module[{},
  dprint["entering MonographForeLink, SelectSelf[] = "<>ToString[SelectSelf[], InputForm]];
	XMLElement[
		"a",
		{"href" -> Quiet @ Check[
			First @ Cases[
				SelectSelf[],
				Rule[ButtonData, s_String] :> DocumentationBuild`Export`DocLinkURL[
					{GetParameter["pacletName"], GetParameter["url"]},
					s,
					GetParameter["urlBaseRemote"],
					GetParameter["CompleteHTMLQ"]
				],
				Infinity
			],
			""
		]},
		{XML`RawXML["&rsaquo;"]}
	]
],
{Cell, "MonographInactiveForeLink"} :> Module[{},
  dprint["MonographInactiveForeLink, SelectSelf[] = "<>ToString[SelectSelf[], InputForm]];
  XMLElement["a", {"class" -> "inactive"}, {XML`RawXML["&rsaquo;"]}]
],

{Cell, "MonographHamburgerLink"} :> Module[{},
  (* dprint["entering MonographHamburgerLink, SelectSelf[] = "<>ToString[SelectSelf[], InputForm]]; *)
  XMLElement["li", {}, {Recurse[]}]
],

{CellGroupData, "MonographHamburgerGroup"} :> Module[{back, fore, burger, self = SelectSelf[]},
  (* dprint["entering MonographHamburgerGroup, SelectSelf[] = "<>ToString[SelectSelf[], InputForm]]; *)
  back =   Cases[self, Cell[_, s_String /; StringMatchQ[s, {"MonographBackLink", "MonographInactiveBackLink"}], ___], Infinity];
  back = back /. Cell[_, "MonographInactiveBackLink", rest:___] -> Cell["", "MonographInactiveBackLink", rest];
dprint["MonographHamburgerGroup: back = "<>ToString[back, InputForm]];
  fore =   Cases[self, Cell[_, s_String /; StringMatchQ[s, {"MonographForeLink", "MonographInactiveForeLink"}], ___], Infinity];
  fore = fore /. Cell[_, "MonographInactiveForeLink", rest:___] -> Cell["", "MonographInactiveForeLink", rest];
dprint["MonographHamburgerGroup: fore = "<>ToString[fore, InputForm]];
  burger = Cases[self, Cell[_, "MonographHamburgerLinks", ___], Infinity];
dprint["MonographHamburgerGroup: burger = "<>ToString[burger, InputForm]];

  XMLElement["div", {"class" -> "overview-menu"}, {
    XMLElement["div", {"class" -> "overview-nav-buttons"}, {
      Transmogrify[back],
      XMLElement["button", {"class" -> "hamburger", lang -> "en"}, {XML`RawXML["&equiv;"]}],
      Transmogrify[fore]
    }],
    XMLElement["ul", {"class" -> "overview-menu-items hide"}, {
      Transmogrify[burger]
    }]
  }]
],

{CellGroupData, "MonographBulletedTextContainer"} :> XMLElement["ul", {"class" -> "bullet-list"}, {Recurse[]}],
{Cell, "MonographBulletedTextContainer"} :> Unevaluated[Sequence[]],
{Cell, "MonographBulletedText"} :> XMLElement["li", {}, {Recurse[]}],

{Cell, "TutorialAbstract"} :> Unevaluated[Sequence[
  XMLElement["div", {"class" -> "TutorialAbstract"}, {
    XMLElement["div", {"class" -> "inner"}, {Recurse[]}]
  }]
]],

{Cell, "Text"} :> WithParameters[{"newline2br" -> True},
	If[MatchQ[SelectParent[], Cell[BoxData[PopupMenuBox[a__]], b___]], 
		Span[{"class" -> "Text"}, {Recurse[]}], 
	P[{"class" -> "Text"}, {Recurse[]}]
	] 
],


{Cell,  "2ColumnTable" | "3ColumnTable"} :>
 				WithParameters[{"tableStyle" -> GetStyle[]},
					(
XMLElement["div", {"class" -> "FTab", "id"-> GetStyle[]}, {
	If[HasOption[CellTags], 
		Map[XMLElement["a", {"name" -> #}, {}] &, 
		StringReplace[Flatten[{GetOption[CellTags]}], ":" -> "_"]     ]],
	Recurse[]	}] 
)  ], 



{Cell, "Picture" | "Table"} :>
Module[{iCounter},
iCounter = IncrementCounter["Image"];

P[{"class" -> "Picture"}, {
  transmogrifyImage[
    ToFileName[{"Files", GetBasenameLa[]}, iCounter <> ".gif"],
    {"alt" -> iCounter <> ".gif", "title" -> iCounter <> ".gif"},
    Inline -> True,
    CropImage -> False,
    ImageFormat -> "GIF",
    "RetinaImage" -> False
  ]
}]
],

{Cell, "ExampleDelimiter"} :> 
DIV[{"class" -> "ExLine"}, {XML`RawXML["<!-- -->"]}],

{Cell, "Subsection" | "SubsectionIcon" | "SubsectionFirst" | "Command" | 
"ContentsTitle" | "InspectorLevel1"} :>
		H2[{"id" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]}],


{Cell, "FlushInput"} :>
	WithParameters[{"newline2br" -> True},
		P[{"class" -> "StandardForm", "id" -> GetStyle[]}, {
			XMLElement["b", {}, {Recurse[]}]  }
			] ], 


{Cell, "ItemizedText"} :> P[{"class" -> (GetStyle[]/.None->"FIXME")}, 
Sequence@{XMLElement["span", {}, {IncrementCounter["ItemizedText"] <> ". ", XML`RawXML["&nbsp;"]}],
Recurse[]
} ],

{Cell, "ItemNumbered"} :> P[{"class" -> (GetStyle[]/.None->"FIXME")}, 
Sequence@{XMLElement["span", {}, {IncrementCounter["ItemNumbered"] <> ". ", XML`RawXML["&nbsp;"]}],
Recurse[]
} ],

{Cell, "Notes"} :> XMLElement["li", Flatten[{
  If[And[HasOption[Background], MatchQ[ GetOption[Background], _Dynamic]], 
    {"class" -> "modified-text"},
    {}
  ]
}], {Recurse[]}],

{Cell, "ItemizedTextParagraph"} :> P[{"class" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]} ], 

{Cell, "ItemParagraph"} :> P[{"class" -> (GetStyle[]/.None->"FIXME")}, {Recurse[]} ], 
(* {CellGroupData, "TOCSection" | "TOCSubsection"} :> Recurse[], 


{Cell,  "TOCSection" | "TOCSubsection" | "TOCSubsubsection"} :> 
	Ol[{"id"->GetStyle[]}, {Li[{"id"->GetStyle[]}, {Recurse[]}]
	}], *)


{CellGroupData, "TutorialNotesList"} :> XMLElement["ul", {"class" -> "Notes"}, {Recurse[]}],
{Cell, "TutorialNotesList"} :> Unevaluated[Sequence[]],

(***** Cell Definitions *****)

{CellGroupData, "RelatedTutorialsSection"} :>
  XMLElement["section", {"id" -> "RelatedTutorials", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["RelatedTutorialsSectionText", GetParameter["language"]]}],
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "RelatedTutorialsSection" } :> Unevaluated[Sequence[]],

{CellGroupData, "TutorialMoreAboutSection" } :>
  XMLElement["section", {"id" -> "RelatedGuides", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["GuideMoreAboutSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "TutorialMoreAboutSection" } :> Unevaluated[Sequence[]],

{CellGroupData, "TutorialRelatedLinksSection"} :> 
  XMLElement["section", {"id" -> "RelatedLinks", "class" -> "related-links"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["TutorialRelatedLinksSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "TutorialRelatedLinksSection" } :> Unevaluated[Sequence[]],

{CellGroupData, "TutorialsTutorialCollectionSection"} :>
  XMLElement["section", {"id" -> "TutorialCollection"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["TutorialCollectionSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "TutorialsTutorialCollectionSection" } :> Unevaluated[Sequence[]],

{CellGroupData, "TutorialCollectionSection" } :>
  XMLElement["section", {"id" -> "TutorialCollection"},	{
    XMLElement["div", {"class" -> "inner"}, {
      XMLElement["h1", {}, {Localized["TutorialCollectionSectionText", GetParameter["language"]]}], 
      XMLElement["ul", {"class" -> "functionList"}, {Recurse[]}]
    }]
  }],
{Cell, "TutorialCollectionSection" } :> Unevaluated[Sequence[]],

{Cell, "RelatedTutorials" | "TutorialRelatedLinks" | "TutorialMoreAbout" | "TutorialCollection"} :> Li[{Recurse[]}],

{Cell, "Graphics" | "ItemizedPicture" | "ItemPicture"} :> 
P[{"class"->GetStyle[]}, {
transmogrifyImage[
  ToFileName[{"Files", GetBasenameLa[]}, IncrementCounter["TutorialGraphics"] <> "_tg.gif"],
  {"alt"->""},
  Inline -> True,
  CropImage -> False,
  "RetinaImage" -> True
]}],

{OutputFormData} :> Transmogrify[SelectChildren[][[2]]]

},

IncludeXMLTransforms -> {{"HTML","Components","Includes.m"}},

DefaultParameters -> {
	(*  "newline2br" -> False,  *)
	"newline2br" -> False,
	"newline2div" -> False,
	"styleUsageMessageBreak" -> "</div>\n<div class='UsageMessage'>",

	(* No Operation code *)
	"noop" -> {
		$LanguageExtension = "en";
	}
}

]
