
(*
 Package to provide Mathematica code for the Wolfram Workbench.
*)

BeginPackage["MEET`"]

Needs[ "MEET`Profiler`"];
Needs["JLink`"];

SetupIDE::usage = "SetupIDE[ debug] sets up the IDE for debugging." 

SetOutputHandler::usage = "SetOutputHandler[] sets up output handler." 

AttachFrontEnd::usage = "AttachFrontEnd[] attaches the session to a frontend."

SetCallbackHandler::usage = "SetCallbackHandler[] sets up the breakpoint handler."

SetMessageBreakpoint::usage = "SetMessageBreakpoint[] sets a message breakpoint."

UpdateBreakpoint::usage = "UpdateBreakpoint[] updates the status of a breakpoint."

GetTaggedFiles::usage = "GetTaggedFiles[] gets the tagged files."

SetDebuggerLinks::usage = "SetDebuggerLinks[] sets the debugger links."

DynamicDebugging::usage = "DynamicDebugging[ True] enables debugging of dynamics."

CallKernelAtomic::usage = "CallKernelAtomic[ expr] evaluates expr in an atomic fashion."

FrontEndTagNotify::usage = "FrontEndTagNotify[ tag, clear] tells the FrontEnd to display or clear a tag."

ResetCodeBreakpoints::usage = "ResetCodeBreakpoints[] resets all code breakpoints in the system."

AddGUIPath::usage = "AddGUIPath[ path] adds path to the list of paths searched by GUIKit."

CanonicalizeFileName::usage = "CanonicalizeFileName[name] returns a canonical version of the filename."

SetWorkbenchCallHandler::usage = ""

ToInputFormString::usage = ""

$WorkbenchJavaVM::usage = "$WorkbenchJavaVM is the JVM used by the Workbench."

SetDynamicTriggerBreakpoint::usage = "SetDynamicTriggerBreakpoint[ True/False] sets or clears dynamic trigger breakpoints. "

UninstallIDE::usage = "UninstallIDE[] uninstalls the IDE"



CreateReaderLink::usage = "CreateReaderLink  "
CreateUILink::usage = "CreateUILink  "
SetCreatedLinks::usage = "SetCreatedLinks  ";

MEET`Utility`JLinkNonlocal = False

SetAssertBreakpoints::usage = "SetAssertBreakpoints[]  "



Begin[ "`Private`"]

MEET`Information`$Version = "MEET Version 0.0.3"
MEET`Information`$VersionNumber = 0.0
MEET`Information`$ReleaseNumber =4



(*
 Code for interacting with the Eclipse debugger.
 
 Some terminology.
 
 $DebuggerMainLink,  the main link used to communicate between the Kernel and Eclipse. 
 
 $DebuggerPreemptiveLink,  a preemptive link that Eclipse can use to do preemptive 
 computations in the Kernel.
*)



(*
 Specifies that we are running from a scrapbook file.
*)

$ScrapbookFile = True;

(*
 This prevents calls from the Kernel while we are already in a 
 call from the Kernel.  I'm not quite sure why it is necessary 
 and should discuss with Todd.  It shows itself if a message 
 is generated by an Expressions eval.
*)

$KernelCall = True;


(*
Dialog for breaks
*)
$DialogForBreaks = False;

(*
 Name the $DebuggerMainLink and create the $DebuggerPreemptiveLink.
 Return the name of the $DebuggerPreemptiveLink, so this can be 
 connected to from Eclipse.
 
 InstallJava is called on the ReaderLink,  so Mathematica 
 will call to Java on this.
 
 There is a UILink,  this is preemptive in the Kernel, and 
 the Workbench normally uses this to call to Mathematica.
 
 There is also an evaluation link,  this is not passed 
 in here,  it is used by the Workbench as a transaction 
 link.  Ie to load a test file during a test run.
 
 There is no JavaPreemptive link, for J/Link 
 to call from Mathematica to the Workbench on a 
 preemptive call when J/Link is already calling from 
 Mathematica to Java.   I don't think this is needed. 
 because when the Kernel calls the Workbnech we don't 
 want a preemptive call to come in.
 
 TODO  remove the commented out code when Beta 1 for 1.1 ships.
  
*)

$WorkbenchJavaVM

SetDebuggerLinks[ readerName_, uiName_, protocol_, dialogForBreaks_, JLinkNonlocal_, defaultJVM_] :=
	Module[ {},
		MEET`Utility`JLinkNonlocal = JLinkNonlocal;
		$DialogForBreaks = False (* dialogForBreaks*);
		$DebuggerReaderLink = LinkConnect[ readerName, LinkProtocol -> protocol];
		If[ JLink`Information`$VersionNumber >= 4,
			InstallJava[ $DebuggerReaderLink, Default -> defaultJVM, CreateExtraLinks -> False];
			If[ !JLinkNonlocal,
				If[ StringMatchQ[ $SystemID, "MacOSX*"],
					fixInstallJavaMac[],
					fixInstallJava[]]];
			,
			UseJVM[x_,y_] := y;
			InstallJava[ $DebuggerReaderLink]];
		fixStripBoxes[];
		$WorkbenchJavaVM = GetJVM[ $DebuggerReaderLink];
		$DebuggerEvaluationLink = $ParentLink;
		$DebuggerUILink = LinkConnect[uiName, LinkProtocol -> protocol];
		$WorkbenchJavaVM["JavaUILink"] = $DebuggerUILink;
		(* Not really part of the link system, but related via J/Link *)
		Internal`SetValueNoTrack[JLink`Package`inPreemptiveCallFromJava, True];
		MathLink`AddSharingLink[ $DebuggerUILink, 
					AllowPreemptive->True,
					MathLink`LinkSwitchPre -> linkSwitchPreUILink,
					MathLink`LinkSwitchPost -> linkSwitchPostUILink]
	]


CreateReaderLink[ protocol_] :=
	(
	$DebuggerReaderLink = LinkCreate[ LinkProtocol -> protocol];
	First[ $DebuggerReaderLink]
	)
	
CreateUILink[ protocol_] :=
	(
	$DebuggerUILink = LinkCreate[ LinkProtocol -> protocol];
	First[ $DebuggerUILink]
	)

SetCreatedLinks[ dialogForBreaks_, JLinkNonlocal_, defaultJVM_] :=
	Module[ {},
		MEET`Utility`JLinkNonlocal = JLinkNonlocal;
		$DialogForBreaks = False (* dialogForBreaks*);
		If[ JLink`Information`$VersionNumber >= 4,
			InstallJava[ $DebuggerReaderLink, Default -> defaultJVM, CreateExtraLinks -> False];
			If[ !JLinkNonlocal,
				If[ StringMatchQ[ $SystemID, "MacOSX*"],
					fixInstallJavaMac[],
					fixInstallJava[]]];
			,
			UseJVM[x_,y_] := y;
			InstallJava[ $DebuggerReaderLink]];
		$WorkbenchJavaVM = GetJVM[ $DebuggerReaderLink];
		$WorkbenchJavaVM["JavaUILink"] = $DebuggerUILink;
		MathLink`AddSharingLink[ $DebuggerUILink, 
					AllowPreemptive->True,
					MathLink`LinkSwitchPre -> linkSwitchPreUILink,
					MathLink`LinkSwitchPost -> linkSwitchPostUILink]
	]

linkSwitchPreUILink[ ___] :=
	Module[ {},
		JLink`Package`inPreemptiveCallFromJava[MEET`$WorkbenchJavaVM] = True;
		If[hasServiceFrontEnd[],
				MathLink`SetServiceFrontEnd[],
				Null]
	]

linkSwitchPostUILink[ oldFrontEnd_] :=
	Module[ {},
		JLink`Package`inPreemptiveCallFromJava[MEET`$WorkbenchJavaVM] = False;
		If[ oldFrontEnd =!= Null, 
			MathLink`RestoreFrontEnd[oldFrontEnd]];
	]


(*
 We need to fix InstallJava in cases where it has not been 
 called. This makes sure we find the native library that is 
 in the Workbench, this is the one that matches the version 
 of JLink we loaded.
 
 On the Mac we should just set JLink`InstallJava`Private`$jlinkAppDir.
*)

fixInstallJavaMac[] :=
	Module[ {file},
		file = ToFileName[ {$InstallationDirectory, "SystemFiles", "Links"}, "JLink"];
		If[ FileType[ file] === Directory,
				JLink`Package`$jlinkAppDir = file;
				Return[Null]];
		file = ToFileName[ {$InstallationDirectory, "AddOns"}, "JLink"];
		If[ FileType[ file] === Directory,
				JLink`Package`$jlinkAppDir = file;
				Return[Null]];
	]

fixInstallJava[] :=
	Module[ {path, propArg},
		path = DirectoryName[DirectoryName[ JLink`Package`$jlinkDir]];
		JLink`Package`$jlinkDir = ToFileName[ {path, "lib"}];
		propArg = "-Dcom.wolfram.jlink.libdir=" <> getNativeLib[path];
		SetOptions[InstallJava, JVMArguments -> propArg]
	]

getNativeLib[path_] :=
	getQuote[] <> ToFileName[path,
		Switch[ $SystemID,
			"Windows",
				"os\\win32",
			"Windows-x86-64",
				"os\\win64",
			"MacOSX",
				"os/macosx/ppc",
			"MacOSX-x86",
				"os/macosx/x86",
			"Linux",
				"os/linux/x86",
			"Linux-x86-64",
				"os/linux/x86_64",
			_,  ""]] <> getQuote[]


getQuote[] :=
	If[ $SystemID === "Windows" || $SystemID === "Windows-x86-64", "\"", "'"]


(*
  Add a rule to remove DebugTags in StripBoxes
*)

fixStripBoxes[] :=
	Module[ {},
		Unprotect[ StripBoxes];
		StripBoxes[ FormBox[ x_, DebugTag[fmt_,_]]] := StripBoxes[ FormBox[x,fmt]] ;
		Protect[ StripBoxes]
	]



(*
 Important: if the returned format is changed, update the class
 com.wolfram.eclipse.MEET.debug.DebugInformationTable. This 
 is not used anymore, but there are comments that refer to it, 
 so I'll leave for the present.
*)

GetTagInformationList[] := 
Map[ Join[ 
	RuntimeTools`GetTagInformation[#], 
	{TagNumber -> #, Expression -> ""}]&, RuntimeTools`GetTags[]];


(*
 This can be rewritten to avoid the GetTagInformation, 
 when we get a new kernel.
 
 I don'nt think this is used now.
*)

GetTaggedFiles[] :=
	Module[{tags},
		RuntimeTools`DebugOn[];
		tags = RuntimeTools`GetTags[];
		tags = Map[RuntimeTools`GetTagInformation, tags];
		tags = Part[tags, All,1,2];
		MakeJavaObject[ Union[ tags]]
		]


(*
 Get the tag from the file info. tagInfo is a list of elements
 {tag, file, lineno, charcount}.
*)

GetTagFromFile[ {filename_, lineno_}, tagInfo_] :=
	Cases[ tagInfo, {tag_,filename,_,lineno}->tag];

breakpointTagList = {};



(*
  Change the state of the tags at filename/lineno to addQ. 
  Implementation of UpdateBreakpoint for a new kernel. 
  This only adds one instance of a breakpoint per line, 
  trying to avoid adding it to the entire CompoundExpression.
  
  This should be called with the filename that was used by 
  the Get instruction.
  
  TODO move some of this into the Kernel.
*)

UpdateBreakpoint[ filename_, lineno_, addQ_:True] :=
AbortProtect[
	Block[ {$KernelCall=False},
	Module[ {newbps, found, len, tag},
		newbps = RuntimeTools`ChangeBreakpoint[ {filename, lineno}, addQ];
		If[ addQ === False || Length[newbps] < 2, Return[ Null]];
(*
 Now make sure that the breakpoint is only set on one expr
 
 TODO,  could improve this by returning more info from ChangeBreakpoint, 
 eg the Expr at the line and then using this rather than having to do 
 the TagToExpr look up.  Probably not very expensive.
*)
		newbps = Reverse[ Sort[ newbps]];		
		(*
		  These are all the tags that were found at the file/line, they 
		  all have a BP set, but we want to set the BP on just one expr, 
		  namely that just after the end of the compound expression.
		*)
		found = False;
		len = Length[ newbps];
		Do[  
			tag = newbps[[i]];
			If[ found || 
				(MatchQ[ RuntimeTools`TagToExpr[tag], HoldForm[_CompoundExpression]] && i =!= len)
				, 
				RuntimeTools`ChangeBreakpoint[ tag, False]
				,
				found = True], {i,len}];
	]]]




(* Creates the Stackframe Java object from one M_ Stack frame
  @param x a M_ Stackframe containing {expression, tag number, number (indicating nesting), {local vars}}
  local vars is a list of Hold[var] -> Hold[ value].   If val is a symbol then we evaluate 
  it,  this gives us the current value of symbols, but not of arguments.
 *)
 

$VariableSizeLimit = 10000;
$VariableShallowLimit = {20,50};
 
formatLocalVariables[ Hold[ var_] -> Hold[ val_Symbol]] :=
 	Block[ {Internal`$ContextMarks = False},
 		{ToString[ HoldForm[var]], formatSizeLimit[Hold[ Evaluate[val]], False, $VariableSizeLimit, $VariableShallowLimit]}
 		]
 		

formatLocalVariables[ Hold[ var_] -> Hold[ val_]] :=
 	Block[ {Internal`$ContextMarks = False},
 		{ToString[ HoldForm[var]], formatSizeLimit[ Hold[ val], False, $VariableSizeLimit, $VariableShallowLimit]}
 		]

formatLocalVariables[ Hold[ var_] -> Hold[ args___]] :=
 	Block[ {Internal`$ContextMarks = False},
 		{ToString[ HoldForm[var]], "Sequence[" <> formatSizeLimit[ Hold[ args], False, $VariableSizeLimit, $VariableShallowLimit] <> "]"}
 		]

formatLocalVariables[ HoldComplete[ var_] -> HoldComplete[ val_Symbol]] :=
 	Block[ {Internal`$ContextMarks = False, values},
 		values = OwnValues[ val];
 		If[ MatchQ[ values, {(_ :> value_)}]
 			,
 			values = Replace[ values,  {(_ :> value_)} -> HoldComplete[value]];
   			{stripHold[ToString[ HoldComplete[var]]], formatSizeLimit[ values, False, $VariableSizeLimit, $VariableShallowLimit]}
			,
  			{stripHold[ToString[ HoldComplete[var]]], formatSizeLimit[ Function[x, HoldComplete[x],{SequenceHold}][val],False, $VariableSizeLimit, $VariableShallowLimit]}
			 ]
 		]

formatLocalVariables[ HoldComplete[ var_] -> HoldComplete[ val_]] :=
 	Block[ {Internal`$ContextMarks = False},
 		{stripHold[ToString[ HoldComplete[var]]], formatSizeLimit[ HoldComplete[ val], False, $VariableSizeLimit, $VariableShallowLimit]}
 		]


stripHold[ x_] :=
	Which[ 
		StringQ[x] && StringMatchQ[x, "HoldComplete*"],
				StringTake[ x, {14, -2}],
		StringQ[x] && StringMatchQ[x, "Hold*"],
				StringTake[ x, {6, -2}],
		True,
			x]



createStackFrame[x_List]:=
   Module[{vars, tagInfo, fName, tag, lineNo, charNo, arg1, arg2},
   
      tag = x[[2]];
   	  If[ tag > 0,
      	tagInfo = RuntimeTools`GetTagInformation[ tag];
      	fName = "FileName" /. tagInfo;
      	fName = CanonicalizeFileName[ fName];
      	lineNo = "LineCount" /. tagInfo;
      	charNo = "CharacterCount" /. tagInfo];
      If[ !StringQ[ fName], fName = ""];
      If[ !IntegerQ[ lineNo], lineNo = 0];
      If[ !IntegerQ[ charNo], charNo = 0];
   	  vars = formatLocalVariables /@ x[[4]];
   	  arg1 = formatSizeLimit[x[[1]], True, 1000, {7,6}];
   	  arg2 = ToString[ tag];
   	  UseJVM[$WorkbenchJavaVM,
   	  	vars = If[ vars === {}, MakeJavaObject[Null], MakeJavaObject[ vars]];
      	JavaNew["com.wolfram.eclipse.MEET.debug.MathematicaStackFrame", arg1, arg2,
      			vars, fName, lineNo, charNo]  
   ]];



(*
 Utility function that formats an expression for the stack display.
 Quite similar to the function the FE uses for formatting on the stack.
 
 If the size is above sizeLimit then Shallow is used to format. 
 Otherwize
	If limitQ is True then it picks the smaller of with and without shallow.
 	If limitQ is False then it does not format with shallow.
*)

formatSizeLimit[expr_, limitQ_, sizeLimit_, shallowLimit_] :=
    Block[ {Internal`$ContextMarks = False},
	    Quiet[
	        Module[ {str1, str2, res},
	            res = If[ RuntimeTools`TreeSize[expr, sizeLimit] === False,
	                      ToString[Shallow[InputForm[expr], shallowLimit]],
	                      str1 = ToString[InputForm[expr]];
	                      If[ limitQ,
	                          str2 = ToString[Shallow[InputForm[expr], shallowLimit]];
	                          If[ StringLength[str2] < StringLength[str1],
	                              str2,
	                              str1
	                          ],
	                          str1
	                      ]
	                  ];
	            Which[
	                StringMatchQ[ res, "HoldComplete*"],
	                    StringDrop[StringDrop[res, 13], -1],
	                StringMatchQ[res, "HoldForm*"],
	                    StringDrop[StringDrop[res, 9], -1],
	                StringMatchQ[res, "Hold*"],
	                    StringDrop[StringDrop[res, 5], -1],
	                True, res
	            ]
	        ]
	    ]
    ]
    
(*
 Fix the stack so that variables that are in scope carry down 
 to lower stack frames.  Scope is detected by there being variables 
 or there being an index that is not -1.
 
 The first element of stack is the first thing to be evaluated, 
 as the evaluation proceeds we go down the stack.
*)

fixStack[ stackIn_] :=
	Module[ {nextvars = {}, newvars = {}, ei, tagi, indexi, varsi, stack = stackIn},
		stack =
		Table[
			newvars = nextvars;
			{ei, tagi, indexi, varsi} = Part[stack, i];
			newvars = Join[newvars,varsi];
			(* 
				If this stack frame has local vars or if 
				indexi is not -1, we push the vars down 
				to the next frame.
			*)
			nextvars = 
				If[ varsi === {} && indexi === -1, 
						{}, newvars];
			{ei, tagi, indexi, newvars}
			, {i,Length[stack]}];
			createStackFrame /@ stack
		]
	

(* Breakpoint handler that is registered with M_. 
   This builds the Java stack trace (ie. a List of Stackframe objects) and calls the
   handleBreakpoint method on the BreakpointHandler (the value of this is set in the
   MathematicaBreakpointHandler class before launching the M_ code).
    @param the ones defined for Breakpoint handlers
    
    CAUTION: if you change the name of this function, update the MathematicaBreakpointHandler 
    
    We also need to determine if this is a BP that should use Dialog or not. 
    This is done with $DialogForBreaks and seeing if this is a preemptive call.
  *)
InvokeBreakpointHandler[Hold[e_, num_]/; $KernelCall, level_:0 ] :=
	AbortProtect[
	 JavaBlock[
		Block[ {$KernelCall = False},
			Module[{stack, useDialogQ},
					stack = RuntimeTools`EvaluationStack[];
					If[ $ScrapbookFile && Length[ stack] > 7,
						stack = Drop[ stack, 7]];
					useDialogQ = $DialogForBreaks && !TrueQ[MathLink`IsPreemptive[]];
					BreakpointHandler@handleBreakpoint[ ToString[$SessionID], fixStack[stack], useDialogQ] ;
					If[ useDialogQ, Dialog[]];
			]
    	]
      ]
	 ];



(* 
    Messagefilter handler; this is registered with the MessageFilter system and 
    is called whenever a Message is generated. This uses the object in BreakpointHandler
    to call handleMessageBreakpoint (and thus trigger a MessageBreakpoint in Eclipse).
    
    @param the ones defined for Message filters 

    CAUTION: if you change the name of this function, update the MathematicaBreakpointHandler 

    Check that $Messages is a list of length more than 1.  If not it 
    means that messages have been disabled. Same for MessageOutputHandler.
    
    At present, all messages breakpoint go to the IDE, and if any message BP 
    is set it will be handled.  Since there is no UI for entering different 
    messages you can only break on all or no messages.  This could be improved, 
    but for now is probably fine.
*)


MessagesAreActive[] := ListQ[$Messages] && MemberQ[$Messages, OutputStream["stdout", _]]


MessageBreakpointsActiveQ = False;

MessageBreakpointHandler[x__ /; $KernelCall && MessageBreakpointsActiveQ]:= 
	AbortProtect[
	  JavaBlock[
		Block[ {$KernelCall = False},
			Module[{args, stack, useDialogQ},
				If[ MessagesAreActive[] && ValueQ[ BreakpointHandler],
					args={x};
					paramList = ReleaseHold[ ReplaceAll[ args[[3]] , Message -> MyMessage]]; 
					stack=RuntimeTools`EvaluationStack[];
					If[ Length[ stack] > 12,
						stack = Drop[ stack, -12]];		
					useDialogQ = $DialogForBreaks && !TrueQ[MathLink`IsPreemptive[]];
					Quiet[BreakpointHandler@handleMessageBreakpoint[ ToString[ $SessionID], fixStack[stack] , (ToString[ paramList[[#]] ])&/@Range[Length[ paramList ]], useDialogQ ]];
					If[ useDialogQ, Dialog[]]; 
					];
			]
	     ]
	   ]
	];



SetMessageBreakpoint[ state_] :=
	MessageBreakpointsActiveQ = state;

(*
 Assertion Handler
*)



AssertionHandler[e_ /; MEET`Private`$KernelCall] :=
    AbortProtect[
      JavaBlock[
    	Block[ {MEET`Private`$KernelCall = False},
			Module[ {stack, useDialogQ, pos},
				stack = RuntimeTools`EvaluationStack[];
                pos = Position[stack, e, 1, 1];
				pos = If[ MatchQ[pos, {{_Integer}}],pos[[1, 1]],Length[stack] - 9];
				If[ pos > 0, stack = Take[stack, pos]];
                stack = fixStack[stack];
                checkFrameForTag[Last[stack], e];
                useDialogQ = $DialogForBreaks && !TrueQ[MathLink`IsPreemptive[]];
                BreakpointHandler@
                          handleBreakpoint[ToString[$SessionID], stack, useDialogQ];
                If[ useDialogQ, Dialog[]];
         	];
		]
      ]
	];


checkFrameForTag[ obj_, HoldComplete[Assert[_, {line_Integer, file_String}]]] :=
	Module[{fixFile},
  		fixFile = CanonicalizeFileName[file];
  		obj@checkTagInformation[line, fixFile]
  	]

SetAssertBreakpoints[ name_, on_] /; $VersionNumber >= 8 :=
	Module[ {},
		If[ name =!= "Assertions" && name =!= "SystemAssertions",
				Return[ $Failed]];
		If[ on, 
			Internal`AddHandler[name, AssertionHandler];
			Internal`AddHandler[name, AssertionHandler];
			,
			Internal`RemoveHandler[name, AssertionHandler];
			Internal`RemoveHandler[name, AssertionHandler];
		]
	]




(* Convenience function to set the MathematicaBreakpointHandler object. This is called
from MathematicaBreakpointHandler before launching the code. 
   @param x the MathematicaBreakpointHandler object reference
   
       CAUTION: if you change the name of this function, update the MathematicaBreakpointHandler 
  *)

SetCallbackHandler[x_]:= 
	(BreakpointHandler = x; MEET`Utility`WorkbenchHandler = x);


(* ******  
Handlers and code for Unstructured output (Message and Print). 
*)

(*
 Used from Java to set the reference to an MathematicaConsoleLaunch$UnstructuredOutputHandler object.
   @param x MathematicaConsoleLaunch$UnstructuredOutputHandler object
   
       CAUTION: if you change the name of this function, update the MathematicaConsoleLaunch
       
   also this Function is used in ParallelDebug.m to clear the OutputHandler
*)
SetOutputHandler[x_]:= OutputHandler = x;

SetWorkbenchCallHandler[ x_] := (MEET`WorkbenchCallHandler = x)



(* 
    Messagefilter handler; this is registered with the MessageFilter system and 
    is called whenever a Message is generated. This uses the object in OutputHandler
    to send Messages to Java (format string not interpreted).
    
    @param the ones defined for Message filters 

    CAUTION: if you change the name of this function, update the MathematicaConsoleLaunch 
*)


MessageOutputHandler[mess_, _, args_ /; $KernelCall]:=
	AbortProtect[
		Block[ {$KernelCall = False},
			Module[{body, tag, prePrint, reject},
				If[ MessagesAreActive[],
    				body = Delete[args, {1, 0}];
    				tag = Take[body, 1];
    				tag[[0]] = HoldForm;
    				prePrint = If[ $MessagePrePrint === Automatic || !ValueQ[$MessagePrePrint], Identity, $MessagePrePrint];
    				body = Map[prePrint, body];
    				body = StringForm @@ body;
    				body[[1]] = mess;
    				body = SequenceForm @@ {tag, ": ", body};
    				reject = rejectMessageQ[ args];
    				If[ !reject,
	    				OutputHandler@handleMessage[ ToString[ $SessionID], ToString[body]]]];
    	]]]

(*
 Fixup necessary to prevent a message on launch, because J/Link creates a 
 loopback link and does some writing to it.  The link gets closed but the 
 link sharing system still tries to write to the link.
*)
rejectMessageQ[ args_] :=
	MatchQ[ args, Hold[Message[ LinkObject::linkn, HoldForm[LinkObject["",__]], __]]]


(*
 Overload Print to work. 
 Later could hookup some type of Event.
*)

Unprotect[Print];
$PrintFlag = True;
MEETDebug`MyPrint[x___] := 
	OutputHandler@handlePrint[ ToString[ $SessionID], ToString[ SequenceForm[x] ] ];

Print[ x___ /; $PrintFlag && $KernelCall] :=
	AbortProtect[
		Block[{$KernelCall = False, $PrintFlag = False}, 
			MEETDebug`MyPrint[x];
			Print[x]
		]];

Protect[Print];


(*
 Provided for calls to the Kernel from Java that might 
 generate messages or be aborted.
*)

SetAttributes[ CallKernelAtomic, HoldAll];
CallKernelAtomic[ x_] :=
	AbortProtect[
		Block[ {$KernelCall = False},
			x
		]];

SetAttributes[ ToInputFormString, SequenceHold];
ToInputFormString[ x_] :=
	ToString[Unevaluated[x]]


(*
 Rules for setting up VariableRules.
*)

SetAttributes[System`VariableValue, HoldAllComplete]
System`VariableValue[x_] := System`VariableValue[HoldComplete[x]]
System`VariableValue[HoldComplete[x_]] := 
	Replace[FindVariable[HoldComplete[x], VariableRules[]], HoldComplete[arg_] -> arg]

FindVariable[ HoldComplete[sym_Symbol], 
    {HoldComplete[stacksym_Symbol] -> HoldComplete[value_], rest___}] :=
  If[SymbolName[Unevaluated[sym]] === SymbolName[Unevaluated[stacksym]], HoldComplete[value], 
      FindVariable[HoldComplete[sym], {rest}]]
 
FindVariable[ HoldComplete[ sym_], _] := HoldComplete[ sym]  

VariableRules[] := 
	Map[ fixHoldInStack,
		Reverse[Flatten[Map[Last, Debugger`EvaluationStack[]]]]]

fixHoldInStack[ Hold[a_] -> Hold[b_]] := HoldComplete[a] -> HoldComplete[b]

fixHoldInStack[ x_] := x


DisableFunction[ fun_] :=
	(Unprotect[ fun]; fun = False)

(*
 Remove any handlers set for BreakPoint or Message, these get 
 set by the FrontEnd debugger which we want to make sure is 
 disabled.  It also disables Debugger`SetBreakpoints because 
 the FrontEnd sends this with empty lists when it launches 
 and this can mess up BPs set by the IDE.  All this code 
 should be made tidier,  but it needs to start with the 
 FrontEnd implementation.   We probably also need to 
 disable the FrontEnd turning on and off the Debugger 
 flag, but this really needs to work in the FrontEnd.
*)

SetupIDE[ debugQ_] :=
	SetupIDE[ debugQ, False, 10000, 7, 6]

SetupIDE[ debugQ_, enableBPsDuringGet_, varLimit_, shallowLimit1_, shallowLimit2_] :=
	AbortProtect[
	Module[ {},
		Workbench`$Debug = debugQ;
		SetupIDECalled = debugQ;
		$VariableSizeLimit = varLimit;
		$VariableShallowLimit = {shallowLimit1, shallowLimit2};
		If[ debugQ, RuntimeTools`DebugOn[ True]];
		RuntimeTools`ToolsUninitialize[];
		Unprotect[ Debugger`MessageBreak];
		Unprotect[ RuntimeTools`Dump`MessageBreak];
		Unprotect[ Debugger`BreakpointFunction];
		Clear[ Debugger`MessageBreak];
		Clear[ RuntimeTools`Dump`MessageBreak];
		Clear[ Debugger`BreakpointFunction];
		Map[ Internal`RemoveHandler["BreakPoint", #] &, 
 			"BreakPoint" /. Internal`Handlers["BreakPoint"]];
 		Internal`AddHandler["MessageTextFilter", MessageOutputHandler];
		If[ debugQ,
			Internal`AddHandler["MessageTextFilter", MessageBreakpointHandler];
			Internal`AddHandler["BreakPoint", InvokeBreakpointHandler];	
			VersionSetup[ ] ;
			If[ enableBPsDuringGet && 
					RuntimeTools`SetBreakpointsToAdd[] === Null && 
					$VersionNumber >= 8,
				Internal`AddHandler["GetFileEvent", FileLoadProcess]
				,
				OverloadGet[]];
			If[ $VersionNumber >= 6, DynamicDebugging[True]];
			];
		DisableFunction[ Debugger`SetBreakpoints];
		DisableFunction[ Debugger`DebugOn];		
		DisableFunction[ RuntimeTools`DebugOff];
		DisableFunction[ Debugger`DebugOff];		
	]];

UninstallIDE[] :=
	Module[ {},
		If[ Workbench`$Debug, Null
				];	
		Map[ Internal`RemoveHandler["BreakPoint", #] &, 
 				"BreakPoint" /. Internal`Handlers["BreakPoint"]];
		Map[ Internal`RemoveHandler["MessageTextFilter", #] &, 
 				"MessageTextFilter" /. Internal`Handlers["MessageTextFilter"]];
		VersionUninstall[];
		JLink`UninstallJava[MEET`$WorkbenchJavaVM];
	]


VersionUninstall[ ] /; $VersionNumber >= 6 :=
	Module[ {},
		Map[ Internal`RemoveHandler["ProcessDebugTags", #] &, 
 				"ProcessDebugTags" /. Internal`Handlers["ProcessDebugTags"]];
	]
	
VersionSetup[ ] /; $VersionNumber >= 6 :=
	Module[ {},
		Internal`AddHandler["ProcessDebugTags", ProcessDebugTagsHandler];
	]


	
(*
 Called when we are launching from a notebook, the name of 
 the notebook is passed as an argument.
*)

AttachFrontEnd[  filename_] :=
	Module[ { },
		$ScrapbookFile = False;
	]



(*
  TODO: remove this definition
*)
canonicalizeFileName[file_] := 
	CanonicalizeFileName[file]


(*
 Fixup for Get so that it works for contexts and Windows 
 filenames.  Also, when the file has been loaded, we need 
 to ask the IDE to resend the list of breakpoints.
  
 TODO move some of this into the Kernel.
*)


CanonicalizeFileName[file_] :=
    Module[{newFile = file, test},
      	If[StringMatchQ[newFile, "*`"],
      		test = System`Private`FindFile[ newFile];
      		newFile =
      			Which[
      				test === $Failed, ContextToFileName[newFile],
      				StringMatchQ[newFile, __ ~~ "Kernel" ~~ _ ~~ "init.m"],
      					StringJoin[ StringDrop[ newFile, -1], "/Kernel/init.m"],
      				StringMatchQ[newFile, __ ~~ "init.m"],
      					StringJoin[ StringDrop[ newFile, -1], "/init.m"],
      				True, ContextToFileName[newFile]]];
		If[$PathnameSeparator === "\\" && 
      		StringMatchQ[newFile, ___ ~~ "\\" ~~ ___],
        	newFile = StringReplace[newFile, "\\" -> "/"]];
      	newFile
      ];
               
fileLoading = ""

OverloadGet[] :=
	(
	Unprotect[Get];
	Get[file_ /; (file =!= fileLoading && 
					$KernelCall && 
					MemberQ[$ContextPath, "System`"]), opts___] :=
    	Block[{newFile, ef, fileLoading},
      		fileLoading = file;
      		ef = Get[file, opts];
      		If[ StringQ[ file],
	      		BreakpointHandler@resetFileBreakpoints[ ToString[$SessionID], file, CanonicalizeFileName[file]]];
      		ef
      		];
 	Protect[Get];
 	)
 
 
 
 
 FileLoadProcess[ HoldComplete[ file_, wrapper_, First]] :=
 	Module[ {bps},
 		bps = BreakpointHandler@getFileBreakpoints[ ToString[$SessionID], file, CanonicalizeFileName[file]];
 		RuntimeTools`SetBreakpointsToAdd[ file, bps]
 	]
 
 FileLoadProcess[ HoldComplete[ file_, wrapper_, Last]] :=
 	Module[ {},
 		RuntimeTools`SetBreakpointsToAdd[ file]
 	]

 
(*
 Utility functions for getting information on files with 
 debug tags.
*)

 GetFileNamesWithTags[] :=
 	Module[{tags, files},
 		tags = RuntimeTools`GetTags[];
		files = Map["FileName" /. RuntimeTools`GetTagInformation[#] &, tags];
		Union[files]
 	]
 	
 GetFileInformation[file_] :=
  Module[{tags, info},
    tags = RuntimeTools`GetTags[];
    info = Map[{#, RuntimeTools`GetTagInformation[#]} &, tags];
    info = Select[info, (("FileName" /. #[[2]]) === file) &];
    Map[{RuntimeTools`TagToExpr[#[[1]]], #[[2]]} &, info]
    ]
 
 
DynamicDebugging[ val: True|False] :=
	Module[ {},
		RuntimeTools`SetExecutionState[ "DebugDynamics" -> val];
		If[ val && $Notebooks, 
 			MathLink`CallFrontEnd[
 				FrontEnd`SetOptions[FrontEnd`$FrontEndSession, 
  							DynamicEvaluationTimeout -> 999999]]];
  	]


FrontEndTagNotify[ tag_Integer, clear:True|False] :=
	Module[ {tmp},
		tmp = System`NotebookInterfaceObject[ tag];
		tmp = {{tmp, If[ clear, "Input", Background -> RGBColor[0.6, 0.6, 1.0]]}};
		tmp = System`DebuggerSetExpressionColoring[ tmp];
		MathLink`CallFrontEnd[ tmp]
	]

ResetCodeBreakpoints[] :=
	MEET`Utility`WorkbenchHandler@resetCodeBreakpoints[]


(*
 ProcessDebugTags handler
*)
ProcessDebugTagsHandler[ file_String] :=
	BreakpointHandler@resetFileBreakpoints[ ToString[$SessionID], file, CanonicalizeFileName[file]];
	

(*
 Fix for GUIKit.  Can't be done with a GUIKit API, since GUIKit 
 might not be loaded.  Later can do with ResourceLocator.
*)
AddGUIPath[path_] :=
	If[ ListQ[ GUIKit`Utility`$ExtraGUIPaths],
			AppendTo[ GUIKit`Utility`$ExtraGUIPaths, path],
			GUIKit`Utility`$ExtraGUIPaths = {path}]



BreakWithArgument[ e_] := RuntimeTools`BreakNow[True]

SetDynamicTriggerBreakpoint[ val_] :=
	If[ MatchQ[ Internal`Handlers[], {___, "ValueTrackTrigger" -> _,___}],
			If[ val, Internal`AddHandler, Internal`RemoveHandler]["ValueTrackTrigger", BreakWithArgument];
	]


End[]

EndPackage[]


 