
(*
 Package to provide Mathematica code for the Wolfram Workbench.
*)


BeginPackage[ "MEET`Profiler`"]



Begin[ "`Private`"]

(*
 Introduce the symbol
*)
System`Profile

(*
 Clear any old definitions.
*)
Unprotect[ Profile];
Clear[ Profile];


Profile[ expr_, args___ /; TestState[] ] :=
	Block[{ computeExpr = HoldForm[expr]},
		RuntimeTools`Profile[ expr, args]
	]


(*
 Tests to see if we are in debug mode, if not 
 give an error and return False.
*)
TestState[] :=
	If[ ("RuntimeAnalysisTools" /. 
				RuntimeTools`ExecutionState["RuntimeAnalysisTools"]) === True, 
			True,
			Message[ Profile::state];
			False]

Profile::state = "The profiler can only operate if the Wolfram Engine is running in a debug mode.";

SetAttributes[ Profile, {HoldAllComplete, Protected}]


(*
 Disable any initial profile functionality.
*)
 
Unprotect[RuntimeTools`ProfileFunction]
Clear[ RuntimeTools`ProfileFunction]

(*
 Formatting function for exprs in the result 
 notebook.
*)
addArgument[ e_] := 
	Block[ {Internal`$ContextMarks = False},
		MakeBoxes[Shallow[e,{7,6}], StandardForm]
		]

(*
 Sorter function for the elements.
*)
	
sortFun[ l_,r_]:=
  l[[4]]-r[[4]]>10.^-6||l[[5]]<r[[5]]


(*
 Function for extracting the cutoff below 
 which nothing will be displayed.
*)

defaultLimit=10.^-5;

getLimit[Automatic]=defaultLimit ;

getLimit[num_]:=
 Module[{num1},
  num1=N[num];
  If[ Head[num1]===Real && num1 >= 0,num1,defaultLimit]
  ]



(*
 Find the tag, this could be in a notebook 
 or it could be in a .m file.
*)

locateTag[ tag_Integer] :=
	Module[ {info, filename, line},
		info = RuntimeTools`GetTagInformation[ tag];
		filename = "FileName" /. info;
		If[ StringQ[ filename], filename = MEET`CanonicalizeFileName[filename]];
		line = "LineCount" /. info;
		If[ !IntegerQ[ line], line = -1];
		MEET`Utility`WorkbenchHandler@locateSource[ tag, filename, line];
	]

	
(*
Build a row of the result.
*)

buildResultRow[ {expr_, tag_, calls_, time_, _}] :=
	Module[ {},
	{ToBoxes[calls],ToBoxes[time],
       ButtonBox[addArgument[expr],Active->True,
        ButtonFunction->(locateTag[ tag]&),
        ButtonEvaluator->Automatic]}
	]
	

RuntimeTools`ProfileFunction[eIn_, limitArg_]:=
	Module[ {nb, e,l, lim, eval},
		lim = getLimit[limitArg];
		e = Sort[eIn, #1[[4]] > #2[[4]]&];
		e = Sort[e, sortFun];
		e = Select[e,#[[4]] >= lim &];
		nb = NotebookCreate[];
		eval = Evaluator /. Options[ EvaluationNotebook[]];
		If[ StringQ[ eval], SetOptions[ nb, Evaluator -> eval]];		
		NotebookWrite[ nb, Cell["Profile Output", "Title"]];
		NotebookWrite[ nb, Cell["Generated by the Wolfram Workbench", "Subsubtitle"]];
		NotebookWrite[ nb, Cell[ calculateDate[], "Subsubtitle"]];
		NotebookWrite[ nb, Cell[ calculateInput[], "Input"]];
		(*MathLink`CallFrontEnd[FrontEnd`SelectionMove[nb,All,Notebook]];*)
		l = Apply[List, Map[ buildResultRow, e]];
		l = Prepend[l,{"Calls","Time","Evaluation"}];
		MathLink`CallFrontEnd[
			FrontEnd`NotebookWrite[nb,
				Notebook[{Cell[BoxData[GridBox[l,ColumnLines->True,
				ColumnAlignments->{Right,Center,Left},
				RowLines->{True,False}]],"Text",
				ButtonBoxOptions->{ButtonExpandable->False }]}]]
		]
	];
    

months = {"January", "Febuary", "March", "April","May", "June",
"July", "August", "September", "October", "November", "December"};

calculateDate[] :=
	Module[ {y,m,d,h,min,sec},
		{y,m,d,h,min,sec} = Date[];
		{h,min,d,y} = Map[ToString,{h,min,d,y}];
		h <> ":" <> min <> "   " <> months[[m]] <> " " <> d <> " " <> y
	]

calculateInput[] :=
	Module[ {lab,box},
		box = ToBoxes[ computeExpr];
		lab = StyleBox[
            		RowBox[{"Input", ":"}],
            		"Text",
            		FontWeight->"Plain"] ;
       	BoxData[GridBox[{{lab, box}}]]
	]
	
	
End[]


EndPackage[]


