
BeginPackage["MeetLogger`", {"MUnit`", "JLink`"}]

MeetLogger::usage =
"MeetLogger[] is an MUnit logger for Wolfram Workbench."

RunTestSuite::usage =
	"RunTestSuite[ testSuite, {files...}] runs a test suite with MUnit."

Begin["`Private`"]

UseJVM[MEET`$WorkbenchJavaVM,
	LoadJavaClass["com.wolfram.eclipse.testing.results.munit.ResultCollector"];
	LoadJavaClass["com.wolfram.eclipse.testing.results.munit.MUnitTestRun"];
	LoadJavaClass["com.wolfram.eclipse.testing.results.TestResult"];
	LoadJavaClass["com.wolfram.eclipse.testing.results.TestSuccess"];
	LoadJavaClass["com.wolfram.eclipse.testing.results.TestFailure"];
	LoadJavaClass["com.wolfram.eclipse.testing.results.MessageFailure"];
	LoadJavaClass["com.wolfram.eclipse.testing.results.TestError"]];

BuildTestSuccess[tr_?TestResultQ] :=
	Module[{},
		UseJVM[MEET`$WorkbenchJavaVM,
			JavaNew["com.wolfram.eclipse.testing.results.TestSuccess", Null, ToString[TestID[tr]]]
		]
	]

BuildTestFailure[tr_?TestResultQ] :=
	Module[{expected, actual},
		UseJVM[MEET`$WorkbenchJavaVM,
			expected = makeString[ ExpectedOutput[tr]];
			actual = makeString[ ActualOutput[tr]];
			JavaNew["com.wolfram.eclipse.testing.results.TestFailure", Null, ToString[TestID[tr]], expected, actual, ToString[TestFailureMessage[tr]]  ]
		]
	]

BuildTestMessageFailure[tr_?TestResultQ] :=
	Module[{expectedMessages,actualMessages},
		UseJVM[MEET`$WorkbenchJavaVM,
			expectedMessages = ToString[ ExpectedMessages[tr]];
			actualMessages = ToString[ ActualMessages[tr]];
			JavaNew["com.wolfram.eclipse.testing.results.MessageFailure", Null, ToString[TestID[tr]], expectedMessages, actualMessages, ToString[TestFailureMessage[tr]] ]
		]
	]

BuildErrorTestFailure[tr_] :=
	Module[{expected, actual},
		UseJVM[MEET`$WorkbenchJavaVM,
			expected = makeString[ ExpectedOutput[tr]];
			actual = makeString[ ActualOutput[tr]];
			JavaNew["com.wolfram.eclipse.testing.results.TestError", Null, ToString[TestID[tr]], expected, actual, ToString[ErrorMessage[tr]] ]
		]
	]



MeetLogger[] :=
	With[{meetLogger = Unique["MeetLogger`Private`meetLogger"], 
			resultCollector = Unique["MeetLogger`Private`resultCollector"], 
			currentFile = Unique["MeetLogger`Private`currentFile"],
			checkFile = Unique["MeetLogger`Private`checkFile"]},
		
		(*  Set up resultCollector variable which will receive all logging notifications *)
		meetLogger /: LogStart[meetLogger, name_:""] :=
			Module[{prov},
				UseJVM[
					MEET`$WorkbenchJavaVM,
					prov = JavaNew["com.wolfram.eclipse.testing.results.munit.MUnitTestRun", 
								ToString[name], 
								MeetLogger`Private`TestRunProject, MeetLogger`Private`TestRunFileName];
					resultCollector = ResultCollector`addTestRun[prov];
					resultCollector@testRunStarted[]];
					currentFile = Null;
			];
		
		checkFile[tr_] := 
			Module[ {testFile},
				(*
				TODO
				TestSource is new in MUnit 1.3
				for compatibility with older versions of MUnit, use StringQ
				When it is safe to assume MUnit 1.3, remove the StringQ test
				*)
				testFile = MUnit`TestSource[ tr];
				If[StringQ[testFile] && testFile =!= currentFile, 
						currentFile = testFile;
						resultCollector@testFileSet[currentFile]]
			] ;
		
		(* TODO
		when it is safe, switch to using LogTestCount
		LogExpressionCount is actually the count of ALL expressions, not just test expressions
		LogTestCount is new in MUnit 1.3, so older versions will still report the wrong value
		*)
		meetLogger /: LogExpressionCount[meetLogger, n_Integer] :=
			resultCollector@numberOfTestExpressions[ToString[TestRunId], n];
		
		meetLogger /: LogSuccess[meetLogger, tr_?TestResultQ] :=
			(checkFile[tr];
			resultCollector@testCaseSuccess[ToString[TestRunId], BuildTestSuccess[tr]]);
		
		meetLogger /: LogFailure[meetLogger, tr_?TestResultQ] :=
			(checkFile[tr];
			resultCollector@testCaseFailure[ToString[TestRunId], BuildTestFailure[tr]]);
		
		meetLogger /: LogMessagesFailure[meetLogger, tr_?TestResultQ] :=
			(checkFile[tr];
			resultCollector@testCaseMessageFailure[ToString[TestRunId], BuildTestMessageFailure[tr]]);
		
		meetLogger /: LogError[meetLogger, tr_?TestResultQ] :=
			(checkFile[tr];
			resultCollector@testCaseError[ToString[TestRunId], BuildErrorTestFailure[tr]]);
		
		meetLogger /: LogEnd[meetLogger, testCnt_, successCnt_, failCnt_, msgFailCnt_, secSkippedCnt_, errorCnt_, abort_] :=
			resultCollector@testRunFinished[ToString[TestRunId]];

		meetLogger /: LogFatal[meetLogger, msg_String] :=
			resultCollector@testRunFatal[msg];
				
		meetLogger
	]


	
	
MeetLoggerSuite[resultCollector_] :=
	With[{meetLogger = Unique["MeetLogger`Private`meetLogger"],
			currentFile = Unique["MeetLogger`Private`currentFile"],
			checkFile = Unique["MeetLogger`Private`checkFile"]},
		
		checkFile[tr_] := 
			Module[ {testFile},
				(*
				TODO
				TestSource is new in MUnit 1.3
				for compatibility with older versions of MUnit, use StringQ
				When it is safe to assume MUnit 1.3, remove the StringQ test
				*)
				testFile = MUnit`TestSource[ tr];
				If[StringQ[testFile] && testFile =!= currentFile, 
						currentFile = testFile;
						resultCollector@testFileSet[currentFile]]
			] ;

		
		meetLogger /: LogStart[meetLogger, name_:""] :=
			resultCollector@testRunStarted[];
		
		(* TODO
		when it is safe, switch to using LogTestCount
		LogExpressionCount is actually the count of ALL expressions, not just test expressions
		LogTestCount is new in MUnit 1.3, so older versions will still report the wrong value
		*)
		meetLogger /: LogExpressionCount[meetLogger, n_Integer] :=
			resultCollector@numberOfTestExpressions[ToString[TestRunId], n];
		
		meetLogger /: LogSuccess[meetLogger, tr_?TestResultQ] :=
			(checkFile[tr];
			resultCollector@testCaseSuccess[ToString[TestRunId], BuildTestSuccess[tr]]);
		
		meetLogger /: LogFailure[meetLogger, tr_?TestResultQ] :=
			(checkFile[tr];
			resultCollector@testCaseFailure[ToString[TestRunId], BuildTestFailure[tr]]);
		
		meetLogger /: LogMessagesFailure[meetLogger, tr_?TestResultQ] :=
			(checkFile[tr];
			resultCollector@testCaseMessageFailure[ToString[TestRunId], BuildTestMessageFailure[tr]]);
		
		meetLogger /: LogError[meetLogger, tr_?TestResultQ] :=
			(checkFile[tr];
			resultCollector@testCaseError[ToString[TestRunId], BuildErrorTestFailure[tr]]);
		
		meetLogger /: LogEnd[meetLogger, testCnt_, successCnt_, failCnt_, msgFailCnt_, secSkippedCnt_, errorCnt_, abort_] :=
			resultCollector@testRunFinished[ToString[TestRunId]];
		
		meetLogger /: LogFatal[meetLogger, msg_String] :=
			resultCollector@testRunFatal[msg];
		
		meetLogger
	]


(*
 Run the test suite on files.
*)
RunTestSuite[ suite_, files_List] :=
	Module[ {dir, resultCollector, testRunObj},
		dir = DirectoryName[ suite];
		UseJVM[
			MEET`$WorkbenchJavaVM,
			testRunObj = JavaNew["com.wolfram.eclipse.testing.results.munit.MUnitTestRun", 
							ToString[name], 
							MeetLogger`Private`TestRunProject, MeetLogger`Private`TestRunFileName,
							files];
			resultCollector = ResultCollector`addTestRun[testRunObj];
		];
		resultCollector@testSuiteStarted[];		
		With[ {resultCollector = resultCollector},
			Map[ (testRunObj@startTestSuiteRun[#];MUnit`TestRun[ToFileName[dir,#], MUnit`Loggers->{MeetLoggerSuite[resultCollector]}])&, files];
			resultCollector@testSuiteFinished[]];
	]
	

makeString[ expr_] :=
	Module[ {str},
		str = ToString[ expr, InputForm];
		If[ StringLength[ str] > 10 && StringTake[ str, 9] === "HoldForm[" && StringTake[str,-1] === "]",
	     	StringTake[ str, {10,-2}], 
	     	str]
	]

End[]

EndPackage[]
