Needs["Transmogrify`"]
Needs["MUnit`"]

basicBodyFile = Export[
	CreateFile[],
	"${body}",
	"Text"
];

Test[
	FillInLayout[basicBodyFile, "Text", "body" -> "Some content."]
	,
	"Some content."
]

(* Test that FillInLayout will throw an exception if any ${..} template slots cannot be
   filled. *)
Test[
	Catch[
		FillInLayout[basicBodyFile, "Text", "notbody" -> "Other."],
		_,
		"CaughtException"[#] &
	]
	,
	"CaughtException"[$Failed]
	,
	{FillInLayout::error}
]

(* Test that FillInLayout will throw an exception if any template argument values are
   not strings. *)
Test[
	Catch[
		FillInLayout[basicBodyFile, "Text", "body" -> {1, 2, 3}],
		_,
		"CaughtException"[#] &
	]
	,
	"CaughtException"[$Failed]
	,
	{FillInLayout::error}
]

(*************************************************************)
(* Tests for hard errors when obsolete behavior is expected. *)
(*************************************************************)

(* Test that obsolete FillInLayout behavior when no template arguments are provides
   throws a hard error. *)
Test[
	Catch[
		FillInLayout[basicBodyFile, "Text"],
		_,
		"CaughtException"[#] &
	]
	,
	"CaughtException"[$Failed]
	,
	{FillInLayout::error}
]

(* Test that a 3rd argument context string form throws a hard error -- this form is obsolete. *)
Test[
	Catch[
		FillInLayout[basicBodyFile, "Text", "SomeContext`", "body" -> "Some content."],
		_,
		"CaughtException"[#] &
	]
	,
	"CaughtException"[$Failed]
	,
	{FillInLayout::error}
]

(* Test that FillInLayout will throw an exception if obsolete 'singlevar' form of template
   arguments is used. *)
Test[
	Catch[
		FillInLayout[basicBodyFile, "Text", {"body"}],
		_,
		"CaughtException"[#] &
	]
	,
	"CaughtException"[$Failed]
	,
	{FillInLayout::error}
]

Test[
	Catch[
		FillInLayout[
			basicBodyFile, "Text", "body" -> "Some content.",
			EmbeddedCodeRule -> ("A custom" -> "rule")
		],
		_,
		"CaughtException"[#] &
	]
	,
	"CaughtException"[$Failed]
	,
	{FillInLayout::error}
]