
(*
 Package to debugging for gridMathematica
*)


BeginPackage[ "MEETParallel`"]

Needs[ "MEETParallel`ParallelProfiler`"]

DebuggerLink::usage = ""

Begin[ "`Private`"]


(*
 Make sure that slaves are launched in debug mode.
*)

If[ "RuntimeAnalysisTools" /. RuntimeTools`ExecutionState["RuntimeAnalysisTools"],
	Parallel`Protected`AppendInitCode[ Parallel`Client`HoldCompound[ RuntimeTools`DebugOn[True]], True]
]

uninitialized = True;

InitializeKernels[] :=
	If[ uninitialized === True,
		uninitialized = False;
		Block[ {Parallel`Debug`Private`ExportEnvironmentWrapper = Identity},
			DistributeDefinitions[ "MEET`*"] ;
			DistributeDefinitions[ "MEETParallel`*"] ;
			DistributeDefinitions[ "JLink`*"] ;
		];
	]


(*
 Should attach to just one slave, but not so useful, 
 since ParallelizeDefinitions can't write to just one slave.
*)


MEET`Utility`LinkCreateOnWB = False


AttachSlave[ slave_] :=
	Module[{remoteSession, name, linkArgs, slaveName},
		InitializeKernels[];
		remoteSession = ToString[ ParallelEvaluate[$SessionID, slave]];
		slaveName = "Slave-" <> ToString[ Parallel`Developer`KernelID[slave]];
		name = Part[ParallelEvaluate[ DebuggerLink = LinkCreate[LinkProtocol->"TCPIP"], slave], 1] ;
		linkArgs = " -linkmode connect -linkname " <> name ;
		ParallelEvaluate[
				MathLink`AddSharingLink[ DebuggerLink, 
					AllowPreemptive->True,Terminating->True],slave];
		MEET`Utility`WorkbenchHandler@attachKernel[ remoteSession, slaveName, linkArgs, "TCPIP", MEET`Utility`LinkCreateOnWB];
		ParallelEvaluate[MEET`SetOutputHandler[ Null], slave];
		slave
	]

Parallel`Debug`Private`RemoteKernelInit = AttachSlave

(*
 The apply of the head is because the definitions in 
 expr are wrapped in a holding function.  This needs 
 to be reapplied before ProcessTags is inserted.
*)

addTags[expr_] := Module[{ef, h},
    h = Head[expr];
    ef = RuntimeTools`AddTags[expr];
    ef = h @@ ef;
    ef = Map[RuntimeTools`ProcessTags, ef, {1}];
    ef
    ]


Parallel`Debug`Private`ExportEnvironmentWrapper = addTags


End[]


EndPackage[]

