
(*
 Package to support profiling for gridMathematica
*)



BeginPackage[ "MEETParallel`ParallelProfiler`"]

Needs["MEETParallel`"]

ParallelProfile::usage = "ParallelProfile[ expr] executes expr and computes a parallel profile report."

Begin[ "`Private`"]


FixTag[tt:{a1_,  tag_, res__}] :=
	Module[ {info, filename, line},
		info = RuntimeTools`GetTagInformation[ tag];
		filename = "FileName" /. info;
		line = "LineCount" /. info;
		If[ !IntegerQ[ line], line = -1];
		{a1, {tag, filename, line}, res}
	]

InitializeSlaves21[] :=
	(
	ParallelizeValues[ FixTag];
	MEET`ParallelDebug`Private`ParallelEvaluate[ 
		Unprotect[RuntimeTools`ProfileFunction];
		Clear[ RuntimeTools`ProfileFunction];
		RuntimeTools`ProfileFunction[ x_, _] := 
			(RuntimeTools`ParallelResult = {"Slave-" <> ToString[Parallel`Parallel`$ProcessorID],Map[ FixTag,  x]})
	])

InitializeSlaves[] :=
	(
	ParallelizeValues[ FixTag];
	ParallelEvaluate[ 
		Unprotect[RuntimeTools`ProfileFunction];
		Clear[ RuntimeTools`ProfileFunction];
		RuntimeTools`ProfileFunction[ x_, _] := 
			(RuntimeTools`ParallelResult = {"Slave-" <> ToString[$KernelID],Map[ FixTag,  x]})
	])

SetAttributes[ParallelProfile, HoldAll]

ParallelProfile[ x_] :=
	(
	AttachSlaves[];
	If[ Parallel`Information`$VersionNumber < 3,
		InitializeSlaves21[],
		InitializeSlaves[]];	
	Block[ {Parallel`Debug`Private`RemoteEvaluateWrapper = RuntimeTools`Profile, 
			RuntimeTools`ProfileFunction = ParallelProfileWorker,
			computeExpr = HoldForm[x]},
		RuntimeTools`Profile[ x]
	])
	
	
ParallelProfileWorker[ x_, limitArg_] :=
	Module[ {data, nb, lim, showMaster},
		lim = getLimit[limitArg];
		Block[ {Parallel`Debug`Private`RemoteEvaluateWrapper, result},
			result = x;
			data = ParallelEvaluate[ RuntimeTools`ParallelResult];
			If[ showMaster,
				data = Prepend[ data,  {"MathKernel-0", result}]];
			];
		data = data /. RuntimeTools`ParallelResult -> Sequence[];
		ParallelEvaluate[ Clear[RuntimeTools`ParallelResult]];
		mainData = data;
		nb = createResultNB[];
		Scan[ addResultToNB[ #[[1]], #[[2]], lim, nb]&, data];
	]
	
(*
 Report generating code,  this could all be merged with 
 the Profiler.m package.
*)

(*
 Formatting function for exprs in the result 
 notebook.
*)
addArgument[ e_] := 
	Block[ {Internal`$ContextMarks = False},
		MakeBoxes[Shallow[e,{7,6}], StandardForm]
		]

(*
 Sorter function for the elements.
*)
	
sortFun[ l_,r_]:=
  l[[4]]-r[[4]]>10.^-6||l[[5]]<r[[5]]


(*
 Function for extracting the cutoff below 
 which nothing will be displayed.
*)

defaultLimit=10.^-5;

getLimit[Automatic]=defaultLimit ;

getLimit[num_]:=
 Module[{num1},
  num1=N[num];
  If[ Head[num1]===Real && num1 >= 0,num1,defaultLimit]
  ]



(*
 Find the tag, this could be in a notebook 
 or it could be in a .m file.
*)

locateTag[ tag_Integer] :=
	Module[ {info, filename, line},
		info = RuntimeTools`GetTagInformation[ tag];
		filename = "FileName" /. info;
		line = "LineCount" /. info;
		If[ !IntegerQ[ line], line = -1];
		MEET`Utility`WorkbenchHandler@locateSource[ tag, filename, line];
	]

locateTag[ tag_Integer, file_, line_] :=
	MEET`Utility`WorkbenchHandler@locateSource[ tag, file, line];

	
locateTag
	
(*
Build a row of the result.
*)

buildResultRow[ {expr_, tag_Integer, calls_, time_, _}] :=
	Module[ {},
	{ToBoxes[calls],ToBoxes[time],
       ButtonBox[addArgument[expr],Active->True,
        ButtonFunction->(locateTag[ tag]&),
        ButtonEvaluator->Automatic]}
	]
	
buildResultRow[ {expr_,{ tag_Integer, fileIn_, line_}, calls_, time_, _}] :=
	With[ {file = MEET`CanonicalizeFileName[fileIn]},
	{ToBoxes[calls],ToBoxes[time],
       ButtonBox[addArgument[expr],Active->True,
        ButtonFunction->(locateTag[ tag, file, line]&),
        ButtonEvaluator->Automatic]}
	]
	

addResultToNB[ title_, eIn_,  lim_, nb_] :=
	Module[ {e},
		If[ StringQ[ title],
			NotebookWrite[ nb, Cell[ title, "Section"]]];
		e = Sort[eIn, #1[[4]] > #2[[4]]&];
		e = Sort[e, sortFun];
		e = Select[e,#[[4]] >= lim &];
		(*MathLink`CallFrontEnd[FrontEnd`SelectionMove[nb,All,Notebook]];*)
		l = Apply[List, Map[ buildResultRow, e]];
		l = Prepend[l,{"Calls","Time","Evaluation"}];
		MathLink`CallFrontEnd[
			FrontEnd`NotebookWrite[nb,
				Notebook[{Cell[BoxData[GridBox[l,ColumnLines->True,
				ColumnAlignments->{Right,Center,Left},
				RowLines->{True,False}]],"Text",
				ButtonBoxOptions->{ButtonExpandable->False }]}]]
		]
	]


createResultNB[ ]:=
	Module[ {nb, eval},
		nb = NotebookCreate[];
		eval = Evaluator /. Options[ EvaluationNotebook[]];
		If[ StringQ[ eval], SetOptions[ nb, Evaluator -> eval]];
		NotebookWrite[ nb, Cell["Profile Output", "Title"]];
		NotebookWrite[ nb, Cell["Generated by the Wolfram Workbench", "Subsubtitle"]];
		NotebookWrite[ nb, Cell[ calculateDate[], "Subsubtitle"]];
		NotebookWrite[ nb, Cell[ calculateInput[], "Input"]];
		nb
	];
    

months = {"January", "Febuary", "March", "April","May", "June",
"July", "August", "September", "October", "November", "December"};

calculateDate[] :=
	Module[ {y,m,d,h,min,sec},
		{y,m,d,h,min,sec} = Date[];
		{h,min,d,y} = Map[ToString,{h,min,d,y}];
		h <> ":" <> min <> "   " <> months[[m]] <> " " <> d <> " " <> y
	]

calculateInput[] :=
	Module[ {lab,box},
		box = ToBoxes[ computeExpr];
		lab = StyleBox[
            		RowBox[{"Input", ":"}],
            		"Text",
            		FontWeight->"Plain"] ;
       	BoxData[GridBox[{{lab, box}}]]
	]


End[]

EndPackage[]

