(* ::Package:: *)

Test::usage =
"Test[input] tests whether input evaluates to True and no messages are generated.\n
Test[input, expected] tests whether input evaluates to expected and no messages are generated.\n
Test[input, expected, messages] tests whether input evaluates to expected and the given messages are generated.\n
Test is also a selector for TestResult."

TestMatch::usage =
"TestMatch[input, expected] and TestMatch[input, expected, messages] are like the corresponding versions of Test except they test MatchQ[input, expected] instead of SameQ."

TestStringMatch::usage =
"TestStringMatch[input, expected] and TestStringMatch[input, expected, messages] are like the corresponding versions of Test except they test StringMatchQ[input, expected] instead of SameQ."

TestFree::usage =
"TestFree[input, expected] and TestFree[input, expected, messages] are like the corresponding versions of Test except they test FreeQ[input, expected] instead of SameQ."

TestStringFree::usage =
"TestStringFree[input, expected] and TestStringFree[input, expected, messages] are like the corresponding versions of Test except they test StringFreeQ[input, expected] instead of SameQ."

TestResultQ::usage =
"TestResultQ[tr] returns True if tr is a valid TestResult, and False otherwise."

FailureMode::usage =
"FailureMode is a selector for TestResult."

TestInput::usage =
"TestInput is a selector for TestResult."

ExpectedOutput::usage =
"ExpectedOutput is a selector for TestResult."

ActualOutput::usage =
"ActualOutput is a selector for TestResult."

ExpectedMessages::usage =
"ExpectedMessages is a selector for TestResult."

ActualMessages::usage =
"ActualMessages is a selector for TestResult."

TestIndex::usage =
"TestIndex is a selector for TestResult. It is used in test runs."

TestIndexIncrement::usage =
"TestIndexIncrement is the default incrementer for TestIndex."

MessagesEquivalenceFunction::usage =
"MessagesEquivalenceFunction is an option to Test and TestMatch that specifies the function that will be used to compare the actual and expected messages\n.
Custom MessagesEquivalenceFunction settings should also take into account that actual messages are passed in wrapped in HoldForm, \
and that a custom ExpectedMessagesWrapper will probably also be necessary to prevent any unwanted evaluations or messages."

TestFailureMessage::usage =
"TestFailureMessage is an option to Test that specifies a message that will be logged if the test fails.\n
TestFailureMessage is also a selector for TestResult."

TestFailureAction::usage =
"TestFailureAction is an option to Test that specifies a string naming an action to take if the test fails.\n
The allowable values are Continue (the default) and Abort.\n
TestFailureAction is also a selector for TestResult."

TestErrorAction::usage =
"TestErrorAction is an option to Test that specifies a string naming an action to take if the test has an error.\n
The allowable values are Continue (the default) and Abort.\n
TestErrorAction is also a selector for TestResult."

TestCPUTimeUsed::usage =
"TestCPUTimeUsed is a selector for test results."

TestAbsoluteTimeUsed::usage =
"TestAbsoluteTimeUsed is a selector for test results."

TestMemoryUsed::usage =
"TestMemoryUsed is a selector for test results."

TestSource::usage = 
"TestSource is an option of TestResult that specifies the source of the test: a file, notebook, or stream."

(* All meta *)

TestTags::usage =
"TestTags is an option to Test that specified meta information about that test. If a test comes from a notebook, its
cell tags are kept as TestTags."

TestClass::usage = "TestClass is deprecated, please use TestTags.";

TestComment::usage =
"TestComment is deprecated, please use TestTags."

TestComments::usage =
"TestComments is deprecated, please use TestTags."

AllTestIndex::usage =
"AllTestIndex is deprecated, please use TestTags."

ErrorMessage::usage =
"ErrorMessage is deprecated, please use TestFailureMessage."

EquivalenceFunction::usage =
"EquivalenceFunction is deprecated, please use SameTest"


Begin["`Package`"]

AddTestFunction

testError

End[]

Begin["`Test`Private`"]

Attributes[Test] = {HoldAllComplete}

Options[Test] = {
	TestID -> None,
	MemoryConstraint -> Infinity,
	TimeConstraint -> Infinity,
	SameTest -> SameQ,
	EquivalenceFunction -> SameQ,
	MessagesEquivalenceFunction -> Testing`MessageMatchQ,

	ErrorMessage -> {},
	TestFailureMessage -> {},
	TestFailureAction -> {},
	TestErrorAction -> {},

	(* all meta options accumulated over time, they will all be collected under the same umbrella *)
	TestTags -> {},
	TestComment -> {},
	TestComments -> {},
	TestClass -> {}
}



(*
	Implementing backward compatibility 
	legacy MUnit was using a lot of symbols to extract the properties from a test result object.
	now all munit functions are returning TestReportObject so it's just a matter of calling the right accessor.
	we should work for removing this compatibility layer in the future
*)

Unprotect[{TestID, SameTest}]

FailureMode[any_TestResultObject]                 := any["Outcome"]
TestInput[any_TestResultObject]                   := any["Input"]
ExpectedOutput[any_TestResultObject]              := any["ExpectedOutput"]
ActualOutput[any_TestResultObject]                := any["ActualOutput"]
ExpectedMessages[any_TestResultObject]            := any["ExpectedMessages"]
ActualMessages[any_TestResultObject]              := any["ActualMessages"]

TestID[any_TestResultObject]                      := any["TestID"]
TestIndex[any_TestResultObject]                   := any["TestKey"]
SameTest[any_TestResultObject]                    := any["SameTest"]
MessagesEquivalenceFunction[any_TestResultObject] := any["SameMessages"]
TestSource[any_TestResultObject]                  := any["TestFileName"]
TestFailureMessage[any_TestResultObject]          := Replace[any["TestFailureMessage"],      {{}|_Missing :> "", {f_} :> f}]
TestFailureAction[any_TestResultObject]           := Replace[any["TestFailureAction"],        _Missing :> "Continue"]
TestErrorAction[any_TestResultObject]             := Replace[any["TestErrorAction"],          _Missing :> "Continue"]
TestCPUTimeUsed[any_TestResultObject]             := any["CPUTimeUsedSeconds"]
TestAbsoluteTimeUsed[any_TestResultObject]        := any["AbsoluteTimeUsedSeconds"]
TestMemoryUsed[any_TestResultObject]              := any["MemoryUsedBytes"]
TestClass[any_TestResultObject]                   := Replace[any["Tags"], {{first_, ___} :> first, {}|_Missing :> None}]
TestTags[any_TestResultObject]                    := Replace[any["Tags"],                     _Missing :> {}]

(* duplicated properties *)
TestComment[any_TestResultObject]                 := TestTags[any]
TestComments[any_TestResultObject]                := TestTags[any]
ErrorMessage[any_TestResultObject]                := TestFailureMessage[any]

(* deprecated *)
AllTestIndex[obj_] := TestIndex[obj]

Protect[{TestID, SameTest}]

TestIndexIncrement = Function[Null, PreIncrement[$dynamicTestIndex], HoldAll];

(* all test functions are now returning TestResultObject those are valid test results *)

TestResultQ[TestResultObject[_?AssociationQ]] :=
	True (* all TestResultObject are valid *)
TestResultQ[_] :=
	False


(* Create helpful variants of Test with different SameTest specifications : TestMatch, TestStringMatch etc. *)

AddTestFunction[testFunc_] :=
	Module[{},
		Attributes[testFunc] = Attributes[Test];
		Options[testFunc] = Options[Test];
		testFunc[input_, expected_:True, Shortest[messages_:{}], opts:OptionsPattern[]] :=
			With[{options = Join[Flatten[{opts}], Options[testFunc]]},
				Test[input, expected, messages, options]
			]
	]

AddTestFunction[TestMatch]

SetOptions[TestMatch, SameTest -> MatchQ]

AddTestFunction[TestStringMatch]

SetOptions[TestStringMatch, SameTest -> StringMatchQ]

AddTestFunction[TestFree]

SetOptions[TestFree, SameTest -> FreeQ]

AddTestFunction[TestStringFree]

SetOptions[TestStringFree, SameTest -> StringFreeQ]






(* Test implementation uses TestEvaluate to avaluate the test and is adding some extra WRI specific metadata *)

extractMeta[NTestError[s:"TestErrorAction"|"TestFailureAction"], args_] :=
	Replace[Flatten[args], {___, any_} :> s -> any]

extractMeta[NTestError[s_String], args_] :=
	Replace[Flatten[args], any:{__} :> s -> any]


Test[input_, expected_:True, Shortest[msg_:{}], opts:OptionsPattern[]] :=

	Testing`$TestEvaluatePostProcessor @ Block[
		{Testing`$TestEvaluatePostProcessor  = Identity},
		Association @ Reap[
			(* 
				Collecting meta with reap/sow. 
				WRI functions can simply do Sow["MyMessage", NTestError["MyCustomKey"]] to add new keys to add object. 
				This can be done by evaluating a Sow call during for example a custom SameTest function call (see NTest for more details).
			*)

			Sow[OptionValue[Test, {opts}, {"TestClass", "TestTags", "TestComment", "TestComments"}], NTestError["Tags"]];
			Sow[OptionValue[Test, {opts}, {"TestErrorAction"}], NTestError["TestErrorAction"]];
			Sow[OptionValue[Test, {opts}, {"TestFailureAction"}], NTestError["TestFailureAction"]];
			Sow[OptionValue[Test, {opts}, {"TestFailureMessage", "ErrorMessage"}], NTestError["TestFailureMessage"]];

			Testing`TestEvaluate @ <|
				"Input"              -> HoldForm[input],
				"ExpectedOutput"     -> HoldForm[expected],
				"ExpectedMessages"   -> HoldForm[msg],

				"SameMessages"       -> OptionValue[Test, {opts}, "MessagesEquivalenceFunction"],
				"SameTest"           -> FirstCase[OptionValue[Test, {opts}, {"SameTest", "EquivalenceFunction"}], Except[SameQ], SameQ],
				
				"TestID"             -> OptionValue[Test, {opts}, "TestID"],
				"MemoryConstraint"   -> OptionValue[Test, {opts}, "MemoryConstraint"],
				"TimeConstraint"     -> OptionValue[Test, {opts}, "TimeConstraint"]
			|>,
			_NTestError,
			extractMeta
		]
	]


	
Test[args___] :=
    testError[args]



Attributes[testError] = {HoldAllComplete}



(* Utility function to create a TestResult object for an Error *)
testError[args___] :=
	Testing`$TestEvaluatePostProcessor @ Block[
		{
			Testing`$TestEvaluatePostProcessor := Append[
				{
					"TestFailureMessage" -> "Incorrect arguments: " <> ToString[Unevaluated[{args}]],
					"Outcome" -> "Error"
				}
			]
		},

		Quiet[
			Test @@ {"Error", "Error", {}, Cases[HoldComplete[args], OptionsPattern[], Infinity]},
			OptionValue::nodef
		]


	]




End[]
