TestSuiteReportObject;
TestSuiteReport;

Begin["`Package`"]


Options[TestSuiteReport] = 
	System`Private`SortOptions@Join[Options[TestReport],
		{"KernelCount" -> Automatic,
		 "TestFileTimeConstraint" -> Infinity, 
		 "TestFileMemoryConstraint" -> Infinity,
		 FileNameForms -> Alternatives @@ {"*.mt", "*.wlt", "*.mt0" (*, "*.nb"*)},
		 "ExcludedFileNameForms" -> None,
		 Initialization :> None,
		 Path -> Automatic}]

TestSuiteReport::invext = "Invalid FileNameForms specification `1`.";
TestSuiteReport::inexcl = "Invalid ExcludedFileNameForms specification `1`.";
TestSuiteReport::invfiles = "Invalid file specification.";
TestSuiteReport::noparkern = "No parallel kernel licenses are available, proceeding with serial evaluation.";
TestSuiteReport::nkernavail = "Only `2` kernels can be launched out of `1` requested.";

End[]


Begin["`TestSuite`Private`"]

AppendTo[$ContextPath, "BoxForm`"];

$links = {};
$abort = False;

$subKernelSetup = KernelConfiguration["localhost"]["KernelCommand"] <> " -mathlink -noicon";

iconSuccess = Dynamic@RawBoxes@FEPrivate`FrontEndResource["MUnitExpressions", "SuccessIcon"]

iconFailure = Dynamic@RawBoxes@FEPrivate`FrontEndResource["MUnitExpressions", "TestFailureIcon"]

iconMessagesFailure = Dynamic@RawBoxes@FEPrivate`FrontEndResource["MUnitExpressions", "MessageFailureIcon"]

iconError = Dynamic@RawBoxes@FEPrivate`FrontEndResource["MUnitExpressions", "ErrorIcon"]

iconWrapper[x_] :=
    Graphics[{Inset[
       Pane[x, ElisionsDump`defaultIconSize, Alignment -> Center]]}, 
     PlotRange -> {{0, 1}, {0, 1}}, Background -> GrayLevel[0.93], 
     Axes -> False, AspectRatio -> 1, 
     ImageSize -> 
      Dynamic@{Automatic, 
        3.5 CurrentValue["FontCapHeight"]/
          AbsoluteCurrentValue[Magnification]}, Frame -> True, 
     FrameTicks -> None, 
     FrameStyle -> Directive[Thickness[Tiny], GrayLevel[0.55]]]

toMixedTime[q : HoldPattern[Quantity][sec_, "Seconds"]] := Which[
  sec < 60, q,
  60 <= sec < 3600, 
  UnitConvert[q, MixedUnit[{"Minutes", "Seconds"}]],
  True, UnitConvert[q, MixedUnit[{"Hours", "Minutes", "Seconds"}]]
]
 
(*SummaryBox code for TestSuiteReportObject*)
TestSuiteReportObject /:
  MakeBoxes[tsro : TestSuiteReportObject[assoc_], fmt_] /; 
  BoxForm`UseIcons && AssociationQ[Unevaluated@assoc] := 
 Module[{alwaysGrid, sometimesGrid, totalfiles, totaltests, icon, crashedFiles, layout},
     totalfiles = Length@tsro["Results"];
     totaltests = tsro["TotalTestsCount"];
     crashedFiles = Select[tsro["Results"], FailureQ[#] &];
     layout = "Files";
     icon = Which[
     	(And @@ tsro["AllTestsSucceeded"]) && (Length@crashedFiles === 0),
     	iconSuccess,
     	Length@crashedFiles > 0,
     	iconError,
     	Total@tsro["TestsFailedWrongResultsCount"] > 0,
     	iconFailure,
     	Total@tsro["TestsFailedWithMessagesCount"] > 0,
     	iconMessagesFailure,
     	Total@tsro["TestsFailedWithErrorsCount"] > 0,
     	iconError,
     	True, (* should never reach this *)
     	iconError
     ];
     alwaysGrid = {
     	{
     		BoxForm`MakeSummaryItem[{"Success rate: ", 
	        	If[ totaltests === 0,Indeterminate, Row[{Round[100. tsro["TotalTestsSucceededCount"]/totaltests], "%"}]]}, fmt],
     		BoxForm`MakeSummaryItem[{"Tests run : ", totaltests}, fmt]
     	 }, 
        {BoxForm`MakeSummaryItem[{"Test files : ", totalfiles}, fmt], If[Length@crashedFiles>0, BoxForm`MakeSummaryItem[{"Crashes : ", Length@crashedFiles}, fmt], Nothing]}};
     sometimesGrid = {#, SpanFromLeft} & /@{
        BoxForm`MakeSummaryItem[{"Succeeded : ", tsro["TotalTestsSucceededCount"]}, fmt],
        BoxForm`MakeSummaryItem[{"Failed : ", tsro["TotalTestsFailedCount"]}, fmt],
        If[tsro["TotalTestsFailedCount"] > 0,
	        Sequence@@{BoxForm`MakeSummaryItem[{"Failed with wrong result : ", tsro["TotalTestsFailedWrongResultsCount"]}, fmt],
	        BoxForm`MakeSummaryItem[{"Failed with messages : ", tsro["TotalTestsFailedWithMessagesCount"]}, fmt],
	        BoxForm`MakeSummaryItem[{"Failed with errors : ", tsro["TotalTestsFailedWithErrorsCount"]}, fmt]},
	        Nothing],
	     BoxForm`MakeSummaryItem[{"Time elapsed : ", Round[toMixedTime@tsro["TotalTimeElapsed"], 0.1]}, fmt]
        };
     BoxForm`ArrangeSummaryBox[TestSuiteReportObject, tsro, iconWrapper@icon, alwaysGrid, sometimesGrid, fmt]
 ]

$testReportObjectProperties = {"Aborted", "AllTestsSucceeded", 
   "TestResults", "TestResultRules", "TestsSucceededCount", "TestsFailedCount",
   "TestsFailedWrongResultsCount", "TestsFailedWithMessagesCount", 
   "TestsFailedWithErrorsCount", 
   "TimeElapsed", "Title"};
$indexProperties = {"TestsSucceededIndices", "TestsFailedIndices", "TestsFailedWrongResultsIndices", 
   "TestsFailedWithMessagesIndices", "TestsFailedWithErrorsIndices"};
$resultProperties = {"TestsSucceeded", "TestsFailed", 
   "TestsFailedWrongResults", "TestsFailedWithMessages", 
   "TestsFailedWithErrors"};
$suiteProperties = {"Results", "TotalTestsCount", "TotalTestsSucceededCount",
	"TotalTestsFailedCount", "TotalTestsFailedWrongResultsCount", "TotalTestsFailedWithMessagesCount",
	"TotalTestsFailedWithErrorsCount", "AbortedTestFiles", "TotalTimeElapsed"};

TestSuiteReportObject[a_Association]["Results"] :=
    a["Results"];
TestSuiteReportObject[a_Association]["TotalTimeElapsed"] := 
	a["TotalTimeElapsed"];
TestSuiteReportObject[a_Association]["TotalTestsCount"] :=
    Total[Length[#["TestResults"]] & /@ Select[a["Results"], ! FailureQ[#] &]];
TestSuiteReportObject[a_Association]["TotalTestsSucceededCount"] :=
    Total[Length[#["TestsSucceeded"]] & /@ Select[a["Results"], ! FailureQ[#] &]];
TestSuiteReportObject[a_Association]["TotalTestsFailedCount"] :=
    Total[#["TestsFailedCount"] & /@ Select[a["Results"], ! FailureQ[#] &]];
TestSuiteReportObject[a_Association]["TotalTestsFailedWrongResultsCount"] :=
    Total[#["TestsFailedWrongResultsCount"] & /@ Select[a["Results"], ! FailureQ[#] &]];
TestSuiteReportObject[a_Association]["TotalTestsFailedWithMessagesCount"] :=
    Total[#["TestsFailedWithMessagesCount"] & /@ Select[a["Results"], ! FailureQ[#] &]];
TestSuiteReportObject[a_Association]["TotalTestsFailedWithErrorsCount"] :=
    Total[#["TestsFailedWithErrorsCount"] & /@ Select[a["Results"], ! FailureQ[#] &]];
TestSuiteReportObject[a_Association]["AbortedTestFiles"] := 
	Select[a["Results"], FailureQ[#] &];
TestSuiteReportObject[a_Association]["Properties"] := 
	Sort@Join[$suiteProperties,$testReportObjectProperties,$indexProperties,$resultProperties];
TestSuiteReportObject[a_Association][prop_] :=
    Select[
    	Map[#[prop] &, Select[a["Results"], ! FailureQ[#] &]], 
    	# =!= <||> && (Union@Values[#]) =!= {<||>} &
    ] /; MemberQ[$resultProperties, prop];
TestSuiteReportObject[a_Association][prop_] :=
    Select[
    	Map[#[prop] &, Select[a["Results"], ! FailureQ[#] &]],
    	# =!= {}&
    ] /; MemberQ[$indexProperties, prop];
TestSuiteReportObject[a_Association][prop_] :=
    Map[#[prop] &, Select[a["Results"], ! FailureQ[#] &]] /; MemberQ[$testReportObjectProperties, prop];
TestSuiteReportObject[_][noprop_] := (Message[TestSuiteReportObject::noprop, noprop];$Failed)

memFailure[limit_]:= Failure["MemoryConstrained",  
	<|"MessageTemplate" -> "Current evaluation has exceeded the memory constraint: `memoryConstraint`",
	  "MessageParameters"-> <|"memoryConstraint" -> limit|>|>
	];
	
timeFailure[limit_]:= Failure["TimeConstrained",  
	<|"MessageTemplate" -> "Current evaluation has exceeded the time constraint: `timeConstraint`",
	"MessageParameters"-> <|"timeConstraint" -> limit|>|>
	];
	
crashFailure[]:=Failure["UnexpectedExit",  
	<|"MessageTemplate" -> "Connection to kernel unexpectedly closed during evaluation.",
	  "MessageParameters"-> <||>|>
	]

dispatchToLink[file_, timeout_, memlimit_:Infinity, init_:None, path_:None] :=
    Module[ {link,pkt},
        link = LinkLaunch[$subKernelSetup];
        AppendTo[$links,link];
        Quiet[LinkRead[link]];
        pkt =  Hold[EvaluatePacket[(Needs["MUnit`"];
        							ReleaseHold@path;
                  					ReleaseHold@init;
                                    RunScheduledTask[Exit[], {timeout + 10}]; (* Allow a grace period of 10 seconds over the limit before we kill the kernel *)
                                    TimeConstrained[
                                       MemoryConstrained[TestReport[file], memlimit, memFailure[memlimit]],
                                       timeout,
                                       timeFailure[timeout]
                                    ])]];
        LinkWriteHeld[link,pkt];
        LinkFlush[link];
        link
    ]

getResAndCloseLink[link_] :=
    Module[ {packet, packetHead},
        Quiet@Check[packet = LinkReadHeld[link],$links = DeleteCases[$links,link];
                                         Return[crashFailure[]]];
        packetHead = Extract[packet, {1, 0}];
        Switch[packetHead,
        	ReturnPacket,
		        LinkClose[link];
		        $links = DeleteCases[$links,link];
		        First@ReleaseHold@packet,
		    (* Did not get a return packet *)
	        _,
	        	$continue
	        	(* TODO : consider special handling of MessagePacket etc.*)
        ]
    ]

testFileQ[path_String] := TrueQ[FileExistsQ[path] && FileType[path]===File];
testFileQ[file:File[str_String]] := testFileQ[str];
testFileQ[_] := False;

testingNotebookQ[path_String?FileExistsQ] := 
	Module[{nbObject, res},
		nbObject = NotebookOpen[path, Visible -> False];
  		res = CurrentValue[nbObject, StyleDefinitions] === FrontEnd`FileName[{"MUnit"}, "MUnit.nb", CharacterEncoding -> "UTF-8"];
  		NotebookClose[nbObject];
  		res
  	];
testingNotebookQ[file:File[str_String]]:= testingNotebookQ[str];
testingNotebookQ[_]:= False;

$primitives = _String | _StringExpression | _RegularExpression;
$formsPattern = Verbatim[Alternatives][$primitives ..] | $primitives | {$primitives ...};

pruneDirectories[dirs : {__?DirectoryQ}] :=
    Module[ {assoc, base, parent},
    (* Initial set of all directories *)
        assoc = AssociationThread[dirs -> dirs];
        Do[
        	base = dir;
            (* Work from the directory up to the root directory *)
            parent = ParentDirectory[base];
		  	While[! SameQ[base, parent],
		  		(*If any of the parent directories are already present then remove this directory*)
		    	base = parent;
		        If[ UnsameQ[Lookup[assoc, base, False], False],
		        	KeyDropFrom[assoc, dir]
		        ];
		    	parent = ParentDirectory[base];
		  	];
            ,
            {dir, dirs}];
        (* Get the minimal set of base directories *)
        Values[assoc]
    ];

TestSuiteReport[dir_String, opts : OptionsPattern[]] :=
    Module[ {files, exts, excl, path, newPath},
        exts = OptionValue[FileNameForms];
        If[ ! MatchQ[exts, $formsPattern ],
            Message[TestSuiteReport::invext, exts];
            Return[$Failed]
        ];
        excl = OptionValue["ExcludedFileNameForms"];
        If[ ! MatchQ[excl, None | $formsPattern ],
            Message[TestSuiteReport::inexcl, excl];
            Return[$Failed]
        ];
        exts = Replace[exts, l_List :> Alternatives@@l]; (* Change list of patterns to Alternatives *)
        files = FileNames[exts, dir, Infinity];
        
        path = OptionValue[Path];
        newPath = Switch[path,
        	Automatic, Append[$Path, dir],
        	_, path
        ];
        
        TestSuiteReport[files, FilterRules[Flatten@{opts}, Except[Path]], Path -> newPath]
    ] /; DirectoryQ[dir]

TestSuiteReport[file_String?testFileQ, opts:OptionsPattern[]] := TestSuiteReport[{file}, opts]

TestSuiteReport[inFiles : {(_String|_File)?testFileQ ...}, opts : OptionsPattern[]] :=
    Module[ {files, numFiles, results, testProgress, initTime, parallel, 
      hangtime, nKernels, startingFiles, remainingFiles, readyLinks, n, nextFiles, returned,
      timeWaiting, waitStart, memconstraint, notebookFiles, testingNotebooks, runningFiles,
      initialization, rawnKernels, availableLicenses, excl, time, path},
        Quiet[LinkClose/@$links];
        $links = {}; (* Should not be necessary *)
        
        (* Make sure all notebooks provided are Testing Notebooks *)
        notebookFiles = Select[inFiles, FileExtension[#]==="nb"&];
        testingNotebooks = Select[notebookFiles, testingNotebookQ];
        files = DeleteCases[inFiles, Alternatives@@Complement[notebookFiles, testingNotebooks]];
        
        (* Remove files that match the exclusions *)
        excl = OptionValue["ExcludedFileNameForms"];
        If[ excl =!= None,
            excl = Replace[excl, l_List :> Alternatives@@l]; (* Change list of patterns to Alternatives *)
            files = DeleteCases[files, _?(StringMatchQ[FileNameTake[#,-1], excl]&)]
        ];
        
        (* Determine the subkernel $Path specification *)
        path = OptionValue[Path];
        path = Switch[path,
            Automatic, (* distribute $Path to subkernels *)
                With[ {p = $Path},
                    Hold[$Path = p]
                ],
            _, 
                With[ {p = path},
                    Hold[$Path = p]
                ]
        ];
        numFiles = Length[files];
        nKernels = OptionValue["KernelCount"];
        If[ !MatchQ[nKernels, (_Integer?(NonNegative[#] && (# <= Developer`$MaxMachineInteger) &)) | Automatic],
            Message[TestSuiteReport::invopt, nKernels, "KernelCount", Automatic];
            nKernels = Automatic
        ];
        rawnKernels = nKernels;
        availableLicenses = $MaxLicenseSubprocesses - $KernelCount;
        nKernels = Replace[nKernels, {Automatic -> Min[$ProcessorCount, availableLicenses],
                                      k_Integer :> Min[k, availableLicenses]}];
        parallel = nKernels > 0;
        Which[ 
            (* Told to use parallel but can't open any kernels *)
            !parallel && (rawnKernels === Automatic || rawnKernels > 0 ), Message[TestSuiteReport::noparkern],
            (* Told to use specific number of kernels but can't launch that many*)
            rawnKernels > 0 && nKernels < rawnKernels, Message[TestSuiteReport::nkernavail, rawnKernels, nKernels]
        ];
        testProgress = 0;
        hangtime = OptionValue["TestFileTimeConstraint"];
        memconstraint = OptionValue["TestFileMemoryConstraint"];
        initialization = OptionValue[Automatic, Automatic, Initialization, Hold];
        runningFiles = {};
        Progress`EvaluateWithProgress[
            If[ parallel,
                CheckAbort[Catch[
                    initTime = AbsoluteTime[];
                    {startingFiles, remainingFiles} = TakeDrop[files, UpTo[nKernels]];
                    results = AssociationMap[dispatchToLink[#, hangtime, memconstraint, initialization, path]&, startingFiles];
                    timeWaiting = 0;
                    waitStart = AbsoluteTime[];
                    While[ (Length[$links] > 0 || Length[remainingFiles] > 0),
                        runningFiles = Replace[Flatten@Position[results, _LinkObject, 1],{Key[a : (_String | _File)]:>a}, 1];
                        If[ Length@$links > 0,
                            readyLinks = Pick[$links, Quiet[LinkReadyQ[$links]]];
                            Which[ 
                                TrueQ[$abort],
                                    (* User has manually terminated the test run *)
                                    LinkClose /@ $links;
                                    (*results = Replace[results, _LinkObject :> $Failed, 1];*)
                                    results = DeleteCases[results, _LinkObject];
                                    $links = {};
                                    remainingFiles = {}; (* don't run any more files - we're aborting *)
                                    Throw[results]
                                ,
                                Length[readyLinks] > 0,
                                      (* There's a ready link - read and update results *)
                                      returned = getResAndCloseLink[First@readyLinks];
                                      If[ returned =!= $continue, (* We got a ReturnPacket *)
                                          results = Replace[results, First@readyLinks -> returned, 1];
                                          testProgress++
                                      ];
                                      (* Reset timer *)
                                      timeWaiting = 0;
                                      waitStart = AbsoluteTime[];
                                      ,
                                timeWaiting > hangtime,
                                    (* All running links have timed out - kill and update results *)
                                     LinkClose /@ $links;
                                     testProgress += Length@$links;
                                     results = Replace[results, Alternatives @@ $links -> timeFailure[hangtime], 1];
                                     $links = {};
                                     (* Reset timer *)
                                     timeWaiting = 0;
                                     waitStart = AbsoluteTime[];
                                ,
                                True,
                                     Pause[0.1];
                                     (* Update timer *)
                                     timeWaiting = AbsoluteTime[] - waitStart;
                            ]
                        ];
                        If[ Length@remainingFiles > 0 && Length[$links] < nKernels,
                            n = Min[Length@remainingFiles, nKernels - Length[$links]];
                            {nextFiles, remainingFiles} = TakeDrop[remainingFiles, n];
                            AppendTo[results, AssociationMap[dispatchToLink[#, hangtime, memconstraint, initialization]&, nextFiles]];
                        ]
                    ];
                    Quiet[LinkClose/@$links];
                    $links = {}; (* should not be necessary *)
                    results
                    ], 
                    (* If aborting:
                    * Close links so that they can be reused
                    * Filter out LinkObject in the result *)
                    Quiet[LinkClose/@$links];
                    $links = {};
                    results = DeleteCases[results, _LinkObject];
                    $Aborted],
                (* Serial code *)
                CheckAbort[Catch[
                    initTime = AbsoluteTime[];
                    ReleaseHold@initialization;
                    runningFiles = {};
                    {runningFiles, remainingFiles} = TakeDrop[files, UpTo[1]];
                    results = <||>;
                    While[Length[remainingFiles]>=0 && Length[runningFiles]>0,
                        If[ TrueQ[$abort],
                            Throw[results]
                        ];
                        AppendTo[results, First@runningFiles -> TestReport[First@runningFiles, FilterRules[Flatten@{opts}, Options[TestReport]]]];
                        testProgress++;
                        runningFiles = {};
                        If[ Length@remainingFiles>0,
                            {runningFiles, remainingFiles} = TakeDrop[remainingFiles, 1];
                        ]
                    ]
                ], $Aborted]
            (* EndIf *)],
                <|
                    "Text"        -> "Running test files : ",
                    "ItemCurrent" :> testProgress,
                    "ItemTotal"   :> numFiles,
                    "ItemName"    :> "files",
                    "Detail"      :> Switch[
                                        Length[runningFiles],
                                        _?(# > 1 &), If[ TrueQ[$Notebooks],
                                                         Column[FileNameTake /@ runningFiles],
                                                         FileNameTake /@ runningFiles
                                                     ],
                                        1, First[FileNameTake /@ runningFiles],
                                        _, None],
                    "Progress"    -> Automatic,
                    "ElapsedTime" :> Automatic
                |>
                , 
                "Buttons" -> <|"Stop" :> ($abort = True)|>, 
                UpdateInterval -> 1
                ];
        time = Quantity[AbsoluteTime[] - initTime,"Seconds"];
        $abort = False;
        TestSuiteReportObject[<|"Results" -> results, "TotalTimeElapsed" -> time|>]
    ];

TestSuiteReport[list_List, options : OptionsPattern[]] :=
    Module[ {exts, dirs, files, newPath, path},
        exts = OptionValue[FileNameForms];
        If[!MatchQ[exts, $formsPattern],
            Message[TestSuiteReport::invext, exts];
            Return[$Failed]
        ];
        If[!MatchQ[list, {(_String?FileExistsQ | _File?FileExistsQ) ...}],
            Message[TestSuiteReport::invfiles];
            Return[$Failed]
        ];
        dirs = Select[list, DirectoryQ];

		path = OptionValue[Path];
		newPath = Switch[path,
        	Automatic, Join[$Path, pruneDirectories[dirs]],
        	_, path
        ];
		
		files = DeleteDuplicates@Join[Complement[list, dirs], Flatten@Map[FileNames[exts, #, Infinity]&, dirs]];
        TestSuiteReport[files, FilterRules[Flatten@{options}, Except[Path]], Path -> newPath]
    ];
    
TestSuiteReport[invalid_, OptionsPattern[]]:= (Message[TestSuiteReport::invfiles]; $Failed)

TestSuiteReport[args___] := $Failed /; System`Private`Arguments[TestSuiteReport[args], 1];



(*===================== Code copied over from GeneralUtilities to remove need to load the full paclet =====================*)

$rts = False;
   
timeString[Indeterminate] := "indeterminate";

timeString[t_Integer] := Block[{$rts = True}, timeString[N[t]]];

timeString[t_Real] := Module[{h,m,s},
	h = Floor[t / 3600];
	m = Mod[Floor[t / 60], 60];
	s = Mod[Floor[t], 60];
	Which[
		h > 0, IntegerString[h] <> "h" <> IntegerString[m, 10, 2] <> "m" <> IntegerString[s, 10, 2] <> "s",
		m > 0, IntegerString[m] <> "m" <> IntegerString[s, 10, 2] <> "s",
		t > 10, IntegerString[s] <> "s",
		$rts, IntegerString[Round[t]] <> "s",
		t >= 1, numstr[t, 2] <> "s",
		t >= 0.1, numstr[t, 3] <> "s",
		t >= 0.01, IntegerString[Round[1000*t]] <> "ms",
		t >= 0.001, numstr[1000*t, 2] <> "ms",
		t == 0, "0s",
		True, IntegerString[Round[1000000*t]] <> "\[Micro]s"
	]
];

numstr[t_, d_] := StringReplace[TextString[NumberForm[t, d]], "." ~~ EndOfString -> ".0"];   
   
End[];