(* ::Package:: *)

(*
WRI.m provides a compatibility layer for the old-style Testing package
These symbols are reproduced from the Testing package.
*)

NTest::usage = 
"NTest[input, expected, prec] tests whether input evaluates to expected with prec numerical precision. \n
NTest[input, expected, prec, messages] tests whether input evaluates to expected with prec numerical precision and the given messages are issued."

ExactTest::usage =
"ExactTest[input, expected] tests whether input evaluates to expected while maintaining expected in unevaluated form.\n
ExactTest[input, expected, messages] tests whether input evaluates to expected while maintaining expected in unevaluated form and the given messages are generated."

OrTest::usage = 
"OrTest[input, {expected1, expected2,...}] tests whether input evaluates to any of the expected1, expected2, etc. \n
OrTest[input, {expected1, expected2,...}, messages] tests whether input evaluates to any of the expected1, expected2, etc. and the given messages are issued."

OrNTest::usage = 
"OrNTest[input, {expected1, expected2,...}, prec] tests whether input evaluates to any of the expected1, expected2, etc. with prec numerical precision. \n
OrNTest[input, {expected1, expected2,...}, prec, messages] tests whether input evaluates to any of the expected1, expected2, etc. with prec numerical precision and the given messages are issued."

NTestMatch::usage = 
"NTestMatch[input, expected, prec] tests whether input matches expected pattern up to prec numerical precision. \n
NTestMatch[input, expected, prec, messages] tests whether input matches expected up to prec numerical precision and the given messages are issued."

ConditionalTest

DefectiveTest

EndIgnore

EndRequirement

ExactTest

ExactInputTest

ExactTestCaveat

NTest

NTestCaveat

OrTest

TestCaveat

TestExecute

TestIgnore

TestRequirement

$DefaultTestManager

$FailurePriority

$FailureReportProlog

$MinimumTestAbortTime

$MemoryMessages

$RunDefectiveTests

$SaveTiming

$ShowMemoryInUse

$ShowReport

$TestAbortTime

$TestAbortTimeInitial

$TestAbortTimeMultiplier

$TestMemoryLimitInitial

$TestMemoryLimit

$TestSearchPath

TestNameTemporary

OrNTest

NTestMatch

Begin["`Package`"]

NTestError

End[]

Begin["`WRI`Private`"]

compatOptionValue[name_, opts_, sym_] :=
	name /. Flatten[opts] /. Options[sym]

Attributes[ConditionalTest] = {HoldAll}

Options[ConditionalTest] = Options[Test]

Attributes[$Hold] = {HoldAll}

Attributes[testtypeheld] = {HoldAll}

(* opts isn't using ___?OptionQ because OptionQ evaluates its arguments and we don't want that *)
(*TODO : Investigate using OptionsPattern[] here instead of checking for Rule/RuleDelayed *)
ConditionalTest[test_, input_, expectedList__List, opts:(_Rule | _RuleDelayed)...] :=
	Module[{expres, expresbool, boolposn, returnargs, t},
		expres = Map[Hold, Hold[{expectedList}], 3];
		expres = ReleaseHold[ReleaseHold[ReleaseHold[expres]]];
		expresbool = Map[ReleaseHold[#[[1]]] &, expres];
		boolposn = Position[expresbool, True];
		returnargs = Drop[expres[[boolposn[[1, 1]]]], 1];
		returnargs = Map[Apply[$Hold, #] &, returnargs];
		With[{returnargs = returnargs},
			testtypeheld[input, t, opts] /. t -> Apply[Sequence, returnargs] /. $Hold[x_] -> x /. testtypeheld :> test
		]
	]

ConditionalTest[args___] :=
	With[{msg = "Incorrect arguments: " <> ToString[Unevaluated[{args}]]},
		testError[msg, args]
	]

Attributes[DefectiveTest] = {HoldAllComplete}


Attributes[ExactTest] = {HoldAll}
Options[ExactTest] = Options[Test]

ExactTest[input_, expected_:True, Shortest[messages_:{}], opts:OptionsPattern[]] :=
	Test[input, Unevaluated[expected], messages, opts]

ExactTest[args___] :=
	With[{msg = "Incorrect arguments: " <> ToString[Unevaluated[{args}]]},
		testError[msg, args]
	]



Attributes[ExactInputTest] = {HoldAll}
Options[ExactInputTest] = Options[Test]

ExactInputTest[input_, expected_:True, Shortest[messages_:{}], opts:OptionsPattern[]] :=
	Test[ToString[input, InputForm], ToString[Unevaluated[expected], InputForm], messages, opts]

ExactInputTest[args___] :=
	With[{msg = "Incorrect arguments: " <> ToString[Unevaluated[{args}]]},
		testError[msg, args]
	]
	
(* Utility functions for NTest when using ULPS *)
CompareNumerical[actual_, expected_, tol_] :=
	Module[{tExpected, tActual, temp},
		(*If[Precision[e1] === Infinity,
			(*Message[CompareNumerical::exact, e1, e2]*)
			Null
		];*)
		tActual = RepNums[ReleaseHold[actual]];
		tExpected = RepNums[ReleaseHold[expected]];
		If[First[tExpected] =!= First[tActual],
			{False},
			temp = NumberCompare[Last[tActual], Last[tExpected], tol];
			If[temp === True, {True}, {False, {"CompareNumerical", {False, temp}}}]
		]
	]

(* RepNums takes an expression and replaces the numbers in it with
   numbered slots.  For instance:
   2.3 + x^4.2  --> {$Number[1] + x^$Number[2], {2.3, 4.2}}
   We then check that the structure is the same and that the numbers
   are the same within tolerances. *)
(*
This version differs from the Testing` package because it uses Replace instead of Map.
This version works correctly for expressions whose heads have Hold* attributes.
In the old Map version, you could end up with results like

{SysBody[(If[NumberQ[#1] && #1 =!= Overflow[], 
      If[Precision[#1] < \[Infinity] || 
        Precision[#1] === MachinePrecision, 
       AppendTo[nums, #1]; $Number[numcount++], #1], #1] &)[1], 
  "<>"], {}}
  
for heads like SysBody that have HoldAll attribute, because the function was never being applied.

*)
RepNums[expr_] :=
	Module[{nums = {}, numcount = 1, t},
		t =
		Replace[
			expr
			,
			{n_?NumberQ :>
				With[{res = If[n =!= Overflow[],
								If[Or[Precision[n] < Infinity, Precision[n] === MachinePrecision],
									AppendTo[nums, n];
									$Number[numcount++]
									,
									n
								]
								,
								n
							]},
					res /; True]}
			,
			{-1}
		];
		{t, nums}
	]

(* NumberCompare0[ ] needs to be Listable *)
Attributes[NumberCompare0] = {Listable}

NumberCompare[actual_, expected_, tol_] :=
    AndNumerical[NumberCompare0[actual, expected, tol]]

NumberCompare0[actual_, expected_, tol_] :=
	Module[{ulpsExp, prec, res, goal},
		ulpsExp = Log10[tol];
		prec = Min[Precision /@ {actual, expected}];
		goal = Min[prec, -(ulpsExp - prec + 1)];
		res = Which[
				(* Ulps near 0. is unreliable, compare absolute error as a fall back *)
				TrueQ[Abs[expected - actual] < tol * $MachineEpsilon], True, 
				True, 
					(* For input less than 1, AccuracyGoal will take effect as the PrecisionGoal will be harder to meet.
					For input more than 1, PrecisionGoal will be used. *)
					Internal`CompareToPAT[expected, actual, AccuracyGoal -> goal, PrecisionGoal -> goal]
				];
		If[TrueQ[res],
			True
			,
			First[res]
		]
	]

AndNumerical[l_] :=
	Module[{numlist = Cases[l, {_, _?NumberQ, _}, {1, -2}]},
		If[numlist == {},
			Apply[And, l],
			MaximalBy[numlist, Part[#, 2] &]
		]
	]



Attributes[NTest] = {HoldAll}

Options[NTest] = Union[Options[Test], {AccuracyGoal -> None, PrecisionGoal -> None, Tolerance -> None}]

(* setting EquivalenceFunction can't work because options need to be passed to CompareToPAT *)

NTest[input_, expected_, Optional[preUlps:(_?NumberQ | None), 10^(3+$MachinePrecision-16)], Shortest[messages_:{}], opts___?OptionQ] :=
	Module[{accuracy, precision, tolerance, ulps},
		accuracy = compatOptionValue[AccuracyGoal, {opts}, NTest];
		precision = compatOptionValue[PrecisionGoal, {opts}, NTest];
		tolerance = compatOptionValue[Tolerance, {opts}, NTest];
		(* treat ulps of None same as default, for compatibility *)
		ulps = preUlps /. None -> 10^(3+$MachinePrecision-16);
		If[{accuracy, precision, tolerance} =!= {None, None, None},
			With[{
				(* use accuracy, precision, tolerance *)
				accuracy = accuracy, precision = precision, tolerance = tolerance,
				opts2 = FilterRules[Join[Flatten[{opts}], Options[NTest]], Except[FilterRules[Options[NTest], Except[Options[Test]]]]]},
				Test[
					input,
					expected,
					messages,
					SameTest -> $NTestPATEquivalenceFunction[{accuracy, precision, tolerance}],
					opts2
				]
			]
			,
			(* use ulps *)
			With[{opts2 = FilterRules[Join[Flatten[{opts}], Options[NTest]], Except[FilterRules[Options[NTest], Except[Options[Test]]]]]},
				Test[
					input,
					expected,
					messages,
					SameTest -> $NTestULPSEquivalenceFunction[ulps],
					opts2
				]
			]
		]
	]

NTest[args___] :=
	With[{msg = "Incorrect arguments: " <> ToString[Unevaluated[{args}]]},
		testError[msg, args]
	]


logNTestFailure[message_, type_] :=
	Sow[
		Switch[type,
			"ULPS",
			"Error (ULPS):   " <> StringJoin[Riffle[MUnit`WRI`Private`InformativeString /@ message, "\n"]]
			,
			"CompareToPAT",
			StringJoin[Riffle[MUnit`WRI`Private`InformativeString /@ message, "\n"]]
			,
			"MatchNumerics",
			"Error (Pattern):   " <> ToString[message]
			,
			"EquivalenceFunction",
			StringRiffle[message]
			, 
			_, ToString[message]
		],
		NTestError["TestFailureMessage"]
	]



InformativeString[_Internal`CompareToPAT] := "The arguments to the numerical comparison function were incorrect."

InformativeString[{string_String, _, {}}] = string;

InformativeString[{Tolerance, -Infinity, _}] := "Tolerance was satisfied"

InformativeString[{pat:Precision | Accuracy, Infinity, _}] := StringJoin[
	ToString[pat],
	"Goal was satisfied"]

InformativeString[{Precision, perr_, {}}] := StringJoin[
	"Actual only matches expected to ",
	ToString[perr],
	" digits of Precision (relative error)"]

InformativeString[{Accuracy, aerr_, {}}] :=StringJoin[
	"Actual only matches expected to ",
	ToString[aerr],
	" digits of Accuracy (absolute error)"]

InformativeString[{Tolerance, terr_, {}}] :=StringJoin[
	"Actual deviates from expected with ",
	ToString[terr],
	" least significant digits (Tolerance)"]

InformativeString[{pat_, perr_, Condition[pos_List, Length[pos] > 0]}] := StringJoin[
	InformativeString[{pat, perr, {}}],
	" at position ",
	ToString[pos]]

InformativeString[{False, ulps_}] := StringJoin[
	"Error (ULPS):   ", 
	ToString[InputForm[ulps]]]

InformativeString[False] := "NOT AN ULPS FAILURE (check types and shape)"

$NTestPATEquivalenceFunction[{accuracy_, precision_, tolerance_}][actual_, expected_] :=
	Module[{res},
		res = Internal`CompareToPAT[
			expected,
			actual,
			AccuracyGoal -> accuracy,
			PrecisionGoal -> precision,
			Tolerance -> tolerance
		];
		If[TrueQ[res],
			True
			,
			logNTestFailure[res, "CompareToPAT"];
			False
		]
	]

$NTestPATEquivalenceFunction[pat__][args___] := 
	(logNTestFailure[{"Incorrect second arguments in $NTestPATEquivalenceFunction[", pat, "][", args, "]"}, "EquivalenceFunction"];
	False);

$NTestULPSEquivalenceFunction[ulps_][HoldForm[actual_], HoldForm[expected_]] := 
	$NTestULPSEquivalenceFunction[ulps][actual, expected];
	
$NTestULPSEquivalenceFunction[ulps_][actual_, expected_] :=
	Module[{res, temp},
		res = CompareNumerical[Hold[actual], Hold[expected], ulps];
		Switch[res,
			{True},
			True
			,
			{False, {"CompareNumerical", {False, _}}},
			temp = res[[2, 2, 2]];
			logNTestFailure[temp, "ULPS"];
			False
			,
			_,
			False
		]
	]

$NTestULPSEquivalenceFunction[ulps__][args___] :=
	(logNTestFailure[{"Incorrect second arguments in $NTestULPSEquivalenceFunction[", ulps, "][", args, "]"}, "EquivalenceFunction"];
	False);

Attributes[OrTest] = Attributes[Test]
Options[OrTest] = Options[Test]
SetOptions[OrTest, SameTest -> Function[{actual, expected}, AnyTrue[expected, # === actual &]]]

OrTest[input_, expected:(_[___]), Shortest[messages_:{}], opts___?OptionQ] :=
	With[{opts2 = Join[Flatten[{opts}], Options[OrTest]]},
		Test[input, expected, messages, opts2]
	]

OrTest[args___] :=
	With[{msg = "Incorrect arguments: " <> ToString[Unevaluated[{args}]]},
		testError[msg, args]
	]


Attributes[TestCaveat] = Attributes[Test]
Options[TestCaveat] = Options[Test]

TestCaveat[input_, expected_, caveat_, Shortest[messages_:{}], opts___?OptionQ] :=
	Test[input, expected, messages, TestFailureMessage :> caveat, opts]

TestCaveat[args___] :=
	With[{msg = "Incorrect arguments: " <> ToString[Unevaluated[{args}]]},
		testError[msg, args]
	]




Attributes[NTestCaveat] = Attributes[NTest]
Options[NTestCaveat] = Options[NTest]

NTestCaveat[input_, expected_, Optional[ulps:(_?NumberQ | None), 10^(3+$MachinePrecision-16)], caveat_, Shortest[messages_:{}], opts___?OptionQ] :=
	NTest[input, expected, ulps, messages, TestFailureMessage :> caveat, opts]

NTestCaveat[args___] :=
	With[{msg = "Incorrect arguments: " <> ToString[Unevaluated[{args}]]},
		testError[msg, args]
	]





Attributes[ExactTestCaveat] = Attributes[ExactTest]
Options[ExactTestCaveat] = Options[ExactTest]

ExactTestCaveat[input_, expected_, caveat_, Shortest[messages_:{}], opts___?OptionQ] :=
	Test[input, expected, messages, TestFailureMessage -> caveat, opts]

ExactTestCaveat[args___] :=
	With[{msg = "Incorrect arguments: " <> ToString[Unevaluated[{args}]]},
		testError[msg, args]
	]
	

(* TestExecute is currently a noop *)
TestExecute = Identity


(*TestRequirement[cond_] :=
	BeginTestSection["", cond]

EndRequirement = EndTestSection*)

TestNameTemporary[name_String:"TestTemp"] := ToFileName[{$TemporaryDirectory}, name]


Attributes[OrNTest] = Attributes[NTest]

Options[OrNTest] = Options[NTest]

OrNTest[input_, expected:(_[___]), Optional[preUlps:(_?NumberQ | None), 10^(3+$MachinePrecision-16)], Shortest[messages_:{}], opts___?OptionQ] :=
	Module[{accuracy, precision, tolerance, ulps},
		accuracy = compatOptionValue[AccuracyGoal, {opts}, OrNTest];
		precision = compatOptionValue[PrecisionGoal, {opts}, OrNTest];
		tolerance = compatOptionValue[Tolerance, {opts}, OrNTest];
		(* treat ulps of None same as default, for compatibility *)
		ulps = preUlps /. None -> 10^(3+$MachinePrecision-16);
		If[{accuracy, precision, tolerance} =!= {None, None, None},
			With[{
				(* use accuracy, precision, tolerance *)
				accuracy = accuracy, precision = precision, tolerance = tolerance,
				opts2 = FilterRules[Join[Flatten[{opts}], Options[OrNTest]], Except[FilterRules[Options[OrNTest], Except[Options[Test]]]]]},
				Test[
					input,
					expected,
					messages,
					SameTest -> $OrNTestPATEquivalenceFunction[{accuracy, precision, tolerance}],
					opts2
				]
			]
			,
			(* use ulps *)
			With[{opts2 = FilterRules[Join[Flatten[{opts}], Options[OrNTest]], Except[FilterRules[Options[OrNTest], Except[Options[Test]]]]]},
				Test[
					input,
					expected,
					messages,
					SameTest -> $OrNTestULPSEquivalenceFunction[ulps],
					opts2
				]
			]
		]
	]

OrNTest[args___] :=
	With[{msg = "Incorrect arguments: " <> ToString[Unevaluated[{args}]]},
		testError[msg, args]
	]

$OrNTestPATEquivalenceFunction[{accuracy_, precision_, tolerance_}][actual_, expected_] :=
	TrueQ[Or @@ ($NTestPATEquivalenceFunction[{accuracy, precision, tolerance}][actual, #]& /@ expected)]
	
$OrNTestPATEquivalenceFunction[pat__][args___] :=
	(logNTestFailure[{"Incorrect second arguments in $OrNTestPATEquivalenceFunction[", pat, "][", args, "]"}, "EquivalenceFunction"];
	False);

$OrNTestULPSEquivalenceFunction[ulps_][actual_, expected_] :=
	TrueQ[Or @@ ($NTestULPSEquivalenceFunction[ulps][actual, #]& /@ expected)]

$OrNTestULPSEquivalenceFunction[ulps__][args___] :=
	(logNTestFailure[{"Incorrect second arguments in $OrNTestULPSEquivalenceFunction[", ulps, "][", args, "]"}, "EquivalenceFunction"];
	False);

(* NTestMatch *)

Attributes[NTestMatch] = Attributes[NTest];

Options[NTestMatch] = Options[NTest];

NTestMatch[input_, expected_, Optional[preUlps:(_?NumberQ | None), 10^(3+$MachinePrecision-16)], Shortest[messages_:{}], opts___?OptionQ] :=
	Module[{accuracy, precision, tolerance, ulps},
		accuracy = compatOptionValue[AccuracyGoal, {opts}, NTestMatch];
		precision = compatOptionValue[PrecisionGoal, {opts}, NTestMatch];
		tolerance = compatOptionValue[Tolerance, {opts}, NTestMatch];
		(* treat ulps of None same as default, for compatibility *)
		ulps = preUlps /. None -> 10^(3+$MachinePrecision-16);
		If[{accuracy, precision, tolerance} =!= {None, None, None},
			With[{
				(* use accuracy, precision, tolerance *)
				accuracy = accuracy, precision = precision, tolerance = tolerance,
				opts2 = FilterRules[Join[Flatten[{opts}], Options[NTestMatch]], Except[FilterRules[Options[NTestMatch], Except[Options[Test]]]]]},
				Test[
					input,
					expected,
					messages,
					SameTest -> $NTestMatchPATEquivalenceFunction[{accuracy, precision, tolerance}],
					opts2
				]
			]
			,
			(* use ulps *)
			With[{opts2 = FilterRules[Join[Flatten[{opts}], Options[NTestMatch]], Except[FilterRules[Options[NTestMatch], Except[Options[Test]]]]]},
				Test[
					input,
					expected,
					messages,
					SameTest -> $NTestMatchULPSEquivalenceFunction[ulps],
					opts2
				]
			]
		]
	]


	
$NTestMatchPATEquivalenceFunction[{accuracy_, precision_, tolerance_}][actual_, expected_] :=
    Module[ {res},
        res = matchNumerics[$NTestPATEquivalenceFunction[{accuracy, precision, tolerance}]][actual, expected];
        If[ TrueQ[And @@ Level[res, {-1}]],
            True,
            logNTestFailure[DeleteCases[res, True, Infinity], "MatchNumerics"];
            False
        ]
    ]
    
$NTestMatchPATEquivalenceFunction[pat__][args___] :=
	(logNTestFailure[{"Incorrect second arguments in $NTestMatchPATEquivalenceFunction[", pat, "][", args, "]"}, "EquivalenceFunction"];
	False);

$NTestMatchULPSEquivalenceFunction[ulps_][actual_, expected_] :=
    Module[ {res},
        res = matchNumerics[$NTestULPSEquivalenceFunction[ulps]][actual, expected];
        If[ TrueQ[And @@ Level[res, {-1}]],
            True,
            logNTestFailure[DeleteCases[res, True, Infinity], "MatchNumerics"];
            False
        ]
    ]
    
$NTestMatchULPSEquivalenceFunction[ulps__][args___] :=
	(logNTestFailure[{"Incorrect second arguments in $NTestMatchULPSEquivalenceFunction[", ulps, "][", args, "]"}, "EquivalenceFunction"];
	False);

NTestMatch[args___] :=
	With[{msg = "Incorrect arguments: " <> ToString[Unevaluated[{args}]]},
		testError[msg, args]
	]

(* NTestMatch utility functions *)

(* $patternHeads is a list of heads that comparePatterns will not see through, so that
numerical comparison will not take place in any subexpressions with the head that is listed in
Complement[$patternHeads, $allowNumericPatterns]. 
*)
$patternHeads = {Alternatives, Blank, BlankSequence, BlankNullSequence, 
   DatePattern, Condition, Except, HoldPattern, KeyValuePattern,
   Longest, MissingValuePattern,
   OrderlessPatternSequence, Pattern, PatternTest, PatternSequence, 
   RuleDelayed, Repeated, RepeatedNull, Shortest, Verbatim, Slot, SlotSequence};

$allowNumericPatterns = {Verbatim[Blank[]], BlankSequence, BlankNullSequence, 
						Alternatives, OrderlessPatternSequence, KeyValuePattern,
						PatternSequence, Repeated, RepeatedNull};

patternQ::usage = "\
checks if an expression contains pattern-matching constructs.
patternQ by default checks for all levels of expression.
patternQ[expr, level, {excludedPattern}] checks if expr is a pattern that does not match the excludedPattern at the level.
";

Options[patternQ] = {"ExcludePatterns" -> {}};
(* Special case for Association becasue FreeQ doesn't go into Associations *)
patternQ[expr_Association, level_: {0, Infinity}, opts : OptionsPattern[]] :=
	patternQ[Normal[expr], level, opts]
	
patternQ[tree_?TreeQ, level_ : {0, Infinity}, opts : OptionsPattern[]] :=
	TrueQ[TreeCount[tree, _?(patternQ[#, level, opts] &), level] > 0]
	
patternQ[expr_, level_ : {0, Infinity}, OptionsPattern[]] :=
    Block[{excludedPatterns = OptionValue["ExcludePatterns"]},
    	!FreeQ[Unevaluated[expr], 
      		Alternatives @@ DeleteCases[$patternHeads, Alternatives @@ excludedPatterns], 
      	level]
    ];
	
patternQ[___] := False;

matchNumerics::usage = "\
checks if two expressions match up to a numerical tolerance recursively.
";

matchNumerics[numericalComparisonFunction_][actual_, expected_] :=
	Which[
		MatchQ[actual, expected], True,
		
		(* Special case for Associations. 
		They are raw objects but MatchQ goes into Associations. *)
		
		MatchQ[{actual, expected}, {_Association, _Association}],
			matchNumerics[numericalComparisonFunction][Normal[actual], Normal[expected]],
		
		!patternQ[expected],
			numericalComparisonFunction[actual, expected],
		
		True,
			comparePatterns[numericalComparisonFunction][##]& @@ 
					(Map[ExpressionTree[#, "HeldSubexpressions", Heads -> True]&, {actual, expected}])
	]

$PrintLevel = 0;

dPrint[level_, printStatements___] :=
	If[$PrintLevel >= level, Print[printStatements]]

comparePatterns[numericalComparisonFunction_][actualTree_, expectedTree_] :=
    Module[ {actualExpression = treeExpression[actualTree],
    		expectedExpression = treeExpression[expectedTree],
            numericPositions, patternPositions, expectedWithTolerance,
            subtreeWithTolerance, root
         	},
        dPrint[1, "Current subtrees:\n actual: ", actualTree, "\n expected: ", expectedTree];
        Which[
         (* Try MatchQ first *)
         
         MatchQ[actualExpression, expectedExpression], dPrint[2, "MatchQ passed in comparePatterns"]; True,
             
         (* If the current subexpression is a pattern that is allowed to be compared numerically,
         convert numerical values that appear in the allowed patterns to accept specified 
         AccuraycGoal/PrecisionGoal/tolerance.
		 *)
         
         True,
         	dPrint[2, "Numerical comparison in pattern in comparePatterns, current expectedTree: ", expectedTree];
         	{numericPositions, patternPositions} = separateNumericAndPattern[expectedTree];
         	 
         	(* replace root directly, bug 421240 *)
         	subtreeWithTolerance = Map[Function[{position}, With[{num = TreeData[TreeExtract[expectedTree, position]]},
												_?(numericalComparisonFunction[#1,num] &)]], numericPositions];
			root = First[TreeExtract[expectedTree, 0]]; (* Extract content from HoldForm. Watch out for evaluation *)
			
			expectedWithTolerance = ReplaceAll[With[{replaced = subtreeWithTolerance}, 
										ReplacePart[root, 
											(* Since HeldExpression splits subexpressions into {Head, arg1, ...},
											subtract one to account for the shift. *)
											Thread[(numericPositions - 1) -> replaced]]], 
									(HoldComplete | HoldForm)[expr__] :> expr];
         	dPrint[3, "New expected with tolerance: ", expectedWithTolerance];
         	
         	MatchQ[actualExpression, expectedWithTolerance]
        ]
    ]

(* Helper functions *)

separateNumericAndPattern::usage = 
"separateNumericAndPattern[expr] returns the positions of numbers (that do not appear in any pattern) and patterns";

Options[separateNumericAndPattern] = 
	{"PatternHeads" -> DeleteCases[$patternHeads, 
							(* Blank should allow numerical input *)
							Alternatives @@ Join[$allowNumericPatterns, {Blank}]]};

separateNumericAndPattern[tree_?TreeQ, opts : OptionsPattern[]] :=
	Module[{patternPositions, numericPositions},
		patternPositions = deleteSubpatterns[Reverse[patternPosition[tree, {1, Infinity}, opts]]];
		numericPositions = Quiet[
							(* TreePosition, just like Position, in some cases 
					        (e.g. TreePosition[ExpressionTree[_?(And @@ (constraints /. #1) &), "HeldSubexpressions"], 
					        	_?(! FreeQ[#1, _?NumericQ, 1] &), Infinity]) 
					        will issue messages. *)
							Complement[TreePosition[tree, _?(! FreeQ[#, _?NumericQ, 1] &), Infinity], 
							patternPositions, SameTest -> superpatternQ]];
		{numericPositions, patternPositions}
	]

(* Pattern processing utility functions *) 
 
Options[patternPosition] = Options[separateNumericAndPattern];
patternPosition[patt_, level_ : {-1}, opts : OptionsPattern[]] :=
    Module[{patterns = OptionValue["PatternHeads"]},
    	Switch[
    		patt,
    		(* TreeData are all wrapped in HoldForm.
    		Only returns the position where the root of the tree is a pattern 
    		that does not allow numerical comparison *)
    		_?TreeQ, TreePosition[patt, _?(! FreeQ[#, Alternatives @@ patterns, {1, 2}] &), level],
    		_, Position[patt, Alternatives @@ patterns, level]
    	]
    ]

subpatternQ::usage = "\
subpatternQ[pattPosition1, pattPosition2] checks if pattPosition2 is a subpattern of pattPosition1,
assuming pattPosition1 and pattPosition2 are indices for two subexpressions in the same expression
"

subpatternQ[pattPosition1_, pattPosition2_] :=
    Which[
        (* Subpatterns are deeper in the expression tree, and therefore, 
        have longer position indices *)
		Length[pattPosition1] > Length[pattPosition2], False
		,
		(* a subtree should have common ancestors as the parent expression *)
    	True, SameQ[pattPosition1, Part[pattPosition2, 1 ;; Length[pattPosition1]]]
    ];
    
superpatternQ[pattPosition1_, pattPosition2_] :=
    subpatternQ[pattPosition2, pattPosition1]

deleteSubpatterns[pos_?ListQ] :=
    DeleteDuplicates[pos, subpatternQ]
    
treeExpression[tree_?TreeQ] :=
	TreeExpression[tree, "Subexpressions", Heads -> True] /. (HoldComplete | HoldForm)[content__] :> content


End[]
