

(* hidden feature, used by MUnit loggers to extract informations needs to be repeated here *)

System`VerificationTest
System`TestReport
System`TestResultObject
System`TestReportObject
System`MemoryConstraint
System`$TestFileName

(*

	Testing`$TestEvaluatePostProcessor and Testing`$TestReportPostProcessor are global processors that will be called every time a test run or a test report is produced.
	TestEvaluate is one of the main user of this function and is calling it to process the "raw" result that is returned.
	
	TestReport is blocking Testing`$TestEvaluatePostProcessor to collect all evaluated test results using Reap / Sow.
	MUnit is also blocking the same symbol to collect all VerificationTest evaluations, using a different method.

	This API is strictly internal and temporary, it has been created to slowly re-implement internals incrementally and it will change in the future.
*)


Testing`MessageMatchQ
Testing`TestEvaluate

(* all $ variables that can be blocked and are restored to defaults by TestEvaluate *)

Testing`$DefaultOptions = {}
Testing`$TestEvaluatePostProcessor := System`TestResultObject
Testing`$TestEvaluatePreProcessor := Identity
Testing`$TestReportPostProcessor := System`TestReportObject
Testing`$TestReportPreProcessor := Identity

$TestFileName := ""


(* utilities *)

processData[data_, funcs___] := Fold[Association[#1, #2[#1]] &, data, {funcs}]



(* MessageMatchQ is the equivalence function that determines if a list of 
expected messages (possibly patterns) matches the actual list of thrown messages *)

(*
patternizeExpectedMessage is a utility function for MessageMatchQ that converts the
expectedMessages input to a standardized format which can be used for easier pattern matching.
Needs to be HoldFirst because we do things with the arguments, where they aren't held by Message any more
*)
SetAttributes[patternizeExpectedMessage, HoldFirst]
patternizeExpectedMessage[name_MessageName] := HoldForm[Message[name, ___]]
(*HoldPattern is necessary for the definition, when Message and MessageName are not blocked yet, they are only blocked at runtime*)
patternizeExpectedMessage[HoldPattern[Message][name_, args___]] :=
	With[{args2 = Sequence @@ HoldForm /@ Unevaluated[{args}]},
		With[{m = Message[name, args2]},
			HoldForm[m]
		]
	]
(* If given a fully specified Message wrapped in HoldForm, don't do any further patternizing on it *)
patternizeExpectedMessage[HoldForm[Message[name_,args___]]]:= HoldForm[Message[name,args]];
patternizeExpectedMessage[expr_/;!FreeQ[expr,_MessageName|_Message]]:= patternizeExpectedMessage/@expr
(* let other patterns fall-through *)
patternizeExpectedMessage[arg_] := arg


SetAttributes[MessageMatchQ, HoldAll]
MessageMatchQ[actualMsgs_, expectedMsgs_] :=  
	Module[{expectedMsgs2},
		Block[{Message, MessageName},
			SetAttributes[Message, HoldAll];
			SetAttributes[MessageName, HoldFirst];
			expectedMsgs2 =
				Switch[expectedMsgs,
					_MessageName | _Message,
					{patternizeExpectedMessage[expectedMsgs]}
					,
					_List,
					Map[patternizeExpectedMessage, expectedMsgs]
					,
					_Alternatives,
					Replace[Map[patternizeExpectedMessage, expectedMsgs],arg:Except[_List]:>{arg},{1}]
					,
					_Repeated | _RepeatedNull,
					{patternizeExpectedMessage[expectedMsgs]}
					,
					_,
					expectedMsgs
				];
			MatchQ[actualMsgs, expectedMsgs2]
		]
	]

(* icons *)

getDefaultIcon["Success"] := getDefaultIcon["SuccessIcon"]
getDefaultIcon["Failure"] := getDefaultIcon["TestFailureIcon"]
getDefaultIcon["MessagesFailure"] := getDefaultIcon["MessageFailureIcon"]
getDefaultIcon["Error"] := getDefaultIcon["ErrorIcon"]
getDefaultIcon[icon_] := 
	Graphics[
		{Inset @ Pane[
			Dynamic @ RawBoxes @ FEPrivate`FrontEndResource["MUnitExpressions", icon], 
			ElisionsDump`defaultIconSize, 
			Alignment -> Center
		]}, 
		PlotRange -> {{0, 1}, {0, 1}}, 
		Background -> GrayLevel[0.93], Axes -> False, AspectRatio -> 1,
		ImageSize -> Dynamic@{Automatic, 3.5 CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[Magnification]},
		Frame -> True, 
		FrameTicks -> None, 
		FrameStyle -> Directive[Thickness[Tiny], GrayLevel[0.55]]
	]



