
$testReportProperties = <|
    "Properties" :>
        Function @ Union[
            Keys[#], {
                "TestResultRules",
                "TestsSucceeded",
                "TestsFailedWrongResults",
                "TestsFailedWithMessages",
                "TestsFailedWithErrors",
                "TestsFailed",
                "AllTestsSucceeded",
                "TestsSucceededCount",
                "TestsFailedCount",
                "TestsFailedWrongResultsCount",
                "TestsFailedWithMessagesCount",
                "TestsFailedWithErrorsCount",
                "TestsFailedKeys",
                "TimeElapsed"
            }
        ], 

    (* backward compatibility *)

    "TestsSucceededIndices"          :> Function @ #["TestsSucceededKeys"],
    "TestsFailedWrongResultsIndices" :> Function @ #["TestsFailedWrongResultsKeys"],
    "TestsFailedWithMessagesIndices" :> Function @ #["TestsFailedWithMessagesKeys"],
    "TestsFailedWithErrorsIndices"   :> Function @ #["TestsFailedWithErrorsKeys"],
    "TestsFailedIndices"             :> $testReportProperties["TestsFailedKeys"],

    (* undocumented internal properties *)
    "Outcome" :> 
        Function @ Which[

            Positive @ Length @ #["TestsFailedWithErrorsKeys"],
            "Error",
        
            Positive @ Length @ #["TestsFailedWrongResultsKeys"],
            "Failure",

            Positive @ Length @ #["TestsFailedWithMessagesKeys"],
            "MessagesFailure",
            
            True,
            "Success"
        ],

    "TestsCount" :> 
        Function @ Length @ #["TestResults"],
    "SuccessRate" :> 
        Function @ Replace[
            Length @ #["TestResults"], {
                0 :> 1,
                t_ :> Length[#["TestsSucceededKeys"]] / t
            }
        ],

    (* documented properties *)

    "TestsFailedKeys" -> Function @ Join[
        #["TestsFailedWrongResultsKeys"],
        #["TestsFailedWithMessagesKeys"],
        #["TestsFailedWithErrorsKeys"]
    ],
    "TestsSucceededCount"          -> Function @ Length @ #["TestsSucceededKeys"],
    "TestsFailedCount"             -> Function @ Subtract[Length @ #["TestResults"], Length @ #["TestsSucceededKeys"]],
    "TestsFailedWrongResultsCount" -> Function @ Length @ #["TestsFailedWrongResultsKeys"],
    "TestsFailedWithMessagesCount" -> Function @ Length @ #["TestsFailedWithMessagesKeys"],
    "TestsFailedWithErrorsCount"   -> Function @ Length @ #["TestsFailedWithErrorsKeys"],
    "TimeElapsed"                  -> Function @ Quantity[N @ Total[#[["TestResults", All, 1, "AbsoluteTimeUsed"]]], "Seconds"],
    "TestsSucceeded" :>
        Function[KeyTake[#["TestResults"], #["TestsSucceededKeys"]]],
    "TestsFailed" :>
        Function @ Association[
            "TestsFailedWrongResults" -> KeyTake[#["TestResults"], #["TestsFailedWrongResultsKeys"]], 
            "TestsFailedWithMessages" -> KeyTake[#["TestResults"], #["TestsFailedWithMessagesKeys"]], 
            "TestsFailedWithErrors"   -> KeyTake[#["TestResults"], #["TestsFailedWithErrorsKeys"]]
        ],
    "TestsFailedWrongResults" :>
        Function @ KeyTake[#["TestResults"], #["TestsFailedWrongResultsKeys"]],
    "TestsFailedWithMessages" :>
        Function @ KeyTake[#["TestResults"], #["TestsFailedWithMessagesKeys"]],
    "TestsFailedWithErrors" :>
        Function @ KeyTake[#["TestResults"], #["TestsFailedWithErrorsKeys"]],
    "TestResultRules" :>
        Function @ Association[
            "TestsSucceeded"          -> KeyTake[#["TestResults"], #["TestsSucceededKeys"]], 
            "TestsFailedWrongResults" -> KeyTake[#["TestResults"], #["TestsFailedWrongResultsKeys"]], 
            "TestsFailedWithMessages" -> KeyTake[#["TestResults"], #["TestsFailedWithMessagesKeys"]],
            "TestsFailedWithErrors"   -> KeyTake[#["TestResults"], #["TestsFailedWithErrorsKeys"]]
        ],
    "AllTestsSucceeded" :>
        Function[SameQ[Length @ #["TestResults"], Length @ #["TestsSucceededKeys"]] && !#["Aborted"]],
    "TestResultsDataset" :>
        Function @ Dataset @ KeyDrop[Map[#[All] &, Values[#["TestResults"]]], {"TestKey", "VerificationTest", "AbsoluteTime"}]
|>

(* Querying TestReportObject *)
TestReportObject[x_Association?AssociationQ][property_String] := 
	Lookup[$testReportProperties, property, Function[#[property]]][x]


(* Querying TestReportObject with a list of properties *)
TestReportObject[meta_Association?AssociationQ][props_List]:= 
	AssociationMap[TestReportObject[meta], props]
TestReportObject[meta_Association?AssociationQ][All] :=
	AssociationMap[TestReportObject[meta], TestReportObject[meta]["Properties"]]
TestReportObject[t_TestReportObject] := t



(* SummaryBox code for TestReportObject *)
TestReportObject /: MakeBoxes[tsao: TestReportObject[props_Association?AssociationQ], fmt_] /; BoxForm`UseIcons := 
	BoxForm`ArrangeSummaryBox[
		TestReportObject, 
		tsao, 
		(* icon *)
		getDefaultIcon @ tsao["Outcome"], 
		(* visible grid *)
		{
			{BoxForm`MakeSummaryItem[{"Title: ", tsao["Title"]}, fmt], SpanFromLeft}, 
			{
				BoxForm`MakeSummaryItem[{"Success rate: ", PercentForm[N @ tsao["SuccessRate"]]}, fmt],
				BoxForm`MakeSummaryItem[{"Tests run: ", tsao["TestsCount"]}, fmt]
			}
		}, 
		(* expandable pane *)
		{#, SpanFromLeft} & /@ {
			BoxForm`MakeSummaryItem[{"Succeeded: ", tsao["TestsSucceededCount"]}, fmt], 
			BoxForm`MakeSummaryItem[{"Failed: ", tsao["TestsFailedCount"]}, fmt], 
			BoxForm`MakeSummaryItem[{"Failed with wrong results: ", tsao["TestsFailedWrongResultsCount"]}, fmt],
			BoxForm`MakeSummaryItem[{"Failed with messages: ", tsao["TestsFailedWithMessagesCount"]}, fmt],
			BoxForm`MakeSummaryItem[{"Failed with errors: ", tsao["TestsFailedWithErrorsCount"]}, fmt],
			BoxForm`MakeSummaryItem[{"Time elapsed: ", tsao["TimeElapsed"]}, fmt]
		}, 
		fmt
	]
