

$testResultProperties = Association @ With[
    {
    	toMagnitude = Function @ Replace @ {
			KeyValuePattern[_[#1, HoldPattern[Quantity[val_, #2]]]] :> val,
			KeyValuePattern[_[#1, HoldPattern[val_Quantity]]] :> QuantityMagnitude[val, #2],
			KeyValuePattern[_[#1, val_]] :> val
		},
    	toQuantity = Function @ Replace @ {
			KeyValuePattern[_[#1, HoldPattern[val_Quantity]]] :> val,
			KeyValuePattern[_[#1, val:_Real|_Integer|_Rational]] :> Quantity[val, #2]
		}

	}, {

	    "Properties" :>
	        Function @ Union[
	            Complement[Keys[#], {"SameMessages"}], {
	            	(* documented computed properties goes here *)

	            	"Created",
	            	"VerificationTest"
	            }
	        ],

	    "CPUTimeUsed"             -> toQuantity["CPUTimeUsed", "Seconds"],
	    "AbsoluteTimeUsed"        -> toQuantity["AbsoluteTimeUsed", "Seconds"],
	    "MemoryUsed"              -> toQuantity["MemoryUsed", "Bytes"],


	    "Created"                 -> Function @ TimeZoneConvert[Block[{$TimeZone = "UTC"}, FromAbsoluteTime[#["AbsoluteTime"]]]],

	    "VerificationTest"        -> Function @ Replace[
			Join[
		    	#Input,
		    	#ExpectedOutput,
		    	#ExpectedMessages,
		    	HoldForm[
			    	TestID -> #TestID,
			    	SameTest -> #SameTest,
			    	TimeConstraint -> #TimeConstraint,
			    	MemoryConstraint -> #MemoryConstraint
			    ]
		    ],
		    _[args___] :> HoldForm[VerificationTest[args]]
	    ],

	    (* backward compatibility *)

	    "TestKey" -> Replace @ {
	    	KeyValuePattern[_["TestIndex"|"TestKey", a_]] :> a,
	    	a_ :> Hash[a]
	    },
	    "TestIndex" :> $testResultProperties["TestKey"],

	    (* Undocumented Properties *)

	    "CPUTimeUsedSeconds"      -> toMagnitude["CPUTimeUsed", "Seconds"],
	    "AbsoluteTimeUsedSeconds" -> toMagnitude["AbsoluteTimeUsed", "Seconds"],
	    "MemoryUsedBytes"         -> toMagnitude["MemoryUsed", "Bytes"]

	}
]

(* Querying TestResultObject *)
TestResultObject[x_Association?AssociationQ][property_String] := 
    Lookup[$testResultProperties, property, Function[#[property]]][x]


(* Querying TestResultObject with a list of properties *)
TestResultObject[meta_Association?AssociationQ][props_List]:= 
    AssociationMap[TestResultObject[meta], props]
TestResultObject[meta_Association?AssociationQ][All] :=
    AssociationMap[TestResultObject[meta], TestResultObject[meta]["Properties"]]
TestResultObject[t_TestResultObject] := t

(* Make sure the TestResultObject association contains all necessary keys for typesetting - see bug 364478 *)
SetAttributes[validTROAssoc, HoldFirst]
validTROAssoc[
	KeyValuePattern @ {
		_["Outcome", "Success"|"Failure"|"MessagesFailure"|"Error"], 
		_["Input", _], 
		_["ExpectedOutput", _], 
		_["ActualOutput", _], 
		_["ExpectedMessages", _], 
		_["ActualMessages", _], 
		_["AbsoluteTimeUsed", _], 
		_["TestID", _]
	}
] = True
validTROAssoc[_] := False


clickToCopy[HoldForm[expr_]] := 
	Mouseover[
		HoldForm[expr], 
		Style[
			EventHandler[
				Tooltip[HoldForm[expr], "Copy to clipboard."], 
		  		{"MouseClicked" :> CopyToClipboard[Unevaluated[expr]]}
	  		], 
	  		Background -> LightGray
	  	]
	];
clickToCopy[expr_] := 
	clickToCopy[HoldForm[expr]]

(* SummaryBox code for TestResultObject *)
TestResultObject /: MakeBoxes[tso:TestResultObject[props_?validTROAssoc], fmt_] /; BoxForm`UseIcons := 
	BoxForm`ArrangeSummaryBox[
		TestResultObject, 
		tso, 
		(* icon *)
		getDefaultIcon @ props["Outcome"], 
		(* visible informations *)
		{
			BoxForm`MakeSummaryItem[{"Outcome: ", props["Outcome"]}, fmt], 
			BoxForm`MakeSummaryItem[{"Test ID: ", clickToCopy@props["TestID"]}, fmt]
		}, 
		(* expandable pane *)
		Join[
			{
				BoxForm`MakeSummaryItem[{"Input: ", ElisionsDump`expandablePane[Short[clickToCopy@props["Input"],2/3]]}, fmt],
				BoxForm`MakeSummaryItem[{"Expected output: ", ElisionsDump`expandablePane[Short[clickToCopy@props["ExpectedOutput"],2/3]]}, fmt], 
				BoxForm`MakeSummaryItem[{"Actual output: ", ElisionsDump`expandablePane[Short[clickToCopy@props["ActualOutput"],2/3]]}, fmt]
			},
			If[
				MatchQ[props, KeyValuePattern[{_["ExpectedMessages", {}|HoldForm[{}]], _["ActualMessages", {}]}]],
				{},
				{
					BoxForm`MakeSummaryItem[{"Expected messages: ", ElisionsDump`expandablePane[Short[clickToCopy@props["ExpectedMessages"],2/3]]}, fmt],
					BoxForm`MakeSummaryItem[{"Actual messages: ", ElisionsDump`expandablePane[Short[clickToCopy@props["ActualMessages"],2/3]]}, fmt]
				}
			]
		], 
		fmt
	]