
(*
 Package to debugging for gridMathematica
*)


BeginPackage[ "MEET`ParallelDebug`", {"MEET`ParallelProfiler`"}]

Needs[ "Parallel`"]

DebuggerLink::usage = ""

Begin[ "`Private`"]

(* PCT backward compatibility *)
If[ Parallel`Information`$VersionNumber < 3.0,
	Parallel`Protected`AppendInitCode = Symbol["Parallel`Parallel`Private`AppendInitCode"];
	Parallel`Developer`KernelID = Symbol["Parallel`Parallel`ProcessorID"];
	Parallel`$KernelID = Symbol["Parallel`Parallel`$ProcessorID"];
	Parallel`ParallelizeValues = Symbol["Parallel`Parallel`ExportEnvironment"];
	MEET`ParallelDebug`Private`ParallelEvaluate = Symbol["Parallel`Parallel`RemoteEvaluate"];
	, (* work around incompatible change *)
	MEET`ParallelDebug`Private`ParallelEvaluate = Symbol["Parallel`ParallelEvaluate"];
]

(*
 Make sure that slaves are launched in debug mode.
*)

If[ "RuntimeAnalysisTools" /. RuntimeTools`ExecutionState["RuntimeAnalysisTools"],
	Parallel`Protected`AppendInitCode[ Parallel`Client`HoldCompound[ RuntimeTools`DebugOn[True]], True]
]

uninitialized = True;

InitializeKernels[] :=
	If[ uninitialized === True,
		uninitialized = False;
		Block[ {Parallel`Debug`Private`ExportEnvironmentWrapper = Identity},
			ParallelizeDefinitions[ "MEET`*"] ;
			ParallelizeDefinitions[ "JLink`*"] ;
		];
	]


(*
 Should attach to just one slave, but not so useful, 
 since ParallelizeDefinitions can't write to just one slave.
*)


MEET`Utility`LinkCreateOnWB = False


AttachSlave[ slave_] :=
	Module[{remoteSession, name, linkArgs},
		InitializeKernels[];
		remoteSession = ToString[ MEET`ParallelDebug`Private`ParallelEvaluate[$SessionID, slave]];
		slaveName = "Slave-" <> ToString[ Parallel`Developer`KernelID[slave]];
		name = Part[MEET`ParallelDebug`Private`ParallelEvaluate[ DebuggerLink = LinkCreate[LinkProtocol->"TCPIP"], slave], 1] ;
		linkArgs = " -linkmode connect -linkname " <> name ;
		MEET`ParallelDebug`Private`ParallelEvaluate[
				MathLink`AddSharingLink[ DebuggerLink, 
					AllowPreemptive->True,Terminating->True],slave];
		MEET`Utility`WorkbenchHandler@attachKernel[ remoteSession, slaveName, linkArgs, "TCPIP", MEET`Utility`LinkCreateOnWB];
		MEET`ParallelDebug`Private`ParallelEvaluate[MEET`SetOutputHandler[ Null], slave];
		slave
	]

Parallel`Debug`Private`RemoteKernelInit = AttachSlave

(*
 The apply of the head is because the definitions in 
 expr are wrapped in a holding function.  This needs 
 to be reapplied before ProcessTags is inserted.
*)

addTags[expr_] := Module[{ef, h},
    h = Head[expr];
    ef = RuntimeTools`AddTags[expr];
    ef = h @@ ef;
    ef = Map[RuntimeTools`ProcessTags, ef, {1}];
    ef
    ]


Parallel`Debug`Private`ExportEnvironmentWrapper = addTags


End[]


EndPackage[]

