(* ::Package:: *)

(* ::Title:: *)
(*Buttons*)

(* ::Section:: *)
(*Information*)

`Information`$ButtonsId = "$Id: Buttons.m,v 1.1 2009/11/09 14:43:55 brenton Exp $"

(* ::Section:: *)
(*Package*)

Begin["`Package`"]

jumpToTestButtonBox

replaceOutputButtonBox

replaceMessagesButtonBox

rerunTestButtonBox

End[]

(* ::Section:: *)
(*Private*)

Begin["`Buttons`Private`"]

jumpToTestButtonBox[nb_NotebookObject, tr_?TestResultQ] :=
	ButtonBox["\<\"Jump to Test\"\>",
		ButtonFunction:>jumpToTestButtonFunction[nb, tr],
		Appearance->Automatic,
		ButtonFrame->"DialogBox",
		Evaluator->Automatic,
		Method->"Preemptive"
	]

(*
nbStale references the (possibly stale) main link, but jumpToTestButtonFunction is being run
on the preemptive link
*)
jumpToTestButtonFunction[nbStale_NotebookObject, tr_?TestResultQ] :=
	Module[{testID = ToExpression["36^^" <> StringTake[TestID[tr], -7]], found, nb},
		Catch[
			(*
			nb is the nb object, but referenced on the preemptive link
			here, $FrontEnd evaluates using the preemptive link
			*)
			nb = nbStale /. _FrontEndObject -> $FrontEnd;
			If[!MemberQ[Notebooks[], nb],
				Throw[nb,
					MUnitErrorTag[{"Test Cell", "Invalid NotebookObject"}]
				]
			];
			SetSelectedNotebook[nb];
			found = NotebookFind[nb, testID, All, CellID];
			If[found === $Failed,
				Throw[HoldForm[NotebookFind][nb, testID, All, CellID],
					MUnitErrorTag[{"Test Cell", "Cannot find CellID. NotebookFind returned $Failed"}]
				]
			]
			,
			_MUnitErrorTag
			,
			Function[{value, tag},
				MessageDialog[Column[{tag[[1,1]], Style[tag[[1,2]], Bold], value}], WindowTitle->"MUnit Error"]
			]	
		]
	]

replaceOutputButtonBox[nb_NotebookObject, tr_?TestResultQ] :=
	ButtonBox["\<\"Replace ExpectedOutput with ActualOutput\"\>",
		ButtonFunction :> replaceOutputButtonFunction[nb, tr],							
		Appearance->Automatic,
		ButtonFrame->"DialogBox",
		Evaluator->Automatic,
		Method->"Queued"
	]

replaceOutputButtonFunction[nbStale_NotebookObject, tr_?TestResultQ] :=
	Module[{testID = ToExpression["36^^" <> StringTake[TestID[tr], -7]], info, grouped, read, cases, found, nb},
		Catch[
			nb = nbStale /. _FrontEndObject -> $FrontEnd;
			If[!MemberQ[Notebooks[], nb],
				Throw[nb,
					MUnitErrorTag[{"Test Cell", "Invalid NotebookObject"}]
				]
			];
			(*SetSelectedNotebook[nbMain];*)
			found = NotebookFind[nb, testID, All, CellID];
			If[found === $Failed,
				Throw[HoldForm[NotebookFind][nb, testID, All, CellID],
					MUnitErrorTag[{"Test Cell", "Cannot find CellID. NotebookFind returned $Failed"}]
				]
			];
			info = Developer`CellInformation[nb];
			grouped = "FirstCellInGroup" /. info[[1]];
			If[grouped,
				SelectionMove[nb, All, CellGroup];
				(* if  grouped, then have to dig around a little *)
				read = NotebookRead[nb];
				cases = Cases[read[[1, 1]], Cell[_, "ExpectedOutput", ___, CellID -> cellID_, ___] :> cellID];
				If[cases == {},
					(* no expected output cell, do something else *)
					cases = Cases[read[[1, 1]], Cell[_, "ExpectedMessages", ___, CellID -> cellID_, ___] :> cellID];
					If[cases == {},
						cases = Cases[read[[1, 1]], Cell[_, "ExpectedMessage", ___, CellID -> cellID_, ___] :> cellID];
						If[cases == {},
							(* no expected messages, do something else *)
							cases = Cases[read[[1, 1]], Cell[_, "TestOptions", ___, CellID -> cellID_, ___] :> cellID];
							If[cases == {},
								(* no test options, insert after test cell *)
								NotebookFind[nb, testID, All, CellID];
								SelectionMove[nb, After, Cell]
								,
								(* insert after test options cell *)
								NotebookFind[nb, cases[[1]], All, CellID];
								SelectionMove[nb, After, Cell]
							]
							,
							(* there are "ExpectedMessage" cells *)
							(* find the first expected message, and iterate until all expected message cells are selected *)
							NotebookFind[nb, cases[[1]], All, CellID];
							Do[FrontEndExecute[FrontEndToken[nb, "SelectNextCell"]], {Length[cases]-1}]
						]
						,
						(* insert after expected messages cell *)
						NotebookFind[nb, cases[[1]], All, CellID];
						SelectionMove[nb, After, Cell]
					]
					,
					(* expected output cell, select it to overwrite *)
					NotebookFind[nb, cases[[1]], All, CellID]
				]
				,
				(* only a test cell, so just insert expected output after *)
				SelectionMove[nb, After, Cell]
			];
			
			NotebookWrite[nb, Cell[BoxData[Function[{x}, ToBoxes[Unevaluated[x]], {HoldFirst}] @@ ActualOutput[tr]], "ExpectedOutput"]];
			
			(* palce cursor after cellgroup *)
			SelectionMove[nb, All, CellGroup];
			SelectionMove[nb, After, CellGroup];
			,
			_MUnitErrorTag
			,
			Function[{value, tag}, MessageDialog[Column[{tag[[1,1]], Style[tag[[1,2]], Bold], value}], WindowTitle->"MUnit Error"]]
		]
	]

replaceMessagesButtonBox[nb_NotebookObject, tr_?TestResultQ] :=
	ButtonBox["\<\"Replace ExpectedMessages with ActualMessages\"\>",
		ButtonFunction :> replaceMessagesButtonFunction[nb, tr],
		Appearance->Automatic,
		ButtonFrame->"DialogBox",
		Evaluator->Automatic,
		Method->"Queued"
	]

replaceMessagesButtonFunction[nbStale_NotebookObject, tr_?TestResultQ] :=
	Module[{testID = ToExpression["36^^" <> StringTake[TestID[tr], -7]], info, grouped, read, cases, found, nb},
		Catch[
			nb = nbStale /. _FrontEndObject -> $FrontEnd;
			If[!MemberQ[Notebooks[], nb],
				Throw[nb,
					MUnitErrorTag[{"Test Cell", "Invalid NotebookObject"}]
				]
			];
			(*SetSelectedNotebook[nbMain];*)
			found = NotebookFind[nb, testID, All, CellID];
			If[found === $Failed,
				Throw[HoldForm[NotebookFind][nb, testID, All, CellID],
					MUnitErrorTag[{"Test Cell", "Cannot find CellID. NotebookFind returned $Failed"}]
				]
			];
			info = Developer`CellInformation[nb];
			grouped = "FirstCellInGroup" /. info[[1]];
			If[grouped,
				SelectionMove[nb, All, CellGroup];
				(* if grouped, then have to dig around a little *)
				read = NotebookRead[nb];
				cases = Cases[read[[1, 1]], Cell[_, "ExpectedMessages", ___, CellID -> cellID_, ___] :> cellID];
				If[cases == {},
					(* possibly there are "ExpectedMessage" cells *)
					cases = Cases[read[[1, 1]], Cell[_, "ExpectedMessage", ___, CellID -> cellID_, ___] :> cellID];
					If[cases == {},
						(* no expected messages cell or expected message cells, do something else *)
						cases = Cases[read[[1, 1]], Cell[_, "TestOptions", ___, CellID -> cellID_, ___] :> cellID];
						If[cases == {},
							(* no test options, insert after test cell *)
							NotebookFind[nb, testID, All, CellID];
							SelectionMove[nb, After, Cell]
							,
							(* insert after test options cell *)
							NotebookFind[nb, cases[[1]], All, CellID];
							SelectionMove[nb, After, Cell]
						]
						,
						(* there are "ExpectedMessage" cells *)
						(* find the first expected message, and iterate until all expected message cells are selected *)
						NotebookFind[nb, cases[[1]], All, CellID];
						Do[FrontEndExecute[FrontEndToken[nb, "SelectNextCell"]], {Length[cases]-1}]
					]
					,
					(* expected messages cell, select it to overwrite *)
					NotebookFind[nb, cases[[1]], All, CellID]
				]
				,
				(* only a test cell, so just insert expected messages after *)
				SelectionMove[nb, After, Cell]
			];
			
			NotebookWrite[nb, Cell[BoxData[ToBoxes[ActualMessages[tr]] //. TagBox[msg_, HoldForm] :> msg], "ExpectedMessages"]];
			
			(* palce cursor after cellgroup *)
			SelectionMove[nb, All, CellGroup];
			SelectionMove[nb, After, CellGroup];
			,
			_MUnitErrorTag
			,
			Function[{value, tag}, MessageDialog[Column[{tag[[1,1]], Style[tag[[1,2]], Bold], value}], WindowTitle->"MUnit Error"]]
		]
	]

rerunTestButtonBox[nb_NotebookObject, tr_?TestResultQ] :=
	ButtonBox["\<\"Rerun Test\"\>",
		ButtonFunction :> rerunTestButtonFunction[nb, tr],
		Appearance->Automatic,
		ButtonFrame->"DialogBox",
		Evaluator->Automatic,
		Method->"Queued"
	]

rerunTestButtonFunction[nbStale_NotebookObject, tr_?TestResultQ] :=
	Module[{testid = ToExpression["36^^" <> StringTake[TestID[tr], -7]], found, nb},
		Catch[
			nb = nbStale /. _FrontEndObject -> $FrontEnd;
			If[!MemberQ[Notebooks[], nb],
				Throw[nb,
					MUnitErrorTag[{"Test Cell", "Invalid NotebookObject"}]
				]
			];
			found = NotebookFind[nb, testid, All, CellID];
			If[found === $Failed,
				Throw[HoldForm[NotebookFind][nb, testid, All, CellID],
					MUnitErrorTag[{"Test Cell", "Cannot find CellID. NotebookFind returned $Failed"}]
				]
			];
			(* setup precondition for cellEvaluationFunction *)
			SelectionMove[nb, All, CellGroup];
			(* we know grouped is True because this is being run from a TestResult cell *)
			cellEvaluationFunction[nb, True]
			,
			_MUnitErrorTag
			,
			Function[{value, tag},
				MessageDialog[Column[{tag[[1,1]], Style[tag[[1,2]], Bold], value}], WindowTitle->"MUnit Error"]
			]
		]
	]

End[]
