(* ::Package:: *)

(* ::Title:: *)
(*Loggers*)

(* ::Section:: *)
(*Information*)

`Information`$LoggersId = "$Id: Loggers.m,v 1.22 2009/12/08 17:17:33 brenton Exp $"

(* ::Section:: *)
(*Public*)

PrintLogger::usage =
"PrintLogger[] is an MUnit logger that writes a simple form of test output to $Output."

NotebookLogger::usage =
"NotebookLogger[] is an MUnit logger that writes test results to a notebook."

BatchPrintLogger::usage =
"BatchPrintLogger[] is an MUnit logger that writes batch test results to $Output."

VerbosePrintLogger::usage =
"VerbosePrintLogger[] is an MUnit logger that writes verbose test results to $Output."


LogStart::usage =
"LogStart[logger, title] "

LogTestRunProgress::usage =
"LogTestRunProgress[logger, i, range] logs the progress of the test run."

LogMessage::usage =
"LogMessage[logger, string] logs a message."

LogSuccess::usage =
"LogSuccess[logger, tr] logs a test success."

LogFailure::usage =
"LogFailure[logger, tr] logs a test failure."

LogMessagesFailure::usage =
"LogMessagesFailure[logger, tr] logs a test messages failure."

LogError::usage =
"LogError[logger, tr] logs a test error."

LogTestID::usage =
"LogTestID[logger, testid] logs a test id before a test is run."

LogFatal::usage =
"LogFatal[logger, string] logs a fatal error."

LogBeginTestSection::usage =
"LogBeginTestSection[logger, section, depth] logs the start of a section."

LogEndTestSection::usage =
"LogEndTestSection[logger] logs the end of a section."

LogCPUTimeUsed::usage =
"LogCPUTimeUsed[logger, time] logs the amount of CPU time used by the test run."

LogAbsoluteTimeUsed::usage =
"LogAbsoluteTimeUsed[logger, time] logs the amount of absolute time used by the test run."

LogMemoryUsed::usage =
"LogMemoryUsed[logger, mem] logs the amount of memory used by the test run."

LogExpressionCount::usage =
"LogExpressionCount[logger, i] logs the number of expressions (including tests) in a test run."

LogEnd::usage =
"LogEnd[logger, tr] logs the end of a test run."

(* ::Section:: *)
(*Package*)

Begin["`Package`"]

End[]

(* ::Section:: *)
(*Private*)

Begin["`Loggers`Private`"]

PrintLogger[] :=
	With[{logger = Unique["MUnit`Loggers`Private`logger"]},
		logger /: LogStart[logger, title_] :=
		If[title === None,
			WriteString[$Output, "Starting Test Run\n"],
			WriteString[$Output, "Starting test run \"" <> title <> "\"\n"]
		];
		logger /: LogBeginTestSection[logger, section_, (*require*)_] :=
			WriteString[$Output, "Starting test section ", section, "\n"];
		logger /: LogEndTestSection[logger] :=
			WriteString[$Output, "\nEnding test section\n"];
		logger /: LogExpressionCount[logger, n_Integer] :=
			WriteString[$Output, "\nTest run had " <> ToString[n, OutputForm] <> If[n == 1, " expression", " expressions"] <> "\n"];
		logger /: LogMessage[logger, msg_String] :=
			WriteString[$Output, "\n" <> msg <> "\n"];
		logger /: LogFatal[logger, msg_String] :=
			WriteString[$Output, "\n" <> msg <> "\n"];
		logger /: LogSuccess[logger, (*tr*)_?TestResultQ] :=
			WriteString[$Output, "."];
		logger /: LogFailure[logger, tr_?TestResultQ] :=
			Module[{msg = TestFailureMessage[tr]},
				WriteString[$Output, "!"];
				If[msg != "", WriteString[$Output, "\n** " <> msg <> " **\n"]]
			];
		logger /: LogMessagesFailure[logger, (*tr*)_?TestResultQ] :=
			WriteString[$Output, "*"];
		logger /: LogError[logger, tr_] :=
			Module[{msg = ErrorMessage[tr]},
				WriteString[$Output, "!"];
				If[msg != "", WriteString[$Output, "\n** " <> msg <> " **\n"]]
			];
		logger /: LogEnd[logger, testCnt_, (*successCnt*)_, failCnt_, msgFailCnt_, secSkippedCnt_, errorCnt_, (*abort*)_] :=
			(
				If[$abort, WriteString[$Output, "Test run stopped before completion.\n"]];
				(*
				TODO: Why does "\n\n" work, but not "\n"?
				WriteString[$Output, "\n"];
				*)
				WriteString[$Output, "\n\n"];
				WriteString[$Output, StringForm["Tests run: `1`,  Failures: `2`,  Messages Failures: `3`,  Sections skipped: `4`, Errors: `5`", testCnt, failCnt, msgFailCnt, secSkippedCnt, errorCnt]]
			);
		logger /: Format[logger, StandardForm] :=
			Interpretation[Row[{RawBoxes["\[SkeletonIndicator]"], RawBoxes["Print Logger"], RawBoxes["\[SkeletonIndicator]"]}], logger];
		logger /: Format[logger, OutputForm] :=
			"-Print Logger-";
		logger
	]

testResultCell[testNb_NotebookObject, tr_?TestResultQ, "TestSuccess", opts___] :=
	Cell[BoxData[RowBox[{PaneBox[ToBoxes[tr], ImageSize->250], jumpToTestButtonBox[testNb, tr]}]], "TestSuccess", opts]

testResultCell[testNb_NotebookObject, tr_?TestResultQ, "TestFailure", opts___] :=
	Cell[BoxData[RowBox[{PaneBox[ToBoxes[tr], ImageSize->250], jumpToTestButtonBox[testNb, tr], replaceOutputButtonBox[testNb, tr]}]], "TestFailure", opts]
	
testResultCell[testNb_NotebookObject, tr_?TestResultQ, "TestMessagesFailure", opts___] :=
	Cell[BoxData[RowBox[{PaneBox[ToBoxes[tr], ImageSize->250], jumpToTestButtonBox[testNb, tr], replaceMessagesButtonBox[testNb, tr]}]], "TestMessagesFailure", opts]
	
testResultCell[testNb_NotebookObject, tr_?TestResultQ, "TestError", opts___] :=
	Cell[BoxData[RowBox[{PaneBox[ToBoxes[tr], ImageSize->250], jumpToTestButtonBox[testNb, tr]}]], "TestError", opts]

depthToSectionStyle[depth_Integer] := depth /. {1 -> "Title", 2 -> "Section", 3 -> "Subsection", 4 -> "Subsubsection", 5 -> "Subsubsubsection", _ -> "Unknownsection"}

sectionStyleToDepth[style_String] := style /. {"Title" -> 1, "Section" -> 2, "Subsection" -> 3, "Subsubsection" -> 4, "Subsubsubsection" -> 5, _ -> $UnknownDepth}

Attributes[initializeNotebook] = {HoldFirst}

initializeNotebook[nb_, title_] :=
	If[Head[nb] =!= NotebookObject,
		nb =
			CreateDocument[
				{},
				WindowTitle -> If[title === None, "MUnit Test Run Results", title],
				StyleDefinitions -> FrontEnd`FileName[{"MUnit"}, "TestResults.nb"],
				Saveable->False,
				(* to accomodate the failure mode pane and the buttons *)
				WindowSize -> {Large, Automatic}
				(* ensure that the test results notebook does not take focus away from the InputNotebook[]
				during the test run, since there are hooks using InputNotebook[] *)
				(*WindowClickSelect -> False*)
			]
	]

(*
testNb is the test notebook object
*)
NotebookLogger[testNb_NotebookObject] :=
	With[{logger = Unique["MUnit`Loggers`Private`nblogger"], resNb = Unique["MUnit`Loggers`Private`nb"]},
		Module[{$depth = 0},
			logger /: LogStart[logger, title_] :=
				Developer`UseFrontEnd[
					initializeNotebook[resNb, title]
				];
			logger /: LogMessage[logger, msg_String] :=
				Developer`UseFrontEnd[
					NotebookWrite[resNb, Cell[msg, "TestMessage"]]
				];
			logger /: LogSuccess[logger, tr_?TestResultQ] :=
				Developer`UseFrontEnd[
					NotebookWrite[resNb, testResultCell[testNb, tr, "TestSuccess", CellTags -> TestTags[tr]]]
				];
			logger /: LogFailure[logger, tr_?TestResultQ] :=
				Developer`UseFrontEnd[
					NotebookWrite[resNb, testResultCell[testNb, tr, "TestFailure", CellTags -> TestTags[tr]]]
				];
			logger /: LogMessagesFailure[logger, tr_?TestResultQ] :=
				Developer`UseFrontEnd[
					NotebookWrite[resNb, testResultCell[testNb, tr, "TestMessagesFailure", CellTags -> TestTags[tr]]]
				];
			logger /: LogError[logger, tr_?TestResultQ] :=
				Developer`UseFrontEnd[
					NotebookWrite[resNb, testResultCell[testNb, tr, "TestError"]]
				];
			logger /: LogFatal[logger, msg_String] :=
				Developer`UseFrontEnd[
					NotebookWrite[resNb, Cell[msg, "TestFatal"]]
				];
			logger /: LogBeginTestSection[logger, section_, require:True|False] :=
				Developer`UseFrontEnd[
					$depth++;
					(*If[$depth == 1,
						SetOptions[nb, WindowTitle -> "MUnit Test Run Results for " <> section]
					];*)
					NotebookWrite[resNb, Cell[If[require, section, TextData[StyleBox[section, FontVariations->{"StrikeThrough"->True}]]], depthToSectionStyle[$depth]]]
				];
			(* LogEndTestSection goes back up the notebook and groups all of the cells *)
			logger /: LogEndTestSection[logger] :=
				Module[{(*info*)},
					Developer`UseFrontEnd[
						(*While[(info = Developer`CellInformation[nb]) =!= $Failed && ("Style" /. info[[1]]) != depthToSectionStyle[$depth],
							FrontEndTokenExecute[nb, "SelectPreviousCell"]
						];
						$depth--;
						FrontEndTokenExecute[nb, "CellGroup"];
						SelectionMove[nb, After, Notebook]*)
						$depth--;
					]
				];
			logger /: LogExpressionCount[logger, n_Integer] :=
				Developer`UseFrontEnd[
					NotebookWrite[resNb, Cell["Test run had " <> ToString[n, OutputForm] <> If[n == 1, " expression", " expressions"], "TestMessage"]]
				];
			logger /: LogCPUTimeUsed[logger, time_] :=
				Developer`UseFrontEnd[
					NotebookWrite[resNb, Cell[TextData[{"Test run used ", Cell[BoxData[ToString[time, InputForm]], "Output"], " seconds of CPU time"}], "TestMessage"]]
				];
			logger /: LogAbsoluteTimeUsed[logger, time_] :=
				Developer`UseFrontEnd[
					NotebookWrite[resNb, Cell[TextData[{"Test run used ", Cell[BoxData[ToString[time, InputForm]], "Output"], " seconds of absolute time"}], "TestMessage"]]
				];
			logger /: LogMemoryUsed[logger, mem_] :=
				Developer`UseFrontEnd[
					NotebookWrite[resNb, Cell["Test run used " <> ToString[mem, OutputForm] <> " bytes of memory", "TestMessage"]]
				];
			logger /: LogEnd[logger, testCnt_, successCnt_, failCnt_, msgFailCnt_, secSkippedCnt_, errorCnt_, abortOrFatal_] :=
				Developer`UseFrontEnd[
					NotebookWrite[resNb,
						Cell[ToString[StringForm["Tests run: `1`,  Failures: `2`,  Messages Failures: `3`,  Sections skipped: `4`, Errors: `5`", testCnt, failCnt, msgFailCnt, secSkippedCnt, errorCnt], OutputForm],
							If[successCnt == testCnt && errorCnt == 0 && !abortOrFatal, "TestRunSuccess", "TestRunFailure"]]];
					(* restore clickability after test run *)
					(*SetOptions[resNb, WindowClickSelect -> True]*)
				];
			logger /: Format[logger, StandardForm] :=
				Interpretation[Row[{RawBoxes["\[SkeletonIndicator]"], RawBoxes["Notebook Logger"], RawBoxes["\[SkeletonIndicator]"]}], logger];
			logger /: Format[logger, OutputForm] :=
				"-Notebook Logger-";
			logger
		]
	]


BatchPrintLogger[] :=
	With[{logger = Unique["MUnit`Loggers`Private`logger"], printLogger = PrintLogger[]},
		Module[{$batchLength = 20, doBatchOutput, batchString = ""},
			doBatchOutput[force_:False] :=
				If[force || StringLength[batchString] >= $batchLength,
					WriteString[$Output, batchString];
					batchString = ""
				];
			logger /: LogStart[logger, title_] :=
				(
					LogStart[printLogger, title];
					batchString = ""
				);
			logger /: LogBeginTestSection[logger, section_, (*require*)_] :=
				(
					batchString = batchString <> "Starting test section " <> section <> "\n";
					doBatchOutput[True]
				);
			logger /: LogEndTestSection[logger] :=
				(
					batchString = batchString <> "\nEnding test section\n";
					doBatchOutput[True]
				);
			logger /: LogExpressionCount[logger, n_Integer] :=
				(
					batchString = batchString <> "\nTest run had " <> ToString[n, OutputForm] <> If[n == 1, " expression", " expressions"] <> "\n";
					doBatchOutput[True]
				);
			logger /: LogMessage[logger, msg_String] :=
				(
					batchString = batchString <> "\n" <> msg <> "\n";
					doBatchOutput[True]
				);
			logger /: LogFatal[logger, msg_String] :=
				(
					batchString = batchString <> "\n" <> msg <> "\n";
					doBatchOutput[True]
				);
			logger /: LogSuccess[logger, _?TestResultQ] :=
				(
					batchString = batchString <> ".";
					doBatchOutput[]
				);
			logger /: LogFailure[logger, tr_?TestResultQ] :=
				Module[{msg = TestFailureMessage[tr]},
					batchString = batchString <> "!" <> If[msg != "", "\n** " <> msg <> " **\n", ""];
					doBatchOutput[]
				];
			logger /: LogMessagesFailure[logger, _?TestResultQ] :=
				(
					batchString = batchString <> "*";
					doBatchOutput[]
				);
			logger /: LogError[logger, tr_?TestResultQ] :=
				Module[{msg = ErrorMessage[tr]},
					batchString = batchString <> "!" <> If[msg != "", "\n** " <> msg <> " **\n", ""];
					doBatchOutput[]
				];
			logger /: LogEnd[logger, testCnt_, successCnt_, failCnt_, msgFailCnt_, secSkippedCnt_, errorCnt_, abort_] :=
				(
					doBatchOutput[True];
					LogEnd[printLogger, testCnt, successCnt, failCnt, msgFailCnt, secSkippedCnt, errorCnt, abort]
				);
			logger /: Format[logger, StandardForm] :=
				Interpretation[Row[{RawBoxes["\[SkeletonIndicator]"], RawBoxes["Batch Print Logger"], RawBoxes["\[SkeletonIndicator]"]}], logger];
			logger /: Format[logger, OutputForm] :=
				"-Batch Print Logger-";
			logger
		]
	]




VerbosePrintLogger[] :=
	With[{logger = Unique["MUnit`Loggers`Private`logger"]},
		logger /: LogStart[logger, title_] :=
		If[title === None,
			WriteString[$Output, "Starting Test Run\n"],
			WriteString[$Output, "Starting test run \"" <> title <> "\"\n"]
		];
		logger /: LogBeginTestSection[logger, section_, (*require*)_] :=
			WriteString[$Output, "Starting test section ", section, "\n"];
		logger /: LogEndTestSection[logger] :=
			WriteString[$Output, "\nEnding test section\n"];
		logger /: LogExpressionCount[logger, n_Integer] :=
			WriteString[$Output, "\nTest run had " <> ToString[n, OutputForm] <> If[n == 1, " expression", " expressions"] <> "\n"];
		logger /: LogMessage[logger, msg_String] :=
			WriteString[$Output, "\n" <> msg <> "\n"];
		logger /: LogFatal[logger, msg_String] :=
			WriteString[$Output, "\n" <> msg <> "\n"];
		logger /: LogSuccess[logger, (*tr*)_?TestResultQ] :=
			WriteString[$Output, "."];
		logger /: LogFailure[logger, tr_?TestResultQ] :=
			Module[{msg = TestFailureMessage[tr]},
				WriteString[$Output, "!\n"];
				WriteString[$Output, "Test number " <> ToString[TestIndex[tr], OutputForm] <> " with TestID " <> ToString[TestID[tr], OutputForm] <> " had a failure.\n"];
				WriteString[$Output, "\tInput: " <> ToString[TestInput[tr], InputForm] <> "\n"];
				WriteString[$Output, "\tExpected output: " <> ToString[ExpectedOutput[tr], InputForm] <> "\n"];
				WriteString[$Output, "\tActual output: " <> ToString[ActualOutput[tr], InputForm] <> "\n"];
				If[msg =!= "", WriteString[$Output, "\n** " <> msg <> " **\n"]]
			];
		logger /: LogMessagesFailure[logger, tr_?TestResultQ] :=
			(
				WriteString[$Output, "*\n"];
				WriteString[$Output, "Test number " <> ToString[TestIndex[tr], OutputForm] <> " with TestID " <> ToString[TestID[tr], OutputForm] <> " had a messages failure.\n"];
				WriteString[$Output, "\tInput: " <> ToString[TestInput[tr], InputForm] <> "\n"];
				WriteString[$Output, "\tExpected messages: " <> ToString[ExpectedMessages[tr], InputForm] <> "\n"];
				WriteString[$Output, "\tActual messages: " <> ToString[ActualMessages[tr], InputForm] <> "\n"];
				If[TestFailureMessage[tr] =!= "", WriteString[$Output, "\n** " <> TestFailureMessage[tr] <> " **\n"]]
			);
		logger /: LogError[logger, tr_?TestResultQ] :=
			Module[{msg = ErrorMessage[tr]},
				WriteString[$Output, "!\n"];
				WriteString[$Output, "Test number " <> ToString[TestIndex[tr], OutputForm] <> " with TestID " <> ToString[TestID[tr], OutputForm] <> " had an error.\n"];
				If[msg =!= "", WriteString[$Output, "\n** " <> msg <> " **\n"]]
			];
		logger /: LogEnd[logger, testCnt_, (*successCnt*)_, failCnt_, msgFailCnt_, secSkippedCnt_, errorCnt_, (*abort*)_] :=
			(
				WriteString[$Output, "\n\n"];
				If[$abort, WriteString[$Output, "Test run stopped before completion.\n"]];
				WriteString[$Output, StringForm["Tests run: `1`,  Failures: `2`,  Messages Failures: `3`,  Sections skipped: `4`, Errors: `5`", testCnt, failCnt, msgFailCnt, secSkippedCnt, errorCnt]]
			);
		logger /: Format[logger, StandardForm] :=
			Interpretation[Row[{RawBoxes["\[SkeletonIndicator]"], RawBoxes["Verbose Print Logger"], RawBoxes["\[SkeletonIndicator]"]}], logger];
		logger /: Format[logger, OutputForm] :=
			"-Verbose Print Logger-";
		logger
	]

End[]
