(* ::Package:: *)

(* ::Title:: *)
(*Notebooks*)

(* ::Section:: *)
(*Information*)

`Information`$NotebooksId = "$Id: Notebooks.m,v 1.32 2009/11/23 14:56:50 brenton Exp $"

(* ::Section:: *)
(*Public*)


NotebookToTests::usage =
""

CellsToTest::usage =
""


TestCellEvaluationFunction::usage =
""

OtherCellEvaluationFunction::usage =
""


(* ::Section:: *)
(*Package*)

Begin["`Package`"]

cellEvaluationFunction

End[]

(* ::Section:: *)
(*Private*)

Begin["`Notebooks`Private`"]

handleTestOptionsContents[box_] :=
	Module[{stripped, stripped1},
		stripped =
			If[MatchQ[box, RowBox[{"{", ___, "}"}]],
				stripped1 = box[[1]];
				If[Length[stripped1] > 2,
					Take[stripped1, {2, -2}]
					,
					{}
				]
				,
				{box}
			];
		Switch[stripped,
			{}, Sequence @@ {}
			,
			{RowBox[{_String ..}]}, Sequence @@ Append[stripped, ","]
			,
			{RowBox[__]}, Sequence @@ Append[stripped[[1, 1]], ","]
			,
			_, Throw[stripped, MUnitErrorTag[{"Test Cell", "Invalid Test Options"}]]
		]
	]

$InterfaceLanguage =
	If[$Notebooks, SystemInformation["FrontEnd", "Language"], $Language]

handleSectionOpts[opts___] :=
	Module[{tags, interfaceEnvironments, languages, products, disabled},
		tags = CellTags /. Flatten[{opts}] /. CellTags -> {};
		(* normalize tags *)
		tags = Flatten[{tags}];
		interfaceEnvironments = Intersection[{"Macintosh", "Windows", "X"}, tags];
		languages = Intersection[{"English", "Japanese", "ChineseSimplified"}, tags];
		products = Intersection[{"Mathematica", "MathematicaPlayer", "MathematicaPlayerPro"}, tags];
		disabled = MemberQ[tags, "False" | "Obsolete" | "Disabled"];
		Which[
			disabled,
			False
			,
			interfaceEnvironments != {},
			ToExpression[RowBox[{"MemberQ", "[", RowBox[{ToBoxes[interfaceEnvironments], ",", ToBoxes[$InterfaceEnvironment]}], "]"}]]
			,
			languages != {},
			ToExpression[RowBox[{"MemberQ", "[", RowBox[{ToBoxes[languages], ",", ToBoxes[$InterfaceLanguage]}], "]"}]]
			,
			products != {},
			ToExpression[RowBox[{"MemberQ", "[", RowBox[{ToBoxes[products], ",", ToBoxes["ProductIDName" /. $ProductInformation]}], "]"}]]
			,
			True,
			True
		]
	]

(* TODO: try to remember why I added "Subtitle" and "Subsubtitle" in the first place, since they are not grouping cell styles... *)
sectionStylePattern =
	"Subtitle" | "Subsubtitle" | "Section" | "Subsection" | "Subsubsection" | "TestSection" |
	(* from function paclet notebook *)
	"TemplatesSection"

platformStylePattern =
	"PlatformSection" | "MacintoshSection" | "WindowsSection" | "XSection"

cellParser[cells_List, OptionsPattern[{testParser, sectionParser}]] :=
	Module[{testIDTitleString, testIDFormat, preserve},
		{testIDTitleString, testIDFormat, preserve} = OptionValue[{TestIDTitleString, TestIDFormat, PreserveDataInSections}];
		ReplaceAll[cells,
			{
				Cell[CellGroupData[{Cell[name_String, "Title", opts___], rest___Cell}, _]] :>
					sectionParser[name, handleSectionOpts[opts], rest, TestIDTitleString -> StringReplace[name, ".nb" -> ""], TestIDFormat -> testIDFormat, PreserveDataInSections -> preserve]
				,
				Cell[CellGroupData[{Cell[name_, "Title", opts___], rest___Cell}, _]] :>
					sectionParser[name, handleSectionOpts[opts], rest, TestIDTitleString -> DataToString[name], TestIDFormat -> testIDFormat, PreserveDataInSections -> preserve]
				,
				Cell[CellGroupData[{Cell[name_, (*style:*)sectionStylePattern, opts___], rest___Cell}, _]] :>
					sectionParser[name, handleSectionOpts[opts], rest, TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat, PreserveDataInSections -> preserve]
				,
				Cell[CellGroupData[{Cell[name_, style:platformStylePattern, (*opts*)___], rest___Cell}, _]] :>
					platformSectionParser[name, style, rest, TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat]
				,
				(* handle non-grouped sections *)
				Cell[name_String, "Title", opts___] :>
					sectionParser[name, handleSectionOpts[opts], TestIDTitleString -> StringReplace[name, ".nb" -> ""], TestIDFormat -> testIDFormat, PreserveDataInSections -> preserve]
				,
				Cell[name_, "Title", opts___] :>
					sectionParser[name, handleSectionOpts[opts], TestIDTitleString -> DataToString[name], TestIDFormat -> testIDFormat, PreserveDataInSections -> preserve]
				,
				Cell[name_, (*style:*)sectionStylePattern, opts___] :>
					sectionParser[name, handleSectionOpts[opts], TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat, PreserveDataInSections -> preserve]
				,
				Cell[name_, style:platformStylePattern, (*opts*)___] :>
					platformSectionParser[name, style, TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat]
				,
				Cell[CellGroupData[{first___Cell, test:Cell[_, "Test", ___], rest:___Cell}, _]] :>
					testParser[{first, test, rest}, TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat]
				,
				test:Cell[_, "Test", ___] :>
					testParser[{test}, TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat]
				,
				Cell[CellGroupData[{Cell[BoxData[e_], "Environ", ___], ___Cell}, _]] :>
					boxParser[e]
				,
				Cell[BoxData[e_], "Environ", ___] :>
					boxParser[e]
				,
				Cell[r:RawData[_], ___] :>
					Throw[r, MUnitErrorTag[{"Cell Parsing", "Read RawData"}]]
				,
				Cell[BoxData[e_], "TestOptions", ___] :>
					Throw[e, MUnitErrorTag[{"Cell Parsing", "Orphaned TestOptions cell"}]]
				,
				Cell[BoxData[e_], "ExpectedOutput", ___] :>
					Throw[e, MUnitErrorTag[{"Cell Parsing", "Orphaned ExpectedOutput cell"}]]
				,
				Cell[BoxData[e_], "ExpectedMessage", ___] :>
					Throw[e, MUnitErrorTag[{"Cell Parsing", "Orphaned ExpectedMessage cell"}]]
				,
				Cell[BoxData[e_], "ExpectedMessages", ___] :>
					Throw[e, MUnitErrorTag[{"Cell Parsing", "Orphaned ExpectedMessages cell"}]]
				,
				Cell[___] :>
					Sequence[]
			}
		]
	]

Options[sectionParser] = {PreserveDataInSections -> True}

sectionParser[name_, active:True|False, rest___Cell, opts:OptionsPattern[{testParser, sectionParser}]] :=
	Module[{preserve = OptionValue[PreserveDataInSections]},
		Sequence @@ {
			boxParser[RowBox[{"BeginTestSection", "[", ToBoxes[If[preserve, name, DataToString[name]]], ",", ToBoxes[active], "]"}]],
			Sequence @@ Flatten[{cellParser[{rest}, opts]}],
			boxParser[RowBox[{"EndTestSection", "[", "]"}]]
		}
	]

platformSectionParser[(*name*)_String, style_String, rest___Cell, OptionsPattern[testParser]] :=
	Module[{testIDTitleString, testIDFormat},
		{testIDTitleString, testIDFormat} = OptionValue[{TestIDTitleString, TestIDFormat}];
		Sequence @@ {
			boxParser[
				RowBox[{
					"BeginTestSection", "[",
						RowBox[{
							ToBoxes[style], ",", RowBox[{"$InterfaceEnvironment", "==", ToBoxes[StringReplace[style, p__ ~~ "Section" :> p]]}]
						}],
					"]"
				}]
			],
			Sequence @@ Flatten[{cellParser[{rest}, TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat]}],
			boxParser[RowBox[{"EndTestSection", "[", "]"}]]
		}
	]


Options[testParser] = {TestIDTitleString -> "Untitled", TestIDFormat -> "CellID"}

testParser[list_List, (*opts:*)OptionsPattern[]] :=
	Module[{testIDTitleString, testIDFormat},
		{testIDTitleString, testIDFormat} = OptionValue[{TestIDTitleString, TestIDFormat}];
		ReplaceAll[list, {
			(* single ExpectedMessages cell *)
			{
				t:Cell[BoxData[_], "Test", topts___?OptionQ],
				to:Cell[BoxData[_], "TestOptions", ___]:Cell[BoxData[RowBox[{"{", "}"}]], "TestOptions"],
				em:Cell[BoxData[_], "ExpectedMessages", ___]:Cell[BoxData[RowBox[{"{", "}"}]], "ExpectedMessages"],
				eo:Cell[BoxData[_], "ExpectedOutput", ___]:Cell[BoxData["True"], "ExpectedOutput"],
				(* if a test result cell is present, just ignore it *)
				tr:Cell[___, "TestSuccess" | "TestFailure" | "TestMessagesFailure" | "TestError", ___]:Cell[BoxData[""], "TestSuccess"]
			} :> testParser0[
					t[[1,1]], {topts},
					to[[1,1]],
					em[[1,1]],
					eo[[1,1]], TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat]
			,
			(* multiple ExpectedMessage cells *)
			{
				t:Cell[BoxData[_], "Test", topts___?OptionQ],
				to:Cell[BoxData[_], "TestOptions", ___]:Cell[BoxData[RowBox[{"{", "}"}]], "TestOptions"],
				ems:Cell[BoxData[_], "ExpectedMessage", ___]..,
				eo:Cell[BoxData[_], "ExpectedOutput", ___]:Cell[BoxData["True"], "ExpectedOutput"],
				(* if a test result cell is present, just ignore it *)
				tr:Cell[___, "TestSuccess" | "TestFailure" | "TestMessagesFailure" | "TestError", ___]:Cell[BoxData[""], "TestSuccess"]
			} :> testParser0[
					t[[1,1]], {topts},
					to[[1,1]],
					(*TODO: dig through linear syntax of message and extract arguments and use newer argument checking functionality*)
					RowBox[{"{", RowBox[Riffle[#[[1, 1, 1, 1]] & /@ {ems}, ","]], "}"}],
					eo[[1,1]], TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat]
			,
			_ :> Throw[list, MUnitErrorTag[{"Test Parsing", "The order of the cells is not understood by MUnit"}]]
		}]
	]

testParser0[tb_, topts_, tob_, emb_, eob_, OptionsPattern[testParser]] :=
	Module[{testIDTitleString, testIDFormat, dateString, cellIDString, testID},
		{testIDTitleString, testIDFormat} = OptionValue[{TestIDTitleString, TestIDFormat}];
		dateString = DateString[Flatten[CellChangeTimes /. Flatten[topts] /. CellChangeTimes :> {AbsoluteTime[]}][[1]], {"Year", "Month", "Day"}, TimeZone -> 0];
		cellIDString = IntegerString[CellID /. Flatten[topts] /. CellID -> 0, 36, 7];
		testID =
			Switch[testIDFormat,
				"CellID",
				CellID /. Flatten[topts] /. CellID -> 0
				,
				"WRI",
				testIDTitleString <> "-" <> dateString <> "-" <> cellIDString
				,
				_,
				CellID /. Flatten[topts] /. CellID -> 0
			];
		(*
		kind of a hack. run all of the boxes through the parser once, in order to get any errors propagated
		Because successful parsing returns an expression wrapped in HoldComplete, it is just easier to run the
		parser again around the entire Test expression
		*)
		boxParser[tb];
		boxParser[eob];
		boxParser[emb];
		boxParser[tob];
		boxParser[
			RowBox[{
				"Test",
				"[",
				RowBox[{
					(* provide a courtesy for those horribly mis-guided people who have multiple inputs in a Test cell *)
					If[Head[tb] === List, RowBox[Riffle[tb, ";"]], tb],
					",",
					eob,
					",",
					emb,
					",",
					handleTestOptionsContents[tob],
					RowBox[{
						"TestID",
						"\[Rule]",
						ToBoxes[testID]
					}],
					",",
					RowBox[{
						"TestTags",
						"\[Rule]",
						ToBoxes[Flatten[{CellTags /. Flatten[topts] /. CellTags -> {}}]]
					}]
				}],
				"]"
			}]
		]
	]

Attributes[blockOutputForms] = {HoldFirst}

blockOutputForms[expr_] :=
	Internal`WithLocalSettings[
		Unprotect[$OutputForms]
		,
		Block[{$OutputForms = {}}, expr]
		,
		Protect[$OutputForms]
	]

(* boxParser converts from boxes -> expressions *)
boxParser[boxes_] :=
	blockOutputForms[Quiet[
		Check[
			If[boxes === "",
				Throw[boxes, MUnitErrorTag[{"Box Parsing", "The boxes are empty"}]]
			];
			ToExpression[boxes, StandardForm, HoldComplete]
			,
			Throw[boxes, MUnitErrorTag[{"Box Parsing", "The boxes could not be parsed"}]]
		]
	]]


CellsToTest[cells:{___Cell}] :=
	cellsToTestsString[{Cell[CellGroupData[cells, Open]]}]

NotebookToTests[nb_NotebookObject, opts:OptionsPattern[sectionParser]] :=
	Module[{nbg, cells, title},
		nbg = NotebookGet[nb];
		If[nbg === $Failed,
			(* nb is an invalid (closed) notebook *)
			Throw[nb, MUnitErrorTag[{"NotebookGet", "$Failed"}]]
		];
		cells = nbg[[1]];
		(* insert a Title cell group if it doesn't exist already *)
		If[!MatchQ[cells, {Cell[CellGroupData[{Cell[_, "Title", ___], __}, _]]}],
			title = WindowTitle /. AbsoluteOptions[nb, WindowTitle];
			cells = {Cell[CellGroupData[{Cell[title, "Title"], Sequence @@ cells}, Open]]}
		];
		cellsToTestsString[cells, opts]
	]

cellsToTestsString[cells_, opts:OptionsPattern[sectionParser]] :=
	Module[{oldContext, oldContextPath, parsed},
		Internal`WithLocalSettings[
			oldContext = $Context;
			oldContextPath = $ContextPath;
			$Context = "MUnit`Notebooks`Private`Sandbox`";
			$ContextPath = {"MUnit`Notebooks`Private`Sandbox`", "System`"};
			,
			parsed = cellParser[cells, TestIDFormat -> "WRI", opts];
			If[parsed == {},
				Throw[cells, MUnitErrorTag[{"Cell Parsing", "Cell Parser returned {}"}]]
			];
			StringDrop[StringJoin[
				# /. {
					HoldComplete[x_] :> ExpressionToString[x] <> "\n\n",
					(* convert errors to strings *)
					e_ :> ToString[e] <> "\n\n"
				}& /@ parsed], -1]
			,
			$Context = oldContext;
			$ContextPath = oldContextPath
		]
	]

TestCellEvaluationFunction[_, _] :=
	Module[{nb, grouped},
		Catch[
			nb = EvaluationNotebook[];
			SelectionMove[nb, All, EvaluationCell];
			grouped = "FirstCellInGroup" /. Developer`CellInformation[nb][[1]];
			If[grouped,
				SelectionMove[nb, All, CellGroup]
			];
			cellEvaluationFunction[nb, grouped]
			,
			_MUnitErrorTag
			,
			Function[{value, tag},
				MessageDialog[Column[{tag[[1,1]], Style[tag[[1,2]], Bold], value}], WindowTitle->"MUnit Error"];
				(* return Null to prevent the MessageDialog's NotebookObject from being returned in the test notebook *)
				Null
			]
		]
	]

OtherCellEvaluationFunction[_, _] :=
	Module[{nb},
		Catch[
			nb = EvaluationNotebook[];
			SelectionMove[nb, All, EvaluationCell];
			SelectionMove[nb, All, CellGroup];
			If[Developer`CellInformation[nb] === $Failed,
				(* cells are not grouped with a Test cell *)
				SelectionMove[nb, After, EvaluationCell];
				Throw[Null, MUnitErrorTag[{"Test", "Malformed Test CellGroup"}]]
				,
				cellEvaluationFunction[nb, True]
			]
			,
			_MUnitErrorTag
			,
			Function[{value, tag},
				MessageDialog[Column[{tag[[1,1]], Style[tag[[1,2]], Bold], value}], WindowTitle->"MUnit Error"];
				(* return Null to prevent the MessageDialog's NotebookObject from being returned in the test notebook *)
				Null
			]
		]
	]


(* TODO: rewrite using CellPrint to be nice, and also some how simulate cell evaluating during test evaluation *)
(*
Precondition: if there is a cell group, then it is selected, if there is only a Test cell, then it is selected
*)
cellEvaluationFunction[nb_NotebookObject, grouped:(True|False)] :=
	Module[{selection, cells, test, tr},
		(*SetOptions[nb, WindowClickSelect -> False];*)
		selection = NotebookRead[nb];
		(* cells is a list of cells, {test} or {test, eo} etc. *)
		cells =
			If[grouped,
				DeleteCases[Cases[{selection}, Cell[CellGroupData[cells:{___Cell}, _]] :> cells][[1]],
					Cell[_, Except["Test" | "TestOptions" | "ExpectedMessages" | "ExpectedMessage" | "ExpectedOutput" ], ___],
					Infinity]
				,
				{selection}
			];
		(* no more harm in writing CellGroupData with one cell, if selection was only a Test cell *)
		NotebookWrite[nb, Cell[CellGroupData[cells, Open]], After];
		Catch[
			test = CellsToTest[cells];
			(* from string -> expression *)
			tr = ToExpression[test];
			If[TestResultQ[tr],
				Replace[FailureMode[tr], {
					"Success" :>
						NotebookWrite[nb, Cell[BoxData[RowBox[{PaneBox[ToBoxes[tr], ImageSize->250], rerunTestButtonBox[nb, tr]}]], "TestSuccess"]]
					 ,
					"MessagesFailure" :>
						NotebookWrite[nb, Cell[BoxData[RowBox[{PaneBox[ToBoxes[tr], ImageSize->250], rerunTestButtonBox[nb, tr], replaceMessagesButtonBox[nb, tr]}]], "TestMessagesFailure"]]
					,
					"Failure" :>
						NotebookWrite[nb, Cell[BoxData[RowBox[{PaneBox[ToBoxes[tr], ImageSize->250], rerunTestButtonBox[nb, tr], replaceOutputButtonBox[nb, tr]}]], "TestFailure"]]
					,
					"Error" :>
						NotebookWrite[nb, Cell[BoxData[RowBox[{PaneBox[ToBoxes[tr], ImageSize->250], rerunTestButtonBox[nb, tr]}]], "TestError"]]
					,
					m_ :> Throw[m, MUnitErrorTag[{"Test", "Unrecognized FailureMode"}]]
				}]
				,
				(*
				if tr is not a test result, then there was something very bad
				*)
				Throw[tr, MUnitErrorTag[{"Test", "Invalid TestResult"}]]
			];
			(*SetOptions[nb, WindowClickSelect -> True]*)
			,
			_MUnitErrorTag
			,
			Function[{value, tag},
				NotebookWrite[nb, Cell[BoxData[ToBoxes[Column[{tag[[1,1]], Style[tag[[1,2]], Bold], value}], StandardForm]], "TestError"]];
				(*SetOptions[nb, WindowClickSelect -> True]*)
			]
		]
	]


End[]
