(* ::Package:: *)

(* ::Title:: *)
(*Palette*)

(* ::Section:: *)
(*Information*)

`Information`$PaletteId = "$Id: Palette.m,v 1.2 2009/11/24 16:43:57 brenton Exp $"

(* ::Section:: *)
(*Public*)

PaletteClear::usage =
""

PaletteConvertCellGroup::usage =
""

PaletteConvertNotebook::usage =
""

PaletteNewTestNotebook::usage =
""

PaletteRun::usage =
""

PaletteAbort::usage =
""

PaletteQuitKernels::usage =
""

PaletteSaveAs::usage =
""

(* ::Section:: *)
(*Package*)

Begin["`Package`"]



End[]

(* ::Section:: *)
(*Private*)

Begin["`Palette`Private`"]



PaletteConvertCellGroup[nb_NotebookObject] :=
	Module[{selection},
		Catch[
			checkStyleDefinitions[nb];
			selection = NotebookRead[nb];
			If[MatchQ[selection, Cell[_CellGroupData]],
				(* make selection conform to {Cell[_CellGroupData]..} pattern *)
				selection = {selection};
			];
			If[!MatchQ[selection, {Cell[_CellGroupData]..}],
				Throw[selection, MUnitErrorTag[{"CellGroup Converting", "Current selection is not 1 or more cell groups"}]]
			];
			Scan[convertCellGroup[nb, #]&, selection];
			,
			_MUnitErrorTag
			,
			Function[{value, tag}, MessageDialog[Column[{tag[[1,1]], Style[tag[[1,2]], Bold], value, If[Length[tag] >= 3, tag[[1,3]], Spacer[{0, 0}]]}], WindowTitle->"MUnit Error"]]
		]
	]

checkStyleDefinitions[nb_] :=
	If[CurrentValue[nb, {StyleDefinitions, "Test"}] == {} ||
		CurrentValue[nb, {StyleDefinitions, "ExpectedMessage"}] == {} ||
		CurrentValue[nb, {StyleDefinitions, "ExpectedOutput"}] == {}
		,
		Throw[Null, MUnitErrorTag[{"CellGroup Converting", "Current stylesheet does not have the necessary styles defined."}]]
	]

convertCellGroup[nb_, Cell[CellGroupData[cells:{__Cell}, _]]] :=
	Module[{},
		Scan[checkCellID, cells];	
		Scan[convertCell[nb, #]&, cells]
	]

checkCellID[Cell[_, _, ___, CellID -> _Integer, ___]] := True
checkCellID[Cell[a_, s_, opts___]] := Throw[Cell[a, s, opts], MUnitErrorTag[{"CellGroup Converting", "Missing CellID"}]]
checkCellID[x_] := Throw[x, MUnitErrorTag[{"CellGroup Converting", "Unrecognized cell"}]]

convertCell[nb_, Cell[a_, s_, opts___]] :=
	Module[{
		newStyle = s /. {
			"Input" -> "Test",
			"Environ" -> "Test",
			"Message" -> "ExpectedMessage",
			"Output" -> "ExpectedOutput",
			w_ :> Throw[w, MUnitErrorTag[{"CellGroup Converting", "Unexpected Cell Style"}]]},
		id = Cases[{opts}, (CellID -> id_) :> id][[1]]},
		NotebookFind[nb, id, All, CellID];
		NotebookWrite[nb, Cell[a, newStyle, opts]]
	]


PaletteConvertNotebook[nb_NotebookObject] :=
	Module[{n},
		SetOptions[nb, StyleDefinitions -> FrontEnd`FileName[{"MUnit"}, "MUnit.nb"]];
		(* get-and-put to add any CellIDs that need to be added *)
		n = NotebookGet[nb];
		NotebookPut[n, nb];
	]

PaletteNewTestNotebook[] :=
	NotebookPut[Get[ToFileName[{$MUnitDirectory, "FrontEnd", "SystemResources"}, "Template.nb"]]]

PaletteRun[nb_NotebookObject] :=
	Module[{title},
		title = WindowTitle /. AbsoluteOptions[nb, WindowTitle];
		TestRun[nb, TestRunTitle -> title, Loggers -> {NotebookLogger[nb]}]
	]

PaletteAbort[nb_NotebookObject] :=
	MathLink`CallFrontEnd[FrontEnd`EvaluatorAbort[Automatic]]

PaletteQuitKernels[include_] :=
	Module[{evals = DeleteCases[CurrentValue[$FrontEnd, EvaluatorNames][[All, 1]], "Local"]},
		If[include, AppendTo[evals, "Local"]];
		MathLink`CallFrontEnd[FrontEnd`EvaluatorQuit[#]]& /@ evals
	]


PaletteSaveAs[nb_NotebookObject] :=
	Module[{stream, windowTitle, tests},
		Catch[
			tests = NotebookToTests[InputNotebook[], PreserveDataInSections -> False];
			windowTitle = WindowTitle /. AbsoluteOptions[nb, WindowTitle];
			$fileName =
				SystemDialogInput["FileSave", {
					If[ValueQ[$fileName] && $fileName =!= $Canceled, ToFileName[{DirectoryName[$fileName]}, windowTitle], windowTitle], {
					"Mathematica Package Test File (*.mt)" -> {"*.mt"}}}
				];
			If[$fileName =!= $Canceled,
				stream = OpenWrite[$fileName];
				(* write out package header comment to let the front end recognize this as a package file *)
				WriteString[stream, "(* ::Package:: *)\n\n"];
				(* probably ok to not check that NotebookToTests returned $Failed *)
				WriteString[stream, tests];
				Close[stream]
			]
			,
			_MUnitErrorTag
			,
			Function[{value, tag},
				MessageDialog[Column[{tag[[1,1]], Style[tag[[1,2]], Bold], value, If[Length[tag] >= 3, tag[[1,3]], Spacer[{0, 0}]]}], WindowTitle->"MUnit Error"]
			]
		];
	]

PaletteClear[nb_NotebookObject] :=
	(
		If[NotebookFind[nb, "TestSuccess", All, CellStyle] =!= $Failed,
			NotebookDelete[nb]];
		If[NotebookFind[nb, "TestFailure", All, CellStyle] =!= $Failed,
			NotebookDelete[nb]];
		If[NotebookFind[nb, "TestMessagesFailure", All, CellStyle] =!= $Failed,
			NotebookDelete[nb]];
		If[NotebookFind[nb, "TestError", All, CellStyle] =!= $Failed,
			NotebookDelete[nb]];
		If[NotebookFind[nb, "Output", All, CellStyle] =!= $Failed,
			NotebookDelete[nb]]
	)



End[]
