(* ::Package:: *)

(* ::Title:: *)
(*Test*)

(* ::Section:: *)
(*Information*)

`Information`$TestId = "$Id: Test.m,v 1.35 2009/12/08 17:20:19 brenton Exp $"

(* ::Section:: *)
(*Public*)

Test::usage =
"Test[input] tests whether input evaluates to True and no messages are generated.\n
Test[input, expected] tests whether input evaluates to expected and no messages are generated.\n
Test[input, expected, messages] tests whether input evaluates to expected and the given messages are generated.\n
Test is also a selector for TestResult."

TestMatch::usage =
"TestMatch[input, expected] and TestMatch[input, expected, messages] are like the corresponding versions of Test except they test MatchQ[input, expected] instead of SameQ."

TestStringMatch::usage =
"TestStringMatch[input, expected] and TestStringMatch[input, expected, messages] are like the corresponding versions of Test except they test StringMatchQ[input, expected] instead of SameQ."

TestFree::usage =
"TestFree[input, expected] and TestFree[input, expected, messages] are like the corresponding versions of Test except they test FreeQ[input, expected] instead of SameQ."

TestStringFree::usage =
"TestStringFree[input, expected] and TestStringFree[input, expected, messages] are like the corresponding versions of Test except they test StringFreeQ[input, expected] instead of SameQ."

TestResultObject::usage =
"TestResultObject is the head of the expression returned by Test and related functions. It is passed to loggers.
TestResultObject contains all the relevant information about a given Test.
Clients should always use selectors to extract the desired element from a TestResultObject.
For example, if tr is a TestResultObject, write Test[tr] instead of tr[[1]] to get the original Test expr out.
The internal structure of TestResultObject is not guaranteed to stay the same from one version of MUnit to the next."

TestResultQ::usage =
"TestResultQ[tr] returns True if tr is a valid TestResultObject, and False otherwise."

FailureMode::usage =
"FailureMode is a selector for TestResultObject."

TestInput::usage =
"TestInput is a selector for TestResultObject."

ExpectedOutput::usage =
"ExpectedOutput is a selector for TestResultObject."

ActualOutput::usage =
"ActualOutput is a selector for TestResultObject."

ExpectedMessages::usage =
"ExpectedMessages is a selector for TestResultObject."

ActualMessages::usage =
"ActualMessages is a selector for TestResultObject."

ErrorMessage::usage =
"ErrorMessage is a selector for TestResultObject."

TestIndex::usage =
"TestIndex is a selector for TestResultObject. It is used in test runs."

TestID::usage =
"TestID is an option to Test and TestMatch that specifies a string used as an identifier for the test.
TestID is also a selector for TestResult."

TestTags::usage =
"TestTags is an option to Test that specified meta information about that test. If a test comes from a notebook, its
cell tags are kept as TestTags."

EquivalenceFunction::usage =
"EquivalenceFunction is an option to Test and TestMatch that specifies the function that will be used to compare the actual and expected results.\n
The default for Test is SameQ, and for TestMatch it is MatchQ.
EquivalenceFunction is also a selector for TestResult."

MessagesEquivalenceFunction::usage =
"MessagesEquivalenceFunction is an option to Test and TestMatch that specifies the function that will be used to compare the actual and expected messages\n.
Custom MessagesEquivalenceFunction settings should also take into account that actual messages are passed in wrapped in HoldForm, \
and that a custom ExpectedMessagesWrapper will probably also be necessary to prevent any unwanted evaluations or messages."

$DefaultMessagesEquivalenceFunction::usage =
"$DefaultMessagesEquivalenceFunction is the default messages equivalence function. \
It is similar to MatchQ and understands Message and MessageName expressions."

ExpectedMessagesWrapper::usage =
"ExpectedMessagesWrapper is an option to Test that specifies the function that will be used to wrap expected messages in a TestResultObject expression. \
The default value is $DefaultExpectedMessagesWrapper, but it should be changed to an appropriate value for preserving custom expressions passed as expected messages."

TestFailureMessage::usage =
"TestFailureMessage is an option to Test that specifies a message that will be logged if the test fails.\n
TestFailureMessage is also a selector for TestResult."

TestFailureAction::usage =
"TestFailureAction is an option to Test that specifies a string naming an action to take if the test fails.\n
The allowable values are Continue (the default) and Abort.\n
TestFailureAction is also a selector for TestResult."

TestErrorAction::usage =
"TestErrorAction is an option to Test that specifies a string naming an action to take if the test has an error.\n
The allowable values are Continue (the default) and Abort.\n
TestErrorAction is also a selector for TestResult."

TestComment::usage =
"TestComment is deprecated."

MemoryConstraint::usage =
"MemoryConstraint is an option for Test that specifes how much memory (in bytes) the test is allowed to use."

$MemoryConstrained::usage =
"$MemoryConstrained is returned as actual output when a test met a memory constraint."

$TimeConstrained::usage =
"$TimeConstrained is returned as actual output when a test met a time constraint."

TestTimeUsed::usage =
"TestTimeUsed is a selector for test results."

TestMemoryUsed::usage =
"TestMemoryUsed is a selector for test results."

$Error::usage =
"$Error is filled in for the arguments of a test result when an error occurs."

TestFileChild::usage = 
"TestFileChild is an option of TestResultObject that specifies the TestFileChild used for the test."

(* ::Section:: *)
(*Package*)

Begin["`Package`"]

AddTestFunction

$TestIndex

testError

End[]

(* ::Section:: *)
(*Private*)

Begin["`Test`Private`"]

(*
set this here so that Tests can be run outside of TestRun
*)
$TestIndex = 0

(*
a side-effect is that MUnit leaks out the bitbucket stream
*)
$bitbucket = Module[{f = Switch[$OperatingSystem, "Windows", "NUL", "MacOSX" | "Unix", "/dev/null"], strs},
	            strs = Streams[f];
	            If[strs == {},
	            	(* bitbucket stream is not open yet, so open it *)
	            	OpenWrite[f]
	            	,
	            	strs[[1]]
	            ]
             ]

Attributes[Test] = {HoldAllComplete}

Options[Test] = {
		EquivalenceFunction -> SameQ,
		MemoryConstraint -> None,
		MessagesEquivalenceFunction -> $DefaultMessagesEquivalenceFunction,
		ExpectedMessagesWrapper -> $DefaultExpectedMessagesWrapper,
		TestFailureMessage -> "",
		TestFailureAction -> "Continue",
		TestErrorAction -> "Continue",
		TestID -> 0,
		TestTags -> {},
		TimeConstraint -> None,
		
		TestComment -> ""
	}

Test[input_, expected_:True, Shortest[expectedMsgs_:{}], opts:OptionsPattern[]] :=
	Module[{testEquivFunc, msgsEquivFunc, expectedMsgsWrapper, testID, actual, expectedEvaled, testEquivFuncEvaled,
		msgsEquivFuncEvaled, actualMsgs, failureMode, testFailureMessage, testFailureAction, testErrorAction, timeConstraint,
		memoryConstraint, timeUsed, memoryUsed, testTags},
		Catch[
			Block[{
				(*
				$Messages needs to be something not likely to be set by a function blocking messages, like no-value $Messages or {},
				in order to be able to tell when it IS set to $Messages or {}, in order to correctly not report those messages
				But it also need to be something not containing stdout, to ACTUALLY suppress all messages
				It also needs to be something that actually works as a OutputStream, like a temp file, otherwise there is a slew of messages
				complaining.
				But there's no reason to open a whole temp file just for one test. Just use the platform-dependent bitbucket
				Now, we'll know if $Messages was changed because it will no longer be {$bitbucket}, a safe bet.
				Who would be writing to the bitbucket in a normal circumstance?
				If there were a way to have a MUnit-specific bitbucket, that would be great. Or to have virtual stream things in Mathematica.
				*)
				$OldMessages = $Messages,
				$Messages = {$bitbucket},
				(* Block these symbols to minimize side-effects *)			
				$DisplayFunction = Identity,
				$SoundDisplayFunction = Identity,
				(* these are to setup a nice environment for tests within tests! *)
				$TestIndex = 0,
				$MessageWithArgsList = {},
				(* even though we don't use it, continue blocking $MessageList. This is to reset the General::stop counter
				and allow
				t:=Test[1/0,ComplexInfinity,{_}];
				{t,t,t}
				to work*)
				$MessageList = {}
				}
				,
				
				(* treat any unrecognized options as an error *)
				Check[
					{testEquivFunc, msgsEquivFunc, expectedMsgsWrapper, testID, testFailureMessage,
						testFailureAction, testErrorAction, timeConstraint, memoryConstraint, testTags} =
						OptionValue[{EquivalenceFunction, MessagesEquivalenceFunction, ExpectedMessagesWrapper, TestID, TestFailureMessage,
							TestFailureAction, TestErrorAction, TimeConstraint, MemoryConstraint, TestTags}]
					,
					Throw["Invalid options: " <> ToString[FilterRules[Flatten[{opts},1], Except[Options[Test]]]], "internalMUnitTestTag"]
					,
					OptionValue::nodef
				];
				
				memoryUsed = MemoryInUse[];
				timeUsed = TimeUsed[];
				
				(* evaluate input*)
				CheckAll[
					(* use Reap here to not double-count any Tests within Tests in a test run *)
					Reap[
						Which[
							timeConstraint === memoryConstraint === None,
							input
							,
							timeConstraint === None,
							MemoryConstrained[input, memoryConstraint, $MemoryConstrained]
							,
							memoryConstraint === None,
							TimeConstrained[input, timeConstraint, $TimeConstrained]
							,
							True,
							TimeConstrained[MemoryConstrained[input, memoryConstraint, $MemoryConstrained], timeConstraint, $TimeConstrained]
						]
						,
						"MUnitTest"
					][[1]]
					,
					Switch[#2,
						Hold[],
						(* everything is fine *)
						actual = #1
						,
						Hold[_Abort],
						(* an Abort was encountered, in test or user action *)
						TestTerminate["User aborted during test input evaluation"];
						Throw["User aborted during test input evaluation", "internalMUnitTestTag"]
						,
						_,
						(* some other thing like Throw or Break *)
						Throw["Unhandled error " <> ToString[HoldForm@@#2] <> " during test input evaluation", "internalMUnitTestTag"]
					]&
				];
				
				timeUsed = TimeUsed[] - timeUsed;
				memoryUsed = MemoryInUse[] - memoryUsed;
				
				(* Capture $MessageWithArgsList before expected is evaluated. *)
				actualMsgs = $MessageWithArgsList;
				
				(* evaluate expected only once, to avoid side-effects *)
				CheckAll[
					(* use Reap here to not double-count any Tests within Tests in a test run *)
					Reap[expected, "MUnitTest"][[1]]
					,
					Switch[#2,
						Hold[],
						(* everything is fine *)
						expectedEvaled = #1
						,
						Hold[_Abort],
						(* an Abort was encountered, in test or user action *)
						TestTerminate["User aborted during expected output evaluation"];
						Throw["User aborted during expected output evaluation", "internalMUnitTestTag"]
						,
						_,
						(* some other thing like Throw or Break *)
						Throw["Unhandled error " <> ToString[HoldForm@@#2] <> " during expected output evaluation", "internalMUnitTestTag"]
					]&
				];
				
				testEquivFuncEvaled = testEquivFunc[actual, expectedEvaled];
				
				msgsEquivFuncEvaled = msgsEquivFunc[actualMsgs, expectedMsgs];
				
				Switch[{testEquivFuncEvaled, msgsEquivFuncEvaled},
					{True, True},
					failureMode = "Success"
					,
					{False, True | False},
					failureMode = "Failure"
					,
					{True, False},
					failureMode = "MessagesFailure"
					,
					{Except[True | False], _},
					Throw["EquivalenceFunction did not return True or False: " <> ToString[testEquivFuncEvaled], "internalMUnitTestTag"]
					,
					{_, Except[True | False]},
					Throw["MessagesEquivalenceFunction did not return True or False: " <> ToString[msgsEquivFuncEvaled], "internalMUnitTestTag"]
				]
			]; (* end Block *)
			(*
			Wrap expected and actual in HoldForm to preserve it. This is neccessary if a TestResultObject is saved to a file,
			then read in a different session, where actual would evaluate to something new.
			*)
			With[{actual = actual},
				Sow[
					TestResultObject[
						HoldForm[Test[input, expected, expectedMsgs, opts]],
						failureMode,
						HoldForm[input],
						HoldForm[expected],
						HoldForm[actual],
						expectedMsgsWrapper[expectedMsgs],
						(* each msg is already wrapped in HoldForm *)
						actualMsgs,
						"",
						TestIndex -> ++$TestIndex,
						TestID -> testID,
						EquivalenceFunction -> testEquivFunc,
						MessagesEquivalenceFunction -> msgsEquivFunc,
						ExpectedMessagesWrapper -> expectedMsgsWrapper,
						TestFailureMessage -> testFailureMessage,
						TestFailureAction -> testFailureAction,
						TestErrorAction -> testErrorAction,
						TestTimeUsed -> timeUsed,
						TestMemoryUsed -> memoryUsed,
						TestTags -> testTags,
						TestFileChild -> $CurrentFileChild
					],
					{"MUnitTest"}
				]
			]
			,
			"internalMUnitTestTag"
			,
			Function[{value, tag},
				(*
				calling testError must wait until outside of the Block, so that $TestIndex can increment correctly
				*)
				testError[value, opts]
			]
		]
	]

Test[args___] :=
	testError["Incorrect arguments: " <> ToString[Unevaluated[{args}]], args]

testError[errMsg_String, ___, OptionsPattern[]] :=
	Module[{testID = OptionValue[Test, TestID]},
		Sow[
			TestResultObject[
				$Error,
				"Error",
				$Error,
				$Error,
				$Error,
				$Error,
				$Error,
				errMsg,
				TestIndex -> ++$TestIndex,
				TestID -> testID
			]
			,
			{"MUnitTest"}
		]
	]

AddTestFunction[testFunc_] :=
	Module[{},
		Attributes[testFunc] = Attributes[Test];
		Options[testFunc] = Options[Test];
		testFunc[input_, expected_:True, Shortest[messages_:{}], opts:OptionsPattern[]] :=
			With[{options = Join[Flatten[{opts}], Options[testFunc]]},
				Test[input, expected, messages, options]
			]
	]

AddTestFunction[TestMatch]

SetOptions[TestMatch, EquivalenceFunction -> MatchQ]

AddTestFunction[TestStringMatch]

SetOptions[TestStringMatch, EquivalenceFunction -> StringMatchQ]

AddTestFunction[TestFree]

SetOptions[TestFree, EquivalenceFunction -> FreeQ]

AddTestFunction[TestStringFree]

SetOptions[TestStringFree, EquivalenceFunction -> StringFreeQ]


Options[TestResultObject] =
		{TestIndex -> 0,
		TestMemoryUsed -> 0,
		TestTimeUsed -> 0,
		TestFileChild -> Null} ~Join~ Options[Test]

TestResultQ[tr_TestResultObject] :=
	MatchQ[tr[[1]], HoldForm[_Test] | $Error]

TestResultQ[_] =
	False

TestResultObject /: Format[tr_TestResultObject?TestResultQ, OutputForm] :=
	Switch[FailureMode[tr],
		"Success",
		StringJoin["-", "Success", "-"]
		,
		"MessagesFailure",
		StringJoin["-", "MessagesFailure: ", ToString[ActualMessages[tr], OutputForm], "-"]
		,
		"Failure",
		StringJoin["-", "Failure: ", ToString[ActualOutput[tr], OutputForm], "-"]
		,
		"Error",
		StringJoin["-", "Error: ", ErrorMessage[tr], "-"]
	]

TestResultObject /: Format[tr_TestResultObject?TestResultQ, StandardForm] :=
	Switch[FailureMode[tr],
		"Success",
		Interpretation[Row[{RawBoxes["\[SkeletonIndicator]"], Row[{RawBoxes["TestResultObject"], RawBoxes["["], "Success", RawBoxes["]"]}], RawBoxes["\[SkeletonIndicator]"]}], tr]
		,
		"MessagesFailure",
		Interpretation[Row[{RawBoxes["\[SkeletonIndicator]"], Row[{RawBoxes["TestResultObject"], RawBoxes["["], Row[{"MessagesFailure:", " ", ActualMessages[tr]}], RawBoxes["]"]}], RawBoxes["\[SkeletonIndicator]"]}], tr]
		,
		"Failure",
		Interpretation[Row[{RawBoxes["\[SkeletonIndicator]"], Row[{RawBoxes["TestResultObject"], RawBoxes["["], Row[{"Failure:", " ", ActualOutput[tr]}], RawBoxes["]"]}], RawBoxes["\[SkeletonIndicator]"]}], tr]
		,
		"Error",
		Interpretation[Row[{RawBoxes["\[SkeletonIndicator]"], Row[{RawBoxes["TestResultObject"], RawBoxes["["], Row[{"Error:", " ", ErrorMessage[tr]}], RawBoxes["]"]}], RawBoxes["\[SkeletonIndicator]"]}], tr]
	]

(* This prevents the Very Large Output panel from appearing in the front end if the parts of the TestResult are very big (such as Graphics) *)
BoxForm`SetSizeCountRule[TestResultObject -> None]

TestResultObject /: Test[            TestResultObject[test_, ___]] :=
	test

TestResultObject /: FailureMode[     TestResultObject[_, failMode_, ___]] :=
	failMode

TestResultObject /: TestInput[       TestResultObject[_, _, input_, ___]] :=
	input

TestResultObject /: ExpectedOutput[  TestResultObject[_, _, _, expected_, ___]] :=
	expected

TestResultObject /: ActualOutput[    TestResultObject[_, _, _, _, actual_, ___]] :=
	actual

TestResultObject /: ExpectedMessages[TestResultObject[_, _, _, _, _, expectMsgs_, ___]] :=
	expectMsgs

TestResultObject /: ActualMessages[  TestResultObject[_, _, _, _, _, _, actualMsgs_, ___]] :=
	actualMsgs

TestResultObject /: ErrorMessage[    TestResultObject[_, _, _, _, _, _, _, errMsg_, ___]] :=
	errMsg

TestResultObject /: TestID[TestResultObject[__, opts:OptionsPattern[]]] :=
	OptionValue[TestResultObject, {opts}, TestID]

TestResultObject /: TestIndex[TestResultObject[__, opts:OptionsPattern[]]] :=
	OptionValue[TestResultObject, {opts}, TestIndex]

TestResultObject /: EquivalenceFunction[TestResultObject[__, opts:OptionsPattern[]]] :=
	OptionValue[TestResultObject, {opts}, EquivalenceFunction]

TestResultObject /: MessagesEquivalenceFunction[TestResultObject[__, opts:OptionsPattern[]]] :=
	OptionValue[TestResultObject, {opts}, MessagesEquivalenceFunction]

TestResultObject /: ExpectedMessagesWrapper[TestResultObject[__, opts:OptionsPattern[]]] :=
	OptionValue[TestResultObject, {opts}, ExpectedMessagesWrapper]

TestResultObject /: TestFailureMessage[TestResultObject[__, opts:OptionsPattern[]]] :=
	OptionValue[TestResultObject, {opts}, TestFailureMessage]

TestResultObject /: TestFailureAction[TestResultObject[__, opts:OptionsPattern[]]] :=
	OptionValue[TestResultObject, {opts}, TestFailureAction]

TestResultObject /: TestErrorAction[TestResultObject[__, opts:OptionsPattern[]]] :=
	OptionValue[TestResultObject, {opts}, TestErrorAction]

TestResultObject /: TestTimeUsed[TestResultObject[__, opts:OptionsPattern[]]] :=
	OptionValue[TestResultObject, {opts}, TestTimeUsed]

TestResultObject /: TestMemoryUsed[TestResultObject[__, opts:OptionsPattern[]]] :=
	OptionValue[TestResultObject, {opts}, TestMemoryUsed]

TestResultObject /: TestTags[TestResultObject[__, opts:OptionsPattern[]]] :=
	OptionValue[TestResultObject, {opts}, TestTags]

TestResultObject /: TestFileChild[TestResultObject[__, opts:OptionsPattern[]]] :=
	OptionValue[TestResultObject, {opts}, TestFileChild]


$MessageWithArgsList = {}

MUnitMessageHandler[ctlString_String, Hold[msg_MessageName], Hold[Message[msg_MessageName, args___HoldForm]]] :=
	Module[{},
		If[(* stdout would have been in $Messages *)
		   If[$Messages === {$bitbucket},
		   	  (* $Messages has (most likely) not changed since invocation of Test, so use the value of $Messages at that time *)
		      (ListQ[$OldMessages] && MemberQ[$OldMessages, OutputStream["stdout", _]]),
		      (* $Messages has been changed, most likely to $Messages or {} *)
		      (ListQ[$Messages] && MemberQ[$Messages, OutputStream["stdout", _]])
		   ]
		   ,
		   AppendTo[$MessageWithArgsList, HoldForm[Message[msg, args]]];
		]
	]

Internal`AddHandler["MessageTextFilter", MUnitMessageHandler]

(*
needs to be HoldFirst because we do things with the arguments, where they aren't held by Message any more
*)
SetAttributes[patternizeExpectedMessage, HoldFirst]
patternizeExpectedMessage[name_MessageName] := HoldForm[Message[name, ___]]
(*HoldPattern is necessary for the definition, when Message and MessageName are not blocked yet, they are only blocked at runtime*)
patternizeExpectedMessage[HoldPattern[Message][name_, args___]] :=
	With[{args2 = Sequence @@ HoldForm /@ Unevaluated[{args}]},
		With[{m = Message[name, args2]},
			HoldForm[m]
		]
	]
(* let patterns fall-through *)
patternizeExpectedMessage[arg_] := arg

SetAttributes[$DefaultMessagesEquivalenceFunction, HoldAllComplete]

$DefaultMessagesEquivalenceFunction[actualMsgs_, expectedMsgs_] :=
	Module[{expectedMsgs2},
		Block[{Message, MessageName},
			SetAttributes[Message, HoldRest];
			SetAttributes[MessageName, HoldFirst];
			expectedMsgs2 =
				Switch[expectedMsgs,
					_MessageName,
					{patternizeExpectedMessage[expectedMsgs]}
					,
					_Message,
					{patternizeExpectedMessage[expectedMsgs]}
					,
					_List,
					Map[patternizeExpectedMessage, expectedMsgs]
					,
					_Alternatives,
					(* from old Testing`-style tests *)
					Map[patternizeExpectedMessage, expectedMsgs, {2}]
					,
					_,
					expectedMsgs
				];
			MatchQ[actualMsgs, expectedMsgs2]
		]
	]

SetAttributes[$DefaultExpectedMessagesWrapper, HoldFirst]

(*
nothing depends on ExpectedMessages, so only wrap entire in HoldForm. The individual args do not
need to be wrapped in HoldForm, like with ActualMessages.
*)
$DefaultExpectedMessagesWrapper[expectedMsg_MessageName] := {HoldForm[expectedMsg]}
$DefaultExpectedMessagesWrapper[expectedMsg_Message] := {HoldForm[expectedMsg]}
$DefaultExpectedMessagesWrapper[expectedMsgs_List] := HoldForm /@ Unevaluated[expectedMsgs]
$DefaultExpectedMessagesWrapper[expectedMsgs_Alternatives] := Map[HoldForm, Unevaluated[expectedMsgs], {2}]
$DefaultExpectedMessagesWrapper[expectedMsgs_] := expectedMsgs


End[]
