(* ::Package:: *)

(* ::Title:: *)
(*Typesetting*)

(* ::Section:: *)
(*Information*)

`Information`$TypesettingId = "$Id: Typesetting.m,v 1.3 2009/11/09 14:43:55 brenton Exp $"

(* ::Section:: *)
(*Package*)

Begin["`Package`"]

ExpressionToString::usage =
"Unfortunately, ExpressionToString only outputs strings of the FullForm of the given expression. This will be fixed in a later version."

DataToString::usage =
"Unfortunately, DataToString only outputs strings of the LinearOutputForm of the given data. This is by design."

End[]

(* ::Section:: *)
(*Private*)

Begin["`Typesetting`Private`"]


Attributes[ExpressionToString] = {HoldAllComplete}

ExpressionToString[x_] := ToString[Unevaluated[FullForm[x]]]


(*
DataToString

text -> return
boxes like SqrtBox, FormBox, etc. -> return pseudo-linear syntax string thing 
boxes like GraphicsBox, GridBox -> -GraphicsBox-, -GridBox-, etc.
*)

DataToString::baddata =
"Invalid data given to DataToString: `1`"

DataToString::badtext =
"Invalid text data given to DataToString: `1`"

DataToString::badbox =
"Invalid box data given to DataToString: `1`"

(*
strings go through because Cell["title", "Title"] is like implicit TextData
*)

DataToString[d_] :=
	Switch[d,
		_String,
		d
		,
		TextData[_List, ___],
		StringJoin[dtsTextData /@ d[[1]]]
		,
		_TextData,
		dtsTextData[d[[1]]]
		,
		_BoxData,
		dtsBoxData[d[[1]]]
		,
		_,
		Throw[d, MUnitErrorTag[{"Bad Data", "DataToString does not understand this type of Data"}]]
	]

(*
dtsTextData accepts inline Cells that come from TextData
*)
dtsTextData[s_String] := s
dtsTextData[c_Cell] := DataToString[c[[1]]]
(*seems kind of inconsistent that things like ButtonBox can just appear by themselves in TextData*)
dtsTextData[b_ButtonBox] := dtsBoxData[b]
dtsTextData[t_] := Throw[t, MUnitErrorTag[{"Bad Text", "DataToString does not understand this type of Text"}]]

dtsBoxData[ButtonBox[b_, ___]] := dtsBoxData[b]
dtsBoxData[FormBox[b_, _]] := dtsBoxData[b]
dtsBoxData[RowBox[list_List]] := StringJoin[dtsBoxData /@ list]
dtsBoxData[StyleBox[b_, ___]] := dtsBoxData[b]
dtsBoxData[SubscriptBox[a_String, b_String]] := a<>"_"<>b
dtsBoxData[SuperscriptBox[a_String, b_String]] := a<>"^"<>b
dtsBoxData[SqrtBox[a_String]] := "@"<>a
dtsBoxData[s_String] := s
dtsBoxData[b_] := Throw[Head[b], MUnitErrorTag[{"Bad Box", "DataToString does not understand this type of box"}]]


End[]
