(* ::Package:: *)

(* ::Title:: *)
(*Typesetting*)

(* ::Section:: *)
(*Information*)

`Information`$FormattingId = "$Id: Formatting.m,v 1.6 2010/10/12 16:41:57 brenton Exp $"

(* ::Section:: *)
(*Package*)

(*

RowBox[{RowBox[{"f", "[", "x_", "]"}], ":=", "\[IndentingNewLine]",
RowBox[{"Module", "[", RowBox[{RowBox[{"{", "y", "}"}], ",", "\[IndentingNewLine]",
RowBox[{RowBox[{"y", "=", RowBox[{"x", "+", "1"}]}], ";", "\[IndentingNewLine]",
RowBox[{"y", "^", "2"}]}]}], "\[IndentingNewLine]",
"]"}]}]

*)

Begin["`Package`"]

(*boxes -> ReadableForm string*)
BoxesToReadableFormString::usage =
"BoxesToString[boxes] converts boxes to a ReadableForm string."

(*data(boxes) -> LinearOutputForm string*)
DataToString::usage =
"DataToString[data] outputs strings of the LinearOutputForm of the given data."

End[]

(* ::Section:: *)
(*Private*)

Begin["`Formatting`Private`"]

(*
takes single Integer, representing a Unicode code point
leave ASCII chars untouched, while converting non-ASCII to ASCII
*)
codePointToReadableForm =
	If[# > 127,
		StringTake[ToString[FromCharacterCode[#], InputForm, CharacterEncoding -> "ASCII"], {2, -2}]
		,
		FromCharacterCode[#]
	]&

stringToReadableFormString[s_String] :=
StringJoin[codePointToReadableForm /@ ToCharacterCode[Characters[s], "Unicode"][[All, 1]]]

(*
Second arg to BoxesToReadableFormString is the lexical environment.
Currently, it is only used to communicate the immediate parent box 
*)

(*
this is to get around the issue that 1`-2 and 1` - 2 are two very different expressions
*)
BoxesToReadableFormString[RowBox[{op1_, "-", op2_}], env_:{}] :=
	StringJoin[{BoxesToReadableFormString[op1, {RowBox}], " - ", BoxesToReadableFormString[op2, {RowBox}]}]
BoxesToReadableFormString[RowBox[{op1_, "+", op2_}], env_:{}] :=
	StringJoin[{BoxesToReadableFormString[op1, {RowBox}], " + ", BoxesToReadableFormString[op2, {RowBox}]}]

BoxesToReadableFormString[FractionBox[op1_, op2_], env_:{}] :=
	Module[{s},
		s = StringJoin[{BoxesToReadableFormString[op1, {FractionBox}], "/", BoxesToReadableFormString[op2, {FractionBox}]}];
		If[env =!= {},
			s = "(" <> s <> ")";
		];
		s
	]

BoxesToReadableFormString[StyleBox[content_, ___], env_:{}] :=
	BoxesToReadableFormString[content, {StyleBox}]
BoxesToReadableFormString[RowBox[row_], env_:{}] :=
	StringJoin[BoxesToReadableFormString[#, {RowBox}]& /@ row]
BoxesToReadableFormString["\[IndentingNewLine]", env_:{}] =
	"\n"
BoxesToReadableFormString[s_String, env_:{}] :=
	stringToReadableFormString[s]
(* can occur when there are multiple lines in a cell *)
BoxesToReadableFormString[list_List, env_:{}] :=
	BoxesToReadableFormString[#, {List}]& /@ list
BoxesToReadableFormString[box_, env_:{}] :=
	Block[{$OutputForms = {}},
		ToExpression[box, StandardForm,
			Function[{e}, stringToReadableFormString[ToString[Unevaluated[e], InputForm]], {HoldFirst}]]
	]

(*
DataToString

text -> return
boxes like SqrtBox, FormBox, etc. -> return pseudo-linear syntax string thing 
boxes like GraphicsBox, GridBox -> -GraphicsBox-, -GridBox-, etc.
*)

DataToString::baddata =
"Invalid data given to DataToString: `1`"

DataToString::badtext =
"Invalid text data given to DataToString: `1`"

DataToString::badbox =
"Invalid box data given to DataToString: `1`"

(*
strings go through because Cell["title", "Title"] is like implicit TextData
*)

DataToString[d_] :=
	Switch[d,
		_String,
		d
		,
		TextData[_List, ___],
		StringJoin[dtsTextData /@ d[[1]]]
		,
		_TextData,
		dtsTextData[d[[1]]]
		,
		_BoxData,
		dtsBoxData[d[[1]]]
		,
		_,
		Throw[{"Value" -> d, "Messages" -> {"Bad Data", "DataToString does not understand this type of Data"}}, MUnitErrorTag]
	]

(*
dtsTextData accepts inline Cells that come from TextData
*)
dtsTextData[s_String] := s
dtsTextData[c_Cell] := DataToString[c[[1]]]
(*seems kind of inconsistent that things like ButtonBox can just appear by themselves in TextData*)
dtsTextData[b_ButtonBox] := dtsBoxData[b]
dtsTextData[b_StyleBox] := dtsBoxData[b]
dtsTextData[t_] := Throw[{"Value" -> t, "Messages" -> {"Bad Text", "DataToString does not understand this type of Text"}}, MUnitErrorTag]

dtsBoxData[ButtonBox[b_, ___]] := dtsBoxData[b]
dtsBoxData[FormBox[b_, _]] := dtsBoxData[b]
dtsBoxData[RowBox[list_List]] := StringJoin[dtsBoxData /@ list]
dtsBoxData[StyleBox[b_, ___]] := dtsBoxData[b]
dtsBoxData[SubscriptBox[a_String, b_String]] := a<>"_"<>b
dtsBoxData[SuperscriptBox[a_String, b_String]] := a<>"^"<>b
dtsBoxData[SqrtBox[a_String]] := "@"<>a
dtsBoxData[s_String] := s
dtsBoxData[b_] := Throw[{"Value" -> Head[b], "Messages" -> {"Bad Box", "DataToString does not understand this type of box"}}, MUnitErrorTag]


End[]
