(* ::Package:: *)

(* ::Title:: *)
(*Notebooks*)

(* ::Section:: *)
(*Information*)

`Information`$NotebooksId = "$Id: Notebooks.m,v 1.47 2010/10/12 16:41:33 brenton Exp $"

(* ::Section:: *)
(*Public*)


NotebookToTests::usage =
""

CellsToTest::usage =
""


TestCellEvaluationFunction::usage =
""

OtherCellEvaluationFunction::usage =
""


(* ::Section:: *)
(*Package*)

Begin["`Package`"]

cellEvaluationFunction

End[]

(* ::Section:: *)
(*Private*)

Begin["`Notebooks`Private`"]

handleTestOptionsContents[box_] :=
	Module[{stripped, stripped1},
		stripped =
			If[MatchQ[box, RowBox[{"{", ___, "}"}]],
				stripped1 = box[[1]];
				If[Length[stripped1] > 2,
					Take[stripped1, {2, -2}]
					,
					{}
				]
				,
				{box}
			];
		Switch[stripped,
			{}, Sequence @@ {}
			,
			{RowBox[{_String ..}]}, Sequence @@ Join[stripped, {",", "\[IndentingNewLine]"}]
			,
			{RowBox[__]}, Sequence @@ Join[stripped[[1, 1]], {",", "\[IndentingNewLine]"}]
			,
			_, Throw[{"Value" -> stripped, "Messages" -> {"Test Cell", "Invalid Test Options"}}, MUnitErrorTag]
		]
	]

getRequireBoxes[{opts___}] :=
	Module[{tags, interfaceEnvironments, languages, products, disabled},
		tags = CellTags /. Flatten[{opts}] /. CellTags -> {};
		(* normalize tags *)
		tags = Flatten[{tags}];
		interfaceEnvironments = Intersection[{"Macintosh", "Windows", "X"}, tags];
		languages = Intersection[{"English", "Japanese", "ChineseSimplified", "Spanish"}, tags];
		products = Intersection[{"Mathematica", "MathematicaPlayer", "MathematicaPlayerPro"}, tags];
		disabled = MemberQ[tags, "False" | "Obsolete" | "Disabled"];
		Which[
			disabled,
			"False"
			,
			Length[interfaceEnvironments] == 1,
			RowBox[{"$InterfaceEnvironment", "==", ToBoxes[interfaceEnvironments[[1]]]}]
			,
			interfaceEnvironments != {},
			RowBox[{"MemberQ", "[", RowBox[{ToBoxes[interfaceEnvironments], ",", "$InterfaceEnvironment"}], "]"}]
			,
			Length[languages] == 1,
			RowBox[{"$Language", "==", ToBoxes[languages[[1]]]}]
			,
			languages != {},
			RowBox[{"MemberQ", "[", RowBox[{ToBoxes[languages], ",", "$Language"}], "]"}]
			,
			Length[products] == 1,
			RowBox[{RowBox[{"(", RowBox[{"\"ProductIDName\"", "/.", "$ProductInformation"}], ")"}], "==", ToBoxes[products[[1]]]}]
			,
			products != {},
			RowBox[{"MemberQ", "[", RowBox[{ToBoxes[products], ",", RowBox[{"\"ProductIDName\"", "/.", "$ProductInformation"}]}], "]"}]
			,
			True,
			"True"
		]
	]

(* TODO: try to remember why I added "Subtitle" and "Subsubtitle" in the first place, since they are not grouping cell styles... *)
sectionStylePattern =
	"Subtitle" | "Subsubtitle" | "Section" | "Subsection" | "Subsubsection" | "TestSection" |
	(* from function paclet notebook *)
	"TemplatesSection"

platformStylePattern =
	"PlatformSection" | "MacintoshSection" | "WindowsSection" | "XSection"

cellParser[cells_List, OptionsPattern[{testParser, sectionParser}]] :=
	Module[{testIDTitleString, testIDFormat, preserve},
		{testIDTitleString, testIDFormat, preserve} = OptionValue[{TestIDTitleString, TestIDFormat, PreserveDataInSections}];
		ReplaceAll[cells,
			{
				Cell[CellGroupData[{Cell[name_String, "Title", opts___], rest___Cell}, _]] :>
					sectionParser[name, {opts}, rest, TestIDTitleString -> StringReplace[name, ".nb" -> ""], TestIDFormat -> testIDFormat, PreserveDataInSections -> preserve]
				,
				Cell[CellGroupData[{Cell[name_, "Title", opts___], rest___Cell}, _]] :>
					sectionParser[name, {opts}, rest, TestIDTitleString -> DataToString[name], TestIDFormat -> testIDFormat, PreserveDataInSections -> preserve]
				,
				Cell[CellGroupData[{Cell[name_, (*style:*)sectionStylePattern, opts___], rest___Cell}, _]] :>
					sectionParser[name, {opts}, rest, TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat, PreserveDataInSections -> preserve]
				,
				Cell[CellGroupData[{Cell[name_, style:platformStylePattern, (*opts*)___], rest___Cell}, _]] :>
					platformSectionParser[name, style, rest, TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat]
				,
				(* handle non-grouped sections *)
				Cell[name_String, "Title", opts___] :>
					sectionParser[name, {opts}, TestIDTitleString -> StringReplace[name, ".nb" -> ""], TestIDFormat -> testIDFormat, PreserveDataInSections -> preserve]
				,
				Cell[name_, "Title", opts___] :>
					sectionParser[name, {opts}, TestIDTitleString -> DataToString[name], TestIDFormat -> testIDFormat, PreserveDataInSections -> preserve]
				,
				Cell[name_, (*style:*)sectionStylePattern, opts___] :>
					sectionParser[name, {opts}, TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat, PreserveDataInSections -> preserve]
				,
				Cell[name_, style:platformStylePattern, (*opts*)___] :>
					platformSectionParser[name, style, TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat]
				,
				Cell[text_String, "Text", (*opts*)___] :>
					textParser[text]
				,
				Cell[CellGroupData[{first___Cell, test:Cell[_, "Test", ___], rest:___Cell}, _]] :>
					testParser[{first, test, rest}, TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat]
				,
				test:Cell[_, "Test", ___] :>
					testParser[{test}, TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat]
				,
				Cell[CellGroupData[{Cell[BoxData[e_], "Environ", opts___], ___Cell}, _]] :>
					Catch[
						validateBoxes[e]
						,
						MUnitErrorTag
						,
						Function[{rules, tag},
							Throw[Append[rules, "CellID" -> OptionValue[Cell, {opts}, CellID]], MUnitErrorTag]
						]
					]
				,
				Cell[BoxData[e_], "Environ", opts___] :>
					Catch[
						validateBoxes[e]
						,
						MUnitErrorTag
						,
						Function[{rules, tag},
							Throw[Append[rules, "CellID" -> OptionValue[Cell, {opts}, CellID]], MUnitErrorTag]
						]
					]
				,
				Cell[r:RawData[_], ___] :>
					Throw[{"Value" -> r, "Messages" -> {"Cell Parsing", "Read RawData"}}, MUnitErrorTag]
				,
				Cell[BoxData[e_], "TestOptions", ___] :>
					Throw[{"Value" -> e, "Messages" -> {"Cell Parsing", "Orphaned TestOptions cell"}}, MUnitErrorTag]
				,
				Cell[BoxData[e_], "ExpectedOutput", ___] :>
					Throw[{"Value" -> e, "Messages" -> {"Cell Parsing", "Orphaned ExpectedOutput cell"}}, MUnitErrorTag]
				,
				Cell[BoxData[e_], "ExpectedMessage", ___] :>
					Throw[{"Value" -> e, "Messages" -> {"Cell Parsing", "Orphaned ExpectedMessage cell"}}, MUnitErrorTag]
				,
				Cell[BoxData[e_], "ExpectedMessages", ___] :>
					Throw[{"Value" -> e, "Messages" -> {"Cell Parsing", "Orphaned ExpectedMessages cell"}}, MUnitErrorTag]
				,
				Cell[___] :>
					Sequence[]
			}
		]
	]

Options[sectionParser] = {PreserveDataInSections -> True}

sectionParser[name_, cellOpts_, rest___Cell, opts:OptionsPattern[{testParser, sectionParser}]] :=
	Module[{requireBoxes = getRequireBoxes[cellOpts], preserve = OptionValue[PreserveDataInSections]},
		Sequence @@ {
			RowBox[{
				"BeginTestSection", "[",
					ToBoxes[If[preserve, name, DataToString[name]]],
					If[requireBoxes === "True", Sequence @@ {}, Sequence @@ {",", requireBoxes}],
				"]"
			}]
			,
			Sequence @@ Flatten[{cellParser[{rest}, opts]}]
			,
			RowBox[{"EndTestSection", "[", "]"}]
		}
	]

platformSectionParser[(*name*)_String, style_String, rest___Cell, OptionsPattern[testParser]] :=
	Module[{testIDTitleString, testIDFormat},
		{testIDTitleString, testIDFormat} = OptionValue[{TestIDTitleString, TestIDFormat}];
		Sequence @@ {
			RowBox[{
				"BeginTestSection", "[",
					RowBox[{
						ToBoxes[style], ",", RowBox[{"$InterfaceEnvironment", "==", ToBoxes[StringReplace[style, p__ ~~ "Section" :> p]]}]
					}],
				"]"
			}]
			,
			Sequence @@ Flatten[{cellParser[{rest}, TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat]}]
			,
			RowBox[{"EndTestSection", "[", "]"}]
		}
	]


Options[testParser] = {TestIDTitleString -> "Untitled", TestIDFormat -> "CellID"}

testParser[list_List, (*opts:*)OptionsPattern[]] :=
	Module[{testIDTitleString, testIDFormat},
		{testIDTitleString, testIDFormat} = OptionValue[{TestIDTitleString, TestIDFormat}];
		ReplaceAll[list, {
			(* single ExpectedMessages cell *)
			{
				t:Cell[BoxData[_], "Test", topts___?OptionQ],
				to:Cell[BoxData[_], "TestOptions", ___]:Cell[BoxData[RowBox[{"{", "}"}]], "TestOptions"],
				em:Cell[BoxData[_], "ExpectedMessages", ___]:Cell[BoxData[RowBox[{"{", "}"}]], "ExpectedMessages"],
				eo:Cell[BoxData[_], "ExpectedOutput", ___]:Cell[BoxData["True"], "ExpectedOutput"],
				(* if a test result cell is present, just ignore it *)
				tr:Cell[___, "TestSuccess" | "TestFailure" | "TestMessagesFailure" | "TestError", ___]:Cell[BoxData[""], "TestSuccess"]
			} :> testParser0[
					t[[1,1]], {topts},
					to[[1,1]], List@@to[[3;;]],
					em[[1,1]], List@@em[[3;;]],
					eo[[1,1]], List@@eo[[3;;]], TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat]
			,
			(* multiple ExpectedMessage cells *)
			{
				t:Cell[BoxData[_], "Test", topts___?OptionQ],
				to:Cell[BoxData[_], "TestOptions", ___]:Cell[BoxData[RowBox[{"{", "}"}]], "TestOptions"],
				ems:Cell[BoxData[_], "ExpectedMessage", ___]..,
				eo:Cell[BoxData[_], "ExpectedOutput", ___]:Cell[BoxData["True"], "ExpectedOutput"],
				(* if a test result cell is present, just ignore it *)
				tr:Cell[___, "TestSuccess" | "TestFailure" | "TestMessagesFailure" | "TestError", ___]:Cell[BoxData[""], "TestSuccess"]
			} :> testParser0[
					t[[1,1]], {topts},
					to[[1,1]], List@@to[[3;;]],
					(*TODO: dig through linear syntax of message and extract arguments and use newer argument checking functionality*)
					(*TODO: better error checking for mal-formed messages cells: empty cells but also edited *)
					RowBox[{"{", RowBox[Riffle[#[[1, 1, 1, 1, 1]] & /@ {ems}, ","]], "}"}], {},
					eo[[1,1]], List@@eo[[3;;]], TestIDTitleString -> testIDTitleString, TestIDFormat -> testIDFormat]
			,
			_ :> Throw[{"Value" -> list, "Messages" -> {"Test Parsing", "The order of the cells is not understood by MUnit"}}, MUnitErrorTag]
		}]
	]

testParser0[tb_, topts_, tob_, toopts_, emb_, emopts_, eob_, eoopts_, OptionsPattern[testParser]] :=
	Module[{testIDTitleString, testIDFormat, dateString, cellID, cellIDString, testID},
		{testIDTitleString, testIDFormat} = OptionValue[{TestIDTitleString, TestIDFormat}];
		dateString = DateString[Flatten[CellChangeTimes /. Flatten[topts] /. CellChangeTimes :> {AbsoluteTime[]}][[1]], {"Year", "Month", "Day"}, TimeZone -> 0];
		cellID = CellID /. topts;
		If[cellID === CellID,
			Throw[{"Value" -> tb, "Messages" -> {"Test Parsing", "Test Cell must have a CellID"}}, MUnitErrorTag]
		];
		cellIDString = IntegerString[cellID, 36, 7];
		testID =
			Switch[testIDFormat,
				"CellID",
				cellID
				,
				"WRI",
				testIDTitleString <> "-" <> dateString <> "-" <> cellIDString
				,
				_,
				cellID
			];
		(*
		kind of a hack. run all of the boxes through the parser once, in order to get any errors propagated
		do each one separately so that we can add respective CellIDs
		*)
		Catch[
			validateBoxes[tb]
			,
			MUnitErrorTag
			,
			Function[{rules, tag},
				Throw[Append[rules, "CellID" -> OptionValue[Cell, topts, CellID]], MUnitErrorTag]
			]
		];
		Catch[
			validateBoxes[tob]
			,
			MUnitErrorTag
			,
			Function[{rules, tag},
				Throw[Append[rules, "CellID" -> OptionValue[Cell, toopts, CellID]], MUnitErrorTag]
			]
		];
		Catch[
			validateBoxes[emb]
			,
			MUnitErrorTag
			,
			Function[{rules, tag},
				Throw[Append[rules, "CellID" -> OptionValue[Cell, emopts, CellID]], MUnitErrorTag]
			]
		];
		Catch[
			validateBoxes[eob]
			,
			MUnitErrorTag
			,
			Function[{rules, tag},
				Throw[Append[rules, "CellID" -> OptionValue[Cell, eoopts, CellID]], MUnitErrorTag]
			]
		];
		RowBox[{
			"Test", "[", RowBox[{"(*", " ", ToBoxes[++$notebookTestIndex], " ", "*)"}], "\[IndentingNewLine]",
				RowBox[{
					(* provide a courtesy for those horribly mis-guided people who have multiple inputs in a Test cell *)
					If[Head[tb] === List,
						(* split into lines *)
						Module[{lines = Split[tb, #1 == #2 || #2 == "\[IndentingNewLine]" || #2 == "\n" &]},
							(Function[line,
								(* line is {RowBox[...], "\[IndentingNewLine]"} *)
								(* may or may not have semi-colons at the end of the lines *)
								If[MatchQ[line, {RowBox[{__, ";"}], "\[IndentingNewLine]"|"\n"}],
									(* already has semi-colon *)
									line
									,
									(* wrap with semi-colon *)
									{RowBox[{line[[1]], ";"}], "\[IndentingNewLine]"}
								]
							] /@ Most[lines]) ~Join~ Last[lines]
						]
						,
						tb
					],
					"\[IndentingNewLine]",
					",", "\[IndentingNewLine]",
					eob, "\[IndentingNewLine]",
					",", "\[IndentingNewLine]",
					If[emb === RowBox[{"{", "}"}], Sequence @@ {}, Sequence @@ {emb, "\[IndentingNewLine]", ",", "\[IndentingNewLine]"}],
					handleTestOptionsContents[tob],
					RowBox[{"TestID", "->", ToBoxes[testID]}],
					Module[{tags = Flatten[{CellTags /. Flatten[topts] /. CellTags -> {}}]},
						If[tags === {}, Sequence @@ {}, Sequence @@ {"\[IndentingNewLine]", ",", "\[IndentingNewLine]", RowBox[{"TestTags", "\[Rule]", ToBoxes[tags]}]}]
					]
				}], "\[IndentingNewLine]",
			"]"
		}]
	]

(* boxParser converts from boxes -> expressions *)
validateBoxes[boxes_] :=
	Quiet[
		If[boxes === "",
			Throw[{"Value" -> boxes, "Messages" -> {"Box Parsing", "The boxes are empty"}}, MUnitErrorTag]
		];
		Module[{oldContext, oldContextPath},
			Internal`WithLocalSettings[
				oldContext = $Context;
				oldContextPath = $ContextPath;
				$Context = "MUnit`Notebooks`Private`Sandbox`";
				$ContextPath = {"MUnit`Notebooks`Private`Sandbox`", "System`"};
				,
				Check[
					Quiet[
						(* do ToExpression to generate any messages *)
						ToExpression[boxes, StandardForm, HoldComplete]
						,
						{Syntax::noinfo}
					]
					,
					Throw[{"Value" -> boxes, "Messages" -> {"Box Parsing", "The boxes could not be parsed"}}, MUnitErrorTag]
				]
				,
				$Context = oldContext;
				$ContextPath = oldContextPath
			]
		];
		boxes
	]

textParser[text_String] :=
	"(* ::Text:: *)\n(*"<>text<>"*)"

CellsToTest[cells:{___Cell}] :=
	cellsToTestsString[{Cell[CellGroupData[cells, Open]]}]

NotebookToTests[nb_NotebookObject, opts:OptionsPattern[sectionParser]] :=
	Module[{nbg, cells, title},
		nbg = NotebookGet[nb];
		If[nbg === $Failed,
			(* nb is an invalid (closed) notebook *)
			Throw[{"Value" -> nb, "Messages" -> {"NotebookGet", "$Failed"}}, MUnitErrorTag]
		];
		cells = nbg[[1]];
		(* insert a Title cell group if it doesn't exist already *)
		If[!MatchQ[cells, {Cell[CellGroupData[{Cell[_, "Title", ___], __}, _]]}],
			title = WindowTitle /. AbsoluteOptions[nb, WindowTitle];
			cells = {Cell[CellGroupData[{Cell[title, "Title"], Sequence @@ cells}, Open]]}
		];
		cellsToTestsString[cells, opts]
	]

(*
useful for running notebooks without a front end
*)
NotebookToTests[nb:Notebook[oldCells_, nbOpts:OptionsPattern[]], opts:OptionsPattern[sectionParser]] :=
	Module[{title, cells},
		(* insert a Title cell group if it doesn't exist already *)
		If[!MatchQ[oldCells, {Cell[CellGroupData[{Cell[_, "Title", ___], __}, _]]}],
			(* cannot use OptionValue mechanism for getting value of WindowTitle, since it complains loudly about
			unrecognized options (since in a stand-alone kernel Options[Notebook] == {} ) *)
			title = WindowTitle /. Flatten[{nbOpts}] /. WindowTitle -> Automatic;
			cells = {Cell[CellGroupData[{Cell[ToBoxes[title], "Title"], Sequence @@ oldCells}, Open]]}
			,
			cells = oldCells
		];
		cellsToTestsString[cells, opts]
	]

cellsToTestsString[cells_, opts:OptionsPattern[sectionParser]] :=
	Module[{parsed},
		Block[{$notebookTestIndex = 0},
			parsed = cellParser[cells, TestIDFormat -> "WRI", opts];
			If[parsed == {},
				Throw[{"Value" -> cells, "Messages" -> {"Cell Parsing", "Cell Parser returned {}"}}, MUnitErrorTag]
			];
			StringDrop[StringJoin[BoxesToReadableFormString[#] <> "\n\n" & /@ parsed], -1]
		]
	]

TestCellEvaluationFunction[_, _] :=
	Module[{nb, grouped},
		Catch[
			nb = EvaluationNotebook[];
			If[nb === $Failed,
				Throw[{"Value" -> Null, "Messages" -> {"Test", "Internal Error: EvaluationNotebook[] returned $Failed"}}, MUnitErrorTag]
			];
			SelectionMove[nb, All, EvaluationCell];
			grouped = "FirstCellInGroup" /. Developer`CellInformation[nb][[1]];
			If[grouped,
				SelectionMove[nb, All, CellGroup]
			];
			cellEvaluationFunction[nb, grouped]
			,
			MUnitErrorTag
			,
			Function[{rules, tag},
				Module[{msgs = "Messages" /. rules, val = "Value" /. rules},
					MessageDialog[Column[{msgs[[1]], Style[msgs[[2]], Bold], val}], WindowTitle -> "MUnit Error"];
					(* return Null to prevent the MessageDialog's NotebookObject from being returned in the test notebook *)
					Null
				]
			]
		]
	]

OtherCellEvaluationFunction[_, _] :=
	Module[{nb},
		Catch[
			nb = EvaluationNotebook[];
			If[nb === $Failed,
				Throw[{"Value" -> Null, "Messages" -> {"Test", "Internal Error: EvaluationNotebook[] returned $Failed"}}, MUnitErrorTag]
			];
			SelectionMove[nb, All, EvaluationCell];
			SelectionMove[nb, All, CellGroup];
			If[Developer`CellInformation[nb] === $Failed,
				(* cells are not grouped with a Test cell *)
				SelectionMove[nb, After, EvaluationCell];
				Throw[{"Value" -> Null, "Messages" -> {"Test", "Malformed Test CellGroup"}}, MUnitErrorTag]
				,
				cellEvaluationFunction[nb, True]
			]
			,
			MUnitErrorTag
			,
			Function[{rules, tag},
				Module[{msgs = "Messages" /. rules, val = "Value" /. rules},
					MessageDialog[Column[{msgs[[1]], Style[msgs[[2]], Bold], val}], WindowTitle -> "MUnit Error"];
					(* return Null to prevent the MessageDialog's NotebookObject from being returned in the test notebook *)
					Null
				]
			]
		]
	]


(* TODO: rewrite using CellPrint to be nice, and also some how simulate cell evaluating during test evaluation *)
(*
Precondition: if there is a cell group, then it is selected, if there is only a Test cell, then it is selected
*)
cellEvaluationFunction[nb_NotebookObject, grouped:(True|False)] :=
	Module[{selection, cells, test, tr},
		selection = NotebookRead[nb];
		(* cells is a list of cells, {test} or {test, eo} etc. *)
		cells =
			If[grouped,
				DeleteCases[Cases[{selection}, Cell[CellGroupData[cells:{___Cell}, _]] :> cells][[1]],
					Cell[_, Except["Test" | "TestOptions" | "ExpectedMessages" | "ExpectedMessage" | "ExpectedOutput" ], ___],
					Infinity]
				,
				{selection}
			];
		(* no more harm in writing CellGroupData with one cell, if selection was only a Test cell *)
		NotebookWrite[nb, Cell[CellGroupData[cells, Open]], After];
		Catch[
			test = CellsToTest[cells];
			(* from string -> expression *)
			tr = ToExpression[test];
			If[TestResultQ[tr],
				Replace[FailureMode[tr], {
					"Success" :>
						NotebookWrite[nb, Cell[BoxData[RowBox[{ToBoxes[tr], rerunTestButtonBox[nb, tr]}]], "TestSuccess"]]
					 ,
					"MessagesFailure" :>
						NotebookWrite[nb, Cell[BoxData[RowBox[{ToBoxes[tr], rerunTestButtonBox[nb, tr], replaceMessagesButtonBox[nb, tr]}]], "TestMessagesFailure"]]
					,
					"Failure" :>
						NotebookWrite[nb, Cell[BoxData[RowBox[{ToBoxes[tr], rerunTestButtonBox[nb, tr], replaceOutputButtonBox[nb, tr]}]], "TestFailure"]]
					,
					"Error" :>
						NotebookWrite[nb, Cell[BoxData[RowBox[{ToBoxes[tr], rerunTestButtonBox[nb, tr]}]], "TestError"]]
					,
					m_ :> Throw[{"Value" -> m, "Messages" -> {"Test", "Unrecognized FailureMode"}}, MUnitErrorTag]
				}]
				,
				(*
				if tr is not a test result, then there was something very bad
				*)
				Throw[{"Value" -> tr, "Messages" -> {"Test", "Invalid TestResult"}}, MUnitErrorTag]
			];
			,
			MUnitErrorTag
			,
			Function[{rules, tag},
				Module[{msgs = "Messages" /. rules, val = "Value" /. rules},
					NotebookWrite[nb, Cell[BoxData[ToBoxes[Column[{msgs[[1]], Style[msgs[[2]], Bold], val}], StandardForm]], "TestError"]];
				]
			]
		]
	]


End[]
