(* ::Package:: *)

(* ::Title:: *)
(*Palette*)

(* ::Section:: *)
(*Information*)

`Information`$PaletteId = "$Id: Palette.m,v 1.9 2010/04/15 03:49:12 brenton Exp $"

(* ::Section:: *)
(*Public*)

PaletteClear::usage =
""

PaletteConvertCellGroup::usage =
""

PaletteConvertNotebook::usage =
""

PaletteNewTestNotebook::usage =
""

PaletteRun::usage =
""

PaletteAbort::usage =
""

PaletteSaveAs::usage =
""

(* ::Section:: *)
(*Package*)

Begin["`Package`"]



End[]

(* ::Section:: *)
(*Private*)

Begin["`Palette`Private`"]



PaletteConvertCellGroup[nb_NotebookObject] :=
	Module[{selection},
		Catch[
			checkStyleDefinitions[nb];
			selection = NotebookRead[nb];
			If[MatchQ[selection, Cell[_CellGroupData]],
				(* make selection conform to {Cell[_CellGroupData]..} pattern *)
				selection = {selection};
			];
			If[!MatchQ[selection, {Cell[_CellGroupData]..}],
				Throw[{"Value" -> selection, "Messages" -> {"CellGroup Converting", "Current selection is not 1 or more cell groups"}}, MUnitErrorTag]
			];
			Scan[convertCellGroup[nb, #]&, selection];
			,
			MUnitErrorTag
			,
			Function[{rules, tag},
				Module[{msgs = "Messages" /. rules, val = "Value" /. rules},
					MessageDialog[Column[{msgs[[1]], Style[msgs[[2]], Bold], val, If[Length[msgs] >= 3, msgs[[3]], Spacer[{0, 0}]]}], WindowTitle -> "MUnit Error"]
				]
			]
		]
	]

checkStyleDefinitions[nb_] :=
	If[CurrentValue[nb, {StyleDefinitions, "Test"}] == {} ||
		CurrentValue[nb, {StyleDefinitions, "ExpectedMessage"}] == {} ||
		CurrentValue[nb, {StyleDefinitions, "ExpectedOutput"}] == {}
		,
		Throw[{"Value" -> Null, "Messages" -> {"CellGroup Converting", "Current stylesheet does not have the necessary styles defined."}}, MUnitErrorTag]
	]

convertCellGroup[nb_, Cell[CellGroupData[cells:{__Cell}, _]]] :=
	Module[{},
		Scan[checkCellID, cells];	
		Scan[convertCell[nb, #]&, cells]
	]

checkCellID[Cell[_, _, ___, CellID -> _Integer, ___]] := True
checkCellID[Cell[a_, s_, opts___]] := Throw[{"Value" -> Cell[a, s, opts], "Messages" -> {"CellGroup Converting", "Missing CellID"}}, MUnitErrorTag]
checkCellID[x_] := Throw[{"Value" -> x, "Messages" -> {"CellGroup Converting", "Unrecognized cell"}}, MUnitErrorTag]

convertCell[nb_, Cell[a_, s_, opts___]] :=
	Module[{
		newStyle = s /. {
			"Input" -> "Test",
			"Code" -> "Test",
			"Environ" -> "Test",
			"Message" -> "ExpectedMessage",
			"Output" -> "ExpectedOutput",
			w_ :> Throw[{"Value" -> w, "Messages" -> {"CellGroup Converting", "Unexpected Cell Style"}}, MUnitErrorTag]},
		id = Cases[{opts}, (CellID -> id_) :> id][[1]]},
		NotebookFind[nb, id, All, CellID];
		NotebookWrite[nb, Cell[a, newStyle, opts]]
	]


PaletteConvertNotebook[nb_NotebookObject] :=
	Module[{n},
		SetOptions[nb, StyleDefinitions -> FrontEnd`FileName[{"MUnit"}, "MUnit.nb"]];
		(* get-and-put to add any CellIDs that need to be added *)
		n = NotebookGet[nb];
		NotebookPut[n, nb];
	]

PaletteNewTestNotebook[] :=
	NotebookPut[Get[ToFileName[{$MUnitDirectory, "FrontEnd", "SystemResources"}, "Template.nb"]]]

PaletteRun[nb_NotebookObject] :=
	Module[{title},
		title = WindowTitle /. AbsoluteOptions[nb, WindowTitle];
		TestRun[nb, TestRunTitle -> title, Loggers -> {NotebookLogger[nb]}]
	]

PaletteAbort[nb_NotebookObject] :=
	MathLink`CallFrontEnd[FrontEnd`EvaluatorAbort[Automatic]]

$oldFileName

PaletteSaveAs[nb_NotebookObject] :=
	Module[{stream, windowTitle, tests},
		Catch[
			windowTitle = WindowTitle /. AbsoluteOptions[nb, WindowTitle];
			$oldFileName =
				SystemDialogInput["FileSave", {
					If[ValueQ[$oldFileName] && $oldFileName =!= $Canceled, ToFileName[{DirectoryName[$oldFileName]}, windowTitle], windowTitle],
					{"Mathematica Package Test File (*.mt)" -> {"*.mt"}}
				}];
			If[$oldFileName =!= $Canceled,
				tests = NotebookToTests[InputNotebook[], PreserveDataInSections -> False];
				stream = OpenWrite[$oldFileName];
				If[stream === $Failed,
					Throw[{"Value" -> $oldFileName, "Messages" -> {"PaletteSaveAs", "OpenWrite returned $Failed"}}, MUnitErrorTag]
				];
				(* write out package header comment to let the front end recognize this as a package file *)
				WriteString[stream, "(* ::Package:: *)\n\n"];
				(* probably ok to not check that NotebookToTests returned $Failed *)
				WriteString[stream, tests];
				Close[stream]
			]
			,
			MUnitErrorTag
			,
			Function[{rules, tag},
				Module[{msgs = "Messages" /. rules, val = "Value" /. rules},
					MessageDialog[Column[{msgs[[1]], Style[msgs[[2]], Bold], val, If[Length[msgs] >= 3, msgs[[3]], Spacer[{0, 0}]]}], WindowTitle -> "MUnit Error"]
				]
			]
		];
	]

PaletteClear[nb_NotebookObject] :=
	(
		If[NotebookFind[nb, "TestSuccess", All, CellStyle] =!= $Failed,
			NotebookDelete[nb]];
		If[NotebookFind[nb, "TestFailure", All, CellStyle] =!= $Failed,
			NotebookDelete[nb]];
		If[NotebookFind[nb, "TestMessagesFailure", All, CellStyle] =!= $Failed,
			NotebookDelete[nb]];
		If[NotebookFind[nb, "TestError", All, CellStyle] =!= $Failed,
			NotebookDelete[nb]];
		If[NotebookFind[nb, "Output", All, CellStyle] =!= $Failed,
			NotebookDelete[nb]]
	)



End[]
