(* ::Package:: *)

(* ::Title:: *)
(*Test*)

(* ::Section:: *)
(*Information*)

`Information`$TestId = "$Id: Test.m,v 1.52 2010/07/13 22:43:52 brenton Exp $"

(* ::Section:: *)
(*Public*)

Test::usage =
"Test[input] tests whether input evaluates to True and no messages are generated.\n
Test[input, expected] tests whether input evaluates to expected and no messages are generated.\n
Test[input, expected, messages] tests whether input evaluates to expected and the given messages are generated.\n
Test is also a selector for TestResult."

TestMatch::usage =
"TestMatch[input, expected] and TestMatch[input, expected, messages] are like the corresponding versions of Test except they test MatchQ[input, expected] instead of SameQ."

TestStringMatch::usage =
"TestStringMatch[input, expected] and TestStringMatch[input, expected, messages] are like the corresponding versions of Test except they test StringMatchQ[input, expected] instead of SameQ."

TestFree::usage =
"TestFree[input, expected] and TestFree[input, expected, messages] are like the corresponding versions of Test except they test FreeQ[input, expected] instead of SameQ."

TestStringFree::usage =
"TestStringFree[input, expected] and TestStringFree[input, expected, messages] are like the corresponding versions of Test except they test StringFreeQ[input, expected] instead of SameQ."

TestResultObject::usage =
"TestResultObject is the head of the expression returned by Test and related functions. It is passed to loggers.
TestResultObject contains all the relevant information about a given Test.
Clients should always use selectors to extract the desired element from a TestResultObject.
For example, if tr is a TestResultObject, write Test[tr] instead of tr[[1]] to get the original Test expr out.
The internal structure of TestResultObject is not guaranteed to stay the same from one version of MUnit to the next."

TestResultQ::usage =
"TestResultQ[tr] returns True if tr is a valid TestResultObject, and False otherwise."

FailureMode::usage =
"FailureMode is a selector for TestResultObject."

TestInput::usage =
"TestInput is a selector for TestResultObject."

ExpectedOutput::usage =
"ExpectedOutput is a selector for TestResultObject."

ActualOutput::usage =
"ActualOutput is a selector for TestResultObject."

ExpectedMessages::usage =
"ExpectedMessages is a selector for TestResultObject."

ActualMessages::usage =
"ActualMessages is a selector for TestResultObject."

ErrorMessage::usage =
"ErrorMessage is a selector for TestResultObject."

TestIndex::usage =
"TestIndex is a selector for TestResultObject. It is used in test runs."

AllTestIndex::usage =
"AllTestIndex is a selector for TestResultObject. It is used in test runs."

TestID::usage =
"TestID is an option to Test and TestMatch that specifies a string used as an identifier for the test.
TestID is also a selector for TestResult."

TestTags::usage =
"TestTags is an option to Test that specified meta information about that test. If a test comes from a notebook, its
cell tags are kept as TestTags."

ActualOutputSetFunction::usage =
"ActualOutputSetFunction is an option to Test that specifies the function that will be used to evaluate the input \
and set it to a variable.\n
The default is Set. \n
ActualOutputSetFunction is also a selector for TestResult"

ExpectedOutputSetFunction::usage =
"ExpectedOutputSetFunction is an option to Test that specifies the function that will be used to evaluate the expected output \
and set it to a variable.\n
The default is Set. \n
ExpectedOutputSetFunction is also a selector for TestResult"

EquivalenceFunction::usage =
"EquivalenceFunction is an option to Test and TestMatch that specifies the function that will be \
used to compare the actual and expected results.\n
The test input and expected output will have evaluated once before the equivalence function is evaluated.
The default for Test is SameQ, and for TestMatch it is MatchQ.
EquivalenceFunction is also a selector for TestResult."

MessagesEquivalenceFunction::usage =
"MessagesEquivalenceFunction is an option to Test and TestMatch that specifies the function that will be used to compare the actual and expected messages\n.
Custom MessagesEquivalenceFunction settings should also take into account that actual messages are passed in wrapped in HoldForm, \
and that a custom ExpectedMessagesWrapper will probably also be necessary to prevent any unwanted evaluations or messages."

$DefaultMessagesEquivalenceFunction::usage =
"$DefaultMessagesEquivalenceFunction is the default messages equivalence function. \
It is similar to MatchQ and understands Message and MessageName expressions."

ExpectedOutputWrapper::usage =
""

ActualOutputWrapper::usage =
""

ExpectedMessagesWrapper::usage =
"ExpectedMessagesWrapper is an option to Test that specifies the function that will be used to wrap expected messages in a TestResultObject expression. \
The default value is $DefaultExpectedMessagesWrapper, but it should be changed to an appropriate value for preserving custom expressions passed as expected messages."

TestFailureMessage::usage =
"TestFailureMessage is an option to Test that specifies a message that will be logged if the test fails.\n
TestFailureMessage is also a selector for TestResult."

TestFailureAction::usage =
"TestFailureAction is an option to Test that specifies a string naming an action to take if the test fails.\n
The allowable values are Continue (the default) and Abort.\n
TestFailureAction is also a selector for TestResult."

TestErrorAction::usage =
"TestErrorAction is an option to Test that specifies a string naming an action to take if the test has an error.\n
The allowable values are Continue (the default) and Abort.\n
TestErrorAction is also a selector for TestResult."

TestComment::usage =
"TestComment is deprecated."

TestComments::usage =
"TestComments is deprecated."


MemoryConstraint::usage =
"MemoryConstraint is an option for Test that specifes how much memory (in bytes) the test is allowed to use."

$MemoryConstrained::usage =
"$MemoryConstrained is returned as actual output when a test met a memory constraint."

$TimeConstrained::usage =
"$TimeConstrained is returned as actual output when a test met a time constraint."

TestTimeUsed::usage =
"TestTimeUsed is a selector for test results."

TestMemoryUsed::usage =
"TestMemoryUsed is a selector for test results."

$Error::usage =
"$Error is filled in for the arguments of a test result when an error occurs."

TestSource::usage = 
"TestSource is an option of TestResultObject that specifies the source of the test: a file, notebook, or stream."

(* ::Section:: *)
(*Package*)

Begin["`Package`"]

AddTestFunction

testError

End[]

(* ::Section:: *)
(*Private*)

Begin["`Test`Private`"]

(*
a side-effect is that MUnit leaks out the bitbucket stream
*)
$bitbucket = Module[{f = Switch[$OperatingSystem, "Windows", "NUL", "MacOSX" | "Unix", "/dev/null"], strs},
	            strs = Streams[f];
	            If[strs == {},
	            	(* bitbucket stream is not open yet, so open it *)
	            	OpenWrite[f]
	            	,
	            	strs[[1]]
	            ]
             ]

If[ListQ["MessageTextFilter" /. Internal`Handlers[]],
	Internal`AddHandler["MessageTextFilter", MUnitMessageHandler]
]

Attributes[Test] = {HoldAllComplete}

Options[Test] = {
		ActualOutputSetFunction -> Set,
		ExpectedOutputSetFunction -> Set,
		EquivalenceFunction -> SameQ,
		MemoryConstraint -> None,
		ExpectedOutputWrapper -> $DefaultExpectedOutputWrapper,
		ActualOutputWrapper -> $DefaultActualOutputWrapper,
		MessagesEquivalenceFunction -> $DefaultMessagesEquivalenceFunction,
		ExpectedMessagesWrapper -> $DefaultExpectedMessagesWrapper,
		TestFailureMessage -> "",
		TestFailureAction -> "Continue",
		TestErrorAction -> "Abort",
		TestID -> 0,
		TestTags -> {},
		TimeConstraint -> None,
		TestComment -> "",
		TestComments -> ""
	}

(*
Precondition: $Messages is set to the system default of something like {stdout}
*)

Test[input_, expected_:True, Shortest[expectedMsgs_:{}], opts:OptionsPattern[]] :=
	Module[{actualOutputSetFunction, expectedOutputSetFunction, testEquivFunc, msgsEquivFunc,
		expectedOutputWrapper, actualOutputWrapper, expectedMsgsWrapper, testID, actual,
		expectedEvaled, testEquivFuncEvaled, msgsEquivFuncEvaled, actualMsgs, expectedOutputMsgs, testEquivFuncMsgStrings, msgsEquivFuncMsgStrings,
		failureMode, testFailureMessage, testFailureAction, testErrorAction,
		timeConstraint, memoryConstraint, timeUsed, memoryUsed, testTags, actualRes, expectedRes,
		testEquivFuncRes, msgsEquivFuncRes},
		Catch[
			Block[{
				(*
				$Messages needs to be something not likely to be set by a function blocking messages, like no-value $Messages or {},
				in order to be able to tell when it IS set to $Messages or {}, in order to correctly not report those messages
				But it also needs to be something not containing stdout, to ACTUALLY suppress all messages
				It also needs to be something that actually works as a OutputStream, like a temp file, otherwise there is a slew of messages
				complaining.
				But there's no reason to open a whole temp file just for one test. Just use the platform-dependent bitbucket
				Now, we'll know if $Messages was changed because it will no longer be {$bitbucket}, a safe bet.
				Who would be writing to the bitbucket in a normal circumstance?
				If there were a way to have a MUnit-specific bitbucket, that would be great. Or to have virtual stream things in Mathematica.
				*)
				$Messages = {$bitbucket},
				(* Block these symbols to minimize side-effects *)			
				$DisplayFunction = Identity,
				$SoundDisplayFunction = Identity,
				(* these are to setup a nice environment for tests within tests! *)
				$actualTestIndex = 0,
				$allTestIndex = 0,
				$MessageWithArgsList = {},
				(* even though we don't use it, continue blocking $MessageList. This is to reset the General::stop counter
				and allow
				t:=Test[1/0,ComplexInfinity,{_}];
				{t,t,t}
				to work*)
				$MessageList = {},
				MUnitMessageHandler = $MUnitMessageHandler}
				,
	
				(* treat any unrecognized options as an error *)
				Check[
					{actualOutputSetFunction, expectedOutputSetFunction, testEquivFunc, msgsEquivFunc,
						expectedOutputWrapper, actualOutputWrapper, expectedMsgsWrapper, testID, testFailureMessage,
						testFailureAction, testErrorAction, timeConstraint, memoryConstraint, testTags} =
						OptionValue[{ActualOutputSetFunction, ExpectedOutputSetFunction, EquivalenceFunction,
							MessagesEquivalenceFunction, ExpectedOutputWrapper, ActualOutputWrapper,
							ExpectedMessagesWrapper, TestID, TestFailureMessage, TestFailureAction, TestErrorAction,
							TimeConstraint, MemoryConstraint, TestTags}]
					,
					Throw["Invalid options: " <> ToString[FilterRules[Flatten[{opts},1], Except[Options[Test]]]], "internalMUnitTestTag"]
					,
					OptionValue::nodef
				];
				
				memoryUsed = MemoryInUse[];
				timeUsed = TimeUsed[];
				
				actualRes = MUnitCheckAll[
					actualOutputSetFunction[
						actual
						,
						input
					]
					,
					opts
				];
				If[MatchQ[actualRes, $TimeConstrained | $MemoryConstrained],
					actual = actualRes 
				];
				
				timeUsed = TimeUsed[] - timeUsed;
				memoryUsed = MemoryInUse[] - memoryUsed;
				
				actualMsgs = $MessageWithArgsList;
				$MessageWithArgsList = {};
				$MessageList = {};
				
				expectedRes = MUnitCheckAll[
					expectedOutputSetFunction[
						expectedEvaled
						,
						expected
					]
					,
					opts
				];
				If[MatchQ[expectedRes, $TimeConstrained | $MemoryConstrained],
					expectedEvaled = expectedRes 
				];
				
				expectedOutputMsgs = $MessageWithArgsList;
				$MessageWithArgsList = {};
				$MessageList = {};
				
				
				Block[{MUnitMessageHandler = messageStringHandler},
					$MessageStringList = {};
					
					testEquivFuncRes = MUnitCheckAll[
						Set[
							testEquivFuncEvaled
							,
							testEquivFunc[actual, expectedEvaled]
						]
						,
						opts
					];
					
					testEquivFuncMsgStrings = $MessageStringList;
					$MessageStringList = {};
					$MessageList = {};
					
					(*
					TODO:
					if using $DefaultExpectedMessageHandler and given MessageNames as expected messages,
					then make sure to return MessagesNames as the actual messages
					*)
					msgsEquivFuncRes = MUnitCheckAll[
						Set[
							msgsEquivFuncEvaled
							,
							msgsEquivFunc[actualMsgs, expectedMsgs]
						]
						,
						opts
					];
					
					msgsEquivFuncMsgStrings = $MessageStringList;
					$MessageStringList = {};
					$MessageList = {};
				];
				
				failureMode = determineFailureMode[
								{testEquivFuncEvaled, msgsEquivFuncEvaled},
								{testEquivFuncRes, msgsEquivFuncRes},
								{testEquivFuncMsgStrings, msgsEquivFuncMsgStrings}]
				
			]; (* end Block *)
			(*
			Wrap expected and actual in HoldForm to preserve it. This is neccessary if a TestResultObject is saved to a file,
			then read in a different session, where actual would evaluate to something new.
			*)
			Module[{tr},
				tr = Sow[
					newTestResultObject[
						HoldForm[Test[input, expected, expectedMsgs, opts]],
						failureMode,
						HoldForm[input],
						(* this makes sure that if expectedEvaled or actual are delayed, then they are maintained *)
						expectedOutputWrapper[expectedEvaled],
						actualOutputWrapper[actual],
						expectedMsgsWrapper[expectedMsgs],
						(* each msg is already wrapped in HoldForm *)
						actualMsgs,
						"",
						TestIndex -> ++$actualTestIndex,
						AllTestIndex -> $allTestIndex,
						TestID -> testID,
						ActualOutputSetFunction -> actualOutputSetFunction,
						ExpectedOutputSetFunction -> expectedOutputSetFunction,
						EquivalenceFunction -> testEquivFunc,
						MessagesEquivalenceFunction -> msgsEquivFunc,
						ExpectedMessagesWrapper -> expectedMsgsWrapper,
						TestFailureMessage -> testFailureMessage,
						TestFailureAction -> testFailureAction,
						TestErrorAction -> testErrorAction,
						TestTimeUsed -> timeUsed,
						TestMemoryUsed -> memoryUsed,
						TestTags -> testTags,
						TestSource -> $CurrentTestSource
					],
					{"MUnitTest"}
				];
				logTestResult[tr];
				tr
			]
			,
			"internalMUnitTestTag"
			,
			Function[{value, tag},
				(*
				calling testError must wait until outside of the Block, so that $actualTestIndex can increment correctly
				*)
				testError[value, opts]
			]
		]
	]

Test[args___] :=
	testError["Incorrect arguments: " <> ToString[Unevaluated[{args}]], args]


testError[errMsg_String, ___, opts:OptionsPattern[]] :=
	Module[{testID, testErrorAction, tr},
		Quiet[{testID, testErrorAction} = OptionValue[Test, {opts}, {TestID, TestErrorAction}], {OptionValue::nodef}];
		tr = Sow[
			newTestResultObject[
				$Error,
				"Error",
				$Error,
				$Error,
				$Error,
				$Error,
				$Error,
				errMsg,
				TestIndex -> ++$actualTestIndex,
				AllTestIndex -> $allTestIndex,
				TestID -> testID,
				TestSource -> $CurrentTestSource
			]
			,
			{"MUnitTest"}
		];
		logTestResult[tr];
		tr
	]

(*
note: MUnitCheckAll returns Null
*)
SetAttributes[MUnitCheckAll, HoldAllComplete]
MUnitCheckAll[expr_, opts:OptionsPattern[]] :=
	Module[{timeConstraint, memoryConstraint},
		{timeConstraint, memoryConstraint} = OptionValue[Test, {opts}, {TimeConstraint, MemoryConstraint}];
		CheckAll[
			(* use Reap here to not double-count any Tests within Tests in a test run *)
			Reap[
				(* the With here is to allow MemoryConstrained and TimeConstrained look better when they return unevaluated *)
				With[{timeConstraint = timeConstraint, memoryConstraint = memoryConstraint},
					Which[
						timeConstraint === memoryConstraint === None,
						expr
						,
						timeConstraint === None,
						MemoryConstrained[expr, memoryConstraint, $MemoryConstrained]
						,
						memoryConstraint === None,
						TimeConstrained[expr, timeConstraint, $TimeConstrained]
						,
						True,
						TimeConstrained[MemoryConstrained[expr, memoryConstraint, $MemoryConstrained], timeConstraint, $TimeConstrained]
					]
				]
				,
				"MUnitTest"
			][[1]]
			,
			Switch[#2,
				Hold[],
				(* everything is fine *)
				#1
				,
				Hold[_Abort],
				(* an Abort was encountered, in test or user action *)
				TestTerminate["Aborted during evaluation"];
				Throw["Aborted during evaluation", "internalMUnitTestTag"]
				,
				_,
				(* some other thing like Throw or Break *)
				Throw["Unhandled error " <> ToString[HoldForm[#2]], "internalMUnitTestTag"]
			]&
		]
	]

(*
there is a bit of machinery here to determine exactly what
kind of error message to throw and to throw it nicely

The Block[{$Context = "MUnit`Test`Private`"}, ...]'s are to make the string nice, without giving
symbols the full MUnit`Test`Private` name
*)
determineFailureMode[{testEquivFuncEvaled_, msgsEquivFuncEvaled_}, {testEquivFuncRes_, msgsEquivFuncRes_}, {testEquivFuncMsgStrings_, msgsEquivFuncMsgStrings_}] :=
	Module[{testEquivFuncResString, msgsEquivFuncResString},
		Switch[{testEquivFuncEvaled, msgsEquivFuncEvaled},
			{True, True},
			"Success"
			,
			{False, True | False},
			"Failure"
			,
			{True, False},
			"MessagesFailure"
			,
			{Except[True | False], _},
			Block[{$Context = "MUnit`Test`Private`"},
				testEquivFuncResString = ToString[testEquivFuncRes]
			];
			Throw["EquivalenceFunction did not return True or False: \n\n" <> StringJoin[Riffle[testEquivFuncMsgStrings, "\n\n"]] <> "\n\n" <> testEquivFuncResString, "internalMUnitTestTag"]
			,
			{_, Except[True | False]},
			Block[{$Context = "MUnit`Test`Private`"},
				msgsEquivFuncResString = ToString[msgsEquivFuncRes]
			];
			Throw["MessagesEquivalenceFunction did not return True or False: \n\n" <> StringJoin[Riffle[msgsEquivFuncMsgStrings, "\n\n"]] <> "\n\n" <> msgsEquivFuncResString, "internalMUnitTestTag"]
		]
	]

(*
needs to be HoldFirst because we do things with the arguments, where they aren't held by Message any more
*)
SetAttributes[patternizeExpectedMessage, HoldFirst]
patternizeExpectedMessage[name_MessageName] := HoldForm[Message[name, ___]]
(*HoldPattern is necessary for the definition, when Message and MessageName are not blocked yet, they are only blocked at runtime*)
patternizeExpectedMessage[HoldPattern[Message][name_, args___]] :=
	With[{args2 = Sequence @@ HoldForm /@ Unevaluated[{args}]},
		With[{m = Message[name, args2]},
			HoldForm[m]
		]
	]
(* let patterns fall-through *)
patternizeExpectedMessage[arg_] := arg

SetAttributes[$DefaultMessagesEquivalenceFunction, HoldAllComplete]

$DefaultMessagesEquivalenceFunction[actualMsgs_, expectedMsgs_] :=
	Module[{expectedMsgs2},
		Block[{Message, MessageName},
			SetAttributes[Message, HoldRest];
			SetAttributes[MessageName, HoldFirst];
			expectedMsgs2 =
				Switch[expectedMsgs,
					_MessageName | _Message,
					{patternizeExpectedMessage[expectedMsgs]}
					,
					_List,
					Map[patternizeExpectedMessage, expectedMsgs]
					,
					_Alternatives,
					(* from old Testing`-style tests *)
					Map[patternizeExpectedMessage, expectedMsgs, {2}]
					,
					_,
					expectedMsgs
				];
			MatchQ[actualMsgs, expectedMsgs2]
		]
	]

SetAttributes[$DefaultExpectedOutputWrapper, HoldFirst]
$DefaultExpectedOutputWrapper[expectedEvaled_] := HoldForm[expectedEvaled] /. OwnValues[expectedEvaled]

SetAttributes[$DefaultActualOutputWrapper, HoldFirst]
$DefaultActualOutputWrapper[actual_] := HoldForm[actual] /. OwnValues[actual]


SetAttributes[$DefaultExpectedMessagesWrapper, HoldFirst]
(*
nothing depends on ExpectedMessages, so only wrap entire in HoldForm. The individual args do not
need to be wrapped in HoldForm, like with ActualMessages.
*)
$DefaultExpectedMessagesWrapper[expectedMsg_MessageName] := {HoldForm[expectedMsg]}
$DefaultExpectedMessagesWrapper[expectedMsg_Message] := {HoldForm[expectedMsg]}
$DefaultExpectedMessagesWrapper[expectedMsgs_List] := HoldForm /@ Unevaluated[expectedMsgs]
$DefaultExpectedMessagesWrapper[expectedMsgs_Alternatives] := Map[HoldForm, Unevaluated[expectedMsgs], {2}]
$DefaultExpectedMessagesWrapper[expectedMsgs_] := expectedMsgs

(*
The args pattern used to be args___HoldForm, but then we realized that
message arguments are not always wrapped in HoldForm.
Some times they are, some times they are not.
Note that this is different than expected messages, which are never expected to have HoldForm around the args
*)
$MUnitMessageHandler[ctlString_String, Hold[msg_MessageName], Hold[Message[msg_MessageName, args___]]] :=
	(
	If[(* stdout would have been in $Messages *)
		$Messages === {$bitbucket} ||
			(* $Messages has been changed, most likely to $Messages or {} *)
			(ListQ[$Messages] && MemberQ[$Messages, OutputStream["stdout", _]])
		,
		AppendTo[$MessageWithArgsList, HoldForm[Message[msg, args]]];
	])

messageStringHandler[ctlString_String, Hold[msg_MessageName], Hold[Message[msg_MessageName, args___]]] :=
	(
	If[(* stdout would have been in $Messages *)
		$Messages === {$bitbucket} ||
			(* $Messages has been changed, most likely to $Messages or {} *)
			(ListQ[$Messages] && MemberQ[$Messages, OutputStream["stdout", _]])
		,
		Block[{$Context = "MUnit`Test`Private`"},
			AppendTo[$MessageStringList, ToString[StringForm[ctlString, args]]];
		]
	])
	

AddTestFunction[testFunc_] :=
	Module[{},
		Attributes[testFunc] = Attributes[Test];
		Options[testFunc] = Options[Test];
		testFunc[input_, expected_:True, Shortest[messages_:{}], opts:OptionsPattern[]] :=
			With[{options = Join[Flatten[{opts}], Options[testFunc]]},
				Test[input, expected, messages, options]
			]
	]

AddTestFunction[TestMatch]

SetOptions[TestMatch, EquivalenceFunction -> MatchQ]

AddTestFunction[TestStringMatch]

SetOptions[TestStringMatch, EquivalenceFunction -> StringMatchQ]

AddTestFunction[TestFree]

SetOptions[TestFree, EquivalenceFunction -> FreeQ]

AddTestFunction[TestStringFree]

SetOptions[TestStringFree, EquivalenceFunction -> StringFreeQ]



Options[TestResultObject] =
		{TestIndex -> 0,
		AllTestIndex -> 0,
		TestMemoryUsed -> 0,
		TestTimeUsed -> 0,
		TestSource -> Null} ~Join~ Options[Test]

TestResultQ[_] =
	False

newTestResultObject[
	test_,
	failureMode_,
	input_,
	expectedOutput_,
	actualOutput_,
	expectedMsgs_,
	actualMsgs_,
	errorMsg_,
	opts:OptionsPattern[]] :=
	With[{obj = Unique["MUnit`TestResultObjects`TestResultObject"]},
		Module[{},
			obj /: TestResultQ[obj] = True;
			obj /: Format[obj, OutputForm] :=
				Switch[FailureMode[obj],
					"Success",
					StringJoin["-", "Success", "-"]
					,					
					"MessagesFailure",
					StringJoin["-", "MessagesFailure: ", ToString[ActualMessages[obj], OutputForm], "-"]
					,
					"Failure",
					StringJoin["-", "Failure: ", ToString[ActualOutput[obj], OutputForm], "-"]
					,
					"Error",
					StringJoin["-", "Error: ", ErrorMessage[obj], "-"]
				];
			obj /: Format[obj, StandardForm] :=
				Switch[FailureMode[obj],
					"Success",
					Interpretation[Row[{RawBoxes["\[SkeletonIndicator]"], Row[{RawBoxes["TestResultObject"], RawBoxes["["], "Success", RawBoxes["]"]}], RawBoxes["\[SkeletonIndicator]"]}], obj]
					,
					"MessagesFailure",
					Interpretation[Row[{RawBoxes["\[SkeletonIndicator]"], Row[{RawBoxes["TestResultObject"], RawBoxes["["], Row[{"MessagesFailure:", " ", ActualMessages[obj]}], RawBoxes["]"]}], RawBoxes["\[SkeletonIndicator]"]}], obj]
					,
					"Failure",
					Interpretation[Row[{RawBoxes["\[SkeletonIndicator]"], Row[{RawBoxes["TestResultObject"], RawBoxes["["], Row[{"Failure:", " ", ActualOutput[obj]}], RawBoxes["]"]}], RawBoxes["\[SkeletonIndicator]"]}], obj]
					,
					"Error",
					Interpretation[Row[{RawBoxes["\[SkeletonIndicator]"], Row[{RawBoxes["TestResultObject"], RawBoxes["["], Row[{"Error:", " ", ErrorMessage[obj]}], RawBoxes["]"]}], RawBoxes["\[SkeletonIndicator]"]}], obj]
				];
			(* This prevents the Very Large Output panel from appearing in the
			front end if the parts of the TestResult are very big (such as Graphics) *)
			BoxForm`SetSizeCountRule[obj -> None];
			obj /: FailureMode[obj] := failureMode;
			obj /: TestInput[obj] := input;
			obj /: ExpectedOutput[obj] := expectedOutput;
			obj /: ActualOutput[obj] := actualOutput;
			obj /: ExpectedMessages[obj] := expectedMsgs;
			obj /: ActualMessages[obj] := actualMsgs;
			obj /: ErrorMessage[obj] := errorMsg;
			obj /: TestID[obj] := OptionValue[TestResultObject, {opts}, TestID];
			obj /: TestIndex[obj] := OptionValue[TestResultObject, {opts}, TestIndex];
			obj /: AllTestIndex[obj] := OptionValue[TestResultObject, {opts}, AllTestIndex];
			obj /: TestInputSetFunction[obj] := OptionValue[TestResultObject, {opts}, TestInputSetFunction];
			obj /: ExpectedOutputSetFunction[obj] := OptionValue[TestResultObject, {opts}, ExpectedOutputSetFunction];
			obj /: EquivalenceFunction[obj] := OptionValue[TestResultObject, {opts}, EquivalenceFunction];
			obj /: MessagesEquivalenceFunction[obj] := OptionValue[TestResultObject, {opts}, MessagesEquivalenceFunction];
			obj /: ExpectedMessagesWrapper[obj] := OptionValue[TestResultObject, {opts}, ExpectedMessagesWrapper];
			obj /: TestFailureMessage[obj] := OptionValue[TestResultObject, {opts}, TestFailureMessage];
			obj /: TestFailureAction[obj] := OptionValue[TestResultObject, {opts}, TestFailureAction];
			obj /: TestErrorAction[obj] := OptionValue[TestResultObject, {opts}, TestErrorAction];
			obj /: TestTimeUsed[obj] := OptionValue[TestResultObject, {opts}, TestTimeUsed];
			obj /: TestMemoryUsed[obj] := OptionValue[TestResultObject, {opts}, TestMemoryUsed];
			obj /: TestTags[obj] := OptionValue[TestResultObject, {opts}, TestTags];
			obj /: TestSource[obj] := OptionValue[TestResultObject, {opts}, TestSource];
			obj
		]
	]


End[]
