BeginPackage[ "XXFUNCTIONXX`"]

XXFUNCTIONXX::usage = "XXFUNCTIONXX is a plug-in method for NDSolve ... " 

Begin["`Private`"]

(*
	Intialize the method object for integrating the ODE system
	y'[t] == rhs[t, y[t]] within the NDSolve`StateData object state
	with the method options opts.  
	
	stepmode will be either Automatic or "Fixed". When the method is being called as a 
	submethod of a step controller, the stepmode will typically be "Fixed".
	When it is Automatic, this is effectively a request for the method to set up (if possible)
	its own automatic step control.  The NDSolve step controller will determine what to
	do in this case based on the value of the methods "StepControl" property that you can define 
	below.
	
	methoddata can be any data that your method might need to use to integrate the system
	of ODEs.  The data can be in any structure you like since it is private to your method
	and is not used by NDSolve except through your method.
*)
XXFUNCTIONXX /: NDSolve`InitializeMethod[
			XXFUNCTIONXX, 
			stepmode_String, 
			rhs_Experimental`NumericalFunction, 
			state_NDSolve`StateData, 
			opts___] := 
	Module[{methoddata}, 
	    XXFUNCTIONXX[methoddata]
   	]

(*
	Given the ODE system y'[t] == rhs[t, y[t]] along with 
	starting values y[t0] and y'[t0] compute the solution 
	increment dy so that y[t0 + h] == y[t0] + dy
	
	Return should be
	{newh, dy, yph, newobj} where
		newh is the timestep to take for the next step
		dy is the increment.  $Failed can be given to indicate a rejected step
		yph is y'[t0 + h].  If your method does not evaluate this, you can 
			just give it as None.
		newobj is a new method object if any of the data have changed.
		
	If the output is a list with 1 <= length < 4, default values
	{newh, $Failed, None, method} will be used as needed.
*)
XXFUNCTIONXX[methoddata___]@"Step"[rhs_, t0_, h_, y_, yp_] := 
	Module[{ },
    	
    ]
   
(*
	The difference order should be set to a positive integer equal to
	the local truncation order, p, of the method.  It is important to set
	this property if you are planning to use your method with a step
	controller such as DoubleStep.
*)
XXFUNCTIONXX[methoddata___]@"DifferenceOrder" := p

(*
	Set this to the leftmost boundary of the region of absolute stability of the 
	method along the real axis.  It should be a negative real number.
	This is required for step controllers such as DoubleStep that do stiffness testing.
*)
XXFUNCTIONXX[methoddata___]@"LinearStabilityBoundary" := $Failed

(*
	Set this to True for an A-stable method (the region of absolute
	stability includes the entire left-half plane.)
*)
XXFUNCTIONXX[methoddata___]@"StiffMethodQ" := False

(*
	Set this to True for a method that can handle linearly implicit systems
	of the form M.y'[t] == f[t, y[t]]
	where M is a (possibly singular) mass matrix.
*)
XXFUNCTIONXX[methoddata___]@"MassMatrixQ" := False

(*
	Set this True for a symmetric method for which the error is only in terms
	of even powers of the step size h.
*)
XXFUNCTIONXX[methoddata___]@"SymmetricMethodQ" := False

(*
	Change this from the default of "Fixed" to Automatic if your method is set up with
	the capability of choosing the next step size appropriately according to
	the local tolerances derived from PrecisionGoal and AccuracyGoal.
*)
XXFUNCTIONXX[___]@"StepMode" := "Fixed"

(*
	Set this to True if you can provide dense output for your method.
	If you set this to True, you also need to fill in the definition of
	the "GetDenseOutput" method below.
*)
XXFUNCTIONXX[methoddata___]@"DenseOutputQ" := False

(*
	The get dense output method should return data required to
	produce an interpolant of order of at least the method order
	over the interval [te - h, te].  It should return
	{
		{t0, t1, ..., tk},
		{{y00, y01, y02, ...}, 
		 {y10, y11, y12, ...},
		 ...
		 {yk0, yk1, yk2, ...}},
		f
	}
	where
		t0, t1, ..., tk are the time values in the interval [te - h, te]
		yji is a function (or derivative value) at tj 
		f is a function that evaluates the dth derivative of the interpolant at a value t given
			f[d, t - {t0, t1, ..., tk}, {{y00, y01, y02, ...}, ..., {yk0, yk1, yk2, ...}}]
			
	Note that values supplied as arguments y, yp are y[te], y'[te] are the endpoint of the 
	step.  If your dense output function f needs the values at the beginning of the step to 
	do the interpolation, you can use the keyword "Previous" in place of {y00, y01, ...} where 
	t0 = te - h. 
*)
XXFUNCTIONXX[methoddata___]@"GetDenseOutput"[rhs_, t1_, h_, y_, yp_] := $Failed


End[]

EndPackage[]