FormatSpec[
    Name -> "PacletInfo specification",
    RestrictTo[
      FileName -> "PacletInfo.m"
    ],
        RuleErrorMessages[
		RootOrEmpty -> "Illegal setting for Root",
		PacletName -> "Illegal setting for Paclet Name",
		PacletCreator -> "Illegal setting for Paclet Creator",
		PacletVersion -> "Illegal setting for Paclet Version",
		MathematicaVersion -> "Illegal setting for Wolfram Language Version"
    ],

    (* defines which rule to use as start rule  *)
    RootRule -> PacletFormat,
ExpressionSpec[
 (*
   Basically: think of this as a kind of BNF for Paclets: ie. NonTerminal ::= Definition
   ExprLiteral defines the expression expected for this NonTerminal, ie. ExprLiteral["FOO"] means FOO[];
   ExprLiteral["FOO", SomeOtherRule[]] means Foo[ ... ] - where ... is whatever is defined in SomeOtherRule[];
   Regexp[""] is a rule that will check that the element is a String and that the contents must match the given Regex,
      eg. PacletName -> ExprLiteral["Rule", "Name", Regexp["[a-zA-Z]*"]]  will  match: Rule[Name, "Hello"], but not Rule[Name, "Hello42"]
 *)
 
 (*
 TODO: support naming elements to allow to pull out individual values
 *)
 
  PacletFormat -> ExprLiteral["Paclet", 
      CreatorRule[]|Empty[],
      DescriptionRule[]|Empty[],
      Extensions[],
      LoadingRule[]|Empty[],
      PacletNameRule[] ,
      RootRule[]|Empty[], 
      ThumbnailRule[]|Empty[], 
	  UpdatingRule[]|Empty[],
	  VersionRule[] ,
	  WolframVersionRule[]|MathematicaVersionRule[], 
      Orderless -> True 
   ],
 
 PacletNameRule -> ExprLiteral["Rule", "Name", PacletName[] ],
 PacletName -> Regexp["[a-zA-Z][a-zA-Z0-9]*"],
 
 VersionRule -> ExprLiteral["Rule", "Version", PacletVersion[]],
 PacletVersion -> Regexp["\\d+(\.\\d+(\.\\d+(\.\\d+(\.\\d+)?)?)?)?"],
 
 WolframVersionRule -> ExprLiteral["Rule", "WolframVersion", WolframVersion[] ] ,
 MathematicaVersionRule -> ExprLiteral["Rule", "MathematicaVersion", WolframVersion[] ] ,
 WolframVersion -> Regexp["\\d+(\.\\d+)?(\\+)?"],  (*  Fix this Regex *)
 
 
 RootRule -> ExprLiteral["Rule", "Root", Root[] ],
 RootOrEmpty -> Root[] | Regexp[""],
 Root -> Regexp["[a-zA-Z][a-zA-Z0-9]*"],  (* TODO: this is likely to be wrong! Probably needs slash, period an other possible characters *) 
 
 
 CreatorRule -> ExprLiteral["Rule", "Creator", Creator[] ],
 Creator -> Regexp[".*"],
 
 LoadingRule -> ExprLiteral["Rule", "Loading", StringLiteral["Automatic"] | StringLiteral["Manual"] | StringLiteral["Startup"] ],
 
 UpdatingRule -> ExprLiteral["Rule", "Updating", "Automatic"],
 
 DescriptionRule -> ExprLiteral["Rule", "Description", Description[] ],
 Description -> Regexp["(.|\\r|\\n)*"],
 
 ThumbnailRule -> ExprLiteral["Rule", "Thumbnail", Regexp[".*"] ],

  Extensions -> ExprLiteral["Rule", "Extensions", 
  		ExprLiteral["List", 
  			ApplicationExtension[]..., 
  			DocumentationExtension[]..., 
  			FrontEndExtension[]..., 
  			KernelExtension[]...,
  			LibraryLinkExtension[]...
			] ],
  
  DocumentationExtension -> ExprLiteral["List", 
               StringLiteral["Documentation"], 
               ExprLiteral[ "Rule", "Language", Regexp[".*"]] | Empty[],
               ExprLiteral[ "Rule", "LinkBase", Regexp[".*"]] | Empty[],
               ExprLiteral[ "Rule", "MainPage", Anything[]] | Empty[],
               ExprLiteral[ "Rule", "Resources", ExprLiteral["List", Anything[]..]] | Empty[],
               RootRule[] | Empty[],
               Orderless -> True
              ],
  
  FrontEndExtension -> ExprLiteral["List", 
              StringLiteral["FrontEnd"], 
               RootRule[] | Empty[],
               Orderless -> True
              ],
  
  ApplicationExtension -> ExprLiteral["List", 
              StringLiteral["Application"], 
              ExprLiteral[ "Rule", "Context", MathematicaPackageName[]] | Empty[] ,
              RootRule[] | Empty[],
              Orderless -> True
              ],
  
  KernelExtension -> ExprLiteral["List", 
              StringLiteral["Kernel"], 
              ExprLiteral[ "Rule", "Context", MathematicaPackageName[]|ExprLiteral["List", MathematicaPackageName[]..]] | Empty[] ,
              RootRule[] | Empty[],
              ExprLiteral[ "Rule", "Symbols", ExprLiteral["List", Anything[]..]] | Empty[],
              Orderless -> True
              ],
  
  LibraryLinkExtension -> ExprLiteral["List", 
              StringLiteral["LibraryLink"]
              ],
  
  MathematicaPackageName -> Regexp["[a-zA-Z][a-zA-Z0-9]+`"] 
  
 ]
 
]