(* ::Package:: *)

(* :Title: Make.m *)

(* :Authors:
	Andrew Hunt, Shaun McCance, Todd Gayley
	andy@wolfram.com, shaunm@wolfram.com, tgayley@wolfram.com
*)

(* :Package Version: 0.50 *)

(* :Mathematica Version: 7.0 *)
                     
BeginPackage["DocumentationBuild`Make`", {
    "DocumentationBuild`Common`",
    "DocumentationBuild`Utils`",
    "DocumentationBuild`Info`",
    "Transmogrify`"
}]


(******************************************************************************)
(** Usage Statements **)

MakeNotebook::usage = "MakeNotebook[nb] create an in-product notebook\
 from an authoring notebook.";

MakeNotebook::error = "``"
MakeNotebook::warning = "``"
 

Options[MakeNotebook] = {
  "ReleaseVersion" -> "13.1",
  (* ThresholdVersion should be the previous version; for point releases, can be Automatic, but when incrementing to a new integer for ReleaseVersion, 
     ThresholdVersion should be explicitly set.  E.g., when incrementing to ReleaseVersion->12, ThresholdVersion needs to be set to 11.3. *)
  "ThresholdVersion" -> Automatic,
  "BuildMode"->"Normal",
  "Language" -> "English",
  (* WebPage *)
  "Web" -> False,
  "CoreInformation"->Automatic,
  (* IncludeHistoryCell *)
  IncludeHistory -> True,
  "FunctionPaclet" -> False,
  "Debug"->"False",
  "outfile" -> None,
  "outfile2" -> None,
  "RasterizeOutput"->True,
  "FilePath" -> Missing["OptionNotSet", "FilePath"],
  (* V9 EDIT BELOW *)
  "CapitalizeHeadingStyles" -> {
    (*"AllowableOptionsSection",*) "ApplicationNotesSection", 
    "AttributesSection", "CategorizationSection", "TestCasesSection",
    "DesignDiscussionSection", "DocumentStatusSection", 
    (*"ElementsSection",*) "ErrorMessagesSection", (*"ExampleSection",*) 
    "ExtendedExamplesSection", "ExtensionsSection", 
    (*"GuideFunctionsSection", "GuideMoreAboutSection", 
    "GuideTutorialsSection", "ImportExportSection",*) "KeywordsSection", 
    "LegacyMaterialSection", (*"MoreAboutSection",*) 
    "MoreToExploreSection", "MoreInformationSection",
    (*"NotebookInterfaceSection",*) (*"NotesSection",*) (*"OptionsSection", *)
    (*"PrimaryExamplesSection",*) (*"RelatedDemonstrationsSection", *)
    (*"RelatedLinksSection",*) (*"Section",*) (*"SeeAlsoSection",*) 
    "TemplatesSection", (*"TOCSection",*) (*"TutorialsSection",*) 
    (*"RelatedTutorialsSection",*) (*"TutorialMoreAboutSection",*) (*"TutorialCollectionSection",*)
    (*"TutorialRelatedLinksSection",*) (*"TutorialsTutorialCollectionSection",*)
    "UsageMessagesSection", "UsageSection"(*, "IndicatorExampleSection", "IndicatorDescriptionSection", "IndicatorCategorizationSection", "IndicatorAbbreviationSection", "IndicatorFormulaSection"*)
  }
};

$MarketingPageStyleSheet = Null;
$ComparisonPageStyleSheet = Null;
$CoreStyleSheet = Null;


MakeMarketingPageNotebook::usage = "MakeMarketingPageNotebook[nb] takes a production notebook and returns a massaged notebook ready for inclusion in product/on web.";
ExportMarketingWebPage::usage = "ExportMarketingWebPage[file.html, nb, language] takes a production notebook and exports to xml.";
ExportMarketingCategoryWebPages::usage = "";
ExportCapabilitiesWebPage::usage ="";
ExportComparisonWebPage::usage="";
MakeWhatsNewSidebar::usage="";
ExportWhatsNewSidebar::usage="";
MakeMarketingSubpage::usage = "";
MakeMarketingThumbnailTable::usage = "";
MakeSolutionsPageNotebook::usage = "MakeSolutionsPageNotebook[nb] takes a production notebook and returns a massaged notebook ready for Web export.";
ExportSolutionsWebPage::usage =  "ExportSolutionsWebPage[file.html, nb, language] exports Solutions page for Web from output of MakeSolutionsPageNotebook.";
MakeUpdatedSolutionsNotebook::usage = "MakeUpdatedSolutionsNotebook[nb] takes a production notebook and returns a massaged notebook ready for Web export.";
ExportUpdatedSolutionsPage::usage =  "ExportUpdatedSolutionsPage[file.html, nb, language] exports Solutions page for Web from output of MakeUpdatedSolutionsNotebook.";
MakeTMJNotebook::usage= "MakeTMJNotebook[nb] takes a production notebook and returns a massaged notebook ready for Web export.";
ExportTMJWebPage::usage =  "ExportTMJWebPage[file.html, nb, language] exports TMJ expression for Web from output of MakeTMJNotebook.";
MakeInsiderNotebook::usage = "MakeInsiderNotebook[nb] takes a production notebook and returns a massaged notebook ready for Web export.";
ExportInsiderWebPage::usage =  "ExportInsiderWebPage[file.html, nb, language] exports newsletter html or email from output of MakeInsiderNotebook.";
MakeAlphaAPINotebook::usage= "MakeAlphaAPINotebook[nb] takes a production notebook and returns a massaged notebook ready for Web export.";
ExportAlphaAPIWebPage::usage =  "ExportAlphaAPIWebPage[file.html, nb, language] creates html output from MakeAlphaAPINotebook.";


Begin["`Private`"];

(******************************************************************************)
(** MakeNotebook **)

MakeNotebook[nb_String, opts___?OptionQ] :=
  MakeNotebook[GetQuietly[nb], "FilePath" -> nb, opts];

MakeNotebook[nb_Notebook, opts___?((OptionQ[#] && !ListQ[#])&)] :=
  MakeNotebook[nb, GetNotebookInformation[nb, opts], opts];

MakeNotebook[
	nb_Notebook,
	info_List,
	opts:OptionsPattern[MakeNotebook]
] := Module[{
	pacletName, type, lang, uri, expr, poss, webpage, functionPaclet, rootCaptionList,
	stylesheet, styledefs,  indicatorstyles,
	versionYears, historyCell, citationInfo, citationCell,
	overviewLinks, classLabel,
	nbContext,
	outfilelocation, outfilelocation2, buildmode,
	moreAboutCells, moreAboutCellPos, newMoreAboutCells,
	specialKeywords,
	guidetutorialcollectioncells, tutorialcollectiontitleandlinks, linktrailexprs, colortype,
	debug,
	time1,time2,time2a,time3,time4,time5,time6,time7,time8,time9,time10,time11,time12,
	time13,time14,time15,time16,time17,
	makenboutput, autoitalics,
	hasoptions, warnicon, $Rebranded, summary, fnessay,
	GetTableTaggingRules, tabletaggingrules, rasterizeoutput,
	MakeKeyComboBox, closedGroups, CloseGroups, CustomGuideQ
},

    Off[CreateDirectory::filex];

    (* StrokeForm (and others) not loaded, so load it hack *)
    System`StrokeForm;
    System`Absolute;
    System`CellEvaluationLanguage;
    System`GraphicsHighlightColor;


    $Rebranded = True;
    rasterizeoutput = OptionValue["RasterizeOutput"];

    pacletName = "PacletName" /. info;
    pacletName = If[pacletName === "Mathematica", "", pacletName];
    type = "EntityType" /. info;
    lang = OptionValue["Language"];
    uri = "URI" /. info;
    webpage = OptionValue["Web"];
    buildmode = OptionValue["BuildMode"];

	If[!StringQ[uri],
		(* TODO: Make this a DocsError. *)
		DocsWarning[
			"Notebook info \"URI\" value expected to be a String in MakeNotebook: ``",
			InputForm[uri]
		];
	];

	{outfilelocation, outfilelocation2} = Map[
		Replace[OptionValue[MakeNotebook, FilterRules[{opts}, Options[MakeNotebook]], #], {
			valid : (_?StringQ | None) :> valid,
			(* Note:
				The default value for these options used to be False, so this case is for
				backwards compatibility with older code which may pass in False explicitly. *)
			False -> None,
			other_ :> DocsError[
				"Invalid value for MakeNotebook `` option: ``",
				InputForm[#], InputForm[other]
			]
		}] &,
		{"outfile", "outfile2"}
	];

    functionPaclet = OptionValue["FunctionPaclet"];
    debug = OptionValue["Debug"];

    debug = "True";

	Global`AntLog["Starting time1"];
	time1 = ToString @ AbsoluteTiming[
		Replace[OptionValue["CoreInformation"], {
			Automatic :> (
				(* Load only the System core information. *)
				(* Note:
					Only load the system core information if core information is empty.
					This is purely for backwards compatibility of MakeNotebook with
					use-cases that pre-date the DocumentationBuild(Notebooks|HTML|...)
					entry point functions. Eventually, this should be removed. *)
				If[DocumentationBuild`Info`$CoreInformation === <||>,
					SetCoreInformation[lang, Automatic, {"System"}];
				];
			),
			other_ :> (
				DocsError[
					"MakeNotebook: Unsupported \"CoreInformation\" option value."
					<> "MakeNotebook no longer supports a core information option. "
					<> "Call SetCoreInformation directly before calling MakeNotebook. Value: ``",
					other
				];
			)
		}]
	][[1]];
    If[debug==="True",Global`AntLog["Loading CoreInformation: "<>time1]];

	(* check for explicit CellContext setting - present in some/most M- Tutorials *)
	nbContext = (CellContext /. Rest[List @@ nb]);
	specialKeywords = Cases[nb, Cell[a_,"Keywords",Rule[CellLabel,b_]]:>a, Infinity];

    BuildLog["** Entering MakeNotebook **"];
    BuildLog["   Language: "<>lang ];

    expr = nb;

	GetRootCaptionList[nbexpression_]:= Module[{SectionCaptions},
		SectionCaptions = Cases[nbexpression, Cell[__, "GuideText", ___], Infinity];
		SectionCaptions = DeleteCases[SectionCaptions, Cell[__, "GuideText","TOCExcludedObject", ___], Infinity];
		SectionCaptions = Cases[SectionCaptions, Cell[a__, "GuideText", ___]:>a, Infinity];
		SectionCaptions
	];

	rootCaptionList = If[type === "Root Guide",
		GetRootCaptionList[expr],
		""
	];

	indicatorstyles = {
		Cell[
			StyleData["IndicatorExampleSection"],
			Editable -> False,
			CellFrame -> {{0, 0}, {0, 2}},
			ShowCellBracket -> Automatic,
			ShowGroupOpener -> True,
			CellMargins -> {{24, 24}, {6, 18}},
			CellElementSpacings -> {"ClosedGroupTopMargin"->28},
			CellGroupingRules -> {"SectionGrouping", 50},
			CellFrameMargins -> 4,
			CellFrameColor -> RGBColor[
			0.48235300000000003`, 0.5176470000000001, 0.8196080000000001],
			StyleMenuListing -> None,
			FontFamily -> "Verdana",
			FontSize -> 9,
			FontTracking -> "Wide",
			FontColor -> GrayLevel[0]
		],

		Cell[
			StyleData["IndicatorExample"],
			CellMargins -> {{52, 12}, {1, 1}},
			ReturnCreatesNewCell -> True,
			CellChangeTimes -> {3.475262460013097*^9},
			FontFamily -> "Verdana",
			FontSize -> 11
		],

		Cell[
			StyleData["IndicatorDescriptionSection"],
			Editable -> False,
			CellFrame -> {{0, 0}, {0, 2}},
			ShowCellBracket -> Automatic,
			ShowGroupOpener -> True,
			CellMargins -> {{24, 24}, {6, 18}},
			CellElementSpacings -> {"ClosedGroupTopMargin"->28},
			CellGroupingRules -> {"SectionGrouping", 50},
			CellFrameMargins -> 4,
			CellFrameColor -> RGBColor[
			0.48235300000000003`, 0.5176470000000001, 0.8196080000000001],
			StyleMenuListing -> None,
			FontFamily -> "Verdana",
			FontSize -> 9,
			FontTracking -> "Wide",
			FontColor -> GrayLevel[0]
		],

		Cell[
			StyleData["IndicatorDescription"],
			CellMargins -> {{52, 12}, {2, 2}},
			ReturnCreatesNewCell -> True,
			CellChangeTimes -> {3.475262460013297*^9},
			FontFamily -> "Verdana",
			FontSize -> 11
		],

		Cell[StyleData["IndicatorCategorizationSection"],
			Editable -> False,
			CellFrame -> {{0, 0}, {0, 2}},
			ShowCellBracket -> Automatic,
			ShowGroupOpener -> True,
			CellMargins -> {{24, 24}, {6, 18}},
			CellElementSpacings -> {"ClosedGroupTopMargin"->28},
			CellGroupingRules -> {"SectionGrouping", 50},
			CellFrameMargins -> 4,
			CellFrameColor -> RGBColor[
			0.48235300000000003`, 0.5176470000000001, 0.8196080000000001],
			StyleMenuListing -> None,
			FontFamily -> "Verdana",
			FontSize -> 9,
			FontTracking -> "Wide",
			FontColor -> GrayLevel[0]
		],

		Cell[
			StyleData["IndicatorCategorization"],
			CellMargins -> {{52, 12}, {2, 2}},
			ReturnCreatesNewCell -> True,
			CellChangeTimes -> {3.475262460013495*^9},
			FontFamily -> "Verdana",
			FontSize -> 11
		],

		Cell[
			StyleData["IndicatorAbbreviationSection"],
			Editable -> False,
			CellFrame -> {{0, 0}, {0, 2}},
			ShowCellBracket -> Automatic,
			ShowGroupOpener -> True,
			CellMargins -> {{24, 24}, {6, 18}},
			CellElementSpacings -> {"ClosedGroupTopMargin"->28},
			CellGroupingRules -> {"SectionGrouping", 50},
			CellFrameMargins -> 4,
			CellFrameColor -> RGBColor[
				0.48235300000000003`, 0.5176470000000001, 0.8196080000000001
			],
			StyleMenuListing -> None,
			FontFamily -> "Verdana",
			FontSize -> 9,
			FontTracking -> "Wide",
			FontColor -> GrayLevel[0]
		],

		Cell[
			StyleData["IndicatorAbbreviation"],
			CellMargins -> {{52, 12}, {2, 2}},
			ReturnCreatesNewCell -> True,
			CellChangeTimes -> {3.475262460013693*^9},
			FontFamily -> "Verdana",
			FontSize -> 11
		],

		Cell[
			StyleData["IndicatorFormulaSection"],
			Editable -> False,
			CellFrame -> {{0, 0}, {0, 2}},
			ShowCellBracket -> Automatic,
			ShowGroupOpener -> True,
			CellMargins -> {{24, 24}, {6, 18}},
			CellElementSpacings -> {"ClosedGroupTopMargin"->28},
			CellGroupingRules -> {"SectionGrouping", 50},
			CellFrameMargins -> 4,
			CellFrameColor -> RGBColor[
			0.48235300000000003`, 0.5176470000000001, 0.8196080000000001],
			StyleMenuListing -> None,
			FontFamily -> "Verdana",
			FontSize -> 9,
			FontTracking -> "Wide",
			FontColor -> GrayLevel[0]
		],

		Cell[
			StyleData["IndicatorFormula"],
			CellMargins -> {{52, 12}, {2, 2}},
			ReturnCreatesNewCell -> True,
			CellChangeTimes -> {3.475262460013907*^9},
			FontFamily -> "Verdana",
			FontSize -> 11
		],

		Cell[
			StyleData["IndicatorUsage"],
			Editable -> False,
			CellFrame -> {{0, 0}, {0.5, 0.5}},
			ShowCellBracket -> Automatic,
			CellMargins -> {{24, 8}, {6, 6}},
			CellFrameColor -> RGBColor[0.7490200000000001, 0.694118, 0.5529410000000001],
			Hyphenation -> False,
			ParagraphSpacing -> {0.5, 0},
			StyleMenuListing -> None,
			FontFamily -> "Verdana",
			FontSize -> 11,
			GridBoxOptions -> {
				BaseStyle -> {FontFamily -> "Verdana"},
				GridBoxAlignment -> {
					"Columns" -> {{Left}},
					"ColumnsIndexed" -> {},
					"Rows" -> {{Baseline}},
					"RowsIndexed" -> {}
				},
				GridBoxItemSize -> {
					"Columns" -> {
						Scaled[0.003],
						{Scaled[0.996]}
					},
					"ColumnsIndexed" -> {},
					"Rows" -> {{1.}},
					"RowsIndexed" -> {}
				}
			}
		]
	};

    (* This is inserted into the notebook at the end. *)
    stylesheet = Switch[lang,
		"Japanese",
			FrontEnd`FileName[{"Wolfram"}, "Reference-ja.nb"],
		"ChineseSimplified",
			FrontEnd`FileName[{"Wolfram"}, "Reference-zh.nb"],
		_,
			FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]
	];

	(*----------------------*)
	(* Populate `styledefs` *)
	(*----------------------*)

	styledefs = {};

    If[type === "Program",
      styledefs = Join[styledefs, {
			Cell[
				StyleData["InlineFormula"],
				FontSize -> FEPrivate`Switch[FEPrivate`$OperatingSystem,
					"MacOSX", 14,
					"Windows", Inherited*1.025,
					"Unix", 13,
					_, Inherited
				]
			],
			Cell[
				StyleData["ProgramCodeText"],
				FontSize -> FEPrivate`Switch[FEPrivate`$OperatingSystem,
					"MacOSX", 14,
					"Windows", 14,
					"Unix", 13,
					_, Inherited
				]
			]
		}]
	];

	If[MatchQ[type, "Workflow" | "Workflow Guide"],
		styledefs = Join[styledefs, {
			Cell[StyleData["Input"], CellMargins->{{120, 10}, {5, 11}}, FontSize->14, FontFamily->"Source Sans Pro"],
			Cell[StyleData["Output"], CellMargins->{{120, 10}, {11, 5}}, FontSize->14, FontFamily->"Source Sans Pro"],
			Cell[
				StyleData["InlineFormula"],
				FontColor -> GrayLevel[0],
				FontFamily -> "Source Sans Pro",
				FontSize -> FEPrivate`Switch[FEPrivate`$OperatingSystem,
					(*"MacOSX", 14,
					"Windows", Inherited*0.99,
					"Unix", 13,*)
					_, Inherited
				]
			],
			Cell[StyleData["StandardForm"], FontFamily->"Source Sans Pro"],
			Cell[StyleData["CellLabel"], FontFamily->"Helvetica", FontColor->GrayLevel[0.6]],
			Cell[StyleData["Link"], FontFamily->"Source Sans Pro"]
		}]
	];

	If[MatchQ[type, "Message"],
		styledefs = Join[styledefs, {
			Cell[StyleData["ObjectName"], FontColor -> RGBColor[0.866667, 0.066667, 0.]]
		}]
	];

	If[MatchQ[type, "Blockchain"],
		styledefs = Join[styledefs, {
			Cell[StyleData["BlockchainSection", StyleDefinitions->StyleData["ServiceSubsection"]]]
		}]
	];

	If[MatchQ[type, "Tutorial"|"Tech Note"|"Monograph"|"Monograph Overview"],
		styledefs = Join[styledefs, {
			Cell[StyleData["Input"], CellContext -> Notebook],
			Cell[StyleData["Output"], CellContext -> Notebook],
			Cell[StyleData["TutorialMoreAboutSection"], CellGroupingRules -> {"SectionGrouping", 30}],
			Cell[StyleData["RelatedTutorialsSection"], CellGroupingRules -> {"SectionGrouping", 30}],
			Cell[StyleData["TutorialRelatedLinksSection"], CellGroupingRules -> {"SectionGrouping", 30}]
		}]
	];

	If[type === "Overview",
		styledefs = Join[styledefs, {
			Cell[StyleData["Input"], CellContext -> Notebook],
			Cell[StyleData["Output"], CellContext -> Notebook],
			Cell[StyleData["TOCSubsubsection"], CellGroupingRules -> {"SectionGrouping", 69}],
			Cell[StyleData["TutorialMoreAboutSection"], CellGroupingRules -> {"SectionGrouping", 30}],
			Cell[StyleData["RelatedTutorialsSection"], CellGroupingRules -> {"SectionGrouping", 30}],
			Cell[StyleData["TutorialRelatedLinksSection"], CellGroupingRules -> {"SectionGrouping", 30}]
		}]
	];

	(* If[type === "HowTo",
		styledefs = Join[styledefs, {
			Cell[StyleData["Input"], CellContext -> Notebook],
			Cell[StyleData["Output"], CellContext -> Notebook]
		}]
	]; *)

	(* If[type === "Indicator",
		styledefs = Join[styledefs, indicatorstyles]
	]; *)

	If[nbContext === "Global`",
		styledefs = Join[styledefs, {
			Cell[StyleData["Input"], CellContext -> "Global`"],
			Cell[StyleData["Output"], CellContext -> "Global`"]
		}]
	];

	If[nbContext === "Notebook",
		styledefs = Join[styledefs, {
			Cell[StyleData["Input"], CellContext -> Notebook],
			Cell[StyleData["Output"], CellContext -> Notebook]
		}]
	];

	If[nbContext === Notebook,
		styledefs = Join[styledefs, {
			Cell[StyleData["Input"], CellContext -> Notebook],
			Cell[StyleData["Output"], CellContext -> Notebook]
		}]
	];

	If[TrueQ[functionPaclet],
		styledefs = Join[styledefs, {
			Cell[
				StyleData["AnchorBarActionMenu"],
				Background -> RGBColor[0.968627, 0.960784, 0.964706],
				FontColor -> RGBColor[0.0352941, 0.596078, 1.]
			],
			Cell[
				StyleData["Usage"],
				Background -> GrayLevel[0.972549],
				GridBoxOptions -> {
					GridBoxDividers -> {
						"RowsIndexed" -> {
							1 -> Directive[GrayLevel[0.576471], Thickness[3]],
							-1 -> Thickness[0]
						}
					}
				}
			]
		}]
	];

	If[Length[styledefs] > 0,
		stylesheet = Notebook[Join[
			{Cell[StyleData[StyleDefinitions -> stylesheet]]},
			styledefs
		]]
	];
	BuildLog["   Stylesheet: "<>ToString[stylesheet]];



    (* These are inserted into the notebook at the end. *)
	(* FIXME *)
	Global`AntLog["Starting time2"];
	time2 = ToString @ AbsoluteTiming[
		overviewLinks = DocumentationBuild`Utils`GetOverviewButtonLinks["paclet:"<>uri, lang];
	][[1]];
	If[debug==="True",Global`AntLog["Running GetOverviewButtonLinks: "<>time2]];

    (* FIXME: should this be in info? *)
	time2a = ToString @ AbsoluteTiming[
		classLabel = Transmogrify`ConvertToString[GetClassificationBoxes[info, lang, "Rebranded"->$Rebranded]];
    ][[1]];

    If[debug==="True",Global`AntLog["Running ConvertToString: "<>time2a]];

    classLabel = If[functionPaclet, "Mathematica Function Paclet Symbol", classLabel];

	(* List of version numbers -> years list for history cell: {ver->year, ...} *)
	versionYears = {
		"1"->"1988", "1.2"->"1989",
		"2"->"1991", "2.1"->"1992", "2.2"->"1993",
		"3"->"1996",
		"4"->"1999", "4.1"->"2000", "4.2"->"2002",
		"5"->"2003", "5.1"->"2004", "5.2"->"2005",
		"6"->"2007", "6.0.1"->"2007", "6.0.2"->"2008", "6.0.3"->"2008",
		"7"->"2008", "7.0.1"->"2009",
		"8"->"2010", "8.0.1"->"2011", "8.0.4"->"2011",
		"9"->"2012", "9.0.1"->"2013",
		"10"->"2014", "10.0.1"->"2014", "10.0.2"->"2014",
		"10.1"->"2015", "10.1.1"->"2015",
		"10.2"->"2015",
		"10.3"->"2015",
		"10.4"->"2016",
		"11"->"2016",
		"11.1"->"2017", "11.1.1"->"2017",
		"11.2"->"2017",
		"11.3"->"2018",
		"12"->"2019",
		"12.1"->"2020", "12.1.1"->"2020",
		"12.2"->"2020",
		"12.3"->"2021",
		"13"->"2021",
		"13.1"->"2022",
		"13.2"->"2022"
	};

    (* Store the history cell for later insertion *)
	Module[{
		newIn, modIn, newVers, modVers, inc
	},
		inc = IncludeHistory /. {opts} /. Options[MakeNotebook];

		{newIn, modIn} = {"New", "Modified"} /. ("HistoryData" /. info);

		newIn = If[
			(StringQ[newIn] && !StringMatchQ[newIn, RegularExpression["\\s+|\\?+|XX+"]]),
			newIn,
			""
		];

		(* Handle future/UnderDev pages *)
		If[!StringMatchQ[newIn, (DigitCharacter | ".")..] && TrueQ["Underdevelopment" /. info],
			newIn = StringReplace[ToString[$VersionNumber], "."~~EndOfString -> ""]
		];

		modIn = If[(StringQ[modIn] && !StringMatchQ[modIn, RegularExpression["\\s+|\\?+|XX+"]]),
			modIn,
			Null
		];
		(*modIn = If[!StringFreeQ[modIn, ","], Last[StringSplit[modIn, ","]], modIn];*)

		modIn = If[StringQ[modIn],
			StringTrim /@ StringSplit[modIn, ","],
			modIn
		];

		newVers = If[StringFreeQ[newIn, "."],
			newIn <> ".0",
			newIn
		];
		modVers = If[StringFreeQ[#, "."],
			# <> ".0",
			#
		]& /@ modIn;

		(* Substitute in years for version numbers *)
		newIn = ReplaceAll[
			StringReplace[newIn, ".0"~~EndOfString -> ""],
			versionYears
		];
		modIn = ReplaceAll[
			StringReplace[#, ".0"~~EndOfString -> ""]& /@ modIn,
			versionYears
		];

		If[!MatchQ[newIn, $Failed | Null | ""],
			If[! MatchQ[modIn, $Failed | Null | "" | {}],
				historyCell = Cell[
					TextData[{
						Localized["HistoryNewIn", lang, newIn],
						StyleBox["\[NonBreakingSpace]("<> ToString[newVers] <>")", "HistoryVersion"],
						StyleBox["\[NonBreakingSpace]| ", "HistoryDelimiter"],
						Localized["HistoryModIn", lang, First[modIn]],
						StyleBox["\[NonBreakingSpace](" <> ToString[First[modVers]] <> ")", "HistoryVersion"],
						If[Length[modIn] > 1,
							Apply[
								Sequence,
								MapThread[
									{
										StyleBox["\[NonBreakingSpace]\[FilledVerySmallSquare]\[ThinSpace]", "HistoryDelimiter",FontSlant->"Plain"],
										#1,
										StyleBox["\[NonBreakingSpace]("<> ToString[#2] <>")", "HistoryVersion"]
									}&,
									{Rest[modIn], Rest[modVers]}
								],
								{0,1}
							]
							,
							Nothing
						]
					}],
					"History"
				]
				,
				historyCell = Cell[
					TextData[{Localized["HistoryNewIn", lang, newIn], StyleBox["\[NonBreakingSpace]("<> ToString[newVers] <>")", "HistoryVersion"]}],
					"History"
				]
			]
			,
			historyCell = {}
		];

		(* export citation info for use in a bit *)
		citationInfo = {newIn, If[Length[modIn] > 0, Last[modIn], ""]};

		historyCell = If[(inc === True) && (("PacletName" /. info) === "Mathematica") &&
			MatchQ[type, "Symbol" | "Format" | "MathLink C Function" | "Character Name" | "Interpreter" | "Entity" | "Classifier" | "Predictor" |
			"Net Encoder" | "Net Decoder" | "Service Connection" | "Device Connection" | "Compiled Type" | "Embedding Format" | "External Evaluation System" |
			"Front End Object" | "Indicator" | "Method" | "File" | "Program" | "Database Connection" | "Data Structure" | "Text Content" |
      "Remote Batch Computation Provider" | "Featureset Page"],
			{historyCell},
			{}
		];
	];

    (* Citation *)
    citationCell = {};

	If[MemberQ[{"Symbol"}, type],
		citationInfo = DocumentationBuild`Utils`BuildCitation[citationInfo, info, lang];

		If[!FailureQ[citationInfo],
			Assert[MatchQ[citationInfo, {_, _}]];

			citationCell = DocumentationBuild`Utils`BuildCitationCell[lang];
		];
	];

    (*expr = DeleteCases[expr, (Cell[CellGroupData[{#, ___}, ___], ___] | #)&[
        Cell[__,
          "GuideTutorialCollection" | "GuideTutorialCollectionSection" | "TutorialCollectionSection" | "TutorialCollection",
        ___]
      ], Infinity];*)
 
    (* change MoreAbout stub "Autogenerated" with XXX so it's removed natuarlly *)
    expr = expr /. Cell["Autogenerated", "MoreAbout", ___] :> Cell["XXXX", "MoreAbout"];

    (* Convert *newer* style of ModInfo first *)
    expr = expr /. Cell[BoxData[TooltipBox[Cell[v_, "ModInfo", vv___], tt_String]], "ModInfoWrapper"] :>
          Cell[tt, "ModInfo", vv];

    (* Remove new style of ModInfo *)   
    expr = expr /. Cell[BoxData[TooltipBox[Cell[v_, "ModInfo", vv___], ___], ___], ___] :> Cell[v, "ModInfo", vv];
    expr = expr /. TooltipBox[Cell[v_, "ModInfo", vv___], _] :> Cell[v, "ModInfo", vv];

	(* Formatting note: This code includes fix for bug #112931. Original commit: 8fb0698. *)
	BuildLog["-- Delete table rows with XXXX"];
	poss = Position[expr,
		Cell[
			BoxData[GridBox[___]],
			("1ColumnTableMod" | "2ColumnTableMod" | "3ColumnTableMod" | "3ColumnIllustratedTableMod" | "DefinitionBox"),
			___
		],
		Infinity
	];
	Scan[
		Function[{cellpos}, Module[{gridpos, grid},
			gridpos= Join[cellpos, {1, 1}];
			grid = Extract[expr, gridpos];
			If[buildmode === "Future",
				expr,
				expr = Delete[
					expr,
					Map[
						Join[gridpos, {1}, #]&
						,
						Position[
							First[grid],
							{
								___,
								(s_String /; StringMatchQ[s, RegularExpression["\\s*XX+"] | RegularExpression["\\s*xxx+"]])
								| (Cell[s_String /; StringMatchQ[s, RegularExpression["\\s*XX+"] | RegularExpression["\\s*xxx+"]], ___]),
								___
							}
						]
					]
				]
			];
		]],
		poss
	];

    (* Get tagged tables' TaggingRules (autocomplete data) before they're stripped out in the next block of code *)
    GetTableTaggingRules[expr_] :=
      Cases[expr, _["TemplateArgumentValues", c_] :> c, Infinity];

    If[MatchQ[type, "Symbol"],
		tabletaggingrules = GetTableTaggingRules[expr]
	];

    (* determine if custom guide page before tagging rules are removed *)
    If[MatchQ[type, "Guide"],
      CustomGuideQ = TrueQ[Or@@Cases[expr, _["CustomGuideQ", c_] :> c, Infinity]]
	];

	(*-----------------------------------------------*)
	(* Delete a handful of internal, XXX, etc. cells *)
	(*-----------------------------------------------*)

	BuildLog["-- Delete various cells"];
	If[buildmode === "Future",
		expr = DeleteCases[
			expr,
			Alternatives[
				(* Flags and other things we use for internal purposes *)
				(Cell[CellGroupData[{#, ___}, ___], ___] | #)&[
					Cell[__,
					"FutureFlag" | "ExcisedFlag" | "InternalFlag" | "ObsoleteFlag" |
					"TemporaryFlag" | "AwaitingFutureDesignReviewFlag" | "PreviewFlag" |
					"AllowableOptions" | "AllowableOptionsSection" | "ApplicationNotesSection" |
					"Attributes" | "AttributesSection" | "ExampleStatusSection" |
					"Categorization" | "CategorizationSection" | "DesignDiscussionSection" |
					"DocumentStatus" | "DocumentStatusSection" | "Details" | "DetailsSection" |
					"ErrorMessages" | "ErrorMessagesSection" | "ExtensionsSection" |
					"History" | "Keywords" | "KeywordsSection" | "TestCasesSection" |
					"LegacyMaterialSection" | "Synonyms" | "SynonymsSection" |
					"Template" | "TemplatesSection" |
					"UsageMessages" | "UsageMessagesSection" |
					"WebResources" | "WebResourcesSection" | "AuthorDate" | "SecuritySection" | "SecurityDetails",
					___]
				],
				(* Cell groups that don't contain much of anything *)
				Cell[CellGroupData[{_, Cell["" | " ", ___]}, ___], ___],
				(* Cell labels used for modification version *)
				_[CellLabel, s_String] /; StringMatchQ[s, RegularExpression["^\\d[+~Xx]"] ],
				(* All CellChangeTimes and TaggingRules, except in Custom Guide Pages *)
				If[TrueQ[CustomGuideQ],
					_[System`CellChangeTimes, _]
					,
					_[System`CellChangeTimes | TaggingRules, _]
				]
			],
			Infinity
		]
		,
		expr = DeleteCases[
			expr,
			Alternatives[
				(* Flags and other things we use for internal purposes *)
				(Cell[CellGroupData[{#, ___}, ___], ___] | #)&[
					Cell[__,
					"FutureFlag" | "ExcisedFlag" | "InternalFlag" | "ObsoleteFlag" |
					"TemporaryFlag" | "AwaitingFutureDesignReviewFlag" | "PreviewFlag" |
					"AllowableOptions" | "AllowableOptionsSection" | "ApplicationNotesSection" |
					"Attributes" | "AttributesSection" | "ExampleStatusSection" |
					"Categorization" | "CategorizationSection" | "DesignDiscussionSection" |
					"DocumentStatus" | "DocumentStatusSection" | "Details" | "DetailsSection" |
					"ErrorMessages" | "ErrorMessagesSection" | "ExtensionsSection" |
					"History" | "Keywords" | "KeywordsSection" | "TestCasesSection" |
					"LegacyMaterialSection" | "Synonyms" | "SynonymsSection" |
					"Template" | "TemplatesSection" |
					"UsageMessages" | "UsageMessagesSection" |
					"WebResources" | "WebResourcesSection" | "AuthorDate" | "SecuritySection" | "SecurityDetails",
					___]
				],
				(* Cell groups that don't contain much of anything *)
				Cell[CellGroupData[{_, Cell["" | " ", ___]}, ___], ___],
				(* Cell labels used for modification version *)
				_[CellLabel, s_String] /; StringMatchQ[s, RegularExpression["^\\d[+~Xx]"] ],
				(* All CellChangeTimes and TaggingRules, except in Custom Guide Pages *)
				If[TrueQ[CustomGuideQ],
					_[System`CellChangeTimes, _]
					,
					_[System`CellChangeTimes | TaggingRules, _]
				],
				(* All CellDynamicExpressions *)
				_[CellDynamicExpression, _],
				(* FutureExample cells that aren't ModInfo cells *)
				Cell[pre___, "FutureExample" | "TentativeExample", post___, _[CellID, _], ___] /; FreeQ[{pre, post}, "ModInfo"],
				(* XX... cells, just dummy placeholders *)
				Cell[c_String /; StringMatchQ[c, RegularExpression["\\s*XX+"]], ___]
			],
			Infinity
		]
	];

	BuildLog["-- Remove Future tagged table "];
	If[buildmode === "Future",
		expr
		,
		poss = Position[expr, _GridBox, Infinity];
		expr = ReplacePart[
			expr,
			Flatten[Map[
				Function[{pos}, Module[{grid},
					grid = Extract[expr, pos];
					If[
						MatchQ[
							First[grid],
							{___, {___, (Cell|StyleBox)[___, "FutureExample" | "TentativeExample", ___], ___}, ___}
						]
						,
						(pos -> ReplacePart[
							grid,
							{1} -> DeleteCases[First[grid], {___, (Cell|StyleBox)[___, "FutureExample" | "TentativeExample", ___], ___}]
						])
						,
						{}
					]
				]],
				poss
			]]
		]
	];

	(* Remove ConversionSpecial and corresponding CellTag *)
	expr = expr /. {
		Cell[
			c__,
			"ConversionSpecial",
			o1___,
			CellTags -> t_List /; {a___, "InProductOnly", b___},
			o2___
		] :> Cell[c, o1, CellTags->{a,b}, o2]
		,
		Cell[
			c__,
			"ConversionSpecial",
			o1___,
			CellTags -> t_String /; StringMatchQ[t, "InProductOnly"],
			o2___
		] :> Cell[c, o1, o2]
	};

	expr = expr /. {
		Cell[
			c__,
			"ConversionSpecial",
			o1___,
			CellTags -> t_List /; {a___, "OnWebOnly", b___},
			o2___
		] :> Cell[
			c, o1, CellTags->{a,b}, o2,
			ShowCellBracket -> False,
			CellElementSpacings -> {"ClosedCellHeight" -> 0},
			CellOpen -> False
		]
		,
		Cell[
			c__,
			"ConversionSpecial",
			o1___,
			CellTags -> t_String /; StringMatchQ[t, "OnWebOnly"],
			o2___
		] :> Cell[
			c, o1, o2,
			ShowCellBracket -> False,
			CellElementSpacings -> {"ClosedCellHeight" -> 0},
			CellOpen -> False
		]
	};

	(* Remove XX+ tables and preceding notes cells *)
	If[buildmode === "Future",
		expr,
		poss = Position[expr,
			Cell[
				BoxData[GridBox[
					{{_, s_String /; StringMatchQ[s, RegularExpression["\\s*XX+"]], ___}..},
					___
				]],
				("1ColumnTableMod" | "2ColumnTableMod" | "3ColumnTableMod" | "3ColumnIllustratedTableMod"),
				___
			],
			Infinity
		];
		expr = Delete[
			expr,
			Join @@ Map[
				If[MatchQ[Extract[expr, Append[Most[#], Last[#]-1]], Cell[_, "Notes", ___]],
					{Append[Most[#], Last[#]-1], #},
					{#}
				]&
				,
				poss
			]
		]
	];

	Global`AntLog["Starting time3"];
	time3 = ToString @ AbsoluteTiming[
		expr = AdjustModInfo[expr, info, opts] ;
	][[1]];
	If[debug==="True",Global`AntLog["Running AdjustModInfo: "<>time3]];

	(* More About section needs to be populated before anchor bar is created. Only rewrite for Symbols. *)
	Global`AntLog["Starting time4"];
	time4 = ToString @ AbsoluteTiming[
		If[!MemberQ[{"Guide", "Tutorial", "Overview", "Workflow", "Workflow Guide", "Root Guide", "Monograph", "Monograph Overview", "Tech Note", "Example"}, type],
			expr = EnsureCellGroupData[expr, "MoreAboutSection", lang]
		];
	][[1]];
	If[debug==="True",Global`AntLog["Running EnsureCellGroupData: "<>time4]];

	poss = Position[
		expr,
		CellGroupData[{Cell[_, "MoreAboutSection", ___], ___}, ___],
		Infinity,
		1
	];
	BuildLog["   Position of section(s):"<>ToString@moreAboutPos];

	Global`AntLog["Starting time5"];
	time5 = ToString @ AbsoluteTiming[
		If[MatchQ[type, "Symbol"],
			If[Length[poss] > 0,
				poss = First[poss];

				Module[{maCell, maNew, maManual},
					(* maManual allows authors to put in More Abouts that aren't autogenerated *)
					maManual = Cases[
						expr,
						(Cell[c_, ___, "MoreAbout", ___] /; !MatchQ[c, "XXXX"|"Autogenerated"]) :> {
							Transmogrify`ConvertToString[c],
							First[Cases[c, _[ButtonData,ur_String] :> ur, Infinity]]
						},
						Infinity
					];

					maCell = Extract[expr, Join[poss, {1, 1}]];
					maNew = DocumentationBuild`Info`GetSourceInformation["paclet:" <> uri, lang, "Links", "Guides"];

					If[maNew == None,
						maNew = {}
					];

					maNew = DeleteDuplicates[Join[maNew, Select[maManual, FreeQ[Last/@maNew, #[[2]]]&]]];

					If[buildmode === "Future",
						maNew,
						maNew = Select[
							maNew,
							FreeQ[
								{"ExcisedFlag", "FutureFlag", "InternalFlag", "TemporaryFlag"},
								DocumentationBuild`Info`GetSourceInformation[#[[2]], lang, "Flag"]
							]&
						]
					];

					(* 1/15/15: edit below to remove all SummaryOfNewFeatures* and NewIn* guides *)
					maNew = DeleteCases[
						maNew,
						{_, s_String /; StringMatchQ[s, "paclet:guide/AlphabeticalListing" | "paclet:guide/SummaryOfNewFeaturesIn*" | ("paclet:guide/NewIn" ~~ DigitCharacter.. ~~ __)]}
					];

					(* 1/13/15: Commenting out the below to allow heuristic sorting of Related Guides *)
					(*maNew = Sort[maNew,
						Module[{s1, s2, m1, m2},
						{s1, s2} = StringDrop[#, StringPosition[#, "guide/"][[-1, -1]]]& /@ {#1[[2]], #2[[2]]};
						{m1, m2} = StringMatchQ[#, "NewIn*"] & /@ {s1, s2};
						Which[
							m1 && m2, OrderedQ[{s1, s2}],
							m1, False,
							m2, True,
							True, OrderedQ[{s1, s2}] ]]& ];*)

					If[Length[maNew] > 0,
						expr = ReplacePart[
							expr,
							poss -> CellGroupData[
								Join[
									{maCell},
									Map[
										If[Head[#] === Cell,
											DeleteCases[#, r_[CellChangeTimes, _], Infinity]
											,
											Cell[
												TextData[ButtonBox[
													If[(Head@First[#] =!= String && Length@First@First[#] > 0),
														First@First[#],
														First[#]
													],
													BaseStyle -> "Link",
													ButtonData -> Last[#]
												]],
												"MoreAbout",
												CellID->1
											]
										]&,
										maNew
									]
								],
								Open
							]
						]
					];
				]
			]
		];
	][[1]];

	If[debug==="True",Global`AntLog["Create MoreAbout: "<>time5]];



    (* delete GridBox Options on nColumnTableMod styled cells *)
    expr = expr /.
      Cell[a_, n:("1ColumnTableMod"|"2ColumnTableMod"|"3ColumnTableMod"|"3ColumnIllustratedTableMod"|"4ColumnTableMod"), d___, GridBoxOptions->_, y___] :> 
        Cell[a, n, d, y];
        
    expr = expr /. Cell[a___, Rule[CellFrameLabels, {{cf1_, cf2_}, {cf3_, Cell[___, "TextAnnotation",___]}}], b___] :> Cell[a, b];
        
	Global`AntLog["Starting time6"];
	time6 = ToString@AbsoluteTiming[
		expr = ReplacePart[expr, (# -> DocumentationBuild`Utils`MakePacletButton[Extract[expr, #]]) & /@ Position[expr, ButtonBox[_String, ___]]];
	][[1]];

	If[debug==="True",Global`AntLog["Running MakePacletButton: "<>time6]];

	(* Formatting note: e68c54f7: fixing menuitems showing up in pull down menus for tutorials *)
	If[!MatchQ[type, "Workflow"],
		expr = expr /.
			StyleBox[con_, sty:("MenuName"|"FullListing"), opt___] :>
				StyleBox[
					(con /. ButtonBox[bb__] :> Module[{bs = Cases[{bb}, (BaseStyle->s_):>s]},
						Apply[
							ButtonBox,
							Append[
								DeleteCases[{bb}, _[BaseStyle, _]],
								BaseStyle -> Append[If[ListQ[bs], bs, {bs}], sty]
							]
						]
					]),
					sty,
					opt
				];
	];

	Global`AntLog["Starting time7"];
	time7 = ToString @ AbsoluteTiming[
		(* Add label *)
		expr = AddNotebookLabel[expr, info, lang, "BuildMode"->buildmode, "PacletName" -> pacletName, "Rebranded" -> $Rebranded];
	][[1]];

	If[debug==="True", Global`AntLog["Running AddNotebookLabel: "<>time7]];

	(* Formatting note: 4210f70b: removed color bar and linktrails since they are now in the docked cell *)
(*
	time9 = ToString@AbsoluteTiming[
		(* Add color bar *)
		expr = AddHeaderColorBar[expr, info];
	][[1]];

	If[debug==="True",Global`AntLog["Running AddHeaderColorBar: "<>time9]];
*)

	expr = expr /. {
		(* >> and >> -> \[RightGuillemet] *)
		(* Formatting note: 2e57865f: Fix for RightSkeleton bug (#142434). *)
		ButtonBox[c_String /; StringMatchQ[c, RegularExpression["\\s*\\\\\\[RightSkeleton\\]\\s*"]], Rule[BaseStyle,{"Link","PreserveRightSkeletonLink"}], op___] :>
			ButtonBox["PRESERVERIGHTSKELETONLINK", Rule[BaseStyle,"PreserveRightSkeletonLink"],op],

		ButtonBox[c_String /; StringMatchQ[c, RegularExpression["\\s*\\\\\\[RightSkeleton\\]\\s*"]], o___] :>
			ButtonBox["\[RightGuillemet]", o],

		(* ExcludedObject is only for authoring nb *)
		ButtonBox[f__, BaseStyle->{"Link", "ExcludedObject"}, l___] :>
			ButtonBox[f, BaseStyle->"Link", l],
		Cell[f__, "ExcludedObject", l___] :> Cell[f, l],

		(* PrimaryObject is only for authoring nb *)
		ButtonBox[f__, BaseStyle->{"Link", "PrimaryObject"}, l___] :>
			ButtonBox[f, BaseStyle->"Link", l],

		(* Remove second style for Input/Output *)
		Cell[c_, "Input", "HiddenMaterial", o___] :> Cell[c, "Input", o],
		Cell[c_, "Output", "HiddenMaterial", o___] :> Cell[c, "Output", o]
	};

	(* Remove second style for Input, Deemphasis and place in Stylebox *)
	(* Formatting note: bf16105c: Fix Input, Deemphasis for DatabaseLink et al. *)
    Cell[BoxData[a_], "Input", "Deemphasis", b__] :> Cell[BoxData[StyleBox[a, "InputDeemphasis"]], "Input", b];

	expr = expr /. ButtonBox["PRESERVERIGHTSKELETONLINK", Rule[BaseStyle,"PreserveRightSkeletonLink"],opt___] :>
		ButtonBox["\[RightSkeleton]", Rule[BaseStyle,"Link"], opt];

	(* Change SeeAlso strings to links *)
	(* If 'SeeAlso' list is a string, make into a list of Buttons *)
	expr = expr /. {
		Cell[c_String, "SeeAlso", o___] :> InlineFunctionButton[Cell[c, "SeeAlso", o]]
	};

	(* Fix links which are InlineFormula cells and should be Strings *)
	expr = expr /. {
		Cell[
			TextData[Cell[BoxData[ButtonBox[bb__]], "InlineFormula"]],
			sty:("Tutorials"|"RelatedLinks"|"MoreAbout"|"Extensions"),
			o___
		] :> Cell[TextData[ButtonBox[bb]], sty, o]
	};

	Global`AntLog["Starting time10"];
	time10 = ToString @ AbsoluteTiming[
		(* Apply the specific conversions for this type *)
		expr = ConvertNotebook[expr, info, type, opts, "Rebranding" -> $Rebranded];
	][[1]];

	If[debug==="True",Global`AntLog["Running ConvertNotebook: "<>time10]];

    BuildLog["-- Buttons into Paclet uris"];
    (* manipulate some button boxen *)


(*    (* Add label *)
    expr = AddNotebookLabel[expr, info, lang];

    (* Add color bar *)
    expr = AddHeaderColorBar[expr, info];*)

    BuildLog["-- Renumber In/Out labels"];
    (* Renumber In/Out Cell labels *)
	Global`AntLog["Starting time11"];
	time11 = ToString @ AbsoluteTiming[
		expr = RenumberCells[
			expr,
			{
				"PrimaryExamplesSection", "ExtendedExamplesSection", "ExampleSection",
				"ExampleSubsection", "ExampleDelimiter", "Title"
			}
		];
    ][[1]];

    If[debug==="True",Global`AntLog["Running RenumberCells: "<>time11]];

    BuildLog["-- Remove backlinks"];
    expr = expr //. Cell[
		c__,
		CellFrameLabels -> {{x_,  Cell[TextData[ ButtonBox[___, Rule[_, "ExampleBacklink"] , ___]]]}, {y_,  z_}},
		o___
	] :> Cell[c, CellFrameLabels -> {{x, None}, {y, z}}, o];

    (* Convert the old Example link syntax *)
	expr = expr /.
      Cell[BoxData[ButtonBox["Example", o__]], "Input"] :>
        ButtonBox["\[NonBreakingSpace]\[RightGuillemet]", o];

	expr = expr /. {
		(* Remove 'More Examples' link for now... *)
	(*
		Cell[_, "PrimaryExamplesSection", o___] :>
			Cell[
				BoxData[
					InterpretationBox[
						StyleBox[#, "PrimaryExamplesSection"],
						($Line = 0; Null)
					] &[Localized["PrimaryExamplesSectionText", lang]]
				],
				"PrimaryExamplesSection",
				o
			],
	*)
		Cell[_, "ExtendedExamplesSection", o___] :>
			Cell[Localized["ExtendedExamplesSectionText", lang], "ExtendedExamplesSection", o],

		(* remove TOCExcludedObject style *)
		Cell[c__, "TOCExcludedObject", o___] :> Cell[c, o],

		{"Link", "TOCExcludedObject"} :> "Link"
	};

	Global`AntLog["Starting time12"];
	time12 = ToString @ AbsoluteTiming[
		(* Formatting note: 257042b6: Require 3rd party packages to put in ButtonBoxes for
		   inlineformulas.  Automation should only happen for Mathematica and related packages. *)
		Module[{
			pacletBase = StringReplace[pacletName, " Package"->""]
		},
			If[DocumentationBuild`Internal`SystemPacletQ[pacletBase],
				(* create self links *)
				expr = expr /. {
					Cell[BoxData[c_String /; ShippingFunctionQ[c, pacletBase] ], "InlineFormula", o___] :>
						Cell[BoxData[ButtonBox[c, BaseStyle->"Link", ButtonData->"paclet:"<>pacletBase<>If[StringLength[pacletBase]>0,"/",""]<>"ref/"<>c]],
						"InlineFormula",
						o
					],

					Cell[
						BoxData[
							RowBox[{c_String /; ShippingFunctionQ[c, pacletBase], "[", stuff___}, o1___],
							o2___
						],
						"InlineFormula",
						o3___
					] :> Cell[
						BoxData[
							RowBox[{ButtonBox[c, BaseStyle->"Link", ButtonData->"paclet:"<>pacletBase<>If[StringLength[pacletBase]>0,"/",""]<>"ref/"<>c], "[", stuff}, o1],
							o2
						],
						"InlineFormula",
						o3
					],

					Cell[BoxData[RowBox[{c_String /; ShippingFunctionQ[c, pacletBase], "->", stuff___},
						o1___], o2___], "InlineFormula", o3___
					] :> Cell[
						BoxData[
							RowBox[{ButtonBox[c, BaseStyle->"Link", ButtonData->"paclet:"<>pacletBase<>If[StringLength[pacletBase]>0,"/",""]<>"ref/"<>c], "->", stuff}, o1],
							o2
						],
						"InlineFormula",
						o3
					],

					Cell[
						BoxData[
							RowBox[
								{
									RowBox[
										{c_String /; ShippingFunctionQ[c, pacletBase], "[", stuff1___},
										opts1___
									],
									stuff2___
								},
								opts2___
							],
							opts3___
						],
						"InlineFormula",
						opts4___
					] :> Cell[
						BoxData[
							RowBox[
								{
									RowBox[{ButtonBox[c, BaseStyle->"Link", ButtonData->"paclet:"<>pacletBase<>If[StringLength[pacletBase]>0,"/",""]<>"ref/"<>c], "[", stuff1}, opts1],
									stuff2
								},
								opts2
							],
							opts3
						],
						"InlineFormula",
						opts4
					]
				}
		]];
	][[1]];

	If[debug==="True",Global`AntLog["Running ButtonBox Creation: "<>time12]];

	(*--------------------------------*)
	(* Handle \[EmptyVerySmallSquare] *)
	(*--------------------------------*)

	(* TODO: Perform this transformation on the authoring notebooks, and then remove this code?
	         How common is this? *)

	(* change \[EmptyVerySmallSquare] to "something else" *)
	expr = expr //. Cell[
		TextData[{a___, s_String /; StringMatchQ[s, RegularExpression["\\s*\[EmptyVerySmallSquare]\\s*"] ], b___}],
		sty_String /; StringMatchQ[sty, "InlineGuideFunctionListing" | "GuideText" | "RelatedLinks"|"SeeAlso"|"Extensions"|"Tutorials"|"RelatedEncoders"|"RelatedDecoders"],
		o___
	] :> Cell[
		TextData[{a, "\[NonBreakingSpace]", StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", "InlineSeparator"], " ", b}],
		sty,
		o
	];

	(* change \[EmptyVerySmallSquare] to "something else" for even more complex strings *)
	expr = expr //. Cell[
		TextData[{a___, s_String /; StringMatchQ[s, RegularExpression["\"\\w+\"\\s+\[EmptyVerySmallSquare]\\s+\\D+"] ], b___}],
		sty_String /; StringMatchQ[sty, "InlineGuideFunctionListing" | "GuideText" | "RelatedLinks"|"SeeAlso"|"Extensions"|"Tutorials"|"RelatedEncoders"|"RelatedDecoders"],
		o___
	] :> Cell[
		TextData[{
			a,
			"\[NonBreakingSpace]",
			StringSplit[
				s,
				RegularExpression["\\s+\[EmptyVerySmallSquare]\\s+"]
					-> Sequence[" ",StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", "InlineSeparator"], " "]
			],
			b
		}],
		sty,
		o
	];

	expr = expr //. Cell[
		TextData[{a___, s_String /; StringMatchQ[s, RegularExpression["\\s+\[EmptyVerySmallSquare]\\s+"]~~__ ], b___}],
		sty_String /; StringMatchQ[sty, "InlineGuideFunctionListing" | "GuideText" | "RelatedLinks"|"SeeAlso"|"Extensions"|"Tutorials"|"RelatedEncoders"|"RelatedDecoders"],
		o___
	] :> Cell[
		TextData[{
			a,
			"\[NonBreakingSpace]",
			StringSplit[
				s,
				RegularExpression["\\s+\[EmptyVerySmallSquare]\\s+"]
					-> Sequence[" ",StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", "InlineSeparator"], " "]
			],
			b
		}],
		sty,
		o
	];

	expr = expr //. Cell[
		TextData[{
			a___,
			s_String /; StringMatchQ[s, RegularExpression["\"\\w+\"\\s+\[EmptyVerySmallSquare]\\s+"]~~__ ],
			b___
		}],
		sty_String /; StringMatchQ[sty, "InlineGuideFunctionListing" | "GuideText" | "RelatedLinks"|"SeeAlso"|"Extensions"|"Tutorials"|"RelatedEncoders"|"RelatedDecoders"],
		o___
	] :> Cell[
		TextData[{
			a,
			"\[NonBreakingSpace]",
			StringSplit[
				s,
				RegularExpression["\\s+\[EmptyVerySmallSquare]\\s+"]
					-> Sequence[" ", StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", "InlineSeparator"], " "]
			],
			b
		}],
		sty,
		o
	];

	expr = expr //. Cell[
		TextData[{a___, s_String /; StringMatchQ[s, ")"~~RegularExpression["\\s+\[EmptyVerySmallSquare]\\s+\\D+"]~~__ ], b___}],
		sty_String /; StringMatchQ[sty, "InlineGuideFunctionListing" | "GuideText" | "RelatedLinks"|"SeeAlso"|"Extensions"|"Tutorials"|"RelatedEncoders"|"RelatedDecoders"],
		o___
	] :> Cell[
		TextData[{
			a,
			(*"\[NonBreakingSpace]",*)
			StringSplit[
				s,
				RegularExpression["\\s+\[EmptyVerySmallSquare]\\s+"]
					-> Sequence[StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", "InlineSeparator"], " "]
			],
			b
		}],
		sty,
		o
	];

	expr = expr //. Cell[
		TextData[{a___, s_String /; StringMatchQ[s, RegularExpression[" \\D+\\s+\[EmptyVerySmallSquare]\\s+"]~~__ ], b___}],
		sty_String /; StringMatchQ[sty, "InlineGuideFunctionListing" | "GuideText" | "RelatedLinks"|"SeeAlso"|"Extensions"|"Tutorials"|"RelatedEncoders"|"RelatedDecoders"],
		o___
	] :> Cell[
		TextData[{
			a,
			"\[NonBreakingSpace]",
			StringSplit[
				s,
				RegularExpression["\\s+\[EmptyVerySmallSquare]\\s+"]
					-> Sequence[" ",StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", "InlineSeparator"], " "]
			],
			b
		}],
		sty,
		o
	];

	expr = expr //. Cell[
		TextData[{a___, s_String /; StringMatchQ[s, ") \[EmptyVerySmallSquare] " ], b___}],
		sty_String /; StringMatchQ[sty, "InlineGuideFunctionListing" | "GuideText" | "RelatedLinks"|"SeeAlso"|"Extensions"|"Tutorials"],
		o___
	] :> Cell[
		TextData[{a, ")\[NonBreakingSpace]", StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", "InlineSeparator"], " ", b}],
		sty,
		o
	];

    (* GuideTOCLinks need a RightGuillemet *)
    expr = addGuideTOCRightGuillemet[expr];

(*    expr = expr /. {
      (* >> and >> -> \[RightGuillemet] *)
      ButtonBox[c_String /; StringMatchQ[c, RegularExpression["\\s*\\\\\\[RightSkeleton\\]\\s*"]], o___] :> 
        ButtonBox["\[RightGuillemet]", o],

      (* ExcludedObject is only for authoring nb *)
      ButtonBox[f__, BaseStyle->{"Link", "ExcludedObject"}, l___] :>
        ButtonBox[f, BaseStyle->"Link", l],
      Cell[f__, "ExcludedObject", l___] :> Cell[f, l],

      (* PrimaryObject is only for authoring nb *)
      ButtonBox[f__, BaseStyle->{"Link", "PrimaryObject"}, l___] :>
        ButtonBox[f, BaseStyle->"Link", l],

      (* Remove second style for Input/Output *)
      Cell[c_, "Input", "HiddenMaterial", o___] :> Cell[c, "Input", o],
      Cell[c_, "Output", "HiddenMaterial", o___] :> Cell[c, "Output", o]
    };*)

	Global`AntLog["Starting time13"];
	time13 = ToString @ AbsoluteTiming[
		expr = ReplacePart[
			expr,
			(# -> DocumentationBuild`Utils`MakePacletButton[Extract[expr, #]]) &
				/@ Position[expr, ButtonBox[_String, ___]]
		];
	][[1]];

	If[debug==="True",Global`AntLog["Running MakePacletButton2: "<>time13]];

    expr = expr /.
      ButtonBox[c__, BaseStyle->"Link", o1___, ButtonData->p_String/;StringMatchQ[p, "paclet:note/*"], o2___] :>
        ButtonBox[c, BaseStyle->"Hyperlink", o1, 
          ButtonData -> {URL[ DotHTML[StringReplace[p, "paclet:"->"http://reference.wolfram.com/language/"]]],
            None}, o2];

    (* fix split hyperlinks *)
	Global`AntLog["Starting time14"];
	time14 = ToString @ AbsoluteTiming[
		expr = DocumentationBuild`Utils`Private`fixSplitHyperlinks[expr];
    ][[1]];

    If[debug==="True",Global`AntLog["Running fixSplitHyperlinks: "<>time14]];

    (* Have to do this late, because MakeCommon has a rule to kill GridBoxOptions *)
    BuildLog["-- Highlight ModInfo rows"];
	poss = Position[
		expr,
		(* Formatting note: dbbddb7a: fixed the rest of 337047 *)
		Cell[_, "1ColumnTableMod"|"2ColumnTableMod"|"3ColumnTableMod"|"3ColumnIllustratedTableMod"|"4ColumnTableMod", ___],
		Infinity
	];
    BuildLog["   Number of tables to check: "<> ToString@Length@poss];
	Scan[
		Module[{cexpr, grid, options},
			cexpr = Extract[expr, #];

			grid = Replace[Cases[cexpr, _GridBox, Infinity], {
				{first_, ___} :> first,
				{} :> (
					(* TODO: Make this an error? How common is this? *)
					DocsWarning[
						"``: expected Cell with style `` to contain a GridBox. Got: ``",
						uri,
						InputForm[cexpr[[2]]],
						InputForm[cexpr]
					];
					{{}}
				)
			}];

			options = Cases[cexpr, _[GridBoxOptions, s_] :> s];

			If[Length[options] > 0,
				options = First[options]
			];

			BuildLog["   Length opts: "<>ToString@Length@options];
			expr = ReplacePart[
				expr,
				# -> Append[
					DeleteCases[cexpr, _[GridBoxOptions, _]],
					GridBoxOptions -> Join[options, {
						RowBackgrounds -> Map[
							If[MatchQ[First[#], Cell["\[FilledVerySmallSquare]", "TableRowIcon", ___]],
								Dynamic[
									If[CurrentValue[{TaggingRules, "ModificationHighlight"}] === True,
										RGBColor[1, 0.988235, 0.721569],
										None
									]
								],
								None
							]&,
							First@grid
						],
						GridBoxDividers -> {
							"Rows" -> {
								Join[
									{True},
									Map[(Last[#] =!= "\[SpanFromLeft]")&, First@grid]

								]
							}
						}
					}]
				]
			]
		]&,
		poss
	];

    (* Replace any remaining ModInfo with a space TODO: Comment out for internal documentation variant *)
    expr = expr /. Cell[_, "ModInfo", o___] :> Cell[" ", "ModInfo", o];
	expr = If[buildmode === "Future",
		expr,
		DeleteCases[
			expr,
			(Rule|RuleDelayed)[CellDingbat, Cell[s_String /; StringMatchQ[s, RegularExpression["[\\s]*"]], "ModInfo", ___]],
			Infinity
		]
	];
	expr = If[buildmode === "Future",
		expr,
		DeleteCases[
			expr,
			Cell[s_String /; StringMatchQ[s, RegularExpression["[\\s]*"]], "ModInfo", ___],
			Infinity
		]
    ];

    (* remove empty cell groups *)
    BuildLog["-- Remove empty cell groups"];
Global`AntLog["Starting time15"];time15 = ToString@AbsoluteTiming[
    (* V9 EDIT BELOW *)
    expr = 
    	RemoveEmptyCellGroups[expr, 
			(* commenting out some alongside new attractor icon feature *)
			Flatten[{
				"CapitalizeHeadingStyles" /. Options[MakeNotebook],
				{"ExampleSection", "PrimaryExamplesSection", "ExampleSubsection", "ExampleSubsubsection", "GuideFunctionsSection", "GuideFunctionsSubsection", 
					"GuideMoreAboutDelimiter", (*"GuideRelatedLinksSection", *)"GuideDelimiter", (*"GuideTutorialsSection", "GuideMoreAboutSection", "GuideWorkflowsSection",*) 
					"IndicatorExampleSection", "IndicatorDescriptionSection", "IndicatorCategorizationSection", "IndicatorAbbreviationSection", "IndicatorFormulaSection", (*"SeeAlsoSection",*)
                     (*"RelatedLinksSection", "MoreAboutSection", "TutorialsSection", "TutorialRelatedLinksSection", "TutorialMoreAboutSection",
                     "RelatedTutorialsSection", *)"TutorialsTutorialCollectionSection", "RelatedDemonstrationsSection",
                     "FeaturedExampleMoreAboutSection", "FeaturedExampleSeeAlsoSection", "FeaturedExampleRelatedFunctionsSection", 
                     "FeaturedExampleRelatedLinksSection", "DeviceSubsection", "ServiceSubsection", "FunctionEssaySection",
                     "EmbeddingFormatSection", "InterpreterSection", "AlternativeNamesSection", "DesignJustificationSection",
                     "InternalLinksSection", "DesignChoicesSection", "FormatBackground", "NotebookInterfaceSection", "OptionsSection", "ElementsSection", 
                     "ExportElementsSection", "ProgramSubsection", "ProgramSection",
                     "WorkflowRelatedWorkflowsSection", "WorkflowRelatedFunctionsSection", "WorkflowTutorialsSection",
                     "WorkflowHeader", "WorkflowPlatform", "WorkflowNotesHeader", "WorkflowNotesSection", "WorkflowGuideRelatedGuidesSection", 
                     "WorkflowGuideRelatedLinksSection",
                     "WorkflowGuideSection", "MethodSection",
                     "RelatedEncoderSection", "RelatedDecoderSection", "RelatedModels", "ExternalEvaluateSystemSection",
                     (*"RelatedWorkflowsSection", "GuideWorkflowGuidesSection",*) "EntitySection"(*, "RelatedEntitySection"*), "CompiledTypeSection", "CompiledTypeSubsection", 
                     "BlockchainSection", "DatabaseConnectionSection", "ClassifierSection", "PredictorSection", "BatchComputationProviderSection", "QuestionInterfaceSection",
                     "ResourceObjectSection"}
			}] ];
    ][[1]];

    (* Removing empty sections that would get an attractor icon *)
	(* Formatting note: 7ee8e9eb:
		Exclude howtos from a process not meant for them

		https://jira.wolfram.com/jira/browse/WLDOCS-3575
		https://bugs.wolfram.com/show?number=400874
	*)
	If[type =!= "HowTo",
		expr = Module[{
			sectStyles = {
				"GuideRelatedLinksSection", "GuideTutorialsSection", "GuideMoreAboutSection", "GuideWorkflowsSection",
				"SeeAlsoSection", "RelatedLinksSection", "MoreAboutSection", "TutorialsSection", "TechNotesSection",
				"TutorialRelatedLinksSection", "TutorialMoreAboutSection", "RelatedTutorialsSection", "RelatedWorkflowsSection",
				"GuideWorkflowGuidesSection", "RelatedEntitySection"
			}
		},
			DeleteCases[expr, Cell[_String, Alternatives @@ sectStyles, ___], Infinity]
		]
	];

    If[debug==="True",Global`AntLog["Running RemoveEmptyCellGroups2: "<>time15]];

	(* Remove short forms, etc. 'Map (/@)' *)
	(* Formatting note: ccffbc9332884fdfa96c55bf9331f8664bf6b50e: fixing direct hit for <comment> titles *)
	ShortFormTitle[pageTitle_] :=
		Module[{s, o = pageTitle},
			s = StringSplit[pageTitle, " "];
			If[(Length[s] > 1) && StringMatchQ[s[[2]], "(*"],
				First[s],
				o
			]
		];


	(* Italicize AutoItalicWords in MoreAbout cells *)
	(* Formatting note: edecb35a: updates to fix italics for Important Words in More About cells *)
	moreAboutCells = Cases[
		expr,
		Cell[TextData[ButtonBox[_String,__]],"MoreAbout",___],
		Infinity
	];

	(* Formatting note: 62a3c7f6: another attempt at fixing bugs 211500 and 211320 *)
	autoitalics = AutoItalicWords /. DocumentationBuild`Common`PubsEvaluateWithFE[Options[Cell, AutoItalicWords]];

	newMoreAboutCells = Map[ItalicizeMoreAboutKeyWords[#, autoitalics]&, moreAboutCells];

	moreAboutCellPos = Position[
		expr,
		Cell[TextData[ButtonBox[_String,__]], "MoreAbout", ___],
		Infinity
	];

	expr = ReplacePart[expr, Thread[Rule[moreAboutCellPos, newMoreAboutCells]]];

   (* V9 EDIT (NEW) BELOW *)
   (* V11.1 EDIT (NEWER!) BELOW: removed headers and footers because no longer needed *)
   (* Adding spacers to beginnings and ends of sections (but NOT See Also sections) *)
   (*Module[{sections="MoreAboutSection"|"TutorialsSection"|"GuideMoreAboutSection"|"RelatedLinksSection"|
         "GuideMoreAboutSection"|"GuideRelatedLinksSection"|"GuideTutorialCollectionSection"|"GuideTutorialsSection"|
         "TutorialMoreAboutSection"|"NotesSection"|"TutorialRelatedLinksSection"|
         "RelatedTutorialsSection"|"TutorialsTutorialCollectionSection"|"TutorialCollectionSection"|
         "ImportExportSection"|"NotebookInterfaceSection"|"OptionsSection"|"ElementsSection"|
         "IndicatorDescriptionSection"|"IndicatorCategorizationSection"|"IndicatorAbbreviationSection"|
         "IndicatorFormulaSection"|"GuideFunctionsSection"|
         "FeaturedExampleMoreAboutSection"|"FeaturedExampleSeeAlsoSection"|"FeaturedExampleRelatedFunctionsSection"|
         "FeaturedExampleRelatedLinksSection"|"DeviceSubsection"|"ServiceSubsection"|"EmbeddingFormatSection"|
         "InterpreterSection"|"FunctionEssaySection"|"AlternativeNamesSection"|"DesignJustificationSection"|
         "InternalLinksSection"|"DesignChoicesSection",
         numsects,cells, pos, endpos},
     cells = Cases[expr, Cell[CellGroupData[{Cell[___,
         sections, ___],___}, ___], ___], Infinity];
     numsects = 
       Length[cells];
     (* Every section gets a header *)
     expr = expr /. MapThread[Rule, {cells, Insert[#, Cell["","SectionHeaderSpacer"], {1,1,2}]&/@cells}];
     (* The last section does NOT get a footer *)
     cells = Cases[expr, Cell[CellGroupData[{Cell[___,
         sections, ___],___}, ___], ___], Infinity];
     (* special-cased for Message pages, that only have a notes section and examples -- 
        but need a footer in the notes *)
     Do[expr = expr /. cells[[i]]->Insert[cells[[i]],Cell["","SectionFooterSpacer"],{1,1,-1}],
       {i,If[MatchQ[type, "Message"], numsects, numsects-1]}];
     (* Hack for GuideReferenceSection cell since it hasn't grouped yet *)
     pos = Position[expr, Cell[___, "GuideReferenceSection", ___]];
     If[pos =!= {},
        pos = MapAt[#+1&, pos, {1,-1}];
        expr = Insert[expr, Cell["", "SectionHeaderSpacer"], pos];
        If[cells =!= {},
          endpos = {Append[Most[pos[[1]]], Length[expr[[Sequence@@Most[pos[[1]]]]]] + 1]};
          expr = Insert[expr, Cell["", "SectionFooterSpacer"], endpos];
        ];
     ];
     (* Hack for spacer after example cells since loading Manipulate examples breaks everything *)
     If[MatchQ[type, "Symbol"|"Indicator"|"Format"] && 
        MemberQ[expr, Cell[___, "PrimaryExamplesSection"|"IndicatorExampleSection", ___], Infinity],
       Which[
         (pos = Position[expr, Cell[___, "SeeAlsoSection", ___]]) =!= {},
         expr = Insert[expr, Cell["", "SectionFooterSpacer"], pos],
         (pos = Position[expr, Cell[___, "TutorialsSection", ___]]) =!= {},
         expr = Insert[expr, Cell["", "SectionFooterSpacer"], pos],
         (pos = Position[expr, Cell[___, "MoreAboutSection", ___]]) =!= {},
         expr = Insert[expr, Cell["", "SectionFooterSpacer"], pos],
         (pos = Position[expr, Cell[___, "RelatedLinksSection", ___]]) =!= {},
         expr = Insert[expr, Cell["", "SectionFooterSpacer"], pos]
       ]
     ]; 
     (* Guess what?  Another hack.  This time for guides without a Reference section (menu guides) *)
     If[MatchQ[type, "Guide"] && FreeQ[expr, Cell[___, "GuideReferenceSection", ___], Infinity],
       Which[
         (pos = Position[expr, Cell[___, "GuideTutorialCollectionSection", ___]]) =!= {},
         expr = Insert[expr, Cell["", "SectionFooterSpacer"], pos],
         (pos = Position[expr, Cell[___, "GuideMoreAboutSection", ___]]) =!= {},
         expr = Insert[expr, Cell["", "SectionFooterSpacer"], pos]
       ]
     ];        
   ];*)

   (* Removing Tutorial Collections cells *)
	(* Formatting note: b241c2b1:
		removed tutorial collections and updated history cell contents (year instead of version number) *)
	expr = DeleteCases[expr,
		Cell[CellGroupData[
			{
				Cell[
					___,
					"TutorialCollectionSection"|"GuideTutorialCollectionSection"|"TutorialsTutorialCollectionSection",
					___
				],
				___
			},
			___
		]],
		Infinity
	];

   expr = expr/. Cell[a___, b:(
   	"ElementsSection" | 
   	"ExportElementsSection" |
   	"ExampleSection"|
   	"ExampleSubsection"|
   	"ExampleSubsubsection"|
   	"ExtendedExamplesSection" |
   	(*"GuideRelatedLinksSection"| 
   	"GuideTutorialsSection"| 
   	"GuideMoreAboutSection"|*)
    "ImportExportSection" | 
    "NotebookInterfaceSection" | 
    (*"MoreAboutSection" |*)
    "MoreToExploreSection" | 
    "MoreInformationSection" | 
    "OptionsSection" | 
    "PrimaryExamplesSection" |  
    (*"RelatedLinksSection"|
    "RelatedTutorialsSection"|
    "SeeAlsoSection" | 
    "TutorialsSection" | 
    "TutorialCollectionSection" |
    "TutorialsTutorialCollectionSection" |
    "GuideTutorialCollectionSection" |
    "TutorialRelatedLinksSection" |*)
    "UsageDetailsSection" | "NotesSection" |
    (*"TutorialMoreAboutSection" |*) "GuideReferenceSection" |
    "GuideLearningResourcesSection" | (*"FeaturedExampleMoreAboutSection" | 
    "FeaturedExampleSeeAlsoSection" | "FeaturedExampleRelatedFunctionsSection" |
    "FeaturedExampleRelatedLinksSection" | *)"DeviceSubsection" | "ServiceSubsection" | "FunctionEssaySection" | 
    "InterpreterSection" | "FormatBackground" | "IndicatorExampleSection" | "WorkflowPlatform" | "WorkflowHeader" | 
    "WorkflowNotesSection" | "WorkflowNotesHeader" | "EmbeddingFormatSection" | "MethodSection" | "ExternalEvaluateSystemSection" |
    "GuideWorkflowGuidesSection" | "EntitySection" | "CompiledTypeSection" | "BlockchainSection" | "DatabaseConnectionSection" | "ClassifierSection" |
    "PredictorSection" | "BatchComputationProviderSection" | "QuestionInterfaceSection" | "ResourceObjectSection"
   	), c___]:>Cell[a, b, c, System`WholeCellGroupOpener->True, CacheGraphics->False];

	(* Add ColorType to TaggingRules *)
	colortype = Switch["EntityType" /. info,
		"Guide" | "Guide Overview" | "Alphabetical Listing", "Guide",
		"Tutorial" | "Overview" | "Monograph" | "Tech Note", "Tutorial", 
		"HowTo", "Tutorial",
		"Example", "Tutorial",
		"Root Guide", "Root",
		"Workflow" | "Workflow Guide", "Workflow",
		_ , "Symbol"
	];
	colortype = colortype <> "Color";

	(* Formatting note: a8f15f82:
		adding to the built nbs a TaggingRule for whether a symbol has an options table to
		inform the notes section text
	*)
	hasoptions = DocumentationBuild`Info`GetSourceInformation["paclet:"<>uri, lang, "HasOptions"];

    (* add spacer to pass-through sections (to keep text from crashing into openers) *)
    (* V11.1 Redesign edit: remove these spacers *)
    (*expr = ReplaceAll[expr,
      Cell[s_String, some___, 
        sty:("SeeAlsoSection"|"MoreAboutSection"|"TutorialsSection"|"TutorialCollectionSection"|
         "GuideMoreAboutSection"|"RelatedLinksSection"|
         "GuideRelatedLinksSection"|"GuideTutorialCollectionSection"|"GuideLearningResourcesSection"|"GuideTutorialsSection"|
         "TutorialMoreAboutSection"|"TutorialRelatedLinksSection"|
         "RelatedTutorialsSection"|"TutorialsTutorialCollectionSection"|"GuideReferenceSection"|
         "ImportExportSection"|"NotebookInterfaceSection"|"ElementsSection"|"OptionsSection"|
         "IndicatorExampleSection"|"IndicatorDescriptionSection"|"IndicatorCategorizationSection"|
         "IndicatorAbbreviationSection"|"IndicatorFormulaSection"|"PrimaryExamplesSection"|
         "GuideFunctionsSection"|
         "FeaturedExampleMoreAboutSection"|"FeaturedExampleSeeAlsoSection"|"FeaturedExampleRelatedFunctionsSection"|
         "FeaturedExampleRelatedLinksSection"|"DeviceSubsection"|"ServiceSubsection"|"FunctionEssaySection"|
         "EmbeddingFormatSection"|"InterpreterSection"|"RelatedInterpreterSection"|"AlternativeNamesSection"|
         "DesignJustificationSection"|"InternalLinksSection"|"DesignChoicesSection"|"FormatUsageSection"|"FormatBackground"),
      rest___] :>
      Cell[TextData[{Cell[BoxData[ToBoxes[Spacer[6]]]],s}], some, sty, rest]
    ];*)

    (* Handle rebranding terms (change them back to "Mathematica") if $Rebranded is False *)
    (* Also remove rebranding tagging information if $Rebranded is true *)
	If[!TrueQ[$Rebranded],
		expr = ReplaceRepeated[expr, StyleBox[_, "RebrandingTerm"] -> StyleBox["Mathematica", FontSlant->"Italic"]];
		expr = ReplaceRepeated[expr, StyleBox[_, "RebrandingTerm", "ReplacedText"->text_] :> text]
		,
    	expr = ReplaceRepeated[expr, StyleBox[c_, "RebrandingTerm", ___] :> c];
    ];

    (* Remove Under Development banner (replaced by UNFINISHED DRAFT banner), added during 
       ConvertNotebook[] *)
	expr = DeleteCases[
		expr,
		Cell["U N D E R   D E V E L O P M E N T", ___],
		Infinity
	];

    (* Moving Unfinished Draft banner on Interpreter pages because hack hack hack *)
    If[MemberQ[{"Interpreter"}, type] && MemberQ[expr, "DeviceAbstract", Infinity],
		expr = Module[{
			cell = First[Cases[expr, Cell[___, "DeviceAbstract", ___], Infinity]],
			pos,
			bannerpos
		},
			pos = Position[expr, Cell[___, "ObjectNameAlt", ___]];
			bannerpos = MapAt[(# + 1 &), pos, {1, -1}];
			Insert[DeleteCases[expr, cell, Infinity], cell, bannerpos]
		]
	];

    (* Fix bug 246625; apparently ButtonBox -> TemplateBox conversion was messing up summary generation *) 
    summary = DocumentationBuild`Info`GetNotebookSummary[
		expr,
		("EntityType" /. info),
		("Title" /. info)
	];

    (* Convert ButtonBoxes to TemplateBoxes *)
    expr = If[type =!= "Root Guide", DocumentationBuild`Utils`ConvertButtonBoxesToTemplateBoxes[expr], expr];
    expr = If[type =!= "Root Guide", DocumentationBuild`Utils`ConfirmButtonBoxesToTemplateBoxes[expr], expr];
    expr = ReplaceAll[expr, c:Cell[__, "InlineFormula"|"InlineCode", ___] :> Append[c, FontFamily->"Source Sans Pro"]];

    (* Replace Function Essay bit with something more Dynamic *)
    (* Since font changed to Verdana, let's turn off the function name resizing *)
	(* Formatting note: 80ac25d9:
		fix bug 294563 (ref/WolframLanguageData was getting a function essay from Sin
		because of one of its examples)
	*)
    fnessay = (*ReplaceAll[*)
		Flatten[Cases[
			expr,
			Cell[CellGroupData[
				{
					Cell[__, "FunctionEssaySection", ___],
					___,
					cell : (Cell[__, "FunctionEssay", ___] ..),
					Cell[c__ /; FreeQ[{c}, "FunctionEssay"]] ...
				},
				___
			]] :> {cell},
			Infinity
		]]
		(*,
		Cell[some___,"InlineFormula",rest___]
			:> Cell[some,"InlineFormula",rest, FontSize->12]
	]*);

	(* Formatting note: fd29fcae:
		closed Background section by default, removed Show More/Less button, removed
		FcnEssay-related tagging rule
	*)
	expr = If[fnessay =!= {},
	(* comments below reflect openness of section, per SW (4/21/14) *)
	(*With[{
		showmoreimg = ToBoxes@Image[Import[DocsAsset["InternalOnlineImage", "FunctionEssay-showmore-arrow.png"]],
			ImageSize->Magnification[1], BaselinePosition->Baseline],
		showlessimg = ToBoxes@Image[Import[FileNameJoin[
		{"InternalOnlineImage", "FunctionEssay-showless-arrow.png"}]],
			ImageSize->Magnification[1], BaselinePosition->Baseline]},*)

		ReplaceAll[
			expr,
			Cell[
				CellGroupData[{Cell[some___, "FunctionEssaySection", other___], ___}, ___]
			] :> Cell[CellGroupData[
				{
					Cell[some, "FunctionEssaySection", other], 
					With[{
						full = Cell[BoxData[fnessay],"FunctionEssay"],
						first = First[fnessay]
					},
						Cell[
							BoxData[DynamicBox[
								If[(*CurrentValue[EvaluationNotebook[],{TaggingRules,"FunctionEssayOpen"}]*)True,
									full,
									first
								]
							]],
							"FunctionEssay",
							Selectable->True
						]
					]
					(*Cell[BoxData[DynamicBox[If[
					CurrentValue[EvaluationNotebook[],{TaggingRules,"FunctionEssayOpen"}],
					TagBox[ButtonBox[StyleBox[RowBox[{"Show Less ", showlessimg}],"FunctionEssayShowMore"],BaseStyle->"Link",Evaluator->Automatic,Method->"Queued",
						ButtonFunction:>(CurrentValue[EvaluationNotebook[],{TaggingRules,"FunctionEssayOpen"}]=False)],MouseAppearanceTag["LinkHand"]],
					TagBox[ButtonBox[StyleBox[RowBox[{"Show More ", showmoreimg}],"FunctionEssayShowMore"],BaseStyle->"Link",Evaluator->Automatic,Method->"Queued",
						ButtonFunction:>(CurrentValue[EvaluationNotebook[],{TaggingRules,"FunctionEssayOpen"}]=True)],MouseAppearanceTag["LinkHand"]]]]],"FunctionEssayShowMore"],*)
				},
				Dynamic[CurrentValue[EvaluationNotebook[], {TaggingRules, "Openers", "FunctionEssaySection"}, Closed]]
			]]
		]
	(*]*),
		expr
	];

    (* Frame KeyEvents *)
	expr = ReplaceAll[
		expr,
		StyleBox[s_String, some___, "KeyEvent", rest___] :> Cell[s, "KeyEvent", some, rest]
	];
    (* V11.1: now update to new key combo style *)
    MakeKeyComboBox[c_Cell, forcedstyle_:Automatic] := Module[{
		combos, count, style, newcell = c, INERT, keys, replacements, repl2
	},
		replacements = {
			"control" | "ctl" | "Control" | "Ctl" | "Ctrl" -> "ctrl",
			"command" | "Command" | "Cmd" -> "cmd",
			"option" | "Opt" | "Option" -> "opt",
			"escape" | "Escape" | "Esc" -> "esc",
			"delete" | "Delete" | "Del" -> "del",
			"Enter" -> "enter",
			"Return" -> "return",
			"=" -> "\[LongEqual]"
		};

        repl2 = {StartOfString ~~ "f" ~~ (n:(DigitCharacter..)) :> "F"<>n};
        style = If[!MatchQ[forcedstyle, Automatic],
			forcedstyle,
			First[Cases[c, _String]]
		];

        (* step 0: if a simple cell is passed in, return its contents *)
        If[MatchQ[c, Cell[_String, "KeyEvent"]],
			Return[Cell[
				StringReplace[StringReplace[First[c], replacements], repl2],
				style
			]]
		];

        (* First, handle odd case Dynamics *)
        newcell = Module[{counter = 0},
			ReplaceAll[
				c,
				db:(
					Cell[BoxData[DynamicBox[_[_, Cell[__], Cell[__]], ___]],___] |
					Cell[BoxData[FormBox[DynamicBox[_[_, Cell[__], Cell[__]], ___],___]],___]
				) /; FreeQ[db, TemplateBox[__]] :> (
					counter++;
					StyleBox[
						Cell[BoxData[
							DynamicBox @@@ {
								Part[
									Cases[
										c,
										HoldPattern[f_[$OperatingSystem === os_, true_Cell, false_Cell]]
											:> With[{
												true2 = Evaluate[MakeKeyComboBox[true,style]],
												false2 = Evaluate[MakeKeyComboBox[false,style]]
											},
												HoldComplete[f[$OperatingSystem === os,true2,false2]]
											],
										Infinity
									],
									counter
								]
							}
						]],
						"KeyEvent"
					]
				)
			]
		];

		newcell = ReplaceAll[
			newcell,
			StyleBox[boxes_, "KeyEvent"] /; MemberQ[boxes, TemplateBox[__, "Key0"|"Key1"|"Key2"|"Key3", ___], Infinity]
				:> boxes
		];

		newcell = ReplaceAll[
			newcell,
			f_[
				some___,
				Longest[co:(
					((Cell|StyleBox)[__,"KeyEvent", ___] | (Cell|StyleBox)[__,"KeyEventDelimiter", ___] | "+" | "-")..
				)],
				rest___
			] :> f[
				some,
				Replace[INERT[{co}], s:("+"|"-"):>StyleBox[s, "KeyEventDelimiter"], {2}],
				rest
			]
		];

		newcell = ReplaceAll[
			newcell,
			INERT[co_] :> (
				count = Length[Cases[co, _[__, "KeyEventDelimiter", ___]]];

				Which[
					FreeQ[co, "KeyEvent"],
						Sequence @@ Flatten[ReplaceAll[co, StyleBox[s_, "KeyEventDelimiter"]:>s]],
					count > 3,
						Sequence@@co,
					True,
						keys = Cases[
							co,
							(Cell|StyleBox)[
								k_,
								___, "KeyEvent", ___
							] :> Which[
								StringQ[k],
									StringReplace[StringReplace[ToLowerCase[k], replacements], repl2],
								MatchQ[k, DynamicBox[__]],
									ToExpression[StringReplace[StringReplace[ToString[k, InputForm], Most@replacements],repl2]],
								True,
									k
							]
						];

						Cell[BoxData[TemplateBox[
							keys,
							"Key"<>ToString[count],
							BaseStyle->{style, FontWeight->"Plain", FontFamily->"Source Sans Pro"}
						]]]
				]
			)
		];

        ReplaceAll[
			newcell,
			TemplateBox[l_List, rest__] :> TemplateBox[
				Replace[l, td:(TextData|BoxData)[___] :> Cell[td], {1}],
				rest
			]
		]
	];

    (* Transform some weird cases first *)
    expr = ReplaceRepeated[
		expr,
		Cell[
			BoxData[FormBox[
				StyleBox[
					DynamicBox[
						ToBoxes[If[$OperatingSystem === os_, true_String, false_String], _],
						r___
					],
					"KeyEvent"
				],
				___
			]],
			"KeyEvent",
			___
		] :> Cell[BoxData[DynamicBox[
			If[$OperatingSystem === os,
				Cell[true, "KeyEvent"],
				Cell[false, "KeyEvent"]
			],
			r
		]]]
	];

	If[!MatchQ[type, "Tutorial"|"Monograph"|"Tech Note"],
		TimeConstrained[
			expr = FixedPoint[
				ReplaceAll[
					#,
					c:(Cell[con_, some___, style_String, rest___] /; MemberQ[{con},(Cell|StyleBox)[__, "KeyEvent",___], Infinity])
						:> MakeKeyComboBox[c]
				]&,
				expr,
				50
			],
			30,
			Print["KeyEvents timed out!"]
		]
	];

    (* Clean up keys now *)
	(* Formatting note: 1365d151: fixed bug 330726 *)
    If[!FreeQ[expr, TemplateBox[_, ___, s_String /; StringMatchQ[s, "Key*"], ___]],
		expr = ReplaceAll[
			expr,
			TemplateBox[l_List,some___,style:"Key0"|"Key1"|"Key2"|"Key3",rest___] /; ToExpression[Last[Characters[style]]]+1 =!= Length[l]
				:> TemplateBox[l, some, "Key" <> ToString[Length[l]-1], rest]
		]
	];

	(* Check SeeAlsos and guide inline listing links for bad (future, nonexistent) stuff and strip it *)
	expr = Which[
		MatchQ[type, "Symbol" | "Interpreter"],
			DocumentationBuild`Utils`CheckSeeAlsos[expr, lang],
		MatchQ[type, "Guide"],
			(* TODO *)DocumentationBuild`Utils`CheckGuideInlineListings[expr, lang],
		True,
			expr
	];

    (*closedGroups = Which[MemberQ[{"Format", "Interpreter", "Service Connection", "Device Connection", "Embedding Format", "Method", "Indicator"}, type], 
  	"ExampleSection"|"ExampleSubsection"|"ExampleSubsubsection"|"ImportExportSection"|"NotebookInterfaceSection"|"OptionsSection"|
  	"InterpreterSection"|"EmbeddingFormatSection"|"ServiceSubsection"|"DeviceSubsection"|"MethodSection"|"FunctionEssaySection"|"FormatBackground"|"ElementsSection"|
  	"IndicatorExampleSection"(*|"TutorialsSection"*), 
    True, "ExampleSection"|"ExampleSubsection"|"ExampleSubsubsection"|"FunctionEssaySection"];
  
    CloseGroups[nbexpr_Notebook, style_String] := 
      nbexpr /. {
      Cell[CellGroupData[{
         Cell[con_, sty_String /; StringMatchQ[sty, style], o___],
        (*Cell[con_, sty_String /; StringMatchQ[sty, "ExampleSection"|"ExampleSubsection"], o___],*)
          cs___
        }, ___], g___] 
      :> Cell[CellGroupData[{
        Cell[con, sty, o],
          cs
        }, If[(Length[Cases[con, Localized["BasicExamplesSectionText", lang], Infinity]] > 0), Open, Closed]], g],
      Cell[CellGroupData[{
        Cell[con_, "NotesSection", o___],
          cs___
        }, ___], g___]
      :> Cell[CellGroupData[{
        Cell[con, "NotesSection", o],
          cs
        }, (* Details section to be open by default on Message pages: brianv--05/12/16*) 
        If[MatchQ[type, "Message"], Open, Closed]], 
      g]
    };
  
    expr = Fold[CloseGroups, expr, closedGroups];
    
   (* because I just re-closed the groups after setting the dynamic stuff, the openers are broken.  Fixing that here *) 
    Module[{onetimesections = {"FunctionEssaySection","FormatBackground","ImportExportSection","NotebookInterfaceSection",
     "ElementsSection","OptionsSection","IndicatorExampleSection"},groupopenersections, needopenersQ = False},
   
    groupopenersections = Join[{"NotesSection","PrimaryExamplesSection"}, onetimesections, {"ExampleSection", "ExampleSubsection", "ExampleSubsubsection", "InterpreterSection",
     "DeviceSubsection","ServiceSubsection", "EmbeddingFormatSection", "MethodSection"}];
    
    expr = DynamicGroupOpeners[expr, groupopenersections, onetimesections, "Openers"->needopenersQ]];*)

    BuildLog["   Return Notebook expression."];
	Global`AntLog["Starting time16"];
	time16 = ToString @ AbsoluteTiming[
		expr = Notebook[
			Flatten[{
				First[expr],
				ReformatHistoryCell[historyCell, lang],
				citationCell,
				Cell[" ", "FooterCell"]
			}],
			Sequence @@ Flatten @ {
				ScreenStyleEnvironment -> "Working",
				WindowSize -> {900, 830},
				Saveable -> False,
				(* FIXME: we may need to do something different for packages *)
				WindowTitle -> If[type === "Root Guide",
					Localized["RootGuideWindowTitle", lang],
					(StringReplace[
						("Title" /. info),
						{
							" (\[Conditioned])" -> "",
							" (\[Distributed])" -> "",
							" (<->,\[UndirectedEdge])" -> "",
							" (\[DirectedEdge])" -> "",
							"Wolfram Mathematica" -> "Mathematica",
							StartOfString ~~ "Automatic" ~~ EndOfString -> " Automatic",
							StartOfString ~~ "None" ~~ EndOfString -> " None"
						}
					] (*<> " - Wolfram Mathematica"*))
				],
				WindowMargins -> {{0, Automatic}, {Automatic, 0}},
				StyleDefinitions -> stylesheet,
				If[type === "Root Guide",
					WindowElements -> {
						"StatusArea", "MemoryMonitor", "MagnificationPopUp",
						"VerticalScrollBar", "HorizontalScrollBar", "MenuBar"
					},
					{}
				],
				If[TrueQ[CustomGuideQ], DockedCells->{}, {}],
				(* explicit CellContext setting - present in some/most M- Tutorials *)
				If[nbContext =!= CellContext,
					CellContext->nbContext,
					{}
				],
				If[type === "Guide", CacheGraphics->False, {}],
				TaggingRules -> Flatten @ {
					"ModificationHighlight" -> False,
					(*"ColorType"->If[type==="Root Guide", "", colortype],*)
					"ColorType"->"",
					(* dropping link trails in v11.1 *)
					"LinkTrails" -> "",
					If[MatchQ[hasoptions, {} | None | "None"], {}, "HasOptions" -> hasoptions],
					"ExampleCounter" -> 1,
					"Openers" -> GetOpenSections[type],
					"NewStyles" -> True,
					"CitationPopupData" -> citationInfo,
					"ShowCitation" -> False,
					(*"FunctionEssayOpen" -> False,*)
					"RootCaptions" -> rootCaptionList,
					If[overviewLinks === {}, {}, "Overview" -> overviewLinks],
					"Metadata" -> ConvertValuesForSearch[
						{
							"built" -> ToString@Date[],
							"history" -> {"New", "Modified", "Obsolete", "Excised"} /. ("HistoryData" /. info),
							"context" -> ("Context" /. info),
							"keywords" -> ("Keywords" /. info),
							"specialkeywords" -> specialKeywords,
							"tutorialcollectionlinks" ->("TutorialCollectionLinks" /. info),
							"index" -> "IndexQ" /. info,
							"label" -> classLabel,
							"language" -> GetLanguageExtension[lang],
							"paclet" -> ("PacletName" /. info),
							"status" -> ("Flag" /. info /. None -> "None"),
							(* This was replaced to fix bug 235409, but was added to fix bug 226210, so
							check and make sure that both bugs are fixed later.
							"summary" -> If[!StringFreeQ[("Summary" /. info), "XXXX"], "", ("Summary" /. info)],*)
							"summary" -> summary,
							"synonyms" -> ("Synonyms" /. info),
							"tabletags" -> tabletaggingrules,
							(* FIXME: this needs to be special-cased for root guide *)
							"title" -> ShortFormTitle[("Title" /. info)],
							"titlemodifier"->("TitleModifier" /. info),
							"windowtitle" ->("Title" /. info),
							"type" -> ("EntityType" /. info),
							"uri" -> uri
						},
						lang
					]
				}
			}
		];
    ][[1]];

    If[debug==="True",Global`AntLog["Assemble Notebook: "<>time16]];

	expr = If[type === "Root Guide",
		Join[
			DeleteCases[
				expr,
				((CellGrouping|Selectable|ShowGroupOpener|Background) ~ (Rule|RuleDelayed) ~ _)
			],
			Notebook[
				(* Formatting note: 7d123016:
					removing CellGrouping->Manual from root guide -- was causing Learning
					Resources sections to be open when navigating from root
				*)
				(*CellGrouping->Manual,*)
				ShowGroupOpener -> False,
				Selectable -> False,
				Background -> GrayLevel[0.266667]
			]
		]
		,
		expr
	];

	Global`AntLog["MakeNotebook: Starting time17"];
	time17 = ToString @ AbsoluteTiming[Module[{
		nbObj
	},
    Global`AntLog["  CacheStylesheet"];
		CacheStylesheet[expr];

		(*------------------------------------------------*)
		(* Create parent directories for the notebook(s). *)
		(*------------------------------------------------*)

    Global`AntLog["  create parent dirs for notebooks"];
		Scan[
			Function[path,
				If[!DirectoryQ[DirectoryName[path]],
					CreateDirectory[DirectoryName[path]];
				];
			],
			Select[{outfilelocation, outfilelocation2}, # =!= None &]
		];

		(*---------------------------------------*)
		(* Convert graphics to bitmaps and save. *)
		(*---------------------------------------*)

    Global`AntLog["  NotebookPut"];
		nbObj = UsingFrontEnd @ NotebookPut[expr, Visible -> False];

		If[Head[nbObj] =!= NotebookObject,
			DocsError["Error performing NotebookPut of '``': ``", uri, nbObj];
			If[nbObj === $Failed,
				Global`AntLog["Failed expr: ``", expr]
			];
		];

		If[StringQ[outfilelocation2],
			DocsAssert[StringQ[outfilelocation]];

			(* Formatting note: 6125987a:
				attempt at bug 347957, making sure WorkflowFinisher runs on hi-res output
				workflow nbs
			*)
			If[MatchQ[type, "Workflow"],
				nbObj = PubsEvaluateWithFE @ WorkflowFinisher[nbObj, "Web" -> True, "Language" -> lang];
			];

			(* Save a hi-res (not bitmapped) notebook. *)
      Global`AntLog["  NotebookSaveVisible 1: save a hi-res (not bitmapped) notebook"];
			UsingFrontEnd @ NotebookSaveVisible[nbObj, outfilelocation2];
		];

		UsingFrontEnd @ If[
			And[
				rasterizeoutput === True,
				(* Formatting note: f1c6ba39:
					[...] also bypassing ConvertGraphicsToBitmaps for those notebooks that have no
					Output cells
				*)
				(* If there are no output cells, let's skip the step *)
				Cells[
					nbObj,
					CellStyle -> "AnnotatedOutput" | "AnnotatedInput"
						| "AnnotatedExpression" | "Output"
				] =!= {},
				(* Formatting note: 26735109:
					Skip rasterization of ref/RenderingOptions

					This page, like a few others, has problems when being sent through
					ConvertGraphicsToBitmaps - it simply fails to build.  We have a safety
					valve for this situation that allows pages to skip bitmapping.  When
					there's a mysterious failure to build during notebook conversion, this
					is the first thing to try unless obvious clues point elsewhere.

					Utils.m
					- add ref/RenderingOptions to the small list of pages that are allowed
					to skip Output rasterization.
				*)
				!MemberQ[
					{
						(*"DiscretePlot3D.nb", "Circumsphere.nb", "Insphere.nb", "SphericalShell.nb", "DensityPlot3D.nb"*)
						"GraphData.nb",
						"UsingOpenCascadeLink.nb",
						"RenderingOptions.nb",
						"OutputForm.nb"
					},
					FileNameTake[outfilelocation]
				]
			],
      Global`AntLog["  ConvertGraphicsToBitmaps"];
			DocumentationBuild`Private`rastertime = ToString @ AbsoluteTiming[
				ConvertGraphicsToBitmaps[nbObj, type, uri];
			][[1]];
		];

    Global`AntLog["  begin makenboutput"];
		makenboutput = PubsEvaluateWithFE @ Replace[{outfilelocation, outfilelocation2}, {
			{None, None} :> (
        Global`AntLog["  makenboutput 1: NotebookGet"];
				expr = NotebookGet[nbObj];
				(* Remove the expected `Visible -> False` option from the notebook. *)
        Global`AntLog["  remove Visible -> False"];
				expr = DeleteCases[expr, Visible ~ (Rule|RuleDelayed) ~ _];
				expr
			),
			{_?StringQ, _?StringQ | None} :> (
				(* Hack in extra step for workflows *)
				If[MatchQ[type, "Workflow"],
          Global`AntLog["  makenboutput 2: WorkflowFinisher"];
					nbObj = PubsEvaluateWithFE @ WorkflowFinisher[nbObj, "Language" -> lang];
				];

				(* Formatting note: 6125987a:
					attempt at bug 347957, making sure WorkflowFinisher runs on hi-res output workflow nbs *)
				(* re-closing any open groups *)
        Global`AntLog["  makenboutput 3: close any open groups"];
				CurrentValue[nbObj, {TaggingRules, "Openers"}] = GetOpenSections[type];

        Global`AntLog["  makenboutput 4: NotebookSaveVisible"];
				NotebookSaveVisible[nbObj, outfilelocation];

				outfilelocation
			),
			other_ :> DocsError["Invalid combination of outfile option values: ``", InputForm[other]]
		}];

    Global`AntLog["  makenboutput 5: NotebookClose"];
		UsingFrontEnd @ NotebookClose[nbObj];

		(* If this is the form of MakeNotebook which writes to a file, include the search
		   metadata in the result. *)
		If[StringQ[outfilelocation],
      Global`AntLog["  makenboutput 6: GetSearchMetaDataList"];
			makenboutput = {
				makenboutput,
				DocumentationBuild`Utils`GetSearchMetaDataList[expr]
			};
		];
	]][[1]];

    If[debug==="True",Global`AntLog["Use Frontend to build final notebook: "<>time17]];

    makenboutput
];




AddAttractorIcons[cellgroup_, lang_:"English"]:=
If[cellgroup =!= {}, Module[{imgname, type,cells=If[ListQ[cellgroup],cellgroup[[1,1,1]],cellgroup[[1,1]]],
  styletofile, subrefs={}},
type=Replace[First[cells],Cell[__,s_String/;StringMatchQ[s,"*Section"],___]:>s];
type = If[StringQ[type], type, "None"];
styletofile = {"*SeeAlsoSection"->"RelatedFunction.png", "*MoreAboutSection"->"RelatedGuide.png", "*TutorialsSection"->"RelatedTutorial.png", "*TechNotesSection"->"RelatedTechNote.png",
  "*MonographsSection"->"RelatedMonograph.png", "*EntitySection"->"RelatedEntity.png", "*FormatSection"->"RelatedFormat.png", "*InterpreterSection"->"RelatedInterpreter.png", "*WorkflowsSection"->"RelatedWorkflow.png",
  "*WorkflowGuidesSection"->"RelatedWorkflowGuide.png"};
imgname = Which[Evaluate[Sequence@@Flatten[{StringMatchQ[type, #[[1]]], #[[2]]}&/@styletofile]]];
  
If[Length[cells] >= 2 (*If False, just remove entire group; also removing any of these footers from the "remove empty cell groups" section of code*),
 Switch[type,
  "SeeAlsoSection",
  If[Or@@(!StringFreeQ[StringReplace[#,"paclet:"~~___~~"ref/"->""], "/"]& /@ Cases[cells, (_[ButtonData,s_]) :> s, Infinity]),
    {cells, subrefs} = SplitSeeAlsoSubRefs[cells, lang]];
  Cell[BoxData[GridBox[{{DynamicBox[FEPrivate`ImportImage[
      FrontEnd`FileName[{"Documentation", "FooterIcons"}, "RelatedFunction.png"]]],
      GridBox[{
       {StyleBox[Transmogrify`ConvertToString[First[cells[[1]]]], type]},
       If[Length[cells] < 2 || MatchQ[cells[[2]], Cell[TextData[{}], ___]], Nothing, {cells[[2]]}],
       If[subrefs=!={},
         Sequence@@subrefs,
         Nothing]}]
     }}]], type],
  "None",
  cells,
  _,
  If[StringMatchQ[type, "*RelatedLinksSection"],
    cells = TransformRelatedLinksCells[cells];
    Cell[BoxData[GridBox[{{TemplateBox[{29},"Spacer1"],
      (* multicolumn, not working yet
      GridBox[{
        {StyleBox[Transmogrify`ConvertToString[First[cells[[1]]]], type],"\[SpanFromLeft]"},
        Sequence@@(Partition[Module[{l=Rest[cells]},If[OddQ[Length[l]],Append[l,""],l]],2])}]*)
      (* single column, just to get it working for now *)
      GridBox[{
       {RowBox[{TemplateBox[{17},"Spacer1"],StyleBox[Transmogrify`ConvertToString[First[cells[[1]]]], type]}]},
       Sequence@@(List/@Rest[cells])}]
      }}]], type],
      
    With[{imgname=imgname},Cell[BoxData[GridBox[{{DynamicBox[FEPrivate`ImportImage[
      FrontEnd`FileName[{"Documentation", "FooterIcons"}, imgname]]],
      GridBox[{
       {StyleBox[Transmogrify`ConvertToString[First[cells[[1]]]], type]},
       Sequence@@({RowBox[{"\[FilledVerySmallSquare]", #}]}&/@Rest[cells])}]
     }}]], type]]]], {}]], {}]


SplitSeeAlsoSubRefs[cells_List, lang_:"English"] :=
  Module[{types, sections, subrefs, refs = cells, URIToTitleRules, separatorpattern = (" \[EmptyVerySmallSquare] " | " \[FilledVerySmallSquare] " | "\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]" | "\[MediumSpace]\[EmptyVerySmallSquare]\[MediumSpace]"), spacepattern = (" " | "\[NonBreakingSpace]")},
    URIToTitleRules = 
      {
        (* these are ordered per SW *)
        (s_String/;StringMatchQ[s,"paclet:*"]) | "ref"->Localized["SeeAlsoRelatedFunctionsText", lang],
        "entity"->Localized["SeeAlsoRelatedEntityTypesText", lang],
        "interpreter"->Localized["SeeAlsoRelatedInterpreterTypesText", lang],
        "identifier"->Localized["SeeAlsoRelatedIdentificationTypesText", lang],  (* doesn't exist yet *)
        "indicator"->Localized["SeeAlsoRelatedIndicatorTypesText", lang],
        "blockchain"->Localized["SeeAlsoRelatedBlockchainsText", lang],
        "classifier"->Localized["SeeAlsoRelatedClassifiersText", lang],
        "predictor"->Localized["SeeAlsoRelatedPredictorsText", lang],
        "textcontent"->Localized["SeeAlsoRelatedTextContentsText", lang],
        "netencoder"->Localized["SeeAlsoRelatedNetEncodersText", lang],
        "netdecoder"->Localized["SeeAlsoRelatedNetDecodersText", lang],
        "method"->Localized["SeeAlsoRelatedMachineLearningMethodsText", lang],
        "compiledtype"->Localized["SeeAlsoRelatedCompiledTypesText", lang],
        "datastructure"->Localized["SeeAlsoRelatedDataStructuresText", lang],
        "format"->Localized["SeeAlsoRelatedFormatsText", lang],
        "resource"->Localized["SeeAlsoRelatedResourceTypesText", lang],
        "service"->Localized["SeeAlsoRelatedServiceConnectionsText", lang],
        "device"->Localized["SeeAlsoRelatedDeviceTypesText", lang],
        "databaseconnection"->Localized["SeeAlsoRelatedDatabaseConnectionsText", lang],
        "embeddingformat"->Localized["SeeAlsoRelatedExternalEmbeddingFormatsText", lang],
        "externalevaluationsystem"->Localized["SeeAlsoRelatedExternalEvaluationSystemsText", lang],
        "excelfunction"->Localized["SeeAlsoRelatedExcelFunctionsText", lang],
        "c"->Localized["SeeAlsoRelatedCFunctionsText", lang],
        "program"->Localized["SeeAlsoRelatedProgramsText", lang],
        "file"->Localized["SeeAlsoRelatedSystemFilesText", lang],
        "menuitem"->Localized["SeeAlsoRelatedMenuItemsText", lang],
        "frontendobject"->Localized["SeeAlsoRelatedFrontEndTokensText", lang],
        "character"->Localized["SeeAlsoRelatedCharactersText", lang]
      };
    subrefs = Cases[refs, Cell[(TextData|BoxData)[ButtonBox[__, ButtonData->uri_ /; StringMatchQ[uri, "*ref/*/*"], ___]], ___], Infinity];
    refs = DeleteCases[refs, Alternatives @@ subrefs, Infinity];
    (* handling dangling separators *)
    refs = ReplaceAll[refs, StyleBox[c_, "InlineSeparator"] :> c];
    refs = ReplaceRepeated[refs, {some___, separatorpattern, spacepattern..., separatorpattern, rest___} :> {some, " \[EmptyVerySmallSquare] ", rest}];
    refs = ReplaceAll[refs, {spacepattern..., separatorpattern, rest___} :> {rest}];
    refs = ReplaceAll[refs, {some___, separatorpattern, spacepattern...} :> {some}];
    types = Cases[subrefs, _[ButtonData, s_String] :> StringReplace[s, "paclet:"~~___~~"ref/"->""], Infinity];
    types = DeleteDuplicates[StringReplace[#, "/"~~__~~EndOfString -> ""]& /@ types];
    (* sort by approved listing above *)
    types = SortBy[types, Position[URIToTitleRules, #]&];
    sections = types /. URIToTitleRules;
    subrefs = Cases[subrefs, Cell[(TextData|BoxData)[ButtonBox[__,ButtonData-> s_ /; StringMatchQ[s,"*ref/"<>#<>"/*"], ___]],___],Infinity]& /@ types;
    subrefs = Table[Join[{Cell[sections[[i]]<>":  ","SeeAlso"<>Capitalize[types[[i]]],
      FontFamily->"Source Sans Pro", FontWeight->"Plain", FontSlant->If[lang==="English", "Italic", "Plain"], FontSize->14]},subrefs[[i]]],{i,Length[subrefs]}];
    subrefs = {Cell[TextData[{#[[1]],Flatten[Riffle[
      Rest[
       ReplaceAll[#,s_String/;StringMatchQ[s,("\""~~__~~"\"")]:>
         StyleBox[StringReplace[s,"\""->""],"SeeAlsoRelated","SeeAlsoSubLink"]]
       ],{{"\[NonBreakingSpace]", StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]","InlineSeparator"], " "}}, 2]]}], "SeeAlso"]}&/@subrefs;
    {refs, subrefs}
  ];


TransformRelatedLinksCells[cells_List]:=
  Module[{rest = Rest[cells], first = First[cells], iconrules,spacer=TemplateBox[{4},"Spacer1"],
    emptyspacer = (*TemplateBox[{30},"Spacer1"]*)TemplateBox[{16},"Spacer1"]},
    iconrules = <|"Community" -> "RelatedLinks-Community.png", "DataRepo" -> "RelatedLinks-DataRepository.png",
      "Demos" -> "RelatedLinks-Demonstrations.png", "EIWL" -> "RelatedLinks-EIWL.png",
      "FastIntro" -> "RelatedLinks-FastIntro.png", "FuncRepo" -> "RelatedLinks-FunctionRepository.png",
      "Functions" -> "RelatedLinks-Functions.png", "MathWorld" -> "RelatedLinks-MathWorld.png",
      "NNetRepo" -> "RelatedLinks-NeuralNetRepository.png", "NKS" -> "RelatedLinks-NKS.png",
      "TMJ" -> "RelatedLinks-TMJ.png", "WA" -> "RelatedLinks-WolframAlpha.png",
      "WU" -> "RelatedLinks-WolframU.png"|>;
    rest = Replace[rest, Cell[TextData[td_], some___, style_String /; StringMatchQ[style, "*RelatedLinks"], rest___] :>
     Cell[TextData[ReplaceAll[td, (bb:Cases[td,ButtonBox[_String,___],{0,Infinity},1][[1]]) :>
      Module[{s = bb[[1]], other=Rest[bb]},
      Which[StringMatchQ[s, "NKS|Online"] || !FreeQ[{other}, str_String/;StringMatchQ[str, "*wolframscience.com*"]],
        With[{file=iconrules["NKS"]},
          ButtonBox[Cell[TextData[{Cell[BoxData[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation","FooterIcons"},file]]]]],spacer,s}]],Sequence@@other]],
          
        StringMatchQ[s, "*Fast Introduction*"] || !FreeQ[{other}, str_String/;StringMatchQ[str, "*/language/fast-introduction*"]],
        With[{file=iconrules["FastIntro"]},
          ButtonBox[Cell[TextData[{Cell[BoxData[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation","FooterIcons"},file]]]]],spacer,s}]],Sequence@@other]],
          
        StringMatchQ[s, "*Elementary Introduction*"] || !FreeQ[{other}, str_String/;StringMatchQ[str, "*/language/elementary-introduction*"]],
        With[{file=iconrules["EIWL"]},
          ButtonBox[Cell[TextData[{Cell[BoxData[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation","FooterIcons"},file]]]]],spacer,s}]],Sequence@@other]],
          
        StringMatchQ[s, "*Demonstrations with*"|"*Demonstrations related*"] || !FreeQ[{other}, str_String/;StringMatchQ[str, "*demonstrations.wolfram.com*"]],
        With[{file=iconrules["Demos"]},
          ButtonBox[Cell[TextData[{Cell[BoxData[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation","FooterIcons"},file]]]]],spacer,s}]],Sequence@@other]],
          
        StringMatchQ[s, "*Wolfram Functions*"] || !FreeQ[{other}, str_String/;StringMatchQ[str, "*functions.wolfram.com*"]],
        With[{file=iconrules["Functions"]},
          ButtonBox[Cell[TextData[{Cell[BoxData[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation","FooterIcons"},file]]]]],spacer,s}]],Sequence@@other]],
          
        StringMatchQ[s, "*Data Repository*"] || !FreeQ[{other}, str_String/;StringMatchQ[str, "*datarepository.wolframcloud.com*"]],
        With[{file=iconrules["DataRepo"]},
          ButtonBox[Cell[TextData[{Cell[BoxData[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation","FooterIcons"},file]]]]],spacer,s}]],Sequence@@other]],
          
        StringMatchQ[s, "*Function Repository*"] || !FreeQ[{other}, str_String/;StringMatchQ[str, "*resources.wolframcloud.com/FunctionRepository*"]],
        With[{file=iconrules["FuncRepo"]},
          ButtonBox[Cell[TextData[{Cell[BoxData[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation","FooterIcons"},file]]]]],spacer,s}]],Sequence@@other]],
          
        StringMatchQ[s, "*MathWorld*"] || !FreeQ[{other}, str_String/;StringMatchQ[str, "*mathworld.wolfram.com*"]],
        With[{file=iconrules["MathWorld"]},
          ButtonBox[Cell[TextData[{Cell[BoxData[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation","FooterIcons"},file]]]]],spacer,s}]],Sequence@@other]],
          
        StringMatchQ[s, "*Neural Network Repository*" | "*Neural Net Repository*"],
        With[{file=iconrules["NNetRepo"]},
          ButtonBox[Cell[TextData[{Cell[BoxData[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation","FooterIcons"},file]]]]],spacer,s}]],Sequence@@other]],
          
        StringMatchQ[s, "*Wolfram|Alpha*"] || !FreeQ[{other}, str_String/;StringMatchQ[str, "*wolframalpha.com*"]],
        With[{file=iconrules["WA"]},
          ButtonBox[Cell[TextData[{Cell[BoxData[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation","FooterIcons"},file]]]]],spacer,s}]],Sequence@@other]],
          
        StringMatchQ[s, "*Wolfram U*"|"*WolframU*"],
        With[{file=iconrules["WU"]},
          ButtonBox[Cell[TextData[{Cell[BoxData[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation","FooterIcons"},file]]]]],spacer,s}]],Sequence@@other]],
          
        StringMatchQ[s, "*Mathematica Journal*"],
        With[{file=iconrules["TMJ"]},
          ButtonBox[Cell[TextData[{Cell[BoxData[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation","FooterIcons"},file]]]]],spacer,s}]],Sequence@@other]],
          
        StringMatchQ[s, "*Wolfram Community*"] || !FreeQ[{other}, str_String/;StringMatchQ[str, "*community.wolfram.com*"]],
        With[{file=iconrules["Community"]},
          ButtonBox[Cell[TextData[{Cell[BoxData[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation","FooterIcons"},file]]]]],spacer,s}]],Sequence@@other]],
        
        True,
        (* adding bullets to non-product related links *)
        (*ButtonBox[Cell[TextData[{emptyspacer,spacer,s}]], Sequence@@other]*)
        Cell[BoxData[RowBox[{emptyspacer, "\[FilledVerySmallSquare]", ButtonBox[Cell[TextData[{TemplateBox[{2},"Spacer1"], s}]], Sequence@@other]}]]]
        
        ]]]],some,style,rest], {1}];
    Prepend[rest,first]];


AddWFRCrosslinks[seeAlsoCell_, uri_String, lang_String:"English"]:=
  Module[{cgrid, newcell = seeAlsoCell, localizedtype},
    If[MatchQ[newcell, {Cell[BoxData[GridBox[__]], ___]}],
      cgrid = FirstCase[newcell, GridBox[{{StyleBox[__, "SeeAlsoSection"]}, ___}, ___], {}, Infinity];
      If[cgrid =!= {},
        localizedtype = Localized["SeeAlsoRelatedWFRText", lang];
        With[{lt = localizedtype},
        With[{dynMod = 
          DynamicModuleBox[{$CellContext`wfrgrid$$},
            DynamicBox[Quiet[$CellContext`wfrgrid$$=(WFRCrosslinks`SymbolCrosslinkData[uri])];
             If[TrueQ[ListQ[$CellContext`wfrgrid$$]],
              With[{grid=CurrentValue[EvaluationCell[],{TaggingRules,"SeeAlsoGrid"},""]},
                If[MatchQ[grid,_GridBox],Insert[grid, WFRCrosslinks`MakeWFRCrosslinkSeeAlsos[$CellContext`wfrgrid$$, lt],{1,-1}]]],
                  CurrentValue[EvaluationCell[],{TaggingRules,"SeeAlsoGrid"},""]],
              TrackedSymbols:>{WFRCrosslinks`SymbolCrosslinkData}],
            Initialization:>(
              Quiet[Needs["WFRCrosslinks`"]]
              )]},
            newcell = {Append[ReplaceAll[newcell[[1]], cgrid -> dynMod], TaggingRules -> {"SeeAlsoGrid" -> cgrid}]}          
          ]]
      ]
     ];
     
     newcell
  ]


ReformatHistoryCell[historyCell_, lang_String:"English"]:=
  If[MatchQ[Flatten[historyCell], {}] || SameQ[historyCell, ""], {},
  Module[{cell,  
    GetSummaryURI,
	attractorImage = Get[DocsAsset["Graphics", "HistoryIcon-main.m"]]
    },
    GetSummaryURI[v_String]:=
      Module[{num, uri}, num = StringJoin[Take[StringSplit[First@StringCases[v, (DigitCharacter|".")..], "."], 2]];
      If[StringMatchQ[num, DigitCharacter..],
        Which[
         ToExpression[num] < 60,
         uri = None,
         True,
         uri = StringJoin["paclet:guide/SummaryOfNewFeaturesIn", num]
        ],
        uri = None
       ];
      uri];
    
    cell=
      Nest[
       ReplaceAll[#, patt:{Longest[some___], year_String, vers:StyleBox[v_,"HistoryVersion"], rest___}:>
        Module[{uri = GetSummaryURI[v]}, 
         If[StringQ[uri],
        
         {some,
         Cell[BoxData[TemplateBox[{Cell[TextData[{year,vers}]],GetSummaryURI[v]},
           "GrayLink",BaseStyle->"History"]],
           "History"],rest},
           
         patt]]]&, historyCell, Length[Cases[historyCell, StyleBox[_,"HistoryVersion"], Infinity]]];
    Cell[BoxData[GridBox[{{attractorImage, 
      GridBox[{
       {RowBox[{TemplateBox[{4},"Spacer1"],StyleBox[Localized["HistorySectionText", lang], "HistorySection"]}]},
       {RowBox[{TemplateBox[{4},"Spacer1"], cell[[1]]}]}}]
     }}]], "HistorySection"]
    ]];


(******************************************************************************)
(** ConvertNotebook - Alphabetical Listings **)
ConvertNotebook[expr_, info_,
    ("Alphabetical Listing"),
    opts___?OptionQ] := 
  Module[{chars, linkcell, pos, nb=expr},
    chars = Cases[nb, Cell[c_String, ___, "GuideFunctionsSubsection", ___] :> c, Infinity];
    linkcell = Cell[BoxData[GridBox[{TagBox[
       ButtonBox[
         PaneSelectorBox[{False->#, True->StyleBox[#,FontColor->RGBColor[0.854902, 0.396078, 0.145098]]}, Dynamic[CurrentValue["MouseOver"]]], 
       BaseStyle->{"Hyperlink", FontFamily->"Source Sans Pro"}, 
       ButtonFunction:>NotebookLocate[#], ButtonData->"", ButtonNote->""], MouseAppearanceTag["LinkHand"]]& /@ chars}]], 
       "AlphabeticalListingAnchor"];
    pos = Position[nb, Cell[___, "GuideAbstract", ___]];
    If[pos === {}, pos = Position[nb, Cell[___, "GuideTitle", ___]]];
    pos = MapAt[#+1&, pos, {1,-1}];
    If[chars =!= {}, nb = Insert[nb, linkcell, pos]];
    nb
  ];


(******************************************************************************)
(** ConvertNotebook - Example pages **)
ConvertNotebook[expr_, info_,
    ("Example" | "Featured Example"),
    opts___?OptionQ] := 
  Module[{nb=expr, lang},
    lang = "Language" /. {opts} /. Options[MakeNotebook];

    nb = DeleteCases[nb, Cell[___, "ContributorSection" | "Contributor" | "WeightingSection" | "Weight" | "AlternateThumbnail", ___], Infinity];

    BuildLog["-- Localize MoreAbout on example pages"];
    nb = nb /. Cell[c_, "FeaturedExampleMoreAboutSection", o___] :> 
      Cell[Localized["MoreAboutSectionText", lang], "FeaturedExampleMoreAboutSection", o];
      
    BuildLog["-- Localize see also section on example pages"];
    nb = nb /. Cell[c_, "FeaturedExampleSeeAlsoSection", o___] :> 
      Cell[Localized["SeeAlsoSectionText", lang], "FeaturedExampleSeeAlsoSection", o];

    BuildLog["-- Localize RelatedLinks on example pages"];
    nb = nb /. Cell[c_, "FeaturedExampleRelatedLinksSection", o___] :> 
      Cell[Localized["RelatedLinksSectionText", lang], "FeaturedExampleRelatedLinksSection", o];
   
   nb = RenumberCells[nb, {"FeaturedExampleTitle", "GuideDelimiter"}];
   
    nb
  ];


(******************************************************************************)
(** ConvertNotebook - new-style Program (man) pages **)
ConvertNotebook[nb_, info_,
    ("Program"),
    opts___?OptionQ] := 
  Module[{expr=nb, lang, type, uri, seealsos, newseealsos, guides, tutorials},
    lang = "Language" /. {opts} /. Options[MakeNotebook];
    type = "EntityType" /. info;
    uri = "URI" /. info;

    (* add attractor icons to footer links *)
    guides = GetMoreAboutGroup[expr];
    guides = AddAttractorIcons[guides, lang];
    
    seealsos = GetSeeAlsoGroup[expr];
    newseealsos = AddAttractorIcons[seealsos, lang];
    
    tutorials = GetTutorialsGroup[expr];
    tutorials = AddAttractorIcons[tutorials, lang];
    
    expr = ReplaceAll[expr, 
      {Cell[CellGroupData[{Cell[__, "SeeAlsoSection", ___], __}, ___]] -> newseealsos,
       Cell[CellGroupData[{Cell[__, "MoreAboutSection", ___], __}, ___]] -> guides,
       Cell[CellGroupData[{Cell[__, "TutorialsSection", ___], __}, ___]] -> tutorials}];
       
    (* remove tutorial collection *)
    expr = DeleteCases[expr, Cell[CellGroupData[{Cell["Tutorial Collection", ___], ___}, ___]], Infinity];

    BuildLog["-- Localize RelatedLinks on program pages"];
    expr = expr /. Cell[c_, "RelatedLinksSection", o___] :> 
      Cell[Localized["RelatedLinksSectionText", lang], "RelatedLinksSection", o];

    BuildLog["-- Delete various cells"];
    If[buildmode==="Future",
    	expr,
    	expr = DeleteCases[expr, Alternatives[
        Cell[s_String /;
          StringMatchQ[s, RegularExpression[".*XX+"]], ___],
        Cell[TextData[s_List /; Apply[ Or,
        	StringMatchQ[Transmogrify`ConvertToString[#], 
        		RegularExpression[".*XX+.*"]] & /@ s]], "Notes", ___],
        Cell[cc_ /; (And[
            Length[#] > 0,
            StringMatchQ[Transmogrify`ConvertToString[First[#]], RegularExpression["XX+"]]
          ]&[Cases[cc,
            Cell[_, ("InlineFormula"|"InlineGuideFunctionListing"), ___], Infinity]]),
          "InlineGuideFunctionListing" | "GuideText" ___],
        Cell[s_String /;
          StringMatchQ[s, RegularExpression["(XX+( \\. )?)+"]],
          "InlineFormula", ___],
        Cell[s_String /;
          StringMatchQ[s, RegularExpression["(XX+( \\. )?)+"]],
          "InlineGuideFunctionListing", ___],
        Cell[cc_ /;
          (Count[cc, Cell[BoxData[s_String /;
              StringMatchQ[s, RegularExpression["XX+"]]
            ], "InlineGuideFunction", ___], Infinity] > 0),
          "Notes", ___]
    ], Infinity]];


    (* V11.1 Redesign -- removed this bit since it was just changing the SeeAlsos to be in Verdana *)
    (*seealsos = First[Cases[expr, Cell[__, "SeeAlso", ___], Infinity]];
    newseealsos = ReplaceAll[seealsos, 
      ButtonBox[c___]:>StyleBox[ButtonBox[c], FontFamily->"Verdana"]];
    expr = expr /. seealsos :> newseealsos;*)

    (* Convert Notes cells to ProgramNotes cells *)
    expr = expr /. Cell[first__, "Notes", rest___] :> Cell[first, "ProgramNotes", rest];
    expr = expr /. Cell[first__, "ObjectName"|"ObjectNameAlt", rest___] :> Cell[first, "ProgramName", rest];
   
    expr
  ];


(******************************************************************************)
(** ConvertNotebook - Symbols **)
ConvertNotebook[
	nb_,
	info_,
	(
		"AppleScript" | "Character" |  "Character Name" | "File" | "Front End Object"
		| "LibraryLink C Function" | "MathLink C Function" | "UnityLink C# Function" | "Menu Item"
		| "Message" | "Method" | "Symbol" | "System Program" | "Widget" | "Format" | "Screencast"
		| "Indicator" | "Device Connection" | "Service Connection" | "Interpreter" | "Embedding Format"
		| "Net Encoder" | "Net Decoder" | "External Evaluation System" | "Classifier" | "Predictor"
		| "Entity" | "Compiled Type" | "Blockchain" | "Database Connection" | "Data Structure"
		| "Text Content" | "Remote Batch Computation Provider" | "ResourceObject" | "Resource Object" | "Question Interface"
		| "Featureset Page"
	),
    opts___?OptionQ
] := Module[{
	expr, lang, poss, uri, pacletName, type, context,
	awaitingReviewNote, obsolescenceNote, dataObsolescenceNote (* for *Data pages being replaced by entity pages *),
	objectNameCell, objectName, underDevelopmentCell,
	characterNameCell, characterName, characterImageCell, chargridOpts, characterImage, characterImageGrid,
	usageNameCell,
	examplesGroup, notesGroup, seeAlsoGroup, relatedInterpretersGroup, importExportGroup,
	indicatorExampleGroup, indicatorDescriptionGroup, indicatorFormulaGroup, elementGroup,
	exportelementGroup, notebookInterfaceGroup, optionsGroup, tutorialsGroup, relatedLinksGroup,
	moreAboutGroup,  elucidationGroup, tutorialCollectionGroup, otherTypeGroup,
	headerCells,
	nbOrder,
	closedGroups, CloseGroups,
	nksIndexedFunctions, warnicon, fnEssayGroup, fnessay,
	entityGroup,
	notesthumbnails, entitythumbnails, formatthumbnails, devicethumbnails,
	classifierthumbnails, predictorthumbnails, externalevaluationsystemthumbnails,
	othertypethumbnails,
	onetimesections, groupopenersections, pagedelimiter, experimentalbanner,
	hasexamplesQ, hasnotesQ, experimentalQ,
	contextCell,
	workflowsGroup,
	descriptionCell,
	blockchainabstract = {},
	buildmode
},
	BuildLog["** Entering ConvertNotebook - Symbols **"];

	type = "EntityType" /. info;
	pacletName = "PacletName" /. info;
	uri = "URI" /. info;
	lang = "Language" /. {opts} /. Options[MakeNotebook];
	expr = nb;
	experimentalQ = TrueQ["Underdevelopment" /. info];
	context = "Context" /. info;

	buildmode = "BuildMode" /. {opts} /. Options[MakeNotebook];

	DocsAssert[StringQ[buildmode], "BuildMode was not StringQ in ConvertNotebook"];

	(* Load thumbnails just for the specific page type *)
	LoadThumbnails[type];

	(* handle special Device notebook styles *)
	(* Formatting note: 6ca3220f
		overhauled AdjustModInfoUsageCells[ ] to work for Device pages (which have
		multiple double-line Usage[ ] cells per page)
	*)
	expr = If[MemberQ[{"Device Connection"}, type],
		DeleteCases[
			expr,
			(* Formatting note: 863bd457
				checked for an untouched placeholder cell in Device pages to remove it *)
			Cell["General statement of device compatibility/support..."|
				"Listing of some devices in this class..."|"Additional notes ..."|
				"These examples were run on XXXX with XXXX.", ___, "Notes"|"DeviceNotes"|"DeviceExamplePrologue", ___
			],
			Infinity
		]
		,
		expr
	];
	expr = If[MemberQ[{"Device Connection"}, type],
		expr /. Cell[a__, "Notes", b___] :> Cell[a, "DeviceNotes", b]
		,
		expr
	];
	expr = If[MemberQ[{"Device Connection"}, type],
		expr /. Cell[a__, "Usage", b___] :> Cell[a, "DeviceDoubleUsage", b]
		,
		expr
	];
	expr = If[MemberQ[{"Device Connection"}, type],
		ReplaceAll[
			DeleteCases[
				expr,
				Cell[c___, "AlternateThumbnail", ___]
					/; !FreeQ[{c}, s_String /; !StringFreeQ[s, "Thumbnail Placeholder"]],
				Infinity
			],
			Cell[s___, "AlternateThumbnail", r___] :> Cell[s, "DeviceImage", r]
		]
		,
		expr
	];
	expr = If[MemberQ[{"Device Connection"}, type],
		If[FreeQ[expr, Cell[___, "DeviceImage"|"AlternateThumbnail", ___], Infinity],
			DeleteCases[expr, Cell[___, "DeviceImageCaption", ___], Infinity]
			,
			expr
		]
		,
		expr
	];
	(* Clean up "DeviceUsage" cell (remove lines from template with XXXX) *)
	expr = If[MemberQ[{"Device Connection"}, type],
		ReplaceAll[
			expr,
			cell:Cell[___, "DeviceUsage", ___] :>
				ReplaceAll[
					cell,
					s_String /; !StringFreeQ[s, "XXXX"] :>
						StringJoin[Riffle[Select[StringSplit[s, "\n"], StringFreeQ[#, "XXXX"]& ], "\n"]]
				]
		],
		expr
	];

	(* In Service pages, convert "InlineGuideFunction" to "InlineFunctionSans" *)
	expr = If[MemberQ[{"Service Connection"}, type],
		ReplaceAll[
			expr,
			Cell[BoxData[c_] | TextData[c_], some___, "InlineGuideFunction", rest___] :>
				Cell[TextData[c], some, "InlineFunctionSans", rest]
		]
		,
		expr
	];
	(* Again, in service pages, convert "Notes" cells to "ServiceNotes" -- which drops the dingbat
		and aligns the text *)
	expr = If[MemberQ[{"Service Connection"}, type],
		ReplaceAll[
			expr,
			Cell[some___, "Notes", rest___] :>
				Cell[some, "ServiceNotes", rest]
		]
		,
		expr
	];
	(* Formatting note: b05fe862
		attempt to fix bug 265226 in build code (inline listings on service pages
		misformatted)
	*)
	(* Another for service pages: convert xxx . xxx InlineGuideFunctionListings to something better *)
	expr = If[MemberQ[{"Service Connection"}, type],
		ReplaceAll[
			expr,
			Cell[
				c_String /; StringMatchQ[c, ___~~" . "~~___],
				some___,
				"InlineGuideFunctionListing",
				rest___
			] :> Cell[
				TextData[Riffle[
					(StyleBox[#, "InlineFunctionSans"]) & /@ StringSplit[c, " . "],
					StyleBox[" \[FilledVerySmallSquare] ", "InlineSeparator"]
				]],
				some,
				"InlineGuideFunctionListing",
				"InlineGuideFunctionListingGray",
				rest
			]
		]
		,
		expr
	];

	(* do something special for DatabaseConnection pages *)
	expr = If[MemberQ[{"Database Connection"}, type],
		expr /. Cell[a__, "Usage", b___] :> Cell[a, "Notes", b]
		,
		expr
	];

	(* Remove ILD-specific cells -- commented out section types that will stay in the built nb*)
	expr = Module[{
		ildstyles = {"ILDBanner", (*"AlternativeNames", "DesignJustification", *)
		"InternalLinks", "DesignChoicesNumbered", "DesignChoices"}
	},
		DeleteCases[expr, Cell[___, Alternatives@@ildstyles, ___ _]]
	];

	(* Formatting note: a52c2822
		mark empty function essay sections for removal, and otherwise handling function
		essay cells until they're ready for primetime
	*)
	(* Formatting note: 80ac25d9
		fix bug 294563 (ref/WolframLanguageData was getting a function essay from Sin
		because of one of its examples)
	*)
	(* Gather then remove Function Essay, to be re-added later *)
	fnessay = Flatten[Cases[
		expr,
		Cell[
			CellGroupData[
				{
					Cell[__, "FunctionEssaySection", ___],
					___,
					cell : (Cell[__, "FunctionEssay", ___] ..),
					Cell[c__ /; FreeQ[{c}, "FunctionEssay"]] ...
				},
				___
			]
		] :> {cell},
		Infinity
	]];
	expr = DeleteCases[expr, Cell[___, "FunctionEssay", ___], Infinity];

	(* For non-symbol ref pages, add Unfinished Draft banner *)
	expr = OtherRefAddUnfinishedDraftBanner[expr, info, type, opts];

	(* 'AwaitingReviewNote' *)
	awaitingReviewNote = Cases[expr, Cell[_, "AwaitingReviewNote", ___], Infinity];
	awaitingReviewNote = DeleteCases[
		awaitingReviewNote,
		ButtonBox[__, ButtonData -> "paclet:tutorial/ObjectsInMathematicaAwaitingFutureDesignReview", ___],
		Infinity
	];
	expr = DeleteCases[expr, Cell[_, "AwaitingReviewNote", ___], Infinity];

	obsolescenceNote = Cases[expr, Cell[_, "ObsolescenceNote", ___], Infinity];
	expr = DeleteCases[expr, Cell[_, "ObsolescenceNote", ___], Infinity];

	(* Formatting note: 8b5ce7c9
		added support for DataPageEntityNote banners on *Data pages obsoleted by entities
	*)
	dataObsolescenceNote = Cases[expr, Cell[_, "DataPageEntityNote", ___], Infinity];
	expr = DeleteCases[expr, Cell[_, "DataPageEntityNote", ___], Infinity];

	(* V9 EDIT BELOW *)
	(* Formatting note: 572c20e4
		Move population of More About section to occur before creation of anchor bar. This
		fixes problem of missing More About pulldown menus.
	*)
	headerCells = Cases[expr, Cell[_, "SymbolColorBar" | "LinkTrail" | "AnchorBarGrid", ___], Infinity];
	expr = DeleteCases[expr, Cell[_, "SymbolColorBar" | "LinkTrail" | "AnchorBarGrid", ___], Infinity];

	(* save ObjectName cell *)
	objectNameCell = Cases[expr, Cell[cont_, "ObjectName"|"ObjectNameSmall"|"ObjectNameAlt",___], Infinity, 1];
	objectName = Cases[objectNameCell, Cell[cont_String, "ObjectName"|"ObjectNameSmall"|"ObjectNameAlt",___] :> cont, Infinity];
	If[Length[objectName] == 0,
		objectName = Cases[
			objectNameCell,
			Cell[cont:TextData[{ ___}, ___], "ObjectName"|"ObjectNameSmall"|"ObjectNameAlt", ___]
				:> Transmogrify`ConvertToString[cont],
			Infinity
		]
	];
	objectName = ToString[ If[Length@objectName > 0, First@objectName, " "] ];
	objectName = StringReplace[objectName, RegularExpression[" .*"] -> ""];

	characterNameCell = Cases[expr, Cell[cont_, "CharacterName",___], Infinity, 1];
	characterName = Cases[characterNameCell, Cell[cont_String, "CharacterName",___] :> cont, Infinity];
	characterName = ToString[ If[Length@characterName > 0, First@characterName, " "] ];

	chargridOpts = Sequence[
		GridBoxBackground -> {
			"Columns" -> {{GrayLevel[1]}}
		},
		GridBoxDividers -> {
			"Columns" -> {{Directive[AbsoluteThickness[1], GrayLevel[0.596078]]}},
			"Rows" -> {{Directive[AbsoluteThickness[1], GrayLevel[0.596078]]}}
		},
		GridBoxSpacings -> {"Columns" -> {{0.4}}, "Rows" -> {{0.2}}}
	];

	Quiet[
		characterImageCell = First[Cases[
			expr,
			Cell[cont_, "CharacterImage", rest___] :> {First[Cases[cont, GridBox[l_,___]:>l, Infinity]], {rest}},
			Infinity,
			1
		]]
	];
	If[Head[characterImageCell] =!= First,
		characterImageCell = Cell[
			BoxData[GridBox[characterImageCell[[1]], chargridOpts]],
			Sequence@@characterImageCell[[2]]
		];
		characterImage = Cases[
			characterImageCell,
			Cell[cont_String, "CharacterImage", ___] :> cont,
			Infinity
		];
		characterImage = ToString[ If[Length@characterImage > 0, First@characterImage, " "] ]
	];

	expr = If[MemberQ[{"Indicator"}, type],
		expr /. Cell[a__, "Usage", b___] :> Cell[a, "IndicatorUsage", b]
		,
		expr
	];

	(* for Entity pages only: save out description cell, delete it before getting Usage cell *)
	If[MemberQ[{"Entity"}, type],
		descriptionCell = Cases[
			expr,
			Cell[first__, "TextUsage", rest___] :> DeleteCases[Cell[first, "EntityAbstract", rest], "Usage", Infinity],
			Infinity,
			1
		];
		If[Length[descriptionCell] > 0,
			descriptionCell = First@descriptionCell
		];
		expr = DeleteCases[expr, Cell[__, "TextUsage", ___], Infinity]
	];

	(* save Usage cell, delete and add it back later *)
	usageNameCell = Cases[expr, Cell[cont_, "Usage",___], Infinity, 1];
	If[Length[usageNameCell] > 0,
		usageNameCell = First@usageNameCell
	];

	underDevelopmentCell = Cases[expr, Cell["U N D E R   D E V E L O P M E N T",___], Infinity, 1];
	If[Length[underDevelopmentCell] > 0,
		underDevelopmentCell = First@underDevelopmentCell
	];

	(* Add NKS Online links *)
	(* List of M- functions which appear in NKSOnline index from Oyvind 11/29/06 *)
	nksIndexedFunctions = {
		"AiryAi", "And", "Apply", "ArcCos", "ArcSin", "ArcTan", "ArithmeticGeometricMean",
		"AspectRatio", "BernoulliB", "BesselI", "BesselJ", "BesselY", "Binomial", "BitAnd",
		"BitOr", "BitXor", "Block", "CarmichaelLambda", "Catalan", "Catch", "Ceiling",
		"CellularAutomaton", "ChebyshevT", "CoefficientList", "Collect", "Compile",
		"Complement", "ContinuedFraction", "Cos", "Cosh", "Cot", "Coth", "Csc", "Cuboid",
		"Cyclotomic", "DedekindEta", "Delete", "DeleteCases", "Depth", "Derivative", "Det",
		"DigitCount", "Distribute", "Divide", "Divisors", "DivisorSigma", "Do", "Dot",
		"Drop", "Eigenvalues", "Element", "EllipticK", "EllipticTheta", "Equal", "Erf",
		"EulerGamma", "EulerPhi", "Exists", "Exp", "Expand", "ExpIntegralEi", "Export",
		"ExtendedGCD", "Factor", "Factorial", "Factorial2", "FactorInteger", "False",
		"Fibonacci", "FindMinimum", "FindRoot", "Fit", "FixedPoint", "FixedPointList",
		"Flat", "Flatten", "Floor", "Fold", "FoldList", "For", "ForAll", "Fourier",
		"FractionalPart", "FresnelC", "FresnelS", "FromDigits", "FullForm", "FullSimplify",
		"Function", "FunctionExpand", "Gamma", "GCD", "GegenbauerC", "GoldenRatio",
		"Graphics3D", "GroebnerBasis", "HermiteH", "Hypergeometric1F1", "Hypergeometric2F1",
		"HypergeometricPFQ", "If", "Implies", "Inner", "InputForm", "Insert", "IntegerDigits",
		"IntegerExponent", "IntegerQ", "Integrate", "Intersection", "InverseFunction",
		"JacobiP", "JacobiSN", "JacobiSymbol", "Join", "Khinchin", "KleinInvariantJ",
		"KroneckerDelta", "LaguerreL", "LatticeReduce", "LCM", "LeafCount", "LegendreP",
		"Length", "Less", "LinearSolve", "Listable", "ListConvolve", "ListCorrelate",
		"ListPlay", "Log", "LogIntegral", "Map", "MapAt", "MapIndexed", "MatchQ", "MathieuC",
		"MatrixPower", "Max", "MeijerG", "MemberQ", "Mod", "ModularLambda", "MoebiusMu",
		"Multinomial", "MultiplicativeOrder", "Nand", "NDSolve", "Nest", "NestList",
		"NestWhile", "NestWhileList", "NIntegrate", "Nor", "Not", "NSolve", "NullSpace",
		"OddQ", "Or", "Orderless", "Outer", "PadLeft", "Partition", "PartitionsP", "Pi",
		"Play", "Plot3D", "Plus", "PolyGamma", "PolyLog", "PolynomialMod", "PolynomialReduce",
		"PolynomialRemainder", "Position", "Power", "PowerMod", "Prepend", "PrimePi", "PrimeQ",
		"Product", "ProductLog", "PseudoInverse", "Quotient", "Random", "Range", "Raster",
		"RealDigits", "RepeatedNull", "ReplaceAll", "ReplaceList", "ReplacePart",
		"ReplaceRepeated", "Rest", "Resultant", "RiemannSiegelZ", "Root", "RotateLeft",
		"Rule", "Sec", "Sech", "SeedRandom", "Select", "Series", "Simplify", "Sin", "Sinh",
		"SinIntegral", "SixJSymbol", "Slot", "Sort", "SphericalHarmonicY", "Split", "Sqrt",
		"StandardForm", "StieltjesGamma", "StirlingS1", "StirlingS2", "StringJoin",
		"StringLength", "StringPosition", "StringReplace", "StringReplacePart",
		"SubsuperscriptBox", "Sum", "Table", "Take", "Tan", "Tanh", "TensorRank", "Through",
		"Throw", "TimeConstraint", "Times", "ToExpression", "ToString", "Transpose",
		"TrigFactor", "True", "TrueQ", "Union", "WeierstrassP", "While", "Xor", "Zeta",
		"$IterationLimit"
	};

	If[MemberQ[{"Symbol"}, type],
		BuildLog["-- Add nksonline links"];
		If[MemberQ[nksIndexedFunctions, objectName],
			Module[{rcell, pos, rlink},
				rcell = Cell[
					TextData[{
						ButtonBox[
							"NKS|Online",
							BaseStyle -> "Hyperlink",
							(* Formatting note: 63b33818
								fix search URL per comments in bug 354326
							*)
							ButtonData -> {
								URL["http://www.wolframscience.com/nks/search/?collection=nks&query="<>objectName],
								None
							}
						],
						" (",
						StyleBox[
							ButtonBox[
								"A New Kind of Science",
								BaseStyle -> "Hyperlink",
								ButtonData -> {URL["http://www.wolframscience.com/nksonline/"], None}
							],
							FontSlant->"Italic"
						],
						")"
					}],
					"RelatedLinks"
				];
				expr = EnsureCellGroupData[expr, "RelatedLinksSection", lang];
				pos = Position[
					expr,
					Cell[CellGroupData[{Cell[_, "RelatedLinksSection", ___], ___}, ___], ___],
					Infinity
				];
				pos = First[pos];
				rlinks = Extract[expr, pos];
				rlinks[[1, 1]] = Append[rlinks[[1, 1]], rcell];
				expr = ReplacePart[expr, pos->rlinks];
			]
		]
	];

	(* Add Demonstration links *)
	If[MemberQ[{"Symbol"},type],
		If[$DocumentationExportDebug,
			BuildLog["-- Add demonstration site links"]
		];
		If[DocumentationBuild`Info`GetSourceInformation["paclet:" <> uri, lang, "Demonstrations"] === True,
			Module[{pos, rlinks, rcell},
				(* FIXME: get right text, localize *)
				rcell = Cell[
					TextData[Join[
						{Localized["DemonstrationsWith", lang][objectName]},
						Localized["WolframDemonstrationsProject", lang]
					]],
					"RelatedLinks"
				];
				pos = Position[
					expr,
					Cell[CellGroupData[{Cell[_, "RelatedLinksSection", ___], ___}, ___], ___],
					Infinity
				];
				If[Length[pos] != 0,
					pos = First[pos];
					rlinks = Extract[expr, pos];
					rlinks[[1, 1]] = Insert[rlinks[[1, 1]], rcell, 2];
					expr = ReplacePart[expr, pos->rlinks];
					, (* Else *)
					expr = Insert[
						expr,
						Cell[CellGroupData[
							{
								Cell[
									StringReplace[
										Localized["RelatedLinksSectionText", lang],
										{" \[RightGuillemet]" | "\[RightGuillemet]"->""}
									],
									"RelatedLinksSection"
								],
								rcell
							},
							Open
						]],
						First[Position[
							expr,
							Cell[CellGroupData[{Cell[_, "SeeAlsoSection", ___], ___}, ___], ___],
							Infinity
						]]
					];
				];
			]
		]
	];

	BuildLog["-- Rearrange Notes and Examples"];

	(* Move Examples groups directly below Notes per sw *)
	notesGroup = GetNotesGroup[expr, type, lang, opts, "HasOptions" -> DocumentationBuild`Info`GetSourceInformation[uri, lang, "HasOptions"]];
	fnEssayGroup = Cell[CellGroupData[{Cell[Localized["GuideBackgroundSectionText", lang], "FunctionEssaySection"], Sequence @@ Flatten[{fnessay}]}, Open]];
	seeAlsoGroup = GetSeeAlsoGroup[expr, lang];
	examplesGroup = GetExamplesGroup[expr, lang];
	moreAboutGroup = GetMoreAboutGroup[expr, lang];
	relatedLinksGroup = GetRelatedLinksGroup[expr, lang];
	tutorialsGroup = GetTutorialsGroup[expr, lang];
	elucidationGroup = Cases[expr, Cell[_,"Elucidation",___], Infinity];
	indicatorExampleGroup = GetIndicatorExampleGroup[expr, lang];
	indicatorDescriptionGroup = GetIndicatorDescriptionGroup[expr, lang];
	indicatorFormulaGroup = GetIndicatorFormulaGroup[expr, lang];
	tutorialCollectionGroup = GetTutorialCollectionGroup[expr, lang];
	entityGroup = GetRelatedEntityGroup[expr, lang];
	workflowsGroup = GetWorkflowsGroup[expr, lang];
	notesthumbnails = If[MatchQ[type, "Symbol"], GetNotesThumbnails[uri], ""];
	entitythumbnails = If[StringMatchQ[uri, "*ref/entity/*"], GetEntityThumbnails[uri], ""];
	formatthumbnails = If[StringMatchQ[uri, "*ref/format/*"], GetFormatThumbnails[uri], ""];
	devicethumbnails = If[StringMatchQ[uri, "*ref/device/*"], GetDeviceThumbnails[uri], ""];
	classifierthumbnails = If[StringMatchQ[uri, "*ref/classifier/*"], GetClassifierPredictorThumbnails[uri, "Classifier"], ""];
	predictorthumbnails = If[StringMatchQ[uri, "*ref/predictor/*"], GetClassifierPredictorThumbnails[uri, "Predictor"], ""];
	externalevaluationsystemthumbnails = If[StringMatchQ[uri, "*ref/externalevaluationsystem/*"], GetExternalEvaluationSystemThumbnails[uri], ""];
	othertypethumbnails = If[
		MemberQ[
			{
				"Menu Item", "MathLink C Function", "File", "System Program",
				"Net Encoder", "Net Decoder", "Front End Object", "Data Structure",
				"Text Content"
			},
			type
		],
		GetOtherTypeThumbnails[uri, type],
		{}
	];
	pagedelimiter = Cell["", "PageDelimiter"];
	underDevelopmentCell = If[experimentalQ,
		Cell[TextData[{Cell["[", "ObjectNameAltQuotes"], "EXPERIMENTAL", Cell["]", "ObjectNameAltQuotes"]}], "ExperimentalBanner"]
		,
		{}
	];
	contextCell = Switch[context, "System`", {}, _, Cell[context, "ContextNameCell"]];

	importExportGroup = If[MemberQ[{"Format"}, type], GetImportExportGroup[expr, lang], {}];
	elementGroup = If[MemberQ[{"Format"}, type], GetElementGroup[expr, lang, "ElementsSection"], {}];
	exportelementGroup = If[MemberQ[{"Format"}, type], GetElementGroup[expr, lang, "ExportElementsSection"], {}];
	notebookInterfaceGroup = If[MemberQ[{"Format"}, type], GetNotebookInterfaceGroup[expr, lang], {}];
	optionsGroup = If[MemberQ[{"Format"}, type], GetOptionsGroup[expr, lang], {}];
	otherTypeGroup = If[
		MemberQ[
			{
				"Service Connection", "Device Connection", "Interpreter", "Embedding Format",
				"Method", "External Evaluation System", "Classifier", "Predictor", "Entity",
				"Compiled Type", "Blockchain", "Database Connection", "Remote Batch Computation Provider",
				"Question Interface", "ResourceObject", "Resource Object", "Featureset Page"
			},
			type
		],
		GetOtherSectionGroup[expr, lang],
		{}
	];

	(* CHARACTER PAGE: make image / notes grid *)
	Quiet[
		characterImageGrid = DocumentationBuild`Utils`MakeCharacterNotesGrid[characterImageCell, notesGroup]
	];

	(* Formatting note: 747878cb
		fix bug 388287 and other unreported
	*)
	{
		notesGroup,
		importExportGroup,
		elementGroup,
		exportelementGroup,
		notebookInterfaceGroup,
		optionsGroup,
		otherTypeGroup
	} = HighlightSectionHeaderOfGroup[#]& /@ {
		notesGroup,
		importExportGroup,
		elementGroup,
		exportelementGroup,
		notebookInterfaceGroup,
		optionsGroup,
		otherTypeGroup
	};

	(* Formatting note: b53cc4a4
		Add tutorial collection to Symbols.
	*)
	expr = DeleteCases[
		expr,
		(Cell[CellGroupData[{#, ___}, ___], ___] | #)&[
			Cell[
				__,
				"GuideTutorialCollection" | "GuideTutorialCollectionSection" | "TutorialCollectionSection" | "TutorialCollection",
				___
			]
		],
		Infinity
	];

	BuildLog["-- Delete various cells"];
	(* Delete cell groups, to be explicitly assembled later *)
	expr = DeleteCases[
		expr,
		Alternatives[
			(* Remove unwanted groups *)
			(Cell[CellGroupData[{#, ___}, ___], ___] | #)&[
				Cell[
					_,
					"ObjectName" | "ObjectNameSmall" | "ObjectNameAlt" | "CharacterName" | "CharacterImage" | "PrimaryExamplesSection" | "ExtendedExamplesSection" | "SeeAlsoSection"
					| "ImportExportSection" | "ElementsSection" | "ExportElementsSection" | "NotebookInterfaceSection" | "OptionsSection" | "TutorialsSection" | "TechNotesSection"
					| "MoreAboutSection" | "RelatedWorkflowsSection" | "MoreToExploreSection" | "MoreInformationSection" | "RelatedLinksSection"
					| "IndicatorExampleSection"| "IndicatorDescriptionSection"| "IndicatorFormulaSection"
					| "ServiceSubsection" | "DeviceSubsection" | "InterpreterSection" | "EmbeddingFormatSection" | "FunctionEssaySection" | "MethodSection"
					| "ExternalEvaluateSystemSection" | "EntitySection" | "RelatedEntitySection" |
					(* Formatting note: ef9a261a: fixed 374436, impl. WLDOCS-2041 *)
					"CompiledTypeSection" | "RelatedEncoderSection" | "RelatedDecoderSection" | "BlockchainSection" | "RelatedBlockchainSection" | "DatabaseConnectionSection"
					| "ClassifierSection" | "PredictorSection" | "RelatedClassifiersSection" | "RelatedPredictorsSection" | "BatchComputationProviderSection" 
					| "QuestionInterfaceSection" | "ResourceObjectSection",
					___
				]
			],
			Cell[_, "PrimaryExamplesSection", ___],
			Cell["U N D E R   D E V E L O P M E N T",___],
			Cell[_, "SeeAlsoSection", ___],
			Cell[_, "Elucidation", ___]
		],
		Infinity
	];

	BuildLog["-- Remove SeeAlso links to flagged notebooks"];
	seeAlsoGroup = RemoveSeeAlsoGroupLinksToFlaggedSymbols[seeAlsoGroup, pacletName, buildmode];

	(* Add [[Not Ready Yet]] for those symbols that have no examples *)
	(* Removing this per Andre -- 1/22/14 *)
	(*examplesGroup =
	If[FreeQ[examplesGroup, Cell[___, "Input", ___], Infinity],
		Insert[examplesGroup, Cell["[[ Not Ready Yet ]]", "ExamplesUnderConstruction"], MapAt[(#+1)&,
		Position[examplesGroup, Cell[___, "PrimaryExamplesSection", ___]],{1,-1}]],
		examplesGroup];*)

	(* Do example counts *)
	Module[{
		poss, repl, cexpr, cpos, excnt, totalcnt
	},
		poss = Position[
			examplesGroup,
			CellGroupData[{Cell[_, "ExampleSection" | "ExampleSubsection" | "ExampleSubsubsection", ___], ___}, ___],
			Infinity
		];

		excnt = Function[{exex},
			Apply[Plus, Flatten[{
				(* Formatting note: 187145d9
					Copy the fixed example counting code from DocumentationTools
				*)
				(* We don't always add one, because that gives us an off-by-one
				when the section only has subsections.  But we do want to add
				one if there's an actual example directly in the section.
				Note that the {2} level specification excludes anything after
				a delimiter, because ExampleDelimiter groups. *)
				If[
					Count[
						exex,
						Cell[_, "ExampleText" | "Input" | "Code" | "Program", ___]
							| Cell[CellGroupData[{Cell[_, "ExampleText" | "Input" | "Code" | "Program", ___], ___}, ___], ___],
						{2}
					] > 0,
					1,
					0
				],
				(* The number of ExampleDelimiters *)
				Count[
					exex,
					Cell[CellGroupData[{Cell[_, "ExampleDelimiter", ___], ___}, ___], ___],
					{2}
				],
				(* The total count of the contained subsections *)
				Map[
					excnt,
					Cases[
						exex,
						CellGroupData[{Cell[_, "ExampleSubsection" | "ExampleSubsubsection", ___], ___}, ___],
						{3}
					]
				]
			}]]
		];

		examplesGroup = Module[{exprs, positions, testRule},
			(*
				the following is a fix for leaked examples. An example leaks if it is untentavized but lives within a
				tentavized subsection. These leaks would not contribute to the total count, because the counting function
				is examining Cell patterns that have been properly re-flowed by the FE. Leaked examples are not re-flowed.
				This code fixes the syntax into what would be the expression if the FE had re-flowed it, with the important
				caveat that nested leaked examples e.g. examples living in subsubsections have an identical Cell pattern that
				replacement rules can't reach: the original position of the nested examples no longer exist in the notebook
				after the replacement rules are applied (to the outer patterns)
			*)
			exprs = Cases[
				examplesGroup,
				Cell[CellGroupData[
					{cell: Cell[CellGroupData[{___}, ___], ___], ___},
					___
				]] :> cell,
				Infinity
			];
			positions = Position[examplesGroup,
				Cell[CellGroupData[{Cell[
				CellGroupData[{___}, ___], ___], ___}, ___]], Infinity
			];
			ReplacePart[examplesGroup, Thread[positions -> exprs]]
		];

		repl = Map[
			Function[{pos},
				cexpr = Extract[examplesGroup, Join[pos, {1, 1}]];
				(* Position is post-order depth-first, and we want the
					inner-most if there's an InterpretationBox going on. *)
				cpos = First[Position[cexpr, Cell[_, cexpr[[2]], ___], {0, Infinity}]];
				cpos = Join[pos, {1, 1}, cpos, {1}];

				cpos -> TextData[Join[
					If[Head[#] === TextData, First[#], {#}]&[
						Extract[examplesGroup, cpos]
					],
					{
						"\[NonBreakingSpace]\[NonBreakingSpace]",
						Cell[
							"(" <> ToString[excnt[First[Extract[examplesGroup, {pos}]]]] <> ")",
							"ExampleCount"
						]
					}
				]]
			],
			poss
		];

		examplesGroup = ReplacePart[examplesGroup, repl];

		totalcnt = Total @ Flatten @ Cases[
			examplesGroup,
			(Cell[c___,"ExampleSection",___] /; FreeQ[{c}, InterpretationBox, Infinity])
				:> Cases[
					{c},
					Cell[n_String, ___, "ExampleCount", ___] :> ToExpression[n],
					Infinity
				],
			Infinity
		];

		examplesGroup = ReplaceAll[
			examplesGroup,
			{
				Cell[TextData[td_], some___, "PrimaryExamplesSection", rest___]
					:> Cell[
						TextData[Join[
							td,
							{
								"\[NonBreakingSpace]\[NonBreakingSpace]",
								Cell["("<>ToString[totalcnt]<>")", "ExampleCount"]
							}
						]],
						some,
						"PrimaryExamplesSection",
						rest
					],
				Cell[s_String, some___, "PrimaryExamplesSection", rest___]
					:> Cell[
						TextData[{
							Cell[BoxData[ToBoxes[Spacer[6]]]],
							s,
							"\[NonBreakingSpace]\[NonBreakingSpace]",
							Cell["("<>ToString[totalcnt]<>")", "ExampleCount"]
						}],
						some,
						"PrimaryExamplesSection",
						rest
					]
			}
		];
	];

	(* ExampleSection cells are a mess.  It's high time they're cleaned up... *)
	examplesGroup = ReplaceRepeated[
		examplesGroup,
		Cell[
			BoxData[InterpretationBox[display_, ___]],
			((sty:("ExampleSection"|"ExampleSubsection"|"ExampleSubsubsection"))..),
			rest___
		] :> (
			Replace[Unevaluated[display], {
				Cell[content_, ___] :> Cell[content, sty, rest],
				(* Note:
					This case occurs when an Example*Section with the following form
					appears in an authoring notebook:

						Cell[
							BoxData[InterpretationBox[
								Cell["XXXX", "ExampleSubsection"],
								$Line = 0; Null
							]],
							"ExampleSubsection",
							...
						]

					An earlier processing step deletes cells with "XXX" content,
					leaving behind the code to reset the example line numbers.

					A previous iteration of this code would unintentionally evaluate
					the `$Line = 0` statement, causing bug 420578. This new code
					avoids accidentally evaluating the `$Line = 0` statement.

					TODO:
						Fix this issue at the source by more carefully deleting "XXX"
						cells in example sections, to prevent accidentally turning a 2-arg
						InterpretationBox into a 1-arg.
				*)
				HoldPattern[CompoundExpression[$Line = 0, Null]] :> (
					Cell["XXX", sty, rest]
				),
				other_ :> DocsError["unexpected Example*Section InterpretationBox contents: ``", other]
			}]
		)
	];

	(* Examples that require the "ExampleFreeze" DocuTools style may have "outputs" that are not in the
		"Output" style, but "PrintTemporary" instead.  These are to become "Output". See ref/FormFunction
		for examples of such examples *)
	examplesGroup = ReplaceRepeated[
		examplesGroup,
		Cell[c___, "PrintTemporary", more___, "ExampleFreeze", rest___]
			:> Cell[c, "Output", more, "ExampleFreeze", rest, CellLabel->""]
	];

	BuildLog["-- Change SeeAlso Links"];
	seeAlsoGroup = If[
		(*MemberQ[{"Format","Character Name"}, type]*)
		True
		,
		seeAlsoGroup
		,
		seeAlsoGroup /. ButtonBox[c___] :> StyleBox[ButtonBox[c], FontFamily->"Verdana"]
	];

	(* Formatting note: f4f06b77
		implemented support for footer icons (WLDOCS-2151)
	*)
	(* moved related interpreter code to own func *)
	relatedInterpretersGroup = GetRelatedInterpreterGroup[expr, seeAlsoGroup, type, lang];
	expr = DeleteCases[
		expr,
		Cell[CellGroupData[{Cell[__, "RelatedInterpreterSection", ___], ___}, ___]],
		Infinity
	];

	(* Arrange the rearranged notebook *)
	(*expr = MapAt[Flatten@Insert[#,
		{objectNameCell, usageNameCell, notesGroup, examplesGroup, seeAlsoGroup}, 1
		] &, expr, 1];*)
	tutorialCollection = True;

	(* Add "(interpretation type)" to the top of ref/interpreter/* pages *)
	(* Also gray-out doublequotes *)
	If[
		MemberQ[
			{
				"Interpreter","Device Connection","Service Connection", "Indicator", "Method",
				"Classifier", "Predictor", "Net Encoder", "Net Decoder", "External Evaluation System",
				"Entity", "Compiled Type", "Database Connection", "Data Structure", "Text Content",
				"Remote Batch Computation Provider", "Question Interface", "Resource Object", "ResourceObject",
				"Featureset Page"
			},
			type
		]
		,
		(* Formatting note: eb5d3b5d
			fix bug 346010 (parenthetical in title for MLModel pages)
		*)
		(* remove existing parenthetical in source nb *)
		If[StringMatchQ[type, "Classifier" | "Predictor"],
			objectNameCell = Replace[
				objectNameCell,
				s_String /; StringMatchQ[s, "*(*)*"]
					:> StringTrim @ StringReplace[s, "("~~Shortest[__]~~")" -> ""],
				{2}
			]
		];

		(* Formatting note: 2bbd7979
			fix bug 346962 (external evaluation system subtitle)
		*)
		If[StringMatchQ[type, "External Evaluation System"],
			objectNameCell = DeleteCases[
				objectNameCell,
				StyleBox[__, "FilenameExtension"],
				Infinity
			]
		];

		If[MemberQ[underDevelopmentCell, "UnderDevelopment", Infinity],
			With[{cell = underDevelopmentCell},
				objectNameCell = Append[
					ReplaceAll[
						objectNameCell,
						s_String /; StringMatchQ[s, objectName~~Whitespace...] :> If[
							MatchQ[objectNameCell, {Cell[_String,__]}],
							Identity,
							Cell
						][
							TextData[Flatten[{
								Cell["\"", "InterpreterTitleQuotes"],
								StringReplace[objectName, "\""->""],
								Cell["\"", "InterpreterTitleQuotes"],
								" ",
								Switch[type,
									"Interpreter",
										Cell[Localized["InterpreterInterpretationType", lang], "InterpreterSubtitle"],
									"Service Connection",
										Cell[Localized["ServiceSubtitle", lang], "ServiceSubtitle"],
									"Device Connection",
										{},
									"Indicator",
										Cell[Localized["IndicatorSubtitle", lang], "ServiceSubtitle"],
									"Method",
										Cell[Localized["MethodSubtitle", lang], "ServiceSubtitle"],
									"Entity",
										Cell[Localized["EntitySubtitle", lang], "InterpreterSubtitle"],
									"Compiled Type",
										Cell[Localized["CompiledTypeSubtitle", lang], "InterpreterSubtitle"],
									"Text Content",
										Cell[Localized["TextContentSubtitle", lang], "InterpreterSubtitle"],
									_,
										""
								]
							}]]
						]
					],
					ReplaceAll[cell, "UnderDevelopment" -> "DeviceAbstract"]
				]
			]
			,
			objectNameCell = ReplaceAll[
				objectNameCell,
				s_String /; StringMatchQ[s, objectName~~Whitespace...] :> If[
					MatchQ[objectNameCell, {Cell[_String,__]}],
					Identity,
					Cell
				][
					(* additional special handling for Entity pages *)
					TextData[Flatten[{
						If[MatchQ[type, "Entity"],
							Nothing,
							Cell["\"", "InterpreterTitleQuotes"]
						],
						(* Formatting note: 667d3051
							fix bug 365141 (and other unreported)
						*)
						If[MatchQ[type, "Entity"],
							StyleBox[#, FontColor->RGBColor[0.396078431372549, 0.20784313725490197, 0.06274509803921569]]
							,
							#
						]& @ StringReplace[objectName, "\"" -> ""]
						,
						If[MatchQ[type, "Entity"],
							Nothing,
							Cell["\"", "InterpreterTitleQuotes"]
						],
						" ",
						Switch[type,
							"Interpreter",
								Cell[Localized["InterpreterInterpretationType", lang], "InterpreterSubtitle"],
							"Service Connection",
								Cell[Localized["ServiceSubtitle", lang], "ServiceSubtitle"],
							"Device Connection",
								{},
							"Indicator",
								Cell[Localized["IndicatorSubtitle", lang], "ServiceSubtitle"],
							"Method",
								Switch[uri,
									(* Formatting note: 21332ce6: fix bug 415585 *)
									"ref/method/MOSEK" | "ref/method/Gurobi" | "ref/method/Xpress",
										Cell[Localized["OSMethodSubtitle", lang], "ServiceSubtitle"],
										(* Formatting note: 4821dd1c: fix bug 408942 *)
									("ref/method/" <> #) & /@ Alternatives[
										"BalloonEmbedding", "BipartiteEmbedding", "CircularMultipartiteEmbedding",
										"DiscreteSpiralEmbedding", "GravityEmbedding", "GridEmbedding", "HighDimensionalEmbedding",
										"LayeredDigraphEmbedding", "LayeredEmbedding", "LinearEmbedding",
										"MultipartiteEmbedding", "PlanarEmbedding", "RadialEmbedding",
										"SpectralEmbedding", "SphericalEmbedding", "SpiralEmbedding",
										"SpringElectricalEmbedding", "SpringEmbedding", "StarEmbedding", "TutteEmbedding"
									],
										Cell[Localized["GLMethodSubtitle", lang], "ServiceSubtitle"],
									_,
										Cell[Localized["MethodSubtitle", lang], "ServiceSubtitle"]
								],
							"Entity",
								{},
							"Compiled Type",
								Cell[Localized["CompiledTypeSubtitle", lang], "InterpreterSubtitle"],
							"Classifier",
								Cell[Localized["ClassifierSubtitle", lang], "ServiceSubtitle"],
							"Predictor",
								Cell[Localized["PredictorSubtitle", lang], "ServiceSubtitle"],
							"Net Encoder",
								Cell[Localized["NetEncoderSubtitle", lang], "ServiceSubtitle"],
							"Net Decoder",
								(* Formatting note: 2bbd7979
									fix bug 346962 (external evaluation system subtitle)
								*)
								Cell[Localized["NetDecoderSubtitle", lang], "ServiceSubtitle"],
							"External Evaluation System",
								Cell[Localized["ExternalEvaluationSystemSubtitle", lang], "ServiceSubtitle"],
							"Database Connection",
								(*Cell[Localized["DatabaseConnectionSubtitle", lang], "ServiceSubtitle"]*)
								{},
							"Data Structure",
								Cell[Localized["DataStructureSubtitle", lang], "ServiceSubtitle"],
							"Text Content",
								(* Formatting note: 19125312: fix bug 398947 *)
								Cell[Localized["TextContentSubtitle", lang], "ServiceSubtitle"],
							"Remote Batch Computation Provider",
								Cell[Localized["BatchComputationProviderSubtitle", lang], "ServiceSubtitle"],
							"Question Interface",
								Cell[Localized["QuestionInterfaceSubtitle", lang], "ServiceSubtitle"],
							"ResourceObject" | "Resource Object",
								Cell[Localized["ResourceObjectSubtitle", lang], "ServiceSubtitle"],
							(* default *)
							_,
								""
						]
					}]]
				]
			]
		]
	];

	(* Formatting note: fc155ee3
		handles WLDOCS-2273, WLDOCS-2663, WLDOCS-2553
	*)
	With[{blockchains = Alternatives["Bitcoin", "ARK", "Ethereum"]},
		If[StringMatchQ[uri, "*"~~blockchains] && MatchQ[type, "Symbol"],
			{objectNameCell, blockchainabstract} = HandleBlockchainRefs[
				objectNameCell,
				objectName,
				First[StringCases[uri, blockchains]]
			];
		]
	];

	(* Similar to above, but not quite the same (WLDOCS-1969) *)
	If[MemberQ[{"MathLink C Function", "LibraryLink C Function"}, type],
		objectNameCell = ReplaceAll[
			objectNameCell,
			s_String /; StringMatchQ[s,objectName~~Whitespace...]
				:> If[
					MatchQ[objectNameCell, {Cell[_String,__]}],
					Identity,
					Cell
				][
					TextData[Flatten[{
						StringReplace[objectName, "()" -> ""],
						" ",
						Switch[type,
							"MathLink C Function" | "LibraryLink C Function",
								Cell[Localized["CFunctionSubtitle", lang], "ServiceSubtitle"]
						]
					}]]
				]
		]
	];

	(* add icon to menu item title cell *)
	If[MemberQ[{"Menu Item"}, type],
		With[{icon = Image[Import[DocsAsset["Images", "MenuItemTitleIcon2x.png"]], Magnification -> 0.5]},
			(* Formatting note: 057a33d5
				fix bugs 389048 and another piece of 382084 as well
			*)
			objectNameCell = Replace[
				ReplaceAll[
					objectNameCell,
					TextData[td_] :> TextData[Join[{Cell[BoxData[ToBoxes[Deploy@icon]]], " "}, td]]
				],
				Cell[c__] :> Cell[c, CacheGraphics -> False],
				{1}
			];
		]
	];

	(* Transform usage cell in Format pages to new inline-bulleted style *)
	If[MemberQ[{"Format"}, type],
		Module[{
			usagetext,
			sep = StyleBox["\[FilledVerySmallSquare] ","FormatUsageSeparator"],
			col1,
			col2
		},
			usagetext = Cases[usageNameCell, Cell[TextData[c_], ___] :> c, Infinity];

			If[MatchQ[usagetext, {}],
				usagetext = Cases[{usageNameCell}, Cell[s_String, ___] :> s, Infinity]
			];
			(* Formatting note: 3b4e01e9
				reworked format bullets to fix bug 284102
			*)
			usagetext = ReplaceRepeated[
				usagetext,
				{some___, first___String, StyleBox[s_, "RebrandingTerm", ___], second___String, Shortest[rest___]}
					:> {some, StringJoin[first, s, second], rest}
			];

			usagetext = ReplaceRepeated[usagetext,StyleBox["\n",__]:>"\n"];

			usagetext = Module[{
				brokentext = Append[
					Prepend[
						Flatten @ ReplaceAll[
							usagetext,
							s_String /; !StringFreeQ[s,"\n"]
								:> StringSplit[s, "\n" -> StyleBox["BREAK"]]
						],
						StyleBox["BREAK"]
					],
					StyleBox["BREAK"]
				]
			},
				DeleteCases[
					ReplaceRepeated[
						brokentext,
						{
							Shortest[some___],
							StyleBox["BREAK"],
							Shortest[cont__] /; FreeQ[{cont},StyleBox["BREAK",Infinity]],
							StyleBox["BREAK"],
							Shortest[rest___]
						} :> {some, RowBox[{cont}], StyleBox["BREAK"], rest}
					],
					StyleBox["BREAK"],
					Infinity
				]
			];

			(* Formatting note: df09a203
				should fix bug 284102
			*)
			usagetext = DeleteCases[usagetext, " " | ""];
			usagetext = Multicolumn[usagetext, 2][[1]];
			usagetext = ReplaceAll[usagetext, Null -> ""];

			{col1, col2} = Transpose[
				Map[
					Cell[
						TextData[If[MatchQ[#,""|" "|Null], #, Flatten[{sep, #}]]],
						"FormatUsage",
						ParagraphIndent -> -10
					]&,
					usagetext,
					{2}
				]
			];

			col1 = List /@ col1;
			col2 = List /@ col2;

			If[!MatchQ[Flatten[DeleteCases[usagetext, Null | "Null", Infinity]], {} | {""}],
				usageNameCell = Cell[CellGroupData[
					{
						Cell[Localized["GuideBackgroundSectionText", lang], "FormatBackground"],
						Cell[
							BoxData[GridBox[
								{{
									GridBox[col1, GridBoxAlignment->{"Columns"->{{Left}},"Rows"->{{Top}}},
									GridBoxItemSize->{"Columns"->{Scaled[1.]}},
									GridBoxSpacings->{"Rows"->{{0.6}}}],
									GridBox[col2, GridBoxAlignment->{"Columns"->{{Left}},"Rows"->{{Top}}},
									GridBoxItemSize->{"Columns"->{Scaled[1.]}},
									GridBoxSpacings->{"Rows"->{{0.6}}}]
								}},
								GridBoxAlignment->{"Columns"->{{Left}},"Rows"->{{Top}}},
								GridBoxItemSize->{"Columns"->{Scaled[0.5],{Scaled[0.5]}}},
								GridBoxSpacings->{"Columns"->{{1.3}}}
							]],
							"FormatUsage",
							CellMargins -> {{24, 24}, {24, 18}}
						]
					},
					Closed
				]]
				,
				usageNameCell = Cell[
					Localized["GuideBackgroundSectionText", lang],
					"FormatBackground"
				]
			];
		]
	];

	hasexamplesQ = FreeQ[
		examplesGroup,
		Cell[s__, "PrimaryExamplesSection", ___] /; MemberQ[{s}, Cell["(0)", "ExampleCount"], Infinity]
	];
	hasnotesQ = !MatchQ[notesGroup, {}];

	(* Formatting note: 114b745a
		fixed bug 367364, also added localization keys for sub-ref see alsos
	*)
	(* final transform of footer groups *)
	seeAlsoGroup = {AddAttractorIcons[seeAlsoGroup, lang]};
	(* make see alsos dynamic for WFR content inclusion, but only for System symbols *)
	If[StringMatchQ[uri, "ref/*"],
		seeAlsoGroup = {AddWFRCrosslinks[seeAlsoGroup, uri, lang]}
	];

	moreAboutGroup = {AddAttractorIcons[moreAboutGroup, lang]};
	tutorialsGroup = {AddAttractorIcons[tutorialsGroup, lang]};
	workflowsGroup = {AddAttractorIcons[workflowsGroup, lang]};
	relatedInterpretersGroup = {AddAttractorIcons[relatedInterpretersGroup, lang]};
	entityGroup = {AddAttractorIcons[entityGroup, lang]};
	relatedLinksGroup = {AddAttractorIcons[relatedLinksGroup, lang]};

	(* Formatting note: ec26144a
		fixed 361632 and other related-see-also bugs/issues
	*)
	(* related Interpreters are now part of the see also group *)
	relatedInterpretersGroup = {};

	formatthumbnails = Replace[formatthumbnails, "" -> {}, {1}];
	devicethumbnails = Replace[devicethumbnails, "" -> {}, {1}];
	classifierthumbnails = Replace[classifierthumbnails, "" -> {}, {1}];
	predictorthumbnails = Replace[predictorthumbnails, "" -> {}, {1}];
	externalevaluationsystemthumbnails = Replace[externalevaluationsystemthumbnails, "" -> {}, {1}];
	othertypethumbnails = Replace[othertypethumbnails, "" -> {}, {1}];

	nbOrder = Which[
		type === "Format",
			{
				underDevelopmentCell,
				objectNameCell,
				notesGroup,
				usageNameCell,
				formatthumbnails[[1]],
				importExportGroup,
				formatthumbnails[[2]],
				notebookInterfaceGroup,
				formatthumbnails[[3]],
				elementGroup,
				formatthumbnails[[4]],
				exportelementGroup,
				formatthumbnails[[5]],
				optionsGroup,
				formatthumbnails[[6]],
				pagedelimiter,
				examplesGroup, seeAlsoGroup, tutorialsGroup, moreAboutGroup, workflowsGroup, relatedLinksGroup
			},
		type === "Indicator",
			{
				underDevelopmentCell,
				objectNameCell,
				usageNameCell,
				elucidationGroup,
				notesGroup,
				If[!MatchQ[othertypethumbnails, ""|{}], othertypethumbnails, Nothing],
				indicatorExampleGroup,
				indicatorDescriptionGroup,
				indicatorFormulaGroup,
				seeAlsoGroup, tutorialsGroup,
				If[tutorialCollection === True, tutorialCollectionGroup, ""],
				moreAboutGroup, workflowsGroup, relatedLinksGroup
			},
		type === "Character Name",
			{
				underDevelopmentCell,
				characterNameCell,
				characterImageGrid,
				importExportGroup,
				notebookInterfaceGroup,
				elementGroup,
				optionsGroup,
				seeAlsoGroup, tutorialsGroup, moreAboutGroup, examplesGroup,
				workflowsGroup, relatedLinksGroup
			},
		type === "Interpreter" || type === "Menu Item",
			{
				underDevelopmentCell,
				objectNameCell,
				usageNameCell,
				elucidationGroup,
				notesGroup,
				If[!MatchQ[othertypethumbnails, ""|{}], othertypethumbnails, Nothing],
				If[TrueQ[hasnotesQ]&& !MatchQ[otherTypeGroup, {}], pagedelimiter, Nothing],
				otherTypeGroup,
				If[MatchQ[otherTypeGroup, {}], {}, pagedelimiter],
				fnEssayGroup,
				examplesGroup,
				relatedInterpretersGroup,
				seeAlsoGroup,
				tutorialsGroup,
				If[tutorialCollection===True, tutorialCollectionGroup, ""],
				moreAboutGroup, workflowsGroup, relatedLinksGroup
			},
		type === "Entity",
			{
				underDevelopmentCell,
				contextCell,
				objectNameCell,
				descriptionCell,
				usageNameCell,
				elucidationGroup,
				notesGroup,
				If[TrueQ[hasnotesQ], pagedelimiter, Nothing],
				otherTypeGroup,
				pagedelimiter,
				fnEssayGroup,
				examplesGroup,
				relatedInterpretersGroup,
				seeAlsoGroup,
				tutorialsGroup,
				If[tutorialCollection === True, tutorialCollectionGroup, ""],
				moreAboutGroup, workflowsGroup, relatedLinksGroup
			},
		type === "Symbol" && !MatchQ[notesthumbnails, ""],
			{
				underDevelopmentCell,
				contextCell,
				objectNameCell,
				blockchainabstract,
				usageNameCell,
				elucidationGroup,
				notesGroup,
				notesthumbnails,
				If[MemberQ[examplesGroup, Cell["(0)", "ExampleCount"], Infinity], Nothing, pagedelimiter],
				otherTypeGroup,
				fnEssayGroup,
				If[FreeQ[fnEssayGroup, Cell[__, "FunctionEssay", ___]](* no fn essay *), {}, pagedelimiter],
				examplesGroup,
				relatedInterpretersGroup,
				seeAlsoGroup,
				tutorialsGroup,
				If[tutorialCollection === True, tutorialCollectionGroup, ""],
				moreAboutGroup, workflowsGroup, relatedLinksGroup
			},
		!TrueQ[hasnotesQ] || MatchQ[notesthumbnails, ""],
			{
				underDevelopmentCell,
				contextCell,
				objectNameCell,
				blockchainabstract,
				usageNameCell,
				elucidationGroup,
				notesGroup,
				If[!MatchQ[othertypethumbnails, ""|{}], othertypethumbnails, Nothing],
				otherTypeGroup,
				If[MatchQ[otherTypeGroup, {}], {}, pagedelimiter],
				fnEssayGroup,
				If[FreeQ[fnEssayGroup, Cell[__, "FunctionEssay", ___]](* no fn essay *), {}, pagedelimiter],
				(*If[FreeQ[examplesGroup, "ExampleText" | "Input"](* no examples *), {}, pagedelimiter], *)
				examplesGroup,
				relatedInterpretersGroup,
				seeAlsoGroup,
				tutorialsGroup,
				If[tutorialCollection === True, tutorialCollectionGroup, ""],
				moreAboutGroup, workflowsGroup, relatedLinksGroup
			},
		True,
			{
				underDevelopmentCell,
				contextCell,
				objectNameCell,
				blockchainabstract,
				usageNameCell,
				elucidationGroup,
				notesGroup,
				If[!MatchQ[othertypethumbnails, ""|{}], othertypethumbnails, Nothing],
				pagedelimiter,
				otherTypeGroup,
				If[MatchQ[otherTypeGroup, {}], {}, pagedelimiter],
				fnEssayGroup,
				If[FreeQ[fnEssayGroup, Cell[__, "FunctionEssay", ___]](* no fn essay *), {}, pagedelimiter],
				(*If[FreeQ[examplesGroup, "ExampleText" | "Input"](* no examples *), {}, pagedelimiter], *)
				examplesGroup,
				relatedInterpretersGroup,
				seeAlsoGroup,
				tutorialsGroup,
				If[tutorialCollection === True, tutorialCollectionGroup, ""],
				moreAboutGroup, workflowsGroup, relatedLinksGroup
			}
	];

	expr = MapAt[
		Flatten @ Insert[#, nbOrder, 1] &,
		expr,
		1
	];

	(* add special Format/Interpreter notebook styles *)
	expr = If[MemberQ[{"Format", "Embedding Format", "External Evaluation System"}, type], expr/.Cell[a__,"Usage",b___]:>Cell[a,"FormatUsage",b], expr];
	expr = If[MemberQ[{"Interpreter", "Classifier", "Predictor", "Text Content"}, type], expr/.Cell[a__,"Usage",b___]:>Cell[a,"InterpreterUsage",b], expr];
	expr = If[MemberQ[{"Format", "Embedding Format", "External Evaluation System"}, type], expr/.Cell[a__,"Notes",b___]:>Cell[a,"FormatNotes",b], expr];
	expr = If[MemberQ[{"Interpreter"}, type], expr/.Cell[a__,"Notes",b___]:>Cell[a,"InterpreterNotes",b], expr];
	expr = If[MemberQ[{"Entity"}, type],
		expr /. Cell[a__, "Usage", b___] :>
			Cell[
				ReplaceAll[
					a,
					StyleBox[TemplateBox[tb__, "Entity", rest___], "Input", sty__]
						:> StyleBox[TemplateBox[tb, "InertEntity", rest], sty]
				],
				"EntityUsage",
				b
			]
		,
		expr
	];

	(* autogenerate Entity content *)
	If[MemberQ[{"Entity"}, type],
		expr = GenerateEntityPageContent[
			expr,
			If[StringMatchQ[objectName, "\""~~__~~"\""],
				ToExpression[objectName],
				objectName
			],
			entitythumbnails,
			lang
		];

		(* add related interpreter link if applicable *)
		Module[{hasRelatedInterpreterQ, pos, cell, inturi},
			inturi = ToPacletURI @ StringReplace[uri, "/entity/" -> "/interpreter/"];

			(* Formatting note: b954b666
				fix bug 368671 and incorrect coloring of ref/c/ see alsos
			*)
			hasRelatedInterpreterQ = !MatchQ[
				GetSourceInformation[inturi, lang],
				None | "None"
			];

			pos = Position[expr, Cell[s_String, ___, "EntityAbstract", ___]];

			If[hasRelatedInterpreterQ && pos =!= {},
				With[{lclstr = Localized["EntityPageRelatedInterpreterText", lang]},
					cell = expr[[Sequence@@pos[[1]]]];

					expr = ReplacePart[
						expr,
						pos -> Replace[
							cell,
							Cell[s_String, ___, "EntityAbstract", ___] :> Cell[
								BoxData[GridBox[{{
									Cell[s, "EntityAbstract"],
									ItemBox["", ItemSize -> Fit],
									Cell[
										BoxData[TemplateBox[
											{StyleBox[lclstr, "EntityRelatedInterpreter"], inturi},
											"GrayLink",
											BaseStyle -> "EntityRelatedInterpreter"
										]],
										"EntityRelatedInterpreter"
									]
								}}]],
								"EntityAbstract"
							]
						]
					]
				]
			]
		]
	];

	(* insert device page thumbnail cells *)
	expr = If[StringMatchQ[uri, "*ref/device/*"],
		Module[{thumbs, thumbsects, newexpr = expr, pos},
			thumbs = devicethumbnails;

			thumbsects = Map[
				Localized["DevicePage"<>#<>"SectionText", lang]&,
				{"Discovery", "Open", "Config", "Properties", "Read", "Write", "Execute", "Close"}
			];

			Do[
				If[!MatchQ[thumbs[[i]], "" | {}],
					pos = Position[
						newexpr,
						Cell[CellGroupData[{Cell[thumbsects[[i]], ___, "DeviceSubsection", ___], ___}, ___]]
					];
					If[pos =!= {},
						newexpr = Insert[newexpr, thumbs[[i]], pos + {{0,1}}]
					]
				],
				{i, 8}
			];

			newexpr
		],
		expr
	];

	(* insert classifier/predictor page thumbnail cells *)
	expr = If[StringMatchQ[uri, "*ref/classifier/*" | "*ref/predictor/*"],
		Module[{thumbs, thumbsects, newexpr = expr, pos, sectstyle},
			{thumbs, sectstyle} = If[StringMatchQ[uri, "*ref/classifier/*"],
				{classifierthumbnails, "ClassifierSection"}
				,
				{predictorthumbnails, "PredictorSection"}
			];
			thumbsects = Map[
				Localized["ClassifierPredictorPage"<>#<>"SectionText", lang]&,
				{"Classes", "Details"}
			];

			Do[
				If[!MatchQ[thumbs[[i]], "" | {}],
					pos = Position[
						newexpr,
						Cell[CellGroupData[{Cell[thumbsects[[i]], ___, sectstyle, ___], ___}, ___]]
					];
					If[pos =!= {},
						newexpr = Insert[newexpr, thumbs[[i]], pos + {{0,1}}]
					]
				],
				{i, 2}
			];

			newexpr
		],
		expr
	];

	(* insert External Evaluation System thumbnails *)
	expr = If[StringMatchQ[uri, "*ref/externalevaluationsystem/*"],
		Module[{thumbs, thumbsects, newexpr = expr, pos},
			thumbs = externalevaluationsystemthumbnails;
			thumbsects = Map[
				Localized["ExternalEvaluationSystem"<>#<>"SectionText", lang]&,
				{"Notes","ExternalEvaluateUsage","DataTypes","UsageNotes"}
			];

			Do[
				If[!MatchQ[thumbs[[i]], "" | {}],
					pos = Position[
						newexpr,
						If[i =!= 1,
							Cell[CellGroupData[{Cell[thumbsects[[i]], ___, "ExternalEvaluateSystemSection", ___], ___}, ___]]
							,
							Cell[CellGroupData[{Cell[__, "NotesSection", ___], ___},___]]
						]
					];
					If[pos =!= {}, newexpr = Insert[newexpr, thumbs[[i]], pos + {{0,1}}]]
				],
				{i, 4}
			];

			newexpr
		],
		expr
	];

	(* add related entity link to interpreter pages if applicable *)
	If[MemberQ[{"Interpreter"}, type],
		Module[{hasRelatedEntityQ, pos, cell, inturi},
			inturi = ToPacletURI @ StringReplace[uri, "/interpreter/" -> "/entity/"];

			hasRelatedEntityQ = !MatchQ[
				GetSourceInformation[inturi, lang],
				None | "None"
			];

			pos = Position[expr, Cell[s_String, ___, "InterpreterUsage", ___]];

			If[hasRelatedEntityQ && pos =!= {},
				With[{lclstr = Localized["InterpreterPageRelatedEntityText", lang]},
					cell = expr[[Sequence@@pos[[1]]]];

					expr = ReplacePart[
						expr,
						pos -> Replace[
							cell,
							Cell[s_String, ___, "InterpreterUsage", ___] :> Cell[
								BoxData[GridBox[{{
									Cell[s, "InterpreterUsage"],
									ItemBox["", ItemSize -> Fit],
									Cell[
										BoxData[TemplateBox[
											{StyleBox[lclstr, "EntityRelatedInterpreter"], inturi},
											"GrayLink",
											BaseStyle -> "EntityRelatedInterpreter"
										]],
										"EntityRelatedInterpreter"
									]
								}}]],
								"InterpreterUsage"
							]
						]
					]
				]
			]
		]
	];

	(* Remove SyntaxTemplateExclusion *)
	expr = expr //. {
		StyleBox[foo_, "SyntaxTemplateExclusion", ___] :> foo,
		Cell[foo_, "InlineFormula", "TemplateExclusion", o___] :> Cell[foo, "InlineFormula", o]
	};

	(* Add ObsolescenceNote *)
	warnicon = Image[
		Import[DocsAsset["Images", "ObsoleteIcon2x.png"], "PNG"],
		Magnification -> 0.5
	];

	obsolescenceNote = If[obsolescenceNote === {},
		obsolescenceNote,
		{Cell[
			BoxData[GridBox[{{
				Cell[
					BoxData[ToBoxes@Deploy[warnicon]],
					Background -> GrayLevel[0.941176]
				],
				Cell[obsolescenceNote[[1, 1]], "ObsolescenceNote", CellFrame -> 0]
			}}]],
			Sequence @@ Rest[obsolescenceNote[[1]]],
			CacheGraphics -> False
		]}
	];
	expr = expr /.
		Cell[c_, sty_String /; StringMatchQ[sty, "ObjectName" | "ObjectNameSmall" | "ObjectNameAlt"], o___]
			:> Cell[CellGroupData[
				Flatten[
					{Cell[c, sty, o], obsolescenceNote},
					1
				],
				Open
			]];

	(* Add "obsolescence note" for *Data pages replaced by entities *)
	dataObsolescenceNote = If[dataObsolescenceNote === {},
		dataObsolescenceNote,
		{Cell[
			BoxData[GridBox[{{
				ItemBox[
					TagBox[
						DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation", "Miscellaneous"}, "DataPage-banner.png"]]],
						Deploy,
						DefaultBaseStyle -> "Deploy"
					],
					Alignment -> Center
				],
				ItemBox[Cell[dataObsolescenceNote[[1, 1]],"DataPageEntityNote",CellFrame->0], Alignment->Center]
			}}]],
			Sequence @@ Rest[dataObsolescenceNote[[1]]],
			CacheGraphics -> False
		]}
	];
	expr = expr /.
		Cell[c_, sty_String/;StringMatchQ[sty, "ObjectName"|"ObjectNameSmall"|"ObjectNameAlt"], o___]
			:> Cell[CellGroupData[
				Flatten[{
					Cell[c, sty, o],
					dataObsolescenceNote
				}, 1],
				Open
			]];

	(* Add WFP-specific note to relevant service pages *)
	If[StringMatchQ[uri, "*/service/BloombergTerminal" | "*/service/BloombergDataLicense" | "*/service/Reuters"],
		Module[{pos, wfpicon},
			wfpicon = Import[DocsAsset["InternalOnlineImage", "WFPIcon.m"]];

			pos = Position[expr, Cell[__, "ServiceAbstract", ___], Infinity, 1];

			expr = Insert[
				expr,
				Cell[
					TextData[{
						Cell[BoxData[TagBox[wfpicon, Deploy, DefaultBaseStyle->"Deploy"]]],
						"  This service connection requires ",
						Cell[BoxData[TemplateBox[
							{"Wolfram Finance Platform \[RightGuillemet]", "http://www.wolfram.com/finance-platform/"},
							"WebLink",
							BaseStyle -> {"PlatformAvailabilityNote"}
						]]]
					}],
					"PlatformAvailabilityNote",
					CacheGraphics -> False,
					CellFrame -> Dynamic[If[$ProgramName =!= "WolframFinancePlatform", Inherited, None]],
					Background -> Dynamic[If[$ProgramName =!= "WolframFinancePlatform", Inherited, None]],
					CellMargins -> Dynamic[If[$ProgramName =!= "WolframFinancePlatform", Inherited, {{0, 0}, {-2, -2}}]],
					CellOpen -> Dynamic[$ProgramName =!= "WolframFinancePlatform"]
				],
				pos
			]
		]
	];

	(* Add Awaiting Review Note *)
	awaitingReviewNote = If[awaitingReviewNote === {},
		awaitingReviewNote,
		{Cell[
			BoxData[GridBox[{{
				Cell[BoxData[ToBoxes@Deploy[warnicon]], Background->GrayLevel[0.941176]],
				Cell[awaitingReviewNote[[1, 1]],"AwaitingReviewNote",CellFrame->0]
			}}]],
			Sequence @@ Rest[awaitingReviewNote[[1]]],
			CacheGraphics -> False
		]}
	];
	expr = expr /.
		Cell[c_, sty_String /; StringMatchQ[sty, "ObjectName"|"ObjectNameSmall"|"ObjectNameAlt"], o___]
			:> Cell[CellGroupData[
				Flatten[{
					Cell[c, sty, o],
					awaitingReviewNote
				}],
				Open
			]];

	If[(*(("PacletName" /. info) === "Mathematica")
		&&*) (Length@Cases[expr, Cell[c_, ___, "ObjectName"|"ObjectNameSmall"|"ObjectNameAlt", ___]:>c, Infinity] > 0),
		BuildLog["-- Add ObjectName Graphic"];\

		expr = AddObjectNameGraphic[expr, info, opts];
	];

	BuildLog["-- Close groups"];

	(*  Close groups and set Dynamic *)
	(* 4/26/14: Sadly, we have to iterate through closedGroups individually, because it seems that
		since these are nested styles, the copies used to replace are being made before the deeper
		replacements occur, so only the highest-level style -- ExampleSection -- is being Closed *)

	closedGroups = Which[
		MemberQ[
			{"Format", "Interpreter", "Service Connection", "Device Connection", "Embedding Format", "Method", "External Evaluation System"},
			type
		],
			"ExampleSection" | "ExampleSubsection" | "ExampleSubsubsection" | "ImportExportSection" | "NotebookInterfaceSection" | "OptionsSection"
			| "InterpreterSection" | "EmbeddingFormatSection" | "ServiceSubsection" | "DeviceSubsection" | "MethodSection" | "ExternalEvaluateSystemSection"(*|"TutorialsSection"*),
		True,
			"ExampleSection" | "ExampleSubsection" | "ExampleSubsubsection"
	];

	CloseGroups[nbexpr_Notebook, style_String] :=
		nbexpr /. {
			Cell[
				CellGroupData[
					{
						Cell[con_, sty_String /; StringMatchQ[sty, style], o___],
						(*Cell[con_, sty_String /; StringMatchQ[sty, "ExampleSection"|"ExampleSubsection"], o___],*)
						cs___
					}, ___
				],
				g___
			] :> Cell[
				CellGroupData[
					{
						Cell[con, sty, o],
						cs
					},
					If[(Length[Cases[con, Localized["BasicExamplesSectionText", lang], Infinity]] > 0),
						Open,
						Closed
					]
				],
				g
			],
			Cell[
				CellGroupData[
					{
						Cell[con_, "NotesSection", o___],
						cs___
					},
					___
				],
				g___
			] :> Cell[
				CellGroupData[
					{
						Cell[con, "NotesSection", o],
						cs
					},
					(* Details section to be open by default on Message pages: brianv--05/12/16*)
					If[MatchQ[type, "Message"],
						Open,
						Closed
					]
				],
				g
			]
		};

	expr = Fold[CloseGroups, expr, closedGroups];

	(* make sure Input and delimiter groups are open *)
	expr = expr /.
		Cell[
			CellGroupData[{Cell[a__, sty_String /; StringMatchQ[sty, "Input"], b___], c___}, Closed],
			e___
		] :> Cell[CellGroupData[{Cell[a, sty, b], c}, Open], e];

	expr = expr /.
		Cell[
			CellGroupData[{Cell[a__, sty_String /; StringMatchQ[sty, "ExampleDelimiter"],b___], c___}, Closed],
			e___
		] :> Cell[CellGroupData[{Cell[a,sty,b], c}, Open], e];

	(* Delete special conversion cells *)
	expr = expr /.
		Cell[
			CellGroupData[{Cell[a__, sty_String /; StringMatchQ[sty, "ExampleDelimiter"],b___], c___}, Closed],
			e___
		] :> Cell[CellGroupData[{Cell[a,sty,b], c}, Open], e];

	expr = DeleteCases[expr, Cell[__, "SuppressedMaterial", ___], Infinity];

	(*  Change headings to have Open/close buttons *)
	expr = expr /.
		Cell[
			con_,
			sty:Alternatives @@ ("CapitalizeHeadingStyles" /. {opts} /. Options[MakeNotebook]),
			o___
		] :> Cell[TextData @ WrapInOpenCloseButton@CapitalizeString[con], sty, o];
 
	(*  Special hack for ref/Underlined failure at 100% magnification in Windows. *)
	expr = expr /.
		Cell[
			BoxData[StyleBox[
				"\<\"Text\"\>",
				StripOnInput -> False,
				FontVariations -> {"Underline" -> True}
			]],
			"Output",
			op___
		] :> Cell[
			BoxData[StyleBox[
				"\<\"Text\"\>",
				StripOnInput -> False,
				FontVariations -> {"Underline" -> True}
			]],
			"Output",
			Magnification -> 1.1,
			op
		];

	(* Final custom pass to convert specific styles *)
	expr = If[MatchQ[type, "MathLink C Function" | "LibraryLink C Function"],
		CFunctionStyleConvert[expr]
		,
		expr
	];

	(* Add header cells back in *)
	expr = MapAt[
		Flatten @ Insert[#, {headerCells}, 1] &,
		expr,
		1
	];

	(* V11.1 Redesign: Need to do dynamic hookup of group openers to tagging rules to use custom openers *)
	onetimesections = {
		"FunctionEssaySection", "FormatBackground", "ImportExportSection",
		"NotebookInterfaceSection", "ElementsSection", "ExportElementsSection",
		"OptionsSection", "IndicatorExampleSection"
	};

	groupopenersections = Join[
		{"NotesSection","PrimaryExamplesSection"},
		onetimesections,
		{
			"ExampleSection", "ExampleSubsection", "ExampleSubsubsection", "InterpreterSection",
			"DeviceSubsection","ServiceSubsection", "EmbeddingFormatSection", "MethodSection",
			"ExternalEvaluateSystemSection", "EntitySection", "CompiledTypeSection", "BlockchainSection",
			"DatabaseConnectionSection", "ClassifierSection", "PredictorSection",
			"BatchComputationProviderSection", "QuestionInterfaceSection", "ResourceObjectSection"
		}
	];
	expr = DynamicGroupOpeners[expr, groupopenersections, onetimesections];

	BuildLog[
		"-- CellChangeTimes still exist? ",
		ToString @ Cases[
			expr,
			Cell[___, System`CellChangeTimes~(Rule|RuleDelayed)~__, ___],
			Infinity
		]
	];

	BuildLog["** Leaving MakeNotebook (Symbol) **"];

	expr
];




HandleBlockchainRefs[cell_, name_String, type_String]:=
  Module[{objectNameCell = cell, abstract, objectName = name},
    (* remove existing parenthetical *)
    objectNameCell = Replace[objectNameCell, s_String/;StringMatchQ[s, "*(*)*"] :> StringTrim@StringReplace[s, "("~~Shortest[__]~~")" -> ""],{2}];
    
    objectNameCell = ReplaceAll[objectNameCell, s_String/;StringMatchQ[s,objectName~~Whitespace...] :> 
      If[MatchQ[objectNameCell, {Cell[_String,__]}], Identity, Cell][TextData[
       Flatten[{objectName,
        "\[MediumSpace]\[MediumSpace]", 
          Cell["(for "<>type<>")", "ServiceSubtitle", FontSize->32, FontWeight->"Plain", FontColor->GrayLevel[.33]]}]
      ]]
    ];
    
    abstract = Cell["$BlockchainBase = \""<>type<>"\"", "ServiceAbstract", FontSize->(Inherited-2), CellMargins->{{Inherited, Inherited}, {10, Inherited}}];
    
    {objectNameCell, abstract}
        
  ];


Options[DynamicGroupOpeners] = {"Openers" -> True};


DynamicGroupOpeners[nbexpr_Notebook, styles_List, onetime_List, opts:OptionsPattern[]]:=
    Module[{cells, counter=0, newnb=nbexpr,rules, onetimesections = onetime, openers = OptionValue["Openers"]},
     With[{
       openicon = Get[DocsAsset["Graphics:Opener", "SectionOpenerOpenOff.m"]],
       closedicon = Get[DocsAsset["Graphics:Opener", "SectionOpenerClosedOff.m"]]
      },
     Switch[counter=0; #,
     (* Cells that appear only ONCE in a notebook *)
     "NotesSection"|"PrimaryExamplesSection",
     newnb=ReplaceAll[newnb, Cell[CellGroupData[{s:Cell[__,#,___],r___},___]]:>
       Cell[CellGroupData[{s, r}, Dynamic[CurrentValue[EvaluationNotebook[], {TaggingRules, "Openers", #},Closed]]]]],
     Alternatives@@onetimesections,
     newnb = ReplaceAll[newnb, Cell[CellGroupData[cells:{Cell[td_,s___,#,rest___],r___},___]]:>
         Cell[CellGroupData[If[TrueQ[openers], {Cell[TextData[Flatten[Join[{Cell[BoxData[DynamicBox[ToBoxes[If[
           MatchQ[CurrentValue[EvaluationNotebook[], {TaggingRules, "Openers", #}],Open|True],
           openicon,
           closedicon]]]]],Cell[BoxData[ToBoxes[Spacer[1]]]]},{td}]]],s,#,rest], r}, cells], 
           Dynamic[CurrentValue[EvaluationNotebook[], {TaggingRules, "Openers", #},Closed]]]]],
     _,
     newnb = ReplaceAll[newnb, Cell[s_String, some___, #, rest___] :> Cell[TextData[s],some,#,rest]];
     cells = Cases[newnb, Cell[CellGroupData[{Cell[TextData[_],___,#,___],___},___]], Infinity];
     Which[MatchQ[cells,{}], newnb,
      (Length[cells]===1)&&!StringMatchQ[#,"Example*ection"|
        (* There's only one predictor, with only one section in it, so override to let 
           thumbnails work. Same below. *)
        "PredictorSection"],
       newnb = ReplaceAll[newnb, Cell[CellGroupData[cells:{Cell[TextData[td_],s___,#,rest___],r___},___]]:>
         Cell[CellGroupData[If[TrueQ[openers], {Cell[TextData[Flatten[Join[{Cell[BoxData[DynamicBox[ToBoxes[If[
           MatchQ[CurrentValue[EvaluationNotebook[], {TaggingRules, "Openers", #}],Open|True],
           openicon,
           closedicon]]]]],Cell[BoxData[ToBoxes[Spacer[1]]]]},{td}]]],s,#,rest], r}, cells], 
           Dynamic[CurrentValue[EvaluationNotebook[], {TaggingRules, "Openers", #},Closed]]]]],
      Length[cells]>1 || StringMatchQ[#,"Example*ection"|"PredictorSection"],
        cells = Position[newnb, Cell[CellGroupData[{Cell[TextData[_],___,#,___],___},___]]];
        rules = Function[cell,With[{count=ToString[counter++], pos=cell},
          If[!MatchQ[pos,{}],Join[pos,{1,2}]:> 
          Hold[CurrentValue[EvaluationNotebook[], {TaggingRules, "Openers", #, count},Closed]]]]]/@cells;
        rules = DeleteCases[rules, Null];
        newnb=ReplacePart[newnb, rules]
        ]]&/@styles;

    ];

    (* Add in opener graphics to cells missing them and attach them to their tagging rules *)
    If[TrueQ[openers],
    newnb = With[{
      openicon = Get[DocsAsset["Graphics:Opener", "SectionOpenerOpenOff.m"]],
      closedicon = Get[DocsAsset["Graphics:Opener", "SectionOpenerClosedOff.m"]]
    },
        Fold[
        ReplaceAll[#, CellGroupData[{c:Cell[TextData[td_]/;FreeQ[td, DynamicBox[___]],rest___],l___}, 
    q:Hold[cv:CurrentValue[EvaluationNotebook[],{TaggingRules, "Openers", s_/;StringMatchQ[s,#2], n_String/;StringMatchQ[n,DigitCharacter..]},___]]]:>
    CellGroupData[{Cell[TextData[Flatten[Join[{Cell[BoxData[DynamicBox[ToBoxes[If[
           MatchQ[cv,Open|True],
           openicon,
           closedicon]]]]],Cell[BoxData[ToBoxes[Spacer[1]]]]},{td}]]],rest],l},Dynamic[cv]]]&, newnb, 
           Complement[styles,Flatten[Join[{"NotesSection","PrimaryExamplesSection"},onetimesections]]]]]];
           
    newnb];


CFunctionStyleConvert[expr_Notebook]:=
  Module[{newnb = expr},
    newnb = ReplaceAll[newnb, Cell[s__, "ObjectName"|"ObjectNameAlt", r___] :> Cell[s, "CFunctionName", r]];
    newnb = ReplaceAll[newnb, Cell[s__, "Usage", r___] :> Cell[s, "CFunctionUsage", r]];
    newnb]


(******************************************************************************)
(** ConvertNotebook - Formats **)
(*ConvertNotebook[nb_, info_,
    ("Format"),
    opts___?OptionQ] := 
  Module[{expr},
    expr = nb;
    expr
  ];*)


(******************************************************************************)
(** ConvertNotebook - Guides **)
ConvertNotebook[nb_, info_,
    entityType:("Guide"),
    opts___?OptionQ] := 
  Module[{uri, lang, expr, labelCell, unfindraftimg, langext, obsolescenceNote, warnicon, handleUpdatedQ},
    BuildLog["** Entering ConvertNotebook - Guides **"];

    uri = "URI" /. info;
    lang = "Language" /. {opts} /. Options[MakeNotebook];
    expr = nb;
    handleUpdatedQ = !StringFreeQ[uri, "SummaryOfNewFeaturesIn"];
    
    (* Convert Alphabetical listings separately *)
    If[!StringFreeQ[uri,"AlphabeticalListing"], expr = ConvertNotebook[nb, info, "Alphabetical Listing", opts]];
    
    (* Convert substyle "UpdatedFunction" to (updated) in guides (currently only SummaryOfNewFeatures guides) *)
    If[TrueQ[handleUpdatedQ],
    	expr = HandleGuideUpdatedFunction[expr, lang]
    ];

    PlaceTutorialCollectionGroup[notebook_]:= Module[{tcg, pos, nbexpr},	 
                 tcg = GetGuideTutorialCollectionGroup[notebook];	 
                 pos = Position[notebook,	 
             Cell[CellGroupData[{	 
                 Cell[_ , "GuideTutorialsSection", ___],	 
                 ___}, ___], ___] ];	 
             pos = MapAt[# + 1 &, pos, {1, -1}];	 
             nbexpr = DeleteCases[notebook, Cell[CellGroupData[{	 
                 Cell[_ , "GuideTutorialCollectionSection", ___],	 
                 ___}, ___], ___], Infinity];	 
 	 
             Insert[nbexpr, tcg, pos]	 
          ];	 
 	 
    expr = If[IncludeTutorialCollectionGroup===False, expr, 
    If[Cases[expr,Cell[CellGroupData[{	 
                 Cell[_ , "GuideTutorialsSection", ___],	 
                 ___}, ___], ___],Infinity]=!={} &&	 
              Cases[expr,Cell[CellGroupData[{	 
                 Cell[_ , "GuideTutorialCollectionSection", ___],	 
                 ___}, ___], ___],Infinity]=!={},	 
                 PlaceTutorialCollectionGroup[expr], 	 
                 expr]
    ];	 
    
    (* special-casing the Recently Added Features guide (with the word clouds) *)
    If[StringMatchQ[uri, "guide/RecentlyAddedFeatures"],
      expr = MakeRecentlyAddedFeaturesGuide[expr, "Language"->lang];
      Return[expr]];
      
    (* Do some Custom Guide Page preprocessing (link handling, etc.) *)
    If[MemberQ[Options[expr], _["CustomGuideQ", True], Infinity],
      expr = ProcessCustomGuidePage[expr, "Language" -> lang]
    ];

    (* V9 CHANGE BELOW *)
    (* Insert Reference section header *)
    (* V111 redesign: removing Reference header altogether *)
    (*expr = Module[{pos,
       override = {"guide/WolframPredictiveInterface", "guide/NotebookShortcuts", "guide/CellGroupsAndOutlining"}},
       If[
         (Cases[expr, 
           Cell[___,"InlineGuideFunction"|"InlineGuideFunctionListing",___],Infinity] === {}) && FreeQ[override, uri], expr,
         pos = Position[expr, Cell["", ___, "GuideFunctionsSection", ___]];
         Insert[expr, Cell[Localized["GuideReferenceSectionText", lang], "GuideReferenceSection"], pos]
    ]];*)

    (* Insert Featured Examples section *)
    (* Commented out to remove per SW: 4/6/15 *)
    (* If re-adding, make sure to reinstate the "ExamplePageLinks" TaggingRule *)
    (*expr = Module[{pos, links = 
         DocumentationBuild`Info`GetSourceInformation[
           uri, lang, "FeaturedExampleLinks"], headercell, next, nextover, prev, prevover, nextdis, prevdis},
       If[MatchQ[links, {} | None | "None"], expr,
       pos = Position[expr, Cell[_, "GuideReferenceSection", ___]];
       {next, nextdis, nextover} = Map[
         ToBoxes[Import[DocsAsset["InternalOnlineImage", #]]]&,
         {"NextSlideButton.png", "NextSlideButton-disabled.png", "NextSlideButton-over.png"}
       ];
       {prev, prevdis, prevover} = Map[
         ToBoxes[Import[DocsAsset["InternalOnlineImage", #]]]&,
         {"PreviousSlideButton.png", "PreviousSlideButton-disabled.png", "PreviousSlideButton-over.png"}
       ];

       headercell = With[{
          prevdis=prevdis,prevover=prevover,prev=prev,
          next=next,nextover=nextover,nextdis=nextdis,
          length=Length[links]},
        If[length > 4, Cell[BoxData[GridBox[{{
        Cell[TextData[{Cell[BoxData[ToBoxes[Spacer[6]]]], Cell[Localized["GuideFeaturedExamplesSectionText", lang], "FeaturedExamplesText"]}]],
        Cell[BoxData[PaneBox[RowBox[{
          ButtonBox[DynamicBox[
            If[CurrentValue[EvaluationNotebook[],{TaggingRules, "ExampleCounter"}]===1, prevdis,
              If[CurrentValue["MouseOver"], 
                TagBox[prevover, MouseAppearanceTag["LinkHand"]], 
                TagBox[prev, MouseAppearanceTag["LinkHand"]]]]], 
            Enabled->Dynamic[CurrentValue[EvaluationNotebook[],{TaggingRules, "ExampleCounter"}] > 1],
            ButtonFunction:>(CurrentValue[EvaluationNotebook[],{TaggingRules, "ExampleCounter"}] = 
              Max[1, CurrentValue[EvaluationNotebook[],{TaggingRules, "ExampleCounter"}] - 1])
          ], "\[NonBreakingSpace]\[NonBreakingSpace]",
          DynamicBox[StyleBox[ToString[CurrentValue[EvaluationNotebook[],{TaggingRules, "ExampleCounter"}]],"ExampleControls"]],
          Cell["\[NonBreakingSpace]\[VerticalSeparator]\[NonBreakingSpace]", "ExamplePositionDivider", FontSize->12],
          StyleBox[ToString[Ceiling[length/4]], "ExampleControls"], "\[NonBreakingSpace]\[NonBreakingSpace]",

          ButtonBox[DynamicBox[
            If[CurrentValue[EvaluationNotebook[],{TaggingRules, "ExampleCounter"}]===Ceiling[length/4], nextdis,
              If[CurrentValue["MouseOver"], 
                TagBox[nextover, MouseAppearanceTag["LinkHand"]], 
                TagBox[next, MouseAppearanceTag["LinkHand"]]]]],
            Enabled->Dynamic[CurrentValue[EvaluationNotebook[],{TaggingRules, "ExampleCounter"}] < Ceiling[length/4]],
            ButtonFunction:>(CurrentValue[EvaluationNotebook[],{TaggingRules, "ExampleCounter"}] = 
              Min[Ceiling[length/4], CurrentValue[EvaluationNotebook[],{TaggingRules, "ExampleCounter"}] + 1])
          ]
        }], Alignment->{Left, Center}]], "ExamplePosition"](*,
         TODO: Add "show all" button for Featured Examples 
        ""*)

       }}]], "GuideFeaturedExamplesSection"],
       Cell[TextData[{Cell[BoxData[ToBoxes[Spacer[6]]]], 
         Cell[Localized["GuideFeaturedExamplesSectionText", lang], "FeaturedExamplesText"]}], "GuideFeaturedExamplesSection", CellFrameMargins->{{20, Inherited}, {6, 6}}]
       ]];

       Insert[expr, Cell[CellGroupData[{
       headercell,
       Cell[BoxData[
          DynamicBox[TemplateBox[{
            CurrentValue[EvaluationNotebook[],{TaggingRules,"ExamplePageLinks"}]},"FeaturedExamplesGrid"]]],
          "FeaturedExamplesGridCell"]
       }]], pos]
    ]];*)

    (* Insert Learning Resources section *)
    (* Commented out to remove Learning Resources per SW: 4/6/15 *)
    (*expr = If[DocumentationBuild`Utils`GetGuideTutorialsSectionList[expr]==={} && 
              DocumentationBuild`Utils`GetGuideHowTosList[expr]==={} &&
              DocumentationBuild`Info`GetSourceInformation[uri, lang, "WebResourcesLinks"]==={}, 
      expr,

      Module[{tempexpr=expr, pos, tclink, vidlink, demolink, commlink, headercell, lrcell, left, right, tutorials, howtos,
                   train,trainover, video,videoover,demo,demoover,comm,commover, demos, tuthowtolist, tuticon, howtoicon},
       {train,trainover} = ToBoxes[Rasterize[
          Import[DocsAsset["InternalOnlineImage", #], "PNG"],
          Background->GrayLevel[0.952941]]]& /@
          {"GuideTrainingCourses.png", "GuideTrainingCourses-over.png"};
       {video,videoover} = ToBoxes[Rasterize[
          Import[DocsAsset["InternalOnlineImage", #], "PNG"],
          Background->GrayLevel[0.952941]]]& /@
          {"GuideVideos.png", "GuideVideos-over.png"};
       {demo,demoover} = ToBoxes[Rasterize[
          Import[DocsAsset["InternalOnlineImage", #], "PNG"],
          Background->GrayLevel[0.952941]]]& /@
          {"GuideDemonstrations.png", "GuideDemonstrations-over.png"};
       {comm,commover} = ToBoxes[Rasterize[
          Import[DocsAsset["InternalOnlineImage", #], "PNG"],
          Background->GrayLevel[0.952941]]]& /@
          {"GuideCommunity.png", "GuideCommunity-over.png"};
       {tuticon, howtoicon} = ToBoxes[Rasterize[
          Import[DocsAsset["InternalOnlineImage", #], "PNG"],
          Background->GrayLevel[0.952941]]]& /@
          {"LearningResourcesTutorial.png", "LearningResourcesHowTo.png"};
       pos = Position[expr, Cell[_, "GuideFeaturedExamplesSection", ___]];
       If[pos === {}, pos = Position[expr, Cell[_, "GuideReferenceSection", ___]]];
       
       headercell = Cell[Localized["GuideLearningResourcesSectionText", lang], "GuideLearningResourcesSection"];
       tutorials = DocumentationBuild`Utils`GetGuideTutorialsSectionList[tempexpr];
       howtos = DocumentationBuild`Utils`GetGuideHowTosList[tempexpr];

       (* Delete old Tutorials section and remove how to links *)
       tempexpr = DeleteCases[tempexpr, 
         (Cell[___, "GuideTutorial", ___]) | 
         (Cell[c___] /; (MemberQ[{c}, "GuideRelatedLinks", Infinity] && 
            MemberQ[{c}, s_String/;StringMatchQ[s, "paclet:*howto/*"],Infinity] &&
            !MemberQ[{c}, "GuideRelatedLinksSection", Infinity])), Infinity];

       tuthowtolist = {
         {If[Length[tutorials] === 0, "", Cell[
           BoxData[
            GridBox[{{tuticon,GridBox[{{""},{Cell["Tutorials", "GuideTutorialsSection"]},
              Sequence@@({Cell[BoxData[TemplateBox[{Cell[#[[1]]],
                If[StringMatchQ[#[[2]], "paclet:*"],#[[2]],"paclet:"<>#[[2]]]},
                "RefLink", BaseStyle->{"GuideTutorial"}]], "GuideTutorial"]}& /@ tutorials)},
            GridBoxDividers->{"Rows" -> {{False}}}]}}, GridBoxSpacings->{"Columns"->{{0.3}}}, 
            GridBoxAlignment->{"Rows" -> {{Top}}}]], "GuideWebResource"]]},
         {If[Length[howtos] === 0, "", Cell[BoxData[GridBox[{{howtoicon,GridBox[{{""},{Cell[Localized["GuideHowTosSectionText", lang], "GuideHowToSection"]},
              Sequence@@({Cell[BoxData[TemplateBox[{Cell[#[[1]]],
                If[StringMatchQ[#[[2]], "paclet:*"],#[[2]],"paclet:"<>#[[2]]]},
                "RefLink", BaseStyle->{"GuideHowTo"}]], "GuideHowTo"]}& /@ howtos)},
                GridBoxDividers->{"Rows" -> {{False}}}]}}, GridBoxSpacings->{"Columns"->{{0.3}}},
                GridBoxAlignment->{"Rows" -> {{Top}}}]], "GuideWebResource"]]}
         };
       tuthowtolist = If[Length[tutorials] === 0, Reverse[tuthowtolist], tuthowtolist];

       {tclink, vidlink, demolink, commlink} = ReplaceAll[(If[Length[#]>0, First[#]]&/@Function[{type}, 
         Flatten[Select[DocumentationBuild`Info`GetSourceInformation[uri, lang, "WebResourcesLinks"],
           MatchQ[#[[2]], type]&]]] /@ {"Training Courses", "Videos", "Demonstrations", "Community"}), None->Null];
       If[demolink === Null,
         Quiet[demolink = First[Flatten[Cases[tempexpr, Cell[c___, "GuideRelatedLinks", ___]:>
           Cases[{c},URL[s_String]/;!StringFreeQ[s, "demonstrations.wolfram.com/search.html?"]:>s, Infinity], Infinity]]]];
         tempexpr = DeleteCases[tempexpr, Cell[c___, "GuideRelatedLinks", ___] /; 
           MemberQ[{c}, s_String/;!StringFreeQ[s, "demonstrations.wolfram.com/search.html?"], Infinity], Infinity]];
       If[!StringQ[demolink], demolink = Null];
       lrcell = Cell[BoxData[GridBox[{{
         left = Cell[BoxData[GridBox[tuthowtolist, GridBoxSpacings -> {"Rows" -> {{2}}}]], 
         "GuideWebResource"],
         right = DeleteCases[Cell[BoxData[GridBox[{
         {Cell[CapitalizeString[Localized["LearningResourcesRelatedWebText", lang]], "GuideWebResourcesSection"], 
          "\[SpanFromLeft]", ToBoxes[Spacer[80]]},
         {Cell[BoxData[TemplateBox[{Cell[Localized["LearningResourcesTrainingCoursesText", lang]], tclink, train, trainover}, "GrayWebLinkWithIcon", BaseStyle->{"GuideWebResourceText"}]]], 
          "\[SpanFromLeft]", ""},
         {Cell[BoxData[TemplateBox[{Cell[Localized["LearningResourcesVideosText", lang]], vidlink, video, videoover}, "GrayWebLinkWithIcon", BaseStyle->{"GuideWebResourceText"}]]], 
          "\[SpanFromLeft]", ""},
         {Cell[BoxData[TemplateBox[{Cell[Localized["LearningResourcesDemonstrationsText", lang]], demolink, demo, demoover}, "GrayWebLinkWithIcon", BaseStyle->{"GuideWebResourceText"}]]], 
          "\[SpanFromLeft]", ""},
         {Cell[BoxData[TemplateBox[{Cell[Localized["LearningResourcesCommunityText", lang]], commlink, comm, commover}, "GrayWebLinkWithIcon", BaseStyle->{"GuideWebResourceText"}]]], 
          "\[SpanFromLeft]", ""}
         }, 
        GridBoxDividers -> {"Rows" -> {False, False, {True}, 
            False}},GridBoxItemSize->{"Columns"->{{Full},Scaled[0.01]}}]], "GuideWebResource"],
         {Cell[BoxData[TemplateBox[c___]/;MemberQ[{c}, {} | Null, Infinity]],___],__}, Infinity]
         }}, GridBoxAlignment -> {"Columns" -> {{Left, Center}}, 
         "Rows" -> {{Top}}},
        GridBoxDividers -> {"Columns" -> {{False, True, False}, False}}]], "GuideLearningResourcesGrid"];
        
        (* If there are no Related Web Resources, don't include the RHS of the grid *)
        (*lrcell = If[MemberQ[lrcell[[1, 1, 1, 1, 2]], TemplateBox[___], Infinity], lrcell, ReplacePart[lrcell,{1,1,1,1,2}->""]];*)
        lrcell = If[MemberQ[lrcell[[1, 1, 1, 1, 2]], TemplateBox[___], Infinity], lrcell, 
          Cell[BoxData[GridBox[{{left}}, GridBoxAlignment -> {"Columns" -> {{Left, Center}}, 
            "Rows" -> {{Top}}}]], "GuideLearningResourcesGrid"]];
        lrcell = If[Length[tutorials] === 0 && Length[howtos] === 0, 
          Cell[BoxData[GridBox[{{right}}, GridBoxAlignment -> {"Columns" -> {{Left, Center}}, 
            "Rows" -> {{Top}}}]], "GuideLearningResourcesGrid"], lrcell];

        Insert[tempexpr, Cell[CellGroupData[{headercell, lrcell}, Closed]], pos]
   ]];*)

    (* 5/20/15: Copied this from the above commented-out code block to make sure these sections
       still get removed *)
    (* Delete old Tutorials section and remove how to links *)
    (* -- only delete tutorials links for System guides *)
    (* 11.1.1: re-adding tutorial links to all guides *)
       expr = Module[{systemQ = StringMatchQ[uri, "guide/*" | "paclet:guide/*"]},
       	DeleteCases[expr, 
         If[(*TrueQ[systemQ]*)False, (Cell[___, "GuideTutorial", ___]) | 
         (Cell[c___] /; (MemberQ[{c}, "GuideRelatedLinks", Infinity] && 
            MemberQ[{c}, s_String/;StringMatchQ[s, "paclet:*howto/*"],Infinity] &&
            !MemberQ[{c}, "GuideRelatedLinksSection", Infinity])),
            (Cell[c___] /; (MemberQ[{c}, "GuideRelatedLinks", Infinity] && 
            MemberQ[{c}, s_String/;StringMatchQ[s, "paclet:*howto/*"],Infinity] &&
            !MemberQ[{c}, "GuideRelatedLinksSection", Infinity]))], Infinity]];

    (* Change "..." to Ellipsis *)
    BuildLog["-- Change '...' to Ellipsis"];
    expr = expr /. d_String /; StringMatchQ[d, " ..."] :> " \[Ellipsis]";   

    BuildLog["-- GuideText LongDash to GuideEmDash style"];
    expr = expr /.
      Cell[TextData[{a___,
          b_String /; StringMatchQ[b, RegularExpression["\\s*\[LongDash]\\s*(.*)"]],
          c___}], "GuideText", d___] :> 
        ReplaceCharacters[
          Cell[TextData[{a, b, c}], "GuideText", d],
          "\[LongDash]",
          StyleBox["\[LongDash]", "GuideEmDash"]];

	obsolescenceNote = Cases[expr, Cell[_, "ObsolescenceNote", ___], Infinity];
    expr = DeleteCases[expr, Cell[_, "ObsolescenceNote", ___], Infinity];

    BuildLog["-- Delete various cells"];
    If[buildmode==="Future",
    	expr,
    	expr = DeleteCases[expr, Alternatives[
        Cell[s_String /;
          StringMatchQ[s, RegularExpression["XX+"]], ___],
        Cell[TextData[s_List /; Apply[ Or,
        	StringMatchQ[Transmogrify`ConvertToString[#], 
        		RegularExpression[".*XX+.*"]] & /@ s]], "GuideText", ___],
        Cell[cc_ /; (And[
            Length[#] > 0,
            StringMatchQ[Transmogrify`ConvertToString[First[#]], RegularExpression["XX+"]]
          ]&[Cases[cc,
            Cell[_, ("InlineFormula"|"InlineGuideFunctionListing"), ___], Infinity]]),
          "InlineGuideFunctionListing" | "GuideText" ___],
        Cell[s_String /;
          StringMatchQ[s, RegularExpression["(XX+( \\. )?)+"]],
          "InlineFormula", ___],
        Cell[s_String /;
          StringMatchQ[s, RegularExpression["(XX+( \\. )?)+"]],
          "InlineGuideFunctionListing", ___],
        Cell[cc_ /;
          (Count[cc, Cell[BoxData[s_String /;
              StringMatchQ[s, RegularExpression["XX+"]]
            ], "InlineGuideFunction", ___], Infinity] > 0),
          "GuideText", ___]
    ], Infinity]];
    
            expr = expr /. {Cell[c_String, "InlineGuideFunctionListing", o___] :> 
        InlineFunctionButton[Cell[c, "InlineGuideFunctionListing", o]]};

	BuildLog["-- Remove links to flagged symbols"];
	expr = RemoveGuideLinksToFlaggedSymbols[expr, uri, entityType, lang, buildmode];

	Scan[
		Function[{pos}, Module[{cell, links, newlinks, text},
			cell = Extract[expr, pos];

			links = Cases[
				cell,
				Cell[
					BoxData[_ButtonBox | StyleBox[_ButtonBox, ___]],
					"InlineFormula",
					___
				],
				Infinity
			];

			If[buildmode === "Future",
				newlinks = links
				,
				newlinks = Select[
					links,
					Function[link, CurrentLinkQ[link, uri, lang, False]]
				]
			];

			If[Length[links] =!= Length[newlinks],
				expr = ReplacePart[
					expr,
					pos -> Join[
						Cell[TextData[
							Flatten[Riffle[
								newlinks,
								{{
									"\[NonBreakingSpace]",
									StyleBox["\[FilledSmallSquare]", "InlineSeparator"],
									" "
								}},
								2
							]]
						]],
						Rest[cell]
					]
				];
			]
		]],
		Position[expr, Cell[_, "InlineGuideFunctionListing", ___]]
	];

    (* Make sure string boxes in an InlineGuideFunction don't get put in a TextData -- 
       they'll show up as raw strings instead of box interpretation *)
    expr = ReplaceAll[expr, Cell[BoxData[s_String /; !StringFreeQ[s, "\!"]], some___, "InlineGuideFunction", rest___] :> 
       Cell[BoxData[s], some, "InlineFunctionSans", rest]];

    (* change unlinked InlineGuideFunction to InlineFunctionSans *)
    expr = ReplaceAll[expr, Cell[BoxData[c_ /; FreeQ[c,GraphicsBox[___]|Graphics3DBox[___]|StyleBox[___,"TI"]]] | 
    	TextData[c_ /; FreeQ[c,GraphicsBox[___]|Graphics3DBox[___]|StyleBox[___,"TI"]]], some___, "InlineGuideFunction", rest___] :> 
  	 Cell[TextData[c], some, "InlineFunctionSans", rest]];

    BuildLog["-- Remove empty cells"];
    (* Remove empty cells *)
    expr = expr //. {
      Cell["", s_String /; !StringMatchQ[s, "GuideFunctionsSection"], ___] :> Sequence[],
      Cell[TextData[{}], s_String /; !StringMatchQ[s, "GuideFunctionsSection"], ___] :> Sequence[],
      BoxData[] :> ""};

    (* Need two types of GuideDelimiter for equal spacing around headings *)
    expr = expr //. 
      CellGroupData[{ce___, Cell[a_, "GuideDelimiter", b___],
          Cell[CellGroupData[{Cell[c_, "GuideFunctionsSubsection", d___], e___}, f___]], g___ },h___] :>
        CellGroupData[{ce, Cell[a, "GuideDelimiterSubsection", b],
          Cell[CellGroupData[{Cell[c, "GuideFunctionsSubsection", d], e}, f]],g},h];
          
    BuildLog["-- Localize MoreAbout on guide pages"];
    expr = expr /. Cell[c_, "GuideMoreAboutSection", o___] :> 
      Cell[Localized["GuideMoreAboutSectionText", lang], "GuideMoreAboutSection", o];
      
    BuildLog["-- Localize Tutorial Collection on guide pages"];
    expr = expr /. Cell[c_, "GuideTutorialCollectionSection", o___] :> 
      Cell[Localized["TutorialCollectionSectionText", lang], "GuideTutorialCollectionSection", o];

    BuildLog["-- Localize RelatedLinks on guide pages"];
    expr = expr /. Cell[c_, "GuideRelatedLinksSection", o___] :> 
      Cell[Localized["RelatedLinksSectionText", lang], "GuideRelatedLinksSection", o];

    BuildLog["-- Localize RelatedTutorials on guide pages"];
    expr = expr /. Cell[c_, "GuideTutorialsSection", o___] :> 
      Cell[Which[c==="Characters", Localized["CharactersSectionText", lang], True, Localized["TutorialsSectionText", lang]], "GuideTutorialsSection", o];

    BuildLog["-- Localize Workflow Guides on guide pages"];
    expr = expr /. Cell[c_, "GuideWorkflowGuidesSection", o___] :> 
      Cell[Localized["WorkflowGuidesSectionText", lang], "GuideWorkflowGuidesSection", o];

    (* Keep empty GuideAbstract for vertical spacing *)
    If[ Length@Cases[expr, Cell[_, "GuideAbstract", ___], Infinity, 1] > 0,
      expr = expr /. 
        Cell[str_String /; StringMatchQ[str, RegularExpression["\\s+|\\?+|XX+"]], "GuideAbstract", ___] :>
          Cell["\[NonBreakingSpace]", "GuideAbstract", CellID->1],
      expr = expr /. Cell[con_, "GuideTitle", o___] :> 
        Cell[CellGroupData[{
          Cell[con, "GuideTitle", o], Cell["\[NonBreakingSpace]", "GuideAbstract", CellID->1] }, Open]]
    ];

    (* Removing leading GuideDelimiters *)
    BuildLog["-- Removing leading GuideDelimiters on guide pages"];
    (* Clear a path by removing some empty cells *)
    expr = RemoveEmptyCellGroups[expr, {"GuideFunctionsSection"}];    
    expr = expr //.
      Cell[CellGroupData[{first___, gfs : Cell[___, "GuideFunctionsSection", ___], 
        Cell[CellGroupData[{Cell[___, "GuideDelimiter", ___], more___},___],___], rest___}, oc___], final___] :> 
        Cell[CellGroupData[{first, gfs, more, rest}, oc], final];

    (* Remove "Functions" per sw *)
    Module[{badheads},
      badheads = Alternatives[
        Cell[c_String/; StringMatchQ[c, "Functions:" | ""], "GuideFunctionsSection", ___],
        Cell[TextData[ButtonBox[s_String, BaseStyle->"GuideTestDrive"]], "GuideTestDrive", ___],
        Cell[TextData[ButtonBox[s_String, BaseStyle->"GuideMainTutorial"]], "GuideMainTutorial", ___]
      ];
      expr = expr /. {xa___, Cell[CellGroupData[{badheads, xb___}, ___], ___], xc___}
        :> {xa, xb, xc};
      expr = DeleteCases[expr, badheads, Infinity];
    ];

    (* change " . " to "something else" *)
    BuildLog["-- Change ' . ' to something else"];
    expr = expr //. { 
      Cell[TextData[{a___, s_String, b___}], 
        sty:("InlineGuideFunctionListing" | "GuideText"), o___] :> 
      Cell[TextData[{a, 
        StringSplit[s, RegularExpression["\\s+\\.\\s+"] -> 
          {"\[NonBreakingSpace]", StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"], " "}], b}
        ], sty, o] };

    (* change " \[EmptyVerySmallSquare] " to InlineSeparator *)
    BuildLog["-- Add MakeInlineSeparator[] "];

    expr = expr //. Cell[c_TextData, sty:("GuideText"|"InlineGuideFunctionListing"), o___] :>
      MakeInlineSeparator[Cell[c, sty, o]];
    BuildLog["-- Change FilledSmallSquare to InlineSeparator"];
    expr = expr //. {StyleBox["\[FilledSmallSquare]", "InlineSeparator"] :> 
      StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"]};
  
    (* Need color StyleBox around GuideFunctionsSubsection buttons *)
    BuildLog["-- Add color StyleBox around GuideFunctionsSubsection buttons"];
        expr = expr /. 
  Cell[c_, "GuideFunctionsSubsection", o___] :> 
   Cell[(c /. 
      ButtonBox[b__] :> 
       Module[{bs = Cases[{b}, (BaseStyle -> s_) :> s]}, 
        Apply[ButtonBox, 
         Append[DeleteCases[{b}, _[BaseStyle, _]], 
          BaseStyle -> 
           Append[If[ListQ[bs], bs, {bs}], 
            "GuideFunctionsSubsection"]]]]), 
    "GuideFunctionsSubsection", o];

    (* Change New In 6 links to a specific style *)
    BuildLog["-- Change NewIn6 links to special link"];

    $NewIn6FunctionList = If[DocumentationBuild`Internal`InternalAssetsDirectory[] === None,
      {}
      ,
      Get[DocsAsset["Internal:Web", FileNameJoin[{"data", "NewIn6FunctionList.m"}]]]
    ];

    expr = expr /. ButtonBox[f_String/;MemberQ[$NewIn6FunctionList, f], s___, BaseStyle->"Link", r___] :> 
      ButtonBox[f, s, BaseStyle->"NewInVersionLink", r];

    (* Add Dynamic for New In Version Links *)
    expr = expr /. 
      Cell[BoxData[ButtonBox[c__, BaseStyle->"NewInVersionLink", o1___], o2___], o3___] :>
        Cell[BoxData[ButtonBox[c, BaseStyle->"NewInVersionLink", o1], o2], o3,
          Background->Dynamic[
            If[CurrentValue[{TaggingRules, "ModificationHighlight"}] === True, 
            RGBColor[1, 0.988235, 0.721569], None]]];

    (* Change function name links on guide pages to be sans *)
    expr = expr /.
      Cell[BoxData[ButtonBox[c__]] | TextData[ButtonBox[c__]] ,
          "InlineGuideFunction" | "InlineFormula" | "InlineCode", o___] :>
        Cell[TextData[{ButtonBox[c]}], "InlineFunctionSans", o];

    (* If any InlineFunctionSans already exist in BoxData[] change to TextData[] *)
    expr = expr /. Cell[BoxData[ButtonBox[c__]], "InlineFunctionSans", o___] :>
      Cell[TextData[{ButtonBox[c]}], "InlineFunctionSans", o];

    If[StringMatchQ["Title" /. info, RegularExpression["\\s*New\\s*in\\s*6\\.0.*"]];
      BuildLog["-- 'New in 6.0' do not add UpdatedIn Graphic"];
      ,
      BuildLog["-- Add UpdatedIn Graphic"];
      expr = AddGuideNewInGraphic[expr, info, opts];
    ];

    (* Fix appearance of unlinked inline function listings (e.g., guide/DateAndTime) 
       secondary style "EmphasizedUnlinked" can be used to keep previous behavior *)
    expr = ReplaceAll[expr, {
      
      c:Cell[BoxData[text_String], some___, "InlineFormula", rest___] /; FreeQ[c, "EmphasizedUnlinked"] :>
      Cell[TextData[text], some, "InlineFunctionSansUnlinked", rest],
      
      c:Cell[s_String, some___, "InlineFunctionSans", rest___] /; FreeQ[c, "EmphasizedUnlinked"] :>
      Cell[s, some, "InlineFunctionSansUnlinked", rest],
      
      c:Cell[TextData[s_ /; FreeQ[s, ButtonBox]], some___, "InlineFunctionSans", rest___] /; FreeQ[c, "EmphasizedUnlinked"] :>
      Cell[TextData[s], some, "InlineFunctionSansUnlinked", rest]
      }];
    expr = DeleteCases[expr, "EmphasizedUnlinked", Infinity];

    (* If "Under Development", add [[UNFINISHED DRAFT]] banner *)
    (* V11.1: changed how [[EXPERIMENTAL]] banner is shown *)
    (*expr = If[MemberQ[expr, "UnderDevelopment", Infinity] || TrueQ["Underdevelopment" /. info],
     langext = lang /. {"English"->"", "Japanese"->"_jp", "ChineseSimplified"->"_zh", "Spanish"->"_es" };
     unfindraftimg = Import[DocsAsset["Graphics", "RefUnfinishedDraft"<>langext<>".m"]];

     ReplaceAll[expr, c : Cell[___, "GuideTitle", ___] :> 
       Cell[TextData[{c, Cell[BoxData[ToBoxes[Spacer[10]]]], Cell[BoxData[ToBoxes[unfindraftimg]]]}], "GuideTitleGrid"]],
     expr];*)
     If[MemberQ[expr, "UnderDevelopment", Infinity] || TrueQ["Underdevelopment" /. info],
       expr = Module[{pos,expcell},
         expcell = Cell[TextData[{Cell["[", "ObjectNameAltQuotes"], "EXPERIMENTAL", Cell["]", "ObjectNameAltQuotes"]}], "ExperimentalBanner"];
         pos = Position[expr, Cell[__, "GuideTitle", ___]];
         Insert[expr, expcell, pos]
         ]
       ];
     
     (* Turn some TI guide listings into TI2, which ups their font size (per andre/SW, 1/30/14) *)
     expr = ReplaceAll[expr, c:Cell[___, "InlineFunctionSans"|"InlineGuideFunction"|"InlineGuideFunctionListing", ___] :> 
     	ReplaceAll[c, {StyleBox[s_, some___, "TI", rest___] :> StyleBox[s, some, "TI2", rest],
     	    s_String /; !StringFreeQ[s,"\"TI\""] :> StringReplace[s,"\"TI\""->"\"TI2\""]}]];
     	    
    (* Add ObsolescenceNote *)
    (* V11.1: and delimiter *)
    warnicon = Image[Import[DocsAsset["Images", "ObsoleteIcon2x.png"], "PNG"], Magnification->0.5];
    obsolescenceNote = If[obsolescenceNote === {}, obsolescenceNote, {Cell[BoxData[GridBox[{{Cell[BoxData[ToBoxes@warnicon]], Cell[obsolescenceNote[[1, 1]], "ObsolescenceNote", CellFrame->0]}}]], Sequence@@Rest[obsolescenceNote[[1]]]]}];
    expr = expr /. 
    Cell[CellGroupData[{Cell[c_, sty_String/;StringMatchQ[sty, "GuideTitle"], o___], rest___},___]] :>
      (Cell[CellGroupData[Flatten[{
        Cell[c, sty, o],
        obsolescenceNote,
        rest,
        Cell["\t", "GuideDelimiterSubsection"]
      }, 1], Open]]);
      
    (* Add icons for visualization functions *)
    expr = ReplaceAll[expr, Cell[BoxData[GraphicsBox[gr_List/;!FreeQ[gr, "Icon"], ___, BoxID->id_String, ___]], ___] :> 
      With[{first = StringTake[id, 1],
        filename = StringJoin[id, ".png"]},
        Cell[BoxData[RowBox[{DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation","SymbolIcons",first},filename]]],"\[MediumSpace]"}]]]]];
      
    (* remove inline separators between entity type page links *)
    If[MemberQ[expr, s_String /; StringMatchQ[s, "paclet:*/entity/*"], Infinity],
    expr = expr /. Cell[l_, some___, "InlineGuideFunctionListing", rest___] :> 
     Cell[l //. {first___, Cell[bb_, f___, sty:"InlineFunctionSans" | "InlineFormula" | "InlineFunctionSansUnlinked", 
           r___] /;  !FreeQ[bb, s_String /; StringMatchQ[s, "paclet:*/entity/*" | "..."]], 
         "", "\[NonBreakingSpace]", StyleBox[__, "InlineSeparator", ___], " ", "", 
         Cell[bb2_, f2___, sty2:"InlineFunctionSans" | "InlineFormula" | "InlineFunctionSansUnlinked", r2___] /; 
           !FreeQ[bb2, s2_String /; StringMatchQ[s2, "paclet:*/entity/*" | "..."]], 
         more___} :> {first, Cell[bb, f, sty, r], "  ", Cell[bb2, f2, sty2, r2], more}, some, 
      "InlineGuideFunctionListing", rest]];

    (* add attractor icons to footer sections *)
    (* first, hard remove any empty sections *)
    If[Cases[expr, Cell[CellGroupData[{Cell[__, #, ___], ___}, ___]], Infinity]==={}, expr = DeleteCases[expr, Cell[__, #, ___], Infinity]]& /@
   	 {"GuideMoreAboutSection", "GuideTutorialsSection", "GuideWorkflowGuidesSection", "GuideRelatedLinksSection"};
    expr = expr /. c:Cell[CellGroupData[{Cell[__, "GuideMoreAboutSection"|"GuideTutorialsSection"|"GuideWorkflowGuidesSection"|"GuideRelatedLinksSection", ___], ___}, ___]] :> AddAttractorIcons[c, lang];
    expr
  ];

HandleGuideUpdatedFunction[nb_Notebook, lang_String:"English"] := 
	Module[{expr = nb, updatedstring, inlinelistinginsert},
		updatedstring = Localized["GuideUpdatedFunction", lang];
		inlinelistinginsert = StyleBox[updatedstring, "GuideText", FontWeight->"Plain"];
		
		(* "GuideText" cells first *)
		expr = ReplaceAll[expr, 
			Cell[TextData[c_] /; !FreeQ[c, "UpdatedFunction"], some___, "GuideText", rest___] :> 
			Cell[TextData[ReplaceAll[c, cell:Cell[BoxData[con_ /; !FreeQ[con, "UpdatedFunction"]], ___, "InlineGuideFunction", ___] :> 
				Splice[{ReplaceAll[cell, {"Link", "UpdatedFunction"} -> "Link"], updatedstring}]]], some, "GuideText", rest]	
		];
		
		(* "InlineGuideFunctionListing" cells *)
		expr = ReplaceAll[expr, 
			Cell[TextData[c_] /; !FreeQ[c, "UpdatedFunction"], some___, "InlineGuideFunctionListing", rest___] :> 
			Cell[TextData[ReplaceAll[c, cell:Cell[BoxData[con_ /; !FreeQ[con, "UpdatedFunction"]], ___, "InlineGuideFunction"|"InlineFormula", ___] :> 
				Splice[{ReplaceAll[cell, {"Link", "UpdatedFunction"} -> "Link"], inlinelistinginsert}]]], some, "InlineGuideFunctionListing", rest]	
		];
		
		expr
	];

Options[MakeRecentlyAddedFeaturesGuide] = {"Language"->"English"};
MakeRecentlyAddedFeaturesGuide[nb_Notebook, OptionsPattern[]]:=
  Module[{expr = nb, sections, newverhighlightsicon, listingmajornewicon, summarylistingicon, blogicon, livestreamicon, import,
    MakeLinkList, GetWordCloud, makeCell, language},

    language = OptionValue["Language"];
    import = ToBoxes[Image[Import[DocsAsset["InternalOnlineImage", #]], Magnification->0.5]]&;

    sections = Cases[expr, Cell[CellGroupData[{Cell[__, "GuideFunctionsSection", ___], ___}, ___]], Infinity];
    newverhighlightsicon = import["NewVersionHighlightsIcon.png"];
    summarylistingicon = import["SummaryListingIcon.png"];
    listingmajornewicon = summarylistingicon;
    blogicon = import["AnnouncementBlogIcon.png"];
    livestreamicon = import["LivestreamIcon2x.png"];
    
    (* adjusting livestream icon position *)
    livestreamicon = AdjustmentBox[livestreamicon, BoxBaselineShift->-.3];
    
    makeCell[s_String, data_List, icon_]:= Module[{bd = data[[Sequence@@(Position[data,s][[1]]+{0,1})]]},
         Cell[BoxData[
            ButtonBox[RowBox[{Cell[BoxData[If[StringMatchQ[s, Localized["RecentFeaturesBlog",language]|Localized["RecentFeaturesLivestream",language]], icon, Append[icon,BaselinePosition->Bottom]]]], "\[NonBreakingSpace]", Cell[s,"GuideFunctionsSubsection",FontSize->15]}],
              BaseStyle->If[ListQ[bd], "Hyperlink", "Link"], ButtonData->bd]], "GuideFunctionsSubsection"]];
    
    MakeLinkList:=
      Module[{mainlinks = 
        Cases[
          DeleteCases[Flatten[{#}], Cell[CellGroupData[{Cell[__,"GuideDelimiter",___], ___},___],___],Infinity], 
         Cell[__,"GuideText", ___], Infinity],
        pointpointlink = Cases[Flatten[{#}], c:Cell[__, "GuideText", ___]/;StringMatchQ[Transmogrify`ConvertToString[c], 
          (DigitCharacter|".").. ~~ ___ ~~ Localized["RecentFeaturesUpdate",language] ~~ ___, IgnoreCase->True], Infinity],
        earlierversionslink = Cases[Flatten[{#}], c:Cell[__, "GuideText", ___]/;StringMatchQ[Transmogrify`ConvertToString[c], 
          Localized["RecentFeaturesEarlierVersions",language], IgnoreCase->True], Infinity],
        textlinks
        },
      textlinks = {Transmogrify`ConvertToString[#], Quiet[Check[First[Cases[#,_[ButtonData, uri_]:>uri, Infinity]],None]]}&/@mainlinks;
      Cell[BoxData[
        RowBox[Flatten[{If[
          MemberQ[First/@textlinks,Localized["RecentFeaturesNewHighlights",language]], {makeCell[Localized["RecentFeaturesNewHighlights",language], textlinks, newverhighlightsicon], "       "}, Nothing],
          If[
          MemberQ[First/@textlinks,Localized["RecentFeaturesBlog",language]], {makeCell[Localized["RecentFeaturesBlog",language], textlinks, blogicon], "     "}, Nothing],
          If[
          MemberQ[First/@textlinks,Localized["RecentFeaturesLivestream",language]], {makeCell[Localized["RecentFeaturesLivestream",language], textlinks, livestreamicon], "     "}, Nothing],
          If[
          MemberQ[First/@textlinks,Localized["RecentFeaturesListingOfUpdates",language]], {makeCell[Localized["RecentFeaturesListingOfUpdates",language], textlinks, listingmajornewicon], "       "}, Nothing],
          If[
          MemberQ[First/@textlinks,Localized["RecentFeaturesSummary",language]], {makeCell[Localized["RecentFeaturesSummary",language], textlinks, summarylistingicon], "       "}, Nothing],
          If[
          Length[pointpointlink]>0, {"|     ", Join[pointpointlink[[1]],Cell[FontSize->13,FontColor->GrayLevel[0.396078431372549]]]}, Nothing]}]]],"GuideText", LineIndent->0,
          LinebreakAdjustments->{1.,10,0,0,10}]] &;
          
      GetWordCloud :=
        Cases[Flatten[#], Cell[__, "Picture", ___], Infinity][[1]] &;
          
      expr = ReplaceAll[expr, fullsec:Cell[CellGroupData[{sec:Cell[__, "GuideFunctionsSection", ___], innards:((Cell[__,"GuideText", ___] | Cell[__,"Picture",___] | 
         Cell[CellGroupData[{Cell[__,"GuideDelimiter",___],___},___]])...), rest___}, r___]]:>
        Cell[CellGroupData[{sec, MakeLinkList[fullsec], GetWordCloud[fullsec]}]]];
        
      expr
  
  ]


Options[ProcessCustomGuidePage] = {"Language" -> "English"};
ProcessCustomGuidePage[nb_Notebook, OptionsPattern[]] :=
  Module[{expr = nb, linktypeURItable, convertLinks, title},
    expr = DeleteCases[expr, Cell[__, "InlineListingAddButton", ___], Infinity];
    expr = DeleteCases[expr, Cell[c_ /; !FreeQ[c, "FunctionPlaceholder"], ___, "InlineGuideFunction", ___], Infinity];
    expr = DeleteCases[expr, Cell[c_ /; !FreeQ[c, "GuideTextPlaceholder"], ___, "GuideText", ___], Infinity];
    expr = DeleteCases[expr, Cell[c_ /; FreeQ[c, "InlineGuideFunction"], ___, "InlineGuideFunctionListing", ___], Infinity];
    linktypeURItable = <|"Symbol" -> "", "Function" -> "", "Format" -> "format/", "Interpreter Type" -> "interpreter/", "Entity Type" -> "entity/", "Data Structure" -> "datastructure/",
      "WSTP Function" -> "c/", "Character" -> "character/", "Classifier" -> "classifier/", "Compiled Type" -> "compiledtype/", "Database Connection" -> "databaseconnection/", 
      "Device Connection" -> "device/", "Embedding Format" -> "embeddingformat/", "External Evaluation System" -> "externalevaluationsystem/", "Built-in File" -> "file/", 
      "Front End Token" -> "frontendobject/", "Financial Indicator" -> "indicator/", "Menu Item" -> "menuitem/", "Method" -> "method/", "Net Decoder" -> "netdecoder/", 
      "Net Encoder" -> "netencoder/", "Predictor" -> "predictor/", "System Program" -> "program/", "Service Connection" -> "service/", "Text Content Type" -> "textcontent/",
      "WFR Function" -> "resourcefunction/",
      (*default case*)"PageType" -> ""|>;
    
    convertLinks[cell_Cell] :=
      ReplaceAll[cell, {
      
          Cell[BoxData[s_String], some___, "InlineGuideFunction", more___, TaggingRules->tr_ /; !FreeQ[tr, "PageType"], rest___] :>
          With[{uri = StringReplace[s, "\""->""], type = "PageType" /. tr},
           Cell[BoxData[ButtonBox[s, BaseStyle->"Link", ButtonData->If[StringMatchQ[type, "WFR Function"], "paclet:" ,"paclet:ref/"] <> linktypeURItable[type] <> uri]], some, "InlineGuideFunction", more, rest]],
          
          Cell[s_String, some___, "GuideMoreAbout", rest___] :> 
          With[{uri = "paclet:guide/" <> StringReplace[Capitalize[s, "AllWords"], Whitespace -> ""]},
            Cell[TextData[ButtonBox[s, BaseStyle->"Link", ButtonData->uri]], some, "GuideMoreAbout", rest]]
         }];
    
    expr = ReplaceAll[expr, c:Cell[__, "GuideText" | "InlineGuideFunctionListing" | "GuideMoreAbout", ___] :> convertLinks[c]];
    
    title = Transmogrify`ConvertToString[FirstCase[expr, Cell[__, "GuideTitle", ___], {}, Infinity]];
    
    expr = AddCustomGuideAnchorMenu[expr, title, OptionValue["Language"]];
    
    expr
  ]


(******************************************************************************)
(** AddCustomGuideAnchorMenu **)
AddCustomGuideAnchorMenu[nb_Notebook, title_String:"", lang_:"English"] :=
  Module[{anchorbarcell, newbar, newcell, cgpicon, cgpbutton, id, testbuild, buildtarget, camelname},
  id = title <> "_" <> FirstCase[nb, _["CustomGuideID", uuid_] :> uuid, "", Infinity];
  camelname = StringReplace[Capitalize[StringReplace[title, "&" -> "And"], "AllWords"], Whitespace -> ""];
  anchorbarcell = FirstCase[nb, Cell[__, "AnchorBarGrid", ___], {}, Infinity];
  testbuild = FirstCase[nb, _["TestBuild", testQ_] :> testQ, False, Infinity];
  buildtarget = FirstCase[nb, _["BuildDestination", testQ_] :> testQ, "Local", Infinity];
  If[anchorbarcell === {}, Return[nb]];
        
  cgpicon = With[{cgpid = id}, DynamicBox[
  
    Switch[DocumentationTools`GuidePages`Private`SavedCGP[cgpid, "Saved"],
    "Loading"|"AddRemove",
    DynamicBox[FEPrivate`FrontEndResource["DocumentationToolsExpressions", "CGPToolbarIconLoading"]],
    True,
    DynamicBox[FEPrivate`FrontEndResource["DocumentationToolsExpressions", "CGPToolbarIconTrue"]],
    False,
    DynamicBox[FEPrivate`FrontEndResource["DocumentationToolsExpressions", "CGPToolbarIconFalse"]],
    _,
    DynamicBox[FEPrivate`FrontEndResource["DocumentationToolsExpressions", "CGPToolbarIcon"]]]
    
   ]];
   
   (* You're building this yourself, so it's "saved" for you *)
   If[!TrueQ[testbuild],
     If[!AssociationQ[DocumentationTools`GuidePages`Private`SavedCGP],
       DocumentationTools`GuidePages`SyncCGPWithCloud["Precedence" -> "Cloud"]
     ];
     Switch[buildtarget,
     "Cloud",
     With[{obj = CloudObject["CustomGuidePages/built/" <> camelname <> ".nb"]},
       DocumentationTools`GuidePages`Private`SavedCGP[id] = 
         Symbol["DocumentationTools`GuidePages`Private`CGPAssocEntry"][id, <|"Saved"->True, "URL" -> obj|>];
       Symbol["DocumentationTools`GuidePages`SyncCGPWithCloud"]
     ],
     _,
     DocumentationTools`GuidePages`Private`SavedCGP[id] = 
       Symbol["DocumentationTools`GuidePages`Private`CGPAssocEntry"][id, <|"Saved"->True|>];
       Symbol["DocumentationTools`GuidePages`SyncCGPWithCloud"]
     ]
   ];
  
  newbar = DocumentationBuild`Make`Private`MakeAnchorBarCell[nb, {}, "IncludeSections" -> {"GuideMoreAbout", "GuideFunction"}];
  
  cgpbutton = 
    With[{cgpid = id, icon = cgpicon, test = testbuild},
    Cell[BoxData[TagBox[DynamicModuleBox[{$CellContext`editTarget$$}, 
      DynamicBox[If[!StringQ[$CellContext`editTarget$$],
        ActionMenuBox[icon, 
        {
          DynamicBox[If[TrueQ[DocumentationTools`GuidePages`Private`SavedCGP[cgpid, "Saved"]],
            StyleBox["Remove Saved Page", FontColor->RGBColor[.75, 0, 0]],
            StyleBox["Save Page", FontColor->RGBColor[0, .75, 0]]]] :>
          Symbol["DocumentationTools`GuidePages`AddRemoveCGP"][cgpid],
          Delimiter,
          "Sync with Cloud" :> (DocumentationTools`GuidePages`Private`SavedCGP[cgpid] = Symbol["DocumentationTools`GuidePages`Private`CGPAssocEntry"][cgpid,<|"Saved"->"AddRemove"|>];
            Symbol["DocumentationTools`GuidePages`SyncCGPWithCloud"]["Precedence"->"Cloud", "UseCache" -> False])
        },
        Appearance->None],
        With[{url = $CellContext`editTarget$$},
          TooltipBox[
            ButtonBox[icon, BaseStyle->"Hyperlink",
              ButtonData:>{URL[url],None}],
            DynamicBox[FEPrivate`FrontEndResource["DocumentationToolsStrings","CGPOpenEditModeTooltip"]]
        ]]]],
        Initialization:>(
          $CellContext`editTarget$$ = If[
            And[
              MatchQ[$EvaluationCloudObject, _CloudObject],
              $WolframID =!= $RequesterWolframID],
            StringReplace[$EvaluationCloudObject[[1]], "/obj/"->"/env/"],
            None
          ];
          Needs["DocumentationTools`GuidePages`"];
          If[Not@AssociationQ[DocumentationTools`GuidePages`Private`SavedCGP], 
            DocumentationTools`GuidePages`Private`SavedCGP = <||>];
            If[(TrueQ[CurrentValue["WolframCloudConnected"]] || TrueQ[$CloudConnected]) (*&& 
        !TrueQ[test]*),
       If[MissingQ[DocumentationTools`GuidePages`Private`SavedCGP[cgpid]], 
         DocumentationTools`GuidePages`Private`SavedCGP[cgpid] = 
           Symbol["DocumentationTools`GuidePages`Private`CGPAssocEntry"][cgpid,<|"Saved"->"Loading"|>]];
       If[DocumentationTools`GuidePages`Private`SavedCGP[cgpid, "Saved"] === "Loading",
        SessionSubmit[
         Symbol["DocumentationTools`GuidePages`SyncCGPWithCloud"]["Precedence"->"Cloud"];
         DocumentationTools`GuidePages`Private`SavedCGP[cgpid] = 
         With[{assoc = DocumentationTools`GuidePages`Private`SavedCGP},
           If[!FreeQ[Keys[assoc], cgpid] && assoc[cgpid, "Saved"] =!= "Loading", 
             assoc[cgpid], 
             Symbol["DocumentationTools`GuidePages`Private`CGPAssocEntry"][cgpid, <|"Saved"->False|>]]]
        ]],
        If[!AssociationQ[DocumentationTools`GuidePages`Private`SavedCGP[cgpid]],
          DocumentationTools`GuidePages`Private`SavedCGP[cgpid] = <||>]])], MouseAppearanceTag["LinkHand"]]]]
    ];
  
  newbar = Replace[newbar, Cell[TextData[{keepcells__, urlmenu_}], cellopts___] :> Cell[TextData[{keepcells, cgpbutton}], cellopts]];

  newcell = Replace[anchorbarcell, Cell[__, "AnchorBar", ___] -> newbar, {Length[FirstPosition[anchorbarcell, "AnchorBar", {}]] - 1}]; 

  Replace[nb, anchorbarcell -> newcell, {Length[FirstPosition[nb, "AnchorBarGrid", {}]] - 1}]
  
  ]


(******************************************************************************)
(** ConvertNotebook - Overviews **)
ConvertNotebook[expr_, info_,
    ("Guide Overview"),
    opts___?OptionQ] := 
  Module[{},
    expr
  ];



(******************************************************************************)
(** ConvertNotebook - Notes **)
ConvertNotebook[expr_, info_,
    ("Note"),
    opts___?OptionQ] := 
  Module[{},
    expr
  ];














(******************************************************************************)
(** ConvertNotebook - Root Guide **)
ConvertNotebook[nb_, info_,
    ("Root Guide"),
    opts___?OptionQ
] := Module[{
	expr, language, rootSectionCaptions, $Rebranded,
	headertext1, headerdividergraphic, headertext2,
	data,
	MakeRootActionMenu, MakeRootWebTile, MakeRootWebActionMenu,
	rootred, rootorange, rootblue, rootgreen, rootteal, rootdarkteal, rootgrey,
	brightrootred, brightrootorange, brightrootblue, brightrootgreen, brightrootteal, brightrootdarkteal, brightrootgrey,
	workflowlinks, workflowgray, brightworkflowgray, resourcegray, brightresourcegray,
	workflowstripecol, workflowstripe, resourcestripecol,
	resourcestripe, wfpRootQ, footerlinks, latestfeaturestileQ, DarkenRootTileColor,
	(* localization variables *)
	footerstring, workflowfooterstr, resourcefooterstr, resourcelinks, builtworkflowsectionstr, builtresourcesectionstr,
	mainRootContent
},
	(* expr = ConvertNotebook[nb, info, "Guide", opts];*)

	(* Add the "Recent Features" tile automatically, only if we're building the System
	   root guide page. *)
	latestfeaturestileQ = Lookup[info, "URI"] === "guide/WolframRoot";

	expr = nb;

	language = "Language" /. {opts} /. Options[MakeNotebook];
	$Rebranded = "Rebranding" /. {opts} /. {"Rebranding" -> False};

	(* Formatting note: 06a78200e2:
		added some hacky code to special case the WFP root guide *)
	wfpRootQ = TrueQ[MemberQ["Keywords" /. info, "wfp"]];
	footerstring = Localized["RootGuideFooterLinks", language];
	workflowfooterstr = Localized["RootGuideWorkflowLinks", language];
	resourcefooterstr = Localized["RootGuideResourceLinks", language];
	{builtworkflowsectionstr, builtresourcesectionstr} = ToUpperCase /@ {workflowfooterstr, resourcefooterstr};

	(* Formatting note: 6f587d1d96: edit to root guide footers to use links from source notebook instead of hardcoded *)
	footerlinks = Cases[
		First @ Cases[
			expr,
			Cell[
				CellGroupData[
					{
						Cell[footerstring, ___, "Section", ___],
						rest__
					},
					___
				]
			] :> {rest},
			Infinity
		],
		ButtonBox[text_, ___, ButtonData -> bd_, ___]
			:> {text, bd},
		Infinity
	];

	workflowlinks = Cases[
		First @ Cases[
			expr,
			Cell[
				CellGroupData[
					{
						Cell[workflowfooterstr, ___, "Section", ___],
						rest__
					},
					___
				]
			] :> {rest},
			Infinity
		],
		ButtonBox[text_, ___, ButtonData -> bd_, ___]
			:> {text, bd},
		Infinity
	];

	resourcelinks = Cases[
		First @ Cases[
			expr,
			Cell[
				CellGroupData[
					{
						Cell[resourcefooterstr, ___, "Section", ___],
						rest__
					},
					___
				]
			] :> {rest},
			Infinity
		],
		ButtonBox[text_, ___, ButtonData -> bd_, ___]
			:> {text, bd},
		Infinity
	];

	(* Formatting note: 5036a36923: Implemented design updates from WLDOCS-2087 *)
    (* hack in data for Fast Intro resources tile *)
    resourcelinks = Replace[
		resourcelinks,
		(* Formatting note: 64700ca121: fix bug 369267, also fix ordering in see-also dropdowns *)
		{"Fast Introductions", __} -> {
			"Fast Introductions",
			{
				{"For Programmers", {URL["https://www.wolfram.com/language/fast-introduction-for-programmers/en/"], None}},
				{"For Math Students", {URL["https://www.wolfram.com/language/fast-introduction-for-math-students/en/"], None}}
			}
		},
		{1}
	];

	(* Formatting note:
		The design for root guide pages is spec'ed most recently in WLDOCS-2087. *)

    (
    LogIt["-- Root guide page"];
    expr = DeleteCases[expr, Cell[_, "LinkTrail", ___], Infinity];
    (
    headertext1 = Localized["RootGuideHeaderText1", language];
    headertext2 = Localized["RootGuideHeaderText2", language];

	headerdividergraphic = ToBoxes[
		Graphics[
			{
				FaceForm[{GrayLevel[1.`],Opacity[1.`]}],
				FilledCurve[{{{0,2,0}, {0,1,0}, {0,1,0}}}, {{{0.3`, 39}, {0.7`, 39}, {0.7`, 12}, {0.3`, 12}}}]
			},
			ImageSize -> {Automatic,35},
			PlotRange -> {{-0.3`, 1.3`}, {8, 41}},
			BaselinePosition -> Scaled[.25]
		]
	];

	data = Cases[
		nb,
		Cell[
			CellGroupData[
				{
					Cell[c_, ___, "GuideFunctionsSection", ___],
					rest__
				},
				___
			],
			___
		] :> {
			ConvertToString[c],
			Cases[
				{rest},
				Cell[co_, ___, sty : ("GuideText" | "GuideDelimiter"), ___] :> If[
					StringMatchQ[sty, "GuideDelimiter"],
					"Delimiter",
					{
						ConvertToString[co],
						Quiet @ Check[
							First @ Cases[co, _[ButtonData, uri_String] :> uri, Infinity],
							{}
						]
					}
				],
				Infinity
			]
		},
		Infinity
	];

	data = ReplaceAll[data, {"Delimiter", rest__} :> {rest}];
	data = ReplaceAll[data, {some__, "Delimiter"} :> {some}];

	If[latestfeaturestileQ && !wfpRootQ,
		data = Append[
			data,
			{
				Localized["RootRecentFeatures", language],
				"paclet:guide/RecentlyAddedFeatures"
			}
		];
	];

    rootred=RGBColor[0.9294117647058824, 0.4, 0.];
    rootorange=RGBColor[0.9058823529411765, 0.6196078431372549, 0.];
    (* used to be blue, is now green *)
    rootblue=RGBColor[0.5411764705882353, 0.7098039215686275, 0.17647058823529413];
    (* for code compatibility, green and teal are both still defined, though they're both the same color now *)
    rootgreen=RGBColor[0.19215686274509805, 0.596078431372549, 0.8];
    rootteal=RGBColor[0.19215686274509805, 0.596078431372549, 0.8];
    rootdarkteal=RGBColor[0.19215686274509805, 0.596078431372549, 0.8];
    rootgrey=RGBColor[{68,68,68}/255.];

    brightrootred=RGBColor[1., 0.396078431372549, 0.00392156862745098];
    brightrootorange=RGBColor[0.9607843137254902, 0.6745098039215687, 0.058823529411764705];
    brightrootblue=RGBColor[0.6, 0.7803921568627451, 0.20784313725490197];
    brightrootgreen=RGBColor[0.21568627450980393, 0.6705882352941176, 0.9019607843137255];
    brightrootteal=RGBColor[0.21568627450980393, 0.6705882352941176, 0.9019607843137255];
    brightrootdarkteal=RGBColor[0.13725490196078433, 0.6352941176470588, 0.8509803921568627];

    workflowgray=(*{GrayLevel[1, 0.3],GrayLevel[1, 0.2],GrayLevel[1, 0.12],GrayLevel[1, 0.09]}*)
      Table[GrayLevel[0.4, 1], 4];

    brightworkflowgray=(*{GrayLevel[1, 0.45],GrayLevel[1, 0.35],GrayLevel[1, 0.25],GrayLevel[1, 0.15]}*)
      Table[RGBColor[0.2549019607843137, 0.7529411764705882, 0.7686274509803922, 1], 4];

    resourcegray = workflowgray;

    brightresourcegray = Table[
		RGBColor[0.33725490196078434, 0.5686274509803921, 0.6392156862745098, 1],
		4
	];

    workflowstripecol = brightworkflowgray[[1]];
    workflowstripe =
      Graphics[{workflowstripecol,Thickness[.02],Line[{{0,0},{1,0}}]},PlotRange->{{0,1},{-.5,.5}},ImageSize->{548,3},AspectRatio->3/548.];
    resourcestripecol = brightresourcegray[[1]];
    resourcestripe =
      Graphics[{resourcestripecol,Thickness[.02],Line[{{0,0},{1,0}}]},PlotRange->{{0,1},{-.5,.5}},ImageSize->{548,3},AspectRatio->3/548.];

    (* This function is to provide the non-hover color, as the hover colors are what's provided by Design *)
    DarkenRootTileColor[col_] := With[{blend = RGBColor[0.29411774928639156, 0.2941175018212802, 0.29411737179856606]},
    	Blend[{col, blend}, .1]];

    Options[MakeRootActionMenu] = Options[MakeRootWebActionMenu] = {
		Appearance -> None,
		ImageSize -> All,
		BaseStyle -> {LineSpacing -> {2, 0}}
	};

	MakeRootActionMenu[header_, mouseover_, contents_List, col_:Automatic, options:OptionsPattern[]] := Module[{
		opt = Join[options, Options[MakeRootActionMenu]]
	},
		With[{
			pairs = If[MatchQ[#, "Delimiter"],
				Delimiter
				,
				#[[1]] :> FEPrivate`FrontEndExecute[FrontEndToken[
					"OpenHelpLink",
					{
						#[[2]],
						FEPrivate`If[CurrentValue["ShiftKey"], Null, FEPrivate`ButtonNotebook[]]
					}
				]]
			]& /@ contents
		},
			ActionMenu[
				Mouseover[header, mouseover],
				pairs,
				opt,
				MenuStyle -> {FontSize -> 14, FontFamily -> "Arial", FontWeight -> "Bold", FontColor -> col}
			]
		]
	];

	MakeRootActionMenu[header_, mouseover_, contents_String, col_:Automatic, options:OptionsPattern[]] :=
		Mouseover[
			header,
			Append[
				Insert[
					mouseover,
					(FEPrivate`FrontEndExecute[FrontEndToken[
						"OpenHelpLink",
						{
							#1,
							FEPrivate`If[CurrentValue["ShiftKey"],Null,FEPrivate`ButtonNotebook[]]
						}
					]]&),
					2
				],
				ButtonData -> contents
			]
		];

	MakeRootWebActionMenu[header_, mouseover_, contents_List, col_:Automatic, options:OptionsPattern[]] := Module[{
		opt = Join[options, Options[MakeRootWebActionMenu]]
	},
		With[{
			pairs = If[MatchQ[#, "Delimiter"],
				Delimiter
				,
				#[[1]] :> FrontEndExecute[{
					NotebookLocate[
						#[[2]],
						"OpenInNewWindow" -> CurrentValue["HyperlinkModifierKey"]
					]
				}]
			]& /@ contents
		},
			ActionMenu[
				Mouseover[header, mouseover],
				pairs,
				opt,
				MenuStyle -> {FontSize -> 14, FontFamily -> "Arial", FontWeight -> "Bold", FontColor -> col}
			]
		]
	];

	MakeRootWebTile[header_, mouseover_, contents_List, col_:Automatic, options:OptionsPattern[]]:=
		If[!MatchQ[contents, {URL[__], None}],
			MakeRootWebActionMenu[header, mouseover, contents, col, options],
			Mouseover[
				header,
				Append[
					Append[
						Append[
							Insert[
								mouseover,
								(FrontEndExecute[{NotebookLocate[#2, "OpenInNewWindow" -> CurrentValue["HyperlinkModifierKey"]]}]&),
								2
							]
							(*, BaseStyle\[Rule]"Hyperlink"*)
						],
						ButtonData -> contents
					],
					ButtonNote->contents[[1,1]]
				]
			]
	];

    makeRootCell = Function[{ex, st},
      Cell[BoxData[
        ex
      ], st]
    ];

    (* Extract information and delete cells *)
    abstract = Cases[expr, Cell[c_, "GuideAbstract", ___] :> c, Infinity, 1];
    expr = DeleteCases[expr, Cell[_, "GuideAbstract" | "FooterCell", ___], Infinity];
    (* Construct new cells *)
    c = If[Length@abstract > 0, First@abstract, " "];

    (* Insert new cells *)
    expr = expr /. {a___, Cell[_, "GuideTitle", ___], b___} :> {
      a,


      (*makeRootCell[
      Cell[BoxData[
        GridBox[{{
          TagBox[ButtonBox[
            Cell[TextData[Localized["RootGSVideos", language]]],
            BaseStyle -> "RootFooterMainLink",
            Background->Dynamic[
              If[CurrentValue["MouseOver"],
               RGBColor[0.596078,0.015686,0.039215], RGBColor[0.686274,0.05098,0.094117]]],
            ButtonData->"http://www.wolfram.com/support/learn/get-started-with-mathematica",
            ButtonNote -> None
          ],MouseAppearanceTag["LinkHand"]],
          TagBox[ButtonBox[
            Cell[TextData[Localized["RootWhatsNew", language]]],
            BaseStyle -> {"Link", "RootFooterMainLink"},
            Background->Dynamic[
              If[CurrentValue["MouseOver"],
               RGBColor[0.596078,0.015686,0.039215], RGBColor[0.686274,0.05098,0.094117]]],
            ButtonData -> "paclet:guide/SummaryOfNewFeaturesIn100",
            ButtonNote -> None
          ], MouseAppearanceTag["LinkHand"]],
          TagBox[ButtonBox[
            Cell[TextData[Localized["RootHowTo", language]]],
            BaseStyle -> {"Link", "RootFooterMainLink"},
            Background->Dynamic[
              If[CurrentValue["MouseOver"],
               RGBColor[0.596078,0.015686,0.039215], RGBColor[0.686274,0.05098,0.094117]]],
            Evaluator -> Automatic,
            ButtonData -> "paclet:guide/HowToTopics",
            ButtonNote -> None
          ], MouseAppearanceTag["LinkHand"]]
        }}]
      ], "RootHeaderGrid"],
    "RootHeader"],*)
      b};

    $i = 0;

    (* If 'GuideText' list is a string, make into a list of Buttons *) 
(* Commenting out 070226 -- andy
    expr = expr /. Cell[c_String, "GuideText", opts___] :> 
      inlineFunctionStringToButtonBox[Cell[c, "GuideText", opts]];
*)
    (* Center Root Guide page *)
    expr = expr /. Rule[WindowMargins,_] :> WindowMargins->Automatic;
    );

	expr = expr /. {
		Cell[a___, "GuideMainTutorial", z___] :> Cell[a, "RootGuideMainTutorial", z],
		Cell[a___, "GuideMoreAbout", z___] :> Cell[
			TextData[Join[
				{"\[ThickSpace]\[FilledVerySmallSquare]\[ThickSpace]\[ThickSpace]\[ThickSpace]"},
				If[MatrixQ[List @@ a],
					a[[1]],
					List @@ a
				]
			]],
			"RootMoreAbout",
			z,
			FontColor -> RGBColor[0.729411764705882, 0.211765, 0.0627450980392157]
		],
		(*Cell[a___, "GuideMoreAbout", z___] :>
		Cell[ TextData@@Join[{"\[ThickSpace]\[FilledVerySmallSquare]\[ThickSpace]\[ThickSpace]\[ThickSpace]"},List@@a] ,
			"RootMoreAbout", z, FontColor->RGBColor[0.729411764705882, 0.21176470588235305`, 0.0627450980392157]],*)
		Cell[a___, "GuideMoreAboutDelimiter", z___] :> Cell[a, "RootMoreAboutDelimiter", z],
		Cell[a___, "GuideMoreAboutSub", z___] :> Cell[
			TextData @@ Join[
				{"\[ThickSpace]\[FilledVerySmallSquare]\[ThickSpace]\[ThickSpace]\[ThickSpace]"},
				List @@ a
			],
			"RootMoreAboutSub",
			z,
			FontColor -> RGBColor[0.729411764705882, 0.211765, 0.0627450980392157]
		]
	};

  (* Root guide page -> multi column links *)
  (* BETA:: Some changes in the following that were hacked in for the M9 beta 1 *)
  iconiter = 0;

	expr = DeleteCases[
		expr,
		Cell[CellGroupData[{Cell[___,"GuideFunctionsSection"|"Section",___],___},___],___],
		Infinity
	];

	expr[[1]] = With[{
		rootredtext=rootred,rootorangetext=rootorange,rootbluetext=rootblue,rootgreentext=rootgreen,roottealtext=rootteal,
		rootredicon=brightrootred,rootorangeicon=brightrootorange,rootblueicon=brightrootblue,
		rootgreenicon=brightrootgreen,roottealicon=brightrootteal, rootdarktealicon=brightrootdarkteal,
		CGPIconCell = 
		  Cell[BoxData[
		  	  
		  	  	DynamicBox[DocumentationTools`GuidePages`rootdisplaycell$$]],
		  	
		  	    Initialization:>(DocumentationTools`GuidePages`rootdisplaycell$$ = StyleBox[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Toolbars", 
		  	      "DocumentationTools"}, "CGP-RootIcon@2x.png"]]], Magnification -> 0.3];
		  	    Needs["DocumentationTools`GuidePages`"];
		  	    If[ValueQ[DocumentationTools`GuidePages`CachedRootGuideMenu],
		  	      DocumentationTools`GuidePages`rootdisplaycell$$ = DocumentationTools`GuidePages`CachedRootGuideMenu,
		  	      SessionSubmit[DocumentationTools`GuidePages`rootdisplaycell$$ = DocumentationTools`GuidePages`CreateRootGuideCGPMenu[], Method->"Idle"]
		  	    ]
		  	)]
		 
		  	
	},

		mainRootContent = Join[
			{{
				(* 02/27/20: use textual header instead of image for hidpi purposes *)
				Item[
					RawBoxes @ Cell[
						BoxData[GridBox[
							{{
								ToBoxes[Spacer[22]],
								StyleBox[headertext1, FontFamily -> "Source Sans Pro", FontColor -> GrayLevel[1], FontSize -> 28],
								ToBoxes[Spacer[8]],
								headerdividergraphic,
								ToBoxes[Spacer[10]],
								StyleBox[headertext2, FontFamily -> "Source Sans Pro", FontColor -> GrayLevel[.7], FontSize -> 28],
								ItemBox["",ItemSize->"Fit"],
								CGPIconCell,
								ToBoxes[Spacer[20]]
							}},
							GridBoxSpacings -> {"Columns" -> {{0}}},
							
							GridBoxAlignment -> {
								"Rows" -> {{Baseline}},
								"Columns" -> {{Center}}
							}
						]],
						PageWidth->Infinity
					],
					Alignment->Left
				]
			}},
			{{
				If[!TrueQ[wfpRootQ],
					Style[
						Row @ Table[
							Module[{
								textcol=(* v12: root dropdowns get black text *)GrayLevel[0.2](*Which[i<=5,rootredtext,
								i<=11,rootorangetext,
								i<=18,rootbluetext,
								i<=20,rootgreentext,
								i>20,roottealtext]*),
								iconcol = Which[
									(i===Length[data] && latestfeaturestileQ),
										rootdarktealicon,
									(* Formatting note: fc155ee3b9: handles WLDOCS-2273, WLDOCS-2663, WLDOCS-2553 *)
									i <=  6, rootredicon,
									i <= 12, rootorangeicon,
									i <= 18, rootblueicon,
									i <= 20, rootgreenicon,
									i >  20, roottealicon
								],
								pad, padh,
								iconover,
								(* shenanigans because higher-math moved up, until new root guide assets are in place *)
								iconimg = Import[DocsAsset["InternalOnlineImage", "RootTileIcon"<>ToString[i]<>".m"]]
							},
								iconimg = ReplaceAll[iconimg, r_[ImageSize,_] :> r[ImageSize, {66.,60.}]];

								pad = 550 - First[ImageDimensions[iconimg]];
								padh = (158.-Last[ImageDimensions[iconimg]]/2) / 2;

								(*iconover = Image[Rasterize[ImagePad[Rasterize[ImagePad[iconimg, {{pad-14,14},{padh,padh}}, None], Background->iconcol],{{0,1},{0,0}}, rootgrey], "Graphics"],Magnification->0.5];
								iconimg = Image[Rasterize[ImagePad[Rasterize[ImagePad[DocumentationBuild`Utils`ChangeImageTransparency[iconimg,0.35], {{pad-14,14},{padh,padh}}, None], Background->DarkenRootTileColor[iconcol]],{{0,1},{0,0}}, rootgrey], "Graphics"],Magnification->0.5]; *)

								(* commented below since icon images are now vectors *)
								(*If[ImageDimensions[iconimg][[1]] === 132,
									iconimg = ImagePad[iconimg, {{-32, 0}, {0, 0}}]];*)

								iconover = Style[
									Overlay[
										{
											Graphics[
												{iconcol, Rectangle[]},
												AspectRatio -> 218/550.,
												PlotRangePadding -> 0,
												ImagePadding -> 0,
												ImageMargins -> 0,
												ImageSize -> {550, 218}
											],
											(* Formatting note: 6eeaad8c24: fixed bug 348735*)
											Item[
												Grid[
													{{
														Spacer[40],
														Style[data[[i,1]], "RootSectionTitle", Magnification -> 2*Inherited]
													}},
													Alignment -> {Left, Center},
													ItemSize -> {{Scaled[.03],Scaled[.80]},Automatic}
												],
												Alignment -> {Left, Center}
											],
											Item[iconimg, Alignment -> {.85,Center}, ItemSize -> Full]
										},
										ImageSize -> {550, 218}
									],
									Magnification->0.5*Inherited
								];
								iconimg = Style[
									Overlay[
										{
											Graphics[
												{DarkenRootTileColor[iconcol], Rectangle[]},
												AspectRatio -> 218/550.,
												PlotRangePadding -> 0,
												ImagePadding -> 0,
												ImageMargins -> 0,
												ImageSize -> {550, 218}
											],
											Item[
												Grid[
													{{
														Spacer[40],
														Style[data[[i,1]], "RootSectionTitle", Magnification -> 2*Inherited]
													}},
													Alignment -> {Left, Center},
													ItemSize -> {{Scaled[.03], Scaled[.80]}, Automatic}
												],
												Alignment -> {Left,Center}
											],
											Item[
												ReplaceAll[iconimg, {
													Opacity[_] -> Opacity[.35],
													FaceForm[col_] :> FaceForm[{col, Opacity[.35]}]
												}],
												Alignment -> {.85, Center},
												ItemSize -> Full
											]
										},
										ImageSize -> {550, 218}
									],
									Magnification -> 0.5 * Inherited
								];

								MouseAppearance[
									MakeRootActionMenu[
										Button[iconimg,
										ImageSize->{276, 80}, Appearance->None],
										Button[iconover,
										ImageSize->{276, 80}, Appearance->None],
										data[[i,2]],
										textcol
									],
									"LinkHand"
								]
							],
							{i,Length[data]}
						],
						(* Formatting note: 7bda0607f5: hopefully fix vertical spacing between root guide tiles *)
						StripOnInput -> False,
						LineSpacing -> {1,2}
					],
					(*else --- WFP Wolfram Finance Platform-specific code below --- had to start with 0 index since icons are numbered*)
					Style[
						Row[
							Table[With[{i = j+1}, Module[{
								textcol = (* v12: root dropdowns get black text *)GrayLevel[0.2](*Which[i<=3,rootbluetext,
								i<=8,rootredtext,
								i<=14,rootorangetext,
								i<=19,rootbluetext,
								i<=21,rootgreentext,
								i>21,roottealtext]*),
								iconcol = Which[
									i <=  3, rootblueicon,
									i <=  8, rootredicon,
									i <= 14, rootorangeicon,
									i <= 19, rootblueicon,
									i <= 21, rootgreenicon,
									i >  21, roottealicon
								],
								pad, padh,
								iconover,
								iconimg = Import[
									DocsAsset[
										"InternalOnlineImage",
										StringJoin[
											"RootTileIcon",
											Which[
												i > 3 && i < 14,
													ToString[i - 3],
												i >= 14 && i < 18,
													ToString[i - 2],
												i >= 18,
													ToString[i-1],
												i === 1,
													"WFP",
												i === 2,
													"16",
												i === 3,
													"12"
											],
											".png"
										]
									]
								]
							},
								pad = 550 - First[ImageDimensions[iconimg]];
								padh = (158. - Last[ImageDimensions[iconimg]]/2) / 2;

								(* Formatting note: 7f6cb3a825: fix bug 349151 (WFP root guide issue) *)
								If[ImageDimensions[iconimg][[1]] === 132,
									iconimg = ImagePad[iconimg, {{-32, 0}, {0, 0}}]
								];

								iconover = Style[
									Overlay[
										{
											Graphics[
												{iconcol, Rectangle[]},
												AspectRatio -> 218/550.,
												PlotRangePadding->0,
												ImagePadding -> 0,
												ImageMargins -> 0,
												ImageSize -> {550, 218}
											],
											Item[
												Grid[
													{{
														Spacer[40],
														Style[data[[i,1]], "RootSectionTitle", Magnification->2*Inherited]
													}},
													Alignment -> {Left, Center},
													ItemSize -> {{Scaled[.03], Scaled[.80]}, Automatic}
												],
												Alignment -> {Left, Center}
											],
											Item[
												Style[
													ImagePad[iconimg, {{0,12}, {0,0}}, None],
													Magnification -> 2*Inherited
												],
												Alignment -> {Right, Center}
											]
										},
										ImageSize -> {550,218}
									],
									Magnification -> 0.5*Inherited
								];
								iconimg = Style[
									Overlay[
										{
											Graphics[
												{DarkenRootTileColor[iconcol], Rectangle[]},
												AspectRatio->218/550.,
												PlotRangePadding->0,
												ImagePadding->0,
												ImageMargins->0,
												ImageSize->{550,218}
											],
											Item[
												Grid[
													{{
														Spacer[40],
														Style[data[[i,1]], "RootSectionTitle", Magnification -> 2*Inherited]
													}},
													Alignment -> {Left, Center},
													ItemSize -> {{Scaled[.03], Scaled[.80]}, Automatic}
												],
												Alignment -> {Left, Center}
											],
											Item[
												Style[
													ImagePad[
														DocumentationBuild`Utils`ChangeImageTransparency[iconimg, 0.35],
														{{0, 12}, {0, 0}},
														None
													],
													Magnification->2*Inherited
												],
												Alignment -> {Right,Center}
											]
										},
										ImageSize -> {550,218}
									],
									Magnification -> 0.5*Inherited
								];

								MouseAppearance[
									MakeRootActionMenu[
										Button[iconimg,
										ImageSize->{276, 80}, Appearance->None],
										Button[iconover,
										ImageSize->{276, 80}, Appearance->None],
										data[[i,2]],
										textcol
									],
									"LinkHand"
								]
							]],
							{j,0,Length[data]-1}]
						],
						StripOnInput -> False,
						LineSpacing -> {1,2}
					]
				]
			}},

			{{ Item[Style["", ShowStringCharacters -> False]] }},
			{{ Item[Style[builtworkflowsectionstr, FontFamily->"Source Sans Pro", FontSize->12, FontColor->GrayLevel[.5], ShowStringCharacters->False],Alignment->Left]}},
			(* Workflow tiles *)
			{{
				Style[
					Row @ Table[
						Module[{
							iconcol = workflowgray[[Ceiling[i/3]]],
							iconcolover = brightworkflowgray[[Ceiling[i/3]]],
							pad, padh,
							iconimg = Image[Table[{0., 0., 0., 0.}, {12}, {132}], ColorSpace -> "RGB"],
							iconover
						},
							pad = 550 - First[ImageDimensions[iconimg]];
							padh = (110.-Last[ImageDimensions[iconimg]]/2)/2;

							iconover = Style[
								Overlay[
									{
										Overlay[
											{
												Graphics[
													{iconcolover, Rectangle[]},
													AspectRatio -> 109/550.,
													PlotRangePadding -> 0, ImagePadding -> 0, ImageMargins -> 0,
													ImageSize -> {550, 109}
												],
												Item[
													Grid[
														{{
															Spacer[15],
															Style[workflowlinks[[i, 1]], "RootSectionTitle", FontSize -> 30],
															Item["", ItemSize -> "Fit"],
															Style["\[RightGuillemet]", "RootSectionTitle", FontSize -> 60],
															Spacer[10]
														}},
														Alignment -> {Left, Center},
														ItemSize -> {{Scaled[.03], Scaled[.80]}, Automatic}
													],
													Alignment -> {Left, Center}
												](*,
												Item[iconimg,Alignment->{.85,Center}]*)
											},
											ImageSize -> {550,112}
										],
										workflowstripe
									},
									Alignment -> {Left, Top}
								],
								Magnification -> 0.5*Inherited
							];

							iconimg = Style[
								Overlay[
									{
										Overlay[
											{
												Graphics[
													{iconcol,Rectangle[]},
													AspectRatio->109/550.,
													PlotRangePadding->0, ImagePadding->0, ImageMargins->0,
													ImageSize->{550,109}
												],
												Item[
													Grid[
														{{
															Spacer[15],
															Style[workflowlinks[[i,1]], "RootSectionTitle", FontSize->30],
															Item["", ItemSize -> "Fit"],
															Style["\[RightGuillemet]", "RootSectionTitle", FontSize->60],
															Spacer[10]
														}},
														Alignment -> {Left, Center},
														ItemSize -> {{Scaled[.03], Scaled[.80]}, Automatic}
													],
													Alignment -> {Left, Center}
												]
												(*,
												Item[ReplaceAll[iconimg,{Opacity[_]->Opacity[.35],FaceForm[col_]:>FaceForm[{col,Opacity[.35]}]}], Alignment->{.85,Center}]*)
											},
											ImageSize -> {550, 112}
										],
										workflowstripe
									},
									Alignment -> {Left,Top}
								],
								Magnification->0.5*Inherited
							];

							(*iconover = Image[Rasterize[ImagePad[Rasterize[ImagePad[iconimg, {{pad-14,14},{padh,padh}}, None], Background->iconcolover],
								{{0,1},{0,0}}, rootgrey], "Graphics"],Magnification->0.5*Inherited];*)
							(*iconimg = Image[Rasterize[ImagePad[Rasterize[ImagePad[ImagePad[iconimg, {{pad-14,14},{padh,padh-3}}, None], {{0, 0}, {0, 3}}, workflowstripecol],
								Background->iconcol],{{0,2},{0,0}}, rootgrey], "Graphics"],Magnification->0.5*Inherited]; *)

							(*iconimg = Quiet@Style[ImageGraphics[iconimg, Method->"Exact"], Magnification->0.5*Inherited];
							iconover = Quiet@Style[ImageGraphics[iconover, Method->"Exact"], Magnification->0.5*Inherited];*)

							MouseAppearance[
								MakeRootActionMenu[
									Button[iconimg,
									ImageSize->{276, 56}, Appearance->None],
									Button[iconover,
									ImageSize->{276, 56}, Appearance->None],
									workflowlinks[[i,2]]
								],
								"LinkHand"
							]
						],
						{i, Length[workflowlinks]}
					],
					StripOnInput -> False,
					LineSpacing -> {1, 1}
				]
			}},

			{{Item[Style["", ShowStringCharacters -> False]]}},
			{{Item[Style[builtresourcesectionstr, FontFamily->"Source Sans Pro", FontSize->12, FontColor->GrayLevel[.5], ShowStringCharacters->False],Alignment->Left]}},
			(* resources tiles *)
			{{
				Style[
					Row @ Table[
						Module[{
							iconcol = resourcegray[[Ceiling[i/3]]],
							iconcolover = brightresourcegray[[Ceiling[i/3]]],
							pad, padh,
							iconimg = Image[Table[{0., 0., 0., 0.}, {12}, {132}], ColorSpace->"RGB"],
							iconover,
							icons = Import[DocsAsset["InternalOnlineImage", "RootGuideResourceIcons.m"]]
						},
							pad = 550 - First[ImageDimensions[iconimg]];
							padh = (110.-Last[ImageDimensions[iconimg]]/2)/2;

							iconover = Style[
								Overlay[
									{
										Overlay[{
											Graphics[{iconcolover,Rectangle[]}, AspectRatio->112/550., PlotRangePadding->0, ImagePadding->0, ImageMargins->0, ImageSize->{550,112}],
											Item[Grid[{{Spacer[15], Style[resourcelinks[[i,1]], "RootSectionTitle", FontSize->30], Item["", ItemSize->"Fit"],
											RawBoxes[StyleBox[icons[[i]],Magnification->Inherited]], Spacer[10]}}, Alignment->{Left, Center},
											ItemSize->{{Scaled[.03],Scaled[.80]},Automatic}],Alignment->{Left,Center}](*,
											Item[iconimg,Alignment->{.85,Center}]*)},ImageSize->{550,112}
										],
										resourcestripe
									},
									Alignment->{Left,Top}
								],
								Magnification -> 0.5*Inherited
							];

							iconimg = Style[
								Overlay[
									{
										Overlay[
											{
												Graphics[
													{iconcol, Rectangle[]},
													AspectRatio->112/550.,
													PlotRangePadding->0, ImagePadding->0, ImageMargins->0,
													ImageSize->{550,112}
												],
												Item[
													Grid[
														{{
															Spacer[15],
															Style[resourcelinks[[i,1]], "RootSectionTitle", FontSize -> 30],
															Item["", ItemSize -> "Fit"],
															RawBoxes[icons[[i]]],
															Spacer[10]
														}},
														Alignment -> {Left, Center},
														ItemSize -> {{Scaled[.03], Scaled[.80]}, Automatic}
													],
													Alignment -> {Left, Center}
												]
												(*,
												Item[ReplaceAll[iconimg,{Opacity[_]->Opacity[.35],FaceForm[col_]:>FaceForm[{col,Opacity[.35]}]}], Alignment->{.85,Center}]*)
											},
											ImageSize -> {550, 112}
										],
										resourcestripe
									},
									Alignment->{Left,Top}
								],
								Magnification -> 0.5*Inherited
							];

							(*iconover = Image[Rasterize[ImagePad[Rasterize[ImagePad[iconimg, {{pad-14,14},{padh,padh}}, None], Background->iconcolover],
								{{0,1},{0,0}}, rootgrey], "Graphics"],Magnification->0.5*Inherited];
							iconimg = Image[Rasterize[ImagePad[Rasterize[ImagePad[ImagePad[iconimg, {{pad-14,14},{padh,padh-3}}, None], {{0, 0}, {0, 3}}, resourcestripecol],
								Background->iconcol],{{0,1},{0,0}}, rootgrey], "Graphics"],Magnification->0.5*Inherited];

							iconimg = Quiet@Style[ImageGraphics[iconimg, Method->"Exact"], Magnification->0.5*Inherited];
							iconover = Quiet@Style[ImageGraphics[iconover, Method->"Exact"], Magnification->0.5*Inherited];*)

							MouseAppearance[
								MakeRootWebTile[
									Button[iconimg,
									ImageSize->{276, 56}, Appearance->None],
									Button[iconover,
									ImageSize->{276, 56}, Appearance->None],
									resourcelinks[[i,2]]
								],
								"LinkHand"
							]
						],
						{i, Length[resourcelinks]}
					],
					StripOnInput -> False,
					LineSpacing -> {1, 1}
				]

			}},

			{{
				Item[
					Style[
						Column[
							{
								Style["", ShowStringCharacters -> False]
								(*Module[{counter=0, iconover, icon},
									iconover = ImagePad[Image[Import[DocsAsset["InternalOnlineImage", #]], Magnification->0.5], {{0,0},{8,0}}, None] & /@ {"FastIntroRootFooterIcon-over.png","EIWLRootFooterIcon-over.png"};
									icon = (*Image[Import[DocsAsset["InternalOnlineImage",If[counter===0,"FastIntroRootFooterIcon.png","EIWLRootFooterIcon.png"]]], Magnification\[Rule]0.5]*)
										DocumentationBuild`Utils`ChangeImageTransparency[#, 0.8]& /@ iconover;

									Row[With[{lctext = StringJoin[{#[[1]](*, " \[RightGuillemet]"*)}],

									hyperlinkQ = MemberQ[#[[2]], URL[__], Infinity]}, counter++;

									If[TrueQ[hyperlinkQ],
										MouseAppearance[
										Button[
											PaneSelector[{
												False -> Row[{icon[[counter]], "\[NonBreakingSpace]\[NonBreakingSpace]\[NonBreakingSpace]", Style[lctext,FontColor->GrayLevel[.85], FontSize->Inherited+4]}],
												True -> Row[{iconover[[counter]], "\[NonBreakingSpace]\[NonBreakingSpace]\[NonBreakingSpace]", Style[lctext,FontColor->GrayLevel[1], FontSize->Inherited+4]}]
										}, Dynamic[CurrentValue["MouseOver"]]],

											FrontEndExecute[{NotebookLocate[#2,"OpenInNewWindow"->CurrentValue["HyperlinkModifierKey"]]}]&,BaseStyle -> "RootFooterLink",
										ButtonData -> #[[2]],
										ButtonNote -> None,Appearance->None], "LinkHand"],

										RawBoxes[TemplateBox[{ToBoxes@PaneSelector[{
												False -> Row[{icon[[counter]], "\[NonBreakingSpace]\[NonBreakingSpace]\[NonBreakingSpace]", Style[lctext,FontColor->GrayLevel[.85], FontSize->Inherited+4]}],
												True -> Row[{iconover[[counter]], "\[NonBreakingSpace]\[NonBreakingSpace]\[NonBreakingSpace]", Style[lctext,FontColor->GrayLevel[1], FontSize->Inherited+4]}]
										}, Dynamic[CurrentValue["MouseOver"]]],#[[2]]},"RefLink",BaseStyle->"RootFooterLink"]]]
											]& /@ footerlinks[[1;;2]]
									(* commented out 082616 to use footer link info from source guide page, replaced with above code --brianv *)
									(* deleted commented out code -- was too big *), Spacer[22]]]*),
								Module[{footerlength = Length[footerlinks[[3;;]]]},
									Row @ Riffle[
										With[{
											lctext = StringJoin[{#[[1]], " \[RightGuillemet]"}],
											hyperlinkQ = MemberQ[#[[2]], URL[__], Infinity]
										},
											If[TrueQ[hyperlinkQ],
												MouseAppearance[
													Button[
														PaneSelector[
															{
																False -> Style[lctext,FontColor->GrayLevel[.733333], FontSize->Inherited+2],
																True -> Style[lctext,FontColor->GrayLevel[0.8], FontSize->Inherited+2]
															},
															Dynamic[CurrentValue["MouseOver"]]
														],
														FrontEndExecute[{NotebookLocate[#2,"OpenInNewWindow"->CurrentValue["HyperlinkModifierKey"]]}]&,
														BaseStyle -> "RootFooterLink",
														ButtonData -> #[[2]],
														ButtonNote -> None,Appearance->None
													],
													"LinkHand"
												]
												,
												RawBoxes[TemplateBox[
													{
														ToBoxes @ PaneSelector[
															{
																False -> Style[lctext,FontColor->GrayLevel[.733333], FontSize->Inherited+2],
																True -> Style[lctext,FontColor->GrayLevel[0.9], FontSize->Inherited+2]
															},
															Dynamic[CurrentValue["MouseOver"]]
														],
														#[[2]]
													},
													"RefLink",
													BaseStyle -> "RootFooterLink"
												]]
											]
										]& /@ footerlinks[[3;;]]
										,
										Prepend[
											Table[
												Row[{
													Spacer[10],
													Style[" ",FontColor->GrayLevel[.8], ShowStringCharacters->False, FontSize->15],
													Spacer[10]
												}],
												footerlength - 2
											],
											Row[{Spacer[24], Style["|",FontColor->GrayLevel[.8], ShowStringCharacters->False, FontSize->15], Spacer[24]}]
										]
										(* commented out 082616 to use footer link info from source guide page, replaced with above code --brianv *)
										(* deleted commented out code -- was too big *)
									]
								]
							},
							Alignment -> Center,
							Spacings -> 1.2
						],
						Magnification -> (Inherited * 1)
					],
					Alignment -> Center
				]
			}}
		];


		Join[
			expr[[1]],
			{
				Cell[
					BoxData @ Cell[
						BoxData @ ToBoxes[
							Deploy @ Grid[
								mainRootContent,
								Alignment -> Center,
								Spacings -> {Automatic, {Automatic,2.2,{Automatic},2.2,Automatic}}
							]
						],
						(* V11.1 change -- make it such that there are no more than 3 columns of tiles *)
						TextAlignment -> Center,
						"RootCell",
						CellSize ->
							(*{
							Dynamic[Max[(350),
							Min[(880),
							((First[CurrentValue[EvaluationNotebook[],WindowSize]]-10)/AbsoluteCurrentValue[EvaluationNotebook[], Magnification])]]],Automatic}*)
							{
								Dynamic[FEPrivate`Which[
									(* Formatting note: 6485192868: fix bug 363786 *)
									(AbsoluteCurrentValue[EvaluationNotebook[],WindowSize][[1]]-10)/AbsoluteCurrentValue[EvaluationNotebook[], Magnification]<600,
										644,
									(AbsoluteCurrentValue[EvaluationNotebook[],WindowSize][[1]]-10)/AbsoluteCurrentValue[EvaluationNotebook[], Magnification]>720,
										880,
									True,
										Automatic
								]],
								Automatic
							}
					],
					"RootCell",
					TextAlignment->Left,
					CellMargins->
						(*{{
						Dynamic[Max[0,
						Floor[(First[CurrentValue[EvaluationNotebook[],WindowSize]]-(880*AbsoluteCurrentValue[EvaluationNotebook[], Magnification]))/(2*AbsoluteCurrentValue[EvaluationNotebook[], Magnification])]]], Inherited},
						{Inherited, Inherited}}*)

						{
							{
								Dynamic[FEPrivate`Which[
									(AbsoluteCurrentValue[EvaluationNotebook[], WindowSize][[1]] - (880*AbsoluteCurrentValue[EvaluationNotebook[], Magnification])) / (2*AbsoluteCurrentValue[EvaluationNotebook[], Magnification]) < 10,
										10,
									True,
										(AbsoluteCurrentValue[EvaluationNotebook[],WindowSize][[1]]-(880*AbsoluteCurrentValue[EvaluationNotebook[], Magnification]))/(2*AbsoluteCurrentValue[EvaluationNotebook[], Magnification])
								]],
								Inherited
							},
							{Inherited, Inherited}
						}
				]
			}
		]
	];

	expr = DeleteCases[expr, Cell[_, "RootMoreAbout", ___], Infinity];

	expr = With[{rootgrey = rootgrey},
		Join[
			DeleteCases[
				expr,
				(CellGrouping|Selectable|ShowGroupOpener|Background) ~ (Rule|RuleDelayed) ~ _
			],
			Notebook[
				(*CellGrouping->Manual,*)
				ShowGroupOpener -> False,
				Selectable -> False,
				Background -> rootgrey
			]
		]
	];

);

	(* Special case for stylebox merging/wrapping on root guide page *)
	expr = expr/. Cell[
			BoxData[ButtonBox[
				PaneSelectorBox[
					{
						Rule[b1__, Cell[TextData[{"MathLink"}]]],
						Rule[b3__, Cell[TextData[{"MathLink"}], "RootDarkLink"]]
					},
					psopt2_
				],
				bbopts2__
			]],
			cellsty2__
		] :> Cell[
			BoxData[ButtonBox[
				PaneSelectorBox[
					{
						Rule[b1,Cell[TextData[{StyleBox["MathLink", FontSlant->Italic]}]]],
						Rule[b3,Cell[TextData[{StyleBox["MathLink", FontSlant->Italic]}], "RootDarkLink"]]
					},
					psopt2
				],
				bbopts2
			]],
			cellsty2
		];

	expr = expr /. TextData[{
			ding_,
			StyleBox[
				Cell[
					BoxData[ButtonBox[
						PaneSelectorBox[
							{
								Rule[a1__, Cell[TextData[{a2__}]]],
								Rule[a3__, Cell[TextData[{a4__}], "RootDarkLink"]]
							},
							psopt1_
						],
						bbopts1__
					]],
					cellsty1__
				],
				sty__
			],
			Cell[
				BoxData[ButtonBox[
					PaneSelectorBox[
						{
							Rule[b1__, Cell[TextData[{b2__}]]],
							Rule[b3__, Cell[TextData[{b4__}], "RootDarkLink"]]
						},
						psopt2_
					],
					bbopts2__
				]],
				cellsty2__
			]
		}] :> TextData[{
			ding,
			Cell[
				BoxData[ButtonBox[
					PaneSelectorBox[
						{
							Rule[a1, Cell[TextData[{StyleBox[a2, sty], b2}]]],
							Rule[a3, Cell[TextData[{StyleBox[a4, sty], b4}], "RootDarkLink"]]
						},
						psopt1
					],
					bbopts1
				]],
				cellsty1
			]
		}];

	expr
];


(******************************************************************************)
(** ConvertNotebook - Workflows & Workflow Guides **)
ConvertNotebook[nb_, info_,
    type:("Workflow" | "Workflow Guide"),
    opts___?OptionQ] := 
  Module[{expr, uri, lang, stylerules, onetimesections,DynamicGroupOpeners, groupopenersections, closedGroups, CloseGroups},
    expr = nb;
    uri = "URI" /. info;
    uri = ToPacletURI[uri];
    lang = "Language" /. {opts} /. Options[MakeNotebook];
    
    (* Convert various cell styles *)
    stylerules = (Cell[s__, #, r___] :> Cell[s, StringJoin["Workflow",#], r])& /@ 
     {"Explanation", "ThingsToKnow", "Details", "RelatedWorkflows", "Text", "Item", "RelatedWorkflowsSection",
      "QAndASection", "Question", "Answer", "TechNotesSection", "TechNote", "GuideTitle", "GuideVideo","MainGuideDescription",
      "GuideDescription","GuideStep","GuideBoxedSmall","PrerequisiteLink","StepRelatedLink","RelatedLink",
      
      "ExplanationSection","ExplanationText","ThingsToKnowSection","ThingsToKnowItem","DetailsItem","StepRelatedWorkflowsSection",
      "StepRelatedWorkflows","RelatedWorkflows","Tutorials", "TutorialsSection","RelatedFunctionsSection", "RelatedFunctions",
      "NotesText", "RelatedGuidesSection", "RelatedGuides", "TechNotesSection", "TechNotes",
      "RelatedLinks", "RelatedLinksSection",
      
      (* Guide Styles *)
      "GuideTitleHeader","GuideTitle","GuideDescription","GuideDivider","GuideSubdivider","GuideGroupHeader","GuideVideo",
      "GuideSectionHeader","GuideText","GuideItem","SubGuideItem"
      };
    
    (* Restyle SeeAlso cell to Related Workflows *)
    stylerules = Join[stylerules, 
       {Cell[s__,"SeeAlsoSection",r___]:>Cell[Sequence@@ReplaceAll[{s},"See Also"->"Related Workflows"],"WorkflowRelatedWorkflowsSection",r],
        Cell[s__,"SeeAlso",r___]:>Cell[s,"WorkflowRelatedWorkflows",r],
        Cell[s__,"NotesSection",r___]:>Cell[s,"WorkflowNotesHeader",r]}];
    
    (* tear out useless stuff from WorkflowNotesSection *)
    expr = ReplaceAll[expr, Cell[_, some___, "WorkflowNotesSection", rest___] :> Cell[Localized["WorkflowNotesSectionText", lang], some, "WorkflowNotesSection", rest]];
    (* same with NotesSection *)
    expr = ReplaceAll[expr, Cell[_, some___, "NotesSection", rest___] :> Cell[Localized["WorkflowNotesSectionText", lang], some, "NotesSection", rest]];

    If[MatchQ[type, "Workflow"], 
    (* close various cell groups *)
    closedGroups = ("WorkflowHeader"|"WorkflowNotesSection"|"NotesSection");
  
    CloseGroups[nbexpr_Notebook, style_String] := 
      nbexpr /. {
      Cell[CellGroupData[{
         Cell[con_, sty_String /; StringMatchQ[sty, style], o___],
        (*Cell[con_, sty_String /; StringMatchQ[sty, "ExampleSection"|"ExampleSubsection"], o___],*)
          cs___
        }, ___], g___] 
      :> Cell[CellGroupData[{
        Cell[con, sty, o],
          cs
        }, If[(Length[Cases[con, Localized["BasicExamplesSectionText", lang], Infinity]] > 0), Open, Closed]], g],
      Cell[CellGroupData[{
        Cell[con_, "NotesSection", o___],
          cs___
        }, ___], g___]
      :> Cell[CellGroupData[{
        Cell[con, "NotesSection", o],
          cs
        }, (* Details section to be open by default on Message pages: brianv--05/12/16*) 
        If[MatchQ[type, "Message"], Open, Closed]], 
      g]
    };
  
    expr = Fold[CloseGroups, expr, closedGroups];
    ];
    
    (* Renumber in/out cells (code just below) *)
    (* Move this code into WorkflowFinisher to prevent style/grouping screwups due to the build not having the DocuTools stylesheets *)
    (*expr = DocumentationBuild`Common`PubsEvaluateWithFE@WorkflowRenumberInOut[expr];*)

    (* Remove cell labels from in/out cells in Explanation section(s)
       and close the section *)
    expr = ReplaceAll[expr, Cell[CellGroupData[{
      Cell[c__, styl:"ExplanationSection"|"ThingsToKnowSection", rest1___],
      others__}, ___]] :> Cell[CellGroupData[{
        Cell[c,styl,rest1],
        Sequence @@ ReplaceAll[{others}, Cell[con__, sty:"Input"|"Output", 
          more___, CellLabel->_, rest___]:>
          Cell[con, sty, more, rest]]},Closed]]];

     (* insert delimiter before footer *) 
     (* don't need it anymore -- added delimiters to the footer section styles *)
     (*Module[{pos=Last[Position[expr,Cell[CellGroupData[{Cell[__,"WorkflowPlatform",___],___},___]]]]},
       If[pos =!= {},
       expr = 
        Insert[expr,Cell["","WorkflowPlatformDelimiter"],
         ReplacePart[pos,-1->pos[[-1]]+1]]]];*)
       
    (* Delete Annotation cells *)
    expr = DeleteCases[expr, Cell[__, "Annotation", ___], Infinity];
    
    (* Remove WorkflowDescription cells that are just "Description" *)
    expr = DeleteCases[expr, Cell["Description"|"XXXX", ___, "WorkflowDescription", ___], Infinity];
    
    (* Remove Input cells with XXXX *)
    expr = DeleteCases[expr, Cell[BoxData["XXXX"], ___, "Input", ___], Infinity];

    (* Edit WorkflowGuideEntry cells to remove < ... > text that Chris uses to distinguish link text from target file name *)
    expr = ReplaceAll[expr, Cell[c_,some___,style:("WorkflowGuideEntry"|"WorkflowGuideSection"),rest___]:>
      Cell[ReplaceAll[c,ButtonBox[s_String,r___]:>ButtonBox[StringReplace[s,Whitespace...~~"<"~~Shortest[__]~~">"->""],r]],some,style,rest]];

	BuildLog["-- Remove links to flagged symbols"];
	expr = RemoveGuideLinksToFlaggedSymbols[expr, uri, type, lang, buildmode];

    (* Run code that duplicates content in headers with [] or [platform name] *)
    expr = DuplicateWorkflowContent[expr];

    (* redundant with code above 
    (* Get rid of file names in <> in links *)
    expr = ReplaceAll[expr, ButtonBox[c_, rest___] :> 
    	ButtonBox[ReplaceAll[c, 
    		s_String/;StringMatchQ[s, "*<"~~((LetterCharacter|DigitCharacter|Whitespace)..)~~">*"]:>
    		StringReplace[StringReplace[s,"<"~~((LetterCharacter|DigitCharacter|Whitespace)..)~~">"->""], "  "->" "]], rest]];*)

    (* Add WorkflowDelimiter between WorkflowSteps *)		
    expr = AddWorkflowStepDelimiters[expr];


    (* Lock down AnnotatedInput cells to prevent editing of annotations in build nbs *)
    expr = LockDownWorkflowAnnotations[expr];
    
    onetimesections = {};
  DynamicGroupOpeners[nbexpr_Notebook, styles_List]:=
    Module[{cells, counter=0, newnb=nbexpr,rules},
     With[{
       openicon = Get[DocsAsset["Graphics:Opener", "WorkflowSectionOpenerOpenOff.m"]],
       closedicon = Get[DocsAsset["Graphics:Opener", "WorkflowSectionOpenerClosedOff.m"]]
     },
     Switch[#,
     (* Cells that appear only ONCE in a notebook *)
     "NotesSection"|"PrimaryExamplesSection",
     newnb=ReplaceAll[newnb, Cell[CellGroupData[{s:Cell[__,#,___],r___},___]]:>
       Cell[CellGroupData[{s, r}, Dynamic[CurrentValue[EvaluationNotebook[], {TaggingRules, "Openers", #},Open]]]]],
     Alternatives@@onetimesections,
     newnb = ReplaceAll[newnb, Cell[CellGroupData[{Cell[td_,s___,#,rest___],r___},___]]:>
         Cell[CellGroupData[{Cell[TextData[Flatten[Join[{Cell[BoxData[DynamicBox[ToBoxes[If[
           MatchQ[CurrentValue[EvaluationNotebook[], {TaggingRules, "Openers", #}],Open|True],
           openicon,
           closedicon]]]]],Cell[BoxData[ToBoxes[Spacer[1]]]]},{td}]]],s,#,rest], r}, 
           Dynamic[CurrentValue[EvaluationNotebook[], {TaggingRules, "Openers", #},Open]]]]],
     _,
     newnb = ReplaceAll[newnb, Cell[s_String, some___, #, rest___] :> Cell[TextData[s],some,#,rest]];
     cells = Cases[newnb, Cell[CellGroupData[{Cell[TextData[_],___,#,___],___},___]], Infinity];
     Which[MatchQ[cells,{}], newnb,
      (Length[cells]===1)&&!StringMatchQ[#,"Example*ection"],
       newnb = ReplaceAll[newnb, Cell[CellGroupData[{Cell[TextData[td_],s___,#,rest___],r___},___]]:>
         Cell[CellGroupData[{Cell[TextData[Flatten[Join[{Cell[BoxData[DynamicBox[ToBoxes[If[
           MatchQ[CurrentValue[EvaluationNotebook[], {TaggingRules, "Openers", #}],Open|True],
           openicon,
           closedicon]]]]],Cell[BoxData[ToBoxes[Spacer[1]]]]},{td}]]],s,#,rest], r}, 
           Dynamic[CurrentValue[EvaluationNotebook[], {TaggingRules, "Openers", #},Open]]]]],
      Length[cells]>1 || StringMatchQ[#,"Example*ection"],
        cells = Position[newnb, Cell[CellGroupData[{Cell[TextData[_],___,#,___],___},___]]];
        rules = Function[cell,With[{count=ToString[counter++], pos=cell},
          If[!MatchQ[pos,{}],Join[pos,{1,2}]-> 
          Dynamic[CurrentValue[EvaluationNotebook[], {TaggingRules, "Openers", #, count},Open]]]]]/@cells;
        rules = DeleteCases[rules, Null];
        newnb=ReplacePart[newnb, rules]
        ]]&/@styles;
       
    ]; newnb];

  groupopenersections = Join[onetimesections, {"WorkflowPlatform","WorkflowHeader","WorkflowNotesSection", "WorkflowNotesHeader", "NotesSection"}];
  expr = DynamicGroupOpeners[expr, groupopenersections]; 

  
  (* Add in opener graphics to cells missing them and attach them to their tagging rules *) 
  expr = With[{
      openicon = Get[DocsAsset["Graphics:Opener", "WorkflowSectionOpenerOpenOff.m"]],
      closedicon = Get[DocsAsset["Graphics:Opener", "WorkflowSectionOpenerClosedOff.m"]]
    },
        Fold[
        ReplaceAll[#, CellGroupData[{c:Cell[TextData[td_]/;FreeQ[td, DynamicBox[___]],rest___],l___}, 
    q:Dynamic[CurrentValue[EvaluationNotebook[],{TaggingRules, "Openers", s_/;StringMatchQ[s,#2], n_String/;StringMatchQ[n,DigitCharacter..]},___]]]:>
    CellGroupData[{Cell[TextData[Flatten[Join[{Cell[BoxData[DynamicBox[ToBoxes[If[
           MatchQ[q[[1]],Open|True],
           openicon,
           closedicon]]]]],Cell[BoxData[ToBoxes[Spacer[1]]]]},{td}]]],rest],l},q]]&, expr, 
           Complement[groupopenersections,Flatten[Join[{},onetimesections]]]]];
    (* Remove CellDingbats from guides *)
    If[StringMatchQ["EntityType"/.info, "Workflow Guide"], 
      expr = ReplaceAll[expr, Cell[c_, some___, _[CellDingbat,_], rest___]:>Cell[c,some,rest]]];
    
    (* Remove broken links *)
    (*expr = DeleteCases[expr, Cell[c_, ___, "WorkflowGuideEntry" | "WorkflowRelatedWorkflows" | "SeeAlso", ___] /; !FreeQ[c, s_String/;(StringMatchQ[s, "paclet:*workflow*/*"] && 
        !StringQ[Documentation`ResolveLink[s]])], Infinity];*)
    
    expr = ReplaceAll[expr, stylerules];
    expr]


(******************************************************************************)
(** DuplicateWorkflowContent **)
(** code that looks for headers with [] or [platform name] and duplicates content from above **)
(** This code is needed to keep from having to maintain multiple places in the source with the same content **)

DuplicateWorkflowContent[expr_Notebook]:=
  Module[{newexpr = expr, generalnames={}, platformspecificnames={}, all, styles = Alternatives["WorkflowHeader","WorkflowStep"]},
    
    (* first, any entire platform sections *)
    If[!FreeQ[newexpr, Cell[co_, ___, "WorkflowPlatform", ___]/;MemberQ[co, s_String/;StringMatchQ[s, "*[*]"], Infinity]],  
      all = Cases[newexpr, StyleBox[_,"WorkflowPlatform", ___], Infinity];
      generalnames = DeleteDuplicates[Transmogrify`ConvertToString/@Select[all, !FreeQ[#, s_String/; StringMatchQ[StringTrim[s], "*[]"]]&]];
      platformspecificnames = DeleteDuplicates[Transmogrify`ConvertToString/@Select[all, !FreeQ[#, s_String/; StringMatchQ[StringTrim[s], "*["~~__~~"]"]]&]];
        
      platformspecificnames = With[{platform = StringReplace[#, (__~~"[")|"]"->""], originalplat = StringTrim[StringReplace[#, ("["~~__~~"]")->""]]},Module[{found = 
       Cases[newexpr, 
           Cell[CellGroupData[{Cell[co_, some___, "WorkflowPlatform", res___]/;MemberQ[co, StyleBox[platform, ___, "WorkflowPlatform", ___], Infinity],__},___],___],Infinity]
        },
        If[found =!= {}, {#, ReplaceAll[found[[1]], StyleBox[platform, some___, "WorkflowPlatform", rest___]:>StyleBox[originalplat, some, "WorkflowPlatform", rest]]}, Nothing]]] & /@ platformspecificnames;
        
      (newexpr = ReplaceRepeated[newexpr, Cell[c_, some___, "WorkflowPlatform", rest___]/;!FreeQ[c, #[[1]]] :> #[[2]]])&/@platformspecificnames;
    ];  
    
    If[!FreeQ[Cases[newexpr, Cell[__,styles,___], Infinity], s_String/;StringMatchQ[StringTrim[s],"*[]"|"*[*]"]],
      all = Cases[newexpr, Cell[__,styles,___], Infinity];
      generalnames = DeleteDuplicates[Transmogrify`ConvertToString/@Select[all, !FreeQ[#, s_String/; StringMatchQ[StringTrim[s], "*[]"]]&]];
      platformspecificnames = DeleteDuplicates[Transmogrify`ConvertToString/@Select[all, !FreeQ[#, s_String/; StringMatchQ[StringTrim[s], "*["~~__~~"]"]]&]];
      
      generalnames = Module[{found = Cases[newexpr, 
        Cell[CellGroupData[{Cell[c_, rest___]/;!FreeQ[{Transmogrify`ConvertToString@c}, StringTrim[StringReplace[Transmogrify`ConvertToString@#,"[]"->""]]], ___},___]],Infinity]},
        If[found =!= {}, {#, found[[1]]}, Nothing]] & /@ generalnames;
      
      platformspecificnames = With[{platform = StringReplace[#, (__~~"[")|"]"->""]},Module[{found = Cases[
       Cases[newexpr, 
           Cell[CellGroupData[{Cell[co_, some___, "WorkflowPlatform", res___]/;MemberQ[co, platform, Infinity],__},___],___],Infinity][[1]], 
        Cell[CellGroupData[{Cell[c_, rest___]/;!FreeQ[{c}, StringTrim[StringReplace[#,"["~~__~~"]"->""]]], ___},___]],Infinity]},
        If[found =!= {}, {#, found[[1]]}, Nothing]]] & /@ platformspecificnames;
      
      (* Because of nesting [], have to do this more than once, and have to do multiple ReplaceRepeated passes to pick everything up, apparently *)  
      Do[(newexpr = ReplaceRepeated[newexpr, Cell[c_, some___, styles, rest___]/;!FreeQ[Transmogrify`ConvertToString@c, #[[1]]] :> #[[2]]])&/@generalnames, Length[styles]];
      Do[(newexpr = ReplaceRepeated[newexpr, Cell[c_, some___, styles, rest___]/;!FreeQ[c, #[[1]]] :> #[[2]]])&/@platformspecificnames, Length[styles]];
    ];
    
    
    newexpr  
  ];


(******************************************************************************)
(** AddWorkflowStepDelimiters **)
(** code that looks in headers for WorkflowStep cells and adds WorkflowDelimiter cells between the steps **)

AddWorkflowStepDelimiters[expr_Notebook] :=
	Module[{headers, newheaders, newexpr = expr},
		newexpr = If[FreeQ[expr, Cell[__, "WorkflowStep", ___]], expr,
			headers = Cases[expr, Cell[CellGroupData[{Cell[__, "WorkflowHeader", ___], __}, ___]], Infinity];
			headers = Select[headers, MemberQ[#, Cell[__, "WorkflowStep", ___], Infinity]&];
			newheaders = ReplaceRepeated[headers, 
				{some___, step1:Cell[CellGroupData[{Cell[__, "WorkflowStep", ___], __}, ___]], step2:Cell[CellGroupData[{Cell[__, "WorkflowStep", ___], __}, ___]], rest___} :>
				{some, step1, Cell["", "WorkflowDelimiter"], step2, rest}];
		    If[headers =!= newheaders, newexpr = ReplacePart[newexpr, Thread[(Position[newexpr,#]&/@headers) -> newheaders]]];
		    newexpr
		];
		newexpr
	];


(******************************************************************************)
(** LockDownWorkflowAnnotations **)
(** code that prevents editing of annotations in workflows **)
LockDownWorkflowAnnotations[expr_Notebook] :=
	Module[{},
		ReplaceAll[expr, Cell[(bd:(TextData|BoxData))[c_], some___, "AnnotatedInput", rest___] :>
			Cell[bd[TagBox[c, Deploy, DefaultBaseStyle->"Deploy"]], some, "AnnotatedInput", rest]]
	];


(******************************************************************************)
(** WorkflowFinisher **)
(** code that runs at the end of MakeNotebook, just for Workflows **)

(* First, some definitions: *)
BuildRelatedFunctions[fcns_List]:=
  If[fcns==={},
   {},
   Cell[TextData[Riffle[Cell[BoxData[TemplateBox[{#,"paclet:ref/"<>#},"RefLink"]]]&/@fcns," \[FilledVerySmallSquare] "]],
     "WorkflowRelatedFunctions", TaggingRules->{"Built"->True}]];

BuildRelatedFunctions[single_String]:= BuildRelatedFunctions[{single}];

BuildRelatedFunctions[cell_Cell]:=
  Module[{stringfcns, existinglinks, return, full, tr},
   tr = Quiet@TrueQ["Built"/.(TaggingRules/.Options[cell, TaggingRules])];
   If[TrueQ[tr],
     return = cell,
   Which[MatchQ[cell, Cell[TextData[Cell[__]], ___]],
    return = cell,
    MatchQ[cell, Cell[TextData[ButtonBox[_String, ___, ButtonData->_String,___]],___, "WorkflowRelatedFunctions",___]],
    return = Cell[BoxData[TemplateBox[Flatten[Cases[cell, ButtonBox[name_String, ___, ButtonData->uri_String, ___]:>{name,uri},Infinity]], "RefLink"]],"WorkflowRelatedFunctions"],
    
    True,
    return = Check[
    existinglinks = Cases[cell, Cell[__, "InlineFormula", ___]|ButtonBox[__, BaseStyle->"Link", ___], Infinity];
    existinglinks={Last[Position[DeleteCases[ReplaceAll[DeleteCases[cell,s_String/;StringMatchQ[StringTrim[s],"."], Infinity],st_String/;!StringFreeQ[st,"."]:>Sequence@@StringTrim/@StringSplit[st,"."]],"",Infinity],#][[1]]],#}&/@existinglinks;
    stringfcns = DeleteCases[StringTrim/@Flatten[StringSplit[#,"."]&/@Select[Flatten@Cases[cell, TextData[{c__}]:>Cases[{c},s_String :> s]],!StringMatchQ[StringTrim[#],"."]&]],""];
    full = Fold[Insert[#, #2[[2]], #2[[1]]]&, stringfcns, existinglinks];
    full = Replace[full, 
      {s_String :> Cell[BoxData[TemplateBox[{s, "paclet:ref/"<>s}, "RefLink"]]],
       ButtonBox[name_String, ___, ButtonData->uri_String, ___]:>Cell[BoxData[TemplateBox[{name, uri}, "RefLink"]]]},{1}];
    Cell[TextData[Riffle[full, " \[FilledVerySmallSquare] "]], "WorkflowRelatedFunctions", TaggingRules->{"Built"->True}],
    cell]
    ]];
    
    return    
  ];
  
BuildRelatedFunctions[{cell_Cell}] := BuildRelatedFunctions[cell];
     
{desktopbutton,cloudbutton,mobilebutton}=
  {
  
   FrameBox[ItemBox[
    StyleBox["\"\\!\\(\\*\\nGraphicsBox[\\n StyleBox[FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {1, 3, 3}, {0, 1, 0}, {1, 3, 3}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {1, 3, 3}, {0, 1, 0}, {1, 3, 3}}}, {{{18., 5.}, {2., 5.}, {2., 16.}, {18., 16.}}, {{17.5, 17.}, {2.5, 17.}, {1.6720000000000002`, 16.999}, {1.001, 16.328}, {1., 15.5}, {1., 4.5}, {1.001, 3.6719999999999997`}, {1.6720000000000002`, 3.001}, {2.5, 3.}, {7., 3.}, {7., 2.}, {4., 2.}, {4., 1.}, {16., 1.}, {16., 2.}, {13., 2.}, {13., 3.}, {17.5, 3.}, {18.328, 3.001}, {18.999, 3.6719999999999997`}, {19., 4.5}, {19., 15.5}, {18.999, 16.328}, {18.328, 16.999}, {17.5, 17.}}}], \\\"iconColors\\\"],\\nAspectRatio->Automatic,\\nImagePadding->{{0, 0}, {8, 0}},\\nImageSize->{30.4140625, 28.},\\nPlotRange->{{0., 20.}, {0., 18.}}]\\)Desktop\"","SelectedPlatformTab",StripOnInput->False],StripOnInput->False],
     Background->Dynamic[If[CurrentValue[EvaluationNotebook[],{TaggingRules,"SelectedPlatform"}]==="Desktop",If[CurrentValue["MouseOver"],RGBColor[1,1,1,0.8],White],RGBColor[1,1,1,0.25]],ImageSizeCache->{9.,{3.,6.}}],FrameStyle->Dynamic[If[CurrentValue["MouseOver"]&&CurrentValue[EvaluationNotebook[],{TaggingRules,"SelectedPlatform"}]==="Desktop",Directive[1,RGBColor[1,1,1,0.]],Directive[1,RGBColor[1,1,1,1]]],ImageSizeCache->{106.,{3.,9.}}],FrameMargins->{{10,10},{4,6}},RoundingRadius->0,StripOnInput->False],
   
   FrameBox[ItemBox[
    StyleBox["\"\\!\\(\\*\\nGraphicsBox[\\n StyleBox[FilledCurveBox[{{{1, 4, 3}, {1, 3, 3}, {0, 1, 0}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}, {0, 1, 0}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}, {0, 1, 0}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}, {0, 1, 0}, {0, 1, 0}, {1, 3, 3}}, {{0, 2, 0}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}}}, {{{6.2891, 12.2559}, {4.399100000000001, 12.215900000000001`}, {2.897100000000001, 10.6559}, {2.9311000000000007`, 8.7659}, {2.9311000000000007`, 8.5709}, {2.9471000000000007`, 8.376900000000001}, {2.978100000000001, 8.184899999999999}, {3.0471000000000004`, 7.741900000000001}, {2.6341000000000006`, 7.591900000000002}, {1.6441000000000008`, 7.043900000000001}, {1.0371000000000006`, 5.9949}, {1.0530999999999997`, 4.863900000000001}, {1.0530999999999997`, 3.4839}, {2.132100000000001, 1.9999000000000002`}, {3.4091000000000005`, 1.9999000000000002`}, {16.5441, 1.9999000000000002`}, {17.8471, 1.9999000000000002`}, {18.9471, 3.5059000000000005`}, {18.9471, 4.913900000000001}, {18.9571, 5.9369000000000005`}, {18.4341, 6.890900000000001}, {17.5661, 7.4319000000000015`}, {17.1491, 7.7159}, {17.2941, 8.2059}, {17.3491, 8.3809}, {17.3761, 8.5629}, {17.3771, 8.7459}, {17.3931, 9.6319}, {16.6931, 10.3659}, {15.807099999999998`, 10.3899}, {15.723099999999999`, 10.3919}, {15.638100000000001`, 10.3859}, {15.554100000000002`, 10.372900000000001`}, {15.066099999999999`, 10.289900000000001`}, {14.9541, 10.783900000000001`}, {14.568100000000001`, 12.6029}, {12.973099999999999`, 13.9109}, {11.1131, 13.930900000000001`}, {9.8481, 13.9289}, {8.6641, 13.308900000000001`}, {7.9411000000000005`, 12.270900000000001`}, {7.7141, 11.9529}, {7.3491, 12.0779}, {7.008100000000001, 12.1959}, {6.6501, 12.2559}, {6.2891, 12.2559}}, {{16.5441, 0.9999000000000002}, {3.4091000000000005`, 0.9999000000000002}, {1.5611000000000006`, 0.9999000000000002}, {0.00009999999999976694, 2.937900000000001}, {0.00009999999999976694, 4.902900000000001}, {-0.0118999999999998, 6.2879000000000005`}, {0.7061000000000002, 7.5769}, {1.8911000000000007`, 8.2949}, {1.8821000000000003`, 8.4039}, {1.8780999999999999`, 8.584900000000001}, {1.8780999999999999`, 8.6959}, {1.8361, 11.180900000000001`}, {3.8051000000000004`, 13.2349}, {6.2891, 13.2989}, {6.6371, 13.3049}, {6.984100000000001, 13.267900000000001`}, {7.3231, 13.1899}, {9.0401, 15.2959}, {12.1401, 15.6099}, {14.2461, 13.892900000000001`}, {15.0141, 13.2669}, {15.574100000000001`, 12.4229}, {15.8551, 11.472900000000001`}, {17.3081, 11.4199}, {18.4511, 10.215900000000001`}, {18.4301, 8.7629}, {18.4301, 8.622900000000001}, {18.4191, 8.4829}, {18.3971, 8.343900000000001}, {19.4181, 7.5149}, {20.007099999999998`, 6.2679}, {20.0001, 4.951900000000001}, {20.0001, 2.9609000000000005`}, {18.417099999999998`, 0.9999000000000002}, {16.5441, 0.9999000000000002}}}], \\\"iconColorsUnselected\\\"],\\nAspectRatio->Automatic,\\nImagePadding->{{0.5, 0.5}, {8, 0}},\\nImageSize->{31.2265625, 26.},\\nPlotRange->{{0., 20.}, {0., 16.}}]\\)Cloud\"","UnselectedPlatformTab",StripOnInput->False],StripOnInput->False],
     Background->Dynamic[If[CurrentValue[EvaluationNotebook[],{TaggingRules,"SelectedPlatform"}]==="Cloud",If[CurrentValue["MouseOver"],RGBColor[1,1,1,0.8],White],RGBColor[1,1,1,0.25]],ImageSizeCache->{9.,{3.,6.}}],FrameStyle->Dynamic[If[CurrentValue["MouseOver"]&&CurrentValue[EvaluationNotebook[],{TaggingRules,"SelectedPlatform"}]==="Cloud",Directive[1,RGBColor[1,1,1,0.]],Directive[1,RGBColor[1,1,1,1]]],ImageSizeCache->{106.,{3.,9.}}],FrameMargins->{{10,10},{5,6}},RoundingRadius->0,StripOnInput->False],
     
   FrameBox[ItemBox[
    StyleBox["\"\\!\\(\\*\\nGraphicsBox[\\n StyleBox[FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {1, 3, 3}, {0, 1, 0}, {1, 3, 3}, {0, 1, 0}, {1, 3, 3}, {0, 1, 0}, {1, 3, 3}}}, {{{12., 4.}, {2., 4.}, {2., 17.}, {12., 17.}}, {{8., 2.}, {6., 2.}, {6., 3.}, {8., 3.}}, {{11.5, 19.}, {2.5, 19.}, {1.671, 19.}, {1., 18.329}, {1., 17.5}, {1., 2.5}, {1., 1.6720000000000002`}, {1.671, 1.}, {2.5, 1.}, {11.5, 1.}, {12.328, 1.}, {13., 1.6720000000000002`}, {13., 2.5}, {13., 17.5}, {13., 18.329}, {12.328, 19.}, {11.5, 19.}}}], \\\"iconColorsUnselected\\\"],\\nAspectRatio->Automatic,\\nImagePadding->{{0, 0}, {8, 0}},\\nImageSize->{15.7421875, Automatic},\\nPlotRange->{{0., 14.}, {0., 20.}}]\\)  Mobile\"","UnselectedPlatformTab",StripOnInput->False],StripOnInput->False],
     Background->Dynamic[If[CurrentValue[EvaluationNotebook[],{TaggingRules,"SelectedPlatform"}]==="Mobile",If[CurrentValue["MouseOver"],RGBColor[1,1,1,0.8],White],RGBColor[1,1,1,0.25]],ImageSizeCache->{9.,{3.,6.}}],FrameStyle->Dynamic[If[CurrentValue["MouseOver"]&&CurrentValue[EvaluationNotebook[],{TaggingRules,"SelectedPlatform"}]==="Mobile",Directive[1,RGBColor[1,1,1,0.]],Directive[1,RGBColor[1,1,1,1]]],ImageSizeCache->{106.,{3.,9.}}],FrameMargins->{{10,10},{3,5}},RoundingRadius->0,StripOnInput->False]
     
  };

{desktopiconsel, desktopiconoff, desktopiconover} = Import[DocsAsset["Graphics:WorkflowTab", "WorkflowDesktopTab"<>#<>".m"]]& /@ {"Selected", "Unselected-off","Unselected-over"};
{cloudiconsel, cloudiconoff, cloudiconover}       = Import[DocsAsset["Graphics:WorkflowTab", "WorkflowCloudTab"<>#<>".m"  ]]& /@ {"Selected", "Unselected-off","Unselected-over"};
{mobileiconsel, mobileiconoff, mobileiconover}    = Import[DocsAsset["Graphics:WorkflowTab", "WorkflowMobileTab"<>#<>".m" ]]& /@ {"Selected", "Unselected-off","Unselected-over"};

desktoptab[language_String] := (desktoptab[language] = Module[{},
  PaneSelectorBox[
    {
      False -> PaneSelectorBox[
        {
          False -> TagBox[FrameBox[GridBox[{{desktopiconoff,  StyleBox[Localized["WorkflowDesktopTab", language], "UnselectedPlatformTab"                                 ]}},GridBoxAlignment->{"Rows"->{{Bottom}}},GridBoxSpacings->{"Rows"->{{1}}}],FrameStyle->GrayLevel[0.9], ImageSize->{Automatic,38}], MouseAppearanceTag["LinkHand"] ],
          True  -> TagBox[FrameBox[GridBox[{{desktopiconover, StyleBox[Localized["WorkflowDesktopTab", language], "UnselectedPlatformTab", FontColor->GrayLevel[0.749019] ]}},GridBoxAlignment->{"Rows"->{{Bottom}}},GridBoxSpacings->{"Rows"->{{1}}}],FrameStyle->GrayLevel[0.9], ImageSize->{Automatic,38}], MouseAppearanceTag["LinkHand"] ]
        },
        Dynamic@CurrentValue["MouseOver"]
      ],
      True -> FrameBox[GridBox[{{desktopiconsel, StyleBox[Localized["WorkflowDesktopTab", language], "SelectedPlatformTab"]}},GridBoxAlignment->{"Rows"->{{Bottom}}},GridBoxSpacings->{"Rows"->{{1}}}],FrameStyle->GrayLevel[0.9],ImageSize->{Automatic,38}]
    },
    Dynamic[CurrentValue[EvaluationNotebook[], {TaggingRules, "SelectedPlatform"}] === "Desktop"]
  ]
]);

cloudtab[language_String] := (

  cloudtab[language] = PaneSelectorBox[
    {
      False -> PaneSelectorBox[
        {
          False -> TagBox[FrameBox[GridBox[{{AdjustmentBox[cloudiconoff,  BoxBaselineShift->-0.4], StyleBox[Localized["WorkflowCloudTab", language], "UnselectedPlatformTab"                                 ]}},GridBoxAlignment->{"Rows"->{{Baseline}}},GridBoxSpacings->{"Rows"->{{1}}}],FrameStyle->GrayLevel[0.9], ImageSize->{Automatic,38}], MouseAppearanceTag["LinkHand"] ],
          True  -> TagBox[FrameBox[GridBox[{{AdjustmentBox[cloudiconover, BoxBaselineShift->-0.4], StyleBox[Localized["WorkflowCloudTab", language], "UnselectedPlatformTab", FontColor->GrayLevel[0.749019] ]}},GridBoxAlignment->{"Rows"->{{Baseline}}},GridBoxSpacings->{"Rows"->{{1}}}],FrameStyle->GrayLevel[0.9], ImageSize->{Automatic,38}], MouseAppearanceTag["LinkHand"] ]
        },
        Dynamic@CurrentValue["MouseOver"]
      ],
      True -> FrameBox[GridBox[{{AdjustmentBox[cloudiconsel, BoxBaselineShift->-0.4], StyleBox[Localized["WorkflowCloudTab", language], "SelectedPlatformTab"]}},GridBoxAlignment->{"Rows"->{{Baseline}}},GridBoxSpacings->{"Rows"->{{1}}}],FrameStyle->GrayLevel[0.9], ImageSize->{Automatic,38}]
    },
    Dynamic[CurrentValue[EvaluationNotebook[], {TaggingRules, "SelectedPlatform"}] === "Cloud"]
  ];
);

mobiletab[language_String] := (mobiletab[language] = PaneSelectorBox[{False ->
  PaneSelectorBox[{False -> TagBox[FrameBox[
    GridBox[{{AdjustmentBox[mobileiconoff, BoxBaselineShift->-0.4], StyleBox[Localized["WorkflowMobileTab", language], "UnselectedPlatformTab"]}},GridBoxAlignment->{"Rows"->{{Baseline}}},GridBoxSpacings->{"Rows"->{{1}}}],FrameStyle->GrayLevel[0.9], ImageSize->{Automatic,38}], MouseAppearanceTag["LinkHand"] ],
    True -> TagBox[FrameBox[
      GridBox[{{AdjustmentBox[mobileiconover, BoxBaselineShift->-0.4], StyleBox[Localized["WorkflowMobileTab", language], "UnselectedPlatformTab", FontColor->GrayLevel[0.749019] ]}},GridBoxAlignment->{"Rows"->{{Baseline}}},GridBoxSpacings->{"Rows"->{{1}}}],FrameStyle->GrayLevel[0.9], ImageSize->{Automatic,38}], MouseAppearanceTag["LinkHand"] ]}, Dynamic@CurrentValue["MouseOver"] ],
  True -> FrameBox[
    GridBox[{{AdjustmentBox[mobileiconsel, BoxBaselineShift->-0.4], StyleBox[Localized["WorkflowMobileTab", language], "SelectedPlatformTab"]}},GridBoxAlignment->{"Rows"->{{Baseline}}},GridBoxSpacings->{"Rows"->{{1}}}],FrameStyle->GrayLevel[0.9], ImageSize->{Automatic,38}]}, Dynamic[CurrentValue[EvaluationNotebook[], {TaggingRules, "SelectedPlatform"}] === "Mobile"] ]);

Options[WorkflowFinisher] = {"Web" -> False, "Language" -> "English"};

WorkflowFinisher[nbobj_NotebookObject, opts:OptionsPattern[]] := Module[{
	nbexpr = NotebookGet[nbobj],
	newobj
},
	NotebookClose[nbobj];
	nbexpr = WorkflowFinisher[nbexpr, opts];
	newobj = NotebookPut[nbexpr];
	newobj
];

WorkflowFinisher[nbexpr_Notebook, opts:OptionsPattern[]] := Module[{
	template,
	contents, tabs,
	footersections,
	result,
	nb = nbexpr,
	webQ, language,
	time
},
   Global`AntLog["Starting WorkflowFinisher..."];

	time = First[AbsoluteTiming[
		webQ = OptionValue["Web"];
		language = OptionValue["Language"];

		If[TrueQ[webQ], (* for 11.3, just do bare minimum finishing on workflows for web build nbs *)
			Module[{relfun},
				nb = DocumentationBuild`Common`PubsEvaluateWithFE @ WorkflowRenumberInOut[nb];

				relfun = BuildRelatedFunctions[
					Quiet[(
						Which[
							MatchQ[#[[1]], s_String /; StringMatchQ[s, __ ~~ "." ~~ __]],
								StringTrim /@ StringSplit[#[[1]], "."],
							MatchQ[#[[1]], _String],
								StringTrim[#[[1]]],
							True,
								#
						]& /@ Cases[nb, Cell[__, "WorkflowRelatedFunctions", ___], Infinity]
					)][[1]]
				];

				result = ReplaceAll[nb, Cell[__, "WorkflowRelatedFunctions", ___] -> relfun];
			]
			,
			(* Formatting note: c038dc86: fix bug 347575 *)
			tabs = StringTrim /@ Cases[nb, StyleBox[s_String, "WorkflowPlatform"] :> s, Infinity];

			DocsAssert[AllTrue[ToUpperCase[tabs], StringMatchQ["DESKTOP" | "CLOUD" | "MOBILE"]]];

			nb = DocumentationBuild`Common`PubsEvaluateWithFE @ WorkflowRenumberInOut[nb];

			template = Get[DocsAsset[
				"WorkflowTemplate",
				StringJoin[
					If[MatchQ[tabs, {}],
						"WorkflowTemplateNoTabs",
						"WorkflowTemplate"
					],
					Switch[language,
						"Japanese",
							"_jp",
						"ChineseSimplified",
							"_zh",
						_,
							""
					],
					".nb"
				]
			]];

			contents = <|
				"Title" -> Cases[nb, Cell[__, "WorkflowTitle", ___], Infinity],
				Map[
					Function[tab, tab -> Cases[
						nb,
						Cell[CellGroupData[
							{
								Cell[c_, ___] /; !FreeQ[
									c,
									StyleBox[s_String /; StringMatchQ[StringTrim[s], tab], "WorkflowPlatform"]
								],
								rest__
							},
							___
						]] :> {rest},
						Infinity
					]],
					tabs
				],
				(* Formatting note: d1764ab6: fix bug 347580 (missing workflow descriptions) *)
				If[MatchQ[tabs, {}],
					"FullContent" -> Cases[
						nb,
						Cell[CellGroupData[{Cell[__, "WorkflowTitle", ___], rest___}, ___], ___]
							:> rest,
						Infinity
					]
					,
					"WorkflowDescription" -> Cases[nb, Cell[__, "WorkflowDescription", ___], Infinity]
				],
				"RelatedWorkflows" -> Cases[nb, Cell[__,"WorkflowRelatedWorkflows", ___], Infinity],
				"RelatedFunctions" -> Quiet[(
					Which[
						MatchQ[#[[1]], s_String /; StringMatchQ[s, __ ~~ "." ~~ __]],
							StringTrim /@ StringSplit[#[[1]], "."],
						MatchQ[#[[1]], _String],
							StringTrim[#[[1]]],
						True,
							#
					]& /@ Cases[nb, Cell[__, "WorkflowRelatedFunctions", ___], Infinity]
				)],
				"RelatedGuides" -> Cases[nb, Cell[__, "WorkflowRelatedGuides", ___], Infinity],
				(* Formatting note: f6f64dfa: fix to include TechNotes in workflows (bug 345785 and others) *)
				"RelatedTechNotes" -> Cases[nb, Cell[__, "WorkflowTechNotes", ___], Infinity],
				"RelatedLinks" -> Cases[nb, Cell[__, "WorkflowRelatedLinks", ___], Infinity],
				"AnchorBar" -> Cases[
					nb,
					Cell[s__, "AnchorBar", r___] :> Cell[
						ReplaceAll[s, FrameBox[c__] :> FrameBox[c, FrameStyle -> None]],
						"AnchorBar",
						r
					],
					Infinity
				],
				"Metadata" -> DeleteCases[Options[nb], _[StyleDefinitions, _]]
			|>;

			footersections = Select[
				{"RelatedWorkflows", "RelatedFunctions", "RelatedGuides", "RelatedTechNotes"},
				!MatchQ[contents[#], {}]&
			];

			result = DeleteCases[
				template,
				Cell[__,"Input"|"Section"|"Output"|"Text",___],
				Infinity
			];

			result = ReplaceAll[result,
				{
					If[!MatchQ[tabs,{}],
						Sequence @@ Map[
							Function[tab,
								Cell[ToUpperCase[tab] <> " PLACEHOLDER", ___]
									-> Sequence @@ (contents[tab][[1]])
							],
							tabs
						]
						,
						Cell["CONTENT PLACEHOLDER", ___] -> Sequence @@ contents["FullContent"]
					],
					Cell["DESCRIPTION PLACEHOLDER", ___] -> If[!MatchQ[contents["WorkflowDescription"], {}],
						Sequence @@ contents["WorkflowDescription"]
						,
						Cell["DELETESECTION"]
					],
					"TITLEPLACEHOLDER" -> contents["Title"][[1, 1]],

					Cell["RELATEDFUNCTIONS PLACEHOLDER", ___] -> Module[{
						rel = If[contents["RelatedFunctions"] === {},
							{},
							BuildRelatedFunctions[contents["RelatedFunctions"][[1]]]
						]
					},
						If[rel === {},
							Cell["DELETESECTION"],
							rel
						]
					],

					Cell["SEEALSO PLACEHOLDER", ___] -> If[!MatchQ[contents["RelatedWorkflows"], {}],
						Sequence @@ contents["RelatedWorkflows"]
						,
						Cell["DELETESECTION"]
					],
					Cell["RELATEDGUIDES PLACEHOLDER", ___] -> If[!MatchQ[contents["RelatedGuides"], {}],
						Sequence @@ contents["RelatedGuides"]
						,
						Cell["DELETESECTION"]
					],
					Cell["RELATEDTECHNOTES PLACEHOLDER", ___] -> If[!MatchQ[contents["RelatedTechNotes"], {}],
						Sequence @@ contents["RelatedTechNotes"],
						Cell["DELETESECTION"]
					],
					Cell["RELATEDLINKS PLACEHOLDER", ___] -> If[!MatchQ[contents["RelatedLinks"], {}],
						Sequence @@ contents["RelatedLinks"],
						Cell["DELETESECTION"]
					],
					Cell[__, "AnchorBar", ___] -> Sequence @@ contents["AnchorBar"],
					Cell[__, "WorkflowTabGrid", ___] -> Cell[
						BoxData[GridBox[
							{{
								If[MemberQ[tabs, "Desktop"],
									ButtonBox[
										desktoptab[language],
										Evaluator -> Automatic,
										ButtonFunction :> (CurrentValue[ButtonNotebook[], {TaggingRules,"SelectedPlatform"}] = "Desktop"),
										Appearance -> None,
										ImageSize -> {Automatic, 38}
									],
									Nothing
								],
								If[MemberQ[tabs, "Cloud"],
									ButtonBox[
										cloudtab[language],
										Evaluator -> Automatic,
										ButtonFunction :> (CurrentValue[ButtonNotebook[], {TaggingRules,"SelectedPlatform"}] = "Cloud"),
										Appearance -> None,
										ImageSize -> {Automatic, 38}
									],
									Nothing
								],
								If[MemberQ[tabs, "Mobile"],
									ButtonBox[
										mobiletab[language],
										Evaluator -> Automatic,
										ButtonFunction :> (CurrentValue[ButtonNotebook[], {TaggingRules,"SelectedPlatform"}] = "Mobile"),
										Appearance -> None,
										ImageSize -> {Automatic, 38}
									],
									Nothing
								]
							}},
							GridBoxSpacings -> {"Columns" -> {{0}}, "Rows" -> {{0}}}
						]],
						"WorkflowTabGrid",
						"WorkflowDockedCell"
					]
				}
			];

			result = ReplaceAll[result, {
				Cell[s_String /; !StringFreeQ[s, "PLACEHOLDER"], ___]
					-> Cell["DELETESECTION"],
				Cell[__, "WorkflowPlatform", ___, CellOpen -> open_, ___, CellID -> id_, ___]
					:> Cell["", "WorkflowPlatform", CellID -> id, CellOpen -> open]
			}];

			result = DeleteCases[
				result,
				Cell[CellGroupData[
					{
						Cell[__, "WorkflowPlatform" | "FooterHeader" | "WorkflowHeader" | "WorkflowStep", ___],
						___,
						Cell["DELETESECTION"],
						___
					},
					___
				], ___],
				Infinity
			];

			result = DeleteCases[result, Cell["DELETESECTION"], Infinity];

			UsingFrontEnd @ Module[{nbobj},
				nbobj = NotebookPut[result];

				SetOptions[nbobj, Sequence @@ contents["Metadata"]];

				(* Formatting note: ae7de169: fix bug 384203 *)
				CurrentValue[nbobj, {TaggingRules, "SelectedPlatform"}] = Quiet[Check[First[tabs], None]];

				CurrentValue[nbobj, {TaggingRules, "Metadata", "WorkflowDockedCell"}] =
					Cell[BoxData[GridBox[
						Transpose[{CurrentValue[nbobj, DockedCells][[4;;]]}],
						GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Center}}}
					]]];

				result = NotebookGet[nbobj];

				NotebookClose[nbobj]
			];
		]
	]];

	Global`AntLog["WorkflowFinisher done! ("<>ToString[time]<>")"];

	result
]


(* WorkflowRenumberInOut -- a special function for workflows that will renumber the cell labels for in/outs *)
(* in: nb expr
   out: nb expr *)
WorkflowRenumberInOut[nb_Notebook] :=
  Module[{counter = 0, nbobj, cells, instart, outstart, inend, outend, info = Developer`CellInformation, newnb, newnbstr},
    instart = "In["; inend = "]:=";
    outstart = "Out["; outend = "]=";
    nbobj = NotebookPut[nb];
    cells = Cells[nbobj, CellStyle->"Input"|"Output"];
    If[!MatchQ[CurrentValue[#, CellLabel], None],
      Switch["Style" /. info[#],
        "Input",
        counter++;
        SetOptions[#, CellLabel->StringJoin[instart, ToString@counter, inend]],
        "Output",
        SetOptions[#, CellLabel->StringJoin[outstart, ToString@counter, outend]]
        ]
       ] & /@ cells;
     newnb = NotebookGet[nbobj];
     NotebookClose[nbobj];
     counter = 0;
     newnbstr = ToString[newnb, InputForm];
     (* clean the slate *)
     newnbstr = StringReplace[newnbstr, {instart~~DigitCharacter..~~inend -> instart<>"xxx"<>inend,
       outstart~~DigitCharacter..~~outend -> outstart<>"xxx"<>outend}];

     (* number them *)
     newnbstr=StringReplace[newnbstr,
       {s:("\""~~instart~~"xxx"~~inend):>
          (++counter;StringReplace[s,"xxx"->ToString[counter]]),
        s:("During evaluation"~~Shortest[__]~~instart~~"xxx"~~inend):>
          StringReplace[s,"xxx"->ToString[counter]],
        s:(outstart~~"xxx"~~outend):>
          StringReplace[s,"xxx"->ToString[counter]]}];
        
     If[SyntaxQ[newnbstr], newnb = ToExpression[newnbstr]];
     newnb
   ]
    


(******************************************************************************)
(** ConvertNotebook - Tutorials **)
ConvertNotebook[nb_, info_,
    ("Tutorial" | "Upgrade Information" | "Overview" | "Monograph" | "Monograph Overview" | "Tech Note"),
    opts___?OptionQ] := 
  Module[{expr, sections, uri, lang, pos, titlepos, jumpboxcell, jumparrow, jumparrowover, type},
	expr = nb;
    uri = "URI" /. info;
    uri = ToPacletURI[uri];
    lang = "Language" /. {opts} /. Options[MakeNotebook];
    type = "EntityType" /. info;
    sections = DocumentationBuild`Info`GetSourceInformation[uri, lang, "TutorialSections"];
    If[sections == None, sections = DocumentationBuild`Navigation`GetTutorialAnchors[uri, nb, lang]];
    If[Length[sections] > 1 && !MatchQ[type, "Overview" | "Monograph Overview"],
      jumparrow = ToBoxes[Import[DocsAsset["Graphics:Anchor", "JumpDownAnchorLinkArrowsPurple.m"]]];
      jumparrowover = ToBoxes[Import[DocsAsset["Graphics:Anchor", "JumpDownAnchorLinkArrowsPurple-over.m"]]];

      sections = 
        TemplateBox[{Cell[#[[1]]],uri<>"#"<>ToString[#[[2]]],jumparrow,jumparrowover}, "GrayLinkWithIcon", BaseStyle->{"TutorialJumpBoxLink"}]& /@ sections;
      sections = 
        Transpose[Partition[sections, Ceiling[Length[sections]/2], Ceiling[Length[sections]/2], {1,1}, ""]];
      titlepos = Position[expr, Cell[__, "Title", ___]];
      pos = MapAt[#+1&, titlepos, {1,-1}];
      jumpboxcell = Cell[BoxData[GridBox[sections]], "TutorialJumpBox"];
      expr = Insert[expr, jumpboxcell, pos];
    ];
    expr = expr /. Cell[a__, "TutorialCollectionSection", b___]:>Cell[a, "TutorialsTutorialCollectionSection", b];
    expr = expr /. Cell[con_,
      sty:Alternatives@@("CapitalizeHeadingStyles" /. {opts} /. Options[MakeNotebook]),
      o___] :>
    Cell[TextData@CapitalizeString[con], sty, o];
    
    BuildLog["-- Localize MoreAbout on tutorials"];
    expr = expr /. Cell[c_, "TutorialMoreAboutSection", o___] :> 
      Cell[Localized["TutorialMoreAboutSectionText", lang], "TutorialMoreAboutSection", o];
      
    BuildLog["-- Localize tutorial collection section on tutorials"];
    expr = expr /. Cell[c_, "TutorialsTutorialCollectionSection", o___] :> 
      Cell[Localized["TutorialCollectionSectionText", lang], "TutorialsTutorialCollectionSection", o];

    BuildLog["-- Localize Related Tutorials on tutorials"];
    expr = expr /. Cell[c_, "RelatedTutorialsSection", o___] :> 
      Cell[Localized["RelatedTutorialsSectionText", lang], "RelatedTutorialsSection", o];
      
   (* Monographs: convert styles, add header cell *)
   If[MemberQ[{"Monograph", "Monograph Overview"}, type],
     expr = ConvertMonographStyles[expr];
     expr = AddMonographHeader[expr, info, lang]];
 
   If[MemberQ[{"Monograph Overview"}, type],
     Module[{pncell}, 
       pncell = Cases[expr, ItemBox[item_, rest___]/;!FreeQ[item,Cell[__, "PacletNameCell", ___]], Infinity][[1]];
       expr = ReplaceAll[expr, {pncell -> "", Cell[title_, "MonographTOCDocumentTitle", rest___]:> 
         Cell[BoxData[GridBox[{{Cases[pncell, Cell[__, "PacletNameCell", ___], Infinity][[1]]}, {Cell[title, "MonographTOCDocumentTitle"]}}]], "MonographTOCDocumentTitle", rest]}]]
     ];
     
   (* Tech Notes: convert styles *)
   If[MemberQ[{"Tech Note"}, type],
     expr = ConvertTechNoteStyles[expr]];
 
   (* add attractor icons to footer sections *)
   (* first, hard remove any empty sections *)
   If[Cases[expr, Cell[CellGroupData[{Cell[__, #, ___], ___}, ___]], Infinity]==={}, expr = DeleteCases[expr, Cell[__, #, ___], Infinity]]& /@
   	 {"RelatedTutorialsSection", "TutorialMoreAboutSection", "TutorialRelatedLinksSection"};
    expr = expr /. c:Cell[CellGroupData[{Cell[__, "RelatedTutorialsSection"|"TutorialMoreAboutSection"|"TutorialRelatedLinksSection", ___], ___}, ___]] :> AddAttractorIcons[c, lang];
 
   (* Add updated/new graphic *)
   expr = AddObjectNameGraphic[expr, info, opts];
   
   expr
  ];
  


(******************************************************************************)
(** ConvertMonographStyles **)
ConvertMonographStyles[nb_Notebook]:=
 Module[{stylelist, stylerules, newnb = nb},
   stylelist = {"Title", "Text", "Section", "Subsection", "Subsubsection", "Subsubsubsection", "Subsubsubsubsection", "MathCaption", "TableHeader", 
   	 "TableText", "BulletedText", "Reference", "TOCDocumentTitle", "TOCChapter", 
     "TOCSection", "TOCSubsection"};
   stylerules = Thread[(Cell[some__, #, rest___]&/@stylelist) :> Evaluate@(Cell[some, #, rest]&/@(StringJoin["Monograph", #]& /@ stylelist))];
   newnb = ReplaceAll[newnb, stylerules];
   newnb = DeleteCases[newnb, Cell[__, "MonographTOCSubsection", ___], Infinity];
   newnb
 ]


(******************************************************************************)
(** ConvertTechNoteStyles **)
ConvertTechNoteStyles[nb_Notebook]:=
 Module[{stylelist, stylerules, newnb = nb},
   stylelist = {"Title", "TutorialJumpBox", "Section", "Text", "TableText", "Caption", "MathCaption"};
   stylerules = Thread[(Cell[some__, #, rest___]&/@stylelist) :> Evaluate@(Cell[some, #, rest]&/@(StringJoin["TechNote", StringReplace[#, "Tutorial"->""]]& /@ stylelist))];
   newnb = ReplaceAll[newnb, stylerules];
   newnb
 ]


(******************************************************************************)
(** AddMonographHeader **)
AddMonographHeader[nb_Notebook, info_, lang_:"English"]:=
 Module[{pos, newnb = nb, cell, monographname, title, uri, parent},
  title = "Title" /. info;
  uri = "URI" /. info;
  If[!StringMatchQ[uri, "*Overview"],
  parent = Module[{tmp = (DocumentationBuild`Info`GetSourceInformation["paclet:"<>uri,lang,"LinkTrails"])},
    If[tmp === None || Length[tmp[[1]]]<=1,
      None,
      tmp[[1, -2]]]];
  If[parent =!= None,
  monographname = DocumentationBuild`Info`GetSourceInformation[parent, lang, "Title"];
  pos = Position[newnb, Cell[__, "MonographTitle", ___]];
  cell = Cell[BoxData[GridBox[{{Cell[monographname, "MonographHeader"], 
    ItemBox["",ItemSize->"Fit"], MakeMonographButtonList[uri, parent, lang]},
    {"", "", ""}}]], "MonographHeaderGrid"];
  newnb = Insert[newnb, cell, pos]]];
  newnb]


(******************************************************************************)
(** MakeMonographButtonList **)
MakeMonographButtonList[uri_String, parent_String, lang_:"English"]:=
 Module[{navdata, back, forward, ret, siblings},
   navdata = DocumentationBuild`Info`GetSourceInformation[uri, lang, "ForwardBackNavData"];
   ret = If[MatchQ[navdata, {}|None|"None"], "",
     (* there's nav data to process *)
     {back, forward} = {"Back", "Forward"} /. navdata;
     (* if pointing at the next overview, cut link *)
     If[StringMatchQ[forward, "*Overview"], forward = None];
     
     (* populate hamburger menu list *)
     siblings = DocumentationBuild`Info`GetSourceInformation[parent, "English", "Links", "Chapters"];
     siblings = Replace[siblings,{TextData[bb:ButtonBox[n_,___,ButtonData->bd_]],___}:>{n,bd},{1}];
     
     Cell[BoxData[RowBox[{
       If[back =!= None, TagBox[ButtonBox[
         PaneSelectorBox[{False -> DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation", "Monographs"}, "Monograph-Back.png"]]],
           True -> DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation", "Monographs"}, "Monograph-Back-over.png"]]]},
           Dynamic[CurrentValue["MouseOver"]]], BaseStyle->"Link", ButtonData->back], MouseAppearanceTag["LinkHand"]],
         TagBox[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation", "Monographs"}, "Monograph-Back-inactive.png"]]], Deploy, DefaultBaseStyle->"Deploy"]
         ], TemplateBox[{3},"Spacer1"],
       
       TagBox[MakeActionMenu[PaneSelectorBox[{False -> DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation", "Monographs"}, "Monograph-Hamburger.png"]]],
           True -> DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation", "Monographs"}, "Monograph-Hamburger-over.png"]]]},
           Dynamic[CurrentValue["MouseOver"]]], {"TOCChapter", FontSize->14}, siblings, FieldSize->{Automatic,{1.5, 10}}], MouseAppearanceTag["LinkHand"]],TemplateBox[{3},"Spacer1"],
           
       If[forward =!= None, TagBox[ButtonBox[
         PaneSelectorBox[{False -> DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation", "Monographs"}, "Monograph-Forward.png"]]],
           True -> DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation", "Monographs"}, "Monograph-Forward-over.png"]]]},
           Dynamic[CurrentValue["MouseOver"]]], BaseStyle->"Link", ButtonData->forward], MouseAppearanceTag["LinkHand"]],
         TagBox[DynamicBox[FEPrivate`ImportImage[FrontEnd`FileName[{"Documentation", "Monographs"}, "Monograph-Forward-inactive.png"]]], Deploy, DefaultBaseStyle->"Deploy"]
         ]}]]]
   ];
   
   ret
 ]


(******************************************************************************)
(** ConvertNotebook - HowTo **)
ConvertNotebook[nb_, info_,
    ("HowTo"),
    opts___?OptionQ] := 
  Module[{expr, language, obsolescenceNote, screenshot, screenuri, warnicon},
	expr = nb;
	language = "Language" /. {opts} /. Options[MakeNotebook];
	
  obsolescenceNote = Cases[expr, Cell[_, "ObsolescenceNote", ___], Infinity];
  expr = DeleteCases[expr, Cell[_, "ObsolescenceNote", ___], Infinity];

  expr = expr /. Cell[p_, "Title", o___] :>
    Cell[
      TextData[Join[
        {
          Cell[Localized["HowToPrefix", language], "HowToTitlePrefix"],
          " "
        },
        If[Head[p] === TextData, First[p], {p}]
      ]],
      "HowToTitle", o];
  expr = expr /. Cell[p_, "Text", o___] :> Cell[p, "HowToText", o];

  (* Add ObsolescenceNote *)
  warnicon = Image[Import[DocsAsset["Images", "ObsoleteIcon2x.png"], "PNG"], Magnification->0.5];

  obsolescenceNote = If[obsolescenceNote === {}, obsolescenceNote, {Cell[BoxData[GridBox[{{Cell[BoxData[ToBoxes@warnicon]], Cell[obsolescenceNote[[1, 1]]]}}]], Sequence@@Rest[obsolescenceNote[[1]]]]}];
  expr = expr /. 
    Cell[c_, sty_String/;StringMatchQ[sty, "Title"|"HowToTitle"], o___] :>
      Cell[CellGroupData[Flatten[{
        Cell[c, sty, o],
        obsolescenceNote
      }, 1], Open]];

  screenshot = Flatten[
   Cases[expr, Cell[img_, ___, "AlternateThumbnail", ___] :> Cases[img, GraphicsBox[___], Infinity], Infinity]];
  If[Length[screenshot] > 0, screenshot = First[screenshot]];

  expr = DeleteCases[expr, Alternatives[
      Cell[TextData[Cell[BoxData[""]]], "ScreencastMarker", ___],
      Cell["paclet:screencast/XXXX", "ScreencastURI", ___],
      Cell[___, "AlternateThumbnail", ___]
    ],
    Infinity];

  expr = expr /. Cell[cont_, "SeeAlso", o___] :>
    Cell[
      cont /. {
        StyleBox[ButtonBox[c___], "MenuName"] :> StyleBox[ButtonBox[c], "MenuName", FontSize->12]},
      "SeeAlso", o];


  (*  NEW V9 CODE -- old code is commented out just below *)
  Module[{abspos,abstract,htxt, screencell},
    screenuri = Cases[expr, Cell[uri_, "ScreencastURI", ___] :> uri, Infinity, 1];
    If[Length[screenuri] > 0,
      screenuri = StringReplace[First[screenuri],
        "paclet:screencast/" -> "http://reference.wolfram.com/mathematica/screencasts/"
      ];
      abspos = First@Position[expr, Cell[_, "HowToAbstract", ___], Infinity, 1];
      abstract = Extract[expr, abspos];
      expr = Delete[expr, abspos];
      htxt = Localized["VideoHowTo", language];
      screencell = Cell[BoxData[TagBox[ButtonBox[
        With[{screenoff = ToBoxes[ImagePad[ImageTake[ToExpression@screenshot, {3, -3}, {3, -3}], 2, GrayLevel[.8]]], 
              screenover = ToBoxes[ImagePad[ImageTake[ImagePad[
                ImageTake[ToExpression@screenshot, {3, -3}, {3, -3}], 2, GrayLevel[.8]], 
                {2, -2}, {2, -2}], 1, RGBColor[.854902, .396078, .145098]]]},
        DynamicBox[If[CurrentValue["MouseOver"], screenover, screenoff]]], 
        BaseStyle->"Hyperlink", 
        ButtonData->{URL[screenuri], None}],MouseAppearanceTag["LinkHand"]]], "HowToScreencastLink"];
      expr = Insert[expr, screencell, abspos];
      expr = Insert[expr, abstract, abspos];
      
  ]
  ];

  (*Module[{screenimg, abspos, abstract, htxt},
    screenuri = Cases[expr, Cell[uri_, "ScreencastURI", ___] :> uri, Infinity, 1];
    If[Length[screenuri] > 0,
      screenuri = StringReplace[First[screenuri],
        "paclet:screencast/" -> "http://reference.wolfram.com/mathematica/screencasts/"
      ];
      screenimg = First@Cases[
        Cases[expr, Cell[_, "ScreencastMarker", ___], Infinity, 1],
        _GraphicsBox, Infinity, 1];
      screenimg = ToBoxes[Import[DocsAsset["InternalOnlineImage", "ScreencastLink.png"], {"PNG", "Graphics"}]];
      abspos = First@Position[expr, Cell[_, "HowToAbstract", ___], Infinity, 1];
      abstract = Extract[expr, abspos];
      htxt = Localized["VideoHowTo", language];
      abstract = Cell[
        BoxData[GridBox[{{
            Cell[First[abstract]],
            Cell[BoxData[
              ButtonBox[
                PaneSelectorBox[{
                  False->GridBox[{{screenimg},
                    {Cell[TextData[StyleBox[htxt, "HowToScreencastLink"]]]}}],
                  True->GridBox[{{screenimg},
                    {Cell[TextData[StyleBox[htxt, "HowToScreencastRedLink"]]]}}]
                },  Dynamic[CurrentValue["MouseOver"]]],
                (*ButtonStyle->"HowToScreencastLink",*)
                BaseStyle->"HowToScreencastLink",
                ButtonData->{URL[screenuri], None}]
            ], "HowToScreencastLink"]
          }}
        ]],
        Sequence@@Rest[abstract]];
      expr = ReplacePart[expr, abspos -> abstract];
  ]];*)
  expr = DeleteCases[expr, Cell[_, "ScreencastURI"|"ScreencastMarker"|"Time", ___], Infinity];

  expr = expr /. Cell[c_, "ScreencastPointerText", o___] :>
    Cell[
      c /. ButtonBox[s_, ___] :>
        ButtonBox[s, BaseStyle->{"Hyperlink", FontWeight->"Bold"},
          ButtonData->{URL[screenuri], None}],
      "ScreencastPointerText", o];

(*  expr = KillEmptyCellGroups[expr, {"MoreToExploreSection", "MoreInformationSection"}];

  (*  Change headings to have Open/close buttons *)
  expr = expr /. Cell[con_, sty_String /; StringMatchQ[sty, Alternatives@@Flatten@{"Styles" /. Options[DocumentationTools`Utilities`CapitalizeHeadings] } ], o___] :>
                   Cell[TextData@WrapInOpenCloseButton@con, sty, o];*)

  (* FIXME: cells not being renumbered *)
  expr = RenumberCells[expr, {"HowToAbstract", "HowToDelimiter"}];
  
  (* Replace Under Development with [[Unfinished Draft]] banner *)
  (* V11.1: changed how [[EXPERIMENTAL]] banner is displayed *)
  (*expr = Module[{langext, unfindraftimg},If[MemberQ[expr, "UnderDevelopment", Infinity] || 
  	      TrueQ["Underdevelopment"/.info],
     langext = language /. {"English"->"", "Japanese"->"_jp", "ChineseSimplified"->"_zh", "Spanish"->"_es" };
     unfindraftimg = Import[DocsAsset["Graphics", "RefUnfinishedDraft"<>langext<>".m"]];
     ReplaceAll[expr, c : Cell[___, "HowToTitle", ___] :> 
       Cell[TextData[{c, Cell[BoxData[ToBoxes[Spacer[10]]]], Cell[BoxData[ToBoxes[unfindraftimg]]]}], "HowToTitleGrid"]],
     expr]];*)
     If[MemberQ[expr, "UnderDevelopment", Infinity] || TrueQ["Underdevelopment" /. info],
       expr = Module[{pos,expcell},
         expcell = Cell[TextData[{Cell["[", "ObjectNameAltQuotes"], "EXPERIMENTAL", Cell["]", "ObjectNameAltQuotes"]}], "ExperimentalBanner"];
         pos = Position[expr, Cell[__, "HowToTitle", ___]];
         Insert[expr, expcell, pos]
         ]
       ];
  
    (*expr = expr /. Cell[con_,
      sty:Alternatives@@("CapitalizeHeadingStyles" /. {opts} /. Options[MakeNotebook]),
      o___] :>
    Cell[TextData@WrapInOpenCloseButton@CapitalizeString[con], sty, o];*)
    
    (* Hard remove all empty footer sections as a consequence of attractor icon additions *)
    If[Cases[expr, Cell[CellGroupData[{Cell[__, #, ___], ___}, ___]], Infinity]==={}, expr = DeleteCases[expr, Cell[__, #, ___], Infinity]]& /@
   	 {"MoreAboutSection", "TutorialsSection", "SeeAlsoSection", "RelatedLinksSection"};
    
    expr = expr /. Cell[con_,
      sty:Alternatives@@("CapitalizeHeadingStyles" /. {opts} /. Options[MakeNotebook]),
      o___] :>
    Cell[TextData@WrapInOpenCloseButton@CapitalizeString[con], sty, o];
    
    BuildLog["-- Localize MoreAbout on howtos"];
    expr = expr /. Cell[c_, "MoreAboutSection", o___] :>
      Cell[Localized["MoreAboutSectionText", language], "MoreAboutSection", o];

    BuildLog["-- Localize Related Tutorials on howtos"];
    expr = expr /. Cell[c_, "TutorialsSection", o___] :> 
      Cell[Localized["TutorialsSectionText", language], "TutorialsSection", o];
      
    BuildLog["-- Localize Related Links on howtos"];
    expr = expr /. Cell[c_, "RelatedLinksSection", o___] :> 
      Cell[Localized["RelatedLinksSectionText", language], "RelatedLinksSection", o];
      
    BuildLog["-- Localize SeeAlso on howtos"];
    expr = expr /. Cell[c_, "SeeAlsoSection", o___] :> 
      Cell[Localized["SeeAlsoSectionText", language], "SeeAlsoSection", o];

expr = DeleteCases[expr, Cell[_,  "LinkTrail", ___], Infinity];
  

  expr
  ];


(******************************************************************************)
(** ConvertNotebook - default **)
ConvertNotebook[expr_, info_, _, opts___?OptionQ] := expr;


(******************************************************************************)
(** Internal **)
RemoveGuideLinksToFlaggedSymbols[
	expr0_,
	uri_,
	entityType_?StringQ,
	lang_,
	buildmode_
] := Module[{
	expr = expr0,
	containerCellStyle,
	linkCellStyle,
	filterWhenMissingSourceInfoQ,
	delpos
},
	{containerCellStyle, linkCellStyle, filterWhenMissingSourceInfoQ} = Replace[entityType, {
		"Guide" -> {
			"GuideText",
			"InlineGuideFunction",
			False
		},
		("Workflow" | "Workflow Guide") -> {
			"WorkflowGuideEntry" | "SeeAlso",
			"WorkflowGuideEntry" | "SeeAlso",
			True
		},
		_ :> DocsError[
			"Unexpected page type: ``: ``",
			InputForm[entityType], InputForm[uri]
		]
	}];

	delpos = Reap[Scan[
		Function[{pos}, Module[{cell, links, newlinks, text},
			cell = Extract[expr, pos];

			links = Cases[
				cell,
				Cell[
					(BoxData|TextData)[_ButtonBox | StyleBox[_ButtonBox, ___]],
					linkCellStyle,
					___
				],
				Infinity
			];

			If[Length[links] > 0,
				If[buildmode === "Future",
					newlinks = links
					,
					newlinks = Select[
						links,
						Function[link, CurrentLinkQ[link, uri, lang, filterWhenMissingSourceInfoQ]]
					]
				];

				If[Length[links] =!= Length[newlinks],
					If[Length[newlinks] > 0,
						text = Cases[cell, _TextData, Infinity, 1][[1, 1]];

						text = Drop[
							text,
							(* TODO:
							    If `entityType` is a Workflow page, should this style
							    be different? Prior to this refactoring, this code was
								duplicated in the ConvertNotebook definitions for "Guide"
								and ("Workflow" | "Workflow Guide") page types, implying
								that this value being hard-coded here may simple be an
								accidentally omitted change in Workflow version of the
								code. *)
							Position[text, Cell[BoxData[_ButtonBox], "InlineGuideFunction", ___]][[-1, 1]]
						];

						expr = ReplacePart[
							expr,
							pos -> Join[
								Cell[TextData[Join[Riffle[newlinks, ", "], text]]],
								Rest[cell]
							]
						];
						, (* Else - removing all links *)
						Sow[pos];
					];
				]
			];
		]],
		Position[expr, Cell[_, containerCellStyle, ___]]
	]];

	If[Length[delpos[[2]]] > 0,
		expr = Delete[expr, delpos[[2, 1]] ]
	];

	expr
];

(******************************************************************************)
(** Internal **)
RemoveSeeAlsoGroupLinksToFlaggedSymbols[
	seeAlsoGroup0_,
	pacletName_,
	buildmode_
] := Module[{
	seeAlsoGroup = seeAlsoGroup0
},
	DocsAssert[StringQ[pacletName]];
	DocsAssert[StringQ[buildmode]];

	Scan[
		Function[{pos}, Module[{cell, origcells, newcells},
			cell = Extract[seeAlsoGroup, pos];

			origcells = Cases[
				cell,
				Cell[BoxData[_ButtonBox, ___], ___]
				| Cell[BoxData[StyleBox[_ButtonBox, ___], ___], ___]
				| Cell[BoxData[TemplateBox[___], ___], ___],
				Infinity
			];

			(* TODO-andy If ButtonData exists, it has to be used! *)
			If[buildmode === "Future",
				newcells = origcells
				,
				newcells = Select[
					origcells,
					Function[link, CurrentSeeAlsoLinkQ[link, pacletName]]
				]
			];

			If[Length[origcells] =!= Length[newcells],
				seeAlsoGroup = ReplacePart[
					seeAlsoGroup,
					pos -> Join[
						Cell[TextData[
							Flatten[Riffle[
								newcells,
								{{
									"\[NonBreakingSpace]",
									StyleBox[
										"\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]",
										"InlineSeparator"
									],
									" "
								}},
								2
							]]
						]],
						Rest[cell]
					]
				]
			];
		]],
		Position[seeAlsoGroup, Cell[_TextData, "SeeAlso", ___], Infinity]
	];

	seeAlsoGroup
]

(******************************************************************************)
(** Internal **)
(* This function returns True if `link` is a Cell containing a link to a URI which is
   "current", in the sense that it is up-to-date, not obsolete or futurized. *)
CurrentLinkQ[
	link_,
	uri_,
	lang_,
	(* TODO:
		This parameter and the behavior surrounding it is under-defined. It really
		shouldn't exist, except at the moment source information is always missing when
		building user documentation (because the source info files for System docs are
		too big at the moment to distribute with DocumentationBuild), so this must be set
		to False during user documentation builds to avoid all of their links being
		filtered out.

		Ideally, source info would always be available, and we would simply issue a
		warning if source info is missing for a particular symbol.
	*)
	filterWhenMissingSourceInfoQ_?BooleanQ
] := Module[{buri},
	DocsAssert[
		MatchQ[link, Cell[___]],
		"expected link expression with head Cell, got: ``",
		InputForm[link]
	];

	(* If the link cell itself is flagged, remove this link. *)
	If[
		MatchQ[
			link,
			Cell[___, "FutureExample" | "TentativeExample", ___]
			| Cell[_[StyleBox[___, "FutureExample" | "TentativeExample", ___], ___], ___]
			| Cell[_[ButtonBox[___, BaseStyle->{___, "BrighterFlag", ___}, ___], ___], ___]
		],
		Return[False, Module];
	];

	buri = Cases[link, _[ButtonData, u_] :> u, Infinity];

	buri = Replace[buri, {
		(* TODO: Warn in this case about an "implicit link destination URI"? *)
		(* FIXME: This is incorrect when processing non-System documentation. *)
		{} :> "paclet:ref/" <> Transmogrify`ConvertToString[link[[1, 1, 1]]],
		{val_?StringQ, rest___} :> (
			If[Length[{rest} > 0],
				DocsWarning[
					"link cell unexpectedly has multiple ButtonData values: ``",
					InputForm[link]
				];
			];
			val
		),
		(* If the link is pointing at a web URL, always include it. *)
		{{url:URL[_?StringQ], None}} :> (
			Return[True, Module];
		),
		_ :> DocsError[
			"link cell has unexpected form: ``",
			InputForm[link]
		]
	}];

	DocsAssert[StringQ[buri]];

	Which[
		(* If no source information is available for the linked page, remove this link. *)
		And[
			filterWhenMissingSourceInfoQ,
			MatchQ[DocumentationBuild`Info`GetSourceInformation[buri, lang], None]
		],
			False,
		(* If this page is marked Obsolete, don't remove links to other Obsolete pages. *)
		DocumentationBuild`Info`GetSourceInformation[uri, lang, "Flag"] === "ObsoleteFlag",
			FreeQ[
				{"InternalFlag", "TemporaryFlag", "FutureFlag", "ExcisedFlag"},
				DocumentationBuild`Info`GetSourceInformation[buri, lang, "Flag"]
			],
		True,
			FreeQ[
				{"ObsoleteFlag", "InternalFlag", "TemporaryFlag", "FutureFlag", "ExcisedFlag"},
				DocumentationBuild`Info`GetSourceInformation[buri, lang, "Flag"]
			]
	]
]

(******************************************************************************)
(** Internal **)
(* This function returns True if `link` is a Cell containing a link to a URI which is
   "current", in the sense that it is up-to-date, not obsolete or futurized. *)
CurrentSeeAlsoLinkQ[
	link_,
	pacletName_
] := Module[{sym, sym2, uri},
	DocsAssert[StringQ[pacletName]];

	If[
		MatchQ[
			link,
			Cell[___, "FutureExample" | "TentativeExample", ___]
			| Cell[BoxData[StyleBox[___, "FutureExample" | "TentativeExample", ___], ___], ___]
		],
		Return[False, Module];
	];

	(* Else *)
	sym = Cases[link, _ButtonBox, Infinity, 1];
	sym2 = Cases[link, _TemplateBox, Infinity, 1];

	(* Formatting note: 636f9a66: fixed 368671 and 368829, restyled SeeAlso sub-refs *)
	uri = Cases[
		sym,
		ButtonData ~(Rule|RuleDelayed)~ uri_String :> uri,
		Infinity,
		1
	];

	uri = If[Length[uri] > 0 && StringMatchQ[First[uri], "paclet:*"],
		(* If Button contains ButtonData, use it *)
		First[uri]
		,
		(* check if the TemplateBox has a URI *)
		If[Length[sym2] > 0 && StringMatchQ[sym2[[1,1,2]], "paclet:*"],
			sym2[[1,1,2]]
			,
			(* no ButtonData so need to create it *)
			sym = Transmogrify`ConvertToString[sym[[1, 1]]];
			sym = StringReplace[sym, "\""->""];

			StringJoin[
				"paclet:",
				If[FreeQ[pacletName, "Mathematica" | ""],
					pacletName <> "/",
					""
				],
				"ref/",
				sym
			]
		]
	];

	ShippingFunctionQ[uri, pacletName]
]

(******************************************************************************)
(** EnsureCellGroupData - Internal **)
EnsureCellGroupData[expr_, type_, language_String:"English"] :=
Module[{allsects, sects, pos, p, nbexpr, newcell, presects, postsects},
  allsects = {"ObjectName", "ObjectNameSmall", "NotesSection", "PrimaryExamplesSection", "SeeAlsoSection",
              "TutorialSection", "RelatedDemonstrationsSection", "RelatedLinksSection",
              "MoreAboutSection", "History", "MoreToExploreSection", "MoreInformationSection"};
  If[Count[expr, Cell[_, type, ___], Infinity] =!= 0, expr,
    p = Position[allsects, type];
    If[p === {},
      presects = postsects = allsects,
      presects = Take[allsects, p[[1,1]] ];
      postsects = Drop[allsects, p[[1,1]] ] ];
    newcell = Cell[CellGroupData[{
      Cell[DocumentationBuild`Utils`CapitalizeString[
        Localized[type<>"Text", language]], type]}, Open]];

    (* We allow the styles not to be in groups, but this will get use both
       the group and the leading cells.  Position is post-order, so we can
       safely take the Last one. *)
    pos = Position[expr,
      Cell[CellGroupData[{Cell[_, Alternatives@@presects, ___], ___}, ___], ___]
       | Cell[_, Alternatives@@presects, ___],
      Infinity];

    If[pos =!= {},
      (* Take the Last and change it so we insert after *)
      pos = Last[pos];
      pos = Append[Array[0&, Length[pos] - 1], 1] + pos,

      (* Else *)
      pos = Position[expr,
        Cell[CellGroupData[{Cell[_, Alternatives@@postsects, ___], ___}, ___], ___]
         | Cell[_, Alternatives@@presects, ___],
        Infinity];
      (* Check if the first position is "inside" the second position.  If
         so, then the first is just the lead cell of the group that is the
         second.  Take the second. *)
      If[pos === {},
        pos = Position[expr, Cell[___, "FooterCell", ___], Infinity],
        If[Length[pos] > 1
            && (Length[pos[[1]]] > Length[pos[[2]]])
            && (Take[pos[[1]], Length[pos[[2]]]] === pos[[2]]),
          pos = pos[[2]],
          pos = pos[[1]]
      ]];
    ];
    If[pos === {},
      nb = ReplacePart[expr, {1} -> Append[First[expr], newcell]],
      (* Else *)
      nb = Insert[expr, newcell, pos]
    ];
    nb]
];


(******************************************************************************)
(** GetNotesGroup - Internal **)
(* get the notes cell group and make necessary adjustments *)
GetNotesGroup[expr_, type_, language_String:"English", opts___?OptionQ] :=
Module[{notesGroup, functionPaclet, hasopts,openicon, closedicon},
  openicon = Get[DocsAsset["Graphics:Opener", "SectionOpenerOpenOff.m"]];
  closedicon = Get[DocsAsset["Graphics:Opener", "SectionOpenerClosedOff.m"]];

  functionPaclet = "FunctionPaclet" /.  {opts} /. Options[MakeNotebook];
  hasopts = "HasOptions" /. Flatten[{opts}] /. ("HasOptions" -> False);
  notesGroup = 
    Cases[expr, 
      Cell[CellGroupData[{
        Cell[_, "ObjectName"|"ObjectNameSmall"|"ObjectNameAlt"|"CharacterImage", ___],
        ___}, ___], ___],
      Infinity, 1];
  notesGroup = DeleteCases[notesGroup, Cell[_, "ObjectName" | "ObjectNameSmall" | "ObjectNameAlt" | "Usage" | "CharacterImage"| "CharacterName" | "Elucidation", ___], Infinity];
  notesGroup = If[notesGroup === {Cell[CellGroupData[{}, Open]]}, {}, notesGroup];
  (* stripping out CellGroupData (we were relying on the style sheet to do this in the past, but
     it seems to have gotten unreliable of late) *)
  notesGroup = Replace[notesGroup, {Cell[CellGroupData[cells_, ___], ___]} :> cells];
  (* Restyle MetaDescription cell back into Notes cell, remove CellTags *)
  notesGroup = ReplaceAll[notesGroup, cell:Cell[__, "MetaDescription", ___] :> 
    DeleteCases[Replace[cell, "MetaDescription"->"Notes", {1}], _[CellTags, "Meta Description"], Infinity]];
  (* add 'more information' heading for notes *)
  notesGroup = Which[MemberQ[{"Character Name", "Format", "Indicator", "Device Connection", "Service Connection", "Method"},type], notesGroup,
  	True,
  notesGroup = With[{openicon = openicon, closedicon = closedicon},If[Length[notesGroup] > 0,
    Cell[CellGroupData[Flatten[{Cell[TextData[{ 
      Cell@BoxData@WrapInOpenCloseButton[
        If[functionPaclet,
			ToBoxes[Import[
				DocsAsset[
					"Internal:Web",
					FileNameJoin[{"html", "images", "mathematicaImages", "MoreInformation.jpg"}]
				],
				"JPG"
			]],
          (* V9 EDIT BELOW *)
          
            Cell[TextData[{Cell[BoxData[DynamicBox[ToBoxes@
              If[MatchQ[CurrentValue[EvaluationNotebook[],{TaggingRules,"Openers","NotesSection"},Closed],Open|True],
                openicon,
                closedicon]]]],Cell[BoxData[ToBoxes[Spacer[1]]]],
              If[TrueQ[hasopts],Localized["NotesSectionText", language],
                Localized["NotesSectionNoOptsText", language]]}], "NotesFrameText"]
            
        ]]
      }], "NotesSection",
      CellGroupingRules->{"SectionGrouping", 50}, CellID->1 ], notesGroup},1], Open]],
    {}]];
  notesGroup]
];


(******************************************************************************)
(** LoadThumbnails - Internal **)

$ThumbnailCache = <| |>;

$ThumbnailAssetFiles = <|
	"Entity"                      -> "EntityThumbnailData.m",
	"Device Connection"           -> "DeviceThumbnailData.m",
	"Format"                      -> "FormatThumbnailData.m",
	"Classifier"                  -> "ClassifierThumbnailData.m",
	"Predictor"                   -> "PredictorThumbnailData.m",
	"External Evaluation System"  -> "ExternalEvaluationSystemThumbnailData.m",

	"Text Content"                -> "TextContentThumbnailData.m",
	"Data Structure"              -> "DataStructureThumbnailData.m",
	"Front End Object"            -> "FrontEndObjectThumbnailData.m",
	"Net Encoder"                 -> "NetEncoderThumbnailData.m",
	"Net Decoder"                 -> "NetDecoderThumbnailData.m",
	"System Program"              -> "SystemProgramThumbnailData.m",
	"File"                        -> "FileThumbnailData.m",
	"MathLink C Function"         -> "MathLinkCFunctionThumbnailData.m",
	"Menu Item"                   -> "MenuItemThumbnailData.m",

	(* NOTE: The `type` being "Symbol" but the file name being "Notes" is not a refactoring
			 mistake. This seems to simply be a historical quirk of naming. Perhaps it
		     could be simplified at some point. *)
	"Symbol"                      -> "NotesThumbnailData.m"
|>;

LoadThumbnails[type_String] := Module[{
	data
},
    Assert[AssociationQ[$ThumbnailCache]];

    If[KeyExistsQ[$ThumbnailCache, type],
      Return[$ThumbnailCache[type]];
    ];

	If[DocumentationBuild`Internal`InternalAssetsDirectory[] === None,
		Return[{}];
	];

	data = Replace[Lookup[$ThumbnailAssetFiles, type], {
		_?MissingQ -> {},
		file_?StringQ :> (
			data = Get[DocsAsset["InternalData", file]];

			If[type === "Symbol",
				(* Note: The structure of the "Symbol" thumbnail data is different. *)
				Uncompress @ data
				,
				Uncompress /@ data
			]
		),
		other_ :> DocsError["Unreachable: unexpected thumbnail asset file value: ``", other]
	}];

	(* Standardize the type of `data`. The Format thumbnail data file has a slightly
	   different structure than the other data files. If these data files are made
	   consistent in the future, then this conversion could be removed. *)
	If[MemberQ[{"Format"}, type],
		DocsAssert[MatchQ[
			data,
			{
				Repeated[
					_?(StringStartsQ["paclet:"]) -> {
						{
							(_?StringQ -> {___Image} | "") ...,
							RepeatedNull["IncludeEllipsis" -> _?BooleanQ, 1]
						} ...
					}
				]
			}
		]];

		data = Map[
			Replace[(key_ -> value_) :> (key -> Flatten[value])],
			data
		];
	];

    (* Cache the thumbnail data we just loaded / uncompressed. *)
    $ThumbnailCache[type] = data;

    data
]

(**********************************************************)
(* Thumbnail Utilities                                    *)
(**********************************************************)

thumbnailMagnifyOverlays[thumbheight0_] := Module[{
	thumbheight = thumbheight0,
	magoff,
	magon
},
	{magoff, magon} = Map[
		Image[#, Magnification -> 0.5] &
		,
		Flatten[ImagePartition[
			Import[DocsAsset["Images", "DetailsMagnify2x.png"]],
			50
		]]
	];

	Map[
		Replace[{mag_, alpha_} :> Overlay[
			{
				Graphics[
					{
						RGBColor[0, 0.666667, 1, alpha],
						Rectangle[{-170 2-20, -thumbheight/2}, {170 2+20, thumbheight/2}]
					},
					ImageSize -> {170 4+40, thumbheight},
					PlotRangePadding -> None,
					ImagePadding -> None
				],
				mag
			},
			Alignment -> {-0.85, Center}
		]],
		{{magon, 0.06}, {magoff, 0}}
	]
]

makeThumbnailsCell[thumbheight_, content_, openSelector_?ListQ] := With[{
	overlay = thumbnailMagnifyOverlays[thumbheight][[1]],
	overlayoff = thumbnailMagnifyOverlays[thumbheight][[2]],
	numthumbs = 4
},
Module[{},
	Cell[
		BoxData @ TagBox[
			ButtonBox[
				StyleBox[
					DynamicBox[
						ToBoxes @ Overlay[
							{
								Grid[{{
									Spacer[10],
									Sequence @@ Map[
										ImageResize[#, {170, Min[70, ImageDimensions[#][[2]] + 5]}] &
										,
										Take[content, UpTo[numthumbs]]
									],
									Spacer[10]
								}}],
								If[CurrentValue["MouseOver"], overlay, overlayoff]
							},
							Alignment -> {Left, Center}
						]
					],
					(* Formatting note: fc2c01031baa2770920ea578c409440195db5548:
						fix bug 329960, and touch up notes thumbnails button in general *)
					Background :> FEPrivate`If[
						FEPrivate`SameQ[CurrentValue[EvaluationNotebook[], Background], None],
						GrayLevel[1],
						CurrentValue[EvaluationNotebook[], Background]
					]
				],
				(* Don't know why the "rule" setting for Appearance keeps the button from
				   getting a "pressed" look when you push it, but it's desired behavior *)
				Appearance -> None,
				ButtonFunction :> (
					CurrentValue[EvaluationNotebook[], openSelector] = Open
				),
				Evaluator -> "System"
			],
			MouseAppearanceTag["LinkHand"]
		],
		"NotesThumbnails",
		(* new code below from jfultz: bug 338675 *)
		CellOpen -> Dynamic[FEPrivate`Switch[
			CurrentValue[EvaluationNotebook[], openSelector],
			True,
				False,
			Open,
				False,
			_,
				True
		]]
	]
]]

processThumbnailData[data0_, openSelector_?ListQ] := Module[{
	data = data0,
	height
},
	data = Replace[data, {
		_?MissingQ | "" :> Return["", Module],
		{images:{___Image}, "IncludeEllipsis" -> _?BooleanQ} :> images,
		images:{___Image} :> images,
		other_ :> DocsError["unexpected thumbnail data form: ``", other]
	}];

	DocsAssert[MatchQ[data, {___Image}]];

	height = If[Length[data] === 1,
		Max[
			(* Note:
				Formula used by just external evaluation system thumbnails, prior to this
				refactoring:
					Round[(ImageDimensions[ data[[1]] ][[2]]+10)/2, 2]
				For some reason ref/externalevaluationsystem pages (and no others) used
				that formula instead of the one below. A glance at a handful of external
				evaluation system pages doesn't suggest any obvious reason why their
				thumbnails ought to be sized differently than other thumbnails, so I'm
				tentatively going to standardize on this formula. *)
			Round[ImageDimensions[ data[[1]] ][[2]], 2] + 10,
			50
		],
		80
	];

	makeThumbnailsCell[height, data, openSelector]
]

(******************************************************************************)

GetThumbnailCells[
	type_?StringQ,
	uri_?StringQ,
	taggingPrefix:{___?StringQ},
	sections : Alternatives[
		{{_?StringQ, _?StringQ}..},
		None
	]
] := Module[{
	newuri = ToPacletURI[uri],
	thumbnails = LoadThumbnails[type],
	entry
},
	(* Handle pages that only have one section that collapses with thumbnails. *)
	If[sections === None,
		entry = Lookup[thumbnails, newuri];

		Return[
			processThumbnailData[
				entry,
				{TaggingRules, "Openers", Sequence @@ taggingPrefix}
			],
			Module
		];
	];

	(* Handle pages that have multiple sections that collapse with thumbnails. *)
	Map[
		Replace[{key_, tag_} :> (
			entry = NestedLookup[thumbnails, {newuri, key}];

			processThumbnailData[
				entry,
				{TaggingRules, "Openers", Sequence @@ taggingPrefix, tag}
			]
		)],
		sections
	]
]


(******************************************************************************)
(** GetNotesThumbnails - Internal **)
GetNotesThumbnails[uri_String] :=
	GetThumbnailCells["Symbol", uri, {"NotesSection"}, None]


(******************************************************************************)
(** GetEntityThumbnails - Internal **)
GetEntityThumbnails[uri_String] :=
	GetThumbnailCells[
		"Entity",
		uri,
		{"EntitySection"},
		{
			{"Properties", "2"},
			{"Details", "3"}
		}
	]


(******************************************************************************)
(** GetFormatThumbnails - Internal **)
GetFormatThumbnails[uri_String] :=
	GetThumbnailCells[
		"Format",
		uri,
		{},
		{#, #}& /@ {
			"FormatBackground",
			"ImportExportSection",
			"NotebookInterfaceSection",
			"ElementsSection",
			"ExportElementsSection",
			"OptionsSection"
		}
	]


(******************************************************************************)
(** GetDeviceThumbnails - Internal **)
GetDeviceThumbnails[uri_String] :=
	GetThumbnailCells[
		"Device Connection",
		uri,
		{"DeviceSubsection"},
		{
			{"Device Discovery", "0"},
			{"Opening the Device", "1"},
			{"Configuring the Device", "2"},
			{"Device Properties", "3"},
			{"Reading Data", "4"},
			{"Writing Data", "5"},
			{"Executing Commands", "6"},
			{"Closing and Releasing Resources", "7"}
		}
	]


(******************************************************************************)
(** GetClassifierPredictorThumbnails - Internal **)
GetClassifierPredictorThumbnails[uri_String, type_String] := With[{
	sectstyle = Switch[type,
		"Classifier",
			"ClassifierSection",
		"Predictor",
			"PredictorSection"
	]
},
	GetThumbnailCells[
		type,
		uri,
		{sectstyle},
		{
			{"Classes", "0"},
			{"Details", "1"}
		}
	]
]


(******************************************************************************)
(** GetExternalEvaluationSystemThumbnails - Internal **)
GetExternalEvaluationSystemThumbnails[uri_String] := Module[{
	newuri = ToPacletURI[uri],
	data
},
	Map[
		With[{
			label = #[[1]],
			sect = #[[2]]
		},
			data = NestedLookup[LoadThumbnails["External Evaluation System"], {newuri, label}];

			processThumbnailData[
				data,
				If[sect =!= "-1",
					{TaggingRules, "Openers", "ExternalEvaluateSystemSection", sect}
					,
					{TaggingRules, "Openers", "NotesSection"}
				]
			]
		]&,
		{
			{"Notes", "-1"},
			{"ExternalEvaluate Usage", "0"},
			{"Data Types", "1"},
			{"Usage Notes", "2"}
		}
	]
]


(******************************************************************************)
(** GetOtherTypeThumbnails - Internal **)
GetOtherTypeThumbnails[uri_String, type_String] :=
	GetThumbnailCells[type, uri, {"NotesSection"}, None]


(******************************************************************************)
(** GetOpenSections - Internal **)
GetOpenSections[pagetype_String] := Switch[pagetype,
	"Message",
		{
			"NotesSection" -> Open,
			"PrimaryExamplesSection" -> Open,
			"ExampleSection" -> {"0" -> Open}
		},
	"Format",
		{
			"PrimaryExamplesSection" -> Open,
			"ExampleSection" -> {"0" -> Open}
		},
	"Indicator",
		{"IndicatorExampleSection" -> Open},
	(* Closing workflow sections by default *)
	"Workflow",
		{
			"WorkflowHeader" -> ((ToString[#] -> Closed)& /@ Range[10]),
			"WorkflowNotesSection" -> Closed
		},
	"Entity" | "Entity Type",
		{
			"EntitySection" -> {"0" -> Open, "1" -> Open},
			"PrimaryExamplesSection" -> Open,
			"ExampleSection" -> {"0" -> Open}
		},
	(* Formatting note: 12496b21: fixed WLDOCS-2525, 2464, 2529 *)
	"Interpreter",
		{
			"NotesSection" -> Open,
			"PrimaryExamplesSection" -> Open,
			"ExampleSection" -> {"0" -> Open},
			"InterpreterSection" -> {"0" -> Open, "1" -> Open, "2" -> Open, "3" -> Open}
		},
	"Database Connection",
		{
			"NotesSection" -> Open,
			"PrimaryExamplesSection" -> Open,
			"ExampleSection" -> {"0" -> Open},
			"DatabaseConnectionSection" -> {"0" -> Open, "1" -> Open, "2" -> Open, "3" -> Open}
		},
	"Compiled Type",
		{
			"PrimaryExamplesSection" -> Open,
			"ExampleSection" -> {"0" -> Open},
			"NotesSection" -> Open,
			"CompiledTypeSection" -> {"0" -> Open, "1" -> Open, "2" -> Open, "3" -> Open, "4" -> Open}
		},
	"ResourceObject" | "Resource Object",
		{
			"PrimaryExamplesSection" -> Open,
			"ExampleSection" -> {"0" -> Open},
			"NotesSection" -> Open
		},
	_,
		{
			"PrimaryExamplesSection" -> Open,
			"ExampleSection" -> {"0" -> Open}
		}
];


(******************************************************************************)
(** GetImportExportGroup - Internal **)
(* get the Import/Export cell group and make necessary adjustments *)
GetImportExportGroup[expr_, language_String:"English"] := Module[{importExportGroup},
  importExportGroup = 
    Cases[expr, 
      Cell[CellGroupData[{
        Cell[_, "ImportExportSection", ___],
        ___}, ___], ___],
      Infinity, 1];
  importExportGroup= importExportGroup /. Cell[a_,"ImportExportSection",b_] :> 
  		Cell[Localized[Switch[a, "Import", "ImportSectionText", "Export", "ExportSectionText", _, "ImportExportSectionText"], language],"ImportExportSection",b];
  importExportGroup
];


(******************************************************************************)
(** GetElementGroup - Internal **)
(* get the ElementsSection cell group and make necessary adjustments *)
(* 071318 - added option to allow extraction of ExportElementsSection group *)
GetElementGroup[expr_, language_String:"English", style_String:"ElementsSection"] := Module[{elementGroup},
  elementGroup = 
    Cases[expr, 
      Cell[CellGroupData[{
        Cell[_, style, ___],
        ___}, ___], ___],
      Infinity, 1];
  elementGroup = elementGroup /. Cell[_,style,b_] :> 
  		Cell[Localized[style<>"Text", language],style,b];
  elementGroup
];


(******************************************************************************)
(** GetNotebookInterfaceGroup - Internal **)
(* get the NotebookInterfaceSection cell group and make necessary adjustments *)
GetNotebookInterfaceGroup[expr_, language_String:"English"] := Module[{notebookInterfaceGroup},
  notebookInterfaceGroup = 
    Cases[expr, 
      Cell[CellGroupData[{
        Cell[_, "NotebookInterfaceSection", ___],
        ___}, ___], ___],
      Infinity, 1];
  notebookInterfaceGroup = notebookInterfaceGroup /. Cell[_,"NotebookInterfaceSection",b_] :> 
  		Cell[Localized["NotebookInterfaceSectionText", language],"NotebookInterfaceSection",b];
  notebookInterfaceGroup
];


(******************************************************************************)
(** GetOptionsGroup - Internal **)
(* get the OptionsSection cell group and make necessary adjustments *)
GetOptionsGroup[expr_, language_String:"English"] := Module[{optionsGroup},
  optionsGroup = 
    Cases[expr, 
      Cell[CellGroupData[{
        Cell[_, "OptionsSection", ___],
        ___}, ___], ___],
      Infinity, 1];
  optionsGroup = optionsGroup /. Cell[_,"OptionsSection",b_] :> 
  		Cell[Localized["OptionsSectionText", language],"OptionsSection",b];
  optionsGroup
];

(******************************************************************************)
(** GetOtherSectionGroup - Internal **)
(* get the OptionsSection cell group and make necessary adjustments *)
GetOtherSectionGroup[expr_, language_String:"English"] := Module[{sectionGroup,styles},
  styles = {"ServiceSubsection","DeviceSubsection","InterpreterSection","EmbeddingFormatSection", "MethodSection","ExternalEvaluateSystemSection","EntitySection","CompiledTypeSection",
            "BlockchainSection", "DatabaseConnectionSection", "ClassifierSection", "PredictorSection", "BatchComputationProviderSection", "QuestionInterfaceSection", "ResourceObjectSection"};
  sectionGroup = 
    Cases[expr, 
      Cell[CellGroupData[{
        Cell[_, Alternatives@@styles, ___],
        ___}, ___], ___],
      Infinity];
  sectionGroup
];

(******************************************************************************)
(** GetOptionsGroup - Internal **)
(* get the OptionsSection cell group and make necessary adjustments *)
GetTutorialsGroup[expr_, language_String:"English"] := Module[{tutorialsGroup, styles = Alternatives["TutorialsSection", "TechNotesSection"]},
  tutorialsGroup = 
    Cases[expr, 
      Cell[CellGroupData[{
        Cell[_, styles, ___],
        ___}, ___], ___],
      Infinity, 1];
  tutorialsGroup = tutorialsGroup /. Cell[_, sty:styles,b_] :> 
          (* V9 EDIT BELOW *)
  		Cell[Localized["TutorialsSectionText", language],sty,b];
  tutorialsGroup
];

(******************************************************************************)
(** GetRelatedLinksGroup - Internal **)
(* get the OptionsSection cell group and make necessary adjustments *)
GetRelatedLinksGroup[expr_, language_String:"English"] := Module[{relatedLinksGroup},

  relatedLinksGroup = 
    Cases[expr, 
      Cell[CellGroupData[{
        Cell[_, "RelatedLinksSection", ___],
        ___}, ___], ___],
      Infinity, 1];
      
  relatedLinksGroup = relatedLinksGroup /. Cell[_,"RelatedLinksSection",b_] :> 
          (* V9 EDIT BELOW *)
  		Cell[Localized["RelatedLinksSectionText", language],"RelatedLinksSection",b];
  relatedLinksGroup
];


(******************************************************************************)
(** GenerateEntityPageContent - Internal **)
(* fill in autogenerated content on Entity-type pages *)
GenerateEntityPageContent[expr_Notebook, type_String, {propthumbnail_:"", detthumbnail_:""}, lang_:"English"]:=
  Module[{sampleent, sampleclass, proptable, newexpr = expr, 
      rowicon = Cell["      ", "TableRowIcon"], pos, 
      thumbs = {propthumbnail, detthumbnail}, 
      thumbsects = {Localized["EntityPagePropertiesSectionText", lang], Localized["EntityPageDetailsSectionText", lang]},
      ellipsis = StyleBox["\[Ellipsis]", "Text", FontWeight->"Bold"]},
    sampleent = ToBoxes/@EntityValue[type, "SampleEntities"];
    sampleclass = ToBoxes/@EntityValue[type, "SampleEntityClasses"];
    proptable = With[{props = EntityValue[type, "Properties"]}, Transpose[{CanonicalName@props, EntityValue[props, "Description"]}]];
    newexpr = ReplaceAll[newexpr, Cell[s_,___]/;(FreeQ[s, CellGroupData] && !FreeQ[s, st_String/;!StringFreeQ[st, "[Autogenerated Sample Entities]"]]) :> 
    	If[!MatchQ[sampleent, {}], 
    		Cell[BoxData[RowBox[Append[sampleent, ellipsis]]], "Notes", LineIndent->0,CellDingbat->None, FontSize->12],
    		Nothing]];
    newexpr = ReplaceAll[newexpr, Cell[s_,___]/;(FreeQ[s, CellGroupData] && !FreeQ[s, st_String/;!StringFreeQ[st, "[Autogenerated Sample Classes]"]]) :> 
    	If[!MatchQ[sampleclass, {}],
    		Cell[BoxData[RowBox[Append[sampleclass, ellipsis]]], "Notes", LineIndent->0,CellDingbat->None, FontSize->12],
    		Nothing]];
    If[ListQ[proptable],  
    	proptable = Cell[BoxData[GridBox[{rowicon, #[[1]], 
    	  Cell[If[StringQ[#[[2]]],#[[2]],BoxData@ToBoxes[#[[2]]]], "TableText", ShowStringCharacters->False]}&/@proptable]], "2ColumnTableMod"];
        newexpr = ReplaceAll[newexpr, Cell[s_,___]/;(FreeQ[s, CellGroupData] && !FreeQ[s, st_String/;!StringFreeQ[st, "[Autogenerated Properties]"]]) :> 
          proptable],
        newexpr = ReplaceAll[newexpr, Cell[s_,___]/;(FreeQ[s, CellGroupData] && !FreeQ[s, st_String/;!StringFreeQ[st, "[Autogenerated Properties]"]]) :> 
          Nothing]];
    (* insert thumbnail cells *)
    Do[If[thumbs[[i]] =!= "",
     pos = Position[newexpr, Cell[CellGroupData[{Cell[thumbsects[[i]], ___, "EntitySection", ___], ___}, ___]]];
     If[pos =!= {}, newexpr = Insert[newexpr, thumbs[[i]], pos + {{0,1}}]]],
     {i, 2}];
    newexpr
  ];


(******************************************************************************)
(** GetOptionsGroup - Internal **)
(* get the OptionsSection cell group and make necessary adjustments *)
GetMoreAboutGroup[expr_, language_String:"English"] := Module[{moreAboutGroup},  

  moreAboutGroup = 
    Cases[expr, 
      Cell[CellGroupData[{
        Cell[_, "MoreAboutSection", ___],
        ___}, ___], ___],
      Infinity, 1];
      
  moreAboutGroup = moreAboutGroup /. Cell[_,"MoreAboutSection",b___] :> 
          (* V9 EDIT BELOW *)
  		Cell[TextData[StringReplace[Localized["MoreAboutSectionText", language], {" \[RightGuillemet]" | "\[RightGuillemet]"->""}]],"MoreAboutSection",b];
  moreAboutGroup
];


(******************************************************************************)
(** GetRelatedEntityGroup - Internal **)
(* get the RelatedEntitySection cell group and make necessary adjustments *)
GetRelatedEntityGroup[expr_, language_String:"English"] := Module[{entityGroup},  

  entityGroup = 
    Cases[expr, 
      Cell[CellGroupData[{
        Cell[_, "MoreAboutSection", ___],
        ___}, ___], ___],
      Infinity, 1];
      
  entityGroup = entityGroup /. Cell[_,"RelatedEntitySection",b___] :> 
  		Cell[TextData[StringReplace[Localized["RelatedEntitySectionText", language], {" \[RightGuillemet]" | "\[RightGuillemet]"->""}]],"RelatedEntitySection",b];
  entityGroup
];


(******************************************************************************)
(** GetRelatedInterpreterGroup - Internal **)
(* get the GetRelatedInterpreterGroup cell group and make necessary adjustments *)
GetRelatedInterpreterGroup[expr_, seeAlsoGroup_, type_, lang_String:"English"] := Module[{relatedInterpretersGroup,saGroup = seeAlsoGroup},  

(* Take out interpreter links from See Alsos, put in own section; build that section from scratch *)
  If[MemberQ[{"Interpreter"}, type], relatedInterpretersGroup = Cell[CellGroupData[{
    Cell[Localized["RelatedInterpretersSectionText", lang], "RelatedInterpreterSection"],
    Cell[TextData[Flatten[
       Riffle[Cases[saGroup, Cell[c_, ___, "InlineFormula", ___] /; 
       MemberQ[c, s_String/;StringMatchQ[s,"paclet:ref/interpreter/*"], Infinity], Infinity], 
          {{"\[NonBreakingSpace]",StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]","InlineSeparator"]," "}}]]], "RelatedInterpreters"]
    }, Open]];
    saGroup = DeleteCases[saGroup, Cell[c_, ___, "InlineFormula", ___] /; 
       MemberQ[c, s_String/;StringMatchQ[s,"paclet:ref/interpreter/*"], Infinity], Infinity];
    saGroup = ReplaceRepeated[saGroup, {
      {first___, " \[EmptyVerySmallSquare] ", " \[EmptyVerySmallSquare] ", last___} :> {first, " \[EmptyVerySmallSquare] ", last},
      {" \[EmptyVerySmallSquare] ", last__} :> {last},
      {first__, " \[EmptyVerySmallSquare] "} :> {first}
      }],
      
    relatedInterpretersGroup = Cases[expr, 
      Cell[CellGroupData[{
        Cell[_, "RelatedInterpreterSection", ___],
        ___}, ___], ___],
      Infinity, 1];
      relatedInterpretersGroup = relatedInterpretersGroup /. Cell[_,"RelatedInterpreterSection",b___] :> 
  		Cell[TextData[StringReplace[Localized["RelatedInterpretersSectionText", lang], {" \[RightGuillemet]" | "\[RightGuillemet]"->""}]],"RelatedInterpreterSection",b]
    ];
    
    relatedInterpretersGroup
    ];


(******************************************************************************)
(** GetSeeAlsoGroup - Internal **)
(* get the see Also cell group and make necessary adjustments *)
GetSeeAlsoGroup[expr_, language_String:"English"] := Module[{seeAlsoCellGroup, seeAlsoGroup, interpreterGroup, entityGroup, 
  encoderGroup, decoderGroup, blockchainGroup, predictorGroup, classifierGroup, allGroups, divider = " \[EmptyVerySmallSquare] ", GetSubGroup,
  sectionStyles = {"RelatedInterpreterSection", "RelatedEntitySection", "RelatedEncoderSection", "RelatedDecoderSection", "RelatedBlockchainSection",
           "RelatedClassifiersSection", "RelatedPredictorsSection"}},
  
  GetSubGroup[sectStyle_String]:=
    Flatten[Cases[Cases[expr, 
      Cell[CellGroupData[{
        Cell[_, sectStyle, ___],
        rest___}, ___], ___]:>rest,
      Infinity, 1], Cell[TextData[lis:({(Cell[___]|_String)..} | Cell[___] | _String)], ___] :> If[!ListQ[lis], {lis}, lis], Infinity]];
  
  seeAlsoCellGroup = 
    Cases[expr, 
      Cell[CellGroupData[{
        Cell[_ , "SeeAlsoSection", ___],
        ___}, ___], ___],
      Infinity, 1];
  seeAlsoGroup = 
    Flatten[Cases[seeAlsoCellGroup, Cell[TextData[lis:({(Cell[___]|_String)..} | Cell[___] | _String)], ___] :> lis, Infinity]];
  If[Count[seeAlsoCellGroup, Cell[__, "SeeAlso", ___], Infinity] > 1,
    seeAlsoCellGroup = Replace[seeAlsoCellGroup, Cell[CellGroupData[{sect:Cell[__, "SeeAlsoSection", ___], first:Cell[TextData[_], ___, "SeeAlso", ___],
      Cell[TextData[_], ___, "SeeAlso", ___]...}, openq___]] :> Cell[CellGroupData[{sect, ReplacePart[first, 1->TextData[seeAlsoGroup]]}, openq]], {1}]
   ];
  
  {interpreterGroup, entityGroup, encoderGroup, decoderGroup, blockchainGroup, classifierGroup, predictorGroup} = Map[GetSubGroup, sectionStyles];  
  
  allGroups = Riffle[DeleteCases[Join[seeAlsoGroup, entityGroup, interpreterGroup, 
    encoderGroup, decoderGroup, blockchainGroup, classifierGroup, predictorGroup], _String], divider];
  seeAlsoCellGroup = ReplaceAll[seeAlsoCellGroup, 
    {If[Length[seeAlsoGroup] === 1, (TextData[seeAlsoGroup] | TextData[seeAlsoGroup[[1]]]), TextData[seeAlsoGroup]] :> TextData[allGroups],
     Cell["XXXX", some___, "SeeAlso", rest___] :> Cell[TextData[allGroups], some, "SeeAlso", rest],
     sect:{Cell[__, "SeeAlsoSection", ___]} :> Join[sect, allGroups]
    }];
  seeAlsoCellGroup = If[seeAlsoCellGroup === {Cell[CellGroupData[{}, Open]]}, {}, seeAlsoCellGroup];
  (* ensure structure Cell[TextData[{...}], "SeeAlso"] *)
  If[MatchQ[seeAlsoCellGroup, {Cell[CellGroupData[{Cell[__, "SeeAlsoSection", ___], ((Cell[__, "InlineFormula"|"InlineCode", ___]|divider)..)}, ___]]}],
    seeAlsoCellGroup = ReplaceAll[seeAlsoCellGroup, 
      CellGroupData[{sec:Cell[__, "SeeAlsoSection", ___], rest:((Cell[__, "InlineFormula"|"InlineCode", ___]|divider)..)}, openQ___] :>
        CellGroupData[{sec, Cell[TextData[{rest}], "SeeAlso"]}, openQ]]
    ];
  seeAlsoCellGroup = seeAlsoCellGroup /. Cell[_, "SeeAlsoSection", o___] :> 
    (* V9 EDIT BELOW *)
    Cell[TextData[StringReplace[Localized["SeeAlsoSectionText", language], {" \[RightGuillemet]" | "\[RightGuillemet]"->""}]], "SeeAlsoSection", o];
  seeAlsoCellGroup
];


(******************************************************************************)
(** GetWorkflowsGroup - Internal **)
(* get the Workflows cell group and make necessary adjustments *)
GetWorkflowsGroup[expr_, language_String:"English"] := Module[{workflowsGroup},
  
  workflowsGroup = 
    Cases[expr, 
      Cell[CellGroupData[{
        Cell[_ , "RelatedWorkflowsSection", ___],
        ___}, ___], ___],
      Infinity, 1];
  workflowsGroup = If[workflowsGroup === {Cell[CellGroupData[{}, Open]]}, {}, workflowsGroup];
  workflowsGroup = workflowsGroup /. Cell[_, "RelatedWorkflowsSection", o___] :> 
    (* V9 EDIT BELOW *)
    Cell[TextData[StringReplace[Localized["RelatedWorkflowsSectionText", language], {" \[RightGuillemet]" | "\[RightGuillemet]"->""}]], "RelatedWorkflowsSection", o];
    
  (* Remove links to flagged notebooks *)
  workflowsGroup = ReplaceAll[workflowsGroup, Cell[TextData[
    ButtonBox[__, 
      ButtonData->uri_String /; 
       Quiet@Module[{info = DocumentationBuild`Info`GetSourceInformation[uri, "English"]}, info === None || !FreeQ[{"ObsoleteFlag", "InternalFlag", "FutureFlag", "ExcisedFlag"}, "Flag" /. info]], ___]], ___, "RelatedWorkflows", ___] -> Nothing];
  workflowsGroup
];


 (**	 
     GetTutorialCollectionGroup	 
     get the TutorialCollection group and make necessary adjustments	 
 *)	 
 GetTutorialCollectionGroup[expr_, language_String:"English"] := Module[{tutorialCollectionGroup},	 
     tutorialCollectionGroup =	 
         Cases[expr,	 
             Cell[CellGroupData[{	 
                 Cell[_ , "TutorialCollectionSection", ___],	 
                 ___}, ___], ___], Infinity, 1];	 
     tutorialCollectionGroup = If[tutorialCollectionGroup === {Cell[CellGroupData[{}, Open]]}, {}, tutorialCollectionGroup];	 
     tutorialCollectionGroup = tutorialCollectionGroup /. Cell[c_String, "TutorialCollectionSection", o___] :>	 
       Cell[c, "TutorialCollectionSection", o];	 
 tutorialCollectionGroup];	 
 
  	 	 
 (**	 
     GetGuideCollectionGroup	 
     get the TutorialCollection group and make necessary adjustments	 
 *)	 
 GetGuideTutorialCollectionGroup[expr_, language_String:"English"] := Module[{guideTutorialCollectionGroup},	 
     guideTutorialCollectionGroup =	 
         Cases[expr,	 
             Cell[CellGroupData[{	 
                 Cell[_ , "GuideTutorialCollectionSection", ___],	 
                 ___}, ___], ___], Infinity, 1];	 
     guideTutorialCollectionGroup = If[guideTutorialCollectionGroup === {Cell[CellGroupData[{}, Open]]}, {}, guideTutorialCollectionGroup];	 
     guideTutorialCollectionGroup = guideTutorialCollectionGroup /. Cell[c_String, "GuideTutorialCollectionSection", o___] :>	 
       Cell[ c, "GuideTutorialCollectionSection", o];	 
 guideTutorialCollectionGroup];
 
 (******************************************************************************)
(** GetIndicatorExampleGroup - Internal **)
(* get the GetIndicatorExampleGroup cell group and make necessary adjustments *)
GetIndicatorExampleGroup[expr_, language_String:"English"] := Module[{indicatorexampleGroup},
  indicatorexampleGroup = 
    Cases[expr, 
      Cell[CellGroupData[{
        Cell[_, "IndicatorExampleSection", ___],
        ___}, ___], ___],
      Infinity, 1];
  indicatorexampleGroup = indicatorexampleGroup /. Cell[_,"IndicatorExampleSection",b_] :> 
  		Cell[Localized["IndicatorExampleSectionText", language],"IndicatorExampleSection",b];
  indicatorexampleGroup
];

 (******************************************************************************)
(** GetIndicatorDescriptionGroup - Internal **)
(* get the GetIndicatorDescriptionGroup cell group and make necessary adjustments *)
GetIndicatorDescriptionGroup[expr_, language_String:"English"] := Module[{indicatordescriptionGroup},
  indicatordescriptionGroup = 
    Cases[expr, 
      Cell[CellGroupData[{
        Cell[_, "IndicatorDescriptionSection", ___],
        ___}, ___], ___],
      Infinity, 1];
  indicatordescriptionGroup = indicatordescriptionGroup /. Cell[_,"IndicatorDescriptionSection",b_] :> 
  		Cell[Localized["IndicatorDescriptionSectionText", language],"IndicatorDescriptionSection",b];
  indicatordescriptionGroup
];

 (******************************************************************************)
(** GetIndicatorFormulaGroup - Internal **)
(* get the GetIndicatorFormulaGroup cell group and make necessary adjustments *)
GetIndicatorFormulaGroup[expr_, language_String:"English"] := Module[{indicatorformulaGroup},
  indicatorformulaGroup = 
    Cases[expr, 
      Cell[CellGroupData[{
        Cell[_, "IndicatorFormulaSection", ___],
        ___}, ___], ___],
      Infinity, 1];
  indicatorformulaGroup = indicatorformulaGroup /. Cell[_,"IndicatorFormulaSection",b_] :> 
  		Cell[Localized["IndicatorFormulaSectionText", language],"IndicatorFormulaSection",b];
  indicatorformulaGroup
];


(******************************************************************************)
(** GetExamplesGroup - Internal **)
(* get the examples cell group and make necessary adjustments *)
GetExamplesGroup[expr_, language_String:"English"] := Module[
  {primaryExamplesGroup, extendedExamplesGroup, examplesGroup, openicon, closedicon},

  openicon = Get[DocsAsset["Graphics:Opener", "SectionOpenerOpenOff.m"]];
  closedicon = Get[DocsAsset["Graphics:Opener", "SectionOpenerClosedOff.m"]];

  (* Extract primary and extended groups *)
  primaryExamplesGroup = 
    Cases[expr,
      Cell[CellGroupData[{
        Cell[_, "PrimaryExamplesSection", ___],
        ___}, ___], ___],
      Infinity, 1];
  extendedExamplesGroup = 
    Cases[expr,
      Cell[CellGroupData[{
        Cell[_, "ExtendedExamplesSection", ___],
        ___}, ___], ___],
      Infinity, 1];

  (* Remove leading cells *)
  primaryExamplesGroup = Replace[primaryExamplesGroup,
    Cell[CellGroupData[l:{Cell[_, "PrimaryExamplesSection", ___], ___}, _], ___]
      :> Rest[l], Infinity];
  extendedExamplesGroup = Replace[extendedExamplesGroup,
    Cell[CellGroupData[l:{Cell[_, "ExtendedExamplesSection", ___], ___}, _], ___]
      :> Rest[l], Infinity];

  (* Assemble a single "Examples" group *)
  examplesGroup = With[{openicon = openicon, closedicon = closedicon},
    Cell[CellGroupData[Flatten[{
      Cell[TextData[{
        Cell[BoxData[DynamicBox[
          ToBoxes[If[MatchQ[CurrentValue[EvaluationNotebook[], {TaggingRules, "Openers", "PrimaryExamplesSection"}, Open],True|Open],
            openicon,
            closedicon]]]]],
        Cell[BoxData[ToBoxes[Spacer[1]]]],
        Localized["PrimaryExamplesSectionText", language]}], "PrimaryExamplesSection",
        CellTags->"PrimaryExamplesSection", CellID->1],
      Cell[CellGroupData[Flatten@{
        Cell[Localized["BasicExamplesSectionText", language], "ExampleSection",CellID->1],
        primaryExamplesGroup}, Open]],
      extendedExamplesGroup
    }], Open]]];

  (* Remove 'Interactive Examples' *)
  (* If[buildmode==="Future",
  examplesGroup,
  examplesGroup = DeleteCases[examplesGroup,
    Cell[CellGroupData[{
        Cell[BoxData[InterpretationBox[
          Cell["Interactive Examples", "ExampleSection", ___],___]], "ExampleSection",___],
        ___},___],___],
      Infinity]]; *)
      

  (* Remove ExampleDelimiter which follow *Section headings *)
  examplesGroup = DeleteCases[examplesGroup, Cell[CellGroupData[{},Open]], Infinity];
  examplesGroup = DeleteCases[examplesGroup, Cell[CellGroupData[{},Open]], Infinity];
  examplesGroup = DeleteCases[examplesGroup, Cell[CellGroupData[{},Open]], Infinity];
      examplesGroup = examplesGroup //. (
    Cell[CellGroupData[{aa___,
      Cell[_, "ExampleDelimiter", ___],
      Cell[CellGroupData[{
        Cell[a_, "ExampleDelimiter", b___], c___}, d___], e___], f___}, g___], h___]
    :>
    Cell[CellGroupData[{aa,
      Cell[CellGroupData[{
        Cell[a, "ExampleDelimiter", b], c}, d], e], f}, g], h] ); 
        
    examplesGroup = examplesGroup //. (
    Cell[CellGroupData[{aa___,
      Cell[CellGroupData[{Cell[_, "ExampleDelimiter", ___]}, ___],___],
      Cell[CellGroupData[{
        Cell[a_, "ExampleDelimiter", b___], c___}, d___], e___], f___}, g___], h___]
    :>
    Cell[CellGroupData[{aa,
      Cell[CellGroupData[{
        Cell[a, "ExampleDelimiter", b], c}, d], e], f}, g], h] ); 
  
    examplesGroup = examplesGroup //. (
    Cell[CellGroupData[{
      Cell[con_, sty:("ExampleSection" | "ExampleSubsection" | "ExtendedExamplesSection"), o___],
      Cell[_, "ExampleDelimiter", ___], d___}, e___], f___]
    :>
    Cell[CellGroupData[{
        Cell[con, sty, o], d}, e], f] ); 
        
     examplesGroup = examplesGroup //. (
    Cell[CellGroupData[{
      Cell[con_, sty:("ExampleSection" | "ExampleSubsection" | "ExtendedExamplesSection"), o___],
      Cell[CellGroupData[{
        Cell[_, "ExampleDelimiter", ___]}, b___], c___], d___}, e___], f___]
    :>
    Cell[CellGroupData[{
        Cell[con, sty, o], d}, e], f] ); 
        
   examplesGroup = examplesGroup //. (
    Cell[CellGroupData[{
      Cell[con_, sty:("ExampleSection" | "ExampleSubsection" | "ExtendedExamplesSection"), o___],
      Cell[CellGroupData[{
        Cell[_, "ExampleDelimiter", ___], a___}, b___], c___], d___}, e___], f___]
    :>
    Cell[CellGroupData[{
        Cell[con, sty, o],
        Cell[CellGroupData[{ a}, b], c], d}, e], f] );

  (* Close ExamplesSection groups *)
  examplesGroup = examplesGroup /. (
    Cell[CellGroupData[{
      Cell[con_, "ExampleSection", o___],
      cs___}, ___], g___]
    :> 
    Cell[CellGroupData[{
      Cell[con, "ExampleSection", "ExampleSection", o], cs },
      If[(StringQ[con] && StringMatchQ[con, Localized["ExtendedExamplesSectionText", language]]),
        Open, Closed]], g] );

  (* Close ExamplesSubsection groups *)
  examplesGroup = examplesGroup /. (
    Cell[CellGroupData[{
      Cell[con_, "ExampleSubsection", o___],
      cs___}, ___], g___]
    :> 
    Cell[CellGroupData[{
      Cell[con, "ExampleSubsection", "ExampleSubsection", o], cs }, Closed], g] );

  (* If no examples, do not include groups *)
  examplesGroup = 
    If[MatchQ[examplesGroup,
      Cell[CellGroupData[{
        Cell[TextData[{___, Localized["PrimaryExamplesSectionText", language], ___}],
          "PrimaryExamplesSection", ___, CellTags->"PrimaryExamplesSection"]
      }, ___], ___]],
      {},
      examplesGroup
    ];

  examplesGroup];


(******************************************************************************)
(** MakeAnchorBarCell - Internal **)
Options[MakeAnchorBarCell] = {
    "NewIn"->False,
    "IncludeSections"->Automatic,
    "BuildMode" -> "Normal",
    "PacletName" -> "Mathematica"
};
MakeAnchorBarCell[expr_, info_, language_String:"English", opts___?OptionQ] :=
Module[{
    func, styles, list, am, uri, type, buttonList = {}, buildmode, stylesOrder, pacletname, lclstr, wsmlinkQ,
	spacer = "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]",
	translationglobeimg,
	grayicon
},

  (* Assert that `info` is a list of rules *)
  Assert[MatchQ[info, {RepeatedNull[_ -> _]}]];

  BuildLog["** Entering MakeAnchorBarCell **"];
  styles = "IncludeSections" /. {opts} /. Options[MakeAnchorBarCell]; 
  buildmode = "BuildMode" /. {opts} /. Options[MakeAnchorBarCell];
  pacletname = "PacletName" /. {opts} /. Options[MakeAnchorBarCell];
  styles = If[ styles === Automatic, {"Tutorials", "Workflow", "MoreAbout", "SeeAlso" }, styles];
  stylesOrder=If[("EntityType" /. info)==="Tutorial",styles,Reverse@styles];
  translationglobeimg = Import[DocsAsset["Graphics", "TranslationGlobe.m"]];
  grayicon = Import[DocsAsset["Graphics", "AnchorDownArrowOff.m"]];
  uri = "URI" /. info;
  type = "EntityType" /. info;


  BuildLog["-- buttonList: "<>ToString[buttonList] ];
  (* FIXME: localization *)
  Scan[Function[{style},
    BuildLog["-- "<>style];
      lclstr = Switch[style,
          "PrimaryExamplesSection", "ExamplesSection",
          "GuideFunction", "GuideFunction",
          "GuideMoreAbout", "GuideMoreAboutSection",
          "GuideTutorial", "GuideTutorialsSection",
          "MoreAbout", "MoreAboutSection",
          "SeeAlso", "SeeAlsoSection",
          "Tutorials", "TutorialsSection",
          "Workflow", "WorkflowsSection",
          _, style
      ];
      (* Note:
        We use the absolute context path so that this is sure to work even if
        $ContextPath doesn't currently contain DocumentationBuild`Utils`.
      *)
      func = ToExpression@StringJoin["DocumentationBuild`Utils`Get", lclstr, "List"];
      Assert[Developer`SymbolQ[func] && ValueQ[func]];

      list = If[StringMatchQ[uri, "*workflow/*"] && MatchQ[lclstr, "SeeAlsoSection" | "RelatedFunctionsSection"],
        func[expr, "IncludePaclets"->True, "PacletName" -> ("PacletName" /. info), "Language"->language, "BuildMode"->buildmode, "PacletName"->pacletname, "PageType"->"Workflow" ],
        func[expr, "IncludePaclets"->True, "PacletName" -> ("PacletName" /. info), "Language"->language, "BuildMode"->buildmode, "PacletName"->pacletname ]
      ];

      (* Filter out {}, returned when the underlying GetJoinedButtonCaptions[] doesn't
         find a ButtonBox. TODO: GetJoinedButtonCaptions should arguably return None or
         Missing instead of any empty list. *)
      list = Select[list, # =!= {} &];

      (* Assert that `list` is a list of rules or pairs*)
      If[!MatchQ[list, {RepeatedNull[_ -> _]} | {RepeatedNull[{_, _}]}],
        Message[MakeNotebook::warning, StringForm["Unexpected form for '``' link data: `` in ``. Ignoring.", style, list, uri]];
        (* Pretend we didn't process a link at all. This isn't a big enough deal to stop
           the build over. *)
        list = {};
      ];

		If[Flatten[list] =!= {},
			list = Select[
				list,
				(
					MatchQ[
						DocumentationBuild`Info`LookupSourceInformation[
							StringReplace[ToPacletURI[#[[2]]], "#" ~~ __ -> ""],
							language,
							"Flag",
							None
						],
						None | "None"
					]
					(* if it's in builds already, go ahead and let it through *)
					(* Formatting note: 8b60a39f: hopefully fix 418370 *)
					(*|| StringQ[Documentation`ResolveLink[replace[#[[2]]]]]*)
				) &
			];
		];
      BuildLog["-- "<>ToString@{func, list}];
      If[ !MemberQ[{ {{}}, {}}, list],
        If[lclstr === "SeeAlsoSection" && !StringMatchQ[uri, "*workflow/*"],
          list = DelimitAndLabelSubSeeAlsos[list, type, language];
        ];
        am = Cell[BoxData@TagBox[ToBoxes@MakeActionMenu[
          TextCell[Row[{Localized[lclstr<>"Text", language], " ", grayicon}],StripOnInput->True], style,
          list],MouseAppearanceTag["LinkHand"]], LineSpacing->{1.4, 0}];
        buttonList = Join[buttonList, {am}]
    ]], stylesOrder];

  (* add function browser button *)
  buttonList =
    If[False,
      BuildLog["-- Adding function browser button"];
      Append[buttonList, Cell[TextData["function browser"]] ],
      BuildLog["-- No function browser button"];
      buttonList
    ];

  (* add url menu *)
  uri = "URI" /. info;
  wsmlinkQ = StringMatchQ[uri, "*WSMLink/*"];
  buttonList = Append[buttonList,
    Cell[BoxData[TagBox[ToBoxes@ActionMenu[Framed[TextCell[Row[{Localized["url", language], " ", grayicon}],StripOnInput->True]], Flatten@{
      StringReplace[uri, "paclet:" -> ""] :> None,
      With[{uri=uri},
        Localized["Copy Mathematica url", language] :>
          Module[{DocumentationSearch`Private`nb}, 
            DocumentationSearch`Private`nb = 
              NotebookPut[Notebook[{Cell[uri]}, Visible -> False]];
            SelectionMove[DocumentationSearch`Private`nb, All, Notebook];
            FrontEndTokenExecute[DocumentationSearch`Private`nb, "Copy"];
            NotebookClose[DocumentationSearch`Private`nb];
      ]],
      If[ !StringMatchQ[uri, "*ref/message/*"], 
        {
        Delimiter,
        With[{uri=StringJoin[If[TrueQ[wsmlinkQ], "http://reference.wolfram.com/system-modeler/", 
        	"http://reference.wolfram.com/language/"], uri, ".html"]},
          Localized["Copy web url", language] :>
            Module[{DocumentationSearch`Private`nb}, 
              DocumentationSearch`Private`nb = NotebookPut[
                Notebook[{Cell[BoxData@MakeBoxes[Hyperlink[uri], StandardForm], 
                  "Input", System`TextClipboardType -> "PlainText"]}, Visible -> False]];
              SelectionMove[DocumentationSearch`Private`nb, All, Notebook];
              FrontEndTokenExecute[DocumentationSearch`Private`nb, "Copy"];
              NotebookClose[DocumentationSearch`Private`nb];
        ]],
        With[{uri=uri},
          Localized["Go to web url", language] :>
            FrontEndExecute[{NotebookLocate[{URL[
              StringJoin[If[TrueQ[wsmlinkQ], "http://reference.wolfram.com/system-modeler/", 
        	"http://reference.wolfram.com/language/"], uri, ".html"]
            ], None}]}] ]
        },
        (* Else *)
        {} ]
    }, Appearance -> None, MenuAppearance -> Automatic, MenuStyle -> "URLMenu"], MouseAppearanceTag["LinkHand"]]], LineSpacing->{1.4, 0}]];
  
  (* Add translation language menu *)
  (* Removed for 10.1 release -- 2/24/15 *)
  (*If[MatchQ["EntityType" /. info, "Symbol"],
    buttonList = Append[buttonList,
    Cell[BoxData[ToBoxes@ActionMenu[Framed[TextCell[Row[{RawBoxes[AdjustmentBox[translationglobeimg, BoxBaselineShift -> -0.25]], Style[" \[FilledDownTriangle]", "AnchorBarArrow"]}]]], Flatten@{
      Map[Row[{Checkbox[Dynamic[CurrentValue[EvaluationNotebook[], {TaggingRules, "TranslationLanguage"}] === #]], "  "<>#}] :>
        (CurrentValue[EvaluationNotebook[], {TaggingRules, "TranslationLanguage"}] = #)&, translationLanguageList]
    }, Appearance -> None, MenuAppearance -> Automatic, MenuStyle -> "TranslationMenu"]], LineSpacing->{1.4, 0}]]
  ];*)

  buttonList = Flatten[{ Riffle[buttonList, spacer] }];
  BuildLog["-- "<>ToString[Cases[buttonList, ButtonBox[c_,___]:>c,Infinity]] ];
  BuildLog["** Leaving MakeAnchorBarCell **"];

  If[(Length@buttonList > 0) && (First[buttonList] =!= " "),
    Cell[TextData[buttonList], "AnchorBar", CacheGraphics->False], " "]
];

(******************************************************************************)
(** MakeInlineSeparator - Internal **)
MakeInlineSeparator[Cell[TextData[lst_List], o___]] :=
  Module[{new},
    new = Map[MakeInlineSeparatorString, lst];
    Cell[TextData[Flatten@new], o]
  ];
MakeInlineSeparator[c___] := c;  

MakeInlineSeparatorString[str_String] :=
 Sequence @@ 
  StringReplace[
   str, {RegularExpression[
      "(.*)\\s+\[EmptyVerySmallSquare]\\s+(.*)"] :> {"$1", "\[NonBreakingSpace]", 
      StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", "InlineSeparator"], " ", 
      "$2"}}]
MakeInlineSeparatorString[c___] := c;


(******************************************************************************)
(** DelimitAndLabelSubSeeAlsos - Internal **)
DelimitAndLabelSubSeeAlsos[seeAlsos_List, type_String, lang_String:"English"]:=
  Module[{splitlist, URIToTitleRules},
    URIToTitleRules = 
      {
        (* these are ordered per SW *)
        (s_String/;StringMatchQ[s,"paclet:*"]) | "ref"->Localized["SeeAlsoRelatedFunctionsText", lang],
        "entity"->Localized["SeeAlsoRelatedEntityTypesText", lang],
        "interpreter"->Localized["SeeAlsoRelatedInterpreterTypesText", lang],
        "identifier"->Localized["SeeAlsoRelatedIdentificationTypesText", lang],  (* doesn't exist yet *)
        "indicator"->Localized["SeeAlsoRelatedIndicatorTypesText", lang],
        "classifier"->Localized["SeeAlsoRelatedClassifiersText", lang],
        "predictor"->Localized["SeeAlsoRelatedPredictorsText", lang],
        "textcontent"->Localized["SeeAlsoRelatedTextContentsText", lang],
        "netencoder"->Localized["SeeAlsoRelatedNetEncodersText", lang],
        "netdecoder"->Localized["SeeAlsoRelatedNetDecodersText", lang],
        "method"->Localized["SeeAlsoRelatedMachineLearningMethodsText", lang],
        "compiledtype"->Localized["SeeAlsoRelatedCompiledTypesText", lang],
        "datastructure"->Localized["SeeAlsoRelatedDataStructuresText", lang],
        "format"->Localized["SeeAlsoRelatedFormatsText", lang],
        "resource"->Localized["SeeAlsoRelatedResourceTypesText", lang],
        "service"->Localized["SeeAlsoRelatedServiceConnectionsText", lang],
        "device"->Localized["SeeAlsoRelatedDeviceTypesText", lang],
        "databaseconnection"->Localized["SeeAlsoRelatedDatabaseConnectionsText", lang],
        "embeddingformat"->Localized["SeeAlsoRelatedExternalEmbeddingFormatsText", lang],
        "externalevaluationsystem"->Localized["SeeAlsoRelatedExternalEvaluationSystemsText", lang],
        "c"->Localized["SeeAlsoRelatedCFunctionsText", lang],
        "program"->Localized["SeeAlsoRelatedProgramsText", lang],
        "file"->Localized["SeeAlsoRelatedSystemFilesText", lang],
        "menuitem"->Localized["SeeAlsoRelatedMenuItemsText", lang],
        "frontendobject"->Localized["SeeAlsoRelatedFrontEndTokensText", lang],
        "character"->Localized["SeeAlsoRelatedCharactersText", lang]
      };
    splitlist = Riffle[GatherBy[seeAlsos, Most[StringSplit[#[[2]],"/"]]&], Delimiter];
    
    If[MemberQ[splitlist, Delimiter, Infinity],
      (* sort based on list above *)
      splitlist = Riffle[SortBy[DeleteCases[splitlist, Delimiter],
        Position[URIToTitleRules, StringSplit[#[[1,2]], "/"][[-2]]] &], Delimiter];
      
      Module[{delimpos = Reverse[Position[splitlist, Delimiter]], sectpos, secttype},
        
        Map[
          (sectpos = # + {1};
           secttype = StringSplit[Flatten[splitlist[[sectpos]]][[2]],"/"][[-2]] /. URIToTitleRules;
           splitlist = Insert[splitlist, {{Style[secttype, "SeeAlsoSection", FontSize->14, FontSlant->"Italic"], None}}, sectpos]
          )&, delimpos];
        splitlist = Flatten[splitlist, 1];
        (* plain weight for all subtypes *)
        delimpos = Position[splitlist, Delimiter];
        Do[
        splitlist[[i]] = Replace[splitlist[[i]], {name_String, uri_String} :> {Style[StringReplace[name, "\""->""], "SeeAlsoRelated"], uri}],
        {i, Flatten[First[delimpos]][[1]]+1, Length[splitlist]}
        ];
      ],
      splitlist = splitlist[[1]]
    ];
    splitlist
  ];


(******************************************************************************)
(** AdjustModInfo - Internal **)

AdjustModInfo[nb_, info_, opts___?OptionQ] :=
  Module[{expr = nb, poss, buildmode},
  	buildmode = "BuildMode" /. {opts} /. Options[MakeNotebook]; 
    BuildLog["-- Apply ModInfo across tables"];
    poss = Position[expr, Cell[_, "1ColumnTableMod"|"2ColumnTableMod"|"3ColumnTableMod"|"3ColumnIllustratedTableMod", ___], Infinity];
    (*Global`AntLog["Before Scan[]"];*)
    Scan[Function[{pos}, Module[{note, cell, mod, gridp, grid, sp},
      (*Global`AntLog["Running..."];*)
      note = Extract[expr, Append[Most[pos], Last[pos] - 1]];
      If[MatchQ[note, Cell[___,
          CellDingbat~(Rule|RuleDelayed)~Cell[s_String /; (CheckVersion[s, opts] > 0),
          "ModInfo", ___], ___]],
        mod = CellDingbat /. List@@Cases[note, _~(Rule|RuleDelayed)~_];
        cell = Extract[expr, pos];
        gridp = Position[cell, _GridBox, Infinity];
        If[Length[gridp] > 0,
          gridp = First[gridp];
          grid = Extract[cell, gridp];
          sp = False;
          expr = ReplacePart[expr, Join[pos, gridp] ->
              GridBox[Map[
                  Function[{row}, Module[{nrow},
                    nrow = Join[
                      If[(sp && (row[[2]] === "")) || !FreeQ[First[row], "BrighterFlag"] || (CheckVersion[row[[1, 1]]] === Infinity),
                        {First[row]},
                        {mod}],
                      Rest[row]];
                    If[Last[nrow] === "\[SpanFromLeft]", sp = True, sp = False];
                    nrow
                  ]],
                  First[grid]],
                Sequence@@Rest[grid]
          ]];
      ]];
    ]], Select[poss, (Last[#] > 1)&]];
    (*Global`AntLog["After Scan[]"];*)
    (* remove ModInfo prior to threshhold *)
  If[
    (* avoid running CheckVersion on nbs without ModInfo cells *)
    Positive @ Count[expr, Cell[_String, "ModInfo", ___], Infinity]
  , expr = If[
      buildmode==="Future"
    , expr
    , expr /.
        Cell[c_String, "ModInfo", ___] :> Cell["      ", "ModInfo"] /; CheckVersion[c, opts] === 0
    ]
  ,
    expr
  ];

    expr = expr /.
      Cell[con___, CellDingbat~(Rule|RuleDelayed)~Cell[c_String, "ModInfo", ___], o___]
      :> Cell[con, o,
        Background->Dynamic[If[CurrentValue[{TaggingRules, "ModificationHighlight"}]===True, RGBColor[1, 0.988235, 0.721569], None]]
      ] /; (CheckVersion[c, opts] === 1);

    (*Global`AntLog["Usage cell"];*)
    BuildLog["-- Remove ModInfo from Usage cell"];
    expr = AdjustModInfoUsageCells[expr, info, opts];

    (*Global`AntLog["Tables cells"];*)
    BuildLog["-- Remove ModInfo from tables"];
    expr = AdjustModInfoTables[expr, info, opts];
    expr = DeleteCases[expr, Cell[BoxData[GridBox[{},___]], ___], Infinity];

    (*Global`AntLog["Remove tentatives"];*)
    (* Remove "FutureExample" | "TentativeExample" tagged cells *)
    BuildLog["-- Remove TentativeExample tagged cells"];
    Which[
    buildmode==="Future",
    expr,
    ("Guide"===("EntityType"/.info))&&("Underdevelopment"/.info),
    expr = DeleteCases[expr,
      Cell[CellGroupData[{Cell[___, "FutureExample" | "TentativeExample"(*|"ExcludedMaterial"*), ___],
        ___}, ___], ___],
      Infinity];
      
    expr = DeleteCases[expr, Cell[CellGroupData[{Cell[___, "ExcludedMaterial", ___],___},___]],Infinity];
    expr = DeleteCases[expr, Cell[___, "ExcludedMaterial", ___],Infinity];
    expr = DeleteCases[expr, Alternatives[
        Cell[___, "FutureExample" | "TentativeExample" | "ExcludedMaterial", ___],
        Cell[__, CellDingbat->Cell[__, "BrighterFlag", ___], ___]
      ],
      Infinity];
      expr=expr/.Cell[a__, CellDingbat->Cell[__, "TentativeID", b___], c___]:>Cell[a, c],
    True,
    expr = DeleteCases[expr,
      Cell[CellGroupData[{Cell[___, "FutureExample" | "TentativeExample" | "ExcludedMaterial", ___],
        ___}, ___], ___],
      Infinity];
    expr = DeleteCases[expr, Alternatives[
        Cell[___, "FutureExample" | "TentativeExample" | "ExcludedMaterial", ___],
        Cell[__, CellDingbat->Cell[__, "TentativeID" | "BrighterFlag", ___], ___],
        Cell[__, CellDingbat->Cell[s_String /; (CheckVersion[s, opts] === Infinity), "ModInfo", ___], ___]
      ],
      Infinity]
    ];

  If[
    (* avoid running CheckVersion on notebooks without TableRowIcon cells *)
    Positive @ Count[expr, Cell[_String, "TableRowIcon", o___], Infinity]
  , expr = expr /.
      Cell[c_String, "TableRowIcon", o___] :>
      Cell["\[FilledVerySmallSquare]", "TableRowIcon", FontColor->None, o] /; CheckVersion[c, opts] > 0
  , expr
  ];

    (*Global`AntLog["Remove empty ModInfo"];*)
    (* remove empty ModInfo *)
    BuildLog["-- Remove empty ModInfo"];
    expr = expr /.
      Cell[a___, Rule[CellDingbat, Cell["", "ModInfo", ___]], h___] :> Cell[a,h];

    expr];


(******************************************************************************)
(** AdjustModInfoUsageCells **)

AdjustModInfoUsageCells[nb_, info_, opts___?OptionQ] :=
  Module[{expr = nb, poss, repl, usageNameCell},
    BuildLog["** Entering AdjustModInfoUsageCells **"];
    poss = Position[expr, Cell[_, "Usage"|"DeviceDoubleUsage", ___], Infinity];
    repl = Reap[Scan[
      Function[{pos}, Module[{cexpr, spl},
        cexpr = Extract[expr, pos];
        If[Head[First[cexpr]] === TextData,
          spl = Split[cexpr[[1, 1]],
            (!StringQ[#1] || (StringCount[#1, "\n"] == 0))&];
          spl = Map[Function[{sp}, If[
            MatchQ[First[sp],
              Cell[c_String /;
                  !(StringMatchQ[c, RegularExpression["\\s*"]] || CheckVersion[c, opts] =!= Infinity),
                "ModInfo", ___] ],
            Module[{str = Last[sp],
                nl = (StringQ[str] && StringPosition[str, "\n"])},
              If[StringQ[str] && (Length[nl] > 0) &&
                  (nl[[1, 1]] =!= StringLength[str]),
                {StringDrop[str, nl[[1, 1]]]},
                {}]],
            sp]],
            spl];
          spl = Flatten[spl];
          If[StringQ[Last[spl]], Module[{str, nl},
            str = Last[spl];
            nl = StringPosition[str, "\n"];
            If[(Length[nl] != 0) && (nl[[-1, 1]] == StringLength[str]),
              spl[[-1]] = StringDrop[str, -1]]
          ]];
          cexpr = ReplacePart[cexpr, {1, 1} -> spl];
          Sow[pos -> cexpr];
          (*,
          cexpr = Cell[BoxData[GridBox[{ {"", Cell[ First@cexpr ]} }]], "FormatUsage", CellID->1 ];
          Sow[pos -> cexpr];*)
        ]
    ]], poss]];

    If[Length[repl[[2]]] != 0,
      repl = repl[[2, 1]];
      expr = ReplacePart[expr, repl] ];

    poss = Position[expr, Cell[_, "Usage"|"DeviceDoubleUsage",___], Infinity];
    usageNameCell = Extract[expr, poss];
    
    (* Map over potential lists instead of assuming singleton *)
    (*If[Length[usageNameCell] > 0, usageNameCell = First@usageNameCell];*)
    If[!ListQ[usageNameCell], usageNameCell = {usageNameCell}];
    (* turn Usage cell into GridBox for easier ModInfo managemet *)
    Do[If[Head@usageNameCell[[i]] === Cell && Head@First@usageNameCell[[i]] === TextData,
      Module[{rows, rowOpts},
        rows = Split[usageNameCell[[i, 1, 1]],
          (!StringQ[#1] || (StringTake[#1, -1] =!= "\n"))&];
        rows = Map[
          If[StringQ[Last[#]] && StringTake[Last[#], -1] === "\n",
            Join[Most[#], {StringDrop[Last[#], -1]}],
            #]&,
          rows];
        usageNameCell[[i]] = ReplacePart[usageNameCell[[i]], {1} :> BoxData[GridBox[
          Map[
            If[MatchQ[First[#], Cell[_, "ModInfo", ___]],
              {First[#], Cell[TextData[Rest[#]]]},
              {"", Cell[TextData[#]]}
            ]&, rows]
        ]]];
        rowOpts = Map[
            If[MatchQ[First[#], Cell[c_String /; (CheckVersion[c, opts] > 0), "ModInfo", ___]],
              Hold[If[CurrentValue[{TaggingRules, "ModificationHighlight"}]===True,
                RGBColor[1, 0.988235, 0.721569], None]],
              None]&,
            rows];
        If[MemberQ[rowOpts, _Hold],
        	usageNameCell[[i]] = Append[usageNameCell[[i]],
        	  With[{rowOpts = rowOpts}, GridBoxOptions->{GridBoxBackground->
        	  	ReleaseHold[Dynamic[{"Rows"->rowOpts}]]}]]];
    ]];
    usageNameCell[[i]] = usageNameCell[[i]] /. Cell[_, "ModInfo", ___] :> "", {i, Length[usageNameCell]}];

    ReplacePart[expr, MapThread[Rule, {poss, usageNameCell}]]
  ];


(******************************************************************************)
(** AdjustModInfoTable **)
AdjustModInfoTables[nb_, info_, options___?OptionQ] :=
  Module[{expr = nb, poss, repl, buildmode},
  buildmode = "BuildMode" /. {options} /. Options[MakeNotebook]; 
  BuildLog["** Entering AdjustModInfoTables **"];
    poss = Position[expr,
      GridBox[{___, {Cell[_, "ModInfo", ___], ___}, ___}, ___],
      Infinity];
    repl = Reap[Scan[Function[{pos},
      Module[{drows, cpos, cexpr, cepos, gexpr, optpos},
        cpos = Last @ Select[(Take[pos, #]& /@ Range[Length[pos]]),
            (Head[First @ Extract[expr, {#}]] === Cell)& ];
        cepos = Drop[pos, Length[cpos]];
        gexpr = Extract[expr, pos];
        If[buildmode==="Future",
        drows={},
        drows = Position[gexpr,
            {Cell[_, "ModInfo", ___,  "BrighterFlag" | "TentativeExample" | "FutureExample", ___]
            |Cell[_, "ModInfo", ___, CellDingbat->Cell[___, "TentativeID" | "BrighterFlag", ___], ___]
            (* 02/28/20: checking version of modinfo and removing as necessary *)
            |Cell[c_String /; (CheckVersion[c, options] === Infinity), "ModInfo", ___],
            ___},
            Infinity]
        ];

        gexpr = If[
        	buildmode==="Future",
        	gexpr,
        	Delete[gexpr, drows]
        ];

		Scan[
			Function[{rowIdx},
				(* Transform the first element of every row in the GridBox. *)
				gexpr = replaceAt[gexpr, {1, rowIdx, 1}, {
					Cell[a_, "ModInfo", b___] :> Cell[a, "TableRowIcon", b],
					(* TODO: Should this case be a warning? Examples of this occur in the
					         PDEModels docs. *)
					" " :> " ",
					elem_ :> (
						DocsWarning[
							"First element in ModInfo GridBox row has unexpected structure: ``",
							InputForm[elem]
						];
						elem
					)
				}];
			],
			Range @ Length @ First @ gexpr
		];

        drows = Last /@ drows;

        cexpr = Extract[expr, cpos];
        cexpr = ReplacePart[cexpr, {cepos -> gexpr}];

        optpos = Position[cexpr, _[GridBoxOptions, _]];
        If[Length[optpos] != 0, Module[{opts, rowlp, rowl, spacp, spac},
            optpos = First[optpos];
            opts = Extract[cexpr, {optpos}][[1, 2]];

            rowlp = Position[opts, _[RowLines, _]];
            If[Length[rowlp] != 0,
                rowlp = First[rowlp];
                rowl = Extract[opts, {rowlp}][[1, 2]];
                If[ListQ[rowl],
                    rowl = Delete[
                        ReplacePart[rowl,
                            Map[(# -> Max[Extract[rowl, #-1], Extract[rowl, #]])&,
                                drows]],
                        {# - 1}& /@ drows];
                    opts = ReplacePart[opts, rowlp -> (RowLines -> rowl)];
            ]];

            spacp = Position[opts, _[RowSpacings, _]];
            If[Length[spacp] != 0,
                spacp = First[spacp];
                spac = Extract[opts, {spacp}][[1, 2]];
                If[ListQ[spac],
                    spac = Delete[
                        ReplacePart[spac,
                            Map[(# -> Max[Extract[spac, #-1], Extract[spac, #]])&,
                                drows]],
                        {# - 1}& /@ drows];
                    opts = ReplacePart[opts, spacp -> (RowSpacings -> spac)];
            ]];

            cexpr = ReplacePart[cexpr,
                optpos -> (GridBoxOptions -> opts)];
        ]];
        Sow[cpos -> cexpr];
    ]], poss]];
    BuildLog["** Leaving AdjustModInfoTables **"];

    If[Length[repl[[2]]] != 0,
        repl = repl[[2, 1]];
        ReplacePart[expr, repl],
        (* Else *)
        expr
  ]];


(******************************************************************************)
(** HighlightSectionHeaderOfGroup **)
(** Add mod highlighting to section header of any section that contains a highlighting cell **)
HighlightSectionHeaderOfGroup[group_]:=
  Module[{isList = True, newgroup, 
          bg = Background->Dynamic[If[CurrentValue[{TaggingRules, "ModificationHighlight"}]===True, RGBColor[1, 0.988235, 0.721569], None]]},
   If[group === {}, Return[group]];
   If[!ListQ[group], isList = False; newgroup = {group}, newgroup = group];
   Do[If[MatchQ[newgroup[[i]], Cell[CellGroupData[{__},___]]],
     If[!FreeQ[newgroup[[i]], "ModificationHighlight" | Cell["\[FilledVerySmallSquare]", "TableRowIcon", ___]],
     newgroup[[i]] = Replace[newgroup[[i]], Cell[CellGroupData[{head_Cell, rest___}, openq___]] :> Cell[CellGroupData[{Append[head,bg],rest},openq]]]
       ]
     ],{i,Length[newgroup]}];
   If[!TrueQ[isList], newgroup[[1]], newgroup]
   ]


(******************************************************************************)
(** RenumberCells **)
(** 
    RenumberCells[nbexpr, {"section1", ...}]
    This function attempts to snag all the In[]/Out[] cell labels
    and renumber each "section" starting from 1.  In order to do 
    this, it must have the in/out cells within a CellGroupData[]
    expression where the first cell style is "section"

    in:         Notebook expression | notebook filename
    out:        modified Notebook expression
    options:    DelimiterStyles

**)

Options[RenumberCells] = {DelimiterStyles->{"ExampleDelimiter"}, InputStyles->{"Input"}}

RenumberCells[$Failed,___] = $Failed

RenumberCells[file_String, sections__String, opts___?OptionQ] :=
  RenumberCells[file,{sections}, opts]

RenumberCells[file_String, sections:{__String}, opts___?OptionQ] :=
  RenumberCells[Import[file,"NB"],sections, opts]

RenumberCells[nb_Notebook, sections__String, opts___?OptionQ] :=
  RenumberCells[nb, {sections}, opts]

RenumberCells[nb_Notebook, sections:{__String}, opts___?OptionQ] := 
  Module[{expr=nb,delStyles,chkDelimiter=False, inStyles},

    (* parse opts *)
    delStyles = DelimiterStyles /. {opts} /. Options[RenumberCells];
    inStyles = InputStyles /. {opts} /. Options[RenumberCells];

    If[MatchQ[delStyles, { __String }],
      chkDelimiter=True; delStyles=Alternatives@@delStyles];

    (* Map over each section style to renumber *)
    Scan[
      Function[{style}, Module[{pos,sectionCells},
        pos = Position[expr, CellGroupData[{Cell[___,style,___],___},___]];

        (* only process if we got something to do *)
        If[pos=!={},
          (* get sections to process *)
          sectionCells = Extract[expr, pos];
          (* loop over each cell group we found *)
          newSections = Map[
            Function[{scell}, Block[
                {stuff = scell[[1]], open = scell[[2]], count = 0, looper},
              looper = Function[{this},
                Which[
                  MatchQ[this,
                    Cell[CellGroupData[{Cell[___, "MathCaption", ___], ___}, ___], ___]],
                  ReplacePart[this, {1, 1} -> (looper /@ this[[1, 1]])],

                  (* match input output cell groups *)
                  MatchQ[this,
                    Cell[CellGroupData[{Cell[___, Alternatives@@inStyles, ___], ___}, ___], ___]],
                  SetCellInOutNumbers[this,++count,InputStyles->inStyles],

                  (* match lone innie / outties *)
                  MatchQ[this,
                    Cell[___,  Alternatives @@ Join[inStyles, {"Output"}],___]],
                  SetCellInOutNumbers[this,++count,InputStyles->inStyles],

                  (* reset @ specified delimiters if we need to *)
                  chkDelimiter && MatchQ[this,
                    Cell[___,delStyles,___]],
                  count = 0; this,

                  (* otherwise return unmodified *)
                  True,
                  this
              ]];
            CellGroupData[looper /@ stuff, open]
          ]], sectionCells];
         
          (* replace old sections with new *) 
          Scan[(expr=ReplacePart[expr,#[[1]]->#[[2]]])&, Transpose[{pos,newSections}]]
        ]
      ]],
      sections];

    (* return processed nb expr *)
    expr
  ];


(******************************************************************************)
(** SetCellInOutNumbers - Internal **)
SetCellInOutNumbers[expr_,num_Integer,funcopts___?OptionQ] :=
  Module[{old, inStyles},
    inStyles = InputStyles /. {funcopts} /. Options[RenumberCells];
    expr /. 
    {
      (* rename all Input cell labels *)
      Cell[stuff___,st:(Alternatives@@inStyles),opts___]:> (
        Cell[stuff,st,
          Sequence@@DeleteCases[{opts},_[CellLabel,_]],
          CellLabel->"In["<>ToString[num]<>"]:="
        ]),

      (* don't forget any errors... for now this is hardcoded.*)
      Cell[stuff___,"Message",opts___]:>(
        Cell[stuff,"Message",
          Sequence@@DeleteCases[{opts},_[CellLabel,_]],
          CellLabel->("During evaluation of In["<>ToString[num]<>"]:=")
        ]),

      (* 
         output cells _need _ to have a cell label otherwise
         we have to guess if it should have //*Form or whatnot
         if, for some reason, there is no celllabel already,
        just hope "Out[num]=" is ok.
      *)
      Cell[stuff___,"Output",opts___]:>(
        old=(Cases[{opts},_[CellLabel,l_]:>l]/.{a_}->a);
        Switch[old,
          _String, 
            CellLabel->StringReplace[old,
              RegularExpression["Out\\[\\d+\\]"]->(
                "Out["<>ToString[num]<>"]")],
          _, 
            CellLabel->"Out["<>ToString[num]<>"]="
        ]//
        Cell[stuff,"Output",
          Sequence@@DeleteCases[{opts},_[CellLabel,_]], #]&
      )
    }
  ];


(******************************************************************************)
(** WrapInOpenCloseButton - Internal **)

(* WrapInOpenCloseButton[expr_]:=
  ButtonBox[expr, 
    BaseStyle->None,
    ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
    FrontEnd`SelectedNotebook[], All, ButtonCell], FrontEndToken["OpenCloseGroup"]}]& ),  
    ButtonFrame->"None", Evaluator->None, Method->"Preemptive"] *)

WrapInOpenCloseButton[expr_]:=
ButtonBox[expr, 
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[FrontEnd`SelectedNotebook[], All, ButtonCell],
    FrontEndToken["OpenCloseGroup"],
    FrontEnd`SelectionMove[FrontEnd`SelectedNotebook[], After, CellContents]
  }]& ),  
  ButtonFrame->"None", Evaluator->None, Method->"Preemptive"]


(******************************************************************************)
(** AddHeaderColorBar - Internal **)

AddHeaderColorBar[nb_Notebook, info_List, opts___?OptionQ] :=
  Module[{colorBarCell, type, linkTrailQ, expr=nb},
    linkTrailQ = If[Length@Flatten@Cases[nb, Cell[_,"LinkTrail",___], Infinity] > 0, True, False];
    type = Switch["EntityType" /. info,
      "Guide" | "Guide Overview" | "Alphabetical Listing", "Guide",
      "Tutorial", "Tutorial", "HowTo", "Tutorial",
      "Root Guide", None,
      _ , "Symbol"
    ];
    If[type =!= None,
      colorBarCell = If[linkTrailQ,
        Cell[" ", type<>"ColorBar", CellMargins->{{Inherited,Inherited},{-5,0}}],
        Cell[" ", type<>"ColorBar"]];
      MapAt[Flatten@Insert[#, {colorBarCell}, 1] &, expr, 1],
      expr
    ]
  ];


(******************************************************************************)
(** AddNotebookLabel - Internal **)

Options[AddNotebookLabel] = {"BuildMode"->"Normal", "PacletName"->"Mathematica", "Rebranded"->False};

AddNotebookLabel[nb_Notebook, info_List, lang_String, opts___?OptionQ]:=
  Module[{expr=nb, anchorBar, pacletNameCell, pn, anchorBarGrid, gridOpts, buildmode, pacletName, URI, $Rebranded,
   pacletnamecellbg = RGBColor[0.023529, 0.427451, 0.729412],
   pacletnamecellguidebg = RGBColor[0.850980, 0.396078, 0],
   pacletnamecellcharbg = GrayLevel[0.6],
   pacletnamecelltutorialbg = RGBColor[0.7450980392156863, 0.3254901960784314, 0.1411764705882353],
   pacletnamecelltechnotebg = RGBColor[0.3843137254901961, 0.4627450980392157, 0.7019607843137254],
   pacletnamecellcfunctionbg = RGBColor[0.247059, 0.498039, 0.121569],
   pacletnamecellmessagebg = RGBColor[0.86667, 0.06667, 0.],
   pacletnamecellworkflowbg = RGBColor[0.2157, 0.6588, 0.6745],
   pacletnamecellpackagebg = RGBColor[0.490196, 0.576471, 0.690196],
   pacletnamecellentitybg = RGBColor[0.996078431372549, 0.8901960784313725, 0.7215686274509804],
   pacletnamecellentitytext = RGBColor[0.396078431372549, 0.20784313725490197, 0.06274509803921569],
   pacletnamecelldefaultbg = GrayLevel[0.541176],
   pncellitemsize = Full,
   pncellafterspacer = ToBoxes[Spacer[8]],pncellbeforespacer = ToBoxes[Spacer[12]]},
    (* do not include nb label on Guide or Root Guide pages *)
    If[MemberQ[{(*"Guide", *)"Root Guide"}, ("EntityType" /. info)], Return[expr] ];
	(* make anchor bar *)
	buildmode = "BuildMode" /. {opts} /. Options[AddNotebookLabel];
	pacletName = "PacletName" /. {opts} /. Options[AddNotebookLabel];
	pn = "PacletName" /. info /. Options[AddNotebookLabel];
	URI = "URI" /. {info} /. Options[AddNotebookLabel];
	$Rebranded = "Rebranded" /. {opts} /. Options[AddNotebookLabel];
    anchorBar = MakeAnchorBarCell[expr, info, lang,
    	Which[ MatchQ[("EntityType" /. info), "Tutorial"|"Monograph"|"Monograph Overview"], 
    	  	"IncludeSections"->{"RelatedTutorialsSection", "TutorialMoreAboutSection", "TutorialFunction"},
    	  	("EntityType" /. info) === "Guide" && URI === {"guide/AlphabeticalListing"},
    	  	"IncludeSections"->{"GuideTutorial", "GuideMoreAbout"},
    	  	("EntityType" /. info) === "Guide", 
    	  	"IncludeSections"->{"GuideTutorial", "GuideMoreAbout", "GuideFunction"},
    	  	MatchQ[("EntityType" /. info), "ExamplePage" | "Example"], 
    	  	"IncludeSections"->{"FeaturedExampleTutorialsSection", "FeaturedExampleMoreAboutSection"},
    	  	("EntityType" /. info) === "Workflow",
    	  	"IncludeSections"->{"SeeAlsoSection", "RelatedFunctionsSection", "RelatedGuidesSection", "TechNotesSection"},
    	  	True, "IncludeSections"->Automatic ],
          If[Flatten[{Cases[expr, StyleBox[_, "ModInfo", ___], Infinity], 
              Cases[expr, Cell[_, "ModInfo", ___], Infinity]}] =!= {}, 
            (* FIXME: This will prohibit non-M- documentation to not have a "New in ..." graphic 
               This isn't an issue since we don't support that option in the authoring environment
               today, but might someday *)
            "NewIn"->If[("PacletName" /. info) =!= "Mathematica", False, True], 
            "NewIn"->False], "BuildMode"->buildmode, "PacletName"->pacletName
        ];  
	gridOpts = 
  	  If[Length@Cases[anchorBar, _ActionMenuBox, Infinity] < 3,
        GridBoxOptions -> { GridBoxItemSize->{"Columns" -> { Scaled[0.65], { Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}}}
        ,
        CellID -> 1
      ];
    BuildLog["-- Assemble anchor bar grid"];
    (* V9 EDIT BELOW *)
    pacletNameCell = 
      CapitalizeString[Transmogrify`ConvertToString[GetClassificationBoxes[info, lang, "Rebranded"->$Rebranded]]];
      
    anchorBarGrid = 
    	Cell[BoxData[GridBox[{
			{If[pacletNameCell === " ", " ", 
			  GridBox[{{ItemBox[Cell[BoxData[
			    RowBox[{pncellbeforespacer, 
			      If[!MatchQ["EntityType"/.info, "Entity"],
			    	Cell[pacletNameCell, "PacletNameCell", TextAlignment->Center],
			    	Cell[pacletNameCell, "PacletNameCell", TextAlignment->Center, FontColor->pacletnamecellentitytext]
			      ], pncellafterspacer}]], TextAlignment->Center], Background->
			  Switch["EntityType"/.info, 
			    "Symbol", 
			      If[StringMatchQ[pn, "Mathematica"],
			        pacletnamecellbg,
			        pacletnamecellpackagebg
			      ],
			    "Guide", pacletnamecellguidebg,
			    "Character Name", pacletnamecellcharbg,
			    "Tutorial"|"HowTo"|"Monograph"|"Monograph Overview", pacletnamecelltutorialbg,
			    "Overview" | "Tech Note", pacletnamecelltechnotebg,
			    (* removing these per design WLDOCS-1969 *)
			    (*"MathLink C Function"|"LibraryLink C Function"|*)"Program", pacletnamecellcfunctionbg,
			    "Message", pacletnamecellmessagebg,
			    "Workflow"|"Workflow Guide", pacletnamecellworkflowbg,
			    "Entity", pacletnamecellentitybg,
			    _, pacletnamecelldefaultbg],
			    ItemSize->pncellitemsize], ""}}, 
			    GridBoxAlignment->{"Rows"->{{Center}}},
			    GridBoxItemSize->{"Columns" -> {Full, Scaled[.02]}, "Rows"->{{2.5}}}]], 
			 (* 11.2 bit to remove drop down menus for one specific page *)
			 If[!StringMatchQ[URI[[1]],"guide/RecentlyAddedFeatures"], anchorBar, Nothing]}
		}(*,
		GridFrame->True,
		FrameStyle->GrayLevel[1],
		GridFrameMargins->{{.4, .4},{.5, -1.5}}*)]], "AnchorBarGrid", gridOpts, CellID->1];
    (* do not include nb label on Guide or Root Guide pages *)
    MapAt[Flatten@Insert[#, {anchorBarGrid}, 1] &, expr, 1]
  ];


(******************************************************************************)
(** CheckVersion - Internal **)
(** Returns:
      Infinity if s is greater than ReleaseVersion
      0 if s is less than Threshold Version
      some positive integer if s is between ThresholdVersion and ReleaseVersion
    Note that t<=s<=r currently only returns 1, but it might return other
    integers if we implement multiple threshholds.
**)
CheckVersion[s_String, opts___?OptionQ] :=
Module[{rversion, tversion, rtup, ttup, stup, str},
  (*Global`AntLog["Starting CheckVersion with s = " <> s];*)
  (* test for Future tagged Modinfo *)
  If[
  	StringMatchQ[s, RegularExpression["\\s*F\\s*"]] ||
  	!StringFreeQ[s, CharacterRange["a","z"]],
  	(*Global`AntLog["Ending CheckVersion (Returned s)"];*)Return[Infinity];
  ];
  rversion = "ReleaseVersion" /. {opts} /. Options[MakeNotebook];
  tversion = "ThresholdVersion" /. {opts} /. Options[MakeNotebook];
  
  (* fix for multiple versions in s from, e.g., tagging the same cell as modified in more than one version. 
       obviously, this will not be well-defined for point-point versions, but it shouldn't outright break *)
  str = If[!StringFreeQ[s, ","], First[ReverseSortBy[StringSplit[StringReplace[s,"+"|"~"->""], ","], ToExpression]], s];
    
  
  (* updating this code for robustness for 11.2 (so 11.1 stuff isn't highlighted as updated in 11.2) *)
  If[tversion === Automatic, 
    tversion = StringSplit[rversion, "."];
    If[Length[tversion] > 1 && ToExpression[tversion[[2]]] >= 1,
      tversion = StringJoin[Riffle[Flatten@{tversion[[1]],ToString[ToExpression[tversion[[2]]]-1]},"."]]
      ];
  ];
  (* end 11.2 update *)
  
  {rtup, ttup, stup} = Map[
    Function[{vers}, Module[{v, tup},
      v = StringCases[vers, RegularExpression["^[0-9\\.]+"]];
      If[v === {}, Return[{}], v = v[[1]]];
      tup = StringSplit[v, "."];
      ToExpression /@ tup
    ]],
    {rversion, tversion, str}];
  Which[
    LexCmp[stup, rtup] === 1, (*Global`AntLog["Ending CheckVersion (Infinity)"];*)Infinity,
    LexCmp[stup, ttup] > 0, (*Global`AntLog["Ending CheckVersion (1)"];*)1,
    True, (*Global`AntLog["Ending CheckVersion (0)"];*)0
]];


(******************************************************************************)
(** LexCmp - Internal **)
LexCmp[s1_, s2_] :=
  Which[
    s1 === s2 === {}, 0,
    s1 === {}, -1,
    s2 === {}, 1,
    s1[[1]] < s2[[1]], -1,
    s1[[1]] > s2[[1]], 1,
    True, LexCmp[Rest[s1], Rest[s2]]
  ];


(******************************************************************************)
(** ShippingFunctionQ - Internal **)
ShippingFunctionQ[___]:= False;
ShippingFunctionQ[func_String, ""] := ShippingFunctionQ[func];
ShippingFunctionQ[func_String, pacletName_String:"Mathematica"] := Module[{
	ref, flag,
	base
},
	(* Avoid looking at applications outside of Mathematica *)
	If[!DocumentationBuild`Internal`SystemPacletQ[pacletName],
		Return[True, Module];
	];

	base = If[StringMatchQ[pacletName, "Mathematica"],
		"",
		pacletName <> "/"
	];

	ref = If[StringMatchQ[func, "paclet:*"],
		func,
		"paclet:" <> base <> "ref/" <> func
	];

	If[HasSourceInformation[ref, "English"],
		flag = DocumentationBuild`Info`GetSourceInformation[ref, "English", "Flag"];
		FreeQ[{"ObsoleteFlag", "InternalFlag", "FutureFlag", "ExcisedFlag"}, flag]
		,
		(* Else *)
		False
	]
];


(******************************************************************************)
(** InlineFunctionButton **)
InlineFunctionButton[str_String] := InlineFunctionButton[ Cell[str,"SeeAlso"] ];
InlineFunctionButton[c___] := (Message[InlineFunctionButton::arg, c]; $Failed);
InlineFunctionButton[Cell[c_String, sty_, opts___] ] :=
Module[{n, f, fList},
  If[(Head@c === String),
    f = StringReplace[c, {"\n"->"", "\r"->""}];
    Which[
      (* . separator *)
      StringCount[f, RegularExpression["\\s+\\.\\s+"]] > 0, 
      (
        f = StringSplit[f, RegularExpression["\\s+\\.\\s+"]];
        fList = 
          Cell[BoxData[ButtonBox[#, BaseStyle -> "Link"]], "InlineFormula"] & /@ DeleteCases[f, s_String /; StringMatchQ[s, RegularExpression["XX+"]]];
        n = Flatten[Riffle[ fList, {{" ", StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"], " "}}], 1];
        Cell[BoxData[RowBox[n]], sty, opts]
      ),
      (* LongDash separator *)
      StringCount[f, RegularExpression["\\s+\[LongDash]\\s+"]] > 0, 
      (* for now, do nothing *)
        Cell[c, sty, opts],
      (*  *)
      StringCount[f, RegularExpression["\\s+\[EmptyVerySmallSquare]\\s+"]] > 0, 
      (
        f = StringSplit[f, RegularExpression["\\s+\[EmptyVerySmallSquare]\\s+"]];
        (* Commented out parts of the below line to fix autolinking of purposely unlinked inline
           listings (purposely because they have EmptyVerySmallSquares but aren't links) 
           See guide/InterpretingStrings (changed 4/2/14). *)
        fList = 
          Cell[BoxData[(*ButtonBox[*)#(*, BaseStyle -> "Link"]*)], "InlineFormula", "InlineGuideFunctionListingGray"] & /@ DeleteCases[f, s_String /; StringMatchQ[s, RegularExpression["XX+"]]];
        n = Flatten[Riffle[ fList, {{" ", StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"], " "}}], 1];
        Cell[BoxData[RowBox[n]], sty, opts]
      ),
     True, 
       Cell[BoxData[RowBox[{Cell[BoxData[ButtonBox[c, BaseStyle -> "Link"]], "InlineFormula"]}]], sty, opts]
    ]    
  ,
  (* Return original input *)
  Cell[c, sty, opts]
  ]
]
InlineFunctionButton::arg = "String expected as input but value was:`1`";

(******************************************************************************)
(** OtherRefAddUnfinishedDraftBanner - Internal **)
OtherRefAddUnfinishedDraftBanner[nbExpr_, info_, type_, opts___?OptionQ]:=
  Module[{expr = nbExpr, unfindraftimg, langext,pos, bannerpos},
  	langext = "Language" /. {opts} /. Options[MakeNotebook] /. {"English"->"", "Japanese"->"_jp", "ChineseSimplified"->"_zh",
  	"Spanish"->"_es" };
  	(* V11.1: changed how [[EXPERIMENTAL]] banner is displayed (so we don't need this function at all anymore. I'm no-oping it. *)
  	If[MatchQ[type, (*"Symbol"*)_], Return[expr]];
  	expr = If[MemberQ[expr, "UnderDevelopment", Infinity] || TrueQ["Underdevelopment"/.info],
      unfindraftimg = Import[DocsAsset["Graphics", "RefUnfinishedDraft"<>langext<>".m"]];
      Switch[type,
        "Device Connection",
        pos = Position[expr, Cell[___, "ObjectNameAlt", ___]];
        bannerpos = MapAt[(# + 1 &), pos, {1, -1}];
        Insert[expr, Cell[BoxData[ToBoxes[unfindraftimg]], "DeviceAbstract"], bannerpos],
        "Interpreter",
        pos = Position[expr, Cell[___, "ObjectNameAlt", ___]];
        bannerpos = MapAt[(# + 1 &), pos, {1, -1}];
        Insert[expr, Cell[BoxData[ToBoxes[unfindraftimg]], "DeviceAbstract"], bannerpos],
        _,
        expr
      ],
  	  expr];
    expr
  ];

(******************************************************************************)
(** translateTerm (from LarryA) - Internal **)
(** This function is for the translations at the tops of ref pages **)

translateTerm[term_String, language_String] :=
    DynamicModule[{translation},
     DynamicWrapper[
      Dynamic[translation],
      (
       FEPrivate`Set[translation,
        FEPrivate`TranslateKeyword[term, language]];
       FEPrivate`If[FEPrivate`SameQ[translation, $Failed],
        FEPrivate`Set[translation, ""]];
       translation
       )
      ]
     ];

(* TODO: update as new languages are added *)
translationLanguageList = {"English", "Japanese", "ChineseSimplified", "Korean", "Spanish"};

(******************************************************************************)
(** AddObjectNameGraphic - Internal **)
AddObjectNameGraphic[nbExpr_, info_, opts___?OptionQ]:=
Module[{pos, expr=nbExpr, rast, rastover, rasthide, rasthideover, unfindraftimg, langext, language, ce, dyn, histData, hist, uri, 
        translationboxes, type, NewInImgGen, NewImgNoVersion, UpdatedInImgGen, UpdatedImgNoVersion, rastNoVersion, platforminfo, icons,
        (*imagespathlist = DocsAsset["InternalOnlineImage", ...],*) updatedText, showChangesText, hideChangesText,
	newObjectNameCell
},
  LogIt["** Entering addObjectNameGraphic **"];
  (*lang = If[language === "English", "", "_jp"];*)
  
  NewInImgGen[version_String, lang:"English"]:=
   Style[Graphics[Text[Style[Localized["BannerNewIn", lang][version],
     FontFamily->"Source Sans Pro",
     FontSize->18,FontSlant->"Italic",
     FontWeight->"Bold",FontColor->RGBColor[.968627,.466667,0]], {Right, Center}, {Right, Center}], ImageSize->{125, 24}], "NewInGraphic"];
     
  NewInImgGen[version_String, lang:"Japanese"]:=
   Style[Graphics[Text[Style[Localized["BannerNewIn", lang][version],
     FontFamily->"Hiragino Kaku Gothic ProN",
     FontSize->18,
     FontWeight->"Bold",FontColor->RGBColor[.968627,.466667,0]], {Right, Center}, {Right, Center}], ImageSize->{180, 24}], "NewInGraphic"];
     
  NewInImgGen[version_String, lang:"ChineseSimplified"]:=
   Style[Graphics[Text[Style[Localized["BannerNewIn", lang][version],
     FontFamily->"Microsoft YaHei",
     FontSize->18,
     FontWeight->"Bold",FontColor->RGBColor[.968627,.466667,0]], {Right, Center}, {Right, Center}], ImageSize->{180, 24}], "NewInGraphic"];
     
  NewInImgGen[version_String, lang_] :=
   NewInImgGen[version, "English"];
   
  UpdatedInImgGen[version_String,lang:"English"]:=
   Style[Graphics[Text[Style[Localized["BannerUpdatedIn", lang][version],
     FontFamily->"Source Sans Pro",
     FontSize->17,FontSlant->"Italic",
     FontWeight->"Bold",FontColor->RGBColor[.968627,.466667,0]], {Right, Center}, {Right, Center}], ImageSize->{125, 24}], "NewInGraphic"];
  
  UpdatedInImgGen[version_String,lang:"Japanese"]:=
   Style[Graphics[Text[Style[Localized["BannerUpdatedIn", lang][version],
     FontFamily->"Hiragino Kaku Gothic ProN",
     FontSize->17,
     FontWeight->"Bold",FontColor->RGBColor[.968627,.466667,0]], {Right, Center}, {Right, Center}], ImageSize->{185, 24}], "NewInGraphic"];
  
  UpdatedInImgGen[version_String,lang:"ChineseSimplified"]:=
   Style[Graphics[Text[Style[Localized["BannerUpdatedIn", lang][version],
     FontFamily->"Microsoft YaHei",
     FontSize->17,
     FontWeight->"Bold",FontColor->RGBColor[.968627,.466667,0]], {Right, Center}, {Right, Center}], ImageSize->{185, 24}], "NewInGraphic"]; 
  
  UpdatedInImgGen[version_String, lang_] :=
   UpdatedInImgGen[version, "English"];
   
  UpdatedImgNoVersion[lang:"English"]:=
   Style[Graphics[Text[Style[Localized["BannerUpdatedNoVersion", lang],
     FontFamily->"Source Sans Pro",
     FontSize->18,FontSlant->"Italic",
     FontWeight->"Bold",FontColor->RGBColor[.968627,.466667,0]], {Right, Center}, {Right, Center}], ImageSize->{125, 24}], "NewInGraphic"];
     
   UpdatedImgNoVersion[lang:"Japanese"]:=
   Style[Graphics[Text[Style[Localized["BannerUpdatedNoVersion", lang],
     FontFamily->"Hiragino Kaku Gothic ProN",
     FontSize->18,
     FontWeight->"Bold",FontColor->RGBColor[.968627,.466667,0]], {Right, Center}, {Right, Center}], ImageSize->{180, 24}], "NewInGraphic"];
     
   UpdatedImgNoVersion[lang:"ChineseSimplified"]:=
   Style[Graphics[Text[Style[Localized["BannerUpdatedNoVersion", lang],
     FontFamily->"Microsoft YaHei",
     FontSize->18,
     FontWeight->"Bold",FontColor->RGBColor[.968627,.466667,0]], {Right, Center}, {Right, Center}], ImageSize->{180, 24}], "NewInGraphic"];
     
   UpdatedImgNoVersion[_] := UpdatedImgNoVersion["English"];
  
  (* TODO:FIXME: These localization graphics should be integrated better...  *)
  (* Adding NewImgNoVersion[lang] to display "NEW" instead of "NEW IN xxx" per sw *)
  (* andyh 2016-08-03 *)
  NewImgNoVersion["English"]:=
   Style[Graphics[Text[Style[Localized["BannerNewNoVersion", "English"],
     FontFamily->"Source Sans Pro",
     FontSize->18,FontSlant->"Italic",
     FontWeight->"Bold",FontColor->RGBColor[.968627,.466667,0]], {Right, Center}, {Right, Center}], ImageSize->{125, 24}], "NewInGraphic"];
     
  NewImgNoVersion["Japanese"]:=
   Style[Graphics[Text[Style[Localized["BannerNewNoVersion", "Japanese"],
     FontFamily->"Hiragino Kaku Gothic ProN",
     FontSize->18,
     FontWeight->"Bold",FontColor->RGBColor[.968627,.466667,0]], {Right, Center}, {Right, Center}], ImageSize->{180, 24}], "NewInGraphic"];
     
  NewImgNoVersion["ChineseSimplified"]:=
   Style[Graphics[Text[Style[Localized["BannerNewNoVersion", "ChineseSimplified"],
     FontFamily->"Microsoft YaHei",
     FontSize->18,
     FontWeight->"Bold",FontColor->RGBColor[.968627,.466667,0]], {Right, Center}, {Right, Center}], ImageSize->{180, 24}], "NewInGraphic"];
     
  NewImgNoVersion[lang_] := NewImgNoVersion["English"];
  
  uri = "URI"/.info;
  type = "EntityType" /. info;
  histData = "HistoryData" /. {info} /. Options[MakeNotebook];
  hist = {histData[[1,1,2]],histData[[1,2,2]], histData[[1,3,2]], histData[[1,4,2]]};
  (* get latest modified version *)
  If[!StringFreeQ[hist[[2]], ","],
    hist[[2]] = ReverseSortBy[StringTrim/@StringSplit[hist[[2]],","],ToExpression][[1]]];
  language = "Language" /. {opts} /. Options[MakeNotebook];
  langext = "Language" /. {opts} /. Options[MakeNotebook] /. {"English"->"", "Japanese"->"_jp", "ChineseSimplified"->"_zh",
  	"ChineseTraditional"->"", "Spanish"->"_es" };
  pos = Position[expr, Cell[_, "ObjectName"|"ObjectNameSmall"|"ObjectNameAlt"|"Title"|"MonographTitle"|"TechNoteTitle", ___]];
  translationboxes = If[MatchQ[type, "Symbol"] && StringMatchQ[uri, "ref/*"], 
  	With[{title = Last[StringSplit[uri, "/"]]}, ToBoxes[
    Style[With[{defaultLang = "English"}, DynamicModule[{result, translationlang},     
     Dynamic[translationlang = FrontEnd`CurrentValue[$FrontEnd, "Language", defaultLang]; With[{translationlang = translationlang}, FE`Evaluate[FEPrivate`If[
      FEPrivate`SameQ[FEPrivate`Set[result, FEPrivate`TranslateKeyword[title, translationlang]], $Failed], "", StringJoin["(", result, ")"]]]]]]], "ObjectNameTranslation"]
   ]], ""];

	(* load cloud-required, etc. page data *)
	(* defines which icons a page gets. empty list is supported everywhere, anything in the list means that one or more icons are different *)
	platforminfo = DocumentationBuild`Settings`FunctionPrereqs[uri];
	(* we don't care about URIsWithoutDesktopSupport right now
	   removing URIsWithServiceCredits for now, to work around 412782 *)
	platforminfo = DeleteCases[platforminfo, "URIsWithoutDesktopSupport" | "URIsWithServiceCredits"];

  icons = (* commenting out for 11.2 until icon-data is better *)
  (*Cell[BoxData[TagBox[RowBox[ToBoxes[Tooltip[With[{img=Image[ImageData[Image[Import[
   FileNameJoin[Join[imagespathlist,{"PlatformAvailability", #[[1]]}]]]]]/.{0.,0.,0.,a_}:>{1,1,1,a},Magnification->.5,ColorSpace->"RGB"]},
     PaneSelector[{False->SetAlphaChannel[img, 0.5], True->img}, Dynamic[CurrentValue["MouseOver"]]]],#[[2]], TooltipStyle->"CloudIconTooltipStyle"]]&/@
    {If[MemberQ[platforminfo, "URIsWithServiceCredits"],
      {"integratedServicesCoin@2.png", "This feature accesses external services that require Wolfram Service Credits."},
      Nothing],
     If[MemberQ[platforminfo, "URIsWithoutDesktopSupport"],
      {"desktopUnknown@2.png", Localized["CloudIconTooltipText", language]},
      {"desktopAvailable@2.png", "This feature is fully supported in Wolfram Desktop systems."}],
     (* removing mobile icons for now *) 
     (*If[MemberQ[platforminfo, "URIsWithoutCloudSupport"],
      {"mobileUnavailable@2.png", "This feature is not supported in the Wolfram Cloud mobile app."},
      {"mobileAvailable@2.png", "This feature is fully supported in the Wolfram Cloud mobile app."}],*)
     If[MemberQ[platforminfo, "URIsWithoutCloudSupport"],
      {"cloudUnavailable@2.png", "This feature is not supported in the Wolfram Cloud."},
      If[MemberQ[platforminfo, "URIsWithoutPublicCloudSupport"],
      {"cloudUnknown@2.png", "This feature is not supported in the Wolfram Cloud, but is supported in suitably configured Wolfram Enterprise Cloud systems."},
      {"cloudAvailable@2.png", "This feature is fully supported in the Wolfram Cloud."}]
      ]}], Deploy, DefaultBaseStyle->"Deploy"]], "PlatformIconGrid"]*)Nothing;

	Clear[c];

	Which[
		(
			(Length[hist] > 0)
			&& StringMatchQ[
				hist[[1]],
				Alternatives[
					RegularExpression["\\s*13\\.*[0-9]* *"],
					(* include futurized && UD pages here *)
					"??"
				]
			] &&
			(* exclude pages that would qualify for both new and updated tags *)
			!StringMatchQ[hist[[2]], RegularExpression["\\s*13\\.*[0-9]* *"]]
		), (
			rast = NewInImgGen[hist[[1]], language];

			unfindraftimg = Import[DocsAsset["Graphics", "RefUnfinishedDraft"<>langext<>".m"]];

			newObjectNameCell = Cases[
				expr,
				Cell[c_, b : "ObjectName" | "ObjectNameSmall" | "ObjectNameAlt" | "Title" | "TechNoteTitle" | "MonographTitle", ___] :> (
					If[MemberQ[expr, "UnderDevelopment", Infinity] || TrueQ["Underdevelopment" /. info],
						(*Cell[TextData[{*)
							Cell[c, b]
						(*, Cell[BoxData[ToBoxes[Spacer[10]]]], Cell[BoxData[ToBoxes[unfindraftimg]]]}]]*)
						,
						Cell[TextData[{
							Cell[c, b],
							Cell[BoxData[ToBoxes[Spacer[8]]]],
							Cell[BoxData[translationboxes], "ObjectNameTranslation"]
						}]]
					]
				),
				Infinity
			];

			newObjectNameCell = Replace[newObjectNameCell, {
				{} :> (
					DocsWarning["`` is missing page name cell.", uri];
					(* pos will be {} *)
					None
				),
				{first_, ___} :> first
			}];

			(* Formatting note: 057a33d5: fix bugs 389048 and another piece of 382084 as well *)
			c = Cell[
				BoxData[GridBox[{{
					newObjectNameCell,
					Cell[
						BoxData[TooltipBox[
							ToBoxes @ Style[Deploy[rast], "NewInGraphic"],
							Localized["New in 13.1", language],
							ActionDelay -> .3
						]],
						Magnification -> 1
					],
					icons
				}}]],
				"ObjectNameGrid",
				CacheGraphics -> False
			];
			pos = Position[
				expr,
				Cell[_, "ObjectName" | "ObjectNameSmall" | "ObjectNameAlt" | "Title" | "TechNoteTitle" | "MonographTitle", ___]
			];
		),
    ((Length@hist > 1) && StringMatchQ[hist[[2]], RegularExpression["\\s*13\\.*[0-9]* *"]]), (
      (* how many actual updates in this nb *)
      dyn = Plus[
        Length[Cases[Cases[nbExpr, Cell[__, "Usage", ___], Infinity],
          {TaggingRules, "ModificationHighlight"},
          Infinity, 1]],
        Length[Cases[nbExpr,
          Cell[__, "Notes"|"FormatNotes"|"Text"|"MonographText"|"TechNoteText"|"DefinitionBox", ___,
            Background->Dynamic[If[CurrentValue[{TaggingRules, "ModificationHighlight"}] === True, __]],
          ___], Infinity, 1]],
        Length[Cases[Cases[nbExpr, Cell[__, "1ColumnTableMod"|"2ColumnTableMod" | "3ColumnTableMod"|"3ColumnIllustratedTableMod", ___], Infinity],
          GridBox[{___, {Cell["\[FilledVerySmallSquare]", "TableRowIcon", ___], ___}, ___}, ___],
          Infinity, 1]]
      ];
      If[dyn > 0,
        updatedText = Localized["BannerUpdatedNoVersion", language];
        showChangesText = Localized["BannerUpdatedShowChanges", language];
        hideChangesText = Localized["BannerUpdatedHideChanges", language];
        (*updatedText = If[StringQ[updatedText], StringJoin[Riffle[Characters[updatedText],"\[VeryThinSpace]"]], "UPDATED"];*)
        rast = GridBox[{{StyleBox[updatedText, FontWeight->"Plain",FontFamily->"Source Sans Pro",FontTracking->"Wide",FontSize->20,FontSlant->"Italic",FontColor->RGBColor[.9,.6,.1]]},
          {StyleBox[showChangesText,FontWeight->"Plain",FontFamily->"Source Sans Pro",FontSize->14,FontColor->GrayLevel[.55]]}},
          GridBoxSpacings->{"Rows"->{{0.25}}},
          GridBoxAlignment->{"Rows" -> {Bottom, Top}, "Columns" -> {{Center}}},
          GridBoxItemSize->{"Rows" -> {{0.5}}}
          ];
        rastover = GridBox[{{StyleBox[updatedText, FontWeight->"Plain",FontFamily->"Source Sans Pro",FontTracking->"Wide",FontSize->20,FontSlant->"Italic",FontColor->RGBColor[.9,.6,.1]]},
          {StyleBox[showChangesText,FontWeight->"Plain",FontFamily->"Source Sans Pro",FontSize->14,FontColor->RGBColor[.9,.6,.1]]}},
          GridBoxSpacings->{"Rows"->{{0.25}}},
          GridBoxAlignment->{"Rows" -> {Bottom, Top}, "Columns" -> {{Center}}},
          GridBoxItemSize->{"Rows" -> {{0.5}}}];
        rasthide = GridBox[{{StyleBox[updatedText, FontWeight->"Plain",FontFamily->"Source Sans Pro",FontTracking->"Wide",FontSize->20,FontSlant->"Italic",FontColor->RGBColor[.9,.6,.1]]},
          {StyleBox[hideChangesText,FontWeight->"Plain",FontFamily->"Source Sans Pro",FontSize->14,FontColor->GrayLevel[.55]]}},
          GridBoxSpacings->{"Rows"->{{0.25}}},
          GridBoxAlignment->{"Rows" -> {Bottom, Top}, "Columns" -> {{Center}}},
          GridBoxItemSize->{"Rows" -> {{0.5}}}];
        rasthideover = GridBox[{{StyleBox[updatedText, FontWeight->"Plain",FontFamily->"Source Sans Pro",FontTracking->"Wide",FontSize->20,FontSlant->"Italic",FontColor->RGBColor[.9,.6,.1]]},
          {StyleBox[hideChangesText,FontWeight->"Plain",FontFamily->"Source Sans Pro",FontSize->14,FontColor->RGBColor[.9,.6,.1]]}},
          GridBoxSpacings->{"Rows"->{{0.25}}},
          GridBoxAlignment->{"Rows" -> {Bottom, Top}, "Columns" -> {{Center}}},
          GridBoxItemSize->{"Rows" -> {{0.5}}}];

        ce = Cell[BoxData[TagBox[TooltipBox[
            TogglerBox[Dynamic[
            FrontEnd`CurrentValue[
            FrontEnd`EvaluationNotebook[], {
              TaggingRules, "ModificationHighlight"}, False]], 
              { True-> PaneSelectorBox[{False->rasthide, True->rasthideover}, Dynamic[
    CurrentValue["MouseOver"]]], False->PaneSelectorBox[{False->rast, True->rastover}, Dynamic[
    CurrentValue["MouseOver"]]] }, "\<\"Click me\"\>"], 
                Localized["Toggle changes", language], ActionDelay -> .3], MouseAppearanceTag["LinkHand"]]]];
        unfindraftimg = Import[DocsAsset["Graphics", "RefUnfinishedDraft"<>langext<>".m"]];

        (* assemble ObjectName grid *)

        (* V11.1: changed how [[EXPERIMENTAL]] banner is displayed *)
        c = Cell[BoxData[GridBox[{
          {First[Cases[expr, Cell[c_, b : "ObjectName" | "ObjectNameSmall" | "ObjectNameAlt" | "Title" | "TechNoteTitle" | "MonographTitle", ___] :> 
           If[MemberQ[expr, "UnderDevelopment" | "ExperimentalObject", Infinity] || TrueQ["Underdevelopment"/.info],
        	 (*Cell[TextData[{*)Cell[c, b](*, Cell[BoxData[ToBoxes[Spacer[10]]]], Cell[BoxData[ToBoxes[unfindraftimg]]]}]]*),
        	 Cell[TextData[{Cell[c, b], Cell[BoxData[ToBoxes[Spacer[8]]]], Cell[BoxData[translationboxes], "ObjectNameTranslation"]}]]], Infinity]],
           ce, 
           icons}
          }]], "ObjectNameGrid"],
      (* Else *)
        rast = UpdatedInImgGen[hist[[2]],language];
        c = Cell[BoxData[GridBox[{
          {First[Cases[expr, Cell[c_, b : "ObjectName" | "ObjectNameSmall" | "ObjectNameAlt" | "Title" | "TechNoteTitle" | "MonographTitle", ___] :> 
            If[MemberQ[expr, "UnderDevelopment" | "ExperimentalObject", Infinity] || TrueQ["Underdevelopment"/.info],
        	 (*Cell[TextData[{*)Cell[c, b](*, Cell[BoxData[ToBoxes[Spacer[10]]]], Cell[BoxData[ToBoxes[unfindraftimg]]]}]]*),
        	 Cell[TextData[{Cell[c, b], Cell[BoxData[ToBoxes[Spacer[8]]]], Cell[BoxData[translationboxes], "ObjectNameTranslation"]}]]], Infinity]]  ,
           Cell[BoxData[ToBoxes@rast], Magnification->1, Deployed->True],
           icons}
          }]], "ObjectNameGrid", CacheGraphics->False]
      ];
	),
	(* Functions which were new in previous major version are marked "New" *)
	(* From: https://jira.wolfram.com/jira/browse/WLDOCS-636
		SW wants docs pages from the direct previous integer version of M- to say "New,"
		but newest/current version of M- to say "New in <version number>".
		For example, any new M9 docs pages will have "New," but M10 will say "New in 10," "New in 10.1,"
		"New in 10.2," etc. Then those will be refactored after the next integer release.
	*)
	(* andyh 2016-08-03 *)

	(*  --------------------------------------------------------------
	For display in version x.0
		a page new in (x-1).y /; y > 0 gets "NEW"
		a page new in x.0 gets "NEW IN x"
	For diplsay in version x.z /; z> 0
		a page new in x.0 gets "NEW in X"
		a page new in x.y /; y>0 gets "NEW IN x.y"
	-------------------------------------------------------------- *)
	((Length[hist] > 0) && StringMatchQ[hist[[1]], RegularExpression["\\s*12\\.*[1-9]+ *"]]), (
		rast = NewInImgGen[hist[[1]], language];
		rastNoVersion = NewImgNoVersion[language];
		unfindraftimg = Import[DocsAsset["Graphics", "RefUnfinishedDraft"<>langext<>".m"]];

		newObjectNameCell = Cases[
			expr,
			Cell[c_, b : "ObjectName" | "ObjectNameSmall" | "ObjectNameAlt" | "Title" | "TechNoteTitle" | "MonographTitle", ___] :> (
				If[MemberQ[expr, "UnderDevelopment" | "ExperimentalObject", Infinity] || TrueQ["Underdevelopment" /. info],
					(*Cell[TextData[{*)
						Cell[c, b]
					(*, Cell[BoxData[ToBoxes[Spacer[10]]]], Cell[BoxData[ToBoxes[unfindraftimg]]]}]]*)
					,
					Cell[TextData[{
						Cell[c, b],
						Cell[BoxData[ToBoxes[Spacer[8]]]],
						Cell[BoxData[translationboxes], "ObjectNameTranslation"]
					}]]
				]
			),
			Infinity
		];

		newObjectNameCell = Replace[newObjectNameCell, {
			{} :> (
				DocsWarning["`` is missing page name cell.", uri];
				(* pos will be {} *)
				None
			),
			{first_, ___} :> first
		}];

		(* Add "NEW" banner here... *)
		c = Cell[
			BoxData[GridBox[{{
				newObjectNameCell,
				Cell[
					BoxData[TooltipBox[
						ToBoxes @ Style[Deploy[rastNoVersion], "NewInGraphic"],
						Localized["BannerNewNoVersion", language],
						ActionDelay -> .3
					]],
					Magnification -> 1
				],
				icons
			}}]],
			"ObjectNameGrid",
			CacheGraphics -> False
		];
		pos = Position[
			expr,
			Cell[_, "ObjectName" | "ObjectNameSmall" | "ObjectNameAlt" | "Title" | "TechNoteTitle" | "MonographTitle", ___]
		];
	),
	
	(*
		Fall through
	*)
	True,
		LogIt["   Non match: "<>ToString[{hist, language}] ];
		unfindraftimg = Import[DocsAsset["Graphics", "RefUnfinishedDraft"<>langext<>".m"]];

		If[MatchQ[type, "Symbol"],
			c = Cell[
				BoxData[GridBox[{{
					First[Cases[
						expr,
						Cell[c_, b : "ObjectName" | "ObjectNameSmall" | "ObjectNameAlt" | "Title" | "TechNoteTitle" | "MonographTitle", ___] :> (
							If[MemberQ[expr, "UnderDevelopment" | "ExperimentalObject", Infinity] || TrueQ["Underdevelopment"/.info],
								(*Cell[TextData[{*)Cell[c, b](*, Cell[BoxData[ToBoxes[Spacer[10]]]], Cell[BoxData[ToBoxes[unfindraftimg]]]}]]*),
								Cell[
									TextData[{
										Cell[c, b],
										Cell[BoxData[ToBoxes[Spacer[8]]]],
										Cell[BoxData[translationboxes], "ObjectNameTranslation"]
									}]
								]
							]
						),
						Infinity
					]],
					"\[SpanFromLeft]",
					icons
				}}]],
				"ObjectNameGrid"
			];
		];
	];

	If[ValueQ[c] && Length[pos] > 0,
		expr = ReplacePart[
			expr,
			First[pos] -> AddPrereqTag[c, platforminfo, language]
		]
	];

	LogIt["** Leaving addObjectNameGraphic **"];

	expr
];


AddPrereqTag[c_Cell, platforminfo_, language_:"English"] := Module[{
	newcell = c, actionpoint, gb, new, tagtext
},
	If[Quiet[ListQ[platforminfo] && Length[platforminfo] > 0],
		tagtext = StyleBox[
			Localized["PrerequisiteTag" <> First[platforminfo], language],
			"PrerequisiteTag"
		];

		(* Add prereq tag *)
		If[MatchQ[c, Cell[BoxData[GridBox[{{Cell[___], __}}, ___]], ___, "ObjectNameGrid", ___]],
			actionpoint = c[[1, 1, 1, 1, -1]]; (* last item of single-row grid *)
			If[!FreeQ[actionpoint, StyleBox[__, "NewInGraphic", ___]],
				(* there's a badge here already *)
				If[MatchQ[actionpoint, Cell[BoxData[TooltipBox[StyleBox[TagBox[__, Deploy, ___], __], __]], ___]],
					new = Module[{newpoint = actionpoint[[1, 1, 1, 1, 1]] (* first arg of TagBox *)},
						actionpoint[[1, 1, 1, 1, 1]] = GridBox[{{newpoint}, {tagtext}}];
						actionpoint
					]
					,
					new = GridBox[{{actionpoint}, {tagtext}}]
				];
				newcell[[1, 1, 1, 1, -1]] = new
				,
				(* no badge; just need bare prereq tag *)
				newcell[[1, 1, 1, 1, -1]] = tagtext
			];
			newcell
			,
			c
		]
		,
		(* No prereq tag to add *)
		c
	]
]


(******************************************************************************)
(** AddGuideNewInGraphic - Internal **)

AddGuideNewInGraphic[nb_Notebook, info_, opts___?OptionQ]:=
  Module[{pos, expr=nb, lang, langext, rast, rasthide, funcs, lang, versionNumber},
  	versionNumber = StringSplit[$Version, "."][[1]];
    lang = "Language" /. {opts} /. Options[MakeNotebook];
    langext = If[lang === "English", "", "_jp"];
    funcs = Cases[expr, ButtonBox[__, BaseStyle->"NewInVersionLink", ___], Infinity]; 
    If[Length@funcs > 0, (
      rast = Import[DocsAsset["InternalOnlineImage", "UpdatedIn"<>versionNumber<>langext<>".gif"], "GIF"];
      rasthide = Import[DocsAsset["InternalOnlineImage", "UpdatedIn"<>versionNumber<>"-hide"<>langext<>".gif"], "GIF"];
      c = Cell[BoxData[GridBox[{
        {Cell[First@Cases[expr, Cell[c_String, "GuideTitle", ___]:>c, Infinity], "GuideTitle"],
          Cell[BoxData[TagBox[TooltipBox[
            TogglerBox[Dynamic[
              FrontEnd`CurrentValue[
                FrontEnd`EvaluationNotebook[], {
                  TaggingRules, "ModificationHighlight"}, False]],
              { True-> ToBoxes@rasthide, False->ToBoxes@rast },
              "\<\"Click me\"\>"],
            (* FIXME: use ReleaseVersion *)
            Localized["HistoryNewIn", lang][versionNumber], ActionDelay -> .3], MouseAppearanceTag["LinkHand"]]]]}
        }]], "GuideTitleGrid"];
      pos = Position[expr, Cell[_, "GuideTitle", ___]];
      expr = ReplacePart[expr, First[pos] -> c];
    )];
    expr
  ];

(*AddGuideNewInGraphic[nb_Notebook, info_, opts___?OptionQ]:=
  Module[{pos, expr=nb, lang, langext, rast, rasthide, funcs, lang},
    lang = "Language" /. {opts} /. Options[MakeNotebook];
    langext = If[lang === "English", "", "_jp"];
    funcs = Cases[expr, ButtonBox[__, BaseStyle->"NewInVersionLink", ___], Infinity]; 
    If[Length@funcs > 0, (
      rast = Import[DocsAsset["InternalOnlineImage", "UpdatedIn6"<>langext<>".gif"], "GIF"];
      rasthide = Import[DocsAsset["InternalOnlineImage", "UpdatedIn6-hide"<>langext<>".gif"], "GIF"];

      c = Cell[BoxData[GridBox[{
        {Cell[First@Cases[expr, Cell[c_String, "GuideTitle", ___]:>c, Infinity], "GuideTitle"],
          Cell[BoxData[TooltipBox[
            TogglerBox[Dynamic[
              FrontEnd`CurrentValue[
                FrontEnd`EvaluationNotebook[], {
                  TaggingRules, "ModificationHighlight"}, False]],
              { True-> ToBoxes@rasthide, False->ToBoxes@rast },
              "\<\"Click me\"\>"],
            (* FIXME: use ReleaseVersion *)
            Localized["HistoryNewIn", lang]["6"], ActionDelay -> .3]]]}
        }]], "GuideTitleGrid"];
      pos = Position[expr, Cell[_, "GuideTitle", ___]];
      expr = ReplacePart[expr, First[pos] -> c];
    )];
    expr
  ];*)


(******************************************************************************)
(** addGuideTOCRightGuillemet - Internal **)
addGuideTOCRightGuillemet[nbExpr_]:=
Module[{expr = nbExpr, pos, cexpr},
      (* GuideTOCLinks need a RightGuillemet *)
      pos = Position[expr, Cell[TextData[ButtonBox[_, ___]], "GuideTOCLink", ___], Infinity ];
      Scan[(
          cexpr = Extract[expr, #];
          If[ !StringMatchQ[ ConvertToString[cexpr], "*\[RightGuillemet]"],
          cexpr = cexpr /.
              Cell[TextData[ButtonBox[c_, o1___]], "GuideTOCLink", o2___] :>
              Cell[TextData[ButtonBox[
                  If[Head@c === String,
                     c <> "\[NonBreakingSpace]\[RightGuillemet]",
                     AppendTo[c, "\[NonBreakingSpace]\[RightGuillemet]"] 
                     ],
                  o1]], "GuideTOCLink", o2];
          expr = ReplacePart[expr, # -> cexpr]
        ]
      )&,
      pos];
  expr
];


(******************************************************************************)
(** appendString - Internal **)
appendString[str_String, text_String:" \[RightGuillemet]"]:=
  str <> text;
  

(******************************************************************************)
(** ConvertValuesForSearch - Internal **)

ConvertValuesForSearch[lst_List, lang_String:"English"] := 
  Flatten[ConvertSearchText[{#}, lang] & /@ lst, 1];

ConvertSearchText[a___]:= (Message[ConvertSearchText::arg, a]; $Failed)
ConvertSearchText[{name_ -> value_String}, lang_String:"English"] :=
  {name -> Switch[lang, (*"Japanese", value, "ChineseSimplified", value,*) _, value] };

ConvertSearchText[{name_ -> value_}, lang_String:"English"] := {name -> value };
(*  *)
ConvertSearchText[{"status" -> None}, ___] := {"status" -> "" };

ConvertSearchText::arg = "List of rules expected as input but value was:`1`";










(**)


(* 
    ExportMarketingWebPage[outfile, expr, pageType, opts] 
    this is the main Transmogrify[] wrapper.
    NOTE - outfile = None just returns the raw Transmogrify output
           Test->True causes ExportMarketingWebPage to use TransmogrifyString[]
           the shorthand functions rawmogtest[] and mogtest[], respectively,
           are wrappers for these two functionalities.
*)
ExportMarketingWebPage[_,$Failed,___] = $Failed;

ExportMarketingWebPage[file_String, nb_String, language_String:"English", opts___?OptionQ]:= 
   ExportMarketingWebPage[file, DocumentationBuild`Utils`GetQuietly[nb], language, opts ];
  
ExportMarketingWebPage[file_String, nbExpr_Notebook, language_String:"English", opts___?OptionQ]:=
Developer`UseFrontEnd @
Module[{ext, properties={}, outfile=file, transform, keywordList={}, titleName, res, nbObj, nbfile, nbobj, nbobj2, nbget, tmp, parentDir, nbFile, localcss, 
  layoutfile, popupLayoutfile, urlBaseRemote, expr, infilename, filename, jatitle, description, ExamplesRawMaterialDir, WhatsNewID, MarketingCategoryName, additionalstyles, defaultstylesheet},

  LogIt["** Entering ExportMarketingPage **"];
  Global`AntLog["** Entering ExportMarketingPage **"];

  UseDocumentationBuildXMLTransforms[];

  urlBaseRemote = "http://reference.wolfram.com/language/";
  ext = Switch[language, "Japanese", "ja", "Russian", "ru", "Korean", "ko", "French", "fr", "German", "de", "Portuguese", "pt-br", "Spanish", "es","ChineseSimplified", "zh", _, "en"];
  
  Preview = "Preview" /. {opts};
  ExamplesRawMaterialDir = "ExamplesRawMaterialDir" /. {opts};
  MarketingCategoryName = "CategoryName" /. {opts};
  WhatsNewID = "WhatsNewID" /. {opts};
  infilename = "InFileName" /. {opts};

  transform = {"HTML","MarketingPages.m"};
  layoutfile = DocsAsset["HTMLTransformLayout", "MarketingPages.html"];
  popupLayoutfile = DocsAsset["HTMLTransformLayout", "MarketingPagePopup.html"];

  (* keywords *)
  description = DocumentationBuild`Info`GetNotebookDescription[nbExpr];
  keywordList = DocumentationBuild`Info`GetNotebookKeywords[nbExpr];
  synonymList = DocumentationBuild`Info`GetNotebookSynonyms[nbExpr];
  LogIt["  Keywords: "<>ToString[keywordList]];

  titleName = Switch[language, "Japanese", 
  	(* 
  	jatitle = Cases[nbExpr, Cell[c_, "MarketingTitle", ___,Rule[CellLabel,d_],___] :> d, Infinity];
    jatitle = DeleteCases[jatitle, StyleBox[_, "FullListing"], Infinity];
    jatitle = Transmogrify`ConvertToString[First[jatitle]];
    jatitle = If[StringFreeQ[jatitle, "\n"], jatitle, First@StringSplit[jatitle, "\n"]];
    jatitle = StringReplace[jatitle, {
        """ | """ | "'" | "'" -> "'",
        "\[FilledRightTriangle]" -> ">",
        "-" -> "-",
        "\"" -> ""
    }];
    jatitle
    *)
    DocumentationBuild`Info`GetNotebookTitle[nbExpr, "MarketingPage"],
    "ChineseSimplified",
  	DocumentationBuild`Info`GetNotebookTitle[nbExpr, "MarketingPage"],
  	    "Spanish",
  	DocumentationBuild`Info`GetNotebookTitle[nbExpr, "MarketingPage"],
  	  	    "Russian",
  	DocumentationBuild`Info`GetNotebookTitle[nbExpr, "MarketingPage"],
  	  	    "Korean",
  	DocumentationBuild`Info`GetNotebookTitle[nbExpr, "MarketingPage"],
  	  	  	    "German",
  	DocumentationBuild`Info`GetNotebookTitle[nbExpr, "MarketingPage"],
  	  	  	    "French",
  	DocumentationBuild`Info`GetNotebookTitle[nbExpr, "MarketingPage"],
  	  	    "Portuguese",
  	DocumentationBuild`Info`GetNotebookTitle[nbExpr, "MarketingPage"],
    _,
  	DocumentationBuild`Info`GetNotebookTitle[nbExpr, "MarketingPage"]];

  LogIt["  Title name: "<>ToString[titleName]];
  outfile = 
    If[ StringQ[ext],
      StringReplace[outfile, RegularExpression["(?<!\\.\\w\\w)(\\.html)"]->"$1"<>"."<>ext],
      outfile];
  localcss = StringReplace[DirectoryName@outfile, {DirectoryName@DirectoryName@outfile->"", "\""->"", "/"->"", $PathnameSeparator -> ""}] <> "-localcss";

  (* Create preprocess notebook *)
  LogIt["  MakeMarketingPageNotebook"];
  Global`AntLog["** MakeMarketingPageNotebook **"];
  
  expr = MakeMarketingPageNotebook[nbExpr, language, opts];

  SetOptions[$FrontEnd, StyleSheetPath -> Prepend[CurrentValue[$FrontEnd, StyleSheetPath], DocumentationBuild`$MarketingStylePath]];

  SetOptions[$FrontEndSession, Rule[AutoStyleOptions, {Rule["HighlightUndefinedSymbols", False]}]];
    
    additionalstyles =
             {Cell[StyleData["InlineFormula"], FontFamily->"Source Sans Pro", FontSize->Inherited 1.08],
             Cell[StyleData["MR"], FontFamily->"Source Sans Pro"],
             Cell[StyleData["StandardForm"],FontFamily->"Source Sans Pro"],
             Cell[StyleData["InputForm"],FontFamily->"Source Sans Pro"],
             Cell[StyleData["Input"],  FontFamily->"Source Sans Pro", FontSize->13, LineSpacing->{1.2, 0}],
             Cell[StyleData["Output"], FontSize->13]
             };
             
    defaultstylesheet= Cell[StyleData[StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "MarketingPageStyles.nb"]]];
    
    expr = expr /. 
    {Rule[StyleDefinitions, FrontEnd`FileName[___]]:> Rule[StyleDefinitions, Notebook[Flatten[{defaultstylesheet, additionalstyles}], Rule[AutoStyleOptions,{Rule["HighlightUndefinedSymbols",False]}]]],
     Rule[StyleDefinitions, Notebook[{contents___}, options___]]:> Rule[StyleDefinitions, Notebook[Flatten[{contents, additionalstyles}], options, Rule[AutoStyleOptions,{Rule["HighlightUndefinedSymbols",False]}]]]
    };

    expr = expr /. {Notebook[{contents___}, options___]:> Notebook[{contents}, options, CellLabelAutoDelete->False]};

  Global`AntLog["** About to run a series of FrontEnd commands that apply the various stylesheet options/rules **"];
  Global`AntLog["** About to run NotebookPut **"];
  nbobj2 = NotebookPut[expr];
  Global`AntLog["** Finished running NotebookPut **"];
  Global`AntLog["** About to run NotebookGet **"];
  expr  =  NotebookGet[nbobj2];
  Global`AntLog["** Finished running NotebookGet **"];
  Global`AntLog["** About to run NotebookClose **"];
           NotebookClose[nbobj2];
  Global`AntLog["** Finished running NotebookClose **"];
  Global`AntLog["** Finished running a series of FrontEnd commands that apply the various stylesheet options/rules **"];
 
  nbFile = StringReplace[outfile, ".html"->".nb"];
  CreateDirectory[ DirectoryName@nbFile];

(*   Put[ CurrentValue[$FrontEnd, StyleSheetPath], nbFile];
   Put[Rasterize[nbget], nbFile<>"1.txt"];
   Put[Rasterize[expr], nbFile<>"2.txt"]; *)

  If[DocumentationBuild`Transform`debug === True,
    Put[ expr, nbFile];
  ];

  (* now call Transmogrify with the appropriate parameters set.
     note that language extension by default goes before the .html *)
  LogIt["  Off to transmogrify"];
  Global`AntLog["** Off to Transmogrify **"];
  
  res = Developer`UseFrontEnd @ Transmogrify`Transmogrify[ outfile, expr, transform, 
      Transmogrify`DefaultParameters -> {
      "layoutfile"->layoutfile, 
      "popupLayoutfile"->popupLayoutfile, 
        "la" -> DocumentationBuild`Utils`LanguageAbbreviation[language],
        "languageAbbreviations" -> DocumentationBuild`Utils`languageList[[All, 2]],
        "IsTMJQ" -> False,
        "IsSolutionsQ" -> False,
        "htmlBase" -> urlBaseRemote,
        "pacletName" -> "",
        "CompleteHTMLQ" -> False,
        "url" -> "",
      	"titleName"->titleName, 
      	"Language"->language, "la" -> DocumentationBuild`Utils`LanguageAbbreviation[language], "urlBaseRemote"->urlBaseRemote, "InFileName"->infilename, "WhatsNewID"->WhatsNewID, "MarketingCategoryName"->MarketingCategoryName, "ExamplesRawMaterialDir"->ExamplesRawMaterialDir, "Preview"->Preview, "description"->description, "keywords"->keywordList, "synonyms"->synonymList, "localcss"->localcss, "filename"->outfile}
    ];
    If[Head@res =!= String, First@res, res]

];

Options[ExportMarketingWebPage] = {
  DefaultParameters->{}, Keywords->{}, LogFile->None, Test->False};

ExportMarketingCategoryWebPages[_,$Failed,___] = $Failed;

ExportMarketingCategoryWebPages[outputDir_String, nb_String, language_String:"English", marketingpagedir_, opts___?OptionQ]:= 
  ExportMarketingCategoryWebPages[outputDir, DocumentationBuild`Utils`GetQuietly[nb], language, marketingpagedir, opts ];
ExportMarketingCategoryWebPages[outputDir_String, nbExpr_Notebook, language_String:"English", marketingpagedir_, opts___?OptionQ]:=
Developer`UseFrontEnd @
Module[{ext, properties={}, transform, keywordList={}, titleName, res, nbObj, tmp, parentDir, nbFile, localcss, 
  layoutfile, popupLayoutfile, expr, filename, jatitle, marketingpagefolder=marketingpagedir},

  LogIt["** Entering ExportMarketingCategoryWebPages **"];

  UseDocumentationBuildXMLTransforms[];

  ext = Switch[language, "Japanese", "ja", "Russian", "ru", "Korean", "ko", "German", "de", "French", "fr", "Portuguese", "pt-br", "Spanish", "es", "ChineseSimplified", "zh", _, "en"];

  Preview = "Preview" /. {opts};

  transform = {"HTML","MarketingCategoryPages.m"};
  layoutfile = DocsAsset["HTMLTransformLayout", "MarketingPages.html"];

  res = Developer`UseFrontEnd @ Transmogrify`Transmogrify[ nbExpr, transform, 
      Transmogrify`DefaultParameters -> {
      	"outputDir" -> outputDir,
      	"layoutfile"->layoutfile, 
      	"marketingpagefolder"-> marketingpagefolder,
      	"Language"->language}
    ];
    If[Head@res =!= String, First@res, res]

];

Options[ExportMarketingCategoryWebPages] = {
  DefaultParameters->{}, Keywords->{}, LogFile->None, Test->False};

(* Create Marketing page notebook *)
MakeMarketingPageNotebook[nb_String, language_String:"English", opts___?OptionQ ]:= 
  MakeMarketingPageNotebook[DocumentationBuild`Utils`GetQuietly[nb], language, opts];
MakeMarketingPageNotebook[nbExpr_Notebook, language_String:"English", opts___?OptionQ ]:= 
Module[{transform, expr=nbExpr, pageTitle = "None", keywordList = {}, seeAlsoGroup, poss, exampleCaption },

  (* Change grouped styles to aid in conversion *)
  expr = expr /. {Cell[CellGroupData[{ Cell[a_, "Delimiter", o___], Cell[b_, "MarketingText", op___], cells___ }, oc___]] :>
    Cell[CellGroupData[ Flatten[{ 
      Cell[a, "HighlightSection", o, CellGroupingRules->{"SectionGrouping", 61}], 
      Cell[b, "HighlightItem", op],
      textStyleToItem[{cells}] 
    }], oc]] };

  (* Change '>>' to \[RightGuillemet] *)
  expr = expr /. ButtonBox[">>", r___] :> ButtonBox["\[RightGuillemet]", r];
expr = expr /. Cell[a___, Rule[CellFrameLabels, {{cf1_, cf2_}, {cf3_, Cell[___, "TextAnnotation",___]}}], b___] :> Cell[a, b];
  If[("Preview"/.{opts})===True,
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[___, "FutureExample"(* | "ExcludedMaterial"*), ___], ___}, ___], ___], Infinity];
  expr = DeleteCases[expr, Cell[___, "FutureExample" | "ExcludedMaterial", ___], Infinity];
  expr = DeleteCases[expr, Cell[___, CellDingbat->Cell[__, "BrightFlag", ___], ___], Infinity];
  expr = expr /. Cell[CellGroupData[{Cell[aa___, bb_?"FutureExample" | "TentativeExample" | "ExcludedMaterial", cc___], dd___}, ee___], ff___] :> Cell[CellGroupData[{Cell[aa, cc], dd}, ee], ff];
  expr = expr /. Cell[aa___, bb_?"FutureExample" | "TentativeExample" | "ExcludedMaterial", cc___] :> Cell[aa, cc];
  expr = expr /. Cell[aa___, Rule[CellDingbat,Cell[cc___, "TentativeID" | "BrightFlag", dd___]], ee___]:> Cell[aa, ee]
  ,
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[___, "FutureExample" | "TentativeExample"(* | "ExcludedMaterial"*), ___], ___}, ___], ___], Infinity];
  expr = DeleteCases[expr, Cell[___, "FutureExample" | "TentativeExample" | "ExcludedMaterial", ___], Infinity];
  expr = DeleteCases[expr, Cell[___, CellDingbat->Cell[__, "TentativeID" | "BrightFlag", ___], ___], Infinity]
  ];
  
    expr = DeleteCases[expr, Cell[___, "History", ___], Infinity];
  (* Add in styles used for grouping *)
  expr = expr /.
   Cell[CellGroupData[{Cell[c_, "MarketingTitle", o___], p___}, q___], r___] :> 
   Cell[CellGroupData[{
     Cell[c, "MarketingTitle", o],
     Cell["SUBHEADER", "MarketingTitleSubheader", CellGroupingRules->{"SectionGrouping", 60}],
     Cell[c, "MarketingTitleAlt", o, CellGroupingRules->{"SectionGrouping", 60}],
     Cell[CellGroupData[{Cell["", "MarketingMainSection", CellGroupingRules->{"SectionGrouping", 40}],
     Cell[CellGroupData[{p}, q], r]}]]}]];
  expr = expr /. Cell[c_, "MarketingText", o___] :> 
   Cell[c, "MarketingText", o, CellGroupingRules->{"SectionGrouping", 60}];
     expr = expr /. Cell[c_, "MarketingExampleDetail", o___] :> 
   Cell[c, "MarketingExampleDetail", o, CellGroupingRules->{"SectionGrouping", 60}];

  (*  Make callouts *)
  If[Length@Cases[expr, Cell[_, "Output", ___], Infinity] :> 0,
    exampleCaption = Cases[expr, Cell[_, "MarketingExampleCaption", ___], Infinity];
    expr = DeleteCases[expr, Cell[_, "MarketingExampleCaption", ___], Infinity];
    poss = First @ Position[expr, Cell[_, "Output", ___], Infinity];
    expr = ReplacePart[expr, { poss -> Cell[CellGroupData[Flatten@{ Extract[expr, poss], exampleCaption}]] }];

    expr = expr /. Cell[c_, "MarketingExampleCaption", o___] :> 
     Cell[c, "calloutbox", o, CellGroupingRules->"OutputGrouping", FontColor->None, FontSize->1, CellMargins->{{0,0},{-2,-2}}];
    expr = expr /. Cell[c_, "MarketingExampleDetail", o___] :> 
     Cell[c, "calloutbox", o, CellGroupingRules->"OutputGrouping", FontColor->None, FontSize->1, CellMargins->{{0,0},{-2,-2}}];
  ];

    expr = expr /. Cell[con_, sty___,"AlternateThumbnail", o___] :> Cell[con, "AlternateThumbnail", o];
  

  expr = expr /. Cell[BoxData[Cell[a__, "Graphics", b__]], _, c__] :>
                   Cell[BoxData[Cell[a, NotebookDefault, b]], "Output", c];
  
  (* Change Input cells *)
  expr = expr /. Cell[con_, "Input", o___] /; DocumentationBuild`Utils`IsManipulateQ @ Cell[con, "Input", o] :> Cell[con, "Output", o]; 
  expr = expr /. Cell[con_, "Input", "PopupMaterial", o___] :> Cell[con, "InputPopupMaterial", o];
  expr = expr /. Cell[con_, "Input", "MainPageMaterial", o___] :> Cell[con, "InputMainPageMaterial", o];
  expr = expr /. Cell[con_, "Input", "MainPopupMaterial", o___] :> Cell[con, "InputMainPopupMaterial", o];

  expr = expr /. Cell[con_, "Input", "HiddenMaterial", o___] :> Cell[con, "InputHiddenMaterial", o];
  expr = expr /. Cell[con_, "Input", "SuppressInputPopupText", o___] :> Cell[con, "InputNoClipboardAction", o];
  expr = expr /. Cell[con_, "Program", "HiddenMaterial", o___] :> Cell[con, "ProgramHiddenMaterial", o];
  expr = DeleteCases[expr, Cell[con_, "Input", "SuppressedMaterial", o___], Infinity];
  expr = DeleteCases[expr, Cell[con___, "SuppressedMaterial", o___], Infinity];
  expr = expr /. Cell[con_, "Input", "ThumbnailMaterial", o___] :> Cell[con, "InputThumbnailMaterial", o];
  
  (* TODO: Fix replacement rule to catch multiple styles *)
  (* Catch other style variations *)
  expr = expr /. Cell[con_, "Input", "ConversionSpecial", o___] :> Cell[con, "OutputSpecial", o, CellTags->"NoFlash"];
  expr = expr /. Cell[con_ /; DocumentationBuild`Utils`IsManipulateQ@con, _, "ConversionSpecial", o___] :> 
    Cell[con, "OutputSpecial", o, CellTags->"NoFlash"];
  
(*  expr = expr /. Cell[con_, "Output", "ConversionSpecial", o___] :> Cell[con, "OutputSpecial", o, CellTags->"NoFlash"];*)

  expr = expr /. Cell[con_, "Input", sty_String, o___] :> Cell[con, "Input", o];

  expr = expr /. Cell[con_, sty_String, "TR", o___] :> Cell[con, sty, o, FontFamily->"Times"];
  expr = expr /. Cell[con_, sty_String, "TI", o___] :> Cell[con, sty, o, FontFamily->"Times", FontSlant->"Italic"];

  expr = expr /. Cell[con_, "Message",  "MSG", o___] :> Cell[con, "Message", o];
  
  expr = expr /. Cell[con_, sty_String, "AltThumbnail", o___] :> Cell[con, sty, o, CellTags->"AltThumbnail"];
    expr = expr /. Cell[con_, sty___,"AlternateThumbnail", o___] :> Cell[con, "AlternateThumbnail", o];
  
  expr = expr /. Cell[con_, "AltThumbnail", o___] :> Cell[con, "AltThumbnail", o, CellTags->"AltThumbnail"];

  expr = expr /. Cell[con_, "NotebookImage", "ThumbnailMaterial", o___] :> Cell[con, "NotebookImage", o];

  expr = expr /. Cell[con_, sty1_String,  sty2_String, o___] :> Cell[con, "Output", o]; 

  expr = expr /. Cell[con_, "Graphics"|"Picture"|"Output", "ThumbnailMaterial", o___] :> Cell[con, "Output", o];


  (* Remove XXX* cells *)
  expr = expr /. Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], "MarketingText", d___] :> Cell["xxx", "MarketingText", d];
  expr = expr /. Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], "HighlightItem", d___] :> Cell["xxx", "HighlightItem", d];
  
  expr = DeleteCases[expr, Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], ___], Infinity];
  expr = DeleteCases[expr, Cell[BoxData[c_String /; StringMatchQ[c, RegularExpression["XX*"]]], ___], Infinity];

  (* convert Function string to buttons *)
  expr = expr /. Cell[c_String, "MarketingRelatedFunctions", o___] :> 
    DocumentationBuild`Make`Private`inlineFunctionStringToButtonBox @ Cell[c, "MarketingRelatedFunctions", o];

  (* Remove XXX* cells *)
  expr = DeleteCases[expr, Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], ___], Infinity];
  expr = DeleteCases[expr, 
    Cell[_  sty_String /; StringMatchQ[sty, "DocumentStatusSection" | "DocumentStatus"], ___], Infinity];

  (* Remove empty cell groups *)
  expr = RemoveEmptyCellGroups[expr, {
           "KeywordsSection", "DescriptionSection", "MarketingContentsSection", "MarketingMoreAboutSection", "MarketingRelatedDemonstrationsSection", 
           "MarketingRelatedFunctionsSection", "MarketingSeeAlsoSection", "MarketingSubpageMoreAboutSection", 
           "MarketingSubpageRelatedDemonstrationsSection", "MarketingSubpageRelatedFunctionsSection", "MarketingSubpageSection",
           "MarketingSubpageSeeAlsoSection", "MarketingThumnailSection", "Keywords", "Description", "MarketingRelatedLinksSection", "DocumentStatusSection", "DocumentStatus"
           }];
           
  expr = expr /.
   Cell[CellGroupData[{Cell[c_, "MarketingSeeAlsoSection", o___], p___}, q___], r___] :> 
   Cell[CellGroupData[{
     Cell[c, "MarketingSeeAlsoSection", o],
     Cell["", "MarketingUnorderedList", CellGroupingRules->{"SectionGrouping", 60}],
     p
   }, q], r];
   
   expr = expr /.
   Cell[CellGroupData[{Cell[c_, "MarketingMoreAboutSection", o___], p___}, q___], r___] :> 
   Cell[CellGroupData[{
     Cell[c, "MarketingMoreAboutSection", o],
     Cell["", "MarketingUnorderedList", CellGroupingRules->{"SectionGrouping", 60}],
     p
   }, q], r];
   
   expr = expr /.
   Cell[CellGroupData[{Cell[c_, "MarketingRelatedDemonstrationsSection", o___], p___}, q___], r___] :> 
   Cell[CellGroupData[{
     Cell[c, "MarketingRelatedDemonstrationsSection", o, CellGroupingRules->{"SectionGrouping", 50}],
     Cell["", "MarketingUnorderedList", CellGroupingRules->{"SectionGrouping", 60}],
     p
   }, q], r];
   
expr = expr /.
   Cell[CellGroupData[{Cell[c_, "MarketingRelatedLinksSection", o___], p___}, q___], r___] :> 
   Cell[CellGroupData[{
     Cell[c, "MarketingRelatedLinksSection", o, CellGroupingRules->{"SectionGrouping", 50}],
     Cell["", "MarketingUnorderedList", CellGroupingRules->{"SectionGrouping", 60}],
     p
   }, q], r];
  (* Change See Also title per sw *)
  expr = expr /. Cell[_, "MarketingSeeAlsoSection", o___] :> Cell[Localized["MarketingSeeAlsoSection", language], "MarketingSeeAlsoSection", o];
  expr = expr /. Cell[_, "MarketingMoreAboutSection", o___] :> Cell[Localized["MarketingMoreAboutSection", language], "MarketingMoreAboutSection", o];
  expr = expr /. Cell[_, "MarketingRelatedFunctionsSection", o___] :> Cell[Localized["MarketingRelatedFunctionsSection", language], "MarketingRelatedFunctionsSection", o];
  expr = expr /. Cell[_, "MarketingRelatedLinksSection", o___] :> Cell[Localized["MarketingRelatedLinksSection", language], "MarketingRelatedLinksSection", o];
  expr = expr /. Cell[_, "MarketingRelatedDemonstrationsSection", o___] :> Cell[Localized["MarketingRelatedDemonstrationsSection", language], "MarketingRelatedDemonstrationsSection", o];
  (* Move See Also group to bottom per sw *)
  
  seeAlsoGroup = Cases[expr, Cell[CellGroupData[{Cell[_, "MarketingSeeAlsoSection", ___], ___},___],___], Infinity];
  moreAboutGroup = Cases[expr, Cell[CellGroupData[{Cell[_, "MarketingMoreAboutSection", ___], ___},___],___], Infinity];
  relatedFunctionsGroup=Cases[expr, Cell[CellGroupData[{Cell[_, "MarketingRelatedFunctionsSection", ___], ___},___],___], Infinity];
  relatedLinksGroup=Cases[expr, Cell[CellGroupData[{Cell[_, "MarketingRelatedLinksSection", ___], ___},___],___], Infinity];
  relatedDemonstrationsGroup=Cases[expr, Cell[CellGroupData[{Cell[_, "MarketingRelatedDemonstrationsSection", ___], ___},___],___], Infinity];
  
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "MarketingSeeAlsoSection", ___], ___},___],___], Infinity];
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "MarketingMoreAboutSection", ___], ___},___],___], Infinity];
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "MarketingRelatedFunctionsSection", ___], ___},___],___], Infinity];
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "MarketingRelatedLinksSection", ___], ___},___],___], Infinity];
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[_, "MarketingRelatedDemonstrationsSection", ___], ___},___],___], Infinity];
  
  expr[[1]] = Flatten[{First[expr], relatedFunctionsGroup, moreAboutGroup, relatedDemonstrationsGroup, relatedLinksGroup, seeAlsoGroup}];

  (* Change paclets to URLs *)

  expr = expr /. ButtonBox[c__, BaseStyle->_, o1___, ButtonData->str_String /; StringMatchQ[str, "paclet*"], o2___] :> 
    ButtonBox[c, BaseStyle->"Hyperlink", ButtonData->{URL[Which[Length[StringSplit[str, "#"]] === 2, 
 "http://reference.wolfram.com/language/" <> 
  StringReplace[StringSplit[str, "#"][[1]], "paclet:" -> ""] <> 
  ".html#" <> 
  StringReplace[StringSplit[str, "#"][[2]], "paclet:" -> ""], True, 
 "http://reference.wolfram.com/language/" <> 
  StringReplace[str, "paclet:" -> ""] <> ".html"]], None}, BaseStyle->"Hyperlink", o1, o2];

  expr = expr /. Cell[BoxData[ButtonBox[c_, BaseStyle->"Link", ___]], "InlineFormula", ___] :> 
    Cell[BoxData[ButtonBox[c, ButtonData->{URL["http://reference.wolfram.com/language/ref/"<> StringReplace[c, {"\""->""}] <>".html"], None}, BaseStyle->"Hyperlink"]], "InlineFormula"];

  (* manipulate some button boxen *)
   expr = ReplacePart[expr, (# -> DocumentationBuild`Utils`MakePacletButton[Extract[expr, #]]) & /@ Position[expr, ButtonBox[_String, ___]]];

  expr = expr /. Cell[con_String, "MarketingSeeAlso", o___] :> makeMarketingSeeAlsoButton@Cell[con, "MarketingSeeAlso", o];
  expr = expr /. Cell[con_String, "MarketingMoreAbout", o___] :> makeMarketingMoreAboutButton@Cell[con, "MarketingMoreAbout", o];
  expr = expr /. Cell[con_String, "MarketingRelatedFunctions", o___] :> makeMarketingRelatedFunctionsButton@Cell[con, "MarketingRelatedFunctions", o];
(*
  expr = expr //. { 
    Cell[TextData[{a___, s_String, b___}], 
      sty_String/; StringMatchQ[sty, "MarketingRelatedFunctions"], o___] :> 
    Cell[TextData[Flatten@{a, makeMarketingRelatedFunctionsButton[s], b}
      ], sty, o] };
*)  
  (* change \[EmptyVerySmallSquare] to "something else" *)
  expr = expr //. Cell[TextData[{a___, s_String /; StringMatchQ[s, RegularExpression["\\s+\[EmptyVerySmallSquare]\\s+"] ], b___}], 
    "MarketingRelatedFunctions", o___] :> 
    Cell[TextData[{a, " ", b}], "MarketingRelatedFunctions", o];

  expr = expr /. Cell[BoxData[con_], "MarketingText", o___] :> Cell[BoxData[con], "Output", o];

  (* Renumber In/Out Cell labels *)
  expr = RenumberCells[expr, {"MarketingSubpageSection","ExampleDelimiter"}, 
           InputStyles->{"InputNoClipboardAction", "InputHiddenMaterial", "ExternalLanguage", "InputSuppressedMaterial", "InputThumbnailMaterial", "Input"}];

  expr = DeleteCases[expr, Rule[CellID, ___], Infinity];

  (* Change WindowSize to a width for nicer line wrapping in gifs *)
  expr = DocumentationBuild`Utils`Private`ResetOptions[expr, 
           WindowSize -> {700, 700}, ShowCellBracket->True, (*PageWidth->WindowWidth,*)
           StyleDefinitions -> 
             Notebook[{Cell[ StyleData[ StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "MarketingPageStyles.nb"]]](*, 
               Cell[StyleData["Input"], FontSize -> 11, CellID -> 1,PageWidth->525 ], 
               Cell[StyleData["Output"], FontSize -> 11, CellID -> 1], 
               Cell[StyleData["Graphics"], CellID -> 1, PageWidth->525 ], 
               Cell[StyleData["Picture"], CellID -> 1, PageWidth->525 ], 
               Cell[StyleData["InputHiddenMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputNoClipboardAction", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputSuppressedMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputThumbnailMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1]*)
               }]];
  expr
];

(* Create Solutions page notebook *)
MakeSolutionsPageNotebook[nb_String, language_String:"English", opts___?OptionQ ]:= 
  MakeSolutionsPageNotebook[DocumentationBuild`Utils`GetQuietly[nb], language, opts];
MakeSolutionsPageNotebook[nbExpr_Notebook, language_String:"English", opts___?OptionQ ]:= 
Module[{transform, expr=nbExpr, pageTitle = "None", keywordList = {}, seeAlsoGroup, poss, exampleCaption },

  
    expr = DeleteCases[expr, Cell[___, "History", ___], Infinity];
    expr = DeleteCases[expr, ShowCellTags->True, Infinity];
    expr = DeleteCases[expr, Cell[__, "InternalNoteLink", ___],Infinity];
   	
   	expr = expr /.Cell[TextData[ButtonBox[a_String,
 	BaseStyle->"Hyperlink",
 	ButtonData->{URL[b_String/;
 		StringJoin[
 			Insert[ Riffle[Drop[Drop[StringSplit[b, "/"], 1], -1], "/"],  "http:/" , 1 ]
 		]==="http://reference.wolfram.com/language/ref"], None}, c__]], "SolutionsListing", d__]:>
 	Cell[TextData[StyleBox[ButtonBox[a, BaseStyle->"Hyperlink", ButtonData->{
   	URL[b], None}, c], "InlineFormulaListItem"]], "SolutionsListing", d];


  (* Remove XXX* cells *)
  expr = DeleteCases[expr, Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], ___], Infinity];
  expr = DeleteCases[expr, Cell[BoxData[c_String /; StringMatchQ[c, RegularExpression["XX*"]]], ___], Infinity];

 (* Remove XXX* cells *)
  expr = DeleteCases[expr, Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], ___], Infinity];
  expr = DeleteCases[expr, 
    Cell[_  sty_String /; StringMatchQ[sty, "DocumentStatusSection" | "DocumentStatus"], ___], Infinity];

  (* Remove empty cell groups *)
  expr = RemoveEmptyCellGroups[expr, {
           "KeywordsSection", "Keywords", "DocumentStatusSection", "DocumentStatus"
           }];

  (* Change paclets to URLs *)
  expr = expr /. ButtonBox[c__, BaseStyle->_, o1___, ButtonData->str_String /; StringMatchQ[str, "paclet*"], o2___] :> 
    ButtonBox[c, BaseStyle->"Hyperlink", ButtonData->{URL["http://reference.wolfram.com/language/"<>StringReplace[str,"paclet:"->""]<>".html"], 
    	None}, BaseStyle->"Hyperlink", o1, o2];
    	
  expr = expr /. Cell[BoxData[FormBox[ButtonBox["\[RightGuillemet]", BaseStyle->"Hyperlink",a__], TraditionalForm]]]:>
  ButtonBox["\[RightGuillemet]",BaseStyle->"Hyperlink",a];
  
  (* manipulate some button boxen *)
  expr = ReplacePart[expr, (# -> DocumentationBuild`Utils`MakePacletButton[Extract[expr, #]]) & /@ Position[expr, ButtonBox[_String, ___]]];

  (* Change WindowSize to a width for nicer line wrapping in gifs *)
  expr = DocumentationBuild`Utils`Private`ResetOptions[expr, 
           WindowSize -> {700, 700}, ShowCellBracket->False, Magnification->1,(*PageWidth->WindowWidth,*)
           StyleDefinitions -> 
             Notebook[{Cell[ StyleData[ StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "SolutionsPageStyles.nb"]]], 
               Cell[StyleData["Input"], FontSize -> 11, CellID -> 1,PageWidth->525 ], 
               Cell[StyleData["Output"], FontSize -> 11, CellID -> 1], 
               Cell[StyleData["Graphics"], Magnification -> 1, CellID -> 1, PageWidth->525 ], 
               Cell[StyleData["Picture"], Magnification -> 1, CellID -> 1, PageWidth->525 ], 
               Cell[StyleData["InputHiddenMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputSuppressedMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputThumbnailMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1]
               }]];

  expr
];

(* Create Updated 2012 Solutions page notebook *)
MakeUpdatedSolutionsNotebook[nb_String, language_String:"English", opts___?OptionQ ]:= 
  MakeUpdatedSolutionsNotebook[DocumentationBuild`Utils`GetQuietly[nb], language, opts];
MakeUpdatedSolutionsNotebook[nbExpr_Notebook, language_String:"English", opts___?OptionQ ]:= 
Module[{transform, expr=nbExpr, pageTitle = "None", keywordList = {}, seeAlsoGroup, poss, exampleCaption },

  
    expr = DeleteCases[expr, Cell[___, "History", ___], Infinity];
    expr = DeleteCases[expr, ShowCellTags->True, Infinity];
    expr = DeleteCases[expr, Cell[__, "InternalNoteLink", ___],Infinity];
   	
   	expr = expr /.Cell[TextData[ButtonBox[a_String,
 	BaseStyle->"Hyperlink",
 	ButtonData->{URL[b_String/;
 		StringJoin[
 			Insert[ Riffle[Drop[Drop[StringSplit[b, "/"], 1], -1], "/"],  "http:/" , 1 ]
 		]==="http://reference.wolfram.com/language/ref"], None}, c__]], "SolutionsListing", d__]:>
 	Cell[TextData[StyleBox[ButtonBox[a, BaseStyle->"Hyperlink", ButtonData->{
   	URL[b], None}, c], "InlineFormulaListItem"]], "SolutionsListing", d];


  (* Remove XXX* cells *)
  expr = DeleteCases[expr, Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], ___], Infinity];
  expr = DeleteCases[expr, Cell[BoxData[c_String /; StringMatchQ[c, RegularExpression["XX*"]]], ___], Infinity];

 (* Remove XXX* cells *)
  expr = DeleteCases[expr, Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], ___], Infinity];
  expr = DeleteCases[expr, 
    Cell[_  sty_String /; StringMatchQ[sty, "DocumentStatusSection" | "DocumentStatus"], ___], Infinity];

  (* Remove empty cell groups *)
  expr = RemoveEmptyCellGroups[expr, {
           "KeywordsSection", "Keywords", "DocumentStatusSection", "DocumentStatus"
           }];

  (* Change paclets to URLs *)
  expr = expr /. ButtonBox[c__, BaseStyle->_, o1___, ButtonData->str_String /; StringMatchQ[str, "paclet*"], o2___] :> 
    ButtonBox[c, BaseStyle->"Hyperlink", ButtonData->{URL["http://reference.wolfram.com/language/"<>StringReplace[str,"paclet:"->""]<>".html"], 
    	None}, BaseStyle->"Hyperlink", o1, o2];
    	
  expr = expr /. Cell[BoxData[FormBox[ButtonBox["\[RightGuillemet]", BaseStyle->"Hyperlink",a__], TraditionalForm]]]:>
  ButtonBox["\[RightGuillemet]",BaseStyle->"Hyperlink",a];
  
  (* manipulate some button boxen *)
  expr = ReplacePart[expr, (# -> DocumentationBuild`Utils`MakePacletButton[Extract[expr, #]]) & /@ Position[expr, ButtonBox[_String, ___]]];

  (* Change WindowSize to a width for nicer line wrapping in gifs *)
  expr = DocumentationBuild`Utils`Private`ResetOptions[expr, 
           WindowSize -> {700, 700}, ShowCellBracket->False, Magnification->1,(*PageWidth->WindowWidth,*)
           StyleDefinitions -> 
             Notebook[{Cell[ StyleData[ StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "SolutionsPageStyles.nb"]]], 
               Cell[StyleData["Input"], FontSize -> 11, CellID -> 1,PageWidth->525 ], 
               Cell[StyleData["Output"], FontSize -> 11, CellID -> 1], 
               Cell[StyleData["Graphics"], Magnification -> 1, CellID -> 1, PageWidth->525 ], 
               Cell[StyleData["Picture"], Magnification -> 1, CellID -> 1, PageWidth->525 ], 
               Cell[StyleData["InputHiddenMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputSuppressedMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputThumbnailMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1]
               }]];

  expr
];



(* ExportUpdatedSolutionsPage[outfile, expr, pageType,= opts] *)
ExportUpdatedSolutionsPage[_,$Failed,___] = $Failed;

Options[ExportUpdatedSolutionsPage] = {
    "DefaultParameters" -> {},
    "IsManipulate" -> True,
    "Language" -> "English",
    "SolutionsTest" -> True,
    "TMJTest" -> False
    
};

ExportUpdatedSolutionsPage[file_String, nb_String, language_String:"English", opts___?OptionQ]:= 
  ExportUpdatedSolutionsPage[file, DocumentationBuild`Utils`GetQuietly[nb], language, opts ];
ExportUpdatedSolutionsPage[file_String, nbExpr_Notebook, language_String:"English", opts___?OptionQ]:=
Developer`UseFrontEnd @
Module[{ext, properties={}, outfile=file, transform, keywordList={}, synonymList={}, summaryText="", titleName, res, nbObj, tmp, parentDir, nbFile, localcss, 
  layoutfile, popupLayoutfile, expr, filename, jatitle, altTitleName, nbFileName, isSolutionsQ, isTMJQ, descriptionText},

	isSolutionsQ = "SolutionsTest" /. {opts} /. Options[ExportUpdatedSolutionsPage];
	isTMJQ = "TMJTest" /. {opts} /. Options[ExportUpdatedSolutionsPage];
    nbFileName=opts[[2]];

  LogIt["** Entering ExportUpdatedSolutionsPage **"];

  UseDocumentationBuildXMLTransforms[];

  ext = Switch[language, 
  	"Japanese", "ja", 
  	"Spanish", "es", 
  	"ChineseSimplified", "zh", 
  	"Russian", "ru",
	"Korean", "ko",
	"German", "de",
	"French", "fr",
	"Portuguese", "pt-br", 
  	_, "en"];
  
  Preview = "Preview" /. {opts};
  
  transform = {"HTML","Solutions2012.m"};
  layoutfile = DocsAsset["HTMLTransformLayout", "Solutions2012.html"];

  (* keywords *)
  keywordList = DocumentationBuild`Info`GetNotebookKeywords[nbExpr];
  synonymList = DocumentationBuild`Info`GetNotebookSynonyms[nbExpr];
  summaryText = DocumentationBuild`Info`GetNotebookSummary[nbExpr, "Solutions Pages"];
  
  LogIt["  Keywords: "<>ToString[keywordList]];

  titleName = DocumentationBuild`Info`GetNotebookTitle[nbExpr, "Solution"];
  altTitleName = Cases[nbExpr,Cell[a_,"Categorization",CellLabel->"ForcedPageTitle",b___]:>a,Infinity];
  descriptionText = Last[Cases[nbExpr, CellGroupData[{Cell[a_, "WebMetadataSection", b__], c___}, d___]:>c, Infinity]];

  LogIt["  Title name: "<>ToString[titleName]];
  outfile = 
    If[ StringQ[ext],
      StringReplace[outfile, RegularExpression["(?<!\\.\\w\\w)(\\.html)"]->"."<>ext<>"$1"],
      outfile];
  localcss = StringReplace[DirectoryName@outfile, {DirectoryName@DirectoryName@outfile->"", "\""->"", "/"->"", $PathnameSeparator -> ""}] <> "-localcss";

  (* Create preprocess notebook *)
  LogIt["  MakeUpdatedSolutionsNotebook"];
  
  expr = MakeUpdatedSolutionsNotebook[nbExpr, language, opts];

  LogIt["  NotebookPut"];
  nbObj = DocumentationBuild`Common`PubsEvaluateWithFE@NotebookPut[expr];

  LogIt["  Export"];
  tmp = OpenTemporary[];
  res = Export[First@tmp<>".nb", NotebookGet@nbObj, "NB"];
  FrontEnd`NotebookClose[nbObj];
  expr = Get[First@tmp<>".nb"];
  DeleteFile[First@tmp<>".nb"];
  NotebookClose[nbObj];
  
  (* Merge ButtonBoxes *)
  expr = expr /. a_TextData -> ConvertTextData[a];

  nbFile = StringReplace[outfile, ".html"->".nb"];
  CreateDirectory[ DirectoryName@nbFile];

  If[DocumentationBuild`Transform`debug === True,
    Put[ expr, nbFile];
  ];

  (* now call Transmogrify with the appropriate parameters set.
     note that language extension by default goes before the .html *)
  LogIt["  Off to transmogrify"];

  res = Developer`UseFrontEnd @ Transmogrify`Transmogrify[ outfile, expr, transform, 
      Transmogrify`DefaultParameters -> {"layoutfile"->layoutfile, "popupLayoutfile"->popupLayoutfile, "nbFileName"->nbFileName, "IsSolutionsQ"->isSolutionsQ, "IsTMJQ"->isTMJQ, 
      	"titleName"->titleName, "altTitleName"->altTitleName, "Language"->language, "Preview"->Preview,"keywords"->keywordList, "synonyms"->synonymList, 
      	"summary"->summaryText, "description"->descriptionText, "localcss"->localcss, "filename"->outfile}
    ];
    If[Head@res =!= String, First@res, res]

];



(* Create TMJ page notebook *)
MakeTMJNotebook[nb_String, language_String:"English", opts___?OptionQ ]:= 
  MakeTMJNotebook[DocumentationBuild`Utils`GetQuietly[nb], language, opts];
MakeTMJNotebook[nbExpr_Notebook, language_String:"English", opts___?OptionQ ]:= 
Module[{transform, expr=nbExpr, pageTitle = "None", keywordList = {}, seeAlsoGroup, poss, exampleCaption },

  (* Remove XXX* cells *)
  expr = DeleteCases[expr, Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], ___], Infinity];
  expr = DeleteCases[expr, Cell[BoxData[c_String /; StringMatchQ[c, RegularExpression["XX*"]]], ___], Infinity];

 (* Remove XXX* cells *)
  expr = DeleteCases[expr, Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], ___], Infinity];
  expr = DeleteCases[expr, 
    Cell[_  sty_String /; StringMatchQ[sty, "DocumentStatusSection" | "DocumentStatus"], ___], Infinity];

  (* Remove empty cell groups *)
  expr = RemoveEmptyCellGroups[expr, {
           "KeywordsSection", "Keywords", "DocumentStatusSection", "DocumentStatus"
           }];
           
  expr = expr /. CounterBox["Reference", a_]:> ButtonBox[CounterBox["Reference", a], "ReferenceCounterBox", ButtonData->a];
  
  expr = expr /. CounterBox["DisplayFormulaNumbered", a_]:> ButtonBox[CounterBox["DisplayFormulaNumbered", a], "DisplayFormulaNumbered", ButtonData->a];

  (* Change paclets to URLs *)
  expr = expr /. ButtonBox[c__, BaseStyle->_, o1___, ButtonData->str_String /; StringMatchQ[str, "paclet*"], o2___] :> 
    ButtonBox[c, BaseStyle->"Hyperlink", ButtonData->{URL["http://reference.wolfram.com/language/"<>StringReplace[str,"paclet:"->""]<>".html"], None}, BaseStyle->"Hyperlink", o1, o2];

  (* manipulate some button boxen *)
  expr = ReplacePart[expr, (# -> DocumentationBuild`Utils`MakePacletButton[Extract[expr, #]]) & /@ Position[expr, ButtonBox[_String, ___]]];

  expr = DocumentationBuild`Utils`Private`ResetOptions[expr, 
           WindowSize -> {700, 700}, ShowCellBracket->False, 
           StyleDefinitions -> Import[
				DocsAsset["Internal:Web", FileNameJoin[{"html", "TMJExportStyle.nb"}]],
				"Notebook"
			]
           (*, 
           StyleDefinitions -> 
             Notebook[{Cell[ StyleData[ StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "SolutionsPageStyles.nb"]]], 
               Cell[StyleData["Input"], FontSize -> 11, CellID -> 1,PageWidth->525 ], 
               Cell[StyleData["Output"], FontSize -> 11, CellID -> 1], 
               Cell[StyleData["Graphics"], Magnification -> 1, CellID -> 1, PageWidth->525 ], 
               Cell[StyleData["InputHiddenMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputSuppressedMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputThumbnailMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1]
               }]*)];
               
  expr
];


(* 
    ExportTMJWebPage[outfile, expr, pageType, opts] 
*)
ExportTMJWebPage[_,$Failed,___] = $Failed;

Options[ExportTMJWebPage] = {
    "DefaultParameters" -> {},
    "CompleteHTMLQ" -> True,
    "IsManipulate" -> True,
    "ImageFileDirectory" -> "Files",
    "ImageFilePath" -> "",
    "CoreInformation"-> Automatic,
    "Language" -> "English",
    "TMJTest" -> True
};


ExportTMJWebPage[file_String, nb_String, language_String:"English", opts___?OptionQ]:= 
  ExportTMJWebPage[file, DocumentationBuild`Utils`GetQuietly[nb], language, opts ];
ExportTMJWebPage[file_String, nbExpr_Notebook, language_String:"English", opts___?OptionQ]:=
Developer`UseFrontEnd @
Module[{ext, outfile=file, transform, keywordList={}, synonymList={}, summaryText="", res, nbObj, tmp, CompleteHTMLQ, isTMJQ, 
	htmlBase = "http://reference.wolfram.com", nbFile, localcss, layoutfile, expr, imageFileDir, imageFilePath, pacletName, url="tmj/tmj", extraTransformRules,
	urlBaseRemote = "http://reference.wolfram.com/language/"},

  imageFileDir= "ImageFileDirectory" /. {opts} /. Options[ExportTMJWebPage];
  imageFilePath= "ImageFilePath" /. {opts} /. Options[ExportTMJWebPage];
  CompleteHTMLQ = "CompleteHTMLQ" /. {opts} /. Options[ExportTMJWebPage];
  isTMJQ = "TMJTest" /. {opts} /. Options[ExportTMJWebPage];

  LogIt["** Entering ExportSolutionsPage **"];

  UseDocumentationBuildXMLTransforms[];

  ext = Switch[language,
    "Japanese", "ja",
    "Spanish", "es",
    "ChineseSimplified", "zh",
    _, "en"
  ];

  Preview = "Preview" /. {opts};

  transform = {"HTML","TMJ.m"};
  layoutfile = DocsAsset["HTMLTransformLayout", "TMJ.html"];

  (* keywords *)
  keywordList = DocumentationBuild`Info`GetNotebookKeywords[nbExpr];
  synonymList = DocumentationBuild`Info`GetNotebookSynonyms[nbExpr];
  summaryText = DocumentationBuild`Info`GetNotebookSummary[nbExpr, "Solutions Pages"];
  
  LogIt["  Keywords: "<>ToString[keywordList]];

  LogIt["  Title name: "<>ToString[titleName]];
  outfile = 
    If[ StringQ[ext],
      StringReplace[outfile, RegularExpression["(?<!\\.\\w\\w)(\\.html)"]->"."<>ext<>"$1"],
      outfile];
  localcss = StringReplace[DirectoryName@outfile, {DirectoryName@DirectoryName@outfile->"", "\""->"", "/"->"", $PathnameSeparator -> ""}] <> "-localcss";

  (* Create preprocess notebook *)
  LogIt["  MakeSolutionsPageNotebook"];
  
  expr = MakeTMJNotebook[nbExpr, language, opts];

  LogIt["  NotebookPut"];
  (*nbObj = DocumentationBuild`Common`PubsEvaluateWithFE@NotebookPut[expr];*)
  nbObj = NotebookPut[expr];
            FrontEndExecute[{FrontEnd`NotebookDynamicToLiteral[nbObj]}];

  LogIt["  Export"];
  tmp = OpenTemporary[];
  res = Export[First@tmp<>".nb", NotebookGet@nbObj, "NB"];
  FrontEnd`NotebookClose[nbObj];
  expr = Get[First@tmp<>".nb"];
  DeleteFile[First@tmp<>".nb"];
  NotebookClose[nbObj];
  
  (* Merge ButtonBoxes *)
  expr = expr /. a_TextData -> ConvertTextData[a];
  
  (*expr = expr /. Cell[a__, "MCode"]:>StyleBox[a, "Code"];*)
  expr = expr /. Cell[a__, "MCode"] :> Cell[a, "InlineFormula"]/;MatchQ[a, BoxData[_String]];
  
  expr = expr /.  Cell[TextData[{a__}], "Reference", b___]:>
   Cell[TextData[{"[", CounterBox["ReferenceList"], "] ", a}], "Reference", b];
  expr = expr /.  Cell[a_String, "Reference", b___]:>
   Cell[TextData[{"[", CounterBox["ReferenceList"], "] ", a}], "Reference", b];
  expr = expr /.   Cell[CellGroupData[{Cell["References","Section",x__], Cell[a__, "Reference", b___],
y__},Open]] :> 
	{Cell["References","Section",x], Cell[CellGroupData[{Cell[a, "Reference", b],
y},Open]]};
expr = expr /.   Cell[CellGroupData[{Cell["References","Section",x__], Cell[a_, "Reference", b___],
y__},Open]] :> 
	Sequence[Cell["References","Section"], Cell[CellGroupData[{Cell[a, "Reference", b],
y},Open]]];

expr = expr /. 
Cell[BoxData[FormBox[a_, TraditionalForm]]] :> Cell[BoxData[
    FormBox[a, TraditionalForm]], "Math"];
    
expr = expr /. Cell[a___, TextAlignment->b__, c___] :> Cell[a,c];
expr = expr /. Cell[a___, TextJustification->b__, c___] :> Cell[a,c];
expr = expr /. Cell[a__, b_String, "MakeDisplayImage", c___] :> Cell[a,b<>"MakeDisplayImage",c]; 

extraTransformRules = "{Cell, \"ProgramMakeDisplayImage\"}";
    
expr = expr /.    
    Cell[b_, "None"]:>
    Cell[b];


  nbFile = StringReplace[outfile, ".html"->".nb"];
  Quiet[CreateDirectory[ DirectoryName@nbFile], {CreateDirectory::filex}];

  If[DocumentationBuild`Transform`debug === True,
    Put[ expr, nbFile];
  ];

  (* now call Transmogrify with the appropriate parameters set.
     note that language extension by default goes before the .html *)
  LogIt["  Off to transmogrify"];

  res = Quiet[Developer`UseFrontEnd @ Transmogrify`Transmogrify[ outfile, expr, transform, 
      Transmogrify`DefaultParameters -> {"layoutfile"->layoutfile, "Language"->language, "ImageFileDirectory"->imageFileDir, "ImageFilePath"->imageFilePath, 
      	"keywords"->keywordList, "synonyms"->synonymList, "summary"->summaryText, "localcss"->localcss, "filename"->outfile, "urlBaseRemote"->urlBaseRemote, 
      	"htmlBase"->htmlBase, "CompleteHTMLQ"->CompleteHTMLQ, "pacletName"->pacletName, "url"->url, "IsTMJQ"->isTMJQ, "ExtraRules"->extraTransformRules}
    ], {CreateDirectory::filex}];
    If[Head@res =!= String, First@res, res]

];

Options[ExportMarketingWebPage] = {
  DefaultParameters->{}, Keywords->{}, LogFile->None, Test->False};
  
  
  
  
  
  



(* Create Wolfram Alpha API page notebook *)
MakeAlphaAPINotebook[nb_String, language_String:"English", opts___?OptionQ ]:= 
  MakeAlphaAPINotebook[DocumentationBuild`Utils`GetQuietly[nb], language, opts];
MakeAlphaAPINotebook[nbExpr_Notebook, language_String:"English", opts___?OptionQ ]:= 
Module[{expr=nbExpr, stylesheet},

stylesheet = Cases[Options[expr], Rule[StyleDefinitions, a_Notebook]->a, Infinity][[1]];

  (* Remove XXX* cells *)
  expr = DeleteCases[expr, Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], ___], Infinity];
  expr = DeleteCases[expr, Cell[BoxData[c_String /; StringMatchQ[c, RegularExpression["XX*"]]], ___], Infinity];

 (* Remove XXX* cells *)
  expr = DeleteCases[expr, Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], ___], Infinity];
  expr = DeleteCases[expr, 
    Cell[_  sty_String /; StringMatchQ[sty, "DocumentStatusSection" | "DocumentStatus"], ___], Infinity];

  (* Remove empty cell groups *)
  expr = RemoveEmptyCellGroups[expr, {
           "KeywordsSection", "Keywords", "DocumentStatusSection", "DocumentStatus"
           }];

  (* Change paclets to URLs *)
  expr = expr /. ButtonBox[c__, BaseStyle->_, o1___, ButtonData->str_String /; StringMatchQ[str, "paclet*"], o2___] :> 
    ButtonBox[c, BaseStyle->"Hyperlink", ButtonData->{URL["http://reference.wolfram.com/language/"<>StringReplace[str,"paclet:"->""]<>".html"], None}, BaseStyle->"Hyperlink", o1, o2];

  (* manipulate some button boxen *)
  expr = ReplacePart[expr, (# -> DocumentationBuild`Utils`MakePacletButton[Extract[expr, #]]) & /@ Position[expr, ButtonBox[_String, ___]]];

  expr = DeleteCases[expr, Cell[__, "StrainedGraphics"|"EpigraphAuthor",___], Infinity];

  expr = DocumentationBuild`Utils`Private`ResetOptions[expr, 
           WindowSize -> {750, 825}, ShowCellBracket->False, 
           StyleDefinitions -> stylesheet
];
               
  expr
];


(* 
    ExportTMJWebPage[outfile, expr, pageType, opts] 
*)
ExportAlphaAPIWebPage[_,$Failed,___] = $Failed;

Options[ExportAlphaAPIWebPage] = {
    "DefaultParameters" -> {},
    "CompleteHTMLQ" -> True,
    "IsManipulate" -> True,
    "ImageFileDirectory" -> "Files",
    "ImageFilePath" -> "",
    "CoreInformation"-> Automatic,
    "Language" -> "English",
    "SolutionsTest" -> False,
    "TMJTest" -> False
};




ExportAlphaAPIWebPage[file_String, nb_String, language_String:"English", opts___?OptionQ]:= 
  ExportAlphaAPIWebPage[file, DocumentationBuild`Utils`GetQuietly[nb], language, opts ];
ExportAlphaAPIWebPage[file_String, nbExpr_Notebook, language_String:"English", opts___?OptionQ]:=
Developer`UseFrontEnd @
Module[{ext, outfile=file, transform, keywordList={}, synonymList={}, summaryText="", res, nbObj, tmp, CompleteHTMLQ, isTMJQ, isSolutionsQ, 
	htmlBase = "http://www.wolframalpha.com", nbFile, localcss, layoutfile, expr, imageFileDir, imageFilePath, pacletName, url="tmj/tmj", extraTransformRules,
	urlBaseRemote = "http://www.wolframalpha.com/api/"},
	
  imageFileDir= "ImageFileDirectory" /. {opts} /. Options[ExportAlphaAPIWebPage];
  imageFilePath= "ImageFilePath" /. {opts} /. Options[ExportAlphaAPIWebPage];
  CompleteHTMLQ = "CompleteHTMLQ" /. {opts} /. Options[ExportAlphaAPIWebPage];
  isTMJQ = "TMJTest" /. {opts} /. Options[ExportAlphaAPIWebPage];
  isSolutionsQ = "SolutionsTest" /. {opts} /. Options[ExportSolutionsWebPage];

  LogIt["** Entering ExportSolutionsPage **"];

  UseDocumentationBuildXMLTransforms[];

  ext = Switch[language, 
  	"Japanese", "ja", 
  	"Spanish", "es", 
  	"ChineseSimplified", "zh", 
  	_, "en"];

  Preview = "Preview" /. {opts};

  transform = {"HTML","WolframAlphaAPI.m"};
  layoutfile = DocsAsset["HTMLTransformLayout", "WolframAlphaAPI.html"];


  LogIt["  Title name: "<>ToString[titleName]];
  outfile = 
    If[ StringQ[ext],
      StringReplace[outfile, RegularExpression["(?<!\\.\\w\\w)(\\.html)"]->"."<>ext<>"$1"],
      outfile];
  localcss = StringReplace[DirectoryName@outfile, {DirectoryName@DirectoryName@outfile->"", "\""->"", "/"->"", $PathnameSeparator -> ""}] <> "-localcss";

  (* Create preprocess notebook *)
  LogIt["  MakeAlphaAPINotebook"];
  
  expr = MakeAlphaAPINotebook[nbExpr, language, opts];

  LogIt["  NotebookPut"];
  (*nbObj = DocumentationBuild`Common`PubsEvaluateWithFE@NotebookPut[expr];*)
  nbObj = NotebookPut[expr];
            FrontEndExecute[{FrontEnd`NotebookDynamicToLiteral[nbObj]}];

  LogIt["  Export"];
  tmp = OpenTemporary[];
  res = Export[First@tmp<>".nb", NotebookGet@nbObj, "NB"];
  FrontEnd`NotebookClose[nbObj];
  expr = Get[First@tmp<>".nb"];
  DeleteFile[First@tmp<>".nb"];
  NotebookClose[nbObj];
  
  (* Merge ButtonBoxes *)
  expr = expr /. a_TextData -> ConvertTextData[a];
  
  expr = expr /. Cell[a__, "MCode"] :> Cell[a, "InlineFormula"]/;MatchQ[a, BoxData[_String]];

expr = expr /. 
Cell[BoxData[FormBox[a_, TraditionalForm]]] :> Cell[BoxData[
    FormBox[a, TraditionalForm]], "Math"];
    
expr = expr /. Cell[a___, TextAlignment->b__, c___] :> Cell[a,c];
expr = expr /. Cell[a___, TextJustification->b__, c___] :> Cell[a,c];
expr = expr /. Cell[a__, b_String, "MakeDisplayImage", c___] :> Cell[a,b<>"MakeDisplayImage",c]; 
    
expr = expr /.    
    Cell[b_, "None"]:>
    Cell[b];
    
expr = expr /. {Cell[a__String, "ModInfo"], b__}:>{b};


  nbFile = StringReplace[outfile, ".html"->".nb"];
  Quiet[CreateDirectory[ DirectoryName@nbFile], {CreateDirectory::filex}];

  If[DocumentationBuild`Transform`debug === True,
    Put[ expr, nbFile];
  ];

  (* now call Transmogrify with the appropriate parameters set.
     note that language extension by default goes before the .html *)
  LogIt["  Off to transmogrify"];

  res = Quiet[Developer`UseFrontEnd @ Transmogrify`Transmogrify[ outfile, expr, transform, 
      Transmogrify`DefaultParameters -> {"layoutfile"->layoutfile, "Language"->language, "ImageFileDirectory"->imageFileDir, "ImageFilePath"->imageFilePath, 
      	"keywords"->keywordList, "synonyms"->synonymList, "summary"->summaryText, "localcss"->localcss, "filename"->outfile, "urlBaseRemote"->urlBaseRemote, 
      	"htmlBase"->htmlBase, "CompleteHTMLQ"->CompleteHTMLQ, "pacletName"->pacletName, "url"->url, "IsTMJQ"->isTMJQ,  "IsSolutionsQ"->isSolutionsQ, "ExtraRules"->extraTransformRules}
    ], {CreateDirectory::filex}];
    If[Head@res =!= String, First@res, res]

];



(* 
    ExportSolutionsWebPage[outfile, expr, pageType,= opts] 
    this is the main Transmogrify[] wrapper.
    NOTE - outfile = None just returns the raw Transmogrify output
           Test->True causes ExportMarketingWebPage to use TransmogrifyString[]
           the shorthand functions rawmogtest[] and mogtest[], respectively,
           are wrappers for these two functionalities.
*)
ExportSolutionsWebPage[_,$Failed,___] = $Failed;

Options[ExportSolutionsWebPage] = {
    "DefaultParameters" -> {},
    "IsManipulate" -> True,
    "Language" -> "English",
    "SolutionsTest" -> True,
    "TMJTest" -> False
    
};

ExportSolutionsWebPage[file_String, nb_String, language_String:"English", opts___?OptionQ]:= 
  ExportSolutionsWebPage[file, DocumentationBuild`Utils`GetQuietly[nb], language, opts ];
ExportSolutionsWebPage[file_String, nbExpr_Notebook, language_String:"English", opts___?OptionQ]:=
Developer`UseFrontEnd @
Module[{ext, properties={}, outfile=file, transform, keywordList={}, synonymList={}, summaryText="", titleName, res, nbObj, tmp, parentDir, nbFile, localcss, 
  layoutfile, popupLayoutfile, expr, filename, jatitle, altTitleName, nbFileName, isSolutionsQ, isTMJQ, descriptionText},

	isSolutionsQ = "SolutionsTest" /. {opts} /. Options[ExportSolutionsWebPage];
	isTMJQ = "TMJTest" /. {opts} /. Options[ExportSolutionsWebPage];
    nbFileName=opts[[2]];
    
    

  LogIt["** Entering ExportSolutionsPage **"];

  UseDocumentationBuildXMLTransforms[];

  ext = Switch[language, 
  	"Japanese", "ja", 
  	"Spanish", "es", 
  	"ChineseSimplified", "zh", 
  	"Russian", "ru",
	"Korean", "ko",
	"German", "de",
    "French", "fr",
	"Portuguese", "pt-br", 
  	_, "en"];

  Preview = "Preview" /. {opts};

  transform = {"HTML","SolutionsPages.m"};
  layoutfile = DocsAsset["HTMLTransformLayout", "SolutionsPages.html"];
  popupLayoutfile = DocsAsset["HTMLTransformLayout", "SolutionsPagesPopup.html"];

  (* keywords *)
  keywordList = DocumentationBuild`Info`GetNotebookKeywords[nbExpr];
  synonymList = DocumentationBuild`Info`GetNotebookSynonyms[nbExpr];
  summaryText = DocumentationBuild`Info`GetNotebookSummary[nbExpr, "Solutions Pages"];
  
  LogIt["  Keywords: "<>ToString[keywordList]];

  titleName = DocumentationBuild`Info`GetNotebookTitle[nbExpr, "Solution"];
  altTitleName = Cases[nbExpr,Cell[a_,"Categorization",CellLabel->"ForcedPageTitle",b___]:>a,Infinity];
  descriptionText = Last[Cases[nbExpr, CellGroupData[{Cell[a_, "WebMetadataSection", b__], c___}, d___]:>c, Infinity]];

  LogIt["  Title name: "<>ToString[titleName]];
  outfile = 
    If[ StringQ[ext],
      StringReplace[outfile, RegularExpression["(?<!\\.\\w\\w)(\\.html)"]->"."<>ext<>"$1"],
      outfile];
  localcss = StringReplace[DirectoryName@outfile, {DirectoryName@DirectoryName@outfile->"", "\""->"", "/"->"", $PathnameSeparator -> ""}] <> "-localcss";

  (* Create preprocess notebook *)
  LogIt["  MakeSolutionsPageNotebook"];
  
  expr = MakeSolutionsPageNotebook[nbExpr, language, opts];

  LogIt["  NotebookPut"];
  nbObj = DocumentationBuild`Common`PubsEvaluateWithFE@NotebookPut[expr];

  LogIt["  Export"];
  tmp = OpenTemporary[];
  res = Export[First@tmp<>".nb", NotebookGet@nbObj, "NB"];
  FrontEnd`NotebookClose[nbObj];
  expr = Get[First@tmp<>".nb"];
  DeleteFile[First@tmp<>".nb"];
  NotebookClose[nbObj];
  
  (* Merge ButtonBoxes *)
  expr = expr /. a_TextData -> ConvertTextData[a];

  nbFile = StringReplace[outfile, ".html"->".nb"];
  CreateDirectory[ DirectoryName@nbFile];

  If[DocumentationBuild`Transform`debug === True,
    Put[ expr, nbFile];
  ];

  (* now call Transmogrify with the appropriate parameters set.
     note that language extension by default goes before the .html *)
  LogIt["  Off to transmogrify"];

  res = Developer`UseFrontEnd @ Transmogrify`Transmogrify[ outfile, expr, transform, 
      Transmogrify`DefaultParameters -> {"layoutfile"->layoutfile, "popupLayoutfile"->popupLayoutfile, "nbFileName"->nbFileName, "IsSolutionsQ"->isSolutionsQ, "IsTMJQ"->isTMJQ, 
      	"titleName"->titleName, "altTitleName"->altTitleName, "Language"->language, "Preview"->Preview,"keywords"->keywordList, "synonyms"->synonymList, 
      	"summary"->summaryText, "description"->descriptionText, "localcss"->localcss, "filename"->outfile}
    ];
    If[Head@res =!= String, First@res, res]

];



 ChangeInsiderTextStyle[text__] := Module[{blah},
   blah = 
    text /. {
      Cell[a_, "WolframInsiderSection", Rule[CellTags,"tmj"], b___]:>
      Cell[a, "WolframInsiderSection", Rule[CellTags,"journal"],b],
      Cell[a__, "WolframInsiderText", b___] :> 
      Cell[a, "WolframInsiderTextRight", b],
      ButtonBox[a__,BaseStyle->"Hyperlink",b__] :> 
      ButtonBox[a,BaseStyle->"NewHyperlink",b]
      
      };
   blah
   ];
   
   
 ChangeFeatureTextGrouping[text__] := Module[{blah},
   blah = 
    text /. {
    Cell[a__, "WolframInsiderImage", b___] :> 
      Cell[a, "WolframInsiderImage", b, CellGroupingRules->{"GroupTogetherNestedGrouping", 16000}, CellTags->"ModifyFeatureImage"], 
    Cell[a__, "WolframInsiderImageTitle", b___] :> 
      Cell[a, "WolframInsiderImageTitle", b, CellGroupingRules->{"GroupTogetherNestedGrouping", 16000}]
      };
   blah
   ];


FindAndReplaceInsiderText[exp__]:=(ReplacePart[exp, (# -> ChangeInsiderTextStyle[Extract[exp, #]]) & /@ 
	Union[Position[exp, Cell[__, "WolframInsiderText"|"WolframInsiderSection", ___]], 
		Position[exp, _ButtonBox]]
	 ]);


ModifyRightColumn[ex__]:=ReplacePart[ex, (# -> FindAndReplaceInsiderText[Extract[ex, #]]) & /@ Position[ex, 
	Cell[CellGroupData[{Cell["Column 2", "WolframInsiderRightSide",a___], b___}, 
    	c___], d___]|Cell[CellGroupData[{Cell["Column 2", "WolframInsiderRightSide", ___], __}, Open]]  ]  ];


    
    
    
(* Create Insider notebook *)
MakeInsiderNotebook[nb_String, language_String:"English", opts___?OptionQ ]:= 
  MakeInsiderNotebook[DocumentationBuild`Utils`GetQuietly[nb], language, opts];
MakeInsiderNotebook[nbExpr_Notebook, language_String:"English", opts___?OptionQ ]:= 
Module[{transform, expr=nbExpr, pageTitle = "None", keywordList = {}, seeAlsoGroup, poss, exampleCaption },
  
    expr = DeleteCases[expr, Cell[___, "History", ___], Infinity];
    expr = DeleteCases[expr, ShowCellTags->True, Infinity];
    expr = DeleteCases[expr, Cell[__, "InternalNoteLink", ___],Infinity];
   	
   	expr = expr /.Cell[TextData[ButtonBox[a_String,
 	BaseStyle->"Hyperlink",
 	ButtonData->{URL[b_String/;
 		StringJoin[
 			Insert[ Riffle[Drop[Drop[StringSplit[b, "/"], 1], -1], "/"],  "http:/" , 1 ]
 		]==="http://reference.wolfram.com/language/ref"], None}, c__]], "SolutionsListing", d__]:>
 	Cell[TextData[StyleBox[ButtonBox[a, BaseStyle->"Hyperlink", ButtonData->{
   	URL[b], None}, c], "InlineFormulaListItem"]], "SolutionsListing", d];


  (* Remove XXX* cells *)
  expr = DeleteCases[expr, Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], ___], Infinity];
  expr = DeleteCases[expr, Cell[BoxData[c_String /; StringMatchQ[c, RegularExpression["XX*"]]], ___], Infinity];

 (* Remove XXX* cells *)
  expr = DeleteCases[expr, Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], ___], Infinity];
  expr = DeleteCases[expr, 
    Cell[_  sty_String /; StringMatchQ[sty, "DocumentStatusSection" | "DocumentStatus"], ___], Infinity];

  (* Remove empty cell groups *)
  expr = RemoveEmptyCellGroups[expr, {
           "KeywordsSection", "Keywords", "DocumentStatusSection", "DocumentStatus"
           }];
           
  (* Remove SuppressedMaterial section cell group *)
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[a_, "WolframInsiderSection", 
    "SuppressedMaterial", b___], c___, d___}, e___]], Infinity];
  expr = DeleteCases[expr, Cell[a_, "WolframInsiderSection", 
    "SuppressedMaterial", b___], Infinity];
    


  (* Change paclets to URLs *)
  expr = expr /. ButtonBox[c__, BaseStyle->_, o1___, ButtonData->str_String /; StringMatchQ[str, "paclet*"], o2___] :> 
    ButtonBox[c, BaseStyle->"Hyperlink", ButtonData->{URL["http://reference.wolfram.com/language/"<>StringReplace[str,"paclet:"->""]<>".html"], 
    	None}, BaseStyle->"Hyperlink", o1, o2];
    	
  expr = expr /. Cell[BoxData[FormBox[ButtonBox["\[RightGuillemet]", BaseStyle->"Hyperlink",a__], TraditionalForm]]]:>
  ButtonBox["\[RightGuillemet]",BaseStyle->"Hyperlink",a];
  
  expr = expr /. ButtonBox[
  	ButtonBox[a__, BaseStyle->"Hyperlink",b___],
  		BaseStyle->"Hyperlink",
  		b___]:>
  ButtonBox[a,BaseStyle->"Hyperlink", b];
  
  expr = expr /. ButtonBox[a__, BaseStyle->"Hyperlink",
  		ButtonData->{b_String/;StringTake[b,1]==="#", None}, c___]:>
  ButtonBox[a,BaseStyle->"Hyperlink", ButtonData->b, c];
  
  expr = expr /. ButtonBox[a__, BaseStyle->"Hyperlink",
  		ButtonData->{URL[b_String], None}, c___]:>
  ButtonBox[a,BaseStyle->"Hyperlink", ButtonData->b, c];
  
  expr = expr /. ButtonBox[a_, BaseStyle->"Hyperlink", ButtonData->b_String]/;StringMatchQ[StringTake[b, 1],"#"]:>
  ButtonBox[a, BaseStyle->"InternalPageLink", ButtonData->b];
  
  
  (*Change Hyperlink inside of h2*)
  expr = expr /. Cell[TextData[ButtonBox[a__, BaseStyle->"Hyperlink",
  		b___]], "WolframInsiderHeading2", x___]:>
  Cell[TextData[ButtonBox[a,BaseStyle->"H2Hyperlink",b]], "WolframInsiderHeading2", x];
  expr = expr /. Cell[TextData[ButtonBox[a__, BaseStyle->"Hyperlink",
  		b___]], "WolframInsiderHeading3", x___]:>
  Cell[TextData[ButtonBox[a,BaseStyle->"H3Hyperlink",b]], "WolframInsiderHeading3", x];
  
  
  (* manipulate some button boxen *)
  expr = ReplacePart[expr, (# -> DocumentationBuild`Utils`MakePacletButton[Extract[expr, #]]) & /@ Position[expr, ButtonBox[_String, ___]]];
  
 (* The function below is a series of ReplacePart functions that simply finds all the Cells / Groups of a particular style and 
 rewrites them. This one rewrites the notebook expression, to rewrite the Right column, to rewrite the WolframInsiderText cells.*)
  expr = ModifyRightColumn[expr];
  
  	
    (* Extract and change Image grouping for feature section *)
  expr = ReplacePart[expr, (# -> ChangeFeatureTextGrouping[Extract[expr, #]]) & /@ Position[expr, 
  	Cell[CellGroupData[{Cell[_, "WolframInsiderSection", CellTags->"upcomingevents", ___], __}, Open]]]  ];
  

  (* Change WindowSize to a width for nicer line wrapping in gifs *)
  expr = DocumentationBuild`Utils`Private`ResetOptions[expr, 
           WindowSize -> {700, 700}, ShowCellBracket->False, Magnification->1,(*PageWidth->WindowWidth,*)
           StyleDefinitions -> 
             Notebook[{Cell[ StyleData[ StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "WolframInsiderPageStyles.nb"]]](*, 
               Cell[StyleData["Input"], FontSize -> 11, CellID -> 1,PageWidth->525 ], 
               Cell[StyleData["Output"], FontSize -> 11, CellID -> 1], 
               Cell[StyleData["Graphics"], Magnification -> 1, CellID -> 1, PageWidth->525 ], 
               Cell[StyleData["Picture"], Magnification -> 1, CellID -> 1, PageWidth->525 ], 
               Cell[StyleData["InputHiddenMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputSuppressedMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1], 
               Cell[StyleData["InputThumbnailMaterial", StyleDefinitions->StyleData["Input"]], CellID -> 1]*)
               }]];

  expr
];







(* 
    ExporInsiderWebPage[outfile, expr, pageType,= opts] 
    this is the main Transmogrify[] wrapper.
    NOTE - outfile = None just returns the raw Transmogrify output
           Test->True causes ExportMarketingWebPage to use TransmogrifyString[]
           the shorthand functions rawmogtest[] and mogtest[], respectively,
           are wrappers for these two functionalities.
*)
ExportInsiderWebPage[_,$Failed,___] = $Failed;

Options[ExportInsiderWebPage] = {
    "DefaultParameters" -> {},
    "IsManipulate" -> True,
    "Language" -> "English",
    "ReleaseBuild" -> False,
    "SolutionsTest" -> False,
    "TMJTest" -> False
    
};


ExportInsiderWebPage[file_String, nb_String, language_String:"English", opts___?OptionQ]:= 
  ExportInsiderWebPage[file, DocumentationBuild`Utils`GetQuietly[nb], language, opts ];
ExportInsiderWebPage[file_String, nbExpr_Notebook, language_String:"English", opts___?OptionQ]:=
Developer`UseFrontEnd @
Module[{ext, outfile=file, transform, keywordList={}, synonymList={}, summaryText="", titleName, res, nbObj, tmp, 
	nbFile, localcss, layoutfile, emaillayoutfile, expr, fileName, isSolutionsQ, isTMJQ, descriptionText,
	pageType, pageTitle, jira, month, year, releaseImagePath, buildTag, pathTail, isReleaseBuild},

	isSolutionsQ = "SolutionsTest" /. {opts} /. Options[ExportInsiderWebPage];
	isTMJQ = "TMJTest" /. {opts} /. Options[ExportInsiderWebPage];
	isReleaseBuild = "ReleaseBuild" /. {opts} /. Options[ExportInsiderWebPage];
    
  LogIt["** Entering ExportSolutionsPage **"];

  UseDocumentationBuildXMLTransforms[];

  ext = Switch[language, 
  	"Japanese", "ja", 
  	_, "en"];

  Preview = "Preview" /. {opts};

  transform = {"HTML","Insider.m"};
  layoutfile = DocsAsset["HTMLTransformLayout", "Insider.html"];
  emaillayoutfile = DocsAsset["HTMLTransformLayout", "InsiderEmail.html"];

  (* keywords *)
  keywordList = DocumentationBuild`Info`GetNotebookKeywords[nbExpr];
  synonymList = DocumentationBuild`Info`GetNotebookSynonyms[nbExpr];
  summaryText = DocumentationBuild`Info`GetNotebookSummary[nbExpr, "Solutions Pages"];

  LogIt["  Keywords: "<>ToString[keywordList]];

  descriptionText = Last[Cases[nbExpr, CellGroupData[{Cell[a_, "WebMetadataSection", b__], c___}, d___]:>c, Infinity]];
  pageType=Cases[nbExpr, CellGroupData[{Cell[a_, "CategorizationSection", b__], c___}, d___]:>c, Infinity][[1,1]];
  pageTitle=Cases[nbExpr, CellGroupData[{Cell[a_, "WebMetadataSection", b__], c___}, d___]:>c, Infinity][[1,1]];
  
  jira = Cases[nbExpr, Cell[a_, "Categorization", CellLabel -> "Jira Issue", b__] :> a, Infinity];
  jira = If[jira==={}, "", jira[[1]]  ];
  month = Cases[nbExpr, Cell[a_, "Categorization", CellLabel -> "Month", b__] :> a, Infinity];
  month = If[month==={}, "", month[[1]]  ];  
  year = Cases[nbExpr, Cell[a_, "Categorization", CellLabel -> "Year", b__] :> a, Infinity];
  year = If[year==={}, "", year[[1]]  ];
  
  buildTag = jira<>year<>month;
  pathTail = year<>"/"<>month<>"/"<>buildTag;
  
  releaseImagePath="http://msgcache.wolfram.com/"<>
  Switch[pageType,
  	"Commercial", "wolfram-insider/"<>pathTail,
  	"Academic", "wolfram-insider/"<>pathTail,
  	"Publishing", "publishing/"<>pathTail,
  	"Salespartner", "wolfram-insider/sales-partner/"<>pathTail,
    _, "wolfram-insider/"<>"NO_PAGE_TYPE"
  ];
		
  
  fileName=Switch[pageType,
  	"Salespartner", "Salespartner",
  	"Publishing", "Publishing",
  	"Academic", "ACAD",
  _, "COMM"];

  LogIt["  Title name: "<>ToString[titleName]];
  outfile = 
    If[ StringQ[ext],
      StringReplace[outfile, RegularExpression["(?<!\\.\\w\\w)(\\.html)"]->"."<>ext<>"$1"],
      outfile];
  localcss = StringReplace[DirectoryName@outfile, {DirectoryName@DirectoryName@outfile->"", "\""->"", "/"->"", $PathnameSeparator -> ""}] <> "-localcss";

  (* Create preprocess notebook *)
  LogIt["  MakeSolutionsPageNotebook"];
  
  expr = MakeInsiderNotebook[nbExpr, language, opts];

  LogIt["  NotebookPut"];
  nbObj = DocumentationBuild`Common`PubsEvaluateWithFE@NotebookPut[expr];

  LogIt["  Export"];
  tmp = OpenTemporary[];
  res = Export[First@tmp<>".nb", NotebookGet@nbObj, "NB"];
  FrontEnd`NotebookClose[nbObj];
  expr = Get[First@tmp<>".nb"];
  DeleteFile[First@tmp<>".nb"];
  NotebookClose[nbObj];
		
  expr = expr /. CellGroupData[{Cell[b__,"WolframInsiderSection",c__], 
		Cell[d__, "WolframInsiderListItem", e___], x___},y___] :>
		CellGroupData[{Cell[b,"WolframInsiderSection",c], 
		Cell[CellGroupData[{Cell[d, "WolframInsiderListItem", e](*, Cell[" ", "WolframInsiderListItem"]*)}]], x},y]; 
	
(*  expr = expr /. Cell[CellGroupData[{Cell["Column 2","WolframInsiderColumn",x1__],
		Cell[x2__,"WolframInsiderHeading4",x3__],
		Cell[CellGroupData[{a___, Cell[b__,"WolframInsiderListItem",c__], 
		Cell[d__, "WolframInsiderListItemDateLocation", e___], x___},y___]]},x4___]]:>
		
  Cell[CellGroupData[{Cell["Column 2","WolframInsiderColumn",x1],
		Cell[x2,"WolframInsiderHeading4",x3],
	Cell[CellGroupData[{a, 
		CellGroupData[{Cell[b,"WolframInsiderListItem25",c], Cell[d, "WolframInsiderListItemDateLocation5", e]}], x},y], "SpecialGroup"]}, x4]];
*)
  expr = expr /. CellGroupData[{a___, Cell[b__,"WolframInsiderListItem",c__], 
		Cell[d__, "WolframInsiderListItemDateLocation", e___], x___},y___]:>
	Cell[CellGroupData[{a, 
		CellGroupData[{Cell[b,"WolframInsiderListItem2",c], Cell[d, "WolframInsiderListItemDateLocation", e]}]
		, x},y], "NewGroup"];
		
  expr = expr //. CellGroupData[{a___, Cell[b__,"WolframInsiderListItem",c__], 
		Cell[d__, "WolframInsiderListItemDateLocation", e___], x___},y___]:>
	CellGroupData[{a, 
		CellGroupData[{Cell[b,"WolframInsiderListItem2",c], Cell[d, "WolframInsiderListItemDateLocation", e]}]
		, x},y];
		
  expr = expr /. CellGroupData[{a___, Cell[b__,"WolframInsiderListItem",c__], 
		Cell[d__, "WolframInsiderListAnswer", e___], x___},y___]:>
	Cell[CellGroupData[{a, 
		CellGroupData[{Cell[b,"WolframInsiderListItem3",c], Cell[d, "WolframInsiderListAnswer", e]}]
		, x},y], "NewGroup"];
		
  expr = expr //. CellGroupData[{a___, Cell[b__,"WolframInsiderListItem",c__], 
		Cell[d__, "WolframInsiderListAnswer", e___], x___},y___]:>
	CellGroupData[{a, 
		CellGroupData[{Cell[b,"WolframInsiderListItem3",c], Cell[d, "WolframInsiderListAnswer", e]}]
		, x},y];
     
  
  (* Merge ButtonBoxes *)
  expr = expr /. a_TextData -> ConvertTextData[a];

  nbFile = StringReplace[outfile, ".html"->".nb"];
  CreateDirectory[ DirectoryName@nbFile];

  If[DocumentationBuild`Transform`debug === True,
    Put[ expr, nbFile];
  ];

  (* now call Transmogrify with the appropriate parameters set.
     note that language extension by default goes before the .html *)
  LogIt["  Off to transmogrify"];

  res = Developer`UseFrontEnd @ Transmogrify`Transmogrify[ outfile, expr, transform, 
      Transmogrify`DefaultParameters -> {"layoutfile"->layoutfile, "emaillayoutfile"->emaillayoutfile, "fileName"->fileName, "IsSolutionsQ"->isSolutionsQ, "IsTMJQ"->isTMJQ, 
        "Language"->language, "Preview"->Preview,"keywords"->keywordList, "synonyms"->synonymList, "summary"->summaryText, 
      	"description"->descriptionText, "pageTitle"->pageTitle, "localcss"->localcss, "filename"->outfile, "type"->pageType, "imagepath"->releaseImagePath, 
      	"releaseBuild"->isReleaseBuild}
    ];
    If[Head@res =!= String, First@res, res]

];








Options[ExportMarketingWebPage] = {
  DefaultParameters->{}, Keywords->{}, LogFile->None, Test->False};

CellLabelInThumbnailQ[expr_]:=
TrueQ[ Length @ Cases[expr, Cell[_, "InputThumbnailMaterial", ___], Infinity] > 0];

inlineFunctionStringToButtonBox[str_String] := inlineFunctionStringToButtonBox[ Cell[str,"SeeAlso"] ];
inlineFunctionStringToButtonBox[c___] := (Message[inlineFunctionStringToButtonBox::arg, c]; $Failed);
inlineFunctionStringToButtonBox[Cell[c_String, sty_, opts___] ] :=
Module[{n, f, fList},
  If[(Head@c === String),
    f = StringReplace[c, {"\n"->"", "\r"->""}];
    Which[
      (* . separator *)
      StringCount[f, RegularExpression["\\s+\\.\\s+"]] > 0, 
      (
        f = StringSplit[f, RegularExpression["\\s+\\.\\s+"]];
        fList = 
          Cell[BoxData[ButtonBox[#, BaseStyle -> "Link"]], "InlineFormula"] & /@ f;
        n = Flatten[Riffle[ fList, {{" ", StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"], " "}}], 1];
        Cell[BoxData[RowBox[n]], sty, opts]
      ),
      (* LongDash separator *)
      StringCount[f, RegularExpression["\\s+\[LongDash]\\s+"]] > 0, 
      (* for now, do nothing *)
        Cell[c, sty, opts],
      (*  *)
      StringCount[f, RegularExpression["\\s+\[EmptyVerySmallSquare]\\s+"]] > 0, 
      (
        f = StringSplit[f, RegularExpression["\\s+\[EmptyVerySmallSquare]\\s+"]];
        fList = 
          Cell[BoxData[ButtonBox[#, BaseStyle -> "Link"]], "InlineFormula"] & /@ f;
        n = Flatten[Riffle[ fList, {{" ", StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"], " "}}], 1];
        Cell[BoxData[RowBox[n]], sty, opts]
      ),
     True, 
       Cell[BoxData[RowBox[{Cell[BoxData[ButtonBox[c, BaseStyle -> "Link"]], "InlineFormula"]}]], sty, opts]
    ]    
  ,
  (* Return original input *)
  Cell[c, sty, opts]
  ]
]
inlineFunctionStringToButtonBox::arg = "String expected as input but value was:`1`";


makeMarketingSeeAlsoButton[ Cell[con_, sty_, opts___] ]:=
Cell[TextData@ButtonBox[con, BaseStyle->"Hyperlink", ButtonData->{URL["/language/12/"<>ToLowerCase[StringReplace[ Transmogrify`ConvertToString[Cell[con, sty]], {"|"->"",","->"","."->"", "&" -> "And"," "->"-","\[EAcute]"->"e","\[CapitalAHat]\[CapitalZHacek]"->"e", "\[Hyphen]"->"-", "\[LongDash]"->"-",":"->"-","\[Dash]"->"-","-"->"-"}]] <> "/"], None}], sty, opts];

makeMarketingMoreAboutButton[ Cell[con_, sty_, opts___] ]:=
Cell[TextData@ButtonBox[con, BaseStyle->"Hyperlink", ButtonData->{URL["http://reference.wolfram.com/language/guide/" <> DocumentationBuild`Utils`MakeCamelCaseString@Cell[con, sty] <> ".html"], None}], sty, opts];

makeMarketingRelatedFunctionsButton[ c_ ]:=
Module[{f, fList},
  If[(Head@c === String),
    f = StringReplace[c, {"\n"->"", "\r"->""}];
    Which[
      (* . separator *)
      StringCount[f, RegularExpression["\\s+\\.\\s+"]] > 0, 
      (
        f = StringSplit[f, RegularExpression["\\s+\\.\\s+"]];
        fList = 
          Cell[BoxData[ButtonBox[#, BaseStyle->"Hyperlink", ButtonData->{URL["http://reference.wolfram.com/language/ref/" <> StringReplace[#, {"\""->""}] <> ".html"], None}]], "InlineFormula"] & /@ f;
        Flatten[Riffle[ fList, {{" ", StyleBox["\[FilledVerySmallSquare]", "InlineSeparator"], " "}}], 1]
      ),
      (* LongDash separator *)
      StringCount[f, RegularExpression["\\s+\[LongDash]\\s+"]] > 0, 
      (* for now, do nothing *)
        c,
     True, 
       ButtonBox[c, BaseStyle->"Hyperlink", ButtonData->{URL["http://reference.wolfram.com/language/ref/" <> c <> ".html"], None}]
    ]    
  ,
  (* Return original input *)
  c
  ]
];

makeMarketingRelatedFunctionsButton[ TextData[c_List] ]:= TextData[Flatten@{makeMarketingRelatedFunctionsButton/@c}];


(* Make marketing thumbnail table *)
MakeMarketingThumbnailTable[expr_]:=
Module[{tds, list },
  (* expr is a list of pairs {thumbail, caption} *)

tds = {XMLElement["td", {"class" -> "thumHolder"}, {#[[1]]}], 
       XMLElement["td", {"class" -> "thumCaption"}, {#[[2]]}]} & /@ expr;

list = Partition[tds, 3, 3, {1, 1}, "xxx"];

(* TODO: fold this line into code above  *)
list = list /. "xxx" -> {XMLElement["td", {}, {" "}], XMLElement["td", {}, {" "}]};

  XMLElement["table", {"class" -> "thumbnailTable", "border"->"0", "cellspacing"->"0", "cellpadding"->"0"},
   Flatten@{
    {XMLElement["tr", {}, Flatten@{#[[1, 1]], #[[2, 1]], #[[3, 1]]}], 
       XMLElement["tr", {}, 
        Flatten@{#[[1, 2]], #[[2, 2]], #[[3, 2]]}]} & /@ list}]
];


MakeBodyCaptionPair[expr_]:=
 Flatten@{expr, Cases[expr, Cell[_, "MarketingSubpageSection", ___], Infinity]};


MakeFirstCategory[{id_, title_, htmlpage_}] := 
 Module[{}, 
  XMLElement[
   "div", {"class" -> "sb_item " <> id}, {XMLElement[
     "a", {"href" -> htmlpage}, {title}], 
    XMLElement[
     "div", {"class" -> "sb_line"}, {XML`RawXML["<!-- -->"]}], 
    XMLElement[
     "div", {"class" -> "sb_separator"}, {XML`RawXML["<!-- -->"]}]}]];

MakeRegularCategory[{id_, title_, htmlpage_}] := 
 Module[{}, 
  XMLElement[
   "div", {"class" -> "sb_item " <> id}, {XMLElement[
     "div", {"class" -> "sb_line"}, {XML`RawXML["<!-- -->"]}], 
    XMLElement["a", {"href" -> htmlpage}, {title}], 
    XMLElement[
     "div", {"class" -> "sb_line"}, {XML`RawXML["<!-- -->"]}], 
    XMLElement[
     "div", {"class" -> "sb_separator"}, {XML`RawXML["<!-- -->"]}]}]];

MakeLastCategory[{id_, title_, htmlpage_}] := 
 Module[{}, 
  XMLElement[
   "div", {"class" -> "sb_item " <> id}, {XMLElement[
     "div", {"class" -> "sb_line"}, {XML`RawXML["<!-- -->"]}], 
    XMLElement["a", {"href" -> htmlpage}, {title}], 
    XMLElement[
     "div", {"class" -> "sb_line"}, {XML`RawXML["<!-- -->"]}]}]];

MakeOneRegularSubCategory[{id_, title_, htmlpage_}] := 
 Module[{}, 
  XMLElement[
   "div", {"class" -> "sb_item " <> id}, {XMLElement[
     "div", {"class" -> "sb_line"}, {XML`RawXML["<!-- -->"]}], 
    XMLElement["a", {"href" -> htmlpage}, {title}], 
    XMLElement[
     "div", {"class" -> "sb_line"}, {XML`RawXML["<!-- -->"]}]}]];

MakeRegularSubCategory[{subcatid_, idtitlehtmlpage_List}] := 
 Module[{}, 
  XMLElement["div", {"class" -> "subcat " <> subcatid}, 
   Join[Delete[
     Delete[Map[
       MakeOneRegularSubCategory[{#[[1]], #[[2]], #[[3]]}] &, 
       idtitlehtmlpage], { -1, -1, -1}], {1, 3, 1}], {XMLElement[
      "div", {"class" -> "sb_subline"}, {XML`RawXML["<!-- -->"]}]}]]];

MakeLastSubCategory[{subcatid_, idtitlehtmlpage_List}] := 
 Module[{}, 
  XMLElement["div", {"class" -> "subcat " <> subcatid}, 
   Delete[Delete[
     Map[MakeOneRegularSubCategory[{#[[1]], #[[2]], #[[3]]}] &, 
      idtitlehtmlpage], { -1, -1, -1}], {1, 3, 1}]]];

MakeWhatsNewSidebar[cattree_List] := 
 Module[{catlength}, catlength = Length[cattree]; 
  XMLElement["div", {"id" -> "sidebar"}, 
   Flatten[{XMLElement[
      "div", {"id" -> "sb_top"}, {XML`RawXML["<!-- -->"]}], 
     XMLElement[
      "div", {"class" -> "sb_item sb_first whatsnewAll"}, {XMLElement[
        "a", {"href" -> 
          "/mathematica/new-in-9/index.html"}, {"All What's New Features"}]}],
     XMLElement[
      "div", {"class" -> "sb_line always_on"}, {XML`RawXML[
        "<!-- -->"]}],
     Switch[catlength, 
      1, {MakeLastCategory[{cattree[[1, 1, 1]], cattree[[1, 1, 2]], 
         cattree[[1, 1, 3]]}], MakeLastSubCategory[cattree[[1, 2]]]}, 
      2, {MakeFirstCategory[{cattree[[1, 1, 1]], cattree[[1, 1, 2]], 
         cattree[[1, 1, 3]]}], 
       MakeRegularSubCategory[cattree[[1, 2]]], 
       MakeLastCategory[{cattree[[2, 1, 1]], cattree[[2, 1, 2]], 
         cattree[[2, 1, 3]]}], 
       MakeLastSubCategory[
        cattree[[2, 2]]]}, _, {MakeFirstCategory[{cattree[[1, 1, 1]], 
         cattree[[1, 1, 2]], cattree[[1, 1, 3]]}], 
       MakeRegularSubCategory[cattree[[1, 2]]], 
       Table[{MakeRegularCategory[{cattree[[i, 1, 1]], 
           cattree[[i, 1, 2]], cattree[[i, 1, 3]]}], 
         MakeRegularSubCategory[cattree[[i, 2]]]}, {i, 2, 
         catlength - 1}], 
       MakeLastCategory[{cattree[[-1, 1, 1]], cattree[[-1, 1, 2]], 
         cattree[[-1, 1, 3]]}], 
       MakeLastSubCategory[cattree[[-1, 2]]]}],
     XMLElement[
      "div", {"id" -> "sb_bot"}, {XML`RawXML["<!-- -->"]}]}]]];


ExportWhatsNewSidebar[_,$Failed,___] = $Failed;

ExportWhatsNewSidebar[outputfile_, nb_String, language_String:"English", marketingpagedir_, opts___?OptionQ]:= 
  ExportWhatsNewSidebar[outputfile, DocumentationBuild`Utils`GetQuietly[nb], language, marketingpagedir, opts ];
ExportWhatsNewSidebar[outputfile_, nbExpr_Notebook, language_String:"English", marketingpagedir_, opts___?OptionQ]:=
Developer`UseFrontEnd @
Module[{ext, transform,  res, res2, marketingpagefolder=marketingpagedir},

  LogIt["** Entering ExportWhatsNewSidebar **"];

  UseDocumentationBuildXMLTransforms[];

  ext = Switch[language, "Japanese", "ja", "ChineseSimplified", "zh", _, "en"];
  
  transform = {"HTML","MarketingSidebar.m"};

  res = Developer`UseFrontEnd @ Transmogrify`Transmogrify[ nbExpr, transform, 
      Transmogrify`DefaultParameters -> {
      	"marketingpagefolder"-> marketingpagefolder,
      	"Language"->language}
    ];
    res2 = ExportString[MakeWhatsNewSidebar[res], "XML"]<>"\n<div style=\"clear: both\"><!-- --></div>";
    
    Export[outputfile, res2 , "Lines"]

];

Options[ExportWhatsNewSidebar] = {
  DefaultParameters->{}, Keywords->{}, LogFile->None, Test->False};


(* 
    ExportComparisonWebPage[outfile, expr, pageType, opts] 
    this is the main Transmogrify[] wrapper.
    NOTE - outfile = None just returns the raw Transmogrify output
           Test->True causes ExportComparisonWebPage to use TransmogrifyString[]
           the shorthand functions rawmogtest[] and mogtest[], respectively,
           are wrappers for these two functionalities.
*)

ExportComparisonWebPage[_,$Failed,___] = $Failed;

ExportComparisonWebPage[file_String, nb_String, language_String:"English", opts___?OptionQ]:= 
  ExportComparisonWebPage[file, DocumentationBuild`Utils`GetQuietly[nb], language, opts ];
ExportComparisonWebPage[file_String, nbExpr_Notebook, language_String:"English", opts___?OptionQ]:=
Developer`UseFrontEnd @
Module[{ext, properties={}, outfile=file, transform, keywordList={}, titleName, res, nbObj, tmp },

  UseDocumentationBuildXMLTransforms[];

  ext = Switch[language, "Japanese", "ja", "ChineseSimplified", "zh", _, "en"];
  
  transform = {"HTML","ComparisonPages.m"};

  (* keywords *)
  keywordList = DocumentationBuild`Info`GetNotebookKeywords[nbExpr];
  (*titleName = DocumentationBuild`Info`GetNotebookTitle[nbExpr, "MarketingPage"] <> ": Wolfram Mathematica Comparative Analysis";*)
  titleName = Cases[nbExpr, Cell[a_, "ComparisonTitle", b__]:>a, Infinity ] <> ": Wolfram Mathematica Comparative Analysis";
  
  outfile = 
    If[ StringQ[ext],
      StringReplace[outfile,RegularExpression["(?<!\\.\\w\\w)(\\.html)"]->"."<>ext<>"$1"],
      outfile];

  (* Create preprocess notebook *)
  expr = MakeComparisonPageNotebook[nbExpr, language, opts];

  nbObj = DocumentationBuild`Common`PubsEvaluateWithFE@NotebookPut[expr];
  tmp = OpenTemporary[];
  res=Export[First@tmp<>".nb", NotebookGet@nbObj, "NB"];
  FrontEnd`NotebookClose[nbObj];
  expr = Get[First@tmp<>".nb"];
  DeleteFile[First@tmp<>".nb"];
  NotebookClose[nbObj];
    
  (* Change WindowSize to a width for nicer line wrapping in gifs *)
  expr = DocumentationBuild`Utils`Private`ResetOptions[expr, 
           WindowSize -> {565, 700}, ShowCellBracket->False (*, PageWidth->WindowWidth*)];

  (* now call Transmogrify with the appropriate parameters set.
     note that language extension by default goes before the .html *)
  res = Developer`UseFrontEnd @ Transmogrify`Transmogrify[ outfile, expr, transform, 
      Transmogrify`DefaultParameters -> {"titleName" -> titleName, "keywords"->keywordList, "Language"->language}
    ];
    If[Head@res =!= String, First@res, res]
];

Options[ExportComparisonWebPage] = {
  DefaultParameters->{}, LanguageExtension->"en", Keywords->{}, LogFile->None, Test->False};


(**

**)
ExportCapabilitiesWebPage[file_String, nb_String, language_String:"English", opts___?OptionQ]:= 
  ExportCapabilitiesWebPage[file, DocumentationBuild`Utils`GetQuietly[nb], language, opts ];
ExportCapabilitiesWebPage[file_String, nbExpr_Notebook, language_String:"English", opts___?OptionQ]:=
Developer`UseFrontEnd @
Module[{ext, properties={}, outfile=file, transform, layoutfile, keywordList={}, titleName, res, nbObj, tmp },

  UseDocumentationBuildXMLTransforms[];

  ext = LanguageExtension /. {opts} /. Options[ExportCapabilitiesWebPage];
  transform = {"HTML","CapabilitiesPages.m"};
  layoutfile = DocsAsset["HTMLTransformLayout", "MarketingPages.html"];

  (* meta info *)
  keywordList = GetNotebookKeywords[nbExpr];
  synonymList = GetNotebookSynonyms[nbExpr];
  titleName = GetNotebookTitle[nbExpr, "MarketingPage"];

  outfile = 
    If[ StringQ[ext],
      StringReplace[outfile,RegularExpression["(?<!\\.\\w\\w)(\\.html)"]->"."<>ext<>"$1"],
      outfile];

  (* Create preprocess notebook *)
  expr = MakeCapabilityPageNotebook[nbExpr, language, opts];

  nbObj = DocumentationBuild`Common`PubsEvaluateWithFE@NotebookPut[expr];
  tmp = OpenTemporary[];
  res=Export[First@tmp<>".nb", NotebookGet@nbObj, "NB"];
  FrontEnd`NotebookClose[nbObj];
  expr = Get[First@tmp<>".nb"];
  DeleteFile[First@tmp<>".nb"];
  NotebookClose[nbObj];
    
  (* Change WindowSize to a width for nicer line wrapping in gifs *)
  expr = DocumentationBuild`Utils`ResetOptions[expr, 
           WindowSize -> {565, 700}, ShowCellBracket->False (*, PageWidth->WindowWidth*)];

  (* now call Transmogrify with the appropriate parameters set.
     note that language extension by default goes before the .html *)
  res = Developer`UseFrontEnd @ Transmogrify`Transmogrify[ outfile, expr, transform, 
      Transmogrify`DefaultParameters -> {"layoutfile"->layoutfile, "titleName" -> titleName, "keywords"->keywordList, 
	  "synonyms" -> synonymList}
    ];
    If[Head@res =!= String, First@res, res]
];

Options[ExportCapabilitiesWebPage] = {
  DefaultParameters->{}, LanguageExtension->"en", Keywords->{}, LogFile->None, Test->False};



(* Create Comparison page notebook *)
MakeCapabilityPageNotebook[nb_String, language_String:"English", opts___?OptionQ ]:= 
  MakeCapabilityPageNotebook[DocumentationBuild`Utils`GetQuietly[nb], language, opts];
MakeCapabilityPageNotebook[nbExpr_Notebook, language_String:"English", opts___?OptionQ ]:= 
Module[{transform, expr=nbExpr, pageTitle = "None", keywordList = {} },

  (* Change paclets to URLs *)
  expr = expr /. ButtonBox[c__, BaseStyle->_, o1___, ButtonData->str_String /; StringMatchQ[str, "paclet*"], o2___] :> 
    ButtonBox[c, ButtonData->{URL["http://reference.wolfram.com/language/"<>StringReplace[str,"paclet:"->""]<>".html"], None}, BaseStyle->"Hyperlink", o1, o2];

  (* Change '>>' to \[RightGuillemet] *)
  expr = expr /. ButtonBox[">>", r___] :> ButtonBox["\[RightGuillemet]", r];
  (* Remove "FutureExample" | "TentativeExample" tagged cells *)
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[___, "FutureExample" | "TentativeExample"(* | "ExcludedMaterial"*), ___], ___}, ___], ___], Infinity];
  expr = DeleteCases[expr, Cell[__, "FutureExample" | "TentativeExample" | "ExcludedMaterial", ___], Infinity];
  expr = DeleteCases[expr, Cell[__, CellDingbat->Cell[__, "TentativeID" | "BrightFlag" | "", ___], ___], Infinity];
    
  (* Remove XXX* cells *)
  expr = DeleteCases[expr, Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], ___], Infinity];
  expr = DeleteCases[expr, 
    Cell[_  sty_String /; StringMatchQ[sty, "DocumentStatusSection" | "DocumentStatus"], ___], Infinity];
  

  (* Remove empty cell groups *)
  expr = RemoveEmptyCellGroups[expr, {
           "CapabilitiesFeaturesSection", "CapabilitiesMoreAboutSection", "CapabilitiesRelatedDemonstrationsSection", 
           "CapabilitiesRelatedFunctionsSection", "CapabilitiesRelatedLinksSection", 
           "CapabilitiesSeeAlsoSection", "CapabilitiesSlideshowSection", 
           "DocumentStatus", "DocumentStatusSection", "Keywords", "KeywordsSection"
           }];
  (* Change paclets to URLs *)
  expr = expr /. ButtonBox[c__, BaseStyle->_, o1___, ButtonData->str_String /; StringMatchQ[str, "paclet*"], o2___] :> 
    ButtonBox[c, BaseStyle->"Hyperlink", ButtonData->{URL["http://reference.wolfram.com/language/"<>StringReplace[str,"paclet:"->""]<>".html"], None}, BaseStyle->"Hyperlink", o1, o2];

  (* manipulate some button boxen *)
   expr = ReplacePart[expr, (# -> DocumentationBuild`Utils`MakePacletButton[Extract[expr, #]]) & /@ Position[expr, Position[expr, ButtonBox[_String, ___]]]];

  (* Renumber In/Out Cell labels *)
  expr = DocumentationBuild`Export`RenumberCells[expr, {"CapabilitiesFeaturesSection", "CapabilitiesSlideshowSection", "CapabilitiesItem"}];

  expr
];




(*  Comparison front page SSI file listing *)
ExportComparisonTable[file_String, dir_String] :=
Module[{d},
  (* stet as CreateDirectory rather than createDirectory since d is ignored; it doesn't matter here whether d is a dir name (pre-10.4 behavior) or $Failed (10.4 behavior) *)
  d = CreateDirectory[DirectoryName@file];
  Export[file, MakeComparisonTable[dir], "XML", "AttributeQuoting"->"\"", "Entities"->{"\[LongDash]"->"&mdash;", "'"->"'"}]
];
MakeComparisonTable[dir_] := Module[{data, data2divs, files},
  files = Sort[FileBaseName[#]& /@ FileNames["*.nb", dir], cmpfunc];
  data = Map[
    Function[{file}, Module[{nb, base},
      base = FileBaseName[file];
      nb = DocumentationBuild`Utils`GetQuietly[file];

      Transmogrify`ConvertToString /@ {
        If[Count[nb, Cell[_, "FutureFlag", ___], 2] != 0,
          "FutureFlag", ""],
        base,
        First@Cases[nb, Cell[_, "ComparisonTitle", ___], Infinity, 1],
        MakeComparisonListingCell@First@Cases[nb, Cell[_, "ComparisonListing", ___], Infinity, 1]
      }
    ]],
    ToFileName[dir, #<>".nb"]& /@ files ];
  data2divs = {
    XMLElement["div", {"class" -> "comptitle"}, {
      XMLElement["a", {"href" -> ("content/" <> #[[1]] <> ".html")}, { #[[2]] }]
    }],
    XMLElement["div", {"class" -> "complisting"}, {
      XMLElement["a", {"href" -> ("content/" <> #[[1]] <> ".html")}, { #[[3]] }]
    }]
  }&;
  XMLElement["div", {}, MapThread[
    Function[{datum, class},
      XMLElement["div", {"class" -> class}, {
        XMLElement["table", {"class" -> "comptable"}, {
          XMLElement["tr", {}, Map[
            XMLElement["td", {"valign" -> "top", "width" -> "33%"}, Flatten@Map[data2divs, #]]&,
            {Take[datum, #], Take[datum, {#+1, 2*#}], Drop[datum, 2*#]}&[
              Ceiling[Length[datum]/3]]
          ]]
        }]
      }]
    ],
    {{Cases[data, {"", r___} :> {r}], Cases[data, {"FutureFlag", r___} :> {r}]},
     {"", "Future"}}
  ]]
];


(* Sort letters first, then numbers, ... *)
cmpfunc[s1_, s2_] := Which[
  s1 === "", True, s2 === "", False,
  True, Module[{f1, f2},
    {f1, f2} = StringTake[#, 1] & /@ {s1, s2};
    Which[
      f1 === f2, Apply[cmpfunc, StringDrop[#, 1]& /@ {s1, s2}],
      LetterQ[f1] && LetterQ[f2], OrderedQ[{f1, f2}],
      LetterQ[f1], True,
      LetterQ[f2], False,
      DigitQ[f1] && DigitQ[f2], OrderedQ[{f1, f2}],
      DigitQ[f1], True,
      DigitQ[f2], False,
      True, OrderedQ[{f1, f2}]
]]];


(* Create Comparison page notebook *)
MakeComparisonPageNotebook[nb_String, language_String:"English", opts___?OptionQ ]:= 
  MakeComparisonPageNotebook[DocumentationBuild`Utils`GetQuietly[nb], language, opts];
MakeComparisonPageNotebook[nbExpr_Notebook, language_String:"English", opts___?OptionQ ]:= 
Module[{transform, expr=nbExpr, pageTitle = "None", keywordList = {} },

  (* Change paclets to URLs *)
  expr = expr /. ButtonBox[c__, BaseStyle->_, o1___, ButtonData->str_String /; StringMatchQ[str, "paclet*"], o2___] :> 
    ButtonBox[c, 
      ButtonData->{URL["http://reference.wolfram.com/language/"<>StringReplace[str,"paclet:"->""]<>".html"], None}, 
      BaseStyle->"Hyperlink", o1, o2];

  (* Change '>>' to \[RightGuillemet] *)
  expr = expr /. ButtonBox[">>", r___] :> ButtonBox["\[RightGuillemet]", r];
  (* Remove "FutureExample" | "TentativeExample" tagged cells *)
  expr = DeleteCases[expr, Cell[CellGroupData[{Cell[___, "FutureExample" | "TentativeExample"(* | "ExcludedMaterial"*), ___], ___}, ___], ___], Infinity];
  expr = DeleteCases[expr, Cell[__, "FutureExample" | "TentativeExample" | "ExcludedMaterial", ___], Infinity];
  expr = DeleteCases[expr, Cell[__, CellDingbat->Cell[__, "TentativeID" | "BrightFlag", ___], ___], Infinity];
    
  (* Remove XXX* cells *)
  expr = DeleteCases[expr, Cell[c_String /; StringMatchQ[c, RegularExpression["XX*"]], ___], Infinity];
  expr = DeleteCases[expr, 
    Cell[_  sty_String /; StringMatchQ[sty, "DocumentStatusSection" | "DocumentStatus"], ___], Infinity];
  
  expr = expr /. Cell[c_, "ComparisonAbstract", o___] :> Cell[c, "ComparisonText", o];

  expr = expr /. Cell[CellGroupData[{Cell[con___], Cell[c1_, "ComparisonListing", o1___], Cell[c2_, "ComparisonText", o2___], rest___ },g1___],g2___] :>
                  Cell[CellGroupData[{Cell[con], Cell[c1, "ComparisonListing", o1], Cell[c2, "ComparisonTextFirst", o2], rest}, g1], g2];

  expr = expr /. Cell[con_String, "ComparisonSeeAlsoSection", o___] :> Cell["See Also Analyses On:", "ComparisonSeeAlsoSection", o];
  expr = expr /. Cell[con_String, "ComparisonSeeAlso", o___] :> makeComparisonSeeAlsoButton@Cell[con, "ComparisonSeeAlso", o];

  (* Remove empty cell groups *)
  expr = RemoveEmptyCellGroups[expr, {
           "ComparisonAdvantagesSection", (*"ComparisonAdvantages", "ComparisonFeatures", *)
           "ComparisonFeaturesSection", "ComparisonInteroperabilitySection", (*"ComparisonInteroperability", *)
           "ComparisonSeeAlsoSection", "ComparisonSubpageSection", (*"ComparisonListing", "ComparisonSeeAlso", *)
           (*"ComparisonText", "ComparisonTitle",*) "DocumentStatus", "DocumentStatusSection", "Keywords", "KeywordsSection",
           "ComparisonInternalInformationSection",
           "ComparisonRelatedLinksSection"
           }];

  (* Change paclets to URLs *)
  expr = expr /. ButtonBox[c__, BaseStyle->_, o1___, ButtonData->str_String /; StringMatchQ[str, "paclet*"], o2___] :> 
    ButtonBox[c, BaseStyle->"Hyperlink", ButtonData->{URL["http://reference.wolfram.com/language/"<>StringReplace[str,"paclet:"->""]<>".html"], None}, BaseStyle->"Hyperlink", o1, o2];

  (* manipulate some button boxen *)
   expr = ReplacePart[expr, (# -> DocumentationBuild`Utils`MakePacletButton[Extract[expr, #]]) & /@ Position[expr, ButtonBox[_String, ___]]];

  If[ DocumentationBuild`Make`$ComparisonPageStyleSheet =!= Null,
    expr = expr /. Rule[StyleDefinitions, {_, _}] :> DocumentationBuild`Make`$ComparisonPageStyleSheet;
  ];

  expr
];



GetWindowSizeCellTag[expr_]:=
Module[{tags, ws, t},
  tags = Union@Flatten@Cases[expr, Cell[___, CellTags->t_, ___] :>t, Infinity];
  ws = 
    DeleteCases[
      StringCases[tags, 
        RegularExpression[
          "{WindowSize\\s*->\\s*{\\s*(\\d+)\\s*,\\s*(\\d+)\\s*}}"] -> {"$1", "$2"}], {}];
  If[Length@ws > 0, ToExpression@Flatten[{ First@ws }], {}]
];


makeComparisonSeeAlsoButton[ Cell[con_, sty_, opts___] ]:=
Cell[TextData@ButtonBox[con, BaseStyle->"Hyperlink", ButtonData->{URL["/products/mathematica/analysis/content/"<>DocumentationBuild`Utils`MakeCamelCaseString@Cell[con, sty] <> ".html"], None}], sty, opts];


textStyleToItem[l___]:=
Module[{ce=l},
  ce = ce /. Cell[c_, "MarketingText", o___] :> Cell[c, "HighlightItem", o];
  ce
];


hasMarketingGraphicQ[expr_]:=
Module[{lst, num},
  lst = Cases[expr, CellGroupData[{Cell[_, "MarketingContentsSection", ___], ___}, ___], Infinity];
  num = Cases[lst, 
          Cell[c_ /; Length @ 
            Cases[c, StyleBox[s_String /; StringMatchQ[s, "*Placeholder*"], ___], Infinity] > 0, "Output", ___], Infinity];
  If[ Length @ num > 0, False, True] 
];


HiddenInputQ[___]:= False;
HiddenInputQ[list_List]:= Apply[Or, Not /@ StringFreeQ[list, "HiddenMaterial"]];

HideCellLabelQ[___]:= False;
HideCellLabelQ[list_List] := 
 TrueQ[
    Or @@ StringMatchQ[list, "*HiddenMaterial"] || 
      !(
      Or @@ StringMatchQ[list, "Input" | "InputThumbnailMaterial" ] && 
      Or @@ StringMatchQ[list, "Output" | "OutputThumbnailMaterial" | "Graphics" | "Picture"]
      )
    ];

(* Replacements *)
characterReplace[str_String, search_String, rep_] :=
  Flatten@Riffle[StringSplit[str, search], rep]

characterReplace[Cell[TextData[lst_List], r___], search_String, rep_] :=
 Cell[TextData[Flatten[
    If[Head@# === String,
       characterReplace[#, search, rep],
       #
       ] & /@ lst]], r
  ]

characterReplace[Cell[str_String, r___], search_String, rep_] :=
  characterReplace[str, search, rep]



MakeComparisonListingCell[Cell[c_, "ComparisonListing", ___]]:= MakeComparisonListingCell @ c;
MakeComparisonListingCell[TextData[lst_List]]:= MakeComparisonListingCell @ lst;
MakeComparisonListingCell[TextData[con_]]:= MakeComparisonListingCell[{con}];
MakeComparisonListingCell[str_]:=
Module[{elems, list},
  (* split list on "," *)
  elems = Transmogrify`ConvertToString @ str;
  elems = StringSplit[ elems, RegularExpression[",\\s*"]];
  elems = DeleteCases[elems, "" | "(" | ")" | "," | ", " | "...)" | ", ...)", Infinity];
  elems = If[Head@#===String, FixedPoint[StringReplace[#, {"(" | ")" | "..." -> ""}]&, #], #]&/@elems;
  elems = DeleteCases[elems, "" | "(" | ")" | "," | ", " | "...)" | ", ...)", Infinity];
(*  list = If[(StringLength@StringJoin@@Take[elems, 3] < 35, Take[elems, 2], Take[elems, 3], elems];*)
  Cell[TextData[ 
    Flatten@Join[{"("}, Riffle[Join[If[Length@elems > 2, Take[elems, 3], elems], {"... )"}], ", "]]], "ComparisonListing"]
];


MakeComparisonItemCell[Cell[con_, sty_String, opts___]]:= 
  Cell[TextData[Flatten@{ MakeComparisonItemCell@con }], sty, opts];
MakeComparisonItemCell[TextData[lst_List]]:= MakeComparisonItemCell @ lst;
MakeComparisonItemCell[TextData[con_]]:= MakeComparisonItemCell[{con}];
MakeComparisonItemCell[con_]:=
Module[{elems, str},
  str = Transmogrify`ConvertToString @ con;
  If[ StringLength@str > 86,
    Print["Too long: "<>ToString@StringLength@str],
    Print["Just right: "<>ToString@StringLength@str]
  ];
  con
];



(* *** *)


End[]
EndPackage[]
